#ifdef GET_SVE_BUILTINS
BUILTIN(__builtin_sve_svaba_n_s16, "q8sq8sq8ss", "n")
BUILTIN(__builtin_sve_svaba_n_s32, "q4iq4iq4ii", "n")
BUILTIN(__builtin_sve_svaba_n_s64, "q2Wiq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svaba_n_s8, "q16Scq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svaba_n_u16, "q8Usq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svaba_n_u32, "q4Uiq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svaba_n_u64, "q2UWiq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svaba_n_u8, "q16Ucq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svaba_s16, "q8sq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svaba_s32, "q4iq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svaba_s64, "q2Wiq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svaba_s8, "q16Scq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svaba_u16, "q8Usq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svaba_u32, "q4Uiq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svaba_u64, "q2UWiq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svaba_u8, "q16Ucq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svabalb_n_s16, "q8sq8sq16ScSc", "n")
BUILTIN(__builtin_sve_svabalb_n_s32, "q4iq4iq8ss", "n")
BUILTIN(__builtin_sve_svabalb_n_s64, "q2Wiq2Wiq4ii", "n")
BUILTIN(__builtin_sve_svabalb_n_u16, "q8Usq8Usq16UcUc", "n")
BUILTIN(__builtin_sve_svabalb_n_u32, "q4Uiq4Uiq8UsUs", "n")
BUILTIN(__builtin_sve_svabalb_n_u64, "q2UWiq2UWiq4UiUi", "n")
BUILTIN(__builtin_sve_svabalb_s16, "q8sq8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svabalb_s32, "q4iq4iq8sq8s", "n")
BUILTIN(__builtin_sve_svabalb_s64, "q2Wiq2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svabalb_u16, "q8Usq8Usq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svabalb_u32, "q4Uiq4Uiq8Usq8Us", "n")
BUILTIN(__builtin_sve_svabalb_u64, "q2UWiq2UWiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svabalt_n_s16, "q8sq8sq16ScSc", "n")
BUILTIN(__builtin_sve_svabalt_n_s32, "q4iq4iq8ss", "n")
BUILTIN(__builtin_sve_svabalt_n_s64, "q2Wiq2Wiq4ii", "n")
BUILTIN(__builtin_sve_svabalt_n_u16, "q8Usq8Usq16UcUc", "n")
BUILTIN(__builtin_sve_svabalt_n_u32, "q4Uiq4Uiq8UsUs", "n")
BUILTIN(__builtin_sve_svabalt_n_u64, "q2UWiq2UWiq4UiUi", "n")
BUILTIN(__builtin_sve_svabalt_s16, "q8sq8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svabalt_s32, "q4iq4iq8sq8s", "n")
BUILTIN(__builtin_sve_svabalt_s64, "q2Wiq2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svabalt_u16, "q8Usq8Usq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svabalt_u32, "q4Uiq4Uiq8Usq8Us", "n")
BUILTIN(__builtin_sve_svabalt_u64, "q2UWiq2UWiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svabd_f16_m, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svabd_f16_x, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svabd_f16_z, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svabd_f32_m, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svabd_f32_x, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svabd_f32_z, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svabd_f64_m, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svabd_f64_x, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svabd_f64_z, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svabd_n_f16_m, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svabd_n_f16_x, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svabd_n_f16_z, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svabd_n_f32_m, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svabd_n_f32_x, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svabd_n_f32_z, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svabd_n_f64_m, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svabd_n_f64_x, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svabd_n_f64_z, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svabd_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svabd_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svabd_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svabd_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svabd_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svabd_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svabd_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svabd_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svabd_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svabd_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svabd_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svabd_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svabd_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svabd_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svabd_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svabd_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svabd_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svabd_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svabd_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svabd_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svabd_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svabd_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svabd_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svabd_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svabd_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svabd_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svabd_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svabd_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svabd_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svabd_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svabd_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svabd_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svabd_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svabd_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svabd_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svabd_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svabd_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svabd_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svabd_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svabd_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svabd_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svabd_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svabd_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svabd_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svabd_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svabd_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svabd_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svabd_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svabdlb_n_s16, "q8sq16ScSc", "n")
BUILTIN(__builtin_sve_svabdlb_n_s32, "q4iq8ss", "n")
BUILTIN(__builtin_sve_svabdlb_n_s64, "q2Wiq4ii", "n")
BUILTIN(__builtin_sve_svabdlb_n_u16, "q8Usq16UcUc", "n")
BUILTIN(__builtin_sve_svabdlb_n_u32, "q4Uiq8UsUs", "n")
BUILTIN(__builtin_sve_svabdlb_n_u64, "q2UWiq4UiUi", "n")
BUILTIN(__builtin_sve_svabdlb_s16, "q8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svabdlb_s32, "q4iq8sq8s", "n")
BUILTIN(__builtin_sve_svabdlb_s64, "q2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svabdlb_u16, "q8Usq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svabdlb_u32, "q4Uiq8Usq8Us", "n")
BUILTIN(__builtin_sve_svabdlb_u64, "q2UWiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svabdlt_n_s16, "q8sq16ScSc", "n")
BUILTIN(__builtin_sve_svabdlt_n_s32, "q4iq8ss", "n")
BUILTIN(__builtin_sve_svabdlt_n_s64, "q2Wiq4ii", "n")
BUILTIN(__builtin_sve_svabdlt_n_u16, "q8Usq16UcUc", "n")
BUILTIN(__builtin_sve_svabdlt_n_u32, "q4Uiq8UsUs", "n")
BUILTIN(__builtin_sve_svabdlt_n_u64, "q2UWiq4UiUi", "n")
BUILTIN(__builtin_sve_svabdlt_s16, "q8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svabdlt_s32, "q4iq8sq8s", "n")
BUILTIN(__builtin_sve_svabdlt_s64, "q2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svabdlt_u16, "q8Usq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svabdlt_u32, "q4Uiq8Usq8Us", "n")
BUILTIN(__builtin_sve_svabdlt_u64, "q2UWiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svabs_f16_m, "q8hq8hq16bq8h", "n")
BUILTIN(__builtin_sve_svabs_f16_x, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svabs_f16_z, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svabs_f32_m, "q4fq4fq16bq4f", "n")
BUILTIN(__builtin_sve_svabs_f32_x, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svabs_f32_z, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svabs_f64_m, "q2dq2dq16bq2d", "n")
BUILTIN(__builtin_sve_svabs_f64_x, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svabs_f64_z, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svabs_s16_m, "q8sq8sq16bq8s", "n")
BUILTIN(__builtin_sve_svabs_s16_x, "q8sq16bq8s", "n")
BUILTIN(__builtin_sve_svabs_s16_z, "q8sq16bq8s", "n")
BUILTIN(__builtin_sve_svabs_s32_m, "q4iq4iq16bq4i", "n")
BUILTIN(__builtin_sve_svabs_s32_x, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svabs_s32_z, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svabs_s64_m, "q2Wiq2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svabs_s64_x, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svabs_s64_z, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svabs_s8_m, "q16Scq16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svabs_s8_x, "q16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svabs_s8_z, "q16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svacge_f16, "q16bq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svacge_f32, "q16bq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svacge_f64, "q16bq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svacge_n_f16, "q16bq16bq8hh", "n")
BUILTIN(__builtin_sve_svacge_n_f32, "q16bq16bq4ff", "n")
BUILTIN(__builtin_sve_svacge_n_f64, "q16bq16bq2dd", "n")
BUILTIN(__builtin_sve_svacgt_f16, "q16bq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svacgt_f32, "q16bq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svacgt_f64, "q16bq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svacgt_n_f16, "q16bq16bq8hh", "n")
BUILTIN(__builtin_sve_svacgt_n_f32, "q16bq16bq4ff", "n")
BUILTIN(__builtin_sve_svacgt_n_f64, "q16bq16bq2dd", "n")
BUILTIN(__builtin_sve_svacle_f16, "q16bq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svacle_f32, "q16bq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svacle_f64, "q16bq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svacle_n_f16, "q16bq16bq8hh", "n")
BUILTIN(__builtin_sve_svacle_n_f32, "q16bq16bq4ff", "n")
BUILTIN(__builtin_sve_svacle_n_f64, "q16bq16bq2dd", "n")
BUILTIN(__builtin_sve_svaclt_f16, "q16bq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svaclt_f32, "q16bq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svaclt_f64, "q16bq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svaclt_n_f16, "q16bq16bq8hh", "n")
BUILTIN(__builtin_sve_svaclt_n_f32, "q16bq16bq4ff", "n")
BUILTIN(__builtin_sve_svaclt_n_f64, "q16bq16bq2dd", "n")
BUILTIN(__builtin_sve_svadalp_s16_m, "q8sq16bq8sq16Sc", "n")
BUILTIN(__builtin_sve_svadalp_s16_x, "q8sq16bq8sq16Sc", "n")
BUILTIN(__builtin_sve_svadalp_s16_z, "q8sq16bq8sq16Sc", "n")
BUILTIN(__builtin_sve_svadalp_s32_m, "q4iq16bq4iq8s", "n")
BUILTIN(__builtin_sve_svadalp_s32_x, "q4iq16bq4iq8s", "n")
BUILTIN(__builtin_sve_svadalp_s32_z, "q4iq16bq4iq8s", "n")
BUILTIN(__builtin_sve_svadalp_s64_m, "q2Wiq16bq2Wiq4i", "n")
BUILTIN(__builtin_sve_svadalp_s64_x, "q2Wiq16bq2Wiq4i", "n")
BUILTIN(__builtin_sve_svadalp_s64_z, "q2Wiq16bq2Wiq4i", "n")
BUILTIN(__builtin_sve_svadalp_u16_m, "q8Usq16bq8Usq16Uc", "n")
BUILTIN(__builtin_sve_svadalp_u16_x, "q8Usq16bq8Usq16Uc", "n")
BUILTIN(__builtin_sve_svadalp_u16_z, "q8Usq16bq8Usq16Uc", "n")
BUILTIN(__builtin_sve_svadalp_u32_m, "q4Uiq16bq4Uiq8Us", "n")
BUILTIN(__builtin_sve_svadalp_u32_x, "q4Uiq16bq4Uiq8Us", "n")
BUILTIN(__builtin_sve_svadalp_u32_z, "q4Uiq16bq4Uiq8Us", "n")
BUILTIN(__builtin_sve_svadalp_u64_m, "q2UWiq16bq2UWiq4Ui", "n")
BUILTIN(__builtin_sve_svadalp_u64_x, "q2UWiq16bq2UWiq4Ui", "n")
BUILTIN(__builtin_sve_svadalp_u64_z, "q2UWiq16bq2UWiq4Ui", "n")
BUILTIN(__builtin_sve_svadclb_n_u32, "q4Uiq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svadclb_n_u64, "q2UWiq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svadclb_u32, "q4Uiq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svadclb_u64, "q2UWiq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svadclt_n_u32, "q4Uiq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svadclt_n_u64, "q2UWiq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svadclt_u32, "q4Uiq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svadclt_u64, "q2UWiq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svadd_f16_m, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svadd_f16_x, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svadd_f16_z, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svadd_f32_m, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svadd_f32_x, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svadd_f32_z, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svadd_f64_m, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svadd_f64_x, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svadd_f64_z, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svadd_n_f16_m, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svadd_n_f16_x, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svadd_n_f16_z, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svadd_n_f32_m, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svadd_n_f32_x, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svadd_n_f32_z, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svadd_n_f64_m, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svadd_n_f64_x, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svadd_n_f64_z, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svadd_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svadd_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svadd_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svadd_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svadd_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svadd_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svadd_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svadd_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svadd_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svadd_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svadd_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svadd_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svadd_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svadd_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svadd_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svadd_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svadd_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svadd_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svadd_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svadd_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svadd_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svadd_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svadd_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svadd_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svadd_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svadd_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svadd_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svadd_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svadd_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svadd_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svadd_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svadd_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svadd_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svadd_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svadd_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svadd_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svadd_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svadd_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svadd_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svadd_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svadd_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svadd_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svadd_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svadd_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svadd_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svadd_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svadd_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svadd_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svadda_f16, "hq16bhq8h", "n")
BUILTIN(__builtin_sve_svadda_f32, "fq16bfq4f", "n")
BUILTIN(__builtin_sve_svadda_f64, "dq16bdq2d", "n")
BUILTIN(__builtin_sve_svaddhnb_n_s16, "q16Scq8ss", "n")
BUILTIN(__builtin_sve_svaddhnb_n_s32, "q8sq4ii", "n")
BUILTIN(__builtin_sve_svaddhnb_n_s64, "q4iq2WiWi", "n")
BUILTIN(__builtin_sve_svaddhnb_n_u16, "q16Ucq8UsUs", "n")
BUILTIN(__builtin_sve_svaddhnb_n_u32, "q8Usq4UiUi", "n")
BUILTIN(__builtin_sve_svaddhnb_n_u64, "q4Uiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svaddhnb_s16, "q16Scq8sq8s", "n")
BUILTIN(__builtin_sve_svaddhnb_s32, "q8sq4iq4i", "n")
BUILTIN(__builtin_sve_svaddhnb_s64, "q4iq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svaddhnb_u16, "q16Ucq8Usq8Us", "n")
BUILTIN(__builtin_sve_svaddhnb_u32, "q8Usq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svaddhnb_u64, "q4Uiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svaddhnt_n_s16, "q16Scq16Scq8ss", "n")
BUILTIN(__builtin_sve_svaddhnt_n_s32, "q8sq8sq4ii", "n")
BUILTIN(__builtin_sve_svaddhnt_n_s64, "q4iq4iq2WiWi", "n")
BUILTIN(__builtin_sve_svaddhnt_n_u16, "q16Ucq16Ucq8UsUs", "n")
BUILTIN(__builtin_sve_svaddhnt_n_u32, "q8Usq8Usq4UiUi", "n")
BUILTIN(__builtin_sve_svaddhnt_n_u64, "q4Uiq4Uiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svaddhnt_s16, "q16Scq16Scq8sq8s", "n")
BUILTIN(__builtin_sve_svaddhnt_s32, "q8sq8sq4iq4i", "n")
BUILTIN(__builtin_sve_svaddhnt_s64, "q4iq4iq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svaddhnt_u16, "q16Ucq16Ucq8Usq8Us", "n")
BUILTIN(__builtin_sve_svaddhnt_u32, "q8Usq8Usq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svaddhnt_u64, "q4Uiq4Uiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svaddlb_n_s16, "q8sq16ScSc", "n")
BUILTIN(__builtin_sve_svaddlb_n_s32, "q4iq8ss", "n")
BUILTIN(__builtin_sve_svaddlb_n_s64, "q2Wiq4ii", "n")
BUILTIN(__builtin_sve_svaddlb_n_u16, "q8Usq16UcUc", "n")
BUILTIN(__builtin_sve_svaddlb_n_u32, "q4Uiq8UsUs", "n")
BUILTIN(__builtin_sve_svaddlb_n_u64, "q2UWiq4UiUi", "n")
BUILTIN(__builtin_sve_svaddlb_s16, "q8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svaddlb_s32, "q4iq8sq8s", "n")
BUILTIN(__builtin_sve_svaddlb_s64, "q2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svaddlb_u16, "q8Usq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svaddlb_u32, "q4Uiq8Usq8Us", "n")
BUILTIN(__builtin_sve_svaddlb_u64, "q2UWiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svaddlbt_n_s16, "q8sq16ScSc", "n")
BUILTIN(__builtin_sve_svaddlbt_n_s32, "q4iq8ss", "n")
BUILTIN(__builtin_sve_svaddlbt_n_s64, "q2Wiq4ii", "n")
BUILTIN(__builtin_sve_svaddlbt_s16, "q8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svaddlbt_s32, "q4iq8sq8s", "n")
BUILTIN(__builtin_sve_svaddlbt_s64, "q2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svaddlt_n_s16, "q8sq16ScSc", "n")
BUILTIN(__builtin_sve_svaddlt_n_s32, "q4iq8ss", "n")
BUILTIN(__builtin_sve_svaddlt_n_s64, "q2Wiq4ii", "n")
BUILTIN(__builtin_sve_svaddlt_n_u16, "q8Usq16UcUc", "n")
BUILTIN(__builtin_sve_svaddlt_n_u32, "q4Uiq8UsUs", "n")
BUILTIN(__builtin_sve_svaddlt_n_u64, "q2UWiq4UiUi", "n")
BUILTIN(__builtin_sve_svaddlt_s16, "q8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svaddlt_s32, "q4iq8sq8s", "n")
BUILTIN(__builtin_sve_svaddlt_s64, "q2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svaddlt_u16, "q8Usq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svaddlt_u32, "q4Uiq8Usq8Us", "n")
BUILTIN(__builtin_sve_svaddlt_u64, "q2UWiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svaddp_f16_m, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svaddp_f16_x, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svaddp_f32_m, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svaddp_f32_x, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svaddp_f64_m, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svaddp_f64_x, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svaddp_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svaddp_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svaddp_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svaddp_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svaddp_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svaddp_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svaddp_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svaddp_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svaddp_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svaddp_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svaddp_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svaddp_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svaddp_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svaddp_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svaddp_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svaddp_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svaddv_f16, "hq16bq8h", "n")
BUILTIN(__builtin_sve_svaddv_f32, "fq16bq4f", "n")
BUILTIN(__builtin_sve_svaddv_f64, "dq16bq2d", "n")
BUILTIN(__builtin_sve_svaddv_s16, "Wiq16bq8s", "n")
BUILTIN(__builtin_sve_svaddv_s32, "Wiq16bq4i", "n")
BUILTIN(__builtin_sve_svaddv_s64, "Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svaddv_s8, "Wiq16bq16Sc", "n")
BUILTIN(__builtin_sve_svaddv_u16, "UWiq16bq8Us", "n")
BUILTIN(__builtin_sve_svaddv_u32, "UWiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svaddv_u64, "UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svaddv_u8, "UWiq16bq16Uc", "n")
BUILTIN(__builtin_sve_svaddwb_n_s16, "q8sq8sSc", "n")
BUILTIN(__builtin_sve_svaddwb_n_s32, "q4iq4is", "n")
BUILTIN(__builtin_sve_svaddwb_n_s64, "q2Wiq2Wii", "n")
BUILTIN(__builtin_sve_svaddwb_n_u16, "q8Usq8UsUc", "n")
BUILTIN(__builtin_sve_svaddwb_n_u32, "q4Uiq4UiUs", "n")
BUILTIN(__builtin_sve_svaddwb_n_u64, "q2UWiq2UWiUi", "n")
BUILTIN(__builtin_sve_svaddwb_s16, "q8sq8sq16Sc", "n")
BUILTIN(__builtin_sve_svaddwb_s32, "q4iq4iq8s", "n")
BUILTIN(__builtin_sve_svaddwb_s64, "q2Wiq2Wiq4i", "n")
BUILTIN(__builtin_sve_svaddwb_u16, "q8Usq8Usq16Uc", "n")
BUILTIN(__builtin_sve_svaddwb_u32, "q4Uiq4Uiq8Us", "n")
BUILTIN(__builtin_sve_svaddwb_u64, "q2UWiq2UWiq4Ui", "n")
BUILTIN(__builtin_sve_svaddwt_n_s16, "q8sq8sSc", "n")
BUILTIN(__builtin_sve_svaddwt_n_s32, "q4iq4is", "n")
BUILTIN(__builtin_sve_svaddwt_n_s64, "q2Wiq2Wii", "n")
BUILTIN(__builtin_sve_svaddwt_n_u16, "q8Usq8UsUc", "n")
BUILTIN(__builtin_sve_svaddwt_n_u32, "q4Uiq4UiUs", "n")
BUILTIN(__builtin_sve_svaddwt_n_u64, "q2UWiq2UWiUi", "n")
BUILTIN(__builtin_sve_svaddwt_s16, "q8sq8sq16Sc", "n")
BUILTIN(__builtin_sve_svaddwt_s32, "q4iq4iq8s", "n")
BUILTIN(__builtin_sve_svaddwt_s64, "q2Wiq2Wiq4i", "n")
BUILTIN(__builtin_sve_svaddwt_u16, "q8Usq8Usq16Uc", "n")
BUILTIN(__builtin_sve_svaddwt_u32, "q4Uiq4Uiq8Us", "n")
BUILTIN(__builtin_sve_svaddwt_u64, "q2UWiq2UWiq4Ui", "n")
BUILTIN(__builtin_sve_svadrb_u32base_s32offset, "q4Uiq4Uiq4i", "n")
BUILTIN(__builtin_sve_svadrb_u32base_u32offset, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svadrb_u64base_s64offset, "q2UWiq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svadrb_u64base_u64offset, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svadrd_u32base_s32index, "q4Uiq4Uiq4i", "n")
BUILTIN(__builtin_sve_svadrd_u32base_u32index, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svadrd_u64base_s64index, "q2UWiq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svadrd_u64base_u64index, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svadrh_u32base_s32index, "q4Uiq4Uiq4i", "n")
BUILTIN(__builtin_sve_svadrh_u32base_u32index, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svadrh_u64base_s64index, "q2UWiq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svadrh_u64base_u64index, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svadrw_u32base_s32index, "q4Uiq4Uiq4i", "n")
BUILTIN(__builtin_sve_svadrw_u32base_u32index, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svadrw_u64base_s64index, "q2UWiq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svadrw_u64base_u64index, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svaesd_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svaese_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svaesimc_u8, "q16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svaesmc_u8, "q16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svand_b_z, "q16bq16bq16bq16b", "n")
BUILTIN(__builtin_sve_svand_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svand_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svand_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svand_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svand_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svand_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svand_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svand_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svand_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svand_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svand_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svand_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svand_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svand_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svand_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svand_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svand_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svand_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svand_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svand_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svand_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svand_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svand_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svand_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svand_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svand_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svand_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svand_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svand_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svand_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svand_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svand_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svand_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svand_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svand_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svand_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svand_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svand_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svand_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svand_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svand_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svand_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svand_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svand_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svand_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svand_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svand_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svand_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svandv_s16, "sq16bq8s", "n")
BUILTIN(__builtin_sve_svandv_s32, "iq16bq4i", "n")
BUILTIN(__builtin_sve_svandv_s64, "Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svandv_s8, "Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svandv_u16, "Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svandv_u32, "Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svandv_u64, "UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svandv_u8, "Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svasr_n_s16_m, "q8sq16bq8sUs", "n")
BUILTIN(__builtin_sve_svasr_n_s16_x, "q8sq16bq8sUs", "n")
BUILTIN(__builtin_sve_svasr_n_s16_z, "q8sq16bq8sUs", "n")
BUILTIN(__builtin_sve_svasr_n_s32_m, "q4iq16bq4iUi", "n")
BUILTIN(__builtin_sve_svasr_n_s32_x, "q4iq16bq4iUi", "n")
BUILTIN(__builtin_sve_svasr_n_s32_z, "q4iq16bq4iUi", "n")
BUILTIN(__builtin_sve_svasr_n_s64_m, "q2Wiq16bq2WiUWi", "n")
BUILTIN(__builtin_sve_svasr_n_s64_x, "q2Wiq16bq2WiUWi", "n")
BUILTIN(__builtin_sve_svasr_n_s64_z, "q2Wiq16bq2WiUWi", "n")
BUILTIN(__builtin_sve_svasr_n_s8_m, "q16Scq16bq16ScUc", "n")
BUILTIN(__builtin_sve_svasr_n_s8_x, "q16Scq16bq16ScUc", "n")
BUILTIN(__builtin_sve_svasr_n_s8_z, "q16Scq16bq16ScUc", "n")
BUILTIN(__builtin_sve_svasr_s16_m, "q8sq16bq8sq8Us", "n")
BUILTIN(__builtin_sve_svasr_s16_x, "q8sq16bq8sq8Us", "n")
BUILTIN(__builtin_sve_svasr_s16_z, "q8sq16bq8sq8Us", "n")
BUILTIN(__builtin_sve_svasr_s32_m, "q4iq16bq4iq4Ui", "n")
BUILTIN(__builtin_sve_svasr_s32_x, "q4iq16bq4iq4Ui", "n")
BUILTIN(__builtin_sve_svasr_s32_z, "q4iq16bq4iq4Ui", "n")
BUILTIN(__builtin_sve_svasr_s64_m, "q2Wiq16bq2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svasr_s64_x, "q2Wiq16bq2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svasr_s64_z, "q2Wiq16bq2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svasr_s8_m, "q16Scq16bq16Scq16Uc", "n")
BUILTIN(__builtin_sve_svasr_s8_x, "q16Scq16bq16Scq16Uc", "n")
BUILTIN(__builtin_sve_svasr_s8_z, "q16Scq16bq16Scq16Uc", "n")
BUILTIN(__builtin_sve_svasr_wide_n_s16_m, "q8sq16bq8sUWi", "n")
BUILTIN(__builtin_sve_svasr_wide_n_s16_x, "q8sq16bq8sUWi", "n")
BUILTIN(__builtin_sve_svasr_wide_n_s16_z, "q8sq16bq8sUWi", "n")
BUILTIN(__builtin_sve_svasr_wide_n_s32_m, "q4iq16bq4iUWi", "n")
BUILTIN(__builtin_sve_svasr_wide_n_s32_x, "q4iq16bq4iUWi", "n")
BUILTIN(__builtin_sve_svasr_wide_n_s32_z, "q4iq16bq4iUWi", "n")
BUILTIN(__builtin_sve_svasr_wide_n_s8_m, "q16Scq16bq16ScUWi", "n")
BUILTIN(__builtin_sve_svasr_wide_n_s8_x, "q16Scq16bq16ScUWi", "n")
BUILTIN(__builtin_sve_svasr_wide_n_s8_z, "q16Scq16bq16ScUWi", "n")
BUILTIN(__builtin_sve_svasr_wide_s16_m, "q8sq16bq8sq2UWi", "n")
BUILTIN(__builtin_sve_svasr_wide_s16_x, "q8sq16bq8sq2UWi", "n")
BUILTIN(__builtin_sve_svasr_wide_s16_z, "q8sq16bq8sq2UWi", "n")
BUILTIN(__builtin_sve_svasr_wide_s32_m, "q4iq16bq4iq2UWi", "n")
BUILTIN(__builtin_sve_svasr_wide_s32_x, "q4iq16bq4iq2UWi", "n")
BUILTIN(__builtin_sve_svasr_wide_s32_z, "q4iq16bq4iq2UWi", "n")
BUILTIN(__builtin_sve_svasr_wide_s8_m, "q16Scq16bq16Scq2UWi", "n")
BUILTIN(__builtin_sve_svasr_wide_s8_x, "q16Scq16bq16Scq2UWi", "n")
BUILTIN(__builtin_sve_svasr_wide_s8_z, "q16Scq16bq16Scq2UWi", "n")
BUILTIN(__builtin_sve_svasrd_n_s16_m, "q8sq16bq8sIUWi", "n")
BUILTIN(__builtin_sve_svasrd_n_s16_x, "q8sq16bq8sIUWi", "n")
BUILTIN(__builtin_sve_svasrd_n_s16_z, "q8sq16bq8sIUWi", "n")
BUILTIN(__builtin_sve_svasrd_n_s32_m, "q4iq16bq4iIUWi", "n")
BUILTIN(__builtin_sve_svasrd_n_s32_x, "q4iq16bq4iIUWi", "n")
BUILTIN(__builtin_sve_svasrd_n_s32_z, "q4iq16bq4iIUWi", "n")
BUILTIN(__builtin_sve_svasrd_n_s64_m, "q2Wiq16bq2WiIUWi", "n")
BUILTIN(__builtin_sve_svasrd_n_s64_x, "q2Wiq16bq2WiIUWi", "n")
BUILTIN(__builtin_sve_svasrd_n_s64_z, "q2Wiq16bq2WiIUWi", "n")
BUILTIN(__builtin_sve_svasrd_n_s8_m, "q16Scq16bq16ScIUWi", "n")
BUILTIN(__builtin_sve_svasrd_n_s8_x, "q16Scq16bq16ScIUWi", "n")
BUILTIN(__builtin_sve_svasrd_n_s8_z, "q16Scq16bq16ScIUWi", "n")
BUILTIN(__builtin_sve_svbcax_n_s16, "q8sq8sq8ss", "n")
BUILTIN(__builtin_sve_svbcax_n_s32, "q4iq4iq4ii", "n")
BUILTIN(__builtin_sve_svbcax_n_s64, "q2Wiq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svbcax_n_s8, "q16Scq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svbcax_n_u16, "q8Usq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svbcax_n_u32, "q4Uiq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svbcax_n_u64, "q2UWiq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svbcax_n_u8, "q16Ucq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svbcax_s16, "q8sq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svbcax_s32, "q4iq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svbcax_s64, "q2Wiq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svbcax_s8, "q16Scq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svbcax_u16, "q8Usq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svbcax_u32, "q4Uiq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svbcax_u64, "q2UWiq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svbcax_u8, "q16Ucq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svbdep_n_u16, "q8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svbdep_n_u32, "q4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svbdep_n_u64, "q2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svbdep_n_u8, "q16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svbdep_u16, "q8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svbdep_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svbdep_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svbdep_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svbext_n_u16, "q8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svbext_n_u32, "q4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svbext_n_u64, "q2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svbext_n_u8, "q16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svbext_u16, "q8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svbext_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svbext_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svbext_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svbfdot_f32, "q4fq4fq8yq8y", "n")
BUILTIN(__builtin_sve_svbfdot_lane_f32, "q4fq4fq8yq8yUWi", "n")
BUILTIN(__builtin_sve_svbfdot_n_f32, "q4fq4fq8yy", "n")
BUILTIN(__builtin_sve_svbfmlalb_f32, "q4fq4fq8yq8y", "n")
BUILTIN(__builtin_sve_svbfmlalb_lane_f32, "q4fq4fq8yq8yUWi", "n")
BUILTIN(__builtin_sve_svbfmlalb_n_f32, "q4fq4fq8yy", "n")
BUILTIN(__builtin_sve_svbfmlalt_f32, "q4fq4fq8yq8y", "n")
BUILTIN(__builtin_sve_svbfmlalt_lane_f32, "q4fq4fq8yq8yUWi", "n")
BUILTIN(__builtin_sve_svbfmlalt_n_f32, "q4fq4fq8yy", "n")
BUILTIN(__builtin_sve_svbfmmla_f32, "q4fq4fq8yq8y", "n")
BUILTIN(__builtin_sve_svbgrp_n_u16, "q8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svbgrp_n_u32, "q4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svbgrp_n_u64, "q2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svbgrp_n_u8, "q16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svbgrp_u16, "q8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svbgrp_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svbgrp_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svbgrp_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svbic_b_z, "q16bq16bq16bq16b", "n")
BUILTIN(__builtin_sve_svbic_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svbic_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svbic_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svbic_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svbic_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svbic_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svbic_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svbic_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svbic_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svbic_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svbic_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svbic_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svbic_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svbic_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svbic_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svbic_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svbic_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svbic_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svbic_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svbic_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svbic_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svbic_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svbic_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svbic_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svbic_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svbic_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svbic_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svbic_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svbic_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svbic_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svbic_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svbic_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svbic_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svbic_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svbic_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svbic_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svbic_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svbic_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svbic_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svbic_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svbic_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svbic_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svbic_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svbic_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svbic_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svbic_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svbic_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svbic_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svbrka_b_m, "q16bq16bq16bq16b", "n")
BUILTIN(__builtin_sve_svbrka_b_z, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svbrkb_b_m, "q16bq16bq16bq16b", "n")
BUILTIN(__builtin_sve_svbrkb_b_z, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svbrkn_b_z, "q16bq16bq16bq16b", "n")
BUILTIN(__builtin_sve_svbrkpa_b_z, "q16bq16bq16bq16b", "n")
BUILTIN(__builtin_sve_svbrkpb_b_z, "q16bq16bq16bq16b", "n")
BUILTIN(__builtin_sve_svbsl1n_n_s16, "q8sq8sq8ss", "n")
BUILTIN(__builtin_sve_svbsl1n_n_s32, "q4iq4iq4ii", "n")
BUILTIN(__builtin_sve_svbsl1n_n_s64, "q2Wiq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svbsl1n_n_s8, "q16Scq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svbsl1n_n_u16, "q8Usq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svbsl1n_n_u32, "q4Uiq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svbsl1n_n_u64, "q2UWiq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svbsl1n_n_u8, "q16Ucq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svbsl1n_s16, "q8sq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svbsl1n_s32, "q4iq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svbsl1n_s64, "q2Wiq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svbsl1n_s8, "q16Scq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svbsl1n_u16, "q8Usq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svbsl1n_u32, "q4Uiq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svbsl1n_u64, "q2UWiq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svbsl1n_u8, "q16Ucq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svbsl2n_n_s16, "q8sq8sq8ss", "n")
BUILTIN(__builtin_sve_svbsl2n_n_s32, "q4iq4iq4ii", "n")
BUILTIN(__builtin_sve_svbsl2n_n_s64, "q2Wiq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svbsl2n_n_s8, "q16Scq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svbsl2n_n_u16, "q8Usq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svbsl2n_n_u32, "q4Uiq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svbsl2n_n_u64, "q2UWiq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svbsl2n_n_u8, "q16Ucq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svbsl2n_s16, "q8sq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svbsl2n_s32, "q4iq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svbsl2n_s64, "q2Wiq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svbsl2n_s8, "q16Scq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svbsl2n_u16, "q8Usq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svbsl2n_u32, "q4Uiq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svbsl2n_u64, "q2UWiq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svbsl2n_u8, "q16Ucq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svbsl_n_s16, "q8sq8sq8ss", "n")
BUILTIN(__builtin_sve_svbsl_n_s32, "q4iq4iq4ii", "n")
BUILTIN(__builtin_sve_svbsl_n_s64, "q2Wiq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svbsl_n_s8, "q16Scq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svbsl_n_u16, "q8Usq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svbsl_n_u32, "q4Uiq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svbsl_n_u64, "q2UWiq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svbsl_n_u8, "q16Ucq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svbsl_s16, "q8sq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svbsl_s32, "q4iq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svbsl_s64, "q2Wiq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svbsl_s8, "q16Scq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svbsl_u16, "q8Usq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svbsl_u32, "q4Uiq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svbsl_u64, "q2UWiq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svbsl_u8, "q16Ucq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svcadd_f16_m, "q8hq16bq8hq8hIUWi", "n")
BUILTIN(__builtin_sve_svcadd_f16_x, "q8hq16bq8hq8hIUWi", "n")
BUILTIN(__builtin_sve_svcadd_f16_z, "q8hq16bq8hq8hIUWi", "n")
BUILTIN(__builtin_sve_svcadd_f32_m, "q4fq16bq4fq4fIUWi", "n")
BUILTIN(__builtin_sve_svcadd_f32_x, "q4fq16bq4fq4fIUWi", "n")
BUILTIN(__builtin_sve_svcadd_f32_z, "q4fq16bq4fq4fIUWi", "n")
BUILTIN(__builtin_sve_svcadd_f64_m, "q2dq16bq2dq2dIUWi", "n")
BUILTIN(__builtin_sve_svcadd_f64_x, "q2dq16bq2dq2dIUWi", "n")
BUILTIN(__builtin_sve_svcadd_f64_z, "q2dq16bq2dq2dIUWi", "n")
BUILTIN(__builtin_sve_svcadd_s16, "q8sq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svcadd_s32, "q4iq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svcadd_s64, "q2Wiq2Wiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svcadd_s8, "q16Scq16Scq16ScIUWi", "n")
BUILTIN(__builtin_sve_svcadd_u16, "q8Usq8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svcadd_u32, "q4Uiq4Uiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svcadd_u64, "q2UWiq2UWiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svcadd_u8, "q16Ucq16Ucq16UcIUWi", "n")
BUILTIN(__builtin_sve_svcdot_lane_s32, "q4iq4iq16Scq16ScIUWiIUWi", "n")
BUILTIN(__builtin_sve_svcdot_lane_s64, "q2Wiq2Wiq8sq8sIUWiIUWi", "n")
BUILTIN(__builtin_sve_svcdot_s32, "q4iq4iq16Scq16ScIUWi", "n")
BUILTIN(__builtin_sve_svcdot_s64, "q2Wiq2Wiq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svclasta_bf16, "q8yq16bq8yq8y", "n")
BUILTIN(__builtin_sve_svclasta_f16, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svclasta_f32, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svclasta_f64, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svclasta_n_bf16, "yq16byq8y", "n")
BUILTIN(__builtin_sve_svclasta_n_f16, "hq16bhq8h", "n")
BUILTIN(__builtin_sve_svclasta_n_f32, "fq16bfq4f", "n")
BUILTIN(__builtin_sve_svclasta_n_f64, "dq16bdq2d", "n")
BUILTIN(__builtin_sve_svclasta_n_s16, "sq16bsq8s", "n")
BUILTIN(__builtin_sve_svclasta_n_s32, "iq16biq4i", "n")
BUILTIN(__builtin_sve_svclasta_n_s64, "Wiq16bWiq2Wi", "n")
BUILTIN(__builtin_sve_svclasta_n_s8, "Scq16bScq16Sc", "n")
BUILTIN(__builtin_sve_svclasta_n_u16, "Usq16bUsq8Us", "n")
BUILTIN(__builtin_sve_svclasta_n_u32, "Uiq16bUiq4Ui", "n")
BUILTIN(__builtin_sve_svclasta_n_u64, "UWiq16bUWiq2UWi", "n")
BUILTIN(__builtin_sve_svclasta_n_u8, "Ucq16bUcq16Uc", "n")
BUILTIN(__builtin_sve_svclasta_s16, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svclasta_s32, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svclasta_s64, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svclasta_s8, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svclasta_u16, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svclasta_u32, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svclasta_u64, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svclasta_u8, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svclastb_bf16, "q8yq16bq8yq8y", "n")
BUILTIN(__builtin_sve_svclastb_f16, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svclastb_f32, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svclastb_f64, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svclastb_n_bf16, "yq16byq8y", "n")
BUILTIN(__builtin_sve_svclastb_n_f16, "hq16bhq8h", "n")
BUILTIN(__builtin_sve_svclastb_n_f32, "fq16bfq4f", "n")
BUILTIN(__builtin_sve_svclastb_n_f64, "dq16bdq2d", "n")
BUILTIN(__builtin_sve_svclastb_n_s16, "sq16bsq8s", "n")
BUILTIN(__builtin_sve_svclastb_n_s32, "iq16biq4i", "n")
BUILTIN(__builtin_sve_svclastb_n_s64, "Wiq16bWiq2Wi", "n")
BUILTIN(__builtin_sve_svclastb_n_s8, "Scq16bScq16Sc", "n")
BUILTIN(__builtin_sve_svclastb_n_u16, "Usq16bUsq8Us", "n")
BUILTIN(__builtin_sve_svclastb_n_u32, "Uiq16bUiq4Ui", "n")
BUILTIN(__builtin_sve_svclastb_n_u64, "UWiq16bUWiq2UWi", "n")
BUILTIN(__builtin_sve_svclastb_n_u8, "Ucq16bUcq16Uc", "n")
BUILTIN(__builtin_sve_svclastb_s16, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svclastb_s32, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svclastb_s64, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svclastb_s8, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svclastb_u16, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svclastb_u32, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svclastb_u64, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svclastb_u8, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svcls_s16_m, "q8Usq8Usq16bq8s", "n")
BUILTIN(__builtin_sve_svcls_s16_x, "q8Usq16bq8s", "n")
BUILTIN(__builtin_sve_svcls_s16_z, "q8Usq16bq8s", "n")
BUILTIN(__builtin_sve_svcls_s32_m, "q4Uiq4Uiq16bq4i", "n")
BUILTIN(__builtin_sve_svcls_s32_x, "q4Uiq16bq4i", "n")
BUILTIN(__builtin_sve_svcls_s32_z, "q4Uiq16bq4i", "n")
BUILTIN(__builtin_sve_svcls_s64_m, "q2UWiq2UWiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svcls_s64_x, "q2UWiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svcls_s64_z, "q2UWiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svcls_s8_m, "q16Ucq16Ucq16bq16Sc", "n")
BUILTIN(__builtin_sve_svcls_s8_x, "q16Ucq16bq16Sc", "n")
BUILTIN(__builtin_sve_svcls_s8_z, "q16Ucq16bq16Sc", "n")
BUILTIN(__builtin_sve_svclz_s16_m, "q8Usq8Usq16bq8s", "n")
BUILTIN(__builtin_sve_svclz_s16_x, "q8Usq16bq8s", "n")
BUILTIN(__builtin_sve_svclz_s16_z, "q8Usq16bq8s", "n")
BUILTIN(__builtin_sve_svclz_s32_m, "q4Uiq4Uiq16bq4i", "n")
BUILTIN(__builtin_sve_svclz_s32_x, "q4Uiq16bq4i", "n")
BUILTIN(__builtin_sve_svclz_s32_z, "q4Uiq16bq4i", "n")
BUILTIN(__builtin_sve_svclz_s64_m, "q2UWiq2UWiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svclz_s64_x, "q2UWiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svclz_s64_z, "q2UWiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svclz_s8_m, "q16Ucq16Ucq16bq16Sc", "n")
BUILTIN(__builtin_sve_svclz_s8_x, "q16Ucq16bq16Sc", "n")
BUILTIN(__builtin_sve_svclz_s8_z, "q16Ucq16bq16Sc", "n")
BUILTIN(__builtin_sve_svclz_u16_m, "q8Usq8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svclz_u16_x, "q8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svclz_u16_z, "q8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svclz_u32_m, "q4Uiq4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svclz_u32_x, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svclz_u32_z, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svclz_u64_m, "q2UWiq2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svclz_u64_x, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svclz_u64_z, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svclz_u8_m, "q16Ucq16Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svclz_u8_x, "q16Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svclz_u8_z, "q16Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svcmla_f16_m, "q8hq16bq8hq8hq8hIUWi", "n")
BUILTIN(__builtin_sve_svcmla_f16_x, "q8hq16bq8hq8hq8hIUWi", "n")
BUILTIN(__builtin_sve_svcmla_f16_z, "q8hq16bq8hq8hq8hIUWi", "n")
BUILTIN(__builtin_sve_svcmla_f32_m, "q4fq16bq4fq4fq4fIUWi", "n")
BUILTIN(__builtin_sve_svcmla_f32_x, "q4fq16bq4fq4fq4fIUWi", "n")
BUILTIN(__builtin_sve_svcmla_f32_z, "q4fq16bq4fq4fq4fIUWi", "n")
BUILTIN(__builtin_sve_svcmla_f64_m, "q2dq16bq2dq2dq2dIUWi", "n")
BUILTIN(__builtin_sve_svcmla_f64_x, "q2dq16bq2dq2dq2dIUWi", "n")
BUILTIN(__builtin_sve_svcmla_f64_z, "q2dq16bq2dq2dq2dIUWi", "n")
BUILTIN(__builtin_sve_svcmla_lane_f16, "q8hq8hq8hq8hIUWiIUWi", "n")
BUILTIN(__builtin_sve_svcmla_lane_f32, "q4fq4fq4fq4fIUWiIUWi", "n")
BUILTIN(__builtin_sve_svcmla_lane_s16, "q8sq8sq8sq8sIUWiIUWi", "n")
BUILTIN(__builtin_sve_svcmla_lane_s32, "q4iq4iq4iq4iIUWiIUWi", "n")
BUILTIN(__builtin_sve_svcmla_lane_u16, "q8Usq8Usq8Usq8UsIUWiIUWi", "n")
BUILTIN(__builtin_sve_svcmla_lane_u32, "q4Uiq4Uiq4Uiq4UiIUWiIUWi", "n")
BUILTIN(__builtin_sve_svcmla_s16, "q8sq8sq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svcmla_s32, "q4iq4iq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svcmla_s64, "q2Wiq2Wiq2Wiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svcmla_s8, "q16Scq16Scq16Scq16ScIUWi", "n")
BUILTIN(__builtin_sve_svcmla_u16, "q8Usq8Usq8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svcmla_u32, "q4Uiq4Uiq4Uiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svcmla_u64, "q2UWiq2UWiq2UWiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svcmla_u8, "q16Ucq16Ucq16Ucq16UcIUWi", "n")
BUILTIN(__builtin_sve_svcmpeq_f16, "q16bq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svcmpeq_f32, "q16bq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svcmpeq_f64, "q16bq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svcmpeq_n_f16, "q16bq16bq8hh", "n")
BUILTIN(__builtin_sve_svcmpeq_n_f32, "q16bq16bq4ff", "n")
BUILTIN(__builtin_sve_svcmpeq_n_f64, "q16bq16bq2dd", "n")
BUILTIN(__builtin_sve_svcmpeq_n_s16, "q16bq16bq8ss", "n")
BUILTIN(__builtin_sve_svcmpeq_n_s32, "q16bq16bq4ii", "n")
BUILTIN(__builtin_sve_svcmpeq_n_s64, "q16bq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svcmpeq_n_s8, "q16bq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svcmpeq_n_u16, "q16bq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svcmpeq_n_u32, "q16bq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svcmpeq_n_u64, "q16bq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svcmpeq_n_u8, "q16bq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svcmpeq_s16, "q16bq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svcmpeq_s32, "q16bq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svcmpeq_s64, "q16bq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svcmpeq_s8, "q16bq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svcmpeq_u16, "q16bq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svcmpeq_u32, "q16bq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svcmpeq_u64, "q16bq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svcmpeq_u8, "q16bq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svcmpeq_wide_n_s16, "q16bq16bq8sWi", "n")
BUILTIN(__builtin_sve_svcmpeq_wide_n_s32, "q16bq16bq4iWi", "n")
BUILTIN(__builtin_sve_svcmpeq_wide_n_s8, "q16bq16bq16ScWi", "n")
BUILTIN(__builtin_sve_svcmpeq_wide_s16, "q16bq16bq8sq2Wi", "n")
BUILTIN(__builtin_sve_svcmpeq_wide_s32, "q16bq16bq4iq2Wi", "n")
BUILTIN(__builtin_sve_svcmpeq_wide_s8, "q16bq16bq16Scq2Wi", "n")
BUILTIN(__builtin_sve_svcmpge_f16, "q16bq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svcmpge_f32, "q16bq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svcmpge_f64, "q16bq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svcmpge_n_f16, "q16bq16bq8hh", "n")
BUILTIN(__builtin_sve_svcmpge_n_f32, "q16bq16bq4ff", "n")
BUILTIN(__builtin_sve_svcmpge_n_f64, "q16bq16bq2dd", "n")
BUILTIN(__builtin_sve_svcmpge_n_s16, "q16bq16bq8ss", "n")
BUILTIN(__builtin_sve_svcmpge_n_s32, "q16bq16bq4ii", "n")
BUILTIN(__builtin_sve_svcmpge_n_s64, "q16bq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svcmpge_n_s8, "q16bq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svcmpge_n_u16, "q16bq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svcmpge_n_u32, "q16bq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svcmpge_n_u64, "q16bq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svcmpge_n_u8, "q16bq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svcmpge_s16, "q16bq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svcmpge_s32, "q16bq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svcmpge_s64, "q16bq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svcmpge_s8, "q16bq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svcmpge_u16, "q16bq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svcmpge_u32, "q16bq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svcmpge_u64, "q16bq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svcmpge_u8, "q16bq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svcmpge_wide_n_s16, "q16bq16bq8sWi", "n")
BUILTIN(__builtin_sve_svcmpge_wide_n_s32, "q16bq16bq4iWi", "n")
BUILTIN(__builtin_sve_svcmpge_wide_n_s8, "q16bq16bq16ScWi", "n")
BUILTIN(__builtin_sve_svcmpge_wide_n_u16, "q16bq16bq8UsUWi", "n")
BUILTIN(__builtin_sve_svcmpge_wide_n_u32, "q16bq16bq4UiUWi", "n")
BUILTIN(__builtin_sve_svcmpge_wide_n_u8, "q16bq16bq16UcUWi", "n")
BUILTIN(__builtin_sve_svcmpge_wide_s16, "q16bq16bq8sq2Wi", "n")
BUILTIN(__builtin_sve_svcmpge_wide_s32, "q16bq16bq4iq2Wi", "n")
BUILTIN(__builtin_sve_svcmpge_wide_s8, "q16bq16bq16Scq2Wi", "n")
BUILTIN(__builtin_sve_svcmpge_wide_u16, "q16bq16bq8Usq2UWi", "n")
BUILTIN(__builtin_sve_svcmpge_wide_u32, "q16bq16bq4Uiq2UWi", "n")
BUILTIN(__builtin_sve_svcmpge_wide_u8, "q16bq16bq16Ucq2UWi", "n")
BUILTIN(__builtin_sve_svcmpgt_f16, "q16bq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svcmpgt_f32, "q16bq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svcmpgt_f64, "q16bq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svcmpgt_n_f16, "q16bq16bq8hh", "n")
BUILTIN(__builtin_sve_svcmpgt_n_f32, "q16bq16bq4ff", "n")
BUILTIN(__builtin_sve_svcmpgt_n_f64, "q16bq16bq2dd", "n")
BUILTIN(__builtin_sve_svcmpgt_n_s16, "q16bq16bq8ss", "n")
BUILTIN(__builtin_sve_svcmpgt_n_s32, "q16bq16bq4ii", "n")
BUILTIN(__builtin_sve_svcmpgt_n_s64, "q16bq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svcmpgt_n_s8, "q16bq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svcmpgt_n_u16, "q16bq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svcmpgt_n_u32, "q16bq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svcmpgt_n_u64, "q16bq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svcmpgt_n_u8, "q16bq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svcmpgt_s16, "q16bq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svcmpgt_s32, "q16bq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svcmpgt_s64, "q16bq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svcmpgt_s8, "q16bq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svcmpgt_u16, "q16bq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svcmpgt_u32, "q16bq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svcmpgt_u64, "q16bq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svcmpgt_u8, "q16bq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svcmpgt_wide_n_s16, "q16bq16bq8sWi", "n")
BUILTIN(__builtin_sve_svcmpgt_wide_n_s32, "q16bq16bq4iWi", "n")
BUILTIN(__builtin_sve_svcmpgt_wide_n_s8, "q16bq16bq16ScWi", "n")
BUILTIN(__builtin_sve_svcmpgt_wide_n_u16, "q16bq16bq8UsUWi", "n")
BUILTIN(__builtin_sve_svcmpgt_wide_n_u32, "q16bq16bq4UiUWi", "n")
BUILTIN(__builtin_sve_svcmpgt_wide_n_u8, "q16bq16bq16UcUWi", "n")
BUILTIN(__builtin_sve_svcmpgt_wide_s16, "q16bq16bq8sq2Wi", "n")
BUILTIN(__builtin_sve_svcmpgt_wide_s32, "q16bq16bq4iq2Wi", "n")
BUILTIN(__builtin_sve_svcmpgt_wide_s8, "q16bq16bq16Scq2Wi", "n")
BUILTIN(__builtin_sve_svcmpgt_wide_u16, "q16bq16bq8Usq2UWi", "n")
BUILTIN(__builtin_sve_svcmpgt_wide_u32, "q16bq16bq4Uiq2UWi", "n")
BUILTIN(__builtin_sve_svcmpgt_wide_u8, "q16bq16bq16Ucq2UWi", "n")
BUILTIN(__builtin_sve_svcmple_f16, "q16bq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svcmple_f32, "q16bq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svcmple_f64, "q16bq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svcmple_n_f16, "q16bq16bq8hh", "n")
BUILTIN(__builtin_sve_svcmple_n_f32, "q16bq16bq4ff", "n")
BUILTIN(__builtin_sve_svcmple_n_f64, "q16bq16bq2dd", "n")
BUILTIN(__builtin_sve_svcmple_n_s16, "q16bq16bq8ss", "n")
BUILTIN(__builtin_sve_svcmple_n_s32, "q16bq16bq4ii", "n")
BUILTIN(__builtin_sve_svcmple_n_s64, "q16bq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svcmple_n_s8, "q16bq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svcmple_n_u16, "q16bq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svcmple_n_u32, "q16bq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svcmple_n_u64, "q16bq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svcmple_n_u8, "q16bq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svcmple_s16, "q16bq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svcmple_s32, "q16bq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svcmple_s64, "q16bq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svcmple_s8, "q16bq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svcmple_u16, "q16bq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svcmple_u32, "q16bq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svcmple_u64, "q16bq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svcmple_u8, "q16bq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svcmple_wide_n_s16, "q16bq16bq8sWi", "n")
BUILTIN(__builtin_sve_svcmple_wide_n_s32, "q16bq16bq4iWi", "n")
BUILTIN(__builtin_sve_svcmple_wide_n_s8, "q16bq16bq16ScWi", "n")
BUILTIN(__builtin_sve_svcmple_wide_n_u16, "q16bq16bq8UsUWi", "n")
BUILTIN(__builtin_sve_svcmple_wide_n_u32, "q16bq16bq4UiUWi", "n")
BUILTIN(__builtin_sve_svcmple_wide_n_u8, "q16bq16bq16UcUWi", "n")
BUILTIN(__builtin_sve_svcmple_wide_s16, "q16bq16bq8sq2Wi", "n")
BUILTIN(__builtin_sve_svcmple_wide_s32, "q16bq16bq4iq2Wi", "n")
BUILTIN(__builtin_sve_svcmple_wide_s8, "q16bq16bq16Scq2Wi", "n")
BUILTIN(__builtin_sve_svcmple_wide_u16, "q16bq16bq8Usq2UWi", "n")
BUILTIN(__builtin_sve_svcmple_wide_u32, "q16bq16bq4Uiq2UWi", "n")
BUILTIN(__builtin_sve_svcmple_wide_u8, "q16bq16bq16Ucq2UWi", "n")
BUILTIN(__builtin_sve_svcmplt_f16, "q16bq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svcmplt_f32, "q16bq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svcmplt_f64, "q16bq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svcmplt_n_f16, "q16bq16bq8hh", "n")
BUILTIN(__builtin_sve_svcmplt_n_f32, "q16bq16bq4ff", "n")
BUILTIN(__builtin_sve_svcmplt_n_f64, "q16bq16bq2dd", "n")
BUILTIN(__builtin_sve_svcmplt_n_s16, "q16bq16bq8ss", "n")
BUILTIN(__builtin_sve_svcmplt_n_s32, "q16bq16bq4ii", "n")
BUILTIN(__builtin_sve_svcmplt_n_s64, "q16bq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svcmplt_n_s8, "q16bq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svcmplt_n_u16, "q16bq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svcmplt_n_u32, "q16bq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svcmplt_n_u64, "q16bq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svcmplt_n_u8, "q16bq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svcmplt_s16, "q16bq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svcmplt_s32, "q16bq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svcmplt_s64, "q16bq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svcmplt_s8, "q16bq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svcmplt_u16, "q16bq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svcmplt_u32, "q16bq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svcmplt_u64, "q16bq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svcmplt_u8, "q16bq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svcmplt_wide_n_s16, "q16bq16bq8sWi", "n")
BUILTIN(__builtin_sve_svcmplt_wide_n_s32, "q16bq16bq4iWi", "n")
BUILTIN(__builtin_sve_svcmplt_wide_n_s8, "q16bq16bq16ScWi", "n")
BUILTIN(__builtin_sve_svcmplt_wide_n_u16, "q16bq16bq8UsUWi", "n")
BUILTIN(__builtin_sve_svcmplt_wide_n_u32, "q16bq16bq4UiUWi", "n")
BUILTIN(__builtin_sve_svcmplt_wide_n_u8, "q16bq16bq16UcUWi", "n")
BUILTIN(__builtin_sve_svcmplt_wide_s16, "q16bq16bq8sq2Wi", "n")
BUILTIN(__builtin_sve_svcmplt_wide_s32, "q16bq16bq4iq2Wi", "n")
BUILTIN(__builtin_sve_svcmplt_wide_s8, "q16bq16bq16Scq2Wi", "n")
BUILTIN(__builtin_sve_svcmplt_wide_u16, "q16bq16bq8Usq2UWi", "n")
BUILTIN(__builtin_sve_svcmplt_wide_u32, "q16bq16bq4Uiq2UWi", "n")
BUILTIN(__builtin_sve_svcmplt_wide_u8, "q16bq16bq16Ucq2UWi", "n")
BUILTIN(__builtin_sve_svcmpne_f16, "q16bq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svcmpne_f32, "q16bq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svcmpne_f64, "q16bq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svcmpne_n_f16, "q16bq16bq8hh", "n")
BUILTIN(__builtin_sve_svcmpne_n_f32, "q16bq16bq4ff", "n")
BUILTIN(__builtin_sve_svcmpne_n_f64, "q16bq16bq2dd", "n")
BUILTIN(__builtin_sve_svcmpne_n_s16, "q16bq16bq8ss", "n")
BUILTIN(__builtin_sve_svcmpne_n_s32, "q16bq16bq4ii", "n")
BUILTIN(__builtin_sve_svcmpne_n_s64, "q16bq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svcmpne_n_s8, "q16bq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svcmpne_n_u16, "q16bq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svcmpne_n_u32, "q16bq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svcmpne_n_u64, "q16bq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svcmpne_n_u8, "q16bq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svcmpne_s16, "q16bq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svcmpne_s32, "q16bq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svcmpne_s64, "q16bq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svcmpne_s8, "q16bq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svcmpne_u16, "q16bq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svcmpne_u32, "q16bq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svcmpne_u64, "q16bq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svcmpne_u8, "q16bq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svcmpne_wide_n_s16, "q16bq16bq8sWi", "n")
BUILTIN(__builtin_sve_svcmpne_wide_n_s32, "q16bq16bq4iWi", "n")
BUILTIN(__builtin_sve_svcmpne_wide_n_s8, "q16bq16bq16ScWi", "n")
BUILTIN(__builtin_sve_svcmpne_wide_s16, "q16bq16bq8sq2Wi", "n")
BUILTIN(__builtin_sve_svcmpne_wide_s32, "q16bq16bq4iq2Wi", "n")
BUILTIN(__builtin_sve_svcmpne_wide_s8, "q16bq16bq16Scq2Wi", "n")
BUILTIN(__builtin_sve_svcmpuo_f16, "q16bq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svcmpuo_f32, "q16bq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svcmpuo_f64, "q16bq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svcmpuo_n_f16, "q16bq16bq8hh", "n")
BUILTIN(__builtin_sve_svcmpuo_n_f32, "q16bq16bq4ff", "n")
BUILTIN(__builtin_sve_svcmpuo_n_f64, "q16bq16bq2dd", "n")
BUILTIN(__builtin_sve_svcnot_s16_m, "q8sq8sq16bq8s", "n")
BUILTIN(__builtin_sve_svcnot_s16_x, "q8sq16bq8s", "n")
BUILTIN(__builtin_sve_svcnot_s16_z, "q8sq16bq8s", "n")
BUILTIN(__builtin_sve_svcnot_s32_m, "q4iq4iq16bq4i", "n")
BUILTIN(__builtin_sve_svcnot_s32_x, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svcnot_s32_z, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svcnot_s64_m, "q2Wiq2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svcnot_s64_x, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svcnot_s64_z, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svcnot_s8_m, "q16Scq16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svcnot_s8_x, "q16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svcnot_s8_z, "q16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svcnot_u16_m, "q8Usq8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svcnot_u16_x, "q8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svcnot_u16_z, "q8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svcnot_u32_m, "q4Uiq4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svcnot_u32_x, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svcnot_u32_z, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svcnot_u64_m, "q2UWiq2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svcnot_u64_x, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svcnot_u64_z, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svcnot_u8_m, "q16Ucq16Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svcnot_u8_x, "q16Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svcnot_u8_z, "q16Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svcnt_bf16_m, "q8Usq8Usq16bq8y", "n")
BUILTIN(__builtin_sve_svcnt_bf16_x, "q8Usq16bq8y", "n")
BUILTIN(__builtin_sve_svcnt_bf16_z, "q8Usq16bq8y", "n")
BUILTIN(__builtin_sve_svcnt_f16_m, "q8Usq8Usq16bq8h", "n")
BUILTIN(__builtin_sve_svcnt_f16_x, "q8Usq16bq8h", "n")
BUILTIN(__builtin_sve_svcnt_f16_z, "q8Usq16bq8h", "n")
BUILTIN(__builtin_sve_svcnt_f32_m, "q4Uiq4Uiq16bq4f", "n")
BUILTIN(__builtin_sve_svcnt_f32_x, "q4Uiq16bq4f", "n")
BUILTIN(__builtin_sve_svcnt_f32_z, "q4Uiq16bq4f", "n")
BUILTIN(__builtin_sve_svcnt_f64_m, "q2UWiq2UWiq16bq2d", "n")
BUILTIN(__builtin_sve_svcnt_f64_x, "q2UWiq16bq2d", "n")
BUILTIN(__builtin_sve_svcnt_f64_z, "q2UWiq16bq2d", "n")
BUILTIN(__builtin_sve_svcnt_s16_m, "q8Usq8Usq16bq8s", "n")
BUILTIN(__builtin_sve_svcnt_s16_x, "q8Usq16bq8s", "n")
BUILTIN(__builtin_sve_svcnt_s16_z, "q8Usq16bq8s", "n")
BUILTIN(__builtin_sve_svcnt_s32_m, "q4Uiq4Uiq16bq4i", "n")
BUILTIN(__builtin_sve_svcnt_s32_x, "q4Uiq16bq4i", "n")
BUILTIN(__builtin_sve_svcnt_s32_z, "q4Uiq16bq4i", "n")
BUILTIN(__builtin_sve_svcnt_s64_m, "q2UWiq2UWiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svcnt_s64_x, "q2UWiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svcnt_s64_z, "q2UWiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svcnt_s8_m, "q16Ucq16Ucq16bq16Sc", "n")
BUILTIN(__builtin_sve_svcnt_s8_x, "q16Ucq16bq16Sc", "n")
BUILTIN(__builtin_sve_svcnt_s8_z, "q16Ucq16bq16Sc", "n")
BUILTIN(__builtin_sve_svcnt_u16_m, "q8Usq8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svcnt_u16_x, "q8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svcnt_u16_z, "q8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svcnt_u32_m, "q4Uiq4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svcnt_u32_x, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svcnt_u32_z, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svcnt_u64_m, "q2UWiq2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svcnt_u64_x, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svcnt_u64_z, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svcnt_u8_m, "q16Ucq16Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svcnt_u8_x, "q16Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svcnt_u8_z, "q16Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svcntb, "UWi", "n")
BUILTIN(__builtin_sve_svcntb_pat, "UWiIi", "n")
BUILTIN(__builtin_sve_svcntd, "UWi", "n")
BUILTIN(__builtin_sve_svcntd_pat, "UWiIi", "n")
BUILTIN(__builtin_sve_svcnth, "UWi", "n")
BUILTIN(__builtin_sve_svcnth_pat, "UWiIi", "n")
BUILTIN(__builtin_sve_svcntp_b16, "UWiq16bq16b", "n")
BUILTIN(__builtin_sve_svcntp_b32, "UWiq16bq16b", "n")
BUILTIN(__builtin_sve_svcntp_b64, "UWiq16bq16b", "n")
BUILTIN(__builtin_sve_svcntp_b8, "UWiq16bq16b", "n")
BUILTIN(__builtin_sve_svcntw, "UWi", "n")
BUILTIN(__builtin_sve_svcntw_pat, "UWiIi", "n")
BUILTIN(__builtin_sve_svcompact_f32, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svcompact_f64, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svcompact_s32, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svcompact_s64, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svcompact_u32, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svcompact_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svcreate2_bf16, "q16yq8yq8y", "n")
BUILTIN(__builtin_sve_svcreate2_f16, "q16hq8hq8h", "n")
BUILTIN(__builtin_sve_svcreate2_f32, "q8fq4fq4f", "n")
BUILTIN(__builtin_sve_svcreate2_f64, "q4dq2dq2d", "n")
BUILTIN(__builtin_sve_svcreate2_s16, "q16sq8sq8s", "n")
BUILTIN(__builtin_sve_svcreate2_s32, "q8iq4iq4i", "n")
BUILTIN(__builtin_sve_svcreate2_s64, "q4Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svcreate2_s8, "q32Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svcreate2_u16, "q16Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svcreate2_u32, "q8Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svcreate2_u64, "q4UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svcreate2_u8, "q32Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svcreate3_bf16, "q24yq8yq8yq8y", "n")
BUILTIN(__builtin_sve_svcreate3_f16, "q24hq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svcreate3_f32, "q12fq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svcreate3_f64, "q6dq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svcreate3_s16, "q24sq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svcreate3_s32, "q12iq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svcreate3_s64, "q6Wiq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svcreate3_s8, "q48Scq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svcreate3_u16, "q24Usq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svcreate3_u32, "q12Uiq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svcreate3_u64, "q6UWiq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svcreate3_u8, "q48Ucq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svcreate4_bf16, "q32yq8yq8yq8yq8y", "n")
BUILTIN(__builtin_sve_svcreate4_f16, "q32hq8hq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svcreate4_f32, "q16fq4fq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svcreate4_f64, "q8dq2dq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svcreate4_s16, "q32sq8sq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svcreate4_s32, "q16iq4iq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svcreate4_s64, "q8Wiq2Wiq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svcreate4_s8, "q64Scq16Scq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svcreate4_u16, "q32Usq8Usq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svcreate4_u32, "q16Uiq4Uiq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svcreate4_u64, "q8UWiq2UWiq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svcreate4_u8, "q64Ucq16Ucq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svcvt_bf16_f32_m, "q8yq8yq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_bf16_f32_x, "q8yq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_bf16_f32_z, "q8yq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_f16_f32_m, "q8hq8hq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_f16_f32_x, "q8hq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_f16_f32_z, "q8hq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_f16_f64_m, "q8hq8hq16bq2d", "n")
BUILTIN(__builtin_sve_svcvt_f16_f64_x, "q8hq16bq2d", "n")
BUILTIN(__builtin_sve_svcvt_f16_f64_z, "q8hq16bq2d", "n")
BUILTIN(__builtin_sve_svcvt_f16_s16_m, "q8hq8hq16bq8s", "n")
BUILTIN(__builtin_sve_svcvt_f16_s16_x, "q8hq16bq8s", "n")
BUILTIN(__builtin_sve_svcvt_f16_s16_z, "q8hq16bq8s", "n")
BUILTIN(__builtin_sve_svcvt_f16_s32_m, "q8hq8hq16bq4i", "n")
BUILTIN(__builtin_sve_svcvt_f16_s32_x, "q8hq16bq4i", "n")
BUILTIN(__builtin_sve_svcvt_f16_s32_z, "q8hq16bq4i", "n")
BUILTIN(__builtin_sve_svcvt_f16_s64_m, "q8hq8hq16bq2Wi", "n")
BUILTIN(__builtin_sve_svcvt_f16_s64_x, "q8hq16bq2Wi", "n")
BUILTIN(__builtin_sve_svcvt_f16_s64_z, "q8hq16bq2Wi", "n")
BUILTIN(__builtin_sve_svcvt_f16_u16_m, "q8hq8hq16bq8Us", "n")
BUILTIN(__builtin_sve_svcvt_f16_u16_x, "q8hq16bq8Us", "n")
BUILTIN(__builtin_sve_svcvt_f16_u16_z, "q8hq16bq8Us", "n")
BUILTIN(__builtin_sve_svcvt_f16_u32_m, "q8hq8hq16bq4Ui", "n")
BUILTIN(__builtin_sve_svcvt_f16_u32_x, "q8hq16bq4Ui", "n")
BUILTIN(__builtin_sve_svcvt_f16_u32_z, "q8hq16bq4Ui", "n")
BUILTIN(__builtin_sve_svcvt_f16_u64_m, "q8hq8hq16bq2UWi", "n")
BUILTIN(__builtin_sve_svcvt_f16_u64_x, "q8hq16bq2UWi", "n")
BUILTIN(__builtin_sve_svcvt_f16_u64_z, "q8hq16bq2UWi", "n")
BUILTIN(__builtin_sve_svcvt_f32_f16_m, "q4fq4fq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_f32_f16_x, "q4fq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_f32_f16_z, "q4fq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_f32_f64_m, "q4fq4fq16bq2d", "n")
BUILTIN(__builtin_sve_svcvt_f32_f64_x, "q4fq16bq2d", "n")
BUILTIN(__builtin_sve_svcvt_f32_f64_z, "q4fq16bq2d", "n")
BUILTIN(__builtin_sve_svcvt_f32_s32_m, "q4fq4fq16bq4i", "n")
BUILTIN(__builtin_sve_svcvt_f32_s32_x, "q4fq16bq4i", "n")
BUILTIN(__builtin_sve_svcvt_f32_s32_z, "q4fq16bq4i", "n")
BUILTIN(__builtin_sve_svcvt_f32_s64_m, "q4fq4fq16bq2Wi", "n")
BUILTIN(__builtin_sve_svcvt_f32_s64_x, "q4fq16bq2Wi", "n")
BUILTIN(__builtin_sve_svcvt_f32_s64_z, "q4fq16bq2Wi", "n")
BUILTIN(__builtin_sve_svcvt_f32_u32_m, "q4fq4fq16bq4Ui", "n")
BUILTIN(__builtin_sve_svcvt_f32_u32_x, "q4fq16bq4Ui", "n")
BUILTIN(__builtin_sve_svcvt_f32_u32_z, "q4fq16bq4Ui", "n")
BUILTIN(__builtin_sve_svcvt_f32_u64_m, "q4fq4fq16bq2UWi", "n")
BUILTIN(__builtin_sve_svcvt_f32_u64_x, "q4fq16bq2UWi", "n")
BUILTIN(__builtin_sve_svcvt_f32_u64_z, "q4fq16bq2UWi", "n")
BUILTIN(__builtin_sve_svcvt_f64_f16_m, "q2dq2dq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_f64_f16_x, "q2dq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_f64_f16_z, "q2dq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_f64_f32_m, "q2dq2dq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_f64_f32_x, "q2dq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_f64_f32_z, "q2dq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_f64_s32_m, "q2dq2dq16bq4i", "n")
BUILTIN(__builtin_sve_svcvt_f64_s32_x, "q2dq16bq4i", "n")
BUILTIN(__builtin_sve_svcvt_f64_s32_z, "q2dq16bq4i", "n")
BUILTIN(__builtin_sve_svcvt_f64_s64_m, "q2dq2dq16bq2Wi", "n")
BUILTIN(__builtin_sve_svcvt_f64_s64_x, "q2dq16bq2Wi", "n")
BUILTIN(__builtin_sve_svcvt_f64_s64_z, "q2dq16bq2Wi", "n")
BUILTIN(__builtin_sve_svcvt_f64_u32_m, "q2dq2dq16bq4Ui", "n")
BUILTIN(__builtin_sve_svcvt_f64_u32_x, "q2dq16bq4Ui", "n")
BUILTIN(__builtin_sve_svcvt_f64_u32_z, "q2dq16bq4Ui", "n")
BUILTIN(__builtin_sve_svcvt_f64_u64_m, "q2dq2dq16bq2UWi", "n")
BUILTIN(__builtin_sve_svcvt_f64_u64_x, "q2dq16bq2UWi", "n")
BUILTIN(__builtin_sve_svcvt_f64_u64_z, "q2dq16bq2UWi", "n")
BUILTIN(__builtin_sve_svcvt_s16_f16_m, "q8sq8sq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_s16_f16_x, "q8sq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_s16_f16_z, "q8sq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_s32_f16_m, "q4iq4iq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_s32_f16_x, "q4iq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_s32_f16_z, "q4iq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_s32_f32_m, "q4iq4iq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_s32_f32_x, "q4iq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_s32_f32_z, "q4iq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_s32_f64_m, "q4iq4iq16bq2d", "n")
BUILTIN(__builtin_sve_svcvt_s32_f64_x, "q4iq16bq2d", "n")
BUILTIN(__builtin_sve_svcvt_s32_f64_z, "q4iq16bq2d", "n")
BUILTIN(__builtin_sve_svcvt_s64_f16_m, "q2Wiq2Wiq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_s64_f16_x, "q2Wiq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_s64_f16_z, "q2Wiq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_s64_f32_m, "q2Wiq2Wiq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_s64_f32_x, "q2Wiq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_s64_f32_z, "q2Wiq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_s64_f64_m, "q2Wiq2Wiq16bq2d", "n")
BUILTIN(__builtin_sve_svcvt_s64_f64_x, "q2Wiq16bq2d", "n")
BUILTIN(__builtin_sve_svcvt_s64_f64_z, "q2Wiq16bq2d", "n")
BUILTIN(__builtin_sve_svcvt_u16_f16_m, "q8Usq8Usq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_u16_f16_x, "q8Usq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_u16_f16_z, "q8Usq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_u32_f16_m, "q4Uiq4Uiq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_u32_f16_x, "q4Uiq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_u32_f16_z, "q4Uiq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_u32_f32_m, "q4Uiq4Uiq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_u32_f32_x, "q4Uiq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_u32_f32_z, "q4Uiq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_u32_f64_m, "q4Uiq4Uiq16bq2d", "n")
BUILTIN(__builtin_sve_svcvt_u32_f64_x, "q4Uiq16bq2d", "n")
BUILTIN(__builtin_sve_svcvt_u32_f64_z, "q4Uiq16bq2d", "n")
BUILTIN(__builtin_sve_svcvt_u64_f16_m, "q2UWiq2UWiq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_u64_f16_x, "q2UWiq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_u64_f16_z, "q2UWiq16bq8h", "n")
BUILTIN(__builtin_sve_svcvt_u64_f32_m, "q2UWiq2UWiq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_u64_f32_x, "q2UWiq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_u64_f32_z, "q2UWiq16bq4f", "n")
BUILTIN(__builtin_sve_svcvt_u64_f64_m, "q2UWiq2UWiq16bq2d", "n")
BUILTIN(__builtin_sve_svcvt_u64_f64_x, "q2UWiq16bq2d", "n")
BUILTIN(__builtin_sve_svcvt_u64_f64_z, "q2UWiq16bq2d", "n")
BUILTIN(__builtin_sve_svcvtlt_f32_f16_m, "q4fq4fq16bq8h", "n")
BUILTIN(__builtin_sve_svcvtlt_f32_f16_x, "q4fq16bq8h", "n")
BUILTIN(__builtin_sve_svcvtlt_f64_f32_m, "q2dq2dq16bq4f", "n")
BUILTIN(__builtin_sve_svcvtlt_f64_f32_x, "q2dq16bq4f", "n")
BUILTIN(__builtin_sve_svcvtnt_bf16_f32_m, "q8yq8yq16bq4f", "n")
BUILTIN(__builtin_sve_svcvtnt_f16_f32_m, "q8hq8hq16bq4f", "n")
BUILTIN(__builtin_sve_svcvtnt_f32_f64_m, "q4fq4fq16bq2d", "n")
BUILTIN(__builtin_sve_svcvtx_f32_f64_m, "q4fq4fq16bq2d", "n")
BUILTIN(__builtin_sve_svcvtx_f32_f64_x, "q4fq16bq2d", "n")
BUILTIN(__builtin_sve_svcvtx_f32_f64_z, "q4fq16bq2d", "n")
BUILTIN(__builtin_sve_svcvtxnt_f32_f64_m, "q4fq4fq16bq2d", "n")
BUILTIN(__builtin_sve_svdiv_f16_m, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svdiv_f16_x, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svdiv_f16_z, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svdiv_f32_m, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svdiv_f32_x, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svdiv_f32_z, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svdiv_f64_m, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svdiv_f64_x, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svdiv_f64_z, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svdiv_n_f16_m, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svdiv_n_f16_x, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svdiv_n_f16_z, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svdiv_n_f32_m, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svdiv_n_f32_x, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svdiv_n_f32_z, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svdiv_n_f64_m, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svdiv_n_f64_x, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svdiv_n_f64_z, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svdiv_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svdiv_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svdiv_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svdiv_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svdiv_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svdiv_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svdiv_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svdiv_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svdiv_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svdiv_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svdiv_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svdiv_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svdiv_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svdiv_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svdiv_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svdiv_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svdiv_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svdiv_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svdiv_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svdiv_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svdiv_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svdiv_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svdiv_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svdiv_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svdivr_f16_m, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svdivr_f16_x, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svdivr_f16_z, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svdivr_f32_m, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svdivr_f32_x, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svdivr_f32_z, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svdivr_f64_m, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svdivr_f64_x, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svdivr_f64_z, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svdivr_n_f16_m, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svdivr_n_f16_x, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svdivr_n_f16_z, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svdivr_n_f32_m, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svdivr_n_f32_x, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svdivr_n_f32_z, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svdivr_n_f64_m, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svdivr_n_f64_x, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svdivr_n_f64_z, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svdivr_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svdivr_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svdivr_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svdivr_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svdivr_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svdivr_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svdivr_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svdivr_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svdivr_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svdivr_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svdivr_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svdivr_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svdivr_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svdivr_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svdivr_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svdivr_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svdivr_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svdivr_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svdivr_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svdivr_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svdivr_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svdivr_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svdivr_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svdivr_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svdot_lane_s32, "q4iq4iq16Scq16ScIUWi", "n")
BUILTIN(__builtin_sve_svdot_lane_s64, "q2Wiq2Wiq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svdot_lane_u32, "q4Uiq4Uiq16Ucq16UcIUWi", "n")
BUILTIN(__builtin_sve_svdot_lane_u64, "q2UWiq2UWiq8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svdot_n_s32, "q4iq4iq16ScSc", "n")
BUILTIN(__builtin_sve_svdot_n_s64, "q2Wiq2Wiq8ss", "n")
BUILTIN(__builtin_sve_svdot_n_u32, "q4Uiq4Uiq16UcUc", "n")
BUILTIN(__builtin_sve_svdot_n_u64, "q2UWiq2UWiq8UsUs", "n")
BUILTIN(__builtin_sve_svdot_s32, "q4iq4iq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svdot_s64, "q2Wiq2Wiq8sq8s", "n")
BUILTIN(__builtin_sve_svdot_u32, "q4Uiq4Uiq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svdot_u64, "q2UWiq2UWiq8Usq8Us", "n")
BUILTIN(__builtin_sve_svdup_lane_bf16, "q8yq8yUs", "n")
BUILTIN(__builtin_sve_svdup_lane_f16, "q8hq8hUs", "n")
BUILTIN(__builtin_sve_svdup_lane_f32, "q4fq4fUi", "n")
BUILTIN(__builtin_sve_svdup_lane_f64, "q2dq2dUWi", "n")
BUILTIN(__builtin_sve_svdup_lane_s16, "q8sq8sUs", "n")
BUILTIN(__builtin_sve_svdup_lane_s32, "q4iq4iUi", "n")
BUILTIN(__builtin_sve_svdup_lane_s64, "q2Wiq2WiUWi", "n")
BUILTIN(__builtin_sve_svdup_lane_s8, "q16Scq16ScUc", "n")
BUILTIN(__builtin_sve_svdup_lane_u16, "q8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svdup_lane_u32, "q4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svdup_lane_u64, "q2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svdup_lane_u8, "q16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svdup_n_b16, "q16bb", "n")
BUILTIN(__builtin_sve_svdup_n_b32, "q16bb", "n")
BUILTIN(__builtin_sve_svdup_n_b64, "q16bb", "n")
BUILTIN(__builtin_sve_svdup_n_b8, "q16bb", "n")
BUILTIN(__builtin_sve_svdup_n_bf16, "q8yy", "n")
BUILTIN(__builtin_sve_svdup_n_bf16_m, "q8yq8yq16by", "n")
BUILTIN(__builtin_sve_svdup_n_bf16_x, "q8yq16by", "n")
BUILTIN(__builtin_sve_svdup_n_bf16_z, "q8yq16by", "n")
BUILTIN(__builtin_sve_svdup_n_f16, "q8hh", "n")
BUILTIN(__builtin_sve_svdup_n_f16_m, "q8hq8hq16bh", "n")
BUILTIN(__builtin_sve_svdup_n_f16_x, "q8hq16bh", "n")
BUILTIN(__builtin_sve_svdup_n_f16_z, "q8hq16bh", "n")
BUILTIN(__builtin_sve_svdup_n_f32, "q4ff", "n")
BUILTIN(__builtin_sve_svdup_n_f32_m, "q4fq4fq16bf", "n")
BUILTIN(__builtin_sve_svdup_n_f32_x, "q4fq16bf", "n")
BUILTIN(__builtin_sve_svdup_n_f32_z, "q4fq16bf", "n")
BUILTIN(__builtin_sve_svdup_n_f64, "q2dd", "n")
BUILTIN(__builtin_sve_svdup_n_f64_m, "q2dq2dq16bd", "n")
BUILTIN(__builtin_sve_svdup_n_f64_x, "q2dq16bd", "n")
BUILTIN(__builtin_sve_svdup_n_f64_z, "q2dq16bd", "n")
BUILTIN(__builtin_sve_svdup_n_s16, "q8ss", "n")
BUILTIN(__builtin_sve_svdup_n_s16_m, "q8sq8sq16bs", "n")
BUILTIN(__builtin_sve_svdup_n_s16_x, "q8sq16bs", "n")
BUILTIN(__builtin_sve_svdup_n_s16_z, "q8sq16bs", "n")
BUILTIN(__builtin_sve_svdup_n_s32, "q4ii", "n")
BUILTIN(__builtin_sve_svdup_n_s32_m, "q4iq4iq16bi", "n")
BUILTIN(__builtin_sve_svdup_n_s32_x, "q4iq16bi", "n")
BUILTIN(__builtin_sve_svdup_n_s32_z, "q4iq16bi", "n")
BUILTIN(__builtin_sve_svdup_n_s64, "q2WiWi", "n")
BUILTIN(__builtin_sve_svdup_n_s64_m, "q2Wiq2Wiq16bWi", "n")
BUILTIN(__builtin_sve_svdup_n_s64_x, "q2Wiq16bWi", "n")
BUILTIN(__builtin_sve_svdup_n_s64_z, "q2Wiq16bWi", "n")
BUILTIN(__builtin_sve_svdup_n_s8, "q16ScSc", "n")
BUILTIN(__builtin_sve_svdup_n_s8_m, "q16Scq16Scq16bSc", "n")
BUILTIN(__builtin_sve_svdup_n_s8_x, "q16Scq16bSc", "n")
BUILTIN(__builtin_sve_svdup_n_s8_z, "q16Scq16bSc", "n")
BUILTIN(__builtin_sve_svdup_n_u16, "q8UsUs", "n")
BUILTIN(__builtin_sve_svdup_n_u16_m, "q8Usq8Usq16bUs", "n")
BUILTIN(__builtin_sve_svdup_n_u16_x, "q8Usq16bUs", "n")
BUILTIN(__builtin_sve_svdup_n_u16_z, "q8Usq16bUs", "n")
BUILTIN(__builtin_sve_svdup_n_u32, "q4UiUi", "n")
BUILTIN(__builtin_sve_svdup_n_u32_m, "q4Uiq4Uiq16bUi", "n")
BUILTIN(__builtin_sve_svdup_n_u32_x, "q4Uiq16bUi", "n")
BUILTIN(__builtin_sve_svdup_n_u32_z, "q4Uiq16bUi", "n")
BUILTIN(__builtin_sve_svdup_n_u64, "q2UWiUWi", "n")
BUILTIN(__builtin_sve_svdup_n_u64_m, "q2UWiq2UWiq16bUWi", "n")
BUILTIN(__builtin_sve_svdup_n_u64_x, "q2UWiq16bUWi", "n")
BUILTIN(__builtin_sve_svdup_n_u64_z, "q2UWiq16bUWi", "n")
BUILTIN(__builtin_sve_svdup_n_u8, "q16UcUc", "n")
BUILTIN(__builtin_sve_svdup_n_u8_m, "q16Ucq16Ucq16bUc", "n")
BUILTIN(__builtin_sve_svdup_n_u8_x, "q16Ucq16bUc", "n")
BUILTIN(__builtin_sve_svdup_n_u8_z, "q16Ucq16bUc", "n")
BUILTIN(__builtin_sve_svdupq_lane_bf16, "q8yq8yUWi", "n")
BUILTIN(__builtin_sve_svdupq_lane_f16, "q8hq8hUWi", "n")
BUILTIN(__builtin_sve_svdupq_lane_f32, "q4fq4fUWi", "n")
BUILTIN(__builtin_sve_svdupq_lane_f64, "q2dq2dUWi", "n")
BUILTIN(__builtin_sve_svdupq_lane_s16, "q8sq8sUWi", "n")
BUILTIN(__builtin_sve_svdupq_lane_s32, "q4iq4iUWi", "n")
BUILTIN(__builtin_sve_svdupq_lane_s64, "q2Wiq2WiUWi", "n")
BUILTIN(__builtin_sve_svdupq_lane_s8, "q16Scq16ScUWi", "n")
BUILTIN(__builtin_sve_svdupq_lane_u16, "q8Usq8UsUWi", "n")
BUILTIN(__builtin_sve_svdupq_lane_u32, "q4Uiq4UiUWi", "n")
BUILTIN(__builtin_sve_svdupq_lane_u64, "q2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svdupq_lane_u8, "q16Ucq16UcUWi", "n")
BUILTIN(__builtin_sve_svdupq_n_b16, "q16bbbbbbbbb", "n")
BUILTIN(__builtin_sve_svdupq_n_b32, "q16bbbbb", "n")
BUILTIN(__builtin_sve_svdupq_n_b64, "q16bbb", "n")
BUILTIN(__builtin_sve_svdupq_n_b8, "q16bbbbbbbbbbbbbbbbb", "n")
BUILTIN(__builtin_sve_svdupq_n_bf16, "q8yyyyyyyyy", "n")
BUILTIN(__builtin_sve_svdupq_n_f16, "q8hhhhhhhhh", "n")
BUILTIN(__builtin_sve_svdupq_n_f32, "q4fffff", "n")
BUILTIN(__builtin_sve_svdupq_n_f64, "q2ddd", "n")
BUILTIN(__builtin_sve_svdupq_n_s16, "q8sssssssss", "n")
BUILTIN(__builtin_sve_svdupq_n_s32, "q4iiiii", "n")
BUILTIN(__builtin_sve_svdupq_n_s64, "q2WiWiWi", "n")
BUILTIN(__builtin_sve_svdupq_n_s8, "q16ScScScScScScScScScScScScScScScScSc", "n")
BUILTIN(__builtin_sve_svdupq_n_u16, "q8UsUsUsUsUsUsUsUsUs", "n")
BUILTIN(__builtin_sve_svdupq_n_u32, "q4UiUiUiUiUi", "n")
BUILTIN(__builtin_sve_svdupq_n_u64, "q2UWiUWiUWi", "n")
BUILTIN(__builtin_sve_svdupq_n_u8, "q16UcUcUcUcUcUcUcUcUcUcUcUcUcUcUcUcUc", "n")
BUILTIN(__builtin_sve_sveor3_n_s16, "q8sq8sq8ss", "n")
BUILTIN(__builtin_sve_sveor3_n_s32, "q4iq4iq4ii", "n")
BUILTIN(__builtin_sve_sveor3_n_s64, "q2Wiq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_sveor3_n_s8, "q16Scq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_sveor3_n_u16, "q8Usq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_sveor3_n_u32, "q4Uiq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_sveor3_n_u64, "q2UWiq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_sveor3_n_u8, "q16Ucq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_sveor3_s16, "q8sq8sq8sq8s", "n")
BUILTIN(__builtin_sve_sveor3_s32, "q4iq4iq4iq4i", "n")
BUILTIN(__builtin_sve_sveor3_s64, "q2Wiq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_sveor3_s8, "q16Scq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_sveor3_u16, "q8Usq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_sveor3_u32, "q4Uiq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_sveor3_u64, "q2UWiq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_sveor3_u8, "q16Ucq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_sveor_b_z, "q16bq16bq16bq16b", "n")
BUILTIN(__builtin_sve_sveor_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_sveor_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_sveor_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_sveor_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_sveor_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_sveor_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_sveor_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_sveor_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_sveor_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_sveor_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_sveor_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_sveor_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_sveor_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_sveor_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_sveor_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_sveor_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_sveor_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_sveor_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_sveor_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_sveor_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_sveor_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_sveor_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_sveor_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_sveor_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_sveor_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_sveor_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_sveor_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_sveor_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_sveor_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_sveor_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_sveor_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_sveor_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_sveor_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_sveor_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_sveor_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_sveor_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_sveor_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_sveor_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_sveor_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_sveor_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_sveor_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_sveor_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_sveor_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_sveor_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_sveor_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_sveor_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_sveor_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_sveor_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_sveorbt_n_s16, "q8sq8sq8ss", "n")
BUILTIN(__builtin_sve_sveorbt_n_s32, "q4iq4iq4ii", "n")
BUILTIN(__builtin_sve_sveorbt_n_s64, "q2Wiq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_sveorbt_n_s8, "q16Scq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_sveorbt_n_u16, "q8Usq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_sveorbt_n_u32, "q4Uiq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_sveorbt_n_u64, "q2UWiq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_sveorbt_n_u8, "q16Ucq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_sveorbt_s16, "q8sq8sq8sq8s", "n")
BUILTIN(__builtin_sve_sveorbt_s32, "q4iq4iq4iq4i", "n")
BUILTIN(__builtin_sve_sveorbt_s64, "q2Wiq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_sveorbt_s8, "q16Scq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_sveorbt_u16, "q8Usq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_sveorbt_u32, "q4Uiq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_sveorbt_u64, "q2UWiq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_sveorbt_u8, "q16Ucq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_sveortb_n_s16, "q8sq8sq8ss", "n")
BUILTIN(__builtin_sve_sveortb_n_s32, "q4iq4iq4ii", "n")
BUILTIN(__builtin_sve_sveortb_n_s64, "q2Wiq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_sveortb_n_s8, "q16Scq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_sveortb_n_u16, "q8Usq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_sveortb_n_u32, "q4Uiq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_sveortb_n_u64, "q2UWiq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_sveortb_n_u8, "q16Ucq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_sveortb_s16, "q8sq8sq8sq8s", "n")
BUILTIN(__builtin_sve_sveortb_s32, "q4iq4iq4iq4i", "n")
BUILTIN(__builtin_sve_sveortb_s64, "q2Wiq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_sveortb_s8, "q16Scq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_sveortb_u16, "q8Usq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_sveortb_u32, "q4Uiq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_sveortb_u64, "q2UWiq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_sveortb_u8, "q16Ucq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_sveorv_s16, "sq16bq8s", "n")
BUILTIN(__builtin_sve_sveorv_s32, "iq16bq4i", "n")
BUILTIN(__builtin_sve_sveorv_s64, "Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_sveorv_s8, "Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_sveorv_u16, "Usq16bq8Us", "n")
BUILTIN(__builtin_sve_sveorv_u32, "Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_sveorv_u64, "UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_sveorv_u8, "Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svexpa_f16, "q8hq8Us", "n")
BUILTIN(__builtin_sve_svexpa_f32, "q4fq4Ui", "n")
BUILTIN(__builtin_sve_svexpa_f64, "q2dq2UWi", "n")
BUILTIN(__builtin_sve_svext_bf16, "q8yq8yq8yIUWi", "n")
BUILTIN(__builtin_sve_svext_f16, "q8hq8hq8hIUWi", "n")
BUILTIN(__builtin_sve_svext_f32, "q4fq4fq4fIUWi", "n")
BUILTIN(__builtin_sve_svext_f64, "q2dq2dq2dIUWi", "n")
BUILTIN(__builtin_sve_svext_s16, "q8sq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svext_s32, "q4iq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svext_s64, "q2Wiq2Wiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svext_s8, "q16Scq16Scq16ScIUWi", "n")
BUILTIN(__builtin_sve_svext_u16, "q8Usq8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svext_u32, "q4Uiq4Uiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svext_u64, "q2UWiq2UWiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svext_u8, "q16Ucq16Ucq16UcIUWi", "n")
BUILTIN(__builtin_sve_svextb_s16_m, "q8sq8sq16bq8s", "n")
BUILTIN(__builtin_sve_svextb_s16_x, "q8sq16bq8s", "n")
BUILTIN(__builtin_sve_svextb_s16_z, "q8sq16bq8s", "n")
BUILTIN(__builtin_sve_svextb_s32_m, "q4iq4iq16bq4i", "n")
BUILTIN(__builtin_sve_svextb_s32_x, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svextb_s32_z, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svextb_s64_m, "q2Wiq2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svextb_s64_x, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svextb_s64_z, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svextb_u16_m, "q8Usq8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svextb_u16_x, "q8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svextb_u16_z, "q8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svextb_u32_m, "q4Uiq4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svextb_u32_x, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svextb_u32_z, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svextb_u64_m, "q2UWiq2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svextb_u64_x, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svextb_u64_z, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svexth_s32_m, "q4iq4iq16bq4i", "n")
BUILTIN(__builtin_sve_svexth_s32_x, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svexth_s32_z, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svexth_s64_m, "q2Wiq2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svexth_s64_x, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svexth_s64_z, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svexth_u32_m, "q4Uiq4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svexth_u32_x, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svexth_u32_z, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svexth_u64_m, "q2UWiq2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svexth_u64_x, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svexth_u64_z, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svextw_s64_m, "q2Wiq2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svextw_s64_x, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svextw_s64_z, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svextw_u64_m, "q2UWiq2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svextw_u64_x, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svextw_u64_z, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svget2_bf16, "q8yq16yIUWi", "n")
BUILTIN(__builtin_sve_svget2_f16, "q8hq16hIUWi", "n")
BUILTIN(__builtin_sve_svget2_f32, "q4fq8fIUWi", "n")
BUILTIN(__builtin_sve_svget2_f64, "q2dq4dIUWi", "n")
BUILTIN(__builtin_sve_svget2_s16, "q8sq16sIUWi", "n")
BUILTIN(__builtin_sve_svget2_s32, "q4iq8iIUWi", "n")
BUILTIN(__builtin_sve_svget2_s64, "q2Wiq4WiIUWi", "n")
BUILTIN(__builtin_sve_svget2_s8, "q16Scq32ScIUWi", "n")
BUILTIN(__builtin_sve_svget2_u16, "q8Usq16UsIUWi", "n")
BUILTIN(__builtin_sve_svget2_u32, "q4Uiq8UiIUWi", "n")
BUILTIN(__builtin_sve_svget2_u64, "q2UWiq4UWiIUWi", "n")
BUILTIN(__builtin_sve_svget2_u8, "q16Ucq32UcIUWi", "n")
BUILTIN(__builtin_sve_svget3_bf16, "q8yq24yIUWi", "n")
BUILTIN(__builtin_sve_svget3_f16, "q8hq24hIUWi", "n")
BUILTIN(__builtin_sve_svget3_f32, "q4fq12fIUWi", "n")
BUILTIN(__builtin_sve_svget3_f64, "q2dq6dIUWi", "n")
BUILTIN(__builtin_sve_svget3_s16, "q8sq24sIUWi", "n")
BUILTIN(__builtin_sve_svget3_s32, "q4iq12iIUWi", "n")
BUILTIN(__builtin_sve_svget3_s64, "q2Wiq6WiIUWi", "n")
BUILTIN(__builtin_sve_svget3_s8, "q16Scq48ScIUWi", "n")
BUILTIN(__builtin_sve_svget3_u16, "q8Usq24UsIUWi", "n")
BUILTIN(__builtin_sve_svget3_u32, "q4Uiq12UiIUWi", "n")
BUILTIN(__builtin_sve_svget3_u64, "q2UWiq6UWiIUWi", "n")
BUILTIN(__builtin_sve_svget3_u8, "q16Ucq48UcIUWi", "n")
BUILTIN(__builtin_sve_svget4_bf16, "q8yq32yIUWi", "n")
BUILTIN(__builtin_sve_svget4_f16, "q8hq32hIUWi", "n")
BUILTIN(__builtin_sve_svget4_f32, "q4fq16fIUWi", "n")
BUILTIN(__builtin_sve_svget4_f64, "q2dq8dIUWi", "n")
BUILTIN(__builtin_sve_svget4_s16, "q8sq32sIUWi", "n")
BUILTIN(__builtin_sve_svget4_s32, "q4iq16iIUWi", "n")
BUILTIN(__builtin_sve_svget4_s64, "q2Wiq8WiIUWi", "n")
BUILTIN(__builtin_sve_svget4_s8, "q16Scq64ScIUWi", "n")
BUILTIN(__builtin_sve_svget4_u16, "q8Usq32UsIUWi", "n")
BUILTIN(__builtin_sve_svget4_u32, "q4Uiq16UiIUWi", "n")
BUILTIN(__builtin_sve_svget4_u64, "q2UWiq8UWiIUWi", "n")
BUILTIN(__builtin_sve_svget4_u8, "q16Ucq64UcIUWi", "n")
BUILTIN(__builtin_sve_svhadd_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svhadd_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svhadd_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svhadd_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svhadd_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svhadd_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svhadd_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svhadd_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svhadd_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svhadd_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svhadd_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svhadd_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svhadd_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svhadd_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svhadd_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svhadd_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svhadd_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svhadd_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svhadd_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svhadd_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svhadd_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svhadd_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svhadd_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svhadd_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svhadd_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svhadd_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svhadd_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svhadd_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svhadd_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svhadd_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svhadd_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svhadd_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svhadd_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svhadd_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svhadd_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svhadd_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svhadd_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svhadd_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svhadd_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svhadd_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svhadd_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svhadd_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svhadd_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svhadd_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svhadd_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svhadd_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svhadd_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svhadd_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svhistcnt_s32_z, "q4Uiq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svhistcnt_s64_z, "q2UWiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svhistcnt_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svhistcnt_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svhistseg_s8, "q16Ucq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svhistseg_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svhsub_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svhsub_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svhsub_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svhsub_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svhsub_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svhsub_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svhsub_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svhsub_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svhsub_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svhsub_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svhsub_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svhsub_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svhsub_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svhsub_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svhsub_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svhsub_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svhsub_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svhsub_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svhsub_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svhsub_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svhsub_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svhsub_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svhsub_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svhsub_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svhsub_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svhsub_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svhsub_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svhsub_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svhsub_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svhsub_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svhsub_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svhsub_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svhsub_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svhsub_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svhsub_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svhsub_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svhsub_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svhsub_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svhsub_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svhsub_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svhsub_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svhsub_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svhsub_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svhsub_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svhsub_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svhsub_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svhsub_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svhsub_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svhsubr_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svhsubr_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svhsubr_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svhsubr_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svhsubr_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svhsubr_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svhsubr_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svhsubr_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svhsubr_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svhsubr_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svhsubr_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svhsubr_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svhsubr_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svhsubr_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svhsubr_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svhsubr_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svhsubr_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svhsubr_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svhsubr_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svhsubr_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svhsubr_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svhsubr_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svhsubr_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svhsubr_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svhsubr_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svhsubr_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svhsubr_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svhsubr_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svhsubr_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svhsubr_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svhsubr_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svhsubr_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svhsubr_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svhsubr_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svhsubr_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svhsubr_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svhsubr_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svhsubr_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svhsubr_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svhsubr_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svhsubr_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svhsubr_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svhsubr_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svhsubr_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svhsubr_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svhsubr_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svhsubr_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svhsubr_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svindex_s16, "q8sss", "n")
BUILTIN(__builtin_sve_svindex_s32, "q4iii", "n")
BUILTIN(__builtin_sve_svindex_s64, "q2WiWiWi", "n")
BUILTIN(__builtin_sve_svindex_s8, "q16ScScSc", "n")
BUILTIN(__builtin_sve_svindex_u16, "q8UsUsUs", "n")
BUILTIN(__builtin_sve_svindex_u32, "q4UiUiUi", "n")
BUILTIN(__builtin_sve_svindex_u64, "q2UWiUWiUWi", "n")
BUILTIN(__builtin_sve_svindex_u8, "q16UcUcUc", "n")
BUILTIN(__builtin_sve_svinsr_n_bf16, "q8yq8yy", "n")
BUILTIN(__builtin_sve_svinsr_n_f16, "q8hq8hh", "n")
BUILTIN(__builtin_sve_svinsr_n_f32, "q4fq4ff", "n")
BUILTIN(__builtin_sve_svinsr_n_f64, "q2dq2dd", "n")
BUILTIN(__builtin_sve_svinsr_n_s16, "q8sq8ss", "n")
BUILTIN(__builtin_sve_svinsr_n_s32, "q4iq4ii", "n")
BUILTIN(__builtin_sve_svinsr_n_s64, "q2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svinsr_n_s8, "q16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svinsr_n_u16, "q8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svinsr_n_u32, "q4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svinsr_n_u64, "q2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svinsr_n_u8, "q16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svlasta_bf16, "yq16bq8y", "n")
BUILTIN(__builtin_sve_svlasta_f16, "hq16bq8h", "n")
BUILTIN(__builtin_sve_svlasta_f32, "fq16bq4f", "n")
BUILTIN(__builtin_sve_svlasta_f64, "dq16bq2d", "n")
BUILTIN(__builtin_sve_svlasta_s16, "sq16bq8s", "n")
BUILTIN(__builtin_sve_svlasta_s32, "iq16bq4i", "n")
BUILTIN(__builtin_sve_svlasta_s64, "Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svlasta_s8, "Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svlasta_u16, "Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svlasta_u32, "Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svlasta_u64, "UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svlasta_u8, "Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svlastb_bf16, "yq16bq8y", "n")
BUILTIN(__builtin_sve_svlastb_f16, "hq16bq8h", "n")
BUILTIN(__builtin_sve_svlastb_f32, "fq16bq4f", "n")
BUILTIN(__builtin_sve_svlastb_f64, "dq16bq2d", "n")
BUILTIN(__builtin_sve_svlastb_s16, "sq16bq8s", "n")
BUILTIN(__builtin_sve_svlastb_s32, "iq16bq4i", "n")
BUILTIN(__builtin_sve_svlastb_s64, "Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svlastb_s8, "Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svlastb_u16, "Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svlastb_u32, "Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svlastb_u64, "UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svlastb_u8, "Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svld1_bf16, "q8yq16byC*", "n")
BUILTIN(__builtin_sve_svld1_f16, "q8hq16bhC*", "n")
BUILTIN(__builtin_sve_svld1_f32, "q4fq16bfC*", "n")
BUILTIN(__builtin_sve_svld1_f64, "q2dq16bdC*", "n")
BUILTIN(__builtin_sve_svld1_gather_s32index_f32, "q4fq16bfC*q4i", "n")
BUILTIN(__builtin_sve_svld1_gather_s32index_s32, "q4iq16bSiC*q4i", "n")
BUILTIN(__builtin_sve_svld1_gather_s32index_u32, "q4Uiq16bUiC*q4i", "n")
BUILTIN(__builtin_sve_svld1_gather_s32offset_f32, "q4fq16bfC*q4i", "n")
BUILTIN(__builtin_sve_svld1_gather_s32offset_s32, "q4iq16bSiC*q4i", "n")
BUILTIN(__builtin_sve_svld1_gather_s32offset_u32, "q4Uiq16bUiC*q4i", "n")
BUILTIN(__builtin_sve_svld1_gather_s64index_f64, "q2dq16bdC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1_gather_s64index_s64, "q2Wiq16bSWiC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1_gather_s64index_u64, "q2UWiq16bUWiC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1_gather_s64offset_f64, "q2dq16bdC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1_gather_s64offset_s64, "q2Wiq16bSWiC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1_gather_s64offset_u64, "q2UWiq16bUWiC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1_gather_u32base_f32, "q4fq16bq4Ui", "n")
BUILTIN(__builtin_sve_svld1_gather_u32base_index_f32, "q4fq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u32base_index_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u32base_index_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u32base_offset_f32, "q4fq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u32base_offset_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u32base_offset_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u32base_s32, "q4iq16bq4Ui", "n")
BUILTIN(__builtin_sve_svld1_gather_u32base_u32, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svld1_gather_u32index_f32, "q4fq16bfC*q4Ui", "n")
BUILTIN(__builtin_sve_svld1_gather_u32index_s32, "q4iq16bSiC*q4Ui", "n")
BUILTIN(__builtin_sve_svld1_gather_u32index_u32, "q4Uiq16bUiC*q4Ui", "n")
BUILTIN(__builtin_sve_svld1_gather_u32offset_f32, "q4fq16bfC*q4Ui", "n")
BUILTIN(__builtin_sve_svld1_gather_u32offset_s32, "q4iq16bSiC*q4Ui", "n")
BUILTIN(__builtin_sve_svld1_gather_u32offset_u32, "q4Uiq16bUiC*q4Ui", "n")
BUILTIN(__builtin_sve_svld1_gather_u64base_f64, "q2dq16bq2UWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u64base_index_f64, "q2dq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u64base_index_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u64base_index_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u64base_offset_f64, "q2dq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u64index_f64, "q2dq16bdC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u64index_s64, "q2Wiq16bSWiC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u64index_u64, "q2UWiq16bUWiC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u64offset_f64, "q2dq16bdC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u64offset_s64, "q2Wiq16bSWiC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1_gather_u64offset_u64, "q2UWiq16bUWiC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1_s16, "q8sq16bSsC*", "n")
BUILTIN(__builtin_sve_svld1_s32, "q4iq16bSiC*", "n")
BUILTIN(__builtin_sve_svld1_s64, "q2Wiq16bSWiC*", "n")
BUILTIN(__builtin_sve_svld1_s8, "q16Scq16bScC*", "n")
BUILTIN(__builtin_sve_svld1_u16, "q8Usq16bUsC*", "n")
BUILTIN(__builtin_sve_svld1_u32, "q4Uiq16bUiC*", "n")
BUILTIN(__builtin_sve_svld1_u64, "q2UWiq16bUWiC*", "n")
BUILTIN(__builtin_sve_svld1_u8, "q16Ucq16bUcC*", "n")
BUILTIN(__builtin_sve_svld1_vnum_bf16, "q8yq16byC*Wi", "n")
BUILTIN(__builtin_sve_svld1_vnum_f16, "q8hq16bhC*Wi", "n")
BUILTIN(__builtin_sve_svld1_vnum_f32, "q4fq16bfC*Wi", "n")
BUILTIN(__builtin_sve_svld1_vnum_f64, "q2dq16bdC*Wi", "n")
BUILTIN(__builtin_sve_svld1_vnum_s16, "q8sq16bSsC*Wi", "n")
BUILTIN(__builtin_sve_svld1_vnum_s32, "q4iq16bSiC*Wi", "n")
BUILTIN(__builtin_sve_svld1_vnum_s64, "q2Wiq16bSWiC*Wi", "n")
BUILTIN(__builtin_sve_svld1_vnum_s8, "q16Scq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svld1_vnum_u16, "q8Usq16bUsC*Wi", "n")
BUILTIN(__builtin_sve_svld1_vnum_u32, "q4Uiq16bUiC*Wi", "n")
BUILTIN(__builtin_sve_svld1_vnum_u64, "q2UWiq16bUWiC*Wi", "n")
BUILTIN(__builtin_sve_svld1_vnum_u8, "q16Ucq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svld1ro_bf16, "q8yq16byC*", "n")
BUILTIN(__builtin_sve_svld1ro_f16, "q8hq16bhC*", "n")
BUILTIN(__builtin_sve_svld1ro_f32, "q4fq16bfC*", "n")
BUILTIN(__builtin_sve_svld1ro_f64, "q2dq16bdC*", "n")
BUILTIN(__builtin_sve_svld1ro_s16, "q8sq16bSsC*", "n")
BUILTIN(__builtin_sve_svld1ro_s32, "q4iq16bSiC*", "n")
BUILTIN(__builtin_sve_svld1ro_s64, "q2Wiq16bSWiC*", "n")
BUILTIN(__builtin_sve_svld1ro_s8, "q16Scq16bScC*", "n")
BUILTIN(__builtin_sve_svld1ro_u16, "q8Usq16bUsC*", "n")
BUILTIN(__builtin_sve_svld1ro_u32, "q4Uiq16bUiC*", "n")
BUILTIN(__builtin_sve_svld1ro_u64, "q2UWiq16bUWiC*", "n")
BUILTIN(__builtin_sve_svld1ro_u8, "q16Ucq16bUcC*", "n")
BUILTIN(__builtin_sve_svld1rq_bf16, "q8yq16byC*", "n")
BUILTIN(__builtin_sve_svld1rq_f16, "q8hq16bhC*", "n")
BUILTIN(__builtin_sve_svld1rq_f32, "q4fq16bfC*", "n")
BUILTIN(__builtin_sve_svld1rq_f64, "q2dq16bdC*", "n")
BUILTIN(__builtin_sve_svld1rq_s16, "q8sq16bSsC*", "n")
BUILTIN(__builtin_sve_svld1rq_s32, "q4iq16bSiC*", "n")
BUILTIN(__builtin_sve_svld1rq_s64, "q2Wiq16bSWiC*", "n")
BUILTIN(__builtin_sve_svld1rq_s8, "q16Scq16bScC*", "n")
BUILTIN(__builtin_sve_svld1rq_u16, "q8Usq16bUsC*", "n")
BUILTIN(__builtin_sve_svld1rq_u32, "q4Uiq16bUiC*", "n")
BUILTIN(__builtin_sve_svld1rq_u64, "q2UWiq16bUWiC*", "n")
BUILTIN(__builtin_sve_svld1rq_u8, "q16Ucq16bUcC*", "n")
BUILTIN(__builtin_sve_svld1sb_gather_s32offset_s32, "q4iq16bScC*q4i", "n")
BUILTIN(__builtin_sve_svld1sb_gather_s32offset_u32, "q4Uiq16bScC*q4i", "n")
BUILTIN(__builtin_sve_svld1sb_gather_s64offset_s64, "q2Wiq16bScC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1sb_gather_s64offset_u64, "q2UWiq16bScC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1sb_gather_u32base_offset_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svld1sb_gather_u32base_offset_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svld1sb_gather_u32base_s32, "q4iq16bq4Ui", "n")
BUILTIN(__builtin_sve_svld1sb_gather_u32base_u32, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svld1sb_gather_u32offset_s32, "q4iq16bScC*q4Ui", "n")
BUILTIN(__builtin_sve_svld1sb_gather_u32offset_u32, "q4Uiq16bScC*q4Ui", "n")
BUILTIN(__builtin_sve_svld1sb_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1sb_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1sb_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svld1sb_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svld1sb_gather_u64offset_s64, "q2Wiq16bScC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1sb_gather_u64offset_u64, "q2UWiq16bScC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1sb_s16, "q8sq16bScC*", "n")
BUILTIN(__builtin_sve_svld1sb_s32, "q4iq16bScC*", "n")
BUILTIN(__builtin_sve_svld1sb_s64, "q2Wiq16bScC*", "n")
BUILTIN(__builtin_sve_svld1sb_u16, "q8Usq16bScC*", "n")
BUILTIN(__builtin_sve_svld1sb_u32, "q4Uiq16bScC*", "n")
BUILTIN(__builtin_sve_svld1sb_u64, "q2UWiq16bScC*", "n")
BUILTIN(__builtin_sve_svld1sb_vnum_s16, "q8sq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svld1sb_vnum_s32, "q4iq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svld1sb_vnum_s64, "q2Wiq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svld1sb_vnum_u16, "q8Usq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svld1sb_vnum_u32, "q4Uiq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svld1sb_vnum_u64, "q2UWiq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svld1sh_gather_s32index_s32, "q4iq16bSsC*q4i", "n")
BUILTIN(__builtin_sve_svld1sh_gather_s32index_u32, "q4Uiq16bSsC*q4i", "n")
BUILTIN(__builtin_sve_svld1sh_gather_s32offset_s32, "q4iq16bSsC*q4i", "n")
BUILTIN(__builtin_sve_svld1sh_gather_s32offset_u32, "q4Uiq16bSsC*q4i", "n")
BUILTIN(__builtin_sve_svld1sh_gather_s64index_s64, "q2Wiq16bSsC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1sh_gather_s64index_u64, "q2UWiq16bSsC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1sh_gather_s64offset_s64, "q2Wiq16bSsC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1sh_gather_s64offset_u64, "q2UWiq16bSsC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u32base_index_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u32base_index_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u32base_offset_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u32base_offset_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u32base_s32, "q4iq16bq4Ui", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u32base_u32, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u32index_s32, "q4iq16bSsC*q4Ui", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u32index_u32, "q4Uiq16bSsC*q4Ui", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u32offset_s32, "q4iq16bSsC*q4Ui", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u32offset_u32, "q4Uiq16bSsC*q4Ui", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u64base_index_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u64base_index_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u64index_s64, "q2Wiq16bSsC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u64index_u64, "q2UWiq16bSsC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u64offset_s64, "q2Wiq16bSsC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1sh_gather_u64offset_u64, "q2UWiq16bSsC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1sh_s32, "q4iq16bSsC*", "n")
BUILTIN(__builtin_sve_svld1sh_s64, "q2Wiq16bSsC*", "n")
BUILTIN(__builtin_sve_svld1sh_u32, "q4Uiq16bSsC*", "n")
BUILTIN(__builtin_sve_svld1sh_u64, "q2UWiq16bSsC*", "n")
BUILTIN(__builtin_sve_svld1sh_vnum_s32, "q4iq16bSsC*Wi", "n")
BUILTIN(__builtin_sve_svld1sh_vnum_s64, "q2Wiq16bSsC*Wi", "n")
BUILTIN(__builtin_sve_svld1sh_vnum_u32, "q4Uiq16bSsC*Wi", "n")
BUILTIN(__builtin_sve_svld1sh_vnum_u64, "q2UWiq16bSsC*Wi", "n")
BUILTIN(__builtin_sve_svld1sw_gather_s64index_s64, "q2Wiq16bSiC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1sw_gather_s64index_u64, "q2UWiq16bSiC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1sw_gather_s64offset_s64, "q2Wiq16bSiC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1sw_gather_s64offset_u64, "q2UWiq16bSiC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1sw_gather_u64base_index_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1sw_gather_u64base_index_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1sw_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1sw_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1sw_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svld1sw_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svld1sw_gather_u64index_s64, "q2Wiq16bSiC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1sw_gather_u64index_u64, "q2UWiq16bSiC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1sw_gather_u64offset_s64, "q2Wiq16bSiC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1sw_gather_u64offset_u64, "q2UWiq16bSiC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1sw_s64, "q2Wiq16bSiC*", "n")
BUILTIN(__builtin_sve_svld1sw_u64, "q2UWiq16bSiC*", "n")
BUILTIN(__builtin_sve_svld1sw_vnum_s64, "q2Wiq16bSiC*Wi", "n")
BUILTIN(__builtin_sve_svld1sw_vnum_u64, "q2UWiq16bSiC*Wi", "n")
BUILTIN(__builtin_sve_svld1ub_gather_s32offset_s32, "q4iq16bUcC*q4i", "n")
BUILTIN(__builtin_sve_svld1ub_gather_s32offset_u32, "q4Uiq16bUcC*q4i", "n")
BUILTIN(__builtin_sve_svld1ub_gather_s64offset_s64, "q2Wiq16bUcC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1ub_gather_s64offset_u64, "q2UWiq16bUcC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1ub_gather_u32base_offset_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svld1ub_gather_u32base_offset_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svld1ub_gather_u32base_s32, "q4iq16bq4Ui", "n")
BUILTIN(__builtin_sve_svld1ub_gather_u32base_u32, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svld1ub_gather_u32offset_s32, "q4iq16bUcC*q4Ui", "n")
BUILTIN(__builtin_sve_svld1ub_gather_u32offset_u32, "q4Uiq16bUcC*q4Ui", "n")
BUILTIN(__builtin_sve_svld1ub_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1ub_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1ub_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svld1ub_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svld1ub_gather_u64offset_s64, "q2Wiq16bUcC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1ub_gather_u64offset_u64, "q2UWiq16bUcC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1ub_s16, "q8sq16bUcC*", "n")
BUILTIN(__builtin_sve_svld1ub_s32, "q4iq16bUcC*", "n")
BUILTIN(__builtin_sve_svld1ub_s64, "q2Wiq16bUcC*", "n")
BUILTIN(__builtin_sve_svld1ub_u16, "q8Usq16bUcC*", "n")
BUILTIN(__builtin_sve_svld1ub_u32, "q4Uiq16bUcC*", "n")
BUILTIN(__builtin_sve_svld1ub_u64, "q2UWiq16bUcC*", "n")
BUILTIN(__builtin_sve_svld1ub_vnum_s16, "q8sq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svld1ub_vnum_s32, "q4iq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svld1ub_vnum_s64, "q2Wiq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svld1ub_vnum_u16, "q8Usq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svld1ub_vnum_u32, "q4Uiq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svld1ub_vnum_u64, "q2UWiq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svld1uh_gather_s32index_s32, "q4iq16bUsC*q4i", "n")
BUILTIN(__builtin_sve_svld1uh_gather_s32index_u32, "q4Uiq16bUsC*q4i", "n")
BUILTIN(__builtin_sve_svld1uh_gather_s32offset_s32, "q4iq16bUsC*q4i", "n")
BUILTIN(__builtin_sve_svld1uh_gather_s32offset_u32, "q4Uiq16bUsC*q4i", "n")
BUILTIN(__builtin_sve_svld1uh_gather_s64index_s64, "q2Wiq16bUsC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1uh_gather_s64index_u64, "q2UWiq16bUsC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1uh_gather_s64offset_s64, "q2Wiq16bUsC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1uh_gather_s64offset_u64, "q2UWiq16bUsC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u32base_index_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u32base_index_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u32base_offset_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u32base_offset_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u32base_s32, "q4iq16bq4Ui", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u32base_u32, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u32index_s32, "q4iq16bUsC*q4Ui", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u32index_u32, "q4Uiq16bUsC*q4Ui", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u32offset_s32, "q4iq16bUsC*q4Ui", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u32offset_u32, "q4Uiq16bUsC*q4Ui", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u64base_index_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u64base_index_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u64index_s64, "q2Wiq16bUsC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u64index_u64, "q2UWiq16bUsC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u64offset_s64, "q2Wiq16bUsC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1uh_gather_u64offset_u64, "q2UWiq16bUsC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1uh_s32, "q4iq16bUsC*", "n")
BUILTIN(__builtin_sve_svld1uh_s64, "q2Wiq16bUsC*", "n")
BUILTIN(__builtin_sve_svld1uh_u32, "q4Uiq16bUsC*", "n")
BUILTIN(__builtin_sve_svld1uh_u64, "q2UWiq16bUsC*", "n")
BUILTIN(__builtin_sve_svld1uh_vnum_s32, "q4iq16bUsC*Wi", "n")
BUILTIN(__builtin_sve_svld1uh_vnum_s64, "q2Wiq16bUsC*Wi", "n")
BUILTIN(__builtin_sve_svld1uh_vnum_u32, "q4Uiq16bUsC*Wi", "n")
BUILTIN(__builtin_sve_svld1uh_vnum_u64, "q2UWiq16bUsC*Wi", "n")
BUILTIN(__builtin_sve_svld1uw_gather_s64index_s64, "q2Wiq16bUiC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1uw_gather_s64index_u64, "q2UWiq16bUiC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1uw_gather_s64offset_s64, "q2Wiq16bUiC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1uw_gather_s64offset_u64, "q2UWiq16bUiC*q2Wi", "n")
BUILTIN(__builtin_sve_svld1uw_gather_u64base_index_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1uw_gather_u64base_index_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1uw_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1uw_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svld1uw_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svld1uw_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svld1uw_gather_u64index_s64, "q2Wiq16bUiC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1uw_gather_u64index_u64, "q2UWiq16bUiC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1uw_gather_u64offset_s64, "q2Wiq16bUiC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1uw_gather_u64offset_u64, "q2UWiq16bUiC*q2UWi", "n")
BUILTIN(__builtin_sve_svld1uw_s64, "q2Wiq16bUiC*", "n")
BUILTIN(__builtin_sve_svld1uw_u64, "q2UWiq16bUiC*", "n")
BUILTIN(__builtin_sve_svld1uw_vnum_s64, "q2Wiq16bUiC*Wi", "n")
BUILTIN(__builtin_sve_svld1uw_vnum_u64, "q2UWiq16bUiC*Wi", "n")
BUILTIN(__builtin_sve_svld2_bf16, "q16yq16byC*", "n")
BUILTIN(__builtin_sve_svld2_f16, "q16hq16bhC*", "n")
BUILTIN(__builtin_sve_svld2_f32, "q8fq16bfC*", "n")
BUILTIN(__builtin_sve_svld2_f64, "q4dq16bdC*", "n")
BUILTIN(__builtin_sve_svld2_s16, "q16sq16bSsC*", "n")
BUILTIN(__builtin_sve_svld2_s32, "q8iq16bSiC*", "n")
BUILTIN(__builtin_sve_svld2_s64, "q4Wiq16bSWiC*", "n")
BUILTIN(__builtin_sve_svld2_s8, "q32Scq16bScC*", "n")
BUILTIN(__builtin_sve_svld2_u16, "q16Usq16bUsC*", "n")
BUILTIN(__builtin_sve_svld2_u32, "q8Uiq16bUiC*", "n")
BUILTIN(__builtin_sve_svld2_u64, "q4UWiq16bUWiC*", "n")
BUILTIN(__builtin_sve_svld2_u8, "q32Ucq16bUcC*", "n")
BUILTIN(__builtin_sve_svld2_vnum_bf16, "q16yq16byC*Wi", "n")
BUILTIN(__builtin_sve_svld2_vnum_f16, "q16hq16bhC*Wi", "n")
BUILTIN(__builtin_sve_svld2_vnum_f32, "q8fq16bfC*Wi", "n")
BUILTIN(__builtin_sve_svld2_vnum_f64, "q4dq16bdC*Wi", "n")
BUILTIN(__builtin_sve_svld2_vnum_s16, "q16sq16bSsC*Wi", "n")
BUILTIN(__builtin_sve_svld2_vnum_s32, "q8iq16bSiC*Wi", "n")
BUILTIN(__builtin_sve_svld2_vnum_s64, "q4Wiq16bSWiC*Wi", "n")
BUILTIN(__builtin_sve_svld2_vnum_s8, "q32Scq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svld2_vnum_u16, "q16Usq16bUsC*Wi", "n")
BUILTIN(__builtin_sve_svld2_vnum_u32, "q8Uiq16bUiC*Wi", "n")
BUILTIN(__builtin_sve_svld2_vnum_u64, "q4UWiq16bUWiC*Wi", "n")
BUILTIN(__builtin_sve_svld2_vnum_u8, "q32Ucq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svld3_bf16, "q24yq16byC*", "n")
BUILTIN(__builtin_sve_svld3_f16, "q24hq16bhC*", "n")
BUILTIN(__builtin_sve_svld3_f32, "q12fq16bfC*", "n")
BUILTIN(__builtin_sve_svld3_f64, "q6dq16bdC*", "n")
BUILTIN(__builtin_sve_svld3_s16, "q24sq16bSsC*", "n")
BUILTIN(__builtin_sve_svld3_s32, "q12iq16bSiC*", "n")
BUILTIN(__builtin_sve_svld3_s64, "q6Wiq16bSWiC*", "n")
BUILTIN(__builtin_sve_svld3_s8, "q48Scq16bScC*", "n")
BUILTIN(__builtin_sve_svld3_u16, "q24Usq16bUsC*", "n")
BUILTIN(__builtin_sve_svld3_u32, "q12Uiq16bUiC*", "n")
BUILTIN(__builtin_sve_svld3_u64, "q6UWiq16bUWiC*", "n")
BUILTIN(__builtin_sve_svld3_u8, "q48Ucq16bUcC*", "n")
BUILTIN(__builtin_sve_svld3_vnum_bf16, "q24yq16byC*Wi", "n")
BUILTIN(__builtin_sve_svld3_vnum_f16, "q24hq16bhC*Wi", "n")
BUILTIN(__builtin_sve_svld3_vnum_f32, "q12fq16bfC*Wi", "n")
BUILTIN(__builtin_sve_svld3_vnum_f64, "q6dq16bdC*Wi", "n")
BUILTIN(__builtin_sve_svld3_vnum_s16, "q24sq16bSsC*Wi", "n")
BUILTIN(__builtin_sve_svld3_vnum_s32, "q12iq16bSiC*Wi", "n")
BUILTIN(__builtin_sve_svld3_vnum_s64, "q6Wiq16bSWiC*Wi", "n")
BUILTIN(__builtin_sve_svld3_vnum_s8, "q48Scq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svld3_vnum_u16, "q24Usq16bUsC*Wi", "n")
BUILTIN(__builtin_sve_svld3_vnum_u32, "q12Uiq16bUiC*Wi", "n")
BUILTIN(__builtin_sve_svld3_vnum_u64, "q6UWiq16bUWiC*Wi", "n")
BUILTIN(__builtin_sve_svld3_vnum_u8, "q48Ucq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svld4_bf16, "q32yq16byC*", "n")
BUILTIN(__builtin_sve_svld4_f16, "q32hq16bhC*", "n")
BUILTIN(__builtin_sve_svld4_f32, "q16fq16bfC*", "n")
BUILTIN(__builtin_sve_svld4_f64, "q8dq16bdC*", "n")
BUILTIN(__builtin_sve_svld4_s16, "q32sq16bSsC*", "n")
BUILTIN(__builtin_sve_svld4_s32, "q16iq16bSiC*", "n")
BUILTIN(__builtin_sve_svld4_s64, "q8Wiq16bSWiC*", "n")
BUILTIN(__builtin_sve_svld4_s8, "q64Scq16bScC*", "n")
BUILTIN(__builtin_sve_svld4_u16, "q32Usq16bUsC*", "n")
BUILTIN(__builtin_sve_svld4_u32, "q16Uiq16bUiC*", "n")
BUILTIN(__builtin_sve_svld4_u64, "q8UWiq16bUWiC*", "n")
BUILTIN(__builtin_sve_svld4_u8, "q64Ucq16bUcC*", "n")
BUILTIN(__builtin_sve_svld4_vnum_bf16, "q32yq16byC*Wi", "n")
BUILTIN(__builtin_sve_svld4_vnum_f16, "q32hq16bhC*Wi", "n")
BUILTIN(__builtin_sve_svld4_vnum_f32, "q16fq16bfC*Wi", "n")
BUILTIN(__builtin_sve_svld4_vnum_f64, "q8dq16bdC*Wi", "n")
BUILTIN(__builtin_sve_svld4_vnum_s16, "q32sq16bSsC*Wi", "n")
BUILTIN(__builtin_sve_svld4_vnum_s32, "q16iq16bSiC*Wi", "n")
BUILTIN(__builtin_sve_svld4_vnum_s64, "q8Wiq16bSWiC*Wi", "n")
BUILTIN(__builtin_sve_svld4_vnum_s8, "q64Scq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svld4_vnum_u16, "q32Usq16bUsC*Wi", "n")
BUILTIN(__builtin_sve_svld4_vnum_u32, "q16Uiq16bUiC*Wi", "n")
BUILTIN(__builtin_sve_svld4_vnum_u64, "q8UWiq16bUWiC*Wi", "n")
BUILTIN(__builtin_sve_svld4_vnum_u8, "q64Ucq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svldff1_bf16, "q8yq16byC*", "n")
BUILTIN(__builtin_sve_svldff1_f16, "q8hq16bhC*", "n")
BUILTIN(__builtin_sve_svldff1_f32, "q4fq16bfC*", "n")
BUILTIN(__builtin_sve_svldff1_f64, "q2dq16bdC*", "n")
BUILTIN(__builtin_sve_svldff1_gather_s32index_f32, "q4fq16bfC*q4i", "n")
BUILTIN(__builtin_sve_svldff1_gather_s32index_s32, "q4iq16bSiC*q4i", "n")
BUILTIN(__builtin_sve_svldff1_gather_s32index_u32, "q4Uiq16bUiC*q4i", "n")
BUILTIN(__builtin_sve_svldff1_gather_s32offset_f32, "q4fq16bfC*q4i", "n")
BUILTIN(__builtin_sve_svldff1_gather_s32offset_s32, "q4iq16bSiC*q4i", "n")
BUILTIN(__builtin_sve_svldff1_gather_s32offset_u32, "q4Uiq16bUiC*q4i", "n")
BUILTIN(__builtin_sve_svldff1_gather_s64index_f64, "q2dq16bdC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1_gather_s64index_s64, "q2Wiq16bSWiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1_gather_s64index_u64, "q2UWiq16bUWiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1_gather_s64offset_f64, "q2dq16bdC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1_gather_s64offset_s64, "q2Wiq16bSWiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1_gather_s64offset_u64, "q2UWiq16bUWiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u32base_f32, "q4fq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldff1_gather_u32base_index_f32, "q4fq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u32base_index_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u32base_index_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u32base_offset_f32, "q4fq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u32base_offset_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u32base_offset_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u32base_s32, "q4iq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldff1_gather_u32base_u32, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldff1_gather_u32index_f32, "q4fq16bfC*q4Ui", "n")
BUILTIN(__builtin_sve_svldff1_gather_u32index_s32, "q4iq16bSiC*q4Ui", "n")
BUILTIN(__builtin_sve_svldff1_gather_u32index_u32, "q4Uiq16bUiC*q4Ui", "n")
BUILTIN(__builtin_sve_svldff1_gather_u32offset_f32, "q4fq16bfC*q4Ui", "n")
BUILTIN(__builtin_sve_svldff1_gather_u32offset_s32, "q4iq16bSiC*q4Ui", "n")
BUILTIN(__builtin_sve_svldff1_gather_u32offset_u32, "q4Uiq16bUiC*q4Ui", "n")
BUILTIN(__builtin_sve_svldff1_gather_u64base_f64, "q2dq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u64base_index_f64, "q2dq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u64base_index_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u64base_index_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u64base_offset_f64, "q2dq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u64index_f64, "q2dq16bdC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u64index_s64, "q2Wiq16bSWiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u64index_u64, "q2UWiq16bUWiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u64offset_f64, "q2dq16bdC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u64offset_s64, "q2Wiq16bSWiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1_gather_u64offset_u64, "q2UWiq16bUWiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1_s16, "q8sq16bSsC*", "n")
BUILTIN(__builtin_sve_svldff1_s32, "q4iq16bSiC*", "n")
BUILTIN(__builtin_sve_svldff1_s64, "q2Wiq16bSWiC*", "n")
BUILTIN(__builtin_sve_svldff1_s8, "q16Scq16bScC*", "n")
BUILTIN(__builtin_sve_svldff1_u16, "q8Usq16bUsC*", "n")
BUILTIN(__builtin_sve_svldff1_u32, "q4Uiq16bUiC*", "n")
BUILTIN(__builtin_sve_svldff1_u64, "q2UWiq16bUWiC*", "n")
BUILTIN(__builtin_sve_svldff1_u8, "q16Ucq16bUcC*", "n")
BUILTIN(__builtin_sve_svldff1_vnum_bf16, "q8yq16byC*Wi", "n")
BUILTIN(__builtin_sve_svldff1_vnum_f16, "q8hq16bhC*Wi", "n")
BUILTIN(__builtin_sve_svldff1_vnum_f32, "q4fq16bfC*Wi", "n")
BUILTIN(__builtin_sve_svldff1_vnum_f64, "q2dq16bdC*Wi", "n")
BUILTIN(__builtin_sve_svldff1_vnum_s16, "q8sq16bSsC*Wi", "n")
BUILTIN(__builtin_sve_svldff1_vnum_s32, "q4iq16bSiC*Wi", "n")
BUILTIN(__builtin_sve_svldff1_vnum_s64, "q2Wiq16bSWiC*Wi", "n")
BUILTIN(__builtin_sve_svldff1_vnum_s8, "q16Scq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svldff1_vnum_u16, "q8Usq16bUsC*Wi", "n")
BUILTIN(__builtin_sve_svldff1_vnum_u32, "q4Uiq16bUiC*Wi", "n")
BUILTIN(__builtin_sve_svldff1_vnum_u64, "q2UWiq16bUWiC*Wi", "n")
BUILTIN(__builtin_sve_svldff1_vnum_u8, "q16Ucq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svldff1sb_gather_s32offset_s32, "q4iq16bScC*q4i", "n")
BUILTIN(__builtin_sve_svldff1sb_gather_s32offset_u32, "q4Uiq16bScC*q4i", "n")
BUILTIN(__builtin_sve_svldff1sb_gather_s64offset_s64, "q2Wiq16bScC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1sb_gather_s64offset_u64, "q2UWiq16bScC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1sb_gather_u32base_offset_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldff1sb_gather_u32base_offset_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldff1sb_gather_u32base_s32, "q4iq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldff1sb_gather_u32base_u32, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldff1sb_gather_u32offset_s32, "q4iq16bScC*q4Ui", "n")
BUILTIN(__builtin_sve_svldff1sb_gather_u32offset_u32, "q4Uiq16bScC*q4Ui", "n")
BUILTIN(__builtin_sve_svldff1sb_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1sb_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1sb_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldff1sb_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldff1sb_gather_u64offset_s64, "q2Wiq16bScC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1sb_gather_u64offset_u64, "q2UWiq16bScC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1sb_s16, "q8sq16bScC*", "n")
BUILTIN(__builtin_sve_svldff1sb_s32, "q4iq16bScC*", "n")
BUILTIN(__builtin_sve_svldff1sb_s64, "q2Wiq16bScC*", "n")
BUILTIN(__builtin_sve_svldff1sb_u16, "q8Usq16bScC*", "n")
BUILTIN(__builtin_sve_svldff1sb_u32, "q4Uiq16bScC*", "n")
BUILTIN(__builtin_sve_svldff1sb_u64, "q2UWiq16bScC*", "n")
BUILTIN(__builtin_sve_svldff1sb_vnum_s16, "q8sq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svldff1sb_vnum_s32, "q4iq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svldff1sb_vnum_s64, "q2Wiq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svldff1sb_vnum_u16, "q8Usq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svldff1sb_vnum_u32, "q4Uiq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svldff1sb_vnum_u64, "q2UWiq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_s32index_s32, "q4iq16bSsC*q4i", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_s32index_u32, "q4Uiq16bSsC*q4i", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_s32offset_s32, "q4iq16bSsC*q4i", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_s32offset_u32, "q4Uiq16bSsC*q4i", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_s64index_s64, "q2Wiq16bSsC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_s64index_u64, "q2UWiq16bSsC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_s64offset_s64, "q2Wiq16bSsC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_s64offset_u64, "q2UWiq16bSsC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u32base_index_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u32base_index_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u32base_offset_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u32base_offset_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u32base_s32, "q4iq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u32base_u32, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u32index_s32, "q4iq16bSsC*q4Ui", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u32index_u32, "q4Uiq16bSsC*q4Ui", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u32offset_s32, "q4iq16bSsC*q4Ui", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u32offset_u32, "q4Uiq16bSsC*q4Ui", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u64base_index_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u64base_index_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u64index_s64, "q2Wiq16bSsC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u64index_u64, "q2UWiq16bSsC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u64offset_s64, "q2Wiq16bSsC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1sh_gather_u64offset_u64, "q2UWiq16bSsC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1sh_s32, "q4iq16bSsC*", "n")
BUILTIN(__builtin_sve_svldff1sh_s64, "q2Wiq16bSsC*", "n")
BUILTIN(__builtin_sve_svldff1sh_u32, "q4Uiq16bSsC*", "n")
BUILTIN(__builtin_sve_svldff1sh_u64, "q2UWiq16bSsC*", "n")
BUILTIN(__builtin_sve_svldff1sh_vnum_s32, "q4iq16bSsC*Wi", "n")
BUILTIN(__builtin_sve_svldff1sh_vnum_s64, "q2Wiq16bSsC*Wi", "n")
BUILTIN(__builtin_sve_svldff1sh_vnum_u32, "q4Uiq16bSsC*Wi", "n")
BUILTIN(__builtin_sve_svldff1sh_vnum_u64, "q2UWiq16bSsC*Wi", "n")
BUILTIN(__builtin_sve_svldff1sw_gather_s64index_s64, "q2Wiq16bSiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1sw_gather_s64index_u64, "q2UWiq16bSiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1sw_gather_s64offset_s64, "q2Wiq16bSiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1sw_gather_s64offset_u64, "q2UWiq16bSiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1sw_gather_u64base_index_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1sw_gather_u64base_index_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1sw_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1sw_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1sw_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldff1sw_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldff1sw_gather_u64index_s64, "q2Wiq16bSiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1sw_gather_u64index_u64, "q2UWiq16bSiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1sw_gather_u64offset_s64, "q2Wiq16bSiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1sw_gather_u64offset_u64, "q2UWiq16bSiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1sw_s64, "q2Wiq16bSiC*", "n")
BUILTIN(__builtin_sve_svldff1sw_u64, "q2UWiq16bSiC*", "n")
BUILTIN(__builtin_sve_svldff1sw_vnum_s64, "q2Wiq16bSiC*Wi", "n")
BUILTIN(__builtin_sve_svldff1sw_vnum_u64, "q2UWiq16bSiC*Wi", "n")
BUILTIN(__builtin_sve_svldff1ub_gather_s32offset_s32, "q4iq16bUcC*q4i", "n")
BUILTIN(__builtin_sve_svldff1ub_gather_s32offset_u32, "q4Uiq16bUcC*q4i", "n")
BUILTIN(__builtin_sve_svldff1ub_gather_s64offset_s64, "q2Wiq16bUcC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1ub_gather_s64offset_u64, "q2UWiq16bUcC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1ub_gather_u32base_offset_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldff1ub_gather_u32base_offset_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldff1ub_gather_u32base_s32, "q4iq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldff1ub_gather_u32base_u32, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldff1ub_gather_u32offset_s32, "q4iq16bUcC*q4Ui", "n")
BUILTIN(__builtin_sve_svldff1ub_gather_u32offset_u32, "q4Uiq16bUcC*q4Ui", "n")
BUILTIN(__builtin_sve_svldff1ub_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1ub_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1ub_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldff1ub_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldff1ub_gather_u64offset_s64, "q2Wiq16bUcC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1ub_gather_u64offset_u64, "q2UWiq16bUcC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1ub_s16, "q8sq16bUcC*", "n")
BUILTIN(__builtin_sve_svldff1ub_s32, "q4iq16bUcC*", "n")
BUILTIN(__builtin_sve_svldff1ub_s64, "q2Wiq16bUcC*", "n")
BUILTIN(__builtin_sve_svldff1ub_u16, "q8Usq16bUcC*", "n")
BUILTIN(__builtin_sve_svldff1ub_u32, "q4Uiq16bUcC*", "n")
BUILTIN(__builtin_sve_svldff1ub_u64, "q2UWiq16bUcC*", "n")
BUILTIN(__builtin_sve_svldff1ub_vnum_s16, "q8sq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svldff1ub_vnum_s32, "q4iq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svldff1ub_vnum_s64, "q2Wiq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svldff1ub_vnum_u16, "q8Usq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svldff1ub_vnum_u32, "q4Uiq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svldff1ub_vnum_u64, "q2UWiq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_s32index_s32, "q4iq16bUsC*q4i", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_s32index_u32, "q4Uiq16bUsC*q4i", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_s32offset_s32, "q4iq16bUsC*q4i", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_s32offset_u32, "q4Uiq16bUsC*q4i", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_s64index_s64, "q2Wiq16bUsC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_s64index_u64, "q2UWiq16bUsC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_s64offset_s64, "q2Wiq16bUsC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_s64offset_u64, "q2UWiq16bUsC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u32base_index_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u32base_index_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u32base_offset_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u32base_offset_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u32base_s32, "q4iq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u32base_u32, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u32index_s32, "q4iq16bUsC*q4Ui", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u32index_u32, "q4Uiq16bUsC*q4Ui", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u32offset_s32, "q4iq16bUsC*q4Ui", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u32offset_u32, "q4Uiq16bUsC*q4Ui", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u64base_index_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u64base_index_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u64index_s64, "q2Wiq16bUsC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u64index_u64, "q2UWiq16bUsC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u64offset_s64, "q2Wiq16bUsC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1uh_gather_u64offset_u64, "q2UWiq16bUsC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1uh_s32, "q4iq16bUsC*", "n")
BUILTIN(__builtin_sve_svldff1uh_s64, "q2Wiq16bUsC*", "n")
BUILTIN(__builtin_sve_svldff1uh_u32, "q4Uiq16bUsC*", "n")
BUILTIN(__builtin_sve_svldff1uh_u64, "q2UWiq16bUsC*", "n")
BUILTIN(__builtin_sve_svldff1uh_vnum_s32, "q4iq16bUsC*Wi", "n")
BUILTIN(__builtin_sve_svldff1uh_vnum_s64, "q2Wiq16bUsC*Wi", "n")
BUILTIN(__builtin_sve_svldff1uh_vnum_u32, "q4Uiq16bUsC*Wi", "n")
BUILTIN(__builtin_sve_svldff1uh_vnum_u64, "q2UWiq16bUsC*Wi", "n")
BUILTIN(__builtin_sve_svldff1uw_gather_s64index_s64, "q2Wiq16bUiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1uw_gather_s64index_u64, "q2UWiq16bUiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1uw_gather_s64offset_s64, "q2Wiq16bUiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1uw_gather_s64offset_u64, "q2UWiq16bUiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldff1uw_gather_u64base_index_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1uw_gather_u64base_index_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1uw_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1uw_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldff1uw_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldff1uw_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldff1uw_gather_u64index_s64, "q2Wiq16bUiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1uw_gather_u64index_u64, "q2UWiq16bUiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1uw_gather_u64offset_s64, "q2Wiq16bUiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1uw_gather_u64offset_u64, "q2UWiq16bUiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldff1uw_s64, "q2Wiq16bUiC*", "n")
BUILTIN(__builtin_sve_svldff1uw_u64, "q2UWiq16bUiC*", "n")
BUILTIN(__builtin_sve_svldff1uw_vnum_s64, "q2Wiq16bUiC*Wi", "n")
BUILTIN(__builtin_sve_svldff1uw_vnum_u64, "q2UWiq16bUiC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1_bf16, "q8yq16byC*", "n")
BUILTIN(__builtin_sve_svldnf1_f16, "q8hq16bhC*", "n")
BUILTIN(__builtin_sve_svldnf1_f32, "q4fq16bfC*", "n")
BUILTIN(__builtin_sve_svldnf1_f64, "q2dq16bdC*", "n")
BUILTIN(__builtin_sve_svldnf1_s16, "q8sq16bSsC*", "n")
BUILTIN(__builtin_sve_svldnf1_s32, "q4iq16bSiC*", "n")
BUILTIN(__builtin_sve_svldnf1_s64, "q2Wiq16bSWiC*", "n")
BUILTIN(__builtin_sve_svldnf1_s8, "q16Scq16bScC*", "n")
BUILTIN(__builtin_sve_svldnf1_u16, "q8Usq16bUsC*", "n")
BUILTIN(__builtin_sve_svldnf1_u32, "q4Uiq16bUiC*", "n")
BUILTIN(__builtin_sve_svldnf1_u64, "q2UWiq16bUWiC*", "n")
BUILTIN(__builtin_sve_svldnf1_u8, "q16Ucq16bUcC*", "n")
BUILTIN(__builtin_sve_svldnf1_vnum_bf16, "q8yq16byC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1_vnum_f16, "q8hq16bhC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1_vnum_f32, "q4fq16bfC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1_vnum_f64, "q2dq16bdC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1_vnum_s16, "q8sq16bSsC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1_vnum_s32, "q4iq16bSiC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1_vnum_s64, "q2Wiq16bSWiC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1_vnum_s8, "q16Scq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1_vnum_u16, "q8Usq16bUsC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1_vnum_u32, "q4Uiq16bUiC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1_vnum_u64, "q2UWiq16bUWiC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1_vnum_u8, "q16Ucq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1sb_s16, "q8sq16bScC*", "n")
BUILTIN(__builtin_sve_svldnf1sb_s32, "q4iq16bScC*", "n")
BUILTIN(__builtin_sve_svldnf1sb_s64, "q2Wiq16bScC*", "n")
BUILTIN(__builtin_sve_svldnf1sb_u16, "q8Usq16bScC*", "n")
BUILTIN(__builtin_sve_svldnf1sb_u32, "q4Uiq16bScC*", "n")
BUILTIN(__builtin_sve_svldnf1sb_u64, "q2UWiq16bScC*", "n")
BUILTIN(__builtin_sve_svldnf1sb_vnum_s16, "q8sq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1sb_vnum_s32, "q4iq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1sb_vnum_s64, "q2Wiq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1sb_vnum_u16, "q8Usq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1sb_vnum_u32, "q4Uiq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1sb_vnum_u64, "q2UWiq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1sh_s32, "q4iq16bSsC*", "n")
BUILTIN(__builtin_sve_svldnf1sh_s64, "q2Wiq16bSsC*", "n")
BUILTIN(__builtin_sve_svldnf1sh_u32, "q4Uiq16bSsC*", "n")
BUILTIN(__builtin_sve_svldnf1sh_u64, "q2UWiq16bSsC*", "n")
BUILTIN(__builtin_sve_svldnf1sh_vnum_s32, "q4iq16bSsC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1sh_vnum_s64, "q2Wiq16bSsC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1sh_vnum_u32, "q4Uiq16bSsC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1sh_vnum_u64, "q2UWiq16bSsC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1sw_s64, "q2Wiq16bSiC*", "n")
BUILTIN(__builtin_sve_svldnf1sw_u64, "q2UWiq16bSiC*", "n")
BUILTIN(__builtin_sve_svldnf1sw_vnum_s64, "q2Wiq16bSiC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1sw_vnum_u64, "q2UWiq16bSiC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1ub_s16, "q8sq16bUcC*", "n")
BUILTIN(__builtin_sve_svldnf1ub_s32, "q4iq16bUcC*", "n")
BUILTIN(__builtin_sve_svldnf1ub_s64, "q2Wiq16bUcC*", "n")
BUILTIN(__builtin_sve_svldnf1ub_u16, "q8Usq16bUcC*", "n")
BUILTIN(__builtin_sve_svldnf1ub_u32, "q4Uiq16bUcC*", "n")
BUILTIN(__builtin_sve_svldnf1ub_u64, "q2UWiq16bUcC*", "n")
BUILTIN(__builtin_sve_svldnf1ub_vnum_s16, "q8sq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1ub_vnum_s32, "q4iq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1ub_vnum_s64, "q2Wiq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1ub_vnum_u16, "q8Usq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1ub_vnum_u32, "q4Uiq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1ub_vnum_u64, "q2UWiq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1uh_s32, "q4iq16bUsC*", "n")
BUILTIN(__builtin_sve_svldnf1uh_s64, "q2Wiq16bUsC*", "n")
BUILTIN(__builtin_sve_svldnf1uh_u32, "q4Uiq16bUsC*", "n")
BUILTIN(__builtin_sve_svldnf1uh_u64, "q2UWiq16bUsC*", "n")
BUILTIN(__builtin_sve_svldnf1uh_vnum_s32, "q4iq16bUsC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1uh_vnum_s64, "q2Wiq16bUsC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1uh_vnum_u32, "q4Uiq16bUsC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1uh_vnum_u64, "q2UWiq16bUsC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1uw_s64, "q2Wiq16bUiC*", "n")
BUILTIN(__builtin_sve_svldnf1uw_u64, "q2UWiq16bUiC*", "n")
BUILTIN(__builtin_sve_svldnf1uw_vnum_s64, "q2Wiq16bUiC*Wi", "n")
BUILTIN(__builtin_sve_svldnf1uw_vnum_u64, "q2UWiq16bUiC*Wi", "n")
BUILTIN(__builtin_sve_svldnt1_bf16, "q8yq16byC*", "n")
BUILTIN(__builtin_sve_svldnt1_f16, "q8hq16bhC*", "n")
BUILTIN(__builtin_sve_svldnt1_f32, "q4fq16bfC*", "n")
BUILTIN(__builtin_sve_svldnt1_f64, "q2dq16bdC*", "n")
BUILTIN(__builtin_sve_svldnt1_gather_s64index_f64, "q2dq16bdC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_s64index_s64, "q2Wiq16bSWiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_s64index_u64, "q2UWiq16bUWiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_s64offset_f64, "q2dq16bdC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_s64offset_s64, "q2Wiq16bSWiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_s64offset_u64, "q2UWiq16bUWiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u32base_f32, "q4fq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u32base_index_f32, "q4fq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u32base_index_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u32base_index_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u32base_offset_f32, "q4fq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u32base_offset_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u32base_offset_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u32base_s32, "q4iq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u32base_u32, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u32offset_f32, "q4fq16bfC*q4Ui", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u32offset_s32, "q4iq16bSiC*q4Ui", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u32offset_u32, "q4Uiq16bUiC*q4Ui", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u64base_f64, "q2dq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u64base_index_f64, "q2dq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u64base_index_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u64base_index_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u64base_offset_f64, "q2dq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u64index_f64, "q2dq16bdC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u64index_s64, "q2Wiq16bSWiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u64index_u64, "q2UWiq16bUWiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u64offset_f64, "q2dq16bdC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u64offset_s64, "q2Wiq16bSWiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1_gather_u64offset_u64, "q2UWiq16bUWiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1_s16, "q8sq16bSsC*", "n")
BUILTIN(__builtin_sve_svldnt1_s32, "q4iq16bSiC*", "n")
BUILTIN(__builtin_sve_svldnt1_s64, "q2Wiq16bSWiC*", "n")
BUILTIN(__builtin_sve_svldnt1_s8, "q16Scq16bScC*", "n")
BUILTIN(__builtin_sve_svldnt1_u16, "q8Usq16bUsC*", "n")
BUILTIN(__builtin_sve_svldnt1_u32, "q4Uiq16bUiC*", "n")
BUILTIN(__builtin_sve_svldnt1_u64, "q2UWiq16bUWiC*", "n")
BUILTIN(__builtin_sve_svldnt1_u8, "q16Ucq16bUcC*", "n")
BUILTIN(__builtin_sve_svldnt1_vnum_bf16, "q8yq16byC*Wi", "n")
BUILTIN(__builtin_sve_svldnt1_vnum_f16, "q8hq16bhC*Wi", "n")
BUILTIN(__builtin_sve_svldnt1_vnum_f32, "q4fq16bfC*Wi", "n")
BUILTIN(__builtin_sve_svldnt1_vnum_f64, "q2dq16bdC*Wi", "n")
BUILTIN(__builtin_sve_svldnt1_vnum_s16, "q8sq16bSsC*Wi", "n")
BUILTIN(__builtin_sve_svldnt1_vnum_s32, "q4iq16bSiC*Wi", "n")
BUILTIN(__builtin_sve_svldnt1_vnum_s64, "q2Wiq16bSWiC*Wi", "n")
BUILTIN(__builtin_sve_svldnt1_vnum_s8, "q16Scq16bScC*Wi", "n")
BUILTIN(__builtin_sve_svldnt1_vnum_u16, "q8Usq16bUsC*Wi", "n")
BUILTIN(__builtin_sve_svldnt1_vnum_u32, "q4Uiq16bUiC*Wi", "n")
BUILTIN(__builtin_sve_svldnt1_vnum_u64, "q2UWiq16bUWiC*Wi", "n")
BUILTIN(__builtin_sve_svldnt1_vnum_u8, "q16Ucq16bUcC*Wi", "n")
BUILTIN(__builtin_sve_svldnt1sb_gather_s64offset_s64, "q2Wiq16bScC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1sb_gather_s64offset_u64, "q2UWiq16bScC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1sb_gather_u32base_offset_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldnt1sb_gather_u32base_offset_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldnt1sb_gather_u32base_s32, "q4iq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldnt1sb_gather_u32base_u32, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldnt1sb_gather_u32offset_s32, "q4iq16bScC*q4Ui", "n")
BUILTIN(__builtin_sve_svldnt1sb_gather_u32offset_u32, "q4Uiq16bScC*q4Ui", "n")
BUILTIN(__builtin_sve_svldnt1sb_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1sb_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1sb_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldnt1sb_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldnt1sb_gather_u64offset_s64, "q2Wiq16bScC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1sb_gather_u64offset_u64, "q2UWiq16bScC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_s64index_s64, "q2Wiq16bSsC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_s64index_u64, "q2UWiq16bSsC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_s64offset_s64, "q2Wiq16bSsC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_s64offset_u64, "q2UWiq16bSsC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_u32base_index_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_u32base_index_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_u32base_offset_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_u32base_offset_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_u32base_s32, "q4iq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_u32base_u32, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_u32offset_s32, "q4iq16bSsC*q4Ui", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_u32offset_u32, "q4Uiq16bSsC*q4Ui", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_u64base_index_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_u64base_index_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_u64index_s64, "q2Wiq16bSsC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_u64index_u64, "q2UWiq16bSsC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_u64offset_s64, "q2Wiq16bSsC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1sh_gather_u64offset_u64, "q2UWiq16bSsC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1sw_gather_s64index_s64, "q2Wiq16bSiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1sw_gather_s64index_u64, "q2UWiq16bSiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1sw_gather_s64offset_s64, "q2Wiq16bSiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1sw_gather_s64offset_u64, "q2UWiq16bSiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1sw_gather_u64base_index_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1sw_gather_u64base_index_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1sw_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1sw_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1sw_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldnt1sw_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldnt1sw_gather_u64index_s64, "q2Wiq16bSiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1sw_gather_u64index_u64, "q2UWiq16bSiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1sw_gather_u64offset_s64, "q2Wiq16bSiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1sw_gather_u64offset_u64, "q2UWiq16bSiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1ub_gather_s64offset_s64, "q2Wiq16bUcC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1ub_gather_s64offset_u64, "q2UWiq16bUcC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1ub_gather_u32base_offset_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldnt1ub_gather_u32base_offset_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldnt1ub_gather_u32base_s32, "q4iq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldnt1ub_gather_u32base_u32, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldnt1ub_gather_u32offset_s32, "q4iq16bUcC*q4Ui", "n")
BUILTIN(__builtin_sve_svldnt1ub_gather_u32offset_u32, "q4Uiq16bUcC*q4Ui", "n")
BUILTIN(__builtin_sve_svldnt1ub_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1ub_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1ub_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldnt1ub_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldnt1ub_gather_u64offset_s64, "q2Wiq16bUcC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1ub_gather_u64offset_u64, "q2UWiq16bUcC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_s64index_s64, "q2Wiq16bUsC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_s64index_u64, "q2UWiq16bUsC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_s64offset_s64, "q2Wiq16bUsC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_s64offset_u64, "q2UWiq16bUsC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_u32base_index_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_u32base_index_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_u32base_offset_s32, "q4iq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_u32base_offset_u32, "q4Uiq16bq4UiWi", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_u32base_s32, "q4iq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_u32base_u32, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_u32offset_s32, "q4iq16bUsC*q4Ui", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_u32offset_u32, "q4Uiq16bUsC*q4Ui", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_u64base_index_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_u64base_index_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_u64index_s64, "q2Wiq16bUsC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_u64index_u64, "q2UWiq16bUsC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_u64offset_s64, "q2Wiq16bUsC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1uh_gather_u64offset_u64, "q2UWiq16bUsC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1uw_gather_s64index_s64, "q2Wiq16bUiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1uw_gather_s64index_u64, "q2UWiq16bUiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1uw_gather_s64offset_s64, "q2Wiq16bUiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1uw_gather_s64offset_u64, "q2UWiq16bUiC*q2Wi", "n")
BUILTIN(__builtin_sve_svldnt1uw_gather_u64base_index_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1uw_gather_u64base_index_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1uw_gather_u64base_offset_s64, "q2Wiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1uw_gather_u64base_offset_u64, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svldnt1uw_gather_u64base_s64, "q2Wiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldnt1uw_gather_u64base_u64, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svldnt1uw_gather_u64index_s64, "q2Wiq16bUiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1uw_gather_u64index_u64, "q2UWiq16bUiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1uw_gather_u64offset_s64, "q2Wiq16bUiC*q2UWi", "n")
BUILTIN(__builtin_sve_svldnt1uw_gather_u64offset_u64, "q2UWiq16bUiC*q2UWi", "n")
BUILTIN(__builtin_sve_svlen_bf16, "UWiq8y", "n")
BUILTIN(__builtin_sve_svlen_f16, "UWiq8h", "n")
BUILTIN(__builtin_sve_svlen_f32, "UWiq4f", "n")
BUILTIN(__builtin_sve_svlen_f64, "UWiq2d", "n")
BUILTIN(__builtin_sve_svlen_s16, "UWiq8s", "n")
BUILTIN(__builtin_sve_svlen_s32, "UWiq4i", "n")
BUILTIN(__builtin_sve_svlen_s64, "UWiq2Wi", "n")
BUILTIN(__builtin_sve_svlen_s8, "UWiq16Sc", "n")
BUILTIN(__builtin_sve_svlen_u16, "UWiq8Us", "n")
BUILTIN(__builtin_sve_svlen_u32, "UWiq4Ui", "n")
BUILTIN(__builtin_sve_svlen_u64, "UWiq2UWi", "n")
BUILTIN(__builtin_sve_svlen_u8, "UWiq16Uc", "n")
BUILTIN(__builtin_sve_svlogb_f16_m, "q8sq8sq16bq8h", "n")
BUILTIN(__builtin_sve_svlogb_f16_x, "q8sq16bq8h", "n")
BUILTIN(__builtin_sve_svlogb_f16_z, "q8sq16bq8h", "n")
BUILTIN(__builtin_sve_svlogb_f32_m, "q4iq4iq16bq4f", "n")
BUILTIN(__builtin_sve_svlogb_f32_x, "q4iq16bq4f", "n")
BUILTIN(__builtin_sve_svlogb_f32_z, "q4iq16bq4f", "n")
BUILTIN(__builtin_sve_svlogb_f64_m, "q2Wiq2Wiq16bq2d", "n")
BUILTIN(__builtin_sve_svlogb_f64_x, "q2Wiq16bq2d", "n")
BUILTIN(__builtin_sve_svlogb_f64_z, "q2Wiq16bq2d", "n")
BUILTIN(__builtin_sve_svlsl_n_s16_m, "q8sq16bq8sUs", "n")
BUILTIN(__builtin_sve_svlsl_n_s16_x, "q8sq16bq8sUs", "n")
BUILTIN(__builtin_sve_svlsl_n_s16_z, "q8sq16bq8sUs", "n")
BUILTIN(__builtin_sve_svlsl_n_s32_m, "q4iq16bq4iUi", "n")
BUILTIN(__builtin_sve_svlsl_n_s32_x, "q4iq16bq4iUi", "n")
BUILTIN(__builtin_sve_svlsl_n_s32_z, "q4iq16bq4iUi", "n")
BUILTIN(__builtin_sve_svlsl_n_s64_m, "q2Wiq16bq2WiUWi", "n")
BUILTIN(__builtin_sve_svlsl_n_s64_x, "q2Wiq16bq2WiUWi", "n")
BUILTIN(__builtin_sve_svlsl_n_s64_z, "q2Wiq16bq2WiUWi", "n")
BUILTIN(__builtin_sve_svlsl_n_s8_m, "q16Scq16bq16ScUc", "n")
BUILTIN(__builtin_sve_svlsl_n_s8_x, "q16Scq16bq16ScUc", "n")
BUILTIN(__builtin_sve_svlsl_n_s8_z, "q16Scq16bq16ScUc", "n")
BUILTIN(__builtin_sve_svlsl_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svlsl_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svlsl_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svlsl_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svlsl_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svlsl_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svlsl_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svlsl_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svlsl_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svlsl_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svlsl_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svlsl_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svlsl_s16_m, "q8sq16bq8sq8Us", "n")
BUILTIN(__builtin_sve_svlsl_s16_x, "q8sq16bq8sq8Us", "n")
BUILTIN(__builtin_sve_svlsl_s16_z, "q8sq16bq8sq8Us", "n")
BUILTIN(__builtin_sve_svlsl_s32_m, "q4iq16bq4iq4Ui", "n")
BUILTIN(__builtin_sve_svlsl_s32_x, "q4iq16bq4iq4Ui", "n")
BUILTIN(__builtin_sve_svlsl_s32_z, "q4iq16bq4iq4Ui", "n")
BUILTIN(__builtin_sve_svlsl_s64_m, "q2Wiq16bq2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_s64_x, "q2Wiq16bq2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_s64_z, "q2Wiq16bq2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_s8_m, "q16Scq16bq16Scq16Uc", "n")
BUILTIN(__builtin_sve_svlsl_s8_x, "q16Scq16bq16Scq16Uc", "n")
BUILTIN(__builtin_sve_svlsl_s8_z, "q16Scq16bq16Scq16Uc", "n")
BUILTIN(__builtin_sve_svlsl_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svlsl_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svlsl_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svlsl_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svlsl_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svlsl_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svlsl_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svlsl_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svlsl_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svlsl_wide_n_s16_m, "q8sq16bq8sUWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_n_s16_x, "q8sq16bq8sUWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_n_s16_z, "q8sq16bq8sUWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_n_s32_m, "q4iq16bq4iUWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_n_s32_x, "q4iq16bq4iUWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_n_s32_z, "q4iq16bq4iUWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_n_s8_m, "q16Scq16bq16ScUWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_n_s8_x, "q16Scq16bq16ScUWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_n_s8_z, "q16Scq16bq16ScUWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_n_u16_m, "q8Usq16bq8UsUWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_n_u16_x, "q8Usq16bq8UsUWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_n_u16_z, "q8Usq16bq8UsUWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_n_u32_m, "q4Uiq16bq4UiUWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_n_u32_x, "q4Uiq16bq4UiUWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_n_u32_z, "q4Uiq16bq4UiUWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_n_u8_m, "q16Ucq16bq16UcUWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_n_u8_x, "q16Ucq16bq16UcUWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_n_u8_z, "q16Ucq16bq16UcUWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_s16_m, "q8sq16bq8sq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_s16_x, "q8sq16bq8sq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_s16_z, "q8sq16bq8sq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_s32_m, "q4iq16bq4iq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_s32_x, "q4iq16bq4iq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_s32_z, "q4iq16bq4iq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_s8_m, "q16Scq16bq16Scq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_s8_x, "q16Scq16bq16Scq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_s8_z, "q16Scq16bq16Scq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_u16_m, "q8Usq16bq8Usq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_u16_x, "q8Usq16bq8Usq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_u16_z, "q8Usq16bq8Usq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_u32_m, "q4Uiq16bq4Uiq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_u32_x, "q4Uiq16bq4Uiq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_u32_z, "q4Uiq16bq4Uiq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_u8_m, "q16Ucq16bq16Ucq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_u8_x, "q16Ucq16bq16Ucq2UWi", "n")
BUILTIN(__builtin_sve_svlsl_wide_u8_z, "q16Ucq16bq16Ucq2UWi", "n")
BUILTIN(__builtin_sve_svlsr_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svlsr_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svlsr_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svlsr_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svlsr_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svlsr_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svlsr_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svlsr_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svlsr_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svlsr_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svlsr_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svlsr_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svlsr_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svlsr_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svlsr_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svlsr_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svlsr_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svlsr_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svlsr_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svlsr_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svlsr_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svlsr_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svlsr_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svlsr_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svlsr_wide_n_u16_m, "q8Usq16bq8UsUWi", "n")
BUILTIN(__builtin_sve_svlsr_wide_n_u16_x, "q8Usq16bq8UsUWi", "n")
BUILTIN(__builtin_sve_svlsr_wide_n_u16_z, "q8Usq16bq8UsUWi", "n")
BUILTIN(__builtin_sve_svlsr_wide_n_u32_m, "q4Uiq16bq4UiUWi", "n")
BUILTIN(__builtin_sve_svlsr_wide_n_u32_x, "q4Uiq16bq4UiUWi", "n")
BUILTIN(__builtin_sve_svlsr_wide_n_u32_z, "q4Uiq16bq4UiUWi", "n")
BUILTIN(__builtin_sve_svlsr_wide_n_u8_m, "q16Ucq16bq16UcUWi", "n")
BUILTIN(__builtin_sve_svlsr_wide_n_u8_x, "q16Ucq16bq16UcUWi", "n")
BUILTIN(__builtin_sve_svlsr_wide_n_u8_z, "q16Ucq16bq16UcUWi", "n")
BUILTIN(__builtin_sve_svlsr_wide_u16_m, "q8Usq16bq8Usq2UWi", "n")
BUILTIN(__builtin_sve_svlsr_wide_u16_x, "q8Usq16bq8Usq2UWi", "n")
BUILTIN(__builtin_sve_svlsr_wide_u16_z, "q8Usq16bq8Usq2UWi", "n")
BUILTIN(__builtin_sve_svlsr_wide_u32_m, "q4Uiq16bq4Uiq2UWi", "n")
BUILTIN(__builtin_sve_svlsr_wide_u32_x, "q4Uiq16bq4Uiq2UWi", "n")
BUILTIN(__builtin_sve_svlsr_wide_u32_z, "q4Uiq16bq4Uiq2UWi", "n")
BUILTIN(__builtin_sve_svlsr_wide_u8_m, "q16Ucq16bq16Ucq2UWi", "n")
BUILTIN(__builtin_sve_svlsr_wide_u8_x, "q16Ucq16bq16Ucq2UWi", "n")
BUILTIN(__builtin_sve_svlsr_wide_u8_z, "q16Ucq16bq16Ucq2UWi", "n")
BUILTIN(__builtin_sve_svmad_f16_m, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svmad_f16_x, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svmad_f16_z, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svmad_f32_m, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svmad_f32_x, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svmad_f32_z, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svmad_f64_m, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svmad_f64_x, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svmad_f64_z, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svmad_n_f16_m, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svmad_n_f16_x, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svmad_n_f16_z, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svmad_n_f32_m, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svmad_n_f32_x, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svmad_n_f32_z, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svmad_n_f64_m, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svmad_n_f64_x, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svmad_n_f64_z, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svmad_n_s16_m, "q8sq16bq8sq8ss", "n")
BUILTIN(__builtin_sve_svmad_n_s16_x, "q8sq16bq8sq8ss", "n")
BUILTIN(__builtin_sve_svmad_n_s16_z, "q8sq16bq8sq8ss", "n")
BUILTIN(__builtin_sve_svmad_n_s32_m, "q4iq16bq4iq4ii", "n")
BUILTIN(__builtin_sve_svmad_n_s32_x, "q4iq16bq4iq4ii", "n")
BUILTIN(__builtin_sve_svmad_n_s32_z, "q4iq16bq4iq4ii", "n")
BUILTIN(__builtin_sve_svmad_n_s64_m, "q2Wiq16bq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svmad_n_s64_x, "q2Wiq16bq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svmad_n_s64_z, "q2Wiq16bq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svmad_n_s8_m, "q16Scq16bq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svmad_n_s8_x, "q16Scq16bq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svmad_n_s8_z, "q16Scq16bq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svmad_n_u16_m, "q8Usq16bq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svmad_n_u16_x, "q8Usq16bq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svmad_n_u16_z, "q8Usq16bq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svmad_n_u32_m, "q4Uiq16bq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svmad_n_u32_x, "q4Uiq16bq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svmad_n_u32_z, "q4Uiq16bq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svmad_n_u64_m, "q2UWiq16bq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmad_n_u64_x, "q2UWiq16bq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmad_n_u64_z, "q2UWiq16bq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmad_n_u8_m, "q16Ucq16bq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svmad_n_u8_x, "q16Ucq16bq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svmad_n_u8_z, "q16Ucq16bq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svmad_s16_m, "q8sq16bq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svmad_s16_x, "q8sq16bq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svmad_s16_z, "q8sq16bq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svmad_s32_m, "q4iq16bq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svmad_s32_x, "q4iq16bq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svmad_s32_z, "q4iq16bq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svmad_s64_m, "q2Wiq16bq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmad_s64_x, "q2Wiq16bq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmad_s64_z, "q2Wiq16bq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmad_s8_m, "q16Scq16bq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmad_s8_x, "q16Scq16bq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmad_s8_z, "q16Scq16bq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmad_u16_m, "q8Usq16bq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmad_u16_x, "q8Usq16bq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmad_u16_z, "q8Usq16bq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmad_u32_m, "q4Uiq16bq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmad_u32_x, "q4Uiq16bq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmad_u32_z, "q4Uiq16bq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmad_u64_m, "q2UWiq16bq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmad_u64_x, "q2UWiq16bq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmad_u64_z, "q2UWiq16bq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmad_u8_m, "q16Ucq16bq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmad_u8_x, "q16Ucq16bq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmad_u8_z, "q16Ucq16bq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmatch_s16, "q16bq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svmatch_s8, "q16bq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmatch_u16, "q16bq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmatch_u8, "q16bq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmax_f16_m, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svmax_f16_x, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svmax_f16_z, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svmax_f32_m, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svmax_f32_x, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svmax_f32_z, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svmax_f64_m, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svmax_f64_x, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svmax_f64_z, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svmax_n_f16_m, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svmax_n_f16_x, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svmax_n_f16_z, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svmax_n_f32_m, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svmax_n_f32_x, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svmax_n_f32_z, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svmax_n_f64_m, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svmax_n_f64_x, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svmax_n_f64_z, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svmax_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svmax_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svmax_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svmax_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svmax_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svmax_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svmax_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svmax_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svmax_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svmax_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svmax_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svmax_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svmax_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svmax_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svmax_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svmax_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svmax_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svmax_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svmax_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmax_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmax_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmax_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svmax_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svmax_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svmax_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svmax_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svmax_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svmax_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svmax_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svmax_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svmax_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmax_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmax_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmax_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmax_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmax_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmax_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmax_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmax_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmax_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmax_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmax_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmax_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmax_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmax_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmax_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmax_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmax_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmaxnm_f16_m, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svmaxnm_f16_x, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svmaxnm_f16_z, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svmaxnm_f32_m, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svmaxnm_f32_x, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svmaxnm_f32_z, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svmaxnm_f64_m, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svmaxnm_f64_x, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svmaxnm_f64_z, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svmaxnm_n_f16_m, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svmaxnm_n_f16_x, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svmaxnm_n_f16_z, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svmaxnm_n_f32_m, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svmaxnm_n_f32_x, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svmaxnm_n_f32_z, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svmaxnm_n_f64_m, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svmaxnm_n_f64_x, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svmaxnm_n_f64_z, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svmaxnmp_f16_m, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svmaxnmp_f16_x, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svmaxnmp_f32_m, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svmaxnmp_f32_x, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svmaxnmp_f64_m, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svmaxnmp_f64_x, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svmaxnmv_f16, "hq16bq8h", "n")
BUILTIN(__builtin_sve_svmaxnmv_f32, "fq16bq4f", "n")
BUILTIN(__builtin_sve_svmaxnmv_f64, "dq16bq2d", "n")
BUILTIN(__builtin_sve_svmaxp_f16_m, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svmaxp_f16_x, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svmaxp_f32_m, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svmaxp_f32_x, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svmaxp_f64_m, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svmaxp_f64_x, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svmaxp_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svmaxp_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svmaxp_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svmaxp_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svmaxp_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmaxp_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmaxp_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmaxp_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmaxp_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmaxp_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmaxp_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmaxp_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmaxp_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmaxp_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmaxp_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmaxp_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmaxv_f16, "hq16bq8h", "n")
BUILTIN(__builtin_sve_svmaxv_f32, "fq16bq4f", "n")
BUILTIN(__builtin_sve_svmaxv_f64, "dq16bq2d", "n")
BUILTIN(__builtin_sve_svmaxv_s16, "sq16bq8s", "n")
BUILTIN(__builtin_sve_svmaxv_s32, "iq16bq4i", "n")
BUILTIN(__builtin_sve_svmaxv_s64, "Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svmaxv_s8, "Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svmaxv_u16, "Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svmaxv_u32, "Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svmaxv_u64, "UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svmaxv_u8, "Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svmin_f16_m, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svmin_f16_x, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svmin_f16_z, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svmin_f32_m, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svmin_f32_x, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svmin_f32_z, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svmin_f64_m, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svmin_f64_x, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svmin_f64_z, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svmin_n_f16_m, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svmin_n_f16_x, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svmin_n_f16_z, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svmin_n_f32_m, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svmin_n_f32_x, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svmin_n_f32_z, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svmin_n_f64_m, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svmin_n_f64_x, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svmin_n_f64_z, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svmin_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svmin_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svmin_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svmin_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svmin_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svmin_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svmin_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svmin_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svmin_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svmin_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svmin_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svmin_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svmin_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svmin_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svmin_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svmin_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svmin_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svmin_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svmin_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmin_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmin_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmin_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svmin_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svmin_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svmin_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svmin_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svmin_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svmin_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svmin_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svmin_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svmin_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmin_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmin_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmin_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmin_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmin_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmin_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmin_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmin_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmin_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmin_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmin_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmin_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmin_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmin_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmin_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmin_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmin_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svminnm_f16_m, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svminnm_f16_x, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svminnm_f16_z, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svminnm_f32_m, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svminnm_f32_x, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svminnm_f32_z, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svminnm_f64_m, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svminnm_f64_x, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svminnm_f64_z, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svminnm_n_f16_m, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svminnm_n_f16_x, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svminnm_n_f16_z, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svminnm_n_f32_m, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svminnm_n_f32_x, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svminnm_n_f32_z, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svminnm_n_f64_m, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svminnm_n_f64_x, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svminnm_n_f64_z, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svminnmp_f16_m, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svminnmp_f16_x, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svminnmp_f32_m, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svminnmp_f32_x, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svminnmp_f64_m, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svminnmp_f64_x, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svminnmv_f16, "hq16bq8h", "n")
BUILTIN(__builtin_sve_svminnmv_f32, "fq16bq4f", "n")
BUILTIN(__builtin_sve_svminnmv_f64, "dq16bq2d", "n")
BUILTIN(__builtin_sve_svminp_f16_m, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svminp_f16_x, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svminp_f32_m, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svminp_f32_x, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svminp_f64_m, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svminp_f64_x, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svminp_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svminp_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svminp_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svminp_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svminp_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svminp_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svminp_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svminp_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svminp_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svminp_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svminp_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svminp_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svminp_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svminp_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svminp_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svminp_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svminv_f16, "hq16bq8h", "n")
BUILTIN(__builtin_sve_svminv_f32, "fq16bq4f", "n")
BUILTIN(__builtin_sve_svminv_f64, "dq16bq2d", "n")
BUILTIN(__builtin_sve_svminv_s16, "sq16bq8s", "n")
BUILTIN(__builtin_sve_svminv_s32, "iq16bq4i", "n")
BUILTIN(__builtin_sve_svminv_s64, "Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svminv_s8, "Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svminv_u16, "Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svminv_u32, "Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svminv_u64, "UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svminv_u8, "Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svmla_f16_m, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svmla_f16_x, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svmla_f16_z, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svmla_f32_m, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svmla_f32_x, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svmla_f32_z, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svmla_f64_m, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svmla_f64_x, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svmla_f64_z, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svmla_lane_f16, "q8hq8hq8hq8hIUWi", "n")
BUILTIN(__builtin_sve_svmla_lane_f32, "q4fq4fq4fq4fIUWi", "n")
BUILTIN(__builtin_sve_svmla_lane_f64, "q2dq2dq2dq2dIUWi", "n")
BUILTIN(__builtin_sve_svmla_lane_s16, "q8sq8sq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svmla_lane_s32, "q4iq4iq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svmla_lane_s64, "q2Wiq2Wiq2Wiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svmla_lane_u16, "q8Usq8Usq8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svmla_lane_u32, "q4Uiq4Uiq4Uiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svmla_lane_u64, "q2UWiq2UWiq2UWiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svmla_n_f16_m, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svmla_n_f16_x, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svmla_n_f16_z, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svmla_n_f32_m, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svmla_n_f32_x, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svmla_n_f32_z, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svmla_n_f64_m, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svmla_n_f64_x, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svmla_n_f64_z, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svmla_n_s16_m, "q8sq16bq8sq8ss", "n")
BUILTIN(__builtin_sve_svmla_n_s16_x, "q8sq16bq8sq8ss", "n")
BUILTIN(__builtin_sve_svmla_n_s16_z, "q8sq16bq8sq8ss", "n")
BUILTIN(__builtin_sve_svmla_n_s32_m, "q4iq16bq4iq4ii", "n")
BUILTIN(__builtin_sve_svmla_n_s32_x, "q4iq16bq4iq4ii", "n")
BUILTIN(__builtin_sve_svmla_n_s32_z, "q4iq16bq4iq4ii", "n")
BUILTIN(__builtin_sve_svmla_n_s64_m, "q2Wiq16bq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svmla_n_s64_x, "q2Wiq16bq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svmla_n_s64_z, "q2Wiq16bq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svmla_n_s8_m, "q16Scq16bq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svmla_n_s8_x, "q16Scq16bq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svmla_n_s8_z, "q16Scq16bq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svmla_n_u16_m, "q8Usq16bq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svmla_n_u16_x, "q8Usq16bq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svmla_n_u16_z, "q8Usq16bq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svmla_n_u32_m, "q4Uiq16bq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svmla_n_u32_x, "q4Uiq16bq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svmla_n_u32_z, "q4Uiq16bq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svmla_n_u64_m, "q2UWiq16bq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmla_n_u64_x, "q2UWiq16bq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmla_n_u64_z, "q2UWiq16bq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmla_n_u8_m, "q16Ucq16bq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svmla_n_u8_x, "q16Ucq16bq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svmla_n_u8_z, "q16Ucq16bq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svmla_s16_m, "q8sq16bq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svmla_s16_x, "q8sq16bq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svmla_s16_z, "q8sq16bq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svmla_s32_m, "q4iq16bq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svmla_s32_x, "q4iq16bq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svmla_s32_z, "q4iq16bq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svmla_s64_m, "q2Wiq16bq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmla_s64_x, "q2Wiq16bq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmla_s64_z, "q2Wiq16bq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmla_s8_m, "q16Scq16bq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmla_s8_x, "q16Scq16bq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmla_s8_z, "q16Scq16bq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmla_u16_m, "q8Usq16bq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmla_u16_x, "q8Usq16bq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmla_u16_z, "q8Usq16bq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmla_u32_m, "q4Uiq16bq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmla_u32_x, "q4Uiq16bq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmla_u32_z, "q4Uiq16bq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmla_u64_m, "q2UWiq16bq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmla_u64_x, "q2UWiq16bq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmla_u64_z, "q2UWiq16bq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmla_u8_m, "q16Ucq16bq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmla_u8_x, "q16Ucq16bq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmla_u8_z, "q16Ucq16bq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmlalb_f32, "q4fq4fq8hq8h", "n")
BUILTIN(__builtin_sve_svmlalb_lane_f32, "q4fq4fq8hq8hIUWi", "n")
BUILTIN(__builtin_sve_svmlalb_lane_s32, "q4iq4iq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svmlalb_lane_s64, "q2Wiq2Wiq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svmlalb_lane_u32, "q4Uiq4Uiq8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svmlalb_lane_u64, "q2UWiq2UWiq4Uiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svmlalb_n_f32, "q4fq4fq8hh", "n")
BUILTIN(__builtin_sve_svmlalb_n_s16, "q8sq8sq16ScSc", "n")
BUILTIN(__builtin_sve_svmlalb_n_s32, "q4iq4iq8ss", "n")
BUILTIN(__builtin_sve_svmlalb_n_s64, "q2Wiq2Wiq4ii", "n")
BUILTIN(__builtin_sve_svmlalb_n_u16, "q8Usq8Usq16UcUc", "n")
BUILTIN(__builtin_sve_svmlalb_n_u32, "q4Uiq4Uiq8UsUs", "n")
BUILTIN(__builtin_sve_svmlalb_n_u64, "q2UWiq2UWiq4UiUi", "n")
BUILTIN(__builtin_sve_svmlalb_s16, "q8sq8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmlalb_s32, "q4iq4iq8sq8s", "n")
BUILTIN(__builtin_sve_svmlalb_s64, "q2Wiq2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svmlalb_u16, "q8Usq8Usq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmlalb_u32, "q4Uiq4Uiq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmlalb_u64, "q2UWiq2UWiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmlalt_f32, "q4fq4fq8hq8h", "n")
BUILTIN(__builtin_sve_svmlalt_lane_f32, "q4fq4fq8hq8hIUWi", "n")
BUILTIN(__builtin_sve_svmlalt_lane_s32, "q4iq4iq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svmlalt_lane_s64, "q2Wiq2Wiq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svmlalt_lane_u32, "q4Uiq4Uiq8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svmlalt_lane_u64, "q2UWiq2UWiq4Uiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svmlalt_n_f32, "q4fq4fq8hh", "n")
BUILTIN(__builtin_sve_svmlalt_n_s16, "q8sq8sq16ScSc", "n")
BUILTIN(__builtin_sve_svmlalt_n_s32, "q4iq4iq8ss", "n")
BUILTIN(__builtin_sve_svmlalt_n_s64, "q2Wiq2Wiq4ii", "n")
BUILTIN(__builtin_sve_svmlalt_n_u16, "q8Usq8Usq16UcUc", "n")
BUILTIN(__builtin_sve_svmlalt_n_u32, "q4Uiq4Uiq8UsUs", "n")
BUILTIN(__builtin_sve_svmlalt_n_u64, "q2UWiq2UWiq4UiUi", "n")
BUILTIN(__builtin_sve_svmlalt_s16, "q8sq8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmlalt_s32, "q4iq4iq8sq8s", "n")
BUILTIN(__builtin_sve_svmlalt_s64, "q2Wiq2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svmlalt_u16, "q8Usq8Usq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmlalt_u32, "q4Uiq4Uiq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmlalt_u64, "q2UWiq2UWiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmls_f16_m, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svmls_f16_x, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svmls_f16_z, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svmls_f32_m, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svmls_f32_x, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svmls_f32_z, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svmls_f64_m, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svmls_f64_x, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svmls_f64_z, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svmls_lane_f16, "q8hq8hq8hq8hIUWi", "n")
BUILTIN(__builtin_sve_svmls_lane_f32, "q4fq4fq4fq4fIUWi", "n")
BUILTIN(__builtin_sve_svmls_lane_f64, "q2dq2dq2dq2dIUWi", "n")
BUILTIN(__builtin_sve_svmls_lane_s16, "q8sq8sq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svmls_lane_s32, "q4iq4iq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svmls_lane_s64, "q2Wiq2Wiq2Wiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svmls_lane_u16, "q8Usq8Usq8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svmls_lane_u32, "q4Uiq4Uiq4Uiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svmls_lane_u64, "q2UWiq2UWiq2UWiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svmls_n_f16_m, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svmls_n_f16_x, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svmls_n_f16_z, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svmls_n_f32_m, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svmls_n_f32_x, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svmls_n_f32_z, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svmls_n_f64_m, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svmls_n_f64_x, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svmls_n_f64_z, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svmls_n_s16_m, "q8sq16bq8sq8ss", "n")
BUILTIN(__builtin_sve_svmls_n_s16_x, "q8sq16bq8sq8ss", "n")
BUILTIN(__builtin_sve_svmls_n_s16_z, "q8sq16bq8sq8ss", "n")
BUILTIN(__builtin_sve_svmls_n_s32_m, "q4iq16bq4iq4ii", "n")
BUILTIN(__builtin_sve_svmls_n_s32_x, "q4iq16bq4iq4ii", "n")
BUILTIN(__builtin_sve_svmls_n_s32_z, "q4iq16bq4iq4ii", "n")
BUILTIN(__builtin_sve_svmls_n_s64_m, "q2Wiq16bq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svmls_n_s64_x, "q2Wiq16bq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svmls_n_s64_z, "q2Wiq16bq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svmls_n_s8_m, "q16Scq16bq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svmls_n_s8_x, "q16Scq16bq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svmls_n_s8_z, "q16Scq16bq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svmls_n_u16_m, "q8Usq16bq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svmls_n_u16_x, "q8Usq16bq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svmls_n_u16_z, "q8Usq16bq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svmls_n_u32_m, "q4Uiq16bq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svmls_n_u32_x, "q4Uiq16bq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svmls_n_u32_z, "q4Uiq16bq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svmls_n_u64_m, "q2UWiq16bq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmls_n_u64_x, "q2UWiq16bq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmls_n_u64_z, "q2UWiq16bq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmls_n_u8_m, "q16Ucq16bq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svmls_n_u8_x, "q16Ucq16bq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svmls_n_u8_z, "q16Ucq16bq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svmls_s16_m, "q8sq16bq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svmls_s16_x, "q8sq16bq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svmls_s16_z, "q8sq16bq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svmls_s32_m, "q4iq16bq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svmls_s32_x, "q4iq16bq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svmls_s32_z, "q4iq16bq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svmls_s64_m, "q2Wiq16bq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmls_s64_x, "q2Wiq16bq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmls_s64_z, "q2Wiq16bq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmls_s8_m, "q16Scq16bq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmls_s8_x, "q16Scq16bq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmls_s8_z, "q16Scq16bq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmls_u16_m, "q8Usq16bq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmls_u16_x, "q8Usq16bq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmls_u16_z, "q8Usq16bq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmls_u32_m, "q4Uiq16bq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmls_u32_x, "q4Uiq16bq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmls_u32_z, "q4Uiq16bq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmls_u64_m, "q2UWiq16bq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmls_u64_x, "q2UWiq16bq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmls_u64_z, "q2UWiq16bq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmls_u8_m, "q16Ucq16bq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmls_u8_x, "q16Ucq16bq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmls_u8_z, "q16Ucq16bq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmlslb_f32, "q4fq4fq8hq8h", "n")
BUILTIN(__builtin_sve_svmlslb_lane_f32, "q4fq4fq8hq8hIUWi", "n")
BUILTIN(__builtin_sve_svmlslb_lane_s32, "q4iq4iq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svmlslb_lane_s64, "q2Wiq2Wiq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svmlslb_lane_u32, "q4Uiq4Uiq8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svmlslb_lane_u64, "q2UWiq2UWiq4Uiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svmlslb_n_f32, "q4fq4fq8hh", "n")
BUILTIN(__builtin_sve_svmlslb_n_s16, "q8sq8sq16ScSc", "n")
BUILTIN(__builtin_sve_svmlslb_n_s32, "q4iq4iq8ss", "n")
BUILTIN(__builtin_sve_svmlslb_n_s64, "q2Wiq2Wiq4ii", "n")
BUILTIN(__builtin_sve_svmlslb_n_u16, "q8Usq8Usq16UcUc", "n")
BUILTIN(__builtin_sve_svmlslb_n_u32, "q4Uiq4Uiq8UsUs", "n")
BUILTIN(__builtin_sve_svmlslb_n_u64, "q2UWiq2UWiq4UiUi", "n")
BUILTIN(__builtin_sve_svmlslb_s16, "q8sq8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmlslb_s32, "q4iq4iq8sq8s", "n")
BUILTIN(__builtin_sve_svmlslb_s64, "q2Wiq2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svmlslb_u16, "q8Usq8Usq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmlslb_u32, "q4Uiq4Uiq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmlslb_u64, "q2UWiq2UWiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmlslt_f32, "q4fq4fq8hq8h", "n")
BUILTIN(__builtin_sve_svmlslt_lane_f32, "q4fq4fq8hq8hIUWi", "n")
BUILTIN(__builtin_sve_svmlslt_lane_s32, "q4iq4iq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svmlslt_lane_s64, "q2Wiq2Wiq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svmlslt_lane_u32, "q4Uiq4Uiq8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svmlslt_lane_u64, "q2UWiq2UWiq4Uiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svmlslt_n_f32, "q4fq4fq8hh", "n")
BUILTIN(__builtin_sve_svmlslt_n_s16, "q8sq8sq16ScSc", "n")
BUILTIN(__builtin_sve_svmlslt_n_s32, "q4iq4iq8ss", "n")
BUILTIN(__builtin_sve_svmlslt_n_s64, "q2Wiq2Wiq4ii", "n")
BUILTIN(__builtin_sve_svmlslt_n_u16, "q8Usq8Usq16UcUc", "n")
BUILTIN(__builtin_sve_svmlslt_n_u32, "q4Uiq4Uiq8UsUs", "n")
BUILTIN(__builtin_sve_svmlslt_n_u64, "q2UWiq2UWiq4UiUi", "n")
BUILTIN(__builtin_sve_svmlslt_s16, "q8sq8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmlslt_s32, "q4iq4iq8sq8s", "n")
BUILTIN(__builtin_sve_svmlslt_s64, "q2Wiq2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svmlslt_u16, "q8Usq8Usq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmlslt_u32, "q4Uiq4Uiq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmlslt_u64, "q2UWiq2UWiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmmla_f32, "q4fq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svmmla_f64, "q2dq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svmmla_s32, "q4iq4iq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmmla_u32, "q4Uiq4Uiq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmov_b_z, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svmovlb_s16, "q8sq16Sc", "n")
BUILTIN(__builtin_sve_svmovlb_s32, "q4iq8s", "n")
BUILTIN(__builtin_sve_svmovlb_s64, "q2Wiq4i", "n")
BUILTIN(__builtin_sve_svmovlb_u16, "q8Usq16Uc", "n")
BUILTIN(__builtin_sve_svmovlb_u32, "q4Uiq8Us", "n")
BUILTIN(__builtin_sve_svmovlb_u64, "q2UWiq4Ui", "n")
BUILTIN(__builtin_sve_svmovlt_s16, "q8sq16Sc", "n")
BUILTIN(__builtin_sve_svmovlt_s32, "q4iq8s", "n")
BUILTIN(__builtin_sve_svmovlt_s64, "q2Wiq4i", "n")
BUILTIN(__builtin_sve_svmovlt_u16, "q8Usq16Uc", "n")
BUILTIN(__builtin_sve_svmovlt_u32, "q4Uiq8Us", "n")
BUILTIN(__builtin_sve_svmovlt_u64, "q2UWiq4Ui", "n")
BUILTIN(__builtin_sve_svmsb_f16_m, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svmsb_f16_x, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svmsb_f16_z, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svmsb_f32_m, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svmsb_f32_x, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svmsb_f32_z, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svmsb_f64_m, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svmsb_f64_x, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svmsb_f64_z, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svmsb_n_f16_m, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svmsb_n_f16_x, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svmsb_n_f16_z, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svmsb_n_f32_m, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svmsb_n_f32_x, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svmsb_n_f32_z, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svmsb_n_f64_m, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svmsb_n_f64_x, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svmsb_n_f64_z, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svmsb_n_s16_m, "q8sq16bq8sq8ss", "n")
BUILTIN(__builtin_sve_svmsb_n_s16_x, "q8sq16bq8sq8ss", "n")
BUILTIN(__builtin_sve_svmsb_n_s16_z, "q8sq16bq8sq8ss", "n")
BUILTIN(__builtin_sve_svmsb_n_s32_m, "q4iq16bq4iq4ii", "n")
BUILTIN(__builtin_sve_svmsb_n_s32_x, "q4iq16bq4iq4ii", "n")
BUILTIN(__builtin_sve_svmsb_n_s32_z, "q4iq16bq4iq4ii", "n")
BUILTIN(__builtin_sve_svmsb_n_s64_m, "q2Wiq16bq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svmsb_n_s64_x, "q2Wiq16bq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svmsb_n_s64_z, "q2Wiq16bq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svmsb_n_s8_m, "q16Scq16bq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svmsb_n_s8_x, "q16Scq16bq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svmsb_n_s8_z, "q16Scq16bq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svmsb_n_u16_m, "q8Usq16bq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svmsb_n_u16_x, "q8Usq16bq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svmsb_n_u16_z, "q8Usq16bq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svmsb_n_u32_m, "q4Uiq16bq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svmsb_n_u32_x, "q4Uiq16bq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svmsb_n_u32_z, "q4Uiq16bq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svmsb_n_u64_m, "q2UWiq16bq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmsb_n_u64_x, "q2UWiq16bq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmsb_n_u64_z, "q2UWiq16bq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmsb_n_u8_m, "q16Ucq16bq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svmsb_n_u8_x, "q16Ucq16bq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svmsb_n_u8_z, "q16Ucq16bq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svmsb_s16_m, "q8sq16bq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svmsb_s16_x, "q8sq16bq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svmsb_s16_z, "q8sq16bq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svmsb_s32_m, "q4iq16bq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svmsb_s32_x, "q4iq16bq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svmsb_s32_z, "q4iq16bq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svmsb_s64_m, "q2Wiq16bq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmsb_s64_x, "q2Wiq16bq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmsb_s64_z, "q2Wiq16bq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmsb_s8_m, "q16Scq16bq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmsb_s8_x, "q16Scq16bq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmsb_s8_z, "q16Scq16bq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmsb_u16_m, "q8Usq16bq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmsb_u16_x, "q8Usq16bq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmsb_u16_z, "q8Usq16bq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmsb_u32_m, "q4Uiq16bq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmsb_u32_x, "q4Uiq16bq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmsb_u32_z, "q4Uiq16bq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmsb_u64_m, "q2UWiq16bq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmsb_u64_x, "q2UWiq16bq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmsb_u64_z, "q2UWiq16bq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmsb_u8_m, "q16Ucq16bq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmsb_u8_x, "q16Ucq16bq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmsb_u8_z, "q16Ucq16bq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmul_f16_m, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svmul_f16_x, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svmul_f16_z, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svmul_f32_m, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svmul_f32_x, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svmul_f32_z, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svmul_f64_m, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svmul_f64_x, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svmul_f64_z, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svmul_lane_f16, "q8hq8hq8hIUWi", "n")
BUILTIN(__builtin_sve_svmul_lane_f32, "q4fq4fq4fIUWi", "n")
BUILTIN(__builtin_sve_svmul_lane_f64, "q2dq2dq2dIUWi", "n")
BUILTIN(__builtin_sve_svmul_lane_s16, "q8sq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svmul_lane_s32, "q4iq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svmul_lane_s64, "q2Wiq2Wiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svmul_lane_u16, "q8Usq8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svmul_lane_u32, "q4Uiq4Uiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svmul_lane_u64, "q2UWiq2UWiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svmul_n_f16_m, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svmul_n_f16_x, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svmul_n_f16_z, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svmul_n_f32_m, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svmul_n_f32_x, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svmul_n_f32_z, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svmul_n_f64_m, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svmul_n_f64_x, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svmul_n_f64_z, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svmul_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svmul_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svmul_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svmul_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svmul_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svmul_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svmul_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svmul_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svmul_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svmul_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svmul_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svmul_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svmul_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svmul_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svmul_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svmul_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svmul_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svmul_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svmul_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmul_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmul_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmul_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svmul_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svmul_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svmul_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svmul_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svmul_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svmul_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svmul_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svmul_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svmul_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmul_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmul_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmul_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmul_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmul_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmul_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmul_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmul_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmul_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmul_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmul_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmul_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmul_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmul_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmul_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmul_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmul_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmulh_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svmulh_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svmulh_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svmulh_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svmulh_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svmulh_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svmulh_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svmulh_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svmulh_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svmulh_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svmulh_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svmulh_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svmulh_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svmulh_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svmulh_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svmulh_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svmulh_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svmulh_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svmulh_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmulh_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmulh_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svmulh_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svmulh_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svmulh_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svmulh_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svmulh_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svmulh_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svmulh_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svmulh_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svmulh_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svmulh_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmulh_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmulh_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svmulh_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmulh_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmulh_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmulh_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmulh_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmulh_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmulh_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmulh_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmulh_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmulh_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmulh_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmulh_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svmulh_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmulh_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmulh_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmullb_lane_s32, "q4iq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svmullb_lane_s64, "q2Wiq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svmullb_lane_u32, "q4Uiq8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svmullb_lane_u64, "q2UWiq4Uiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svmullb_n_s16, "q8sq16ScSc", "n")
BUILTIN(__builtin_sve_svmullb_n_s32, "q4iq8ss", "n")
BUILTIN(__builtin_sve_svmullb_n_s64, "q2Wiq4ii", "n")
BUILTIN(__builtin_sve_svmullb_n_u16, "q8Usq16UcUc", "n")
BUILTIN(__builtin_sve_svmullb_n_u32, "q4Uiq8UsUs", "n")
BUILTIN(__builtin_sve_svmullb_n_u64, "q2UWiq4UiUi", "n")
BUILTIN(__builtin_sve_svmullb_s16, "q8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmullb_s32, "q4iq8sq8s", "n")
BUILTIN(__builtin_sve_svmullb_s64, "q2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svmullb_u16, "q8Usq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmullb_u32, "q4Uiq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmullb_u64, "q2UWiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmullt_lane_s32, "q4iq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svmullt_lane_s64, "q2Wiq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svmullt_lane_u32, "q4Uiq8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svmullt_lane_u64, "q2UWiq4Uiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svmullt_n_s16, "q8sq16ScSc", "n")
BUILTIN(__builtin_sve_svmullt_n_s32, "q4iq8ss", "n")
BUILTIN(__builtin_sve_svmullt_n_s64, "q2Wiq4ii", "n")
BUILTIN(__builtin_sve_svmullt_n_u16, "q8Usq16UcUc", "n")
BUILTIN(__builtin_sve_svmullt_n_u32, "q4Uiq8UsUs", "n")
BUILTIN(__builtin_sve_svmullt_n_u64, "q2UWiq4UiUi", "n")
BUILTIN(__builtin_sve_svmullt_s16, "q8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svmullt_s32, "q4iq8sq8s", "n")
BUILTIN(__builtin_sve_svmullt_s64, "q2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svmullt_u16, "q8Usq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svmullt_u32, "q4Uiq8Usq8Us", "n")
BUILTIN(__builtin_sve_svmullt_u64, "q2UWiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svmulx_f16_m, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svmulx_f16_x, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svmulx_f16_z, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svmulx_f32_m, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svmulx_f32_x, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svmulx_f32_z, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svmulx_f64_m, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svmulx_f64_x, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svmulx_f64_z, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svmulx_n_f16_m, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svmulx_n_f16_x, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svmulx_n_f16_z, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svmulx_n_f32_m, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svmulx_n_f32_x, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svmulx_n_f32_z, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svmulx_n_f64_m, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svmulx_n_f64_x, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svmulx_n_f64_z, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svnand_b_z, "q16bq16bq16bq16b", "n")
BUILTIN(__builtin_sve_svnbsl_n_s16, "q8sq8sq8ss", "n")
BUILTIN(__builtin_sve_svnbsl_n_s32, "q4iq4iq4ii", "n")
BUILTIN(__builtin_sve_svnbsl_n_s64, "q2Wiq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svnbsl_n_s8, "q16Scq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svnbsl_n_u16, "q8Usq8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svnbsl_n_u32, "q4Uiq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svnbsl_n_u64, "q2UWiq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svnbsl_n_u8, "q16Ucq16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svnbsl_s16, "q8sq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svnbsl_s32, "q4iq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svnbsl_s64, "q2Wiq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svnbsl_s8, "q16Scq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svnbsl_u16, "q8Usq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svnbsl_u32, "q4Uiq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svnbsl_u64, "q2UWiq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svnbsl_u8, "q16Ucq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svneg_f16_m, "q8hq8hq16bq8h", "n")
BUILTIN(__builtin_sve_svneg_f16_x, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svneg_f16_z, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svneg_f32_m, "q4fq4fq16bq4f", "n")
BUILTIN(__builtin_sve_svneg_f32_x, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svneg_f32_z, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svneg_f64_m, "q2dq2dq16bq2d", "n")
BUILTIN(__builtin_sve_svneg_f64_x, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svneg_f64_z, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svneg_s16_m, "q8sq8sq16bq8s", "n")
BUILTIN(__builtin_sve_svneg_s16_x, "q8sq16bq8s", "n")
BUILTIN(__builtin_sve_svneg_s16_z, "q8sq16bq8s", "n")
BUILTIN(__builtin_sve_svneg_s32_m, "q4iq4iq16bq4i", "n")
BUILTIN(__builtin_sve_svneg_s32_x, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svneg_s32_z, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svneg_s64_m, "q2Wiq2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svneg_s64_x, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svneg_s64_z, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svneg_s8_m, "q16Scq16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svneg_s8_x, "q16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svneg_s8_z, "q16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svnmad_f16_m, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svnmad_f16_x, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svnmad_f16_z, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svnmad_f32_m, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svnmad_f32_x, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svnmad_f32_z, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svnmad_f64_m, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svnmad_f64_x, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svnmad_f64_z, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svnmad_n_f16_m, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svnmad_n_f16_x, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svnmad_n_f16_z, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svnmad_n_f32_m, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svnmad_n_f32_x, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svnmad_n_f32_z, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svnmad_n_f64_m, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svnmad_n_f64_x, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svnmad_n_f64_z, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svnmatch_s16, "q16bq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svnmatch_s8, "q16bq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svnmatch_u16, "q16bq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svnmatch_u8, "q16bq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svnmla_f16_m, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svnmla_f16_x, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svnmla_f16_z, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svnmla_f32_m, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svnmla_f32_x, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svnmla_f32_z, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svnmla_f64_m, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svnmla_f64_x, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svnmla_f64_z, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svnmla_n_f16_m, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svnmla_n_f16_x, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svnmla_n_f16_z, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svnmla_n_f32_m, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svnmla_n_f32_x, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svnmla_n_f32_z, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svnmla_n_f64_m, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svnmla_n_f64_x, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svnmla_n_f64_z, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svnmls_f16_m, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svnmls_f16_x, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svnmls_f16_z, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svnmls_f32_m, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svnmls_f32_x, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svnmls_f32_z, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svnmls_f64_m, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svnmls_f64_x, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svnmls_f64_z, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svnmls_n_f16_m, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svnmls_n_f16_x, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svnmls_n_f16_z, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svnmls_n_f32_m, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svnmls_n_f32_x, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svnmls_n_f32_z, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svnmls_n_f64_m, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svnmls_n_f64_x, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svnmls_n_f64_z, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svnmsb_f16_m, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svnmsb_f16_x, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svnmsb_f16_z, "q8hq16bq8hq8hq8h", "n")
BUILTIN(__builtin_sve_svnmsb_f32_m, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svnmsb_f32_x, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svnmsb_f32_z, "q4fq16bq4fq4fq4f", "n")
BUILTIN(__builtin_sve_svnmsb_f64_m, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svnmsb_f64_x, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svnmsb_f64_z, "q2dq16bq2dq2dq2d", "n")
BUILTIN(__builtin_sve_svnmsb_n_f16_m, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svnmsb_n_f16_x, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svnmsb_n_f16_z, "q8hq16bq8hq8hh", "n")
BUILTIN(__builtin_sve_svnmsb_n_f32_m, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svnmsb_n_f32_x, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svnmsb_n_f32_z, "q4fq16bq4fq4ff", "n")
BUILTIN(__builtin_sve_svnmsb_n_f64_m, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svnmsb_n_f64_x, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svnmsb_n_f64_z, "q2dq16bq2dq2dd", "n")
BUILTIN(__builtin_sve_svnor_b_z, "q16bq16bq16bq16b", "n")
BUILTIN(__builtin_sve_svnot_b_z, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svnot_s16_m, "q8sq8sq16bq8s", "n")
BUILTIN(__builtin_sve_svnot_s16_x, "q8sq16bq8s", "n")
BUILTIN(__builtin_sve_svnot_s16_z, "q8sq16bq8s", "n")
BUILTIN(__builtin_sve_svnot_s32_m, "q4iq4iq16bq4i", "n")
BUILTIN(__builtin_sve_svnot_s32_x, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svnot_s32_z, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svnot_s64_m, "q2Wiq2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svnot_s64_x, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svnot_s64_z, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svnot_s8_m, "q16Scq16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svnot_s8_x, "q16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svnot_s8_z, "q16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svnot_u16_m, "q8Usq8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svnot_u16_x, "q8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svnot_u16_z, "q8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svnot_u32_m, "q4Uiq4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svnot_u32_x, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svnot_u32_z, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svnot_u64_m, "q2UWiq2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svnot_u64_x, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svnot_u64_z, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svnot_u8_m, "q16Ucq16Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svnot_u8_x, "q16Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svnot_u8_z, "q16Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svorn_b_z, "q16bq16bq16bq16b", "n")
BUILTIN(__builtin_sve_svorr_b_z, "q16bq16bq16bq16b", "n")
BUILTIN(__builtin_sve_svorr_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svorr_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svorr_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svorr_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svorr_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svorr_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svorr_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svorr_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svorr_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svorr_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svorr_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svorr_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svorr_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svorr_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svorr_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svorr_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svorr_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svorr_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svorr_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svorr_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svorr_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svorr_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svorr_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svorr_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svorr_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svorr_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svorr_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svorr_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svorr_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svorr_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svorr_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svorr_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svorr_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svorr_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svorr_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svorr_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svorr_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svorr_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svorr_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svorr_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svorr_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svorr_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svorr_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svorr_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svorr_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svorr_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svorr_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svorr_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svorv_s16, "sq16bq8s", "n")
BUILTIN(__builtin_sve_svorv_s32, "iq16bq4i", "n")
BUILTIN(__builtin_sve_svorv_s64, "Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svorv_s8, "Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svorv_u16, "Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svorv_u32, "Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svorv_u64, "UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svorv_u8, "Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svpfalse_b, "q16b", "n")
BUILTIN(__builtin_sve_svpfirst_b, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svpmul_n_u8, "q16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svpmul_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svpmullb_n_u16, "q8Usq16UcUc", "n")
BUILTIN(__builtin_sve_svpmullb_n_u64, "q2UWiq4UiUi", "n")
BUILTIN(__builtin_sve_svpmullb_pair_n_u32, "q4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svpmullb_pair_n_u64, "q2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svpmullb_pair_n_u8, "q16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svpmullb_pair_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svpmullb_pair_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svpmullb_pair_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svpmullb_u16, "q8Usq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svpmullb_u64, "q2UWiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svpmullt_n_u16, "q8Usq16UcUc", "n")
BUILTIN(__builtin_sve_svpmullt_n_u64, "q2UWiq4UiUi", "n")
BUILTIN(__builtin_sve_svpmullt_pair_n_u32, "q4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svpmullt_pair_n_u64, "q2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svpmullt_pair_n_u8, "q16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svpmullt_pair_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svpmullt_pair_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svpmullt_pair_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svpmullt_u16, "q8Usq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svpmullt_u64, "q2UWiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svpnext_b16, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svpnext_b32, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svpnext_b64, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svpnext_b8, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svprfb, "vq16bvC*Ii", "n")
BUILTIN(__builtin_sve_svprfb_gather_s32offset, "vq16bvC*q4iIi", "n")
BUILTIN(__builtin_sve_svprfb_gather_s64offset, "vq16bvC*q2WiIi", "n")
BUILTIN(__builtin_sve_svprfb_gather_u32base, "vq16bq4UiIi", "n")
BUILTIN(__builtin_sve_svprfb_gather_u32base_offset, "vq16bq4UiWiIi", "n")
BUILTIN(__builtin_sve_svprfb_gather_u32offset, "vq16bvC*q4UiIi", "n")
BUILTIN(__builtin_sve_svprfb_gather_u64base, "vq16bq2UWiIi", "n")
BUILTIN(__builtin_sve_svprfb_gather_u64base_offset, "vq16bq2UWiWiIi", "n")
BUILTIN(__builtin_sve_svprfb_gather_u64offset, "vq16bvC*q2UWiIi", "n")
BUILTIN(__builtin_sve_svprfb_vnum, "vq16bvC*WiIi", "n")
BUILTIN(__builtin_sve_svprfd, "vq16bvC*Ii", "n")
BUILTIN(__builtin_sve_svprfd_gather_s32index, "vq16bvC*q4iIi", "n")
BUILTIN(__builtin_sve_svprfd_gather_s64index, "vq16bvC*q2WiIi", "n")
BUILTIN(__builtin_sve_svprfd_gather_u32base, "vq16bq4UiIi", "n")
BUILTIN(__builtin_sve_svprfd_gather_u32base_index, "vq16bq4UiWiIi", "n")
BUILTIN(__builtin_sve_svprfd_gather_u32index, "vq16bvC*q4UiIi", "n")
BUILTIN(__builtin_sve_svprfd_gather_u64base, "vq16bq2UWiIi", "n")
BUILTIN(__builtin_sve_svprfd_gather_u64base_index, "vq16bq2UWiWiIi", "n")
BUILTIN(__builtin_sve_svprfd_gather_u64index, "vq16bvC*q2UWiIi", "n")
BUILTIN(__builtin_sve_svprfd_vnum, "vq16bvC*WiIi", "n")
BUILTIN(__builtin_sve_svprfh, "vq16bvC*Ii", "n")
BUILTIN(__builtin_sve_svprfh_gather_s32index, "vq16bvC*q4iIi", "n")
BUILTIN(__builtin_sve_svprfh_gather_s64index, "vq16bvC*q2WiIi", "n")
BUILTIN(__builtin_sve_svprfh_gather_u32base, "vq16bq4UiIi", "n")
BUILTIN(__builtin_sve_svprfh_gather_u32base_index, "vq16bq4UiWiIi", "n")
BUILTIN(__builtin_sve_svprfh_gather_u32index, "vq16bvC*q4UiIi", "n")
BUILTIN(__builtin_sve_svprfh_gather_u64base, "vq16bq2UWiIi", "n")
BUILTIN(__builtin_sve_svprfh_gather_u64base_index, "vq16bq2UWiWiIi", "n")
BUILTIN(__builtin_sve_svprfh_gather_u64index, "vq16bvC*q2UWiIi", "n")
BUILTIN(__builtin_sve_svprfh_vnum, "vq16bvC*WiIi", "n")
BUILTIN(__builtin_sve_svprfw, "vq16bvC*Ii", "n")
BUILTIN(__builtin_sve_svprfw_gather_s32index, "vq16bvC*q4iIi", "n")
BUILTIN(__builtin_sve_svprfw_gather_s64index, "vq16bvC*q2WiIi", "n")
BUILTIN(__builtin_sve_svprfw_gather_u32base, "vq16bq4UiIi", "n")
BUILTIN(__builtin_sve_svprfw_gather_u32base_index, "vq16bq4UiWiIi", "n")
BUILTIN(__builtin_sve_svprfw_gather_u32index, "vq16bvC*q4UiIi", "n")
BUILTIN(__builtin_sve_svprfw_gather_u64base, "vq16bq2UWiIi", "n")
BUILTIN(__builtin_sve_svprfw_gather_u64base_index, "vq16bq2UWiWiIi", "n")
BUILTIN(__builtin_sve_svprfw_gather_u64index, "vq16bvC*q2UWiIi", "n")
BUILTIN(__builtin_sve_svprfw_vnum, "vq16bvC*WiIi", "n")
BUILTIN(__builtin_sve_svptest_any, "bq16bq16b", "n")
BUILTIN(__builtin_sve_svptest_first, "bq16bq16b", "n")
BUILTIN(__builtin_sve_svptest_last, "bq16bq16b", "n")
BUILTIN(__builtin_sve_svptrue_b16, "q16b", "n")
BUILTIN(__builtin_sve_svptrue_b32, "q16b", "n")
BUILTIN(__builtin_sve_svptrue_b64, "q16b", "n")
BUILTIN(__builtin_sve_svptrue_b8, "q16b", "n")
BUILTIN(__builtin_sve_svptrue_pat_b16, "q16bIi", "n")
BUILTIN(__builtin_sve_svptrue_pat_b32, "q16bIi", "n")
BUILTIN(__builtin_sve_svptrue_pat_b64, "q16bIi", "n")
BUILTIN(__builtin_sve_svptrue_pat_b8, "q16bIi", "n")
BUILTIN(__builtin_sve_svqabs_s16_m, "q8sq8sq16bq8s", "n")
BUILTIN(__builtin_sve_svqabs_s16_x, "q8sq16bq8s", "n")
BUILTIN(__builtin_sve_svqabs_s16_z, "q8sq16bq8s", "n")
BUILTIN(__builtin_sve_svqabs_s32_m, "q4iq4iq16bq4i", "n")
BUILTIN(__builtin_sve_svqabs_s32_x, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svqabs_s32_z, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svqabs_s64_m, "q2Wiq2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svqabs_s64_x, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svqabs_s64_z, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svqabs_s8_m, "q16Scq16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svqabs_s8_x, "q16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svqabs_s8_z, "q16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svqadd_n_s16, "q8sq8ss", "n")
BUILTIN(__builtin_sve_svqadd_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svqadd_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svqadd_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svqadd_n_s32, "q4iq4ii", "n")
BUILTIN(__builtin_sve_svqadd_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svqadd_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svqadd_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svqadd_n_s64, "q2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svqadd_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svqadd_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svqadd_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svqadd_n_s8, "q16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svqadd_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svqadd_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svqadd_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svqadd_n_u16, "q8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svqadd_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svqadd_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svqadd_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svqadd_n_u32, "q4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svqadd_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svqadd_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svqadd_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svqadd_n_u64, "q2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svqadd_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svqadd_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svqadd_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svqadd_n_u8, "q16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svqadd_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svqadd_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svqadd_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svqadd_s16, "q8sq8sq8s", "n")
BUILTIN(__builtin_sve_svqadd_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svqadd_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svqadd_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svqadd_s32, "q4iq4iq4i", "n")
BUILTIN(__builtin_sve_svqadd_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svqadd_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svqadd_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svqadd_s64, "q2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqadd_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqadd_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqadd_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqadd_s8, "q16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqadd_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqadd_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqadd_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqadd_u16, "q8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svqadd_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svqadd_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svqadd_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svqadd_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svqadd_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svqadd_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svqadd_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svqadd_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svqadd_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svqadd_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svqadd_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svqadd_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svqadd_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svqadd_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svqadd_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svqcadd_s16, "q8sq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svqcadd_s32, "q4iq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svqcadd_s64, "q2Wiq2Wiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svqcadd_s8, "q16Scq16Scq16ScIUWi", "n")
BUILTIN(__builtin_sve_svqdecb_n_s32, "iiIUWi", "n")
BUILTIN(__builtin_sve_svqdecb_n_s64, "WiWiIUWi", "n")
BUILTIN(__builtin_sve_svqdecb_n_u32, "UiUiIUWi", "n")
BUILTIN(__builtin_sve_svqdecb_n_u64, "UWiUWiIUWi", "n")
BUILTIN(__builtin_sve_svqdecb_pat_n_s32, "iiIiIUWi", "n")
BUILTIN(__builtin_sve_svqdecb_pat_n_s64, "WiWiIiIUWi", "n")
BUILTIN(__builtin_sve_svqdecb_pat_n_u32, "UiUiIiIUWi", "n")
BUILTIN(__builtin_sve_svqdecb_pat_n_u64, "UWiUWiIiIUWi", "n")
BUILTIN(__builtin_sve_svqdecd_n_s32, "iiIUWi", "n")
BUILTIN(__builtin_sve_svqdecd_n_s64, "WiWiIUWi", "n")
BUILTIN(__builtin_sve_svqdecd_n_u32, "UiUiIUWi", "n")
BUILTIN(__builtin_sve_svqdecd_n_u64, "UWiUWiIUWi", "n")
BUILTIN(__builtin_sve_svqdecd_pat_n_s32, "iiIiIUWi", "n")
BUILTIN(__builtin_sve_svqdecd_pat_n_s64, "WiWiIiIUWi", "n")
BUILTIN(__builtin_sve_svqdecd_pat_n_u32, "UiUiIiIUWi", "n")
BUILTIN(__builtin_sve_svqdecd_pat_n_u64, "UWiUWiIiIUWi", "n")
BUILTIN(__builtin_sve_svqdecd_pat_s64, "q2Wiq2WiIiIUWi", "n")
BUILTIN(__builtin_sve_svqdecd_pat_u64, "q2UWiq2UWiIiIUWi", "n")
BUILTIN(__builtin_sve_svqdecd_s64, "q2Wiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svqdecd_u64, "q2UWiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svqdech_n_s32, "iiIUWi", "n")
BUILTIN(__builtin_sve_svqdech_n_s64, "WiWiIUWi", "n")
BUILTIN(__builtin_sve_svqdech_n_u32, "UiUiIUWi", "n")
BUILTIN(__builtin_sve_svqdech_n_u64, "UWiUWiIUWi", "n")
BUILTIN(__builtin_sve_svqdech_pat_n_s32, "iiIiIUWi", "n")
BUILTIN(__builtin_sve_svqdech_pat_n_s64, "WiWiIiIUWi", "n")
BUILTIN(__builtin_sve_svqdech_pat_n_u32, "UiUiIiIUWi", "n")
BUILTIN(__builtin_sve_svqdech_pat_n_u64, "UWiUWiIiIUWi", "n")
BUILTIN(__builtin_sve_svqdech_pat_s16, "q8sq8sIiIUWi", "n")
BUILTIN(__builtin_sve_svqdech_pat_u16, "q8Usq8UsIiIUWi", "n")
BUILTIN(__builtin_sve_svqdech_s16, "q8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svqdech_u16, "q8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svqdecp_n_s32_b16, "iiq16b", "n")
BUILTIN(__builtin_sve_svqdecp_n_s32_b32, "iiq16b", "n")
BUILTIN(__builtin_sve_svqdecp_n_s32_b64, "iiq16b", "n")
BUILTIN(__builtin_sve_svqdecp_n_s32_b8, "iiq16b", "n")
BUILTIN(__builtin_sve_svqdecp_n_s64_b16, "WiWiq16b", "n")
BUILTIN(__builtin_sve_svqdecp_n_s64_b32, "WiWiq16b", "n")
BUILTIN(__builtin_sve_svqdecp_n_s64_b64, "WiWiq16b", "n")
BUILTIN(__builtin_sve_svqdecp_n_s64_b8, "WiWiq16b", "n")
BUILTIN(__builtin_sve_svqdecp_n_u32_b16, "UiUiq16b", "n")
BUILTIN(__builtin_sve_svqdecp_n_u32_b32, "UiUiq16b", "n")
BUILTIN(__builtin_sve_svqdecp_n_u32_b64, "UiUiq16b", "n")
BUILTIN(__builtin_sve_svqdecp_n_u32_b8, "UiUiq16b", "n")
BUILTIN(__builtin_sve_svqdecp_n_u64_b16, "UWiUWiq16b", "n")
BUILTIN(__builtin_sve_svqdecp_n_u64_b32, "UWiUWiq16b", "n")
BUILTIN(__builtin_sve_svqdecp_n_u64_b64, "UWiUWiq16b", "n")
BUILTIN(__builtin_sve_svqdecp_n_u64_b8, "UWiUWiq16b", "n")
BUILTIN(__builtin_sve_svqdecp_s16, "q8sq8sq16b", "n")
BUILTIN(__builtin_sve_svqdecp_s32, "q4iq4iq16b", "n")
BUILTIN(__builtin_sve_svqdecp_s64, "q2Wiq2Wiq16b", "n")
BUILTIN(__builtin_sve_svqdecp_u16, "q8Usq8Usq16b", "n")
BUILTIN(__builtin_sve_svqdecp_u32, "q4Uiq4Uiq16b", "n")
BUILTIN(__builtin_sve_svqdecp_u64, "q2UWiq2UWiq16b", "n")
BUILTIN(__builtin_sve_svqdecw_n_s32, "iiIUWi", "n")
BUILTIN(__builtin_sve_svqdecw_n_s64, "WiWiIUWi", "n")
BUILTIN(__builtin_sve_svqdecw_n_u32, "UiUiIUWi", "n")
BUILTIN(__builtin_sve_svqdecw_n_u64, "UWiUWiIUWi", "n")
BUILTIN(__builtin_sve_svqdecw_pat_n_s32, "iiIiIUWi", "n")
BUILTIN(__builtin_sve_svqdecw_pat_n_s64, "WiWiIiIUWi", "n")
BUILTIN(__builtin_sve_svqdecw_pat_n_u32, "UiUiIiIUWi", "n")
BUILTIN(__builtin_sve_svqdecw_pat_n_u64, "UWiUWiIiIUWi", "n")
BUILTIN(__builtin_sve_svqdecw_pat_s32, "q4iq4iIiIUWi", "n")
BUILTIN(__builtin_sve_svqdecw_pat_u32, "q4Uiq4UiIiIUWi", "n")
BUILTIN(__builtin_sve_svqdecw_s32, "q4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svqdecw_u32, "q4Uiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svqdmlalb_lane_s32, "q4iq4iq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svqdmlalb_lane_s64, "q2Wiq2Wiq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svqdmlalb_n_s16, "q8sq8sq16ScSc", "n")
BUILTIN(__builtin_sve_svqdmlalb_n_s32, "q4iq4iq8ss", "n")
BUILTIN(__builtin_sve_svqdmlalb_n_s64, "q2Wiq2Wiq4ii", "n")
BUILTIN(__builtin_sve_svqdmlalb_s16, "q8sq8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqdmlalb_s32, "q4iq4iq8sq8s", "n")
BUILTIN(__builtin_sve_svqdmlalb_s64, "q2Wiq2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svqdmlalbt_n_s16, "q8sq8sq16ScSc", "n")
BUILTIN(__builtin_sve_svqdmlalbt_n_s32, "q4iq4iq8ss", "n")
BUILTIN(__builtin_sve_svqdmlalbt_n_s64, "q2Wiq2Wiq4ii", "n")
BUILTIN(__builtin_sve_svqdmlalbt_s16, "q8sq8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqdmlalbt_s32, "q4iq4iq8sq8s", "n")
BUILTIN(__builtin_sve_svqdmlalbt_s64, "q2Wiq2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svqdmlalt_lane_s32, "q4iq4iq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svqdmlalt_lane_s64, "q2Wiq2Wiq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svqdmlalt_n_s16, "q8sq8sq16ScSc", "n")
BUILTIN(__builtin_sve_svqdmlalt_n_s32, "q4iq4iq8ss", "n")
BUILTIN(__builtin_sve_svqdmlalt_n_s64, "q2Wiq2Wiq4ii", "n")
BUILTIN(__builtin_sve_svqdmlalt_s16, "q8sq8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqdmlalt_s32, "q4iq4iq8sq8s", "n")
BUILTIN(__builtin_sve_svqdmlalt_s64, "q2Wiq2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svqdmlslb_lane_s32, "q4iq4iq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svqdmlslb_lane_s64, "q2Wiq2Wiq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svqdmlslb_n_s16, "q8sq8sq16ScSc", "n")
BUILTIN(__builtin_sve_svqdmlslb_n_s32, "q4iq4iq8ss", "n")
BUILTIN(__builtin_sve_svqdmlslb_n_s64, "q2Wiq2Wiq4ii", "n")
BUILTIN(__builtin_sve_svqdmlslb_s16, "q8sq8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqdmlslb_s32, "q4iq4iq8sq8s", "n")
BUILTIN(__builtin_sve_svqdmlslb_s64, "q2Wiq2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svqdmlslbt_n_s16, "q8sq8sq16ScSc", "n")
BUILTIN(__builtin_sve_svqdmlslbt_n_s32, "q4iq4iq8ss", "n")
BUILTIN(__builtin_sve_svqdmlslbt_n_s64, "q2Wiq2Wiq4ii", "n")
BUILTIN(__builtin_sve_svqdmlslbt_s16, "q8sq8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqdmlslbt_s32, "q4iq4iq8sq8s", "n")
BUILTIN(__builtin_sve_svqdmlslbt_s64, "q2Wiq2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svqdmlslt_lane_s32, "q4iq4iq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svqdmlslt_lane_s64, "q2Wiq2Wiq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svqdmlslt_n_s16, "q8sq8sq16ScSc", "n")
BUILTIN(__builtin_sve_svqdmlslt_n_s32, "q4iq4iq8ss", "n")
BUILTIN(__builtin_sve_svqdmlslt_n_s64, "q2Wiq2Wiq4ii", "n")
BUILTIN(__builtin_sve_svqdmlslt_s16, "q8sq8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqdmlslt_s32, "q4iq4iq8sq8s", "n")
BUILTIN(__builtin_sve_svqdmlslt_s64, "q2Wiq2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svqdmulh_lane_s16, "q8sq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svqdmulh_lane_s32, "q4iq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svqdmulh_lane_s64, "q2Wiq2Wiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svqdmulh_n_s16, "q8sq8ss", "n")
BUILTIN(__builtin_sve_svqdmulh_n_s32, "q4iq4ii", "n")
BUILTIN(__builtin_sve_svqdmulh_n_s64, "q2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svqdmulh_n_s8, "q16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svqdmulh_s16, "q8sq8sq8s", "n")
BUILTIN(__builtin_sve_svqdmulh_s32, "q4iq4iq4i", "n")
BUILTIN(__builtin_sve_svqdmulh_s64, "q2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqdmulh_s8, "q16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqdmullb_lane_s32, "q4iq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svqdmullb_lane_s64, "q2Wiq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svqdmullb_n_s16, "q8sq16ScSc", "n")
BUILTIN(__builtin_sve_svqdmullb_n_s32, "q4iq8ss", "n")
BUILTIN(__builtin_sve_svqdmullb_n_s64, "q2Wiq4ii", "n")
BUILTIN(__builtin_sve_svqdmullb_s16, "q8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqdmullb_s32, "q4iq8sq8s", "n")
BUILTIN(__builtin_sve_svqdmullb_s64, "q2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svqdmullt_lane_s32, "q4iq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svqdmullt_lane_s64, "q2Wiq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svqdmullt_n_s16, "q8sq16ScSc", "n")
BUILTIN(__builtin_sve_svqdmullt_n_s32, "q4iq8ss", "n")
BUILTIN(__builtin_sve_svqdmullt_n_s64, "q2Wiq4ii", "n")
BUILTIN(__builtin_sve_svqdmullt_s16, "q8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqdmullt_s32, "q4iq8sq8s", "n")
BUILTIN(__builtin_sve_svqdmullt_s64, "q2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svqincb_n_s32, "iiIUWi", "n")
BUILTIN(__builtin_sve_svqincb_n_s64, "WiWiIUWi", "n")
BUILTIN(__builtin_sve_svqincb_n_u32, "UiUiIUWi", "n")
BUILTIN(__builtin_sve_svqincb_n_u64, "UWiUWiIUWi", "n")
BUILTIN(__builtin_sve_svqincb_pat_n_s32, "iiIiIUWi", "n")
BUILTIN(__builtin_sve_svqincb_pat_n_s64, "WiWiIiIUWi", "n")
BUILTIN(__builtin_sve_svqincb_pat_n_u32, "UiUiIiIUWi", "n")
BUILTIN(__builtin_sve_svqincb_pat_n_u64, "UWiUWiIiIUWi", "n")
BUILTIN(__builtin_sve_svqincd_n_s32, "iiIUWi", "n")
BUILTIN(__builtin_sve_svqincd_n_s64, "WiWiIUWi", "n")
BUILTIN(__builtin_sve_svqincd_n_u32, "UiUiIUWi", "n")
BUILTIN(__builtin_sve_svqincd_n_u64, "UWiUWiIUWi", "n")
BUILTIN(__builtin_sve_svqincd_pat_n_s32, "iiIiIUWi", "n")
BUILTIN(__builtin_sve_svqincd_pat_n_s64, "WiWiIiIUWi", "n")
BUILTIN(__builtin_sve_svqincd_pat_n_u32, "UiUiIiIUWi", "n")
BUILTIN(__builtin_sve_svqincd_pat_n_u64, "UWiUWiIiIUWi", "n")
BUILTIN(__builtin_sve_svqincd_pat_s64, "q2Wiq2WiIiIUWi", "n")
BUILTIN(__builtin_sve_svqincd_pat_u64, "q2UWiq2UWiIiIUWi", "n")
BUILTIN(__builtin_sve_svqincd_s64, "q2Wiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svqincd_u64, "q2UWiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svqinch_n_s32, "iiIUWi", "n")
BUILTIN(__builtin_sve_svqinch_n_s64, "WiWiIUWi", "n")
BUILTIN(__builtin_sve_svqinch_n_u32, "UiUiIUWi", "n")
BUILTIN(__builtin_sve_svqinch_n_u64, "UWiUWiIUWi", "n")
BUILTIN(__builtin_sve_svqinch_pat_n_s32, "iiIiIUWi", "n")
BUILTIN(__builtin_sve_svqinch_pat_n_s64, "WiWiIiIUWi", "n")
BUILTIN(__builtin_sve_svqinch_pat_n_u32, "UiUiIiIUWi", "n")
BUILTIN(__builtin_sve_svqinch_pat_n_u64, "UWiUWiIiIUWi", "n")
BUILTIN(__builtin_sve_svqinch_pat_s16, "q8sq8sIiIUWi", "n")
BUILTIN(__builtin_sve_svqinch_pat_u16, "q8Usq8UsIiIUWi", "n")
BUILTIN(__builtin_sve_svqinch_s16, "q8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svqinch_u16, "q8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svqincp_n_s32_b16, "iiq16b", "n")
BUILTIN(__builtin_sve_svqincp_n_s32_b32, "iiq16b", "n")
BUILTIN(__builtin_sve_svqincp_n_s32_b64, "iiq16b", "n")
BUILTIN(__builtin_sve_svqincp_n_s32_b8, "iiq16b", "n")
BUILTIN(__builtin_sve_svqincp_n_s64_b16, "WiWiq16b", "n")
BUILTIN(__builtin_sve_svqincp_n_s64_b32, "WiWiq16b", "n")
BUILTIN(__builtin_sve_svqincp_n_s64_b64, "WiWiq16b", "n")
BUILTIN(__builtin_sve_svqincp_n_s64_b8, "WiWiq16b", "n")
BUILTIN(__builtin_sve_svqincp_n_u32_b16, "UiUiq16b", "n")
BUILTIN(__builtin_sve_svqincp_n_u32_b32, "UiUiq16b", "n")
BUILTIN(__builtin_sve_svqincp_n_u32_b64, "UiUiq16b", "n")
BUILTIN(__builtin_sve_svqincp_n_u32_b8, "UiUiq16b", "n")
BUILTIN(__builtin_sve_svqincp_n_u64_b16, "UWiUWiq16b", "n")
BUILTIN(__builtin_sve_svqincp_n_u64_b32, "UWiUWiq16b", "n")
BUILTIN(__builtin_sve_svqincp_n_u64_b64, "UWiUWiq16b", "n")
BUILTIN(__builtin_sve_svqincp_n_u64_b8, "UWiUWiq16b", "n")
BUILTIN(__builtin_sve_svqincp_s16, "q8sq8sq16b", "n")
BUILTIN(__builtin_sve_svqincp_s32, "q4iq4iq16b", "n")
BUILTIN(__builtin_sve_svqincp_s64, "q2Wiq2Wiq16b", "n")
BUILTIN(__builtin_sve_svqincp_u16, "q8Usq8Usq16b", "n")
BUILTIN(__builtin_sve_svqincp_u32, "q4Uiq4Uiq16b", "n")
BUILTIN(__builtin_sve_svqincp_u64, "q2UWiq2UWiq16b", "n")
BUILTIN(__builtin_sve_svqincw_n_s32, "iiIUWi", "n")
BUILTIN(__builtin_sve_svqincw_n_s64, "WiWiIUWi", "n")
BUILTIN(__builtin_sve_svqincw_n_u32, "UiUiIUWi", "n")
BUILTIN(__builtin_sve_svqincw_n_u64, "UWiUWiIUWi", "n")
BUILTIN(__builtin_sve_svqincw_pat_n_s32, "iiIiIUWi", "n")
BUILTIN(__builtin_sve_svqincw_pat_n_s64, "WiWiIiIUWi", "n")
BUILTIN(__builtin_sve_svqincw_pat_n_u32, "UiUiIiIUWi", "n")
BUILTIN(__builtin_sve_svqincw_pat_n_u64, "UWiUWiIiIUWi", "n")
BUILTIN(__builtin_sve_svqincw_pat_s32, "q4iq4iIiIUWi", "n")
BUILTIN(__builtin_sve_svqincw_pat_u32, "q4Uiq4UiIiIUWi", "n")
BUILTIN(__builtin_sve_svqincw_s32, "q4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svqincw_u32, "q4Uiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svqneg_s16_m, "q8sq8sq16bq8s", "n")
BUILTIN(__builtin_sve_svqneg_s16_x, "q8sq16bq8s", "n")
BUILTIN(__builtin_sve_svqneg_s16_z, "q8sq16bq8s", "n")
BUILTIN(__builtin_sve_svqneg_s32_m, "q4iq4iq16bq4i", "n")
BUILTIN(__builtin_sve_svqneg_s32_x, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svqneg_s32_z, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svqneg_s64_m, "q2Wiq2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svqneg_s64_x, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svqneg_s64_z, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svqneg_s8_m, "q16Scq16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svqneg_s8_x, "q16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svqneg_s8_z, "q16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svqrdcmlah_lane_s16, "q8sq8sq8sq8sIUWiIUWi", "n")
BUILTIN(__builtin_sve_svqrdcmlah_lane_s32, "q4iq4iq4iq4iIUWiIUWi", "n")
BUILTIN(__builtin_sve_svqrdcmlah_s16, "q8sq8sq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svqrdcmlah_s32, "q4iq4iq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svqrdcmlah_s64, "q2Wiq2Wiq2Wiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svqrdcmlah_s8, "q16Scq16Scq16Scq16ScIUWi", "n")
BUILTIN(__builtin_sve_svqrdmlah_lane_s16, "q8sq8sq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svqrdmlah_lane_s32, "q4iq4iq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svqrdmlah_lane_s64, "q2Wiq2Wiq2Wiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svqrdmlah_n_s16, "q8sq8sq8ss", "n")
BUILTIN(__builtin_sve_svqrdmlah_n_s32, "q4iq4iq4ii", "n")
BUILTIN(__builtin_sve_svqrdmlah_n_s64, "q2Wiq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svqrdmlah_n_s8, "q16Scq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svqrdmlah_s16, "q8sq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svqrdmlah_s32, "q4iq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svqrdmlah_s64, "q2Wiq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqrdmlah_s8, "q16Scq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqrdmlsh_lane_s16, "q8sq8sq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svqrdmlsh_lane_s32, "q4iq4iq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svqrdmlsh_lane_s64, "q2Wiq2Wiq2Wiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svqrdmlsh_n_s16, "q8sq8sq8ss", "n")
BUILTIN(__builtin_sve_svqrdmlsh_n_s32, "q4iq4iq4ii", "n")
BUILTIN(__builtin_sve_svqrdmlsh_n_s64, "q2Wiq2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svqrdmlsh_n_s8, "q16Scq16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svqrdmlsh_s16, "q8sq8sq8sq8s", "n")
BUILTIN(__builtin_sve_svqrdmlsh_s32, "q4iq4iq4iq4i", "n")
BUILTIN(__builtin_sve_svqrdmlsh_s64, "q2Wiq2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqrdmlsh_s8, "q16Scq16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqrdmulh_lane_s16, "q8sq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svqrdmulh_lane_s32, "q4iq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svqrdmulh_lane_s64, "q2Wiq2Wiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svqrdmulh_n_s16, "q8sq8ss", "n")
BUILTIN(__builtin_sve_svqrdmulh_n_s32, "q4iq4ii", "n")
BUILTIN(__builtin_sve_svqrdmulh_n_s64, "q2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svqrdmulh_n_s8, "q16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svqrdmulh_s16, "q8sq8sq8s", "n")
BUILTIN(__builtin_sve_svqrdmulh_s32, "q4iq4iq4i", "n")
BUILTIN(__builtin_sve_svqrdmulh_s64, "q2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqrdmulh_s8, "q16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqrshl_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svqrshl_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svqrshl_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svqrshl_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svqrshl_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svqrshl_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svqrshl_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svqrshl_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svqrshl_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svqrshl_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svqrshl_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svqrshl_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svqrshl_n_u16_m, "q8Usq16bq8Uss", "n")
BUILTIN(__builtin_sve_svqrshl_n_u16_x, "q8Usq16bq8Uss", "n")
BUILTIN(__builtin_sve_svqrshl_n_u16_z, "q8Usq16bq8Uss", "n")
BUILTIN(__builtin_sve_svqrshl_n_u32_m, "q4Uiq16bq4Uii", "n")
BUILTIN(__builtin_sve_svqrshl_n_u32_x, "q4Uiq16bq4Uii", "n")
BUILTIN(__builtin_sve_svqrshl_n_u32_z, "q4Uiq16bq4Uii", "n")
BUILTIN(__builtin_sve_svqrshl_n_u64_m, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svqrshl_n_u64_x, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svqrshl_n_u64_z, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svqrshl_n_u8_m, "q16Ucq16bq16UcSc", "n")
BUILTIN(__builtin_sve_svqrshl_n_u8_x, "q16Ucq16bq16UcSc", "n")
BUILTIN(__builtin_sve_svqrshl_n_u8_z, "q16Ucq16bq16UcSc", "n")
BUILTIN(__builtin_sve_svqrshl_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svqrshl_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svqrshl_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svqrshl_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svqrshl_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svqrshl_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svqrshl_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqrshl_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqrshl_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqrshl_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqrshl_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqrshl_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqrshl_u16_m, "q8Usq16bq8Usq8s", "n")
BUILTIN(__builtin_sve_svqrshl_u16_x, "q8Usq16bq8Usq8s", "n")
BUILTIN(__builtin_sve_svqrshl_u16_z, "q8Usq16bq8Usq8s", "n")
BUILTIN(__builtin_sve_svqrshl_u32_m, "q4Uiq16bq4Uiq4i", "n")
BUILTIN(__builtin_sve_svqrshl_u32_x, "q4Uiq16bq4Uiq4i", "n")
BUILTIN(__builtin_sve_svqrshl_u32_z, "q4Uiq16bq4Uiq4i", "n")
BUILTIN(__builtin_sve_svqrshl_u64_m, "q2UWiq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svqrshl_u64_x, "q2UWiq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svqrshl_u64_z, "q2UWiq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svqrshl_u8_m, "q16Ucq16bq16Ucq16Sc", "n")
BUILTIN(__builtin_sve_svqrshl_u8_x, "q16Ucq16bq16Ucq16Sc", "n")
BUILTIN(__builtin_sve_svqrshl_u8_z, "q16Ucq16bq16Ucq16Sc", "n")
BUILTIN(__builtin_sve_svqrshrnb_n_s16, "q16Scq8sIUWi", "n")
BUILTIN(__builtin_sve_svqrshrnb_n_s32, "q8sq4iIUWi", "n")
BUILTIN(__builtin_sve_svqrshrnb_n_s64, "q4iq2WiIUWi", "n")
BUILTIN(__builtin_sve_svqrshrnb_n_u16, "q16Ucq8UsIUWi", "n")
BUILTIN(__builtin_sve_svqrshrnb_n_u32, "q8Usq4UiIUWi", "n")
BUILTIN(__builtin_sve_svqrshrnb_n_u64, "q4Uiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svqrshrnt_n_s16, "q16Scq16Scq8sIUWi", "n")
BUILTIN(__builtin_sve_svqrshrnt_n_s32, "q8sq8sq4iIUWi", "n")
BUILTIN(__builtin_sve_svqrshrnt_n_s64, "q4iq4iq2WiIUWi", "n")
BUILTIN(__builtin_sve_svqrshrnt_n_u16, "q16Ucq16Ucq8UsIUWi", "n")
BUILTIN(__builtin_sve_svqrshrnt_n_u32, "q8Usq8Usq4UiIUWi", "n")
BUILTIN(__builtin_sve_svqrshrnt_n_u64, "q4Uiq4Uiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svqrshrunb_n_s16, "q16Ucq8sIUWi", "n")
BUILTIN(__builtin_sve_svqrshrunb_n_s32, "q8Usq4iIUWi", "n")
BUILTIN(__builtin_sve_svqrshrunb_n_s64, "q4Uiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svqrshrunt_n_s16, "q16Ucq16Ucq8sIUWi", "n")
BUILTIN(__builtin_sve_svqrshrunt_n_s32, "q8Usq8Usq4iIUWi", "n")
BUILTIN(__builtin_sve_svqrshrunt_n_s64, "q4Uiq4Uiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svqshl_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svqshl_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svqshl_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svqshl_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svqshl_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svqshl_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svqshl_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svqshl_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svqshl_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svqshl_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svqshl_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svqshl_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svqshl_n_u16_m, "q8Usq16bq8Uss", "n")
BUILTIN(__builtin_sve_svqshl_n_u16_x, "q8Usq16bq8Uss", "n")
BUILTIN(__builtin_sve_svqshl_n_u16_z, "q8Usq16bq8Uss", "n")
BUILTIN(__builtin_sve_svqshl_n_u32_m, "q4Uiq16bq4Uii", "n")
BUILTIN(__builtin_sve_svqshl_n_u32_x, "q4Uiq16bq4Uii", "n")
BUILTIN(__builtin_sve_svqshl_n_u32_z, "q4Uiq16bq4Uii", "n")
BUILTIN(__builtin_sve_svqshl_n_u64_m, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svqshl_n_u64_x, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svqshl_n_u64_z, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svqshl_n_u8_m, "q16Ucq16bq16UcSc", "n")
BUILTIN(__builtin_sve_svqshl_n_u8_x, "q16Ucq16bq16UcSc", "n")
BUILTIN(__builtin_sve_svqshl_n_u8_z, "q16Ucq16bq16UcSc", "n")
BUILTIN(__builtin_sve_svqshl_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svqshl_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svqshl_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svqshl_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svqshl_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svqshl_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svqshl_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqshl_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqshl_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqshl_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqshl_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqshl_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqshl_u16_m, "q8Usq16bq8Usq8s", "n")
BUILTIN(__builtin_sve_svqshl_u16_x, "q8Usq16bq8Usq8s", "n")
BUILTIN(__builtin_sve_svqshl_u16_z, "q8Usq16bq8Usq8s", "n")
BUILTIN(__builtin_sve_svqshl_u32_m, "q4Uiq16bq4Uiq4i", "n")
BUILTIN(__builtin_sve_svqshl_u32_x, "q4Uiq16bq4Uiq4i", "n")
BUILTIN(__builtin_sve_svqshl_u32_z, "q4Uiq16bq4Uiq4i", "n")
BUILTIN(__builtin_sve_svqshl_u64_m, "q2UWiq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svqshl_u64_x, "q2UWiq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svqshl_u64_z, "q2UWiq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svqshl_u8_m, "q16Ucq16bq16Ucq16Sc", "n")
BUILTIN(__builtin_sve_svqshl_u8_x, "q16Ucq16bq16Ucq16Sc", "n")
BUILTIN(__builtin_sve_svqshl_u8_z, "q16Ucq16bq16Ucq16Sc", "n")
BUILTIN(__builtin_sve_svqshlu_n_s16_m, "q8Usq16bq8sIUWi", "n")
BUILTIN(__builtin_sve_svqshlu_n_s16_x, "q8Usq16bq8sIUWi", "n")
BUILTIN(__builtin_sve_svqshlu_n_s16_z, "q8Usq16bq8sIUWi", "n")
BUILTIN(__builtin_sve_svqshlu_n_s32_m, "q4Uiq16bq4iIUWi", "n")
BUILTIN(__builtin_sve_svqshlu_n_s32_x, "q4Uiq16bq4iIUWi", "n")
BUILTIN(__builtin_sve_svqshlu_n_s32_z, "q4Uiq16bq4iIUWi", "n")
BUILTIN(__builtin_sve_svqshlu_n_s64_m, "q2UWiq16bq2WiIUWi", "n")
BUILTIN(__builtin_sve_svqshlu_n_s64_x, "q2UWiq16bq2WiIUWi", "n")
BUILTIN(__builtin_sve_svqshlu_n_s64_z, "q2UWiq16bq2WiIUWi", "n")
BUILTIN(__builtin_sve_svqshlu_n_s8_m, "q16Ucq16bq16ScIUWi", "n")
BUILTIN(__builtin_sve_svqshlu_n_s8_x, "q16Ucq16bq16ScIUWi", "n")
BUILTIN(__builtin_sve_svqshlu_n_s8_z, "q16Ucq16bq16ScIUWi", "n")
BUILTIN(__builtin_sve_svqshrnb_n_s16, "q16Scq8sIUWi", "n")
BUILTIN(__builtin_sve_svqshrnb_n_s32, "q8sq4iIUWi", "n")
BUILTIN(__builtin_sve_svqshrnb_n_s64, "q4iq2WiIUWi", "n")
BUILTIN(__builtin_sve_svqshrnb_n_u16, "q16Ucq8UsIUWi", "n")
BUILTIN(__builtin_sve_svqshrnb_n_u32, "q8Usq4UiIUWi", "n")
BUILTIN(__builtin_sve_svqshrnb_n_u64, "q4Uiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svqshrnt_n_s16, "q16Scq16Scq8sIUWi", "n")
BUILTIN(__builtin_sve_svqshrnt_n_s32, "q8sq8sq4iIUWi", "n")
BUILTIN(__builtin_sve_svqshrnt_n_s64, "q4iq4iq2WiIUWi", "n")
BUILTIN(__builtin_sve_svqshrnt_n_u16, "q16Ucq16Ucq8UsIUWi", "n")
BUILTIN(__builtin_sve_svqshrnt_n_u32, "q8Usq8Usq4UiIUWi", "n")
BUILTIN(__builtin_sve_svqshrnt_n_u64, "q4Uiq4Uiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svqshrunb_n_s16, "q16Ucq8sIUWi", "n")
BUILTIN(__builtin_sve_svqshrunb_n_s32, "q8Usq4iIUWi", "n")
BUILTIN(__builtin_sve_svqshrunb_n_s64, "q4Uiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svqshrunt_n_s16, "q16Ucq16Ucq8sIUWi", "n")
BUILTIN(__builtin_sve_svqshrunt_n_s32, "q8Usq8Usq4iIUWi", "n")
BUILTIN(__builtin_sve_svqshrunt_n_s64, "q4Uiq4Uiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svqsub_n_s16, "q8sq8ss", "n")
BUILTIN(__builtin_sve_svqsub_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svqsub_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svqsub_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svqsub_n_s32, "q4iq4ii", "n")
BUILTIN(__builtin_sve_svqsub_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svqsub_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svqsub_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svqsub_n_s64, "q2Wiq2WiWi", "n")
BUILTIN(__builtin_sve_svqsub_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svqsub_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svqsub_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svqsub_n_s8, "q16Scq16ScSc", "n")
BUILTIN(__builtin_sve_svqsub_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svqsub_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svqsub_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svqsub_n_u16, "q8Usq8UsUs", "n")
BUILTIN(__builtin_sve_svqsub_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svqsub_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svqsub_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svqsub_n_u32, "q4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svqsub_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svqsub_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svqsub_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svqsub_n_u64, "q2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svqsub_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svqsub_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svqsub_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svqsub_n_u8, "q16Ucq16UcUc", "n")
BUILTIN(__builtin_sve_svqsub_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svqsub_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svqsub_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svqsub_s16, "q8sq8sq8s", "n")
BUILTIN(__builtin_sve_svqsub_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svqsub_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svqsub_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svqsub_s32, "q4iq4iq4i", "n")
BUILTIN(__builtin_sve_svqsub_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svqsub_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svqsub_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svqsub_s64, "q2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqsub_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqsub_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqsub_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqsub_s8, "q16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqsub_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqsub_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqsub_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqsub_u16, "q8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svqsub_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svqsub_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svqsub_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svqsub_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svqsub_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svqsub_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svqsub_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svqsub_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svqsub_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svqsub_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svqsub_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svqsub_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svqsub_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svqsub_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svqsub_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svqsubr_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svqsubr_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svqsubr_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svqsubr_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svqsubr_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svqsubr_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svqsubr_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svqsubr_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svqsubr_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svqsubr_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svqsubr_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svqsubr_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svqsubr_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svqsubr_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svqsubr_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svqsubr_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svqsubr_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svqsubr_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svqsubr_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svqsubr_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svqsubr_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svqsubr_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svqsubr_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svqsubr_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svqsubr_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svqsubr_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svqsubr_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svqsubr_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svqsubr_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svqsubr_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svqsubr_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqsubr_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqsubr_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svqsubr_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqsubr_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqsubr_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svqsubr_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svqsubr_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svqsubr_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svqsubr_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svqsubr_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svqsubr_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svqsubr_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svqsubr_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svqsubr_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svqsubr_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svqsubr_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svqsubr_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svqxtnb_s16, "q16Scq8s", "n")
BUILTIN(__builtin_sve_svqxtnb_s32, "q8sq4i", "n")
BUILTIN(__builtin_sve_svqxtnb_s64, "q4iq2Wi", "n")
BUILTIN(__builtin_sve_svqxtnb_u16, "q16Ucq8Us", "n")
BUILTIN(__builtin_sve_svqxtnb_u32, "q8Usq4Ui", "n")
BUILTIN(__builtin_sve_svqxtnb_u64, "q4Uiq2UWi", "n")
BUILTIN(__builtin_sve_svqxtnt_s16, "q16Scq16Scq8s", "n")
BUILTIN(__builtin_sve_svqxtnt_s32, "q8sq8sq4i", "n")
BUILTIN(__builtin_sve_svqxtnt_s64, "q4iq4iq2Wi", "n")
BUILTIN(__builtin_sve_svqxtnt_u16, "q16Ucq16Ucq8Us", "n")
BUILTIN(__builtin_sve_svqxtnt_u32, "q8Usq8Usq4Ui", "n")
BUILTIN(__builtin_sve_svqxtnt_u64, "q4Uiq4Uiq2UWi", "n")
BUILTIN(__builtin_sve_svqxtunb_s16, "q16Ucq8s", "n")
BUILTIN(__builtin_sve_svqxtunb_s32, "q8Usq4i", "n")
BUILTIN(__builtin_sve_svqxtunb_s64, "q4Uiq2Wi", "n")
BUILTIN(__builtin_sve_svqxtunt_s16, "q16Ucq16Ucq8s", "n")
BUILTIN(__builtin_sve_svqxtunt_s32, "q8Usq8Usq4i", "n")
BUILTIN(__builtin_sve_svqxtunt_s64, "q4Uiq4Uiq2Wi", "n")
BUILTIN(__builtin_sve_svraddhnb_n_s16, "q16Scq8ss", "n")
BUILTIN(__builtin_sve_svraddhnb_n_s32, "q8sq4ii", "n")
BUILTIN(__builtin_sve_svraddhnb_n_s64, "q4iq2WiWi", "n")
BUILTIN(__builtin_sve_svraddhnb_n_u16, "q16Ucq8UsUs", "n")
BUILTIN(__builtin_sve_svraddhnb_n_u32, "q8Usq4UiUi", "n")
BUILTIN(__builtin_sve_svraddhnb_n_u64, "q4Uiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svraddhnb_s16, "q16Scq8sq8s", "n")
BUILTIN(__builtin_sve_svraddhnb_s32, "q8sq4iq4i", "n")
BUILTIN(__builtin_sve_svraddhnb_s64, "q4iq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svraddhnb_u16, "q16Ucq8Usq8Us", "n")
BUILTIN(__builtin_sve_svraddhnb_u32, "q8Usq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svraddhnb_u64, "q4Uiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svraddhnt_n_s16, "q16Scq16Scq8ss", "n")
BUILTIN(__builtin_sve_svraddhnt_n_s32, "q8sq8sq4ii", "n")
BUILTIN(__builtin_sve_svraddhnt_n_s64, "q4iq4iq2WiWi", "n")
BUILTIN(__builtin_sve_svraddhnt_n_u16, "q16Ucq16Ucq8UsUs", "n")
BUILTIN(__builtin_sve_svraddhnt_n_u32, "q8Usq8Usq4UiUi", "n")
BUILTIN(__builtin_sve_svraddhnt_n_u64, "q4Uiq4Uiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svraddhnt_s16, "q16Scq16Scq8sq8s", "n")
BUILTIN(__builtin_sve_svraddhnt_s32, "q8sq8sq4iq4i", "n")
BUILTIN(__builtin_sve_svraddhnt_s64, "q4iq4iq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svraddhnt_u16, "q16Ucq16Ucq8Usq8Us", "n")
BUILTIN(__builtin_sve_svraddhnt_u32, "q8Usq8Usq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svraddhnt_u64, "q4Uiq4Uiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svrax1_s64, "q2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svrax1_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svrbit_s16_m, "q8sq8sq16bq8s", "n")
BUILTIN(__builtin_sve_svrbit_s16_x, "q8sq16bq8s", "n")
BUILTIN(__builtin_sve_svrbit_s16_z, "q8sq16bq8s", "n")
BUILTIN(__builtin_sve_svrbit_s32_m, "q4iq4iq16bq4i", "n")
BUILTIN(__builtin_sve_svrbit_s32_x, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svrbit_s32_z, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svrbit_s64_m, "q2Wiq2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svrbit_s64_x, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svrbit_s64_z, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svrbit_s8_m, "q16Scq16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svrbit_s8_x, "q16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svrbit_s8_z, "q16Scq16bq16Sc", "n")
BUILTIN(__builtin_sve_svrbit_u16_m, "q8Usq8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svrbit_u16_x, "q8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svrbit_u16_z, "q8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svrbit_u32_m, "q4Uiq4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svrbit_u32_x, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svrbit_u32_z, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svrbit_u64_m, "q2UWiq2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svrbit_u64_x, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svrbit_u64_z, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svrbit_u8_m, "q16Ucq16Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svrbit_u8_x, "q16Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svrbit_u8_z, "q16Ucq16bq16Uc", "n")
BUILTIN(__builtin_sve_svrdffr, "q16b", "n")
BUILTIN(__builtin_sve_svrdffr_z, "q16bq16b", "n")
BUILTIN(__builtin_sve_svrecpe_f16, "q8hq8h", "n")
BUILTIN(__builtin_sve_svrecpe_f32, "q4fq4f", "n")
BUILTIN(__builtin_sve_svrecpe_f64, "q2dq2d", "n")
BUILTIN(__builtin_sve_svrecpe_u32_m, "q4Uiq4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svrecpe_u32_x, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svrecpe_u32_z, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svrecps_f16, "q8hq8hq8h", "n")
BUILTIN(__builtin_sve_svrecps_f32, "q4fq4fq4f", "n")
BUILTIN(__builtin_sve_svrecps_f64, "q2dq2dq2d", "n")
BUILTIN(__builtin_sve_svrecpx_f16_m, "q8hq8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrecpx_f16_x, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrecpx_f16_z, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrecpx_f32_m, "q4fq4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrecpx_f32_x, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrecpx_f32_z, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrecpx_f64_m, "q2dq2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrecpx_f64_x, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrecpx_f64_z, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrev_b16, "q16bq16b", "n")
BUILTIN(__builtin_sve_svrev_b32, "q16bq16b", "n")
BUILTIN(__builtin_sve_svrev_b64, "q16bq16b", "n")
BUILTIN(__builtin_sve_svrev_b8, "q16bq16b", "n")
BUILTIN(__builtin_sve_svrev_bf16, "q8yq8y", "n")
BUILTIN(__builtin_sve_svrev_f16, "q8hq8h", "n")
BUILTIN(__builtin_sve_svrev_f32, "q4fq4f", "n")
BUILTIN(__builtin_sve_svrev_f64, "q2dq2d", "n")
BUILTIN(__builtin_sve_svrev_s16, "q8sq8s", "n")
BUILTIN(__builtin_sve_svrev_s32, "q4iq4i", "n")
BUILTIN(__builtin_sve_svrev_s64, "q2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svrev_s8, "q16Scq16Sc", "n")
BUILTIN(__builtin_sve_svrev_u16, "q8Usq8Us", "n")
BUILTIN(__builtin_sve_svrev_u32, "q4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svrev_u64, "q2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svrev_u8, "q16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svrevb_s16_m, "q8sq8sq16bq8s", "n")
BUILTIN(__builtin_sve_svrevb_s16_x, "q8sq16bq8s", "n")
BUILTIN(__builtin_sve_svrevb_s16_z, "q8sq16bq8s", "n")
BUILTIN(__builtin_sve_svrevb_s32_m, "q4iq4iq16bq4i", "n")
BUILTIN(__builtin_sve_svrevb_s32_x, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svrevb_s32_z, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svrevb_s64_m, "q2Wiq2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svrevb_s64_x, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svrevb_s64_z, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svrevb_u16_m, "q8Usq8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svrevb_u16_x, "q8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svrevb_u16_z, "q8Usq16bq8Us", "n")
BUILTIN(__builtin_sve_svrevb_u32_m, "q4Uiq4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svrevb_u32_x, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svrevb_u32_z, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svrevb_u64_m, "q2UWiq2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svrevb_u64_x, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svrevb_u64_z, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svrevh_s32_m, "q4iq4iq16bq4i", "n")
BUILTIN(__builtin_sve_svrevh_s32_x, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svrevh_s32_z, "q4iq16bq4i", "n")
BUILTIN(__builtin_sve_svrevh_s64_m, "q2Wiq2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svrevh_s64_x, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svrevh_s64_z, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svrevh_u32_m, "q4Uiq4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svrevh_u32_x, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svrevh_u32_z, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svrevh_u64_m, "q2UWiq2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svrevh_u64_x, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svrevh_u64_z, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svrevw_s64_m, "q2Wiq2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svrevw_s64_x, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svrevw_s64_z, "q2Wiq16bq2Wi", "n")
BUILTIN(__builtin_sve_svrevw_u64_m, "q2UWiq2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svrevw_u64_x, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svrevw_u64_z, "q2UWiq16bq2UWi", "n")
BUILTIN(__builtin_sve_svrhadd_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svrhadd_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svrhadd_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svrhadd_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svrhadd_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svrhadd_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svrhadd_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svrhadd_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svrhadd_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svrhadd_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svrhadd_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svrhadd_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svrhadd_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svrhadd_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svrhadd_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svrhadd_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svrhadd_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svrhadd_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svrhadd_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svrhadd_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svrhadd_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svrhadd_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svrhadd_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svrhadd_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svrhadd_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svrhadd_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svrhadd_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svrhadd_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svrhadd_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svrhadd_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svrhadd_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svrhadd_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svrhadd_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svrhadd_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svrhadd_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svrhadd_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svrhadd_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svrhadd_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svrhadd_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svrhadd_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svrhadd_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svrhadd_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svrhadd_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svrhadd_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svrhadd_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svrhadd_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svrhadd_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svrhadd_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svrinta_f16_m, "q8hq8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrinta_f16_x, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrinta_f16_z, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrinta_f32_m, "q4fq4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrinta_f32_x, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrinta_f32_z, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrinta_f64_m, "q2dq2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrinta_f64_x, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrinta_f64_z, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrinti_f16_m, "q8hq8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrinti_f16_x, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrinti_f16_z, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrinti_f32_m, "q4fq4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrinti_f32_x, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrinti_f32_z, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrinti_f64_m, "q2dq2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrinti_f64_x, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrinti_f64_z, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrintm_f16_m, "q8hq8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrintm_f16_x, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrintm_f16_z, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrintm_f32_m, "q4fq4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrintm_f32_x, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrintm_f32_z, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrintm_f64_m, "q2dq2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrintm_f64_x, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrintm_f64_z, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrintn_f16_m, "q8hq8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrintn_f16_x, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrintn_f16_z, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrintn_f32_m, "q4fq4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrintn_f32_x, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrintn_f32_z, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrintn_f64_m, "q2dq2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrintn_f64_x, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrintn_f64_z, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrintp_f16_m, "q8hq8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrintp_f16_x, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrintp_f16_z, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrintp_f32_m, "q4fq4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrintp_f32_x, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrintp_f32_z, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrintp_f64_m, "q2dq2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrintp_f64_x, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrintp_f64_z, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrintx_f16_m, "q8hq8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrintx_f16_x, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrintx_f16_z, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrintx_f32_m, "q4fq4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrintx_f32_x, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrintx_f32_z, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrintx_f64_m, "q2dq2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrintx_f64_x, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrintx_f64_z, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrintz_f16_m, "q8hq8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrintz_f16_x, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrintz_f16_z, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svrintz_f32_m, "q4fq4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrintz_f32_x, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrintz_f32_z, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svrintz_f64_m, "q2dq2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrintz_f64_x, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrintz_f64_z, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svrshl_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svrshl_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svrshl_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svrshl_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svrshl_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svrshl_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svrshl_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svrshl_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svrshl_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svrshl_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svrshl_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svrshl_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svrshl_n_u16_m, "q8Usq16bq8Uss", "n")
BUILTIN(__builtin_sve_svrshl_n_u16_x, "q8Usq16bq8Uss", "n")
BUILTIN(__builtin_sve_svrshl_n_u16_z, "q8Usq16bq8Uss", "n")
BUILTIN(__builtin_sve_svrshl_n_u32_m, "q4Uiq16bq4Uii", "n")
BUILTIN(__builtin_sve_svrshl_n_u32_x, "q4Uiq16bq4Uii", "n")
BUILTIN(__builtin_sve_svrshl_n_u32_z, "q4Uiq16bq4Uii", "n")
BUILTIN(__builtin_sve_svrshl_n_u64_m, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svrshl_n_u64_x, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svrshl_n_u64_z, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svrshl_n_u8_m, "q16Ucq16bq16UcSc", "n")
BUILTIN(__builtin_sve_svrshl_n_u8_x, "q16Ucq16bq16UcSc", "n")
BUILTIN(__builtin_sve_svrshl_n_u8_z, "q16Ucq16bq16UcSc", "n")
BUILTIN(__builtin_sve_svrshl_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svrshl_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svrshl_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svrshl_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svrshl_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svrshl_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svrshl_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svrshl_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svrshl_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svrshl_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svrshl_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svrshl_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svrshl_u16_m, "q8Usq16bq8Usq8s", "n")
BUILTIN(__builtin_sve_svrshl_u16_x, "q8Usq16bq8Usq8s", "n")
BUILTIN(__builtin_sve_svrshl_u16_z, "q8Usq16bq8Usq8s", "n")
BUILTIN(__builtin_sve_svrshl_u32_m, "q4Uiq16bq4Uiq4i", "n")
BUILTIN(__builtin_sve_svrshl_u32_x, "q4Uiq16bq4Uiq4i", "n")
BUILTIN(__builtin_sve_svrshl_u32_z, "q4Uiq16bq4Uiq4i", "n")
BUILTIN(__builtin_sve_svrshl_u64_m, "q2UWiq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svrshl_u64_x, "q2UWiq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svrshl_u64_z, "q2UWiq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svrshl_u8_m, "q16Ucq16bq16Ucq16Sc", "n")
BUILTIN(__builtin_sve_svrshl_u8_x, "q16Ucq16bq16Ucq16Sc", "n")
BUILTIN(__builtin_sve_svrshl_u8_z, "q16Ucq16bq16Ucq16Sc", "n")
BUILTIN(__builtin_sve_svrshr_n_s16_m, "q8sq16bq8sIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_s16_x, "q8sq16bq8sIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_s16_z, "q8sq16bq8sIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_s32_m, "q4iq16bq4iIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_s32_x, "q4iq16bq4iIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_s32_z, "q4iq16bq4iIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_s64_m, "q2Wiq16bq2WiIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_s64_x, "q2Wiq16bq2WiIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_s64_z, "q2Wiq16bq2WiIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_s8_m, "q16Scq16bq16ScIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_s8_x, "q16Scq16bq16ScIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_s8_z, "q16Scq16bq16ScIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_u16_m, "q8Usq16bq8UsIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_u16_x, "q8Usq16bq8UsIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_u16_z, "q8Usq16bq8UsIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_u32_m, "q4Uiq16bq4UiIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_u32_x, "q4Uiq16bq4UiIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_u32_z, "q4Uiq16bq4UiIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_u64_m, "q2UWiq16bq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_u64_x, "q2UWiq16bq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_u64_z, "q2UWiq16bq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_u8_m, "q16Ucq16bq16UcIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_u8_x, "q16Ucq16bq16UcIUWi", "n")
BUILTIN(__builtin_sve_svrshr_n_u8_z, "q16Ucq16bq16UcIUWi", "n")
BUILTIN(__builtin_sve_svrshrnb_n_s16, "q16Scq8sIUWi", "n")
BUILTIN(__builtin_sve_svrshrnb_n_s32, "q8sq4iIUWi", "n")
BUILTIN(__builtin_sve_svrshrnb_n_s64, "q4iq2WiIUWi", "n")
BUILTIN(__builtin_sve_svrshrnb_n_u16, "q16Ucq8UsIUWi", "n")
BUILTIN(__builtin_sve_svrshrnb_n_u32, "q8Usq4UiIUWi", "n")
BUILTIN(__builtin_sve_svrshrnb_n_u64, "q4Uiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svrshrnt_n_s16, "q16Scq16Scq8sIUWi", "n")
BUILTIN(__builtin_sve_svrshrnt_n_s32, "q8sq8sq4iIUWi", "n")
BUILTIN(__builtin_sve_svrshrnt_n_s64, "q4iq4iq2WiIUWi", "n")
BUILTIN(__builtin_sve_svrshrnt_n_u16, "q16Ucq16Ucq8UsIUWi", "n")
BUILTIN(__builtin_sve_svrshrnt_n_u32, "q8Usq8Usq4UiIUWi", "n")
BUILTIN(__builtin_sve_svrshrnt_n_u64, "q4Uiq4Uiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svrsqrte_f16, "q8hq8h", "n")
BUILTIN(__builtin_sve_svrsqrte_f32, "q4fq4f", "n")
BUILTIN(__builtin_sve_svrsqrte_f64, "q2dq2d", "n")
BUILTIN(__builtin_sve_svrsqrte_u32_m, "q4Uiq4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svrsqrte_u32_x, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svrsqrte_u32_z, "q4Uiq16bq4Ui", "n")
BUILTIN(__builtin_sve_svrsqrts_f16, "q8hq8hq8h", "n")
BUILTIN(__builtin_sve_svrsqrts_f32, "q4fq4fq4f", "n")
BUILTIN(__builtin_sve_svrsqrts_f64, "q2dq2dq2d", "n")
BUILTIN(__builtin_sve_svrsra_n_s16, "q8sq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svrsra_n_s32, "q4iq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svrsra_n_s64, "q2Wiq2Wiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svrsra_n_s8, "q16Scq16Scq16ScIUWi", "n")
BUILTIN(__builtin_sve_svrsra_n_u16, "q8Usq8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svrsra_n_u32, "q4Uiq4Uiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svrsra_n_u64, "q2UWiq2UWiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svrsra_n_u8, "q16Ucq16Ucq16UcIUWi", "n")
BUILTIN(__builtin_sve_svrsubhnb_n_s16, "q16Scq8ss", "n")
BUILTIN(__builtin_sve_svrsubhnb_n_s32, "q8sq4ii", "n")
BUILTIN(__builtin_sve_svrsubhnb_n_s64, "q4iq2WiWi", "n")
BUILTIN(__builtin_sve_svrsubhnb_n_u16, "q16Ucq8UsUs", "n")
BUILTIN(__builtin_sve_svrsubhnb_n_u32, "q8Usq4UiUi", "n")
BUILTIN(__builtin_sve_svrsubhnb_n_u64, "q4Uiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svrsubhnb_s16, "q16Scq8sq8s", "n")
BUILTIN(__builtin_sve_svrsubhnb_s32, "q8sq4iq4i", "n")
BUILTIN(__builtin_sve_svrsubhnb_s64, "q4iq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svrsubhnb_u16, "q16Ucq8Usq8Us", "n")
BUILTIN(__builtin_sve_svrsubhnb_u32, "q8Usq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svrsubhnb_u64, "q4Uiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svrsubhnt_n_s16, "q16Scq16Scq8ss", "n")
BUILTIN(__builtin_sve_svrsubhnt_n_s32, "q8sq8sq4ii", "n")
BUILTIN(__builtin_sve_svrsubhnt_n_s64, "q4iq4iq2WiWi", "n")
BUILTIN(__builtin_sve_svrsubhnt_n_u16, "q16Ucq16Ucq8UsUs", "n")
BUILTIN(__builtin_sve_svrsubhnt_n_u32, "q8Usq8Usq4UiUi", "n")
BUILTIN(__builtin_sve_svrsubhnt_n_u64, "q4Uiq4Uiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svrsubhnt_s16, "q16Scq16Scq8sq8s", "n")
BUILTIN(__builtin_sve_svrsubhnt_s32, "q8sq8sq4iq4i", "n")
BUILTIN(__builtin_sve_svrsubhnt_s64, "q4iq4iq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svrsubhnt_u16, "q16Ucq16Ucq8Usq8Us", "n")
BUILTIN(__builtin_sve_svrsubhnt_u32, "q8Usq8Usq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svrsubhnt_u64, "q4Uiq4Uiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svsbclb_n_u32, "q4Uiq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svsbclb_n_u64, "q2UWiq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svsbclb_u32, "q4Uiq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svsbclb_u64, "q2UWiq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svsbclt_n_u32, "q4Uiq4Uiq4UiUi", "n")
BUILTIN(__builtin_sve_svsbclt_n_u64, "q2UWiq2UWiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svsbclt_u32, "q4Uiq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svsbclt_u64, "q2UWiq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svscale_f16_m, "q8hq16bq8hq8s", "n")
BUILTIN(__builtin_sve_svscale_f16_x, "q8hq16bq8hq8s", "n")
BUILTIN(__builtin_sve_svscale_f16_z, "q8hq16bq8hq8s", "n")
BUILTIN(__builtin_sve_svscale_f32_m, "q4fq16bq4fq4i", "n")
BUILTIN(__builtin_sve_svscale_f32_x, "q4fq16bq4fq4i", "n")
BUILTIN(__builtin_sve_svscale_f32_z, "q4fq16bq4fq4i", "n")
BUILTIN(__builtin_sve_svscale_f64_m, "q2dq16bq2dq2Wi", "n")
BUILTIN(__builtin_sve_svscale_f64_x, "q2dq16bq2dq2Wi", "n")
BUILTIN(__builtin_sve_svscale_f64_z, "q2dq16bq2dq2Wi", "n")
BUILTIN(__builtin_sve_svscale_n_f16_m, "q8hq16bq8hs", "n")
BUILTIN(__builtin_sve_svscale_n_f16_x, "q8hq16bq8hs", "n")
BUILTIN(__builtin_sve_svscale_n_f16_z, "q8hq16bq8hs", "n")
BUILTIN(__builtin_sve_svscale_n_f32_m, "q4fq16bq4fi", "n")
BUILTIN(__builtin_sve_svscale_n_f32_x, "q4fq16bq4fi", "n")
BUILTIN(__builtin_sve_svscale_n_f32_z, "q4fq16bq4fi", "n")
BUILTIN(__builtin_sve_svscale_n_f64_m, "q2dq16bq2dWi", "n")
BUILTIN(__builtin_sve_svscale_n_f64_x, "q2dq16bq2dWi", "n")
BUILTIN(__builtin_sve_svscale_n_f64_z, "q2dq16bq2dWi", "n")
BUILTIN(__builtin_sve_svsel_b, "q16bq16bq16bq16b", "n")
BUILTIN(__builtin_sve_svsel_bf16, "q8yq16bq8yq8y", "n")
BUILTIN(__builtin_sve_svsel_f16, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svsel_f32, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svsel_f64, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svsel_s16, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svsel_s32, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svsel_s64, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svsel_s8, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svsel_u16, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svsel_u32, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svsel_u64, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svsel_u8, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svset2_bf16, "q16yq16yIUWiq8y", "n")
BUILTIN(__builtin_sve_svset2_f16, "q16hq16hIUWiq8h", "n")
BUILTIN(__builtin_sve_svset2_f32, "q8fq8fIUWiq4f", "n")
BUILTIN(__builtin_sve_svset2_f64, "q4dq4dIUWiq2d", "n")
BUILTIN(__builtin_sve_svset2_s16, "q16sq16sIUWiq8s", "n")
BUILTIN(__builtin_sve_svset2_s32, "q8iq8iIUWiq4i", "n")
BUILTIN(__builtin_sve_svset2_s64, "q4Wiq4WiIUWiq2Wi", "n")
BUILTIN(__builtin_sve_svset2_s8, "q32Scq32ScIUWiq16Sc", "n")
BUILTIN(__builtin_sve_svset2_u16, "q16Usq16UsIUWiq8Us", "n")
BUILTIN(__builtin_sve_svset2_u32, "q8Uiq8UiIUWiq4Ui", "n")
BUILTIN(__builtin_sve_svset2_u64, "q4UWiq4UWiIUWiq2UWi", "n")
BUILTIN(__builtin_sve_svset2_u8, "q32Ucq32UcIUWiq16Uc", "n")
BUILTIN(__builtin_sve_svset3_bf16, "q24yq24yIUWiq8y", "n")
BUILTIN(__builtin_sve_svset3_f16, "q24hq24hIUWiq8h", "n")
BUILTIN(__builtin_sve_svset3_f32, "q12fq12fIUWiq4f", "n")
BUILTIN(__builtin_sve_svset3_f64, "q6dq6dIUWiq2d", "n")
BUILTIN(__builtin_sve_svset3_s16, "q24sq24sIUWiq8s", "n")
BUILTIN(__builtin_sve_svset3_s32, "q12iq12iIUWiq4i", "n")
BUILTIN(__builtin_sve_svset3_s64, "q6Wiq6WiIUWiq2Wi", "n")
BUILTIN(__builtin_sve_svset3_s8, "q48Scq48ScIUWiq16Sc", "n")
BUILTIN(__builtin_sve_svset3_u16, "q24Usq24UsIUWiq8Us", "n")
BUILTIN(__builtin_sve_svset3_u32, "q12Uiq12UiIUWiq4Ui", "n")
BUILTIN(__builtin_sve_svset3_u64, "q6UWiq6UWiIUWiq2UWi", "n")
BUILTIN(__builtin_sve_svset3_u8, "q48Ucq48UcIUWiq16Uc", "n")
BUILTIN(__builtin_sve_svset4_bf16, "q32yq32yIUWiq8y", "n")
BUILTIN(__builtin_sve_svset4_f16, "q32hq32hIUWiq8h", "n")
BUILTIN(__builtin_sve_svset4_f32, "q16fq16fIUWiq4f", "n")
BUILTIN(__builtin_sve_svset4_f64, "q8dq8dIUWiq2d", "n")
BUILTIN(__builtin_sve_svset4_s16, "q32sq32sIUWiq8s", "n")
BUILTIN(__builtin_sve_svset4_s32, "q16iq16iIUWiq4i", "n")
BUILTIN(__builtin_sve_svset4_s64, "q8Wiq8WiIUWiq2Wi", "n")
BUILTIN(__builtin_sve_svset4_s8, "q64Scq64ScIUWiq16Sc", "n")
BUILTIN(__builtin_sve_svset4_u16, "q32Usq32UsIUWiq8Us", "n")
BUILTIN(__builtin_sve_svset4_u32, "q16Uiq16UiIUWiq4Ui", "n")
BUILTIN(__builtin_sve_svset4_u64, "q8UWiq8UWiIUWiq2UWi", "n")
BUILTIN(__builtin_sve_svset4_u8, "q64Ucq64UcIUWiq16Uc", "n")
BUILTIN(__builtin_sve_svsetffr, "v", "n")
BUILTIN(__builtin_sve_svshllb_n_s16, "q8sq16ScIUWi", "n")
BUILTIN(__builtin_sve_svshllb_n_s32, "q4iq8sIUWi", "n")
BUILTIN(__builtin_sve_svshllb_n_s64, "q2Wiq4iIUWi", "n")
BUILTIN(__builtin_sve_svshllb_n_u16, "q8Usq16UcIUWi", "n")
BUILTIN(__builtin_sve_svshllb_n_u32, "q4Uiq8UsIUWi", "n")
BUILTIN(__builtin_sve_svshllb_n_u64, "q2UWiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svshllt_n_s16, "q8sq16ScIUWi", "n")
BUILTIN(__builtin_sve_svshllt_n_s32, "q4iq8sIUWi", "n")
BUILTIN(__builtin_sve_svshllt_n_s64, "q2Wiq4iIUWi", "n")
BUILTIN(__builtin_sve_svshllt_n_u16, "q8Usq16UcIUWi", "n")
BUILTIN(__builtin_sve_svshllt_n_u32, "q4Uiq8UsIUWi", "n")
BUILTIN(__builtin_sve_svshllt_n_u64, "q2UWiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svshrnb_n_s16, "q16Scq8sIUWi", "n")
BUILTIN(__builtin_sve_svshrnb_n_s32, "q8sq4iIUWi", "n")
BUILTIN(__builtin_sve_svshrnb_n_s64, "q4iq2WiIUWi", "n")
BUILTIN(__builtin_sve_svshrnb_n_u16, "q16Ucq8UsIUWi", "n")
BUILTIN(__builtin_sve_svshrnb_n_u32, "q8Usq4UiIUWi", "n")
BUILTIN(__builtin_sve_svshrnb_n_u64, "q4Uiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svshrnt_n_s16, "q16Scq16Scq8sIUWi", "n")
BUILTIN(__builtin_sve_svshrnt_n_s32, "q8sq8sq4iIUWi", "n")
BUILTIN(__builtin_sve_svshrnt_n_s64, "q4iq4iq2WiIUWi", "n")
BUILTIN(__builtin_sve_svshrnt_n_u16, "q16Ucq16Ucq8UsIUWi", "n")
BUILTIN(__builtin_sve_svshrnt_n_u32, "q8Usq8Usq4UiIUWi", "n")
BUILTIN(__builtin_sve_svshrnt_n_u64, "q4Uiq4Uiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svsli_n_s16, "q8sq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svsli_n_s32, "q4iq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svsli_n_s64, "q2Wiq2Wiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svsli_n_s8, "q16Scq16Scq16ScIUWi", "n")
BUILTIN(__builtin_sve_svsli_n_u16, "q8Usq8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svsli_n_u32, "q4Uiq4Uiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svsli_n_u64, "q2UWiq2UWiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svsli_n_u8, "q16Ucq16Ucq16UcIUWi", "n")
BUILTIN(__builtin_sve_svsm4e_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svsm4ekey_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svsplice_bf16, "q8yq16bq8yq8y", "n")
BUILTIN(__builtin_sve_svsplice_f16, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svsplice_f32, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svsplice_f64, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svsplice_s16, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svsplice_s32, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svsplice_s64, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svsplice_s8, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svsplice_u16, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svsplice_u32, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svsplice_u64, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svsplice_u8, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svsqadd_n_u16_m, "q8Usq16bq8Uss", "n")
BUILTIN(__builtin_sve_svsqadd_n_u16_x, "q8Usq16bq8Uss", "n")
BUILTIN(__builtin_sve_svsqadd_n_u16_z, "q8Usq16bq8Uss", "n")
BUILTIN(__builtin_sve_svsqadd_n_u32_m, "q4Uiq16bq4Uii", "n")
BUILTIN(__builtin_sve_svsqadd_n_u32_x, "q4Uiq16bq4Uii", "n")
BUILTIN(__builtin_sve_svsqadd_n_u32_z, "q4Uiq16bq4Uii", "n")
BUILTIN(__builtin_sve_svsqadd_n_u64_m, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svsqadd_n_u64_x, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svsqadd_n_u64_z, "q2UWiq16bq2UWiWi", "n")
BUILTIN(__builtin_sve_svsqadd_n_u8_m, "q16Ucq16bq16UcSc", "n")
BUILTIN(__builtin_sve_svsqadd_n_u8_x, "q16Ucq16bq16UcSc", "n")
BUILTIN(__builtin_sve_svsqadd_n_u8_z, "q16Ucq16bq16UcSc", "n")
BUILTIN(__builtin_sve_svsqadd_u16_m, "q8Usq16bq8Usq8s", "n")
BUILTIN(__builtin_sve_svsqadd_u16_x, "q8Usq16bq8Usq8s", "n")
BUILTIN(__builtin_sve_svsqadd_u16_z, "q8Usq16bq8Usq8s", "n")
BUILTIN(__builtin_sve_svsqadd_u32_m, "q4Uiq16bq4Uiq4i", "n")
BUILTIN(__builtin_sve_svsqadd_u32_x, "q4Uiq16bq4Uiq4i", "n")
BUILTIN(__builtin_sve_svsqadd_u32_z, "q4Uiq16bq4Uiq4i", "n")
BUILTIN(__builtin_sve_svsqadd_u64_m, "q2UWiq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svsqadd_u64_x, "q2UWiq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svsqadd_u64_z, "q2UWiq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svsqadd_u8_m, "q16Ucq16bq16Ucq16Sc", "n")
BUILTIN(__builtin_sve_svsqadd_u8_x, "q16Ucq16bq16Ucq16Sc", "n")
BUILTIN(__builtin_sve_svsqadd_u8_z, "q16Ucq16bq16Ucq16Sc", "n")
BUILTIN(__builtin_sve_svsqrt_f16_m, "q8hq8hq16bq8h", "n")
BUILTIN(__builtin_sve_svsqrt_f16_x, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svsqrt_f16_z, "q8hq16bq8h", "n")
BUILTIN(__builtin_sve_svsqrt_f32_m, "q4fq4fq16bq4f", "n")
BUILTIN(__builtin_sve_svsqrt_f32_x, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svsqrt_f32_z, "q4fq16bq4f", "n")
BUILTIN(__builtin_sve_svsqrt_f64_m, "q2dq2dq16bq2d", "n")
BUILTIN(__builtin_sve_svsqrt_f64_x, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svsqrt_f64_z, "q2dq16bq2d", "n")
BUILTIN(__builtin_sve_svsra_n_s16, "q8sq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svsra_n_s32, "q4iq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svsra_n_s64, "q2Wiq2Wiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svsra_n_s8, "q16Scq16Scq16ScIUWi", "n")
BUILTIN(__builtin_sve_svsra_n_u16, "q8Usq8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svsra_n_u32, "q4Uiq4Uiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svsra_n_u64, "q2UWiq2UWiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svsra_n_u8, "q16Ucq16Ucq16UcIUWi", "n")
BUILTIN(__builtin_sve_svsri_n_s16, "q8sq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svsri_n_s32, "q4iq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svsri_n_s64, "q2Wiq2Wiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svsri_n_s8, "q16Scq16Scq16ScIUWi", "n")
BUILTIN(__builtin_sve_svsri_n_u16, "q8Usq8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svsri_n_u32, "q4Uiq4Uiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svsri_n_u64, "q2UWiq2UWiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svsri_n_u8, "q16Ucq16Ucq16UcIUWi", "n")
BUILTIN(__builtin_sve_svst1_bf16, "vq16by*q8y", "n")
BUILTIN(__builtin_sve_svst1_f16, "vq16bh*q8h", "n")
BUILTIN(__builtin_sve_svst1_f32, "vq16bf*q4f", "n")
BUILTIN(__builtin_sve_svst1_f64, "vq16bd*q2d", "n")
BUILTIN(__builtin_sve_svst1_s16, "vq16bSs*q8s", "n")
BUILTIN(__builtin_sve_svst1_s32, "vq16bSi*q4i", "n")
BUILTIN(__builtin_sve_svst1_s64, "vq16bSWi*q2Wi", "n")
BUILTIN(__builtin_sve_svst1_s8, "vq16bSc*q16Sc", "n")
BUILTIN(__builtin_sve_svst1_scatter_s32index_f32, "vq16bf*q4iq4f", "n")
BUILTIN(__builtin_sve_svst1_scatter_s32index_s32, "vq16bSi*q4iq4i", "n")
BUILTIN(__builtin_sve_svst1_scatter_s32index_u32, "vq16bUi*q4iq4Ui", "n")
BUILTIN(__builtin_sve_svst1_scatter_s32offset_f32, "vq16bf*q4iq4f", "n")
BUILTIN(__builtin_sve_svst1_scatter_s32offset_s32, "vq16bSi*q4iq4i", "n")
BUILTIN(__builtin_sve_svst1_scatter_s32offset_u32, "vq16bUi*q4iq4Ui", "n")
BUILTIN(__builtin_sve_svst1_scatter_s64index_f64, "vq16bd*q2Wiq2d", "n")
BUILTIN(__builtin_sve_svst1_scatter_s64index_s64, "vq16bSWi*q2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svst1_scatter_s64index_u64, "vq16bUWi*q2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svst1_scatter_s64offset_f64, "vq16bd*q2Wiq2d", "n")
BUILTIN(__builtin_sve_svst1_scatter_s64offset_s64, "vq16bSWi*q2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svst1_scatter_s64offset_u64, "vq16bUWi*q2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svst1_scatter_u32base_f32, "vq16bq4Uiq4f", "n")
BUILTIN(__builtin_sve_svst1_scatter_u32base_index_f32, "vq16bq4UiWiq4f", "n")
BUILTIN(__builtin_sve_svst1_scatter_u32base_index_s32, "vq16bq4UiWiq4i", "n")
BUILTIN(__builtin_sve_svst1_scatter_u32base_index_u32, "vq16bq4UiWiq4Ui", "n")
BUILTIN(__builtin_sve_svst1_scatter_u32base_offset_f32, "vq16bq4UiWiq4f", "n")
BUILTIN(__builtin_sve_svst1_scatter_u32base_offset_s32, "vq16bq4UiWiq4i", "n")
BUILTIN(__builtin_sve_svst1_scatter_u32base_offset_u32, "vq16bq4UiWiq4Ui", "n")
BUILTIN(__builtin_sve_svst1_scatter_u32base_s32, "vq16bq4Uiq4i", "n")
BUILTIN(__builtin_sve_svst1_scatter_u32base_u32, "vq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svst1_scatter_u32index_f32, "vq16bf*q4Uiq4f", "n")
BUILTIN(__builtin_sve_svst1_scatter_u32index_s32, "vq16bSi*q4Uiq4i", "n")
BUILTIN(__builtin_sve_svst1_scatter_u32index_u32, "vq16bUi*q4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svst1_scatter_u32offset_f32, "vq16bf*q4Uiq4f", "n")
BUILTIN(__builtin_sve_svst1_scatter_u32offset_s32, "vq16bSi*q4Uiq4i", "n")
BUILTIN(__builtin_sve_svst1_scatter_u32offset_u32, "vq16bUi*q4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svst1_scatter_u64base_f64, "vq16bq2UWiq2d", "n")
BUILTIN(__builtin_sve_svst1_scatter_u64base_index_f64, "vq16bq2UWiWiq2d", "n")
BUILTIN(__builtin_sve_svst1_scatter_u64base_index_s64, "vq16bq2UWiWiq2Wi", "n")
BUILTIN(__builtin_sve_svst1_scatter_u64base_index_u64, "vq16bq2UWiWiq2UWi", "n")
BUILTIN(__builtin_sve_svst1_scatter_u64base_offset_f64, "vq16bq2UWiWiq2d", "n")
BUILTIN(__builtin_sve_svst1_scatter_u64base_offset_s64, "vq16bq2UWiWiq2Wi", "n")
BUILTIN(__builtin_sve_svst1_scatter_u64base_offset_u64, "vq16bq2UWiWiq2UWi", "n")
BUILTIN(__builtin_sve_svst1_scatter_u64base_s64, "vq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svst1_scatter_u64base_u64, "vq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svst1_scatter_u64index_f64, "vq16bd*q2UWiq2d", "n")
BUILTIN(__builtin_sve_svst1_scatter_u64index_s64, "vq16bSWi*q2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svst1_scatter_u64index_u64, "vq16bUWi*q2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svst1_scatter_u64offset_f64, "vq16bd*q2UWiq2d", "n")
BUILTIN(__builtin_sve_svst1_scatter_u64offset_s64, "vq16bSWi*q2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svst1_scatter_u64offset_u64, "vq16bUWi*q2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svst1_u16, "vq16bUs*q8Us", "n")
BUILTIN(__builtin_sve_svst1_u32, "vq16bUi*q4Ui", "n")
BUILTIN(__builtin_sve_svst1_u64, "vq16bUWi*q2UWi", "n")
BUILTIN(__builtin_sve_svst1_u8, "vq16bUc*q16Uc", "n")
BUILTIN(__builtin_sve_svst1_vnum_bf16, "vq16by*Wiq8y", "n")
BUILTIN(__builtin_sve_svst1_vnum_f16, "vq16bh*Wiq8h", "n")
BUILTIN(__builtin_sve_svst1_vnum_f32, "vq16bf*Wiq4f", "n")
BUILTIN(__builtin_sve_svst1_vnum_f64, "vq16bd*Wiq2d", "n")
BUILTIN(__builtin_sve_svst1_vnum_s16, "vq16bSs*Wiq8s", "n")
BUILTIN(__builtin_sve_svst1_vnum_s32, "vq16bSi*Wiq4i", "n")
BUILTIN(__builtin_sve_svst1_vnum_s64, "vq16bSWi*Wiq2Wi", "n")
BUILTIN(__builtin_sve_svst1_vnum_s8, "vq16bSc*Wiq16Sc", "n")
BUILTIN(__builtin_sve_svst1_vnum_u16, "vq16bUs*Wiq8Us", "n")
BUILTIN(__builtin_sve_svst1_vnum_u32, "vq16bUi*Wiq4Ui", "n")
BUILTIN(__builtin_sve_svst1_vnum_u64, "vq16bUWi*Wiq2UWi", "n")
BUILTIN(__builtin_sve_svst1_vnum_u8, "vq16bUc*Wiq16Uc", "n")
BUILTIN(__builtin_sve_svst1b_s16, "vq16bSc*q8s", "n")
BUILTIN(__builtin_sve_svst1b_s32, "vq16bSc*q4i", "n")
BUILTIN(__builtin_sve_svst1b_s64, "vq16bSc*q2Wi", "n")
BUILTIN(__builtin_sve_svst1b_scatter_s32offset_s32, "vq16bSc*q4iq4i", "n")
BUILTIN(__builtin_sve_svst1b_scatter_s32offset_u32, "vq16bUc*q4iq4Ui", "n")
BUILTIN(__builtin_sve_svst1b_scatter_s64offset_s64, "vq16bSc*q2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svst1b_scatter_s64offset_u64, "vq16bUc*q2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svst1b_scatter_u32base_offset_s32, "vq16bq4UiWiq4i", "n")
BUILTIN(__builtin_sve_svst1b_scatter_u32base_offset_u32, "vq16bq4UiWiq4Ui", "n")
BUILTIN(__builtin_sve_svst1b_scatter_u32base_s32, "vq16bq4Uiq4i", "n")
BUILTIN(__builtin_sve_svst1b_scatter_u32base_u32, "vq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svst1b_scatter_u32offset_s32, "vq16bSc*q4Uiq4i", "n")
BUILTIN(__builtin_sve_svst1b_scatter_u32offset_u32, "vq16bUc*q4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svst1b_scatter_u64base_offset_s64, "vq16bq2UWiWiq2Wi", "n")
BUILTIN(__builtin_sve_svst1b_scatter_u64base_offset_u64, "vq16bq2UWiWiq2UWi", "n")
BUILTIN(__builtin_sve_svst1b_scatter_u64base_s64, "vq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svst1b_scatter_u64base_u64, "vq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svst1b_scatter_u64offset_s64, "vq16bSc*q2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svst1b_scatter_u64offset_u64, "vq16bUc*q2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svst1b_u16, "vq16bUc*q8Us", "n")
BUILTIN(__builtin_sve_svst1b_u32, "vq16bUc*q4Ui", "n")
BUILTIN(__builtin_sve_svst1b_u64, "vq16bUc*q2UWi", "n")
BUILTIN(__builtin_sve_svst1b_vnum_s16, "vq16bSc*Wiq8s", "n")
BUILTIN(__builtin_sve_svst1b_vnum_s32, "vq16bSc*Wiq4i", "n")
BUILTIN(__builtin_sve_svst1b_vnum_s64, "vq16bSc*Wiq2Wi", "n")
BUILTIN(__builtin_sve_svst1b_vnum_u16, "vq16bUc*Wiq8Us", "n")
BUILTIN(__builtin_sve_svst1b_vnum_u32, "vq16bUc*Wiq4Ui", "n")
BUILTIN(__builtin_sve_svst1b_vnum_u64, "vq16bUc*Wiq2UWi", "n")
BUILTIN(__builtin_sve_svst1h_s32, "vq16bSs*q4i", "n")
BUILTIN(__builtin_sve_svst1h_s64, "vq16bSs*q2Wi", "n")
BUILTIN(__builtin_sve_svst1h_scatter_s32index_s32, "vq16bSs*q4iq4i", "n")
BUILTIN(__builtin_sve_svst1h_scatter_s32index_u32, "vq16bUs*q4iq4Ui", "n")
BUILTIN(__builtin_sve_svst1h_scatter_s32offset_s32, "vq16bSs*q4iq4i", "n")
BUILTIN(__builtin_sve_svst1h_scatter_s32offset_u32, "vq16bUs*q4iq4Ui", "n")
BUILTIN(__builtin_sve_svst1h_scatter_s64index_s64, "vq16bSs*q2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svst1h_scatter_s64index_u64, "vq16bUs*q2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svst1h_scatter_s64offset_s64, "vq16bSs*q2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svst1h_scatter_s64offset_u64, "vq16bUs*q2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u32base_index_s32, "vq16bq4UiWiq4i", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u32base_index_u32, "vq16bq4UiWiq4Ui", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u32base_offset_s32, "vq16bq4UiWiq4i", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u32base_offset_u32, "vq16bq4UiWiq4Ui", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u32base_s32, "vq16bq4Uiq4i", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u32base_u32, "vq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u32index_s32, "vq16bSs*q4Uiq4i", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u32index_u32, "vq16bUs*q4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u32offset_s32, "vq16bSs*q4Uiq4i", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u32offset_u32, "vq16bUs*q4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u64base_index_s64, "vq16bq2UWiWiq2Wi", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u64base_index_u64, "vq16bq2UWiWiq2UWi", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u64base_offset_s64, "vq16bq2UWiWiq2Wi", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u64base_offset_u64, "vq16bq2UWiWiq2UWi", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u64base_s64, "vq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u64base_u64, "vq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u64index_s64, "vq16bSs*q2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u64index_u64, "vq16bUs*q2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u64offset_s64, "vq16bSs*q2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svst1h_scatter_u64offset_u64, "vq16bUs*q2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svst1h_u32, "vq16bUs*q4Ui", "n")
BUILTIN(__builtin_sve_svst1h_u64, "vq16bUs*q2UWi", "n")
BUILTIN(__builtin_sve_svst1h_vnum_s32, "vq16bSs*Wiq4i", "n")
BUILTIN(__builtin_sve_svst1h_vnum_s64, "vq16bSs*Wiq2Wi", "n")
BUILTIN(__builtin_sve_svst1h_vnum_u32, "vq16bUs*Wiq4Ui", "n")
BUILTIN(__builtin_sve_svst1h_vnum_u64, "vq16bUs*Wiq2UWi", "n")
BUILTIN(__builtin_sve_svst1w_s64, "vq16bSi*q2Wi", "n")
BUILTIN(__builtin_sve_svst1w_scatter_s64index_s64, "vq16bSi*q2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svst1w_scatter_s64index_u64, "vq16bUi*q2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svst1w_scatter_s64offset_s64, "vq16bSi*q2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svst1w_scatter_s64offset_u64, "vq16bUi*q2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svst1w_scatter_u64base_index_s64, "vq16bq2UWiWiq2Wi", "n")
BUILTIN(__builtin_sve_svst1w_scatter_u64base_index_u64, "vq16bq2UWiWiq2UWi", "n")
BUILTIN(__builtin_sve_svst1w_scatter_u64base_offset_s64, "vq16bq2UWiWiq2Wi", "n")
BUILTIN(__builtin_sve_svst1w_scatter_u64base_offset_u64, "vq16bq2UWiWiq2UWi", "n")
BUILTIN(__builtin_sve_svst1w_scatter_u64base_s64, "vq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svst1w_scatter_u64base_u64, "vq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svst1w_scatter_u64index_s64, "vq16bSi*q2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svst1w_scatter_u64index_u64, "vq16bUi*q2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svst1w_scatter_u64offset_s64, "vq16bSi*q2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svst1w_scatter_u64offset_u64, "vq16bUi*q2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svst1w_u64, "vq16bUi*q2UWi", "n")
BUILTIN(__builtin_sve_svst1w_vnum_s64, "vq16bSi*Wiq2Wi", "n")
BUILTIN(__builtin_sve_svst1w_vnum_u64, "vq16bUi*Wiq2UWi", "n")
BUILTIN(__builtin_sve_svst2_bf16, "vq16by*q16y", "n")
BUILTIN(__builtin_sve_svst2_f16, "vq16bh*q16h", "n")
BUILTIN(__builtin_sve_svst2_f32, "vq16bf*q8f", "n")
BUILTIN(__builtin_sve_svst2_f64, "vq16bd*q4d", "n")
BUILTIN(__builtin_sve_svst2_s16, "vq16bSs*q16s", "n")
BUILTIN(__builtin_sve_svst2_s32, "vq16bSi*q8i", "n")
BUILTIN(__builtin_sve_svst2_s64, "vq16bSWi*q4Wi", "n")
BUILTIN(__builtin_sve_svst2_s8, "vq16bSc*q32Sc", "n")
BUILTIN(__builtin_sve_svst2_u16, "vq16bUs*q16Us", "n")
BUILTIN(__builtin_sve_svst2_u32, "vq16bUi*q8Ui", "n")
BUILTIN(__builtin_sve_svst2_u64, "vq16bUWi*q4UWi", "n")
BUILTIN(__builtin_sve_svst2_u8, "vq16bUc*q32Uc", "n")
BUILTIN(__builtin_sve_svst2_vnum_bf16, "vq16by*Wiq16y", "n")
BUILTIN(__builtin_sve_svst2_vnum_f16, "vq16bh*Wiq16h", "n")
BUILTIN(__builtin_sve_svst2_vnum_f32, "vq16bf*Wiq8f", "n")
BUILTIN(__builtin_sve_svst2_vnum_f64, "vq16bd*Wiq4d", "n")
BUILTIN(__builtin_sve_svst2_vnum_s16, "vq16bSs*Wiq16s", "n")
BUILTIN(__builtin_sve_svst2_vnum_s32, "vq16bSi*Wiq8i", "n")
BUILTIN(__builtin_sve_svst2_vnum_s64, "vq16bSWi*Wiq4Wi", "n")
BUILTIN(__builtin_sve_svst2_vnum_s8, "vq16bSc*Wiq32Sc", "n")
BUILTIN(__builtin_sve_svst2_vnum_u16, "vq16bUs*Wiq16Us", "n")
BUILTIN(__builtin_sve_svst2_vnum_u32, "vq16bUi*Wiq8Ui", "n")
BUILTIN(__builtin_sve_svst2_vnum_u64, "vq16bUWi*Wiq4UWi", "n")
BUILTIN(__builtin_sve_svst2_vnum_u8, "vq16bUc*Wiq32Uc", "n")
BUILTIN(__builtin_sve_svst3_bf16, "vq16by*q24y", "n")
BUILTIN(__builtin_sve_svst3_f16, "vq16bh*q24h", "n")
BUILTIN(__builtin_sve_svst3_f32, "vq16bf*q12f", "n")
BUILTIN(__builtin_sve_svst3_f64, "vq16bd*q6d", "n")
BUILTIN(__builtin_sve_svst3_s16, "vq16bSs*q24s", "n")
BUILTIN(__builtin_sve_svst3_s32, "vq16bSi*q12i", "n")
BUILTIN(__builtin_sve_svst3_s64, "vq16bSWi*q6Wi", "n")
BUILTIN(__builtin_sve_svst3_s8, "vq16bSc*q48Sc", "n")
BUILTIN(__builtin_sve_svst3_u16, "vq16bUs*q24Us", "n")
BUILTIN(__builtin_sve_svst3_u32, "vq16bUi*q12Ui", "n")
BUILTIN(__builtin_sve_svst3_u64, "vq16bUWi*q6UWi", "n")
BUILTIN(__builtin_sve_svst3_u8, "vq16bUc*q48Uc", "n")
BUILTIN(__builtin_sve_svst3_vnum_bf16, "vq16by*Wiq24y", "n")
BUILTIN(__builtin_sve_svst3_vnum_f16, "vq16bh*Wiq24h", "n")
BUILTIN(__builtin_sve_svst3_vnum_f32, "vq16bf*Wiq12f", "n")
BUILTIN(__builtin_sve_svst3_vnum_f64, "vq16bd*Wiq6d", "n")
BUILTIN(__builtin_sve_svst3_vnum_s16, "vq16bSs*Wiq24s", "n")
BUILTIN(__builtin_sve_svst3_vnum_s32, "vq16bSi*Wiq12i", "n")
BUILTIN(__builtin_sve_svst3_vnum_s64, "vq16bSWi*Wiq6Wi", "n")
BUILTIN(__builtin_sve_svst3_vnum_s8, "vq16bSc*Wiq48Sc", "n")
BUILTIN(__builtin_sve_svst3_vnum_u16, "vq16bUs*Wiq24Us", "n")
BUILTIN(__builtin_sve_svst3_vnum_u32, "vq16bUi*Wiq12Ui", "n")
BUILTIN(__builtin_sve_svst3_vnum_u64, "vq16bUWi*Wiq6UWi", "n")
BUILTIN(__builtin_sve_svst3_vnum_u8, "vq16bUc*Wiq48Uc", "n")
BUILTIN(__builtin_sve_svst4_bf16, "vq16by*q32y", "n")
BUILTIN(__builtin_sve_svst4_f16, "vq16bh*q32h", "n")
BUILTIN(__builtin_sve_svst4_f32, "vq16bf*q16f", "n")
BUILTIN(__builtin_sve_svst4_f64, "vq16bd*q8d", "n")
BUILTIN(__builtin_sve_svst4_s16, "vq16bSs*q32s", "n")
BUILTIN(__builtin_sve_svst4_s32, "vq16bSi*q16i", "n")
BUILTIN(__builtin_sve_svst4_s64, "vq16bSWi*q8Wi", "n")
BUILTIN(__builtin_sve_svst4_s8, "vq16bSc*q64Sc", "n")
BUILTIN(__builtin_sve_svst4_u16, "vq16bUs*q32Us", "n")
BUILTIN(__builtin_sve_svst4_u32, "vq16bUi*q16Ui", "n")
BUILTIN(__builtin_sve_svst4_u64, "vq16bUWi*q8UWi", "n")
BUILTIN(__builtin_sve_svst4_u8, "vq16bUc*q64Uc", "n")
BUILTIN(__builtin_sve_svst4_vnum_bf16, "vq16by*Wiq32y", "n")
BUILTIN(__builtin_sve_svst4_vnum_f16, "vq16bh*Wiq32h", "n")
BUILTIN(__builtin_sve_svst4_vnum_f32, "vq16bf*Wiq16f", "n")
BUILTIN(__builtin_sve_svst4_vnum_f64, "vq16bd*Wiq8d", "n")
BUILTIN(__builtin_sve_svst4_vnum_s16, "vq16bSs*Wiq32s", "n")
BUILTIN(__builtin_sve_svst4_vnum_s32, "vq16bSi*Wiq16i", "n")
BUILTIN(__builtin_sve_svst4_vnum_s64, "vq16bSWi*Wiq8Wi", "n")
BUILTIN(__builtin_sve_svst4_vnum_s8, "vq16bSc*Wiq64Sc", "n")
BUILTIN(__builtin_sve_svst4_vnum_u16, "vq16bUs*Wiq32Us", "n")
BUILTIN(__builtin_sve_svst4_vnum_u32, "vq16bUi*Wiq16Ui", "n")
BUILTIN(__builtin_sve_svst4_vnum_u64, "vq16bUWi*Wiq8UWi", "n")
BUILTIN(__builtin_sve_svst4_vnum_u8, "vq16bUc*Wiq64Uc", "n")
BUILTIN(__builtin_sve_svstnt1_bf16, "vq16by*q8y", "n")
BUILTIN(__builtin_sve_svstnt1_f16, "vq16bh*q8h", "n")
BUILTIN(__builtin_sve_svstnt1_f32, "vq16bf*q4f", "n")
BUILTIN(__builtin_sve_svstnt1_f64, "vq16bd*q2d", "n")
BUILTIN(__builtin_sve_svstnt1_s16, "vq16bSs*q8s", "n")
BUILTIN(__builtin_sve_svstnt1_s32, "vq16bSi*q4i", "n")
BUILTIN(__builtin_sve_svstnt1_s64, "vq16bSWi*q2Wi", "n")
BUILTIN(__builtin_sve_svstnt1_s8, "vq16bSc*q16Sc", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_s64index_f64, "vq16bd*q2Wiq2d", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_s64index_s64, "vq16bSWi*q2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_s64index_u64, "vq16bUWi*q2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_s64offset_f64, "vq16bd*q2Wiq2d", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_s64offset_s64, "vq16bSWi*q2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_s64offset_u64, "vq16bUWi*q2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u32base_f32, "vq16bq4Uiq4f", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u32base_index_f32, "vq16bq4UiWiq4f", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u32base_index_s32, "vq16bq4UiWiq4i", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u32base_index_u32, "vq16bq4UiWiq4Ui", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u32base_offset_f32, "vq16bq4UiWiq4f", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u32base_offset_s32, "vq16bq4UiWiq4i", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u32base_offset_u32, "vq16bq4UiWiq4Ui", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u32base_s32, "vq16bq4Uiq4i", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u32base_u32, "vq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u32offset_f32, "vq16bf*q4Uiq4f", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u32offset_s32, "vq16bSi*q4Uiq4i", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u32offset_u32, "vq16bUi*q4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u64base_f64, "vq16bq2UWiq2d", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u64base_index_f64, "vq16bq2UWiWiq2d", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u64base_index_s64, "vq16bq2UWiWiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u64base_index_u64, "vq16bq2UWiWiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u64base_offset_f64, "vq16bq2UWiWiq2d", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u64base_offset_s64, "vq16bq2UWiWiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u64base_offset_u64, "vq16bq2UWiWiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u64base_s64, "vq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u64base_u64, "vq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u64index_f64, "vq16bd*q2UWiq2d", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u64index_s64, "vq16bSWi*q2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u64index_u64, "vq16bUWi*q2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u64offset_f64, "vq16bd*q2UWiq2d", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u64offset_s64, "vq16bSWi*q2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1_scatter_u64offset_u64, "vq16bUWi*q2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1_u16, "vq16bUs*q8Us", "n")
BUILTIN(__builtin_sve_svstnt1_u32, "vq16bUi*q4Ui", "n")
BUILTIN(__builtin_sve_svstnt1_u64, "vq16bUWi*q2UWi", "n")
BUILTIN(__builtin_sve_svstnt1_u8, "vq16bUc*q16Uc", "n")
BUILTIN(__builtin_sve_svstnt1_vnum_bf16, "vq16by*Wiq8y", "n")
BUILTIN(__builtin_sve_svstnt1_vnum_f16, "vq16bh*Wiq8h", "n")
BUILTIN(__builtin_sve_svstnt1_vnum_f32, "vq16bf*Wiq4f", "n")
BUILTIN(__builtin_sve_svstnt1_vnum_f64, "vq16bd*Wiq2d", "n")
BUILTIN(__builtin_sve_svstnt1_vnum_s16, "vq16bSs*Wiq8s", "n")
BUILTIN(__builtin_sve_svstnt1_vnum_s32, "vq16bSi*Wiq4i", "n")
BUILTIN(__builtin_sve_svstnt1_vnum_s64, "vq16bSWi*Wiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1_vnum_s8, "vq16bSc*Wiq16Sc", "n")
BUILTIN(__builtin_sve_svstnt1_vnum_u16, "vq16bUs*Wiq8Us", "n")
BUILTIN(__builtin_sve_svstnt1_vnum_u32, "vq16bUi*Wiq4Ui", "n")
BUILTIN(__builtin_sve_svstnt1_vnum_u64, "vq16bUWi*Wiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1_vnum_u8, "vq16bUc*Wiq16Uc", "n")
BUILTIN(__builtin_sve_svstnt1b_scatter_s64offset_s64, "vq16bSc*q2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1b_scatter_s64offset_u64, "vq16bUc*q2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1b_scatter_u32base_offset_s32, "vq16bq4UiWiq4i", "n")
BUILTIN(__builtin_sve_svstnt1b_scatter_u32base_offset_u32, "vq16bq4UiWiq4Ui", "n")
BUILTIN(__builtin_sve_svstnt1b_scatter_u32base_s32, "vq16bq4Uiq4i", "n")
BUILTIN(__builtin_sve_svstnt1b_scatter_u32base_u32, "vq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svstnt1b_scatter_u32offset_s32, "vq16bSc*q4Uiq4i", "n")
BUILTIN(__builtin_sve_svstnt1b_scatter_u32offset_u32, "vq16bUc*q4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svstnt1b_scatter_u64base_offset_s64, "vq16bq2UWiWiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1b_scatter_u64base_offset_u64, "vq16bq2UWiWiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1b_scatter_u64base_s64, "vq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1b_scatter_u64base_u64, "vq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1b_scatter_u64offset_s64, "vq16bSc*q2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1b_scatter_u64offset_u64, "vq16bUc*q2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_s64index_s64, "vq16bSs*q2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_s64index_u64, "vq16bUs*q2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_s64offset_s64, "vq16bSs*q2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_s64offset_u64, "vq16bUs*q2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_u32base_index_s32, "vq16bq4UiWiq4i", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_u32base_index_u32, "vq16bq4UiWiq4Ui", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_u32base_offset_s32, "vq16bq4UiWiq4i", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_u32base_offset_u32, "vq16bq4UiWiq4Ui", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_u32base_s32, "vq16bq4Uiq4i", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_u32base_u32, "vq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_u32offset_s32, "vq16bSs*q4Uiq4i", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_u32offset_u32, "vq16bUs*q4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_u64base_index_s64, "vq16bq2UWiWiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_u64base_index_u64, "vq16bq2UWiWiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_u64base_offset_s64, "vq16bq2UWiWiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_u64base_offset_u64, "vq16bq2UWiWiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_u64base_s64, "vq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_u64base_u64, "vq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_u64index_s64, "vq16bSs*q2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_u64index_u64, "vq16bUs*q2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_u64offset_s64, "vq16bSs*q2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1h_scatter_u64offset_u64, "vq16bUs*q2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1w_scatter_s64index_s64, "vq16bSi*q2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1w_scatter_s64index_u64, "vq16bUi*q2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1w_scatter_s64offset_s64, "vq16bSi*q2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1w_scatter_s64offset_u64, "vq16bUi*q2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1w_scatter_u64base_index_s64, "vq16bq2UWiWiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1w_scatter_u64base_index_u64, "vq16bq2UWiWiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1w_scatter_u64base_offset_s64, "vq16bq2UWiWiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1w_scatter_u64base_offset_u64, "vq16bq2UWiWiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1w_scatter_u64base_s64, "vq16bq2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1w_scatter_u64base_u64, "vq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1w_scatter_u64index_s64, "vq16bSi*q2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1w_scatter_u64index_u64, "vq16bUi*q2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svstnt1w_scatter_u64offset_s64, "vq16bSi*q2UWiq2Wi", "n")
BUILTIN(__builtin_sve_svstnt1w_scatter_u64offset_u64, "vq16bUi*q2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svsub_f16_m, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svsub_f16_x, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svsub_f16_z, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svsub_f32_m, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svsub_f32_x, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svsub_f32_z, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svsub_f64_m, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svsub_f64_x, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svsub_f64_z, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svsub_n_f16_m, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svsub_n_f16_x, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svsub_n_f16_z, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svsub_n_f32_m, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svsub_n_f32_x, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svsub_n_f32_z, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svsub_n_f64_m, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svsub_n_f64_x, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svsub_n_f64_z, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svsub_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svsub_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svsub_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svsub_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svsub_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svsub_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svsub_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svsub_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svsub_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svsub_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svsub_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svsub_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svsub_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svsub_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svsub_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svsub_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svsub_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svsub_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svsub_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svsub_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svsub_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svsub_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svsub_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svsub_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svsub_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svsub_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svsub_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svsub_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svsub_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svsub_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svsub_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svsub_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svsub_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svsub_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svsub_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svsub_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svsub_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svsub_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svsub_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svsub_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svsub_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svsub_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svsub_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svsub_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svsub_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svsub_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svsub_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svsub_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svsubhnb_n_s16, "q16Scq8ss", "n")
BUILTIN(__builtin_sve_svsubhnb_n_s32, "q8sq4ii", "n")
BUILTIN(__builtin_sve_svsubhnb_n_s64, "q4iq2WiWi", "n")
BUILTIN(__builtin_sve_svsubhnb_n_u16, "q16Ucq8UsUs", "n")
BUILTIN(__builtin_sve_svsubhnb_n_u32, "q8Usq4UiUi", "n")
BUILTIN(__builtin_sve_svsubhnb_n_u64, "q4Uiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svsubhnb_s16, "q16Scq8sq8s", "n")
BUILTIN(__builtin_sve_svsubhnb_s32, "q8sq4iq4i", "n")
BUILTIN(__builtin_sve_svsubhnb_s64, "q4iq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svsubhnb_u16, "q16Ucq8Usq8Us", "n")
BUILTIN(__builtin_sve_svsubhnb_u32, "q8Usq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svsubhnb_u64, "q4Uiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svsubhnt_n_s16, "q16Scq16Scq8ss", "n")
BUILTIN(__builtin_sve_svsubhnt_n_s32, "q8sq8sq4ii", "n")
BUILTIN(__builtin_sve_svsubhnt_n_s64, "q4iq4iq2WiWi", "n")
BUILTIN(__builtin_sve_svsubhnt_n_u16, "q16Ucq16Ucq8UsUs", "n")
BUILTIN(__builtin_sve_svsubhnt_n_u32, "q8Usq8Usq4UiUi", "n")
BUILTIN(__builtin_sve_svsubhnt_n_u64, "q4Uiq4Uiq2UWiUWi", "n")
BUILTIN(__builtin_sve_svsubhnt_s16, "q16Scq16Scq8sq8s", "n")
BUILTIN(__builtin_sve_svsubhnt_s32, "q8sq8sq4iq4i", "n")
BUILTIN(__builtin_sve_svsubhnt_s64, "q4iq4iq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svsubhnt_u16, "q16Ucq16Ucq8Usq8Us", "n")
BUILTIN(__builtin_sve_svsubhnt_u32, "q8Usq8Usq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svsubhnt_u64, "q4Uiq4Uiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svsublb_n_s16, "q8sq16ScSc", "n")
BUILTIN(__builtin_sve_svsublb_n_s32, "q4iq8ss", "n")
BUILTIN(__builtin_sve_svsublb_n_s64, "q2Wiq4ii", "n")
BUILTIN(__builtin_sve_svsublb_n_u16, "q8Usq16UcUc", "n")
BUILTIN(__builtin_sve_svsublb_n_u32, "q4Uiq8UsUs", "n")
BUILTIN(__builtin_sve_svsublb_n_u64, "q2UWiq4UiUi", "n")
BUILTIN(__builtin_sve_svsublb_s16, "q8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svsublb_s32, "q4iq8sq8s", "n")
BUILTIN(__builtin_sve_svsublb_s64, "q2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svsublb_u16, "q8Usq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svsublb_u32, "q4Uiq8Usq8Us", "n")
BUILTIN(__builtin_sve_svsublb_u64, "q2UWiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svsublbt_n_s16, "q8sq16ScSc", "n")
BUILTIN(__builtin_sve_svsublbt_n_s32, "q4iq8ss", "n")
BUILTIN(__builtin_sve_svsublbt_n_s64, "q2Wiq4ii", "n")
BUILTIN(__builtin_sve_svsublbt_s16, "q8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svsublbt_s32, "q4iq8sq8s", "n")
BUILTIN(__builtin_sve_svsublbt_s64, "q2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svsublt_n_s16, "q8sq16ScSc", "n")
BUILTIN(__builtin_sve_svsublt_n_s32, "q4iq8ss", "n")
BUILTIN(__builtin_sve_svsublt_n_s64, "q2Wiq4ii", "n")
BUILTIN(__builtin_sve_svsublt_n_u16, "q8Usq16UcUc", "n")
BUILTIN(__builtin_sve_svsublt_n_u32, "q4Uiq8UsUs", "n")
BUILTIN(__builtin_sve_svsublt_n_u64, "q2UWiq4UiUi", "n")
BUILTIN(__builtin_sve_svsublt_s16, "q8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svsublt_s32, "q4iq8sq8s", "n")
BUILTIN(__builtin_sve_svsublt_s64, "q2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svsublt_u16, "q8Usq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svsublt_u32, "q4Uiq8Usq8Us", "n")
BUILTIN(__builtin_sve_svsublt_u64, "q2UWiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svsubltb_n_s16, "q8sq16ScSc", "n")
BUILTIN(__builtin_sve_svsubltb_n_s32, "q4iq8ss", "n")
BUILTIN(__builtin_sve_svsubltb_n_s64, "q2Wiq4ii", "n")
BUILTIN(__builtin_sve_svsubltb_s16, "q8sq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svsubltb_s32, "q4iq8sq8s", "n")
BUILTIN(__builtin_sve_svsubltb_s64, "q2Wiq4iq4i", "n")
BUILTIN(__builtin_sve_svsubr_f16_m, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svsubr_f16_x, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svsubr_f16_z, "q8hq16bq8hq8h", "n")
BUILTIN(__builtin_sve_svsubr_f32_m, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svsubr_f32_x, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svsubr_f32_z, "q4fq16bq4fq4f", "n")
BUILTIN(__builtin_sve_svsubr_f64_m, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svsubr_f64_x, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svsubr_f64_z, "q2dq16bq2dq2d", "n")
BUILTIN(__builtin_sve_svsubr_n_f16_m, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svsubr_n_f16_x, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svsubr_n_f16_z, "q8hq16bq8hh", "n")
BUILTIN(__builtin_sve_svsubr_n_f32_m, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svsubr_n_f32_x, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svsubr_n_f32_z, "q4fq16bq4ff", "n")
BUILTIN(__builtin_sve_svsubr_n_f64_m, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svsubr_n_f64_x, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svsubr_n_f64_z, "q2dq16bq2dd", "n")
BUILTIN(__builtin_sve_svsubr_n_s16_m, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svsubr_n_s16_x, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svsubr_n_s16_z, "q8sq16bq8ss", "n")
BUILTIN(__builtin_sve_svsubr_n_s32_m, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svsubr_n_s32_x, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svsubr_n_s32_z, "q4iq16bq4ii", "n")
BUILTIN(__builtin_sve_svsubr_n_s64_m, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svsubr_n_s64_x, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svsubr_n_s64_z, "q2Wiq16bq2WiWi", "n")
BUILTIN(__builtin_sve_svsubr_n_s8_m, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svsubr_n_s8_x, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svsubr_n_s8_z, "q16Scq16bq16ScSc", "n")
BUILTIN(__builtin_sve_svsubr_n_u16_m, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svsubr_n_u16_x, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svsubr_n_u16_z, "q8Usq16bq8UsUs", "n")
BUILTIN(__builtin_sve_svsubr_n_u32_m, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svsubr_n_u32_x, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svsubr_n_u32_z, "q4Uiq16bq4UiUi", "n")
BUILTIN(__builtin_sve_svsubr_n_u64_m, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svsubr_n_u64_x, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svsubr_n_u64_z, "q2UWiq16bq2UWiUWi", "n")
BUILTIN(__builtin_sve_svsubr_n_u8_m, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svsubr_n_u8_x, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svsubr_n_u8_z, "q16Ucq16bq16UcUc", "n")
BUILTIN(__builtin_sve_svsubr_s16_m, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svsubr_s16_x, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svsubr_s16_z, "q8sq16bq8sq8s", "n")
BUILTIN(__builtin_sve_svsubr_s32_m, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svsubr_s32_x, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svsubr_s32_z, "q4iq16bq4iq4i", "n")
BUILTIN(__builtin_sve_svsubr_s64_m, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svsubr_s64_x, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svsubr_s64_z, "q2Wiq16bq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svsubr_s8_m, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svsubr_s8_x, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svsubr_s8_z, "q16Scq16bq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svsubr_u16_m, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svsubr_u16_x, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svsubr_u16_z, "q8Usq16bq8Usq8Us", "n")
BUILTIN(__builtin_sve_svsubr_u32_m, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svsubr_u32_x, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svsubr_u32_z, "q4Uiq16bq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svsubr_u64_m, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svsubr_u64_x, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svsubr_u64_z, "q2UWiq16bq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svsubr_u8_m, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svsubr_u8_x, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svsubr_u8_z, "q16Ucq16bq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svsubwb_n_s16, "q8sq8sSc", "n")
BUILTIN(__builtin_sve_svsubwb_n_s32, "q4iq4is", "n")
BUILTIN(__builtin_sve_svsubwb_n_s64, "q2Wiq2Wii", "n")
BUILTIN(__builtin_sve_svsubwb_n_u16, "q8Usq8UsUc", "n")
BUILTIN(__builtin_sve_svsubwb_n_u32, "q4Uiq4UiUs", "n")
BUILTIN(__builtin_sve_svsubwb_n_u64, "q2UWiq2UWiUi", "n")
BUILTIN(__builtin_sve_svsubwb_s16, "q8sq8sq16Sc", "n")
BUILTIN(__builtin_sve_svsubwb_s32, "q4iq4iq8s", "n")
BUILTIN(__builtin_sve_svsubwb_s64, "q2Wiq2Wiq4i", "n")
BUILTIN(__builtin_sve_svsubwb_u16, "q8Usq8Usq16Uc", "n")
BUILTIN(__builtin_sve_svsubwb_u32, "q4Uiq4Uiq8Us", "n")
BUILTIN(__builtin_sve_svsubwb_u64, "q2UWiq2UWiq4Ui", "n")
BUILTIN(__builtin_sve_svsubwt_n_s16, "q8sq8sSc", "n")
BUILTIN(__builtin_sve_svsubwt_n_s32, "q4iq4is", "n")
BUILTIN(__builtin_sve_svsubwt_n_s64, "q2Wiq2Wii", "n")
BUILTIN(__builtin_sve_svsubwt_n_u16, "q8Usq8UsUc", "n")
BUILTIN(__builtin_sve_svsubwt_n_u32, "q4Uiq4UiUs", "n")
BUILTIN(__builtin_sve_svsubwt_n_u64, "q2UWiq2UWiUi", "n")
BUILTIN(__builtin_sve_svsubwt_s16, "q8sq8sq16Sc", "n")
BUILTIN(__builtin_sve_svsubwt_s32, "q4iq4iq8s", "n")
BUILTIN(__builtin_sve_svsubwt_s64, "q2Wiq2Wiq4i", "n")
BUILTIN(__builtin_sve_svsubwt_u16, "q8Usq8Usq16Uc", "n")
BUILTIN(__builtin_sve_svsubwt_u32, "q4Uiq4Uiq8Us", "n")
BUILTIN(__builtin_sve_svsubwt_u64, "q2UWiq2UWiq4Ui", "n")
BUILTIN(__builtin_sve_svsudot_lane_s32, "q4iq4iq16Scq16UcIUWi", "n")
BUILTIN(__builtin_sve_svsudot_n_s32, "q4iq4iq16ScUc", "n")
BUILTIN(__builtin_sve_svsudot_s32, "q4iq4iq16Scq16Uc", "n")
BUILTIN(__builtin_sve_svtbl2_bf16, "q8yq16yq8Us", "n")
BUILTIN(__builtin_sve_svtbl2_f16, "q8hq16hq8Us", "n")
BUILTIN(__builtin_sve_svtbl2_f32, "q4fq8fq4Ui", "n")
BUILTIN(__builtin_sve_svtbl2_f64, "q2dq4dq2UWi", "n")
BUILTIN(__builtin_sve_svtbl2_s16, "q8sq16sq8Us", "n")
BUILTIN(__builtin_sve_svtbl2_s32, "q4iq8iq4Ui", "n")
BUILTIN(__builtin_sve_svtbl2_s64, "q2Wiq4Wiq2UWi", "n")
BUILTIN(__builtin_sve_svtbl2_s8, "q16Scq32Scq16Uc", "n")
BUILTIN(__builtin_sve_svtbl2_u16, "q8Usq16Usq8Us", "n")
BUILTIN(__builtin_sve_svtbl2_u32, "q4Uiq8Uiq4Ui", "n")
BUILTIN(__builtin_sve_svtbl2_u64, "q2UWiq4UWiq2UWi", "n")
BUILTIN(__builtin_sve_svtbl2_u8, "q16Ucq32Ucq16Uc", "n")
BUILTIN(__builtin_sve_svtbl_bf16, "q8yq8yq8Us", "n")
BUILTIN(__builtin_sve_svtbl_f16, "q8hq8hq8Us", "n")
BUILTIN(__builtin_sve_svtbl_f32, "q4fq4fq4Ui", "n")
BUILTIN(__builtin_sve_svtbl_f64, "q2dq2dq2UWi", "n")
BUILTIN(__builtin_sve_svtbl_s16, "q8sq8sq8Us", "n")
BUILTIN(__builtin_sve_svtbl_s32, "q4iq4iq4Ui", "n")
BUILTIN(__builtin_sve_svtbl_s64, "q2Wiq2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svtbl_s8, "q16Scq16Scq16Uc", "n")
BUILTIN(__builtin_sve_svtbl_u16, "q8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svtbl_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svtbl_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svtbl_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svtbx_bf16, "q8yq8yq8yq8Us", "n")
BUILTIN(__builtin_sve_svtbx_f16, "q8hq8hq8hq8Us", "n")
BUILTIN(__builtin_sve_svtbx_f32, "q4fq4fq4fq4Ui", "n")
BUILTIN(__builtin_sve_svtbx_f64, "q2dq2dq2dq2UWi", "n")
BUILTIN(__builtin_sve_svtbx_s16, "q8sq8sq8sq8Us", "n")
BUILTIN(__builtin_sve_svtbx_s32, "q4iq4iq4iq4Ui", "n")
BUILTIN(__builtin_sve_svtbx_s64, "q2Wiq2Wiq2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svtbx_s8, "q16Scq16Scq16Scq16Uc", "n")
BUILTIN(__builtin_sve_svtbx_u16, "q8Usq8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svtbx_u32, "q4Uiq4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svtbx_u64, "q2UWiq2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svtbx_u8, "q16Ucq16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svtmad_f16, "q8hq8hq8hIUWi", "n")
BUILTIN(__builtin_sve_svtmad_f32, "q4fq4fq4fIUWi", "n")
BUILTIN(__builtin_sve_svtmad_f64, "q2dq2dq2dIUWi", "n")
BUILTIN(__builtin_sve_svtrn1_b16, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svtrn1_b32, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svtrn1_b64, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svtrn1_b8, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svtrn1_bf16, "q8yq8yq8y", "n")
BUILTIN(__builtin_sve_svtrn1_f16, "q8hq8hq8h", "n")
BUILTIN(__builtin_sve_svtrn1_f32, "q4fq4fq4f", "n")
BUILTIN(__builtin_sve_svtrn1_f64, "q2dq2dq2d", "n")
BUILTIN(__builtin_sve_svtrn1_s16, "q8sq8sq8s", "n")
BUILTIN(__builtin_sve_svtrn1_s32, "q4iq4iq4i", "n")
BUILTIN(__builtin_sve_svtrn1_s64, "q2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svtrn1_s8, "q16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svtrn1_u16, "q8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svtrn1_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svtrn1_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svtrn1_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svtrn1q_bf16, "q8yq8yq8y", "n")
BUILTIN(__builtin_sve_svtrn1q_f16, "q8hq8hq8h", "n")
BUILTIN(__builtin_sve_svtrn1q_f32, "q4fq4fq4f", "n")
BUILTIN(__builtin_sve_svtrn1q_f64, "q2dq2dq2d", "n")
BUILTIN(__builtin_sve_svtrn1q_s16, "q8sq8sq8s", "n")
BUILTIN(__builtin_sve_svtrn1q_s32, "q4iq4iq4i", "n")
BUILTIN(__builtin_sve_svtrn1q_s64, "q2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svtrn1q_s8, "q16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svtrn1q_u16, "q8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svtrn1q_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svtrn1q_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svtrn1q_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svtrn2_b16, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svtrn2_b32, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svtrn2_b64, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svtrn2_b8, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svtrn2_bf16, "q8yq8yq8y", "n")
BUILTIN(__builtin_sve_svtrn2_f16, "q8hq8hq8h", "n")
BUILTIN(__builtin_sve_svtrn2_f32, "q4fq4fq4f", "n")
BUILTIN(__builtin_sve_svtrn2_f64, "q2dq2dq2d", "n")
BUILTIN(__builtin_sve_svtrn2_s16, "q8sq8sq8s", "n")
BUILTIN(__builtin_sve_svtrn2_s32, "q4iq4iq4i", "n")
BUILTIN(__builtin_sve_svtrn2_s64, "q2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svtrn2_s8, "q16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svtrn2_u16, "q8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svtrn2_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svtrn2_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svtrn2_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svtrn2q_bf16, "q8yq8yq8y", "n")
BUILTIN(__builtin_sve_svtrn2q_f16, "q8hq8hq8h", "n")
BUILTIN(__builtin_sve_svtrn2q_f32, "q4fq4fq4f", "n")
BUILTIN(__builtin_sve_svtrn2q_f64, "q2dq2dq2d", "n")
BUILTIN(__builtin_sve_svtrn2q_s16, "q8sq8sq8s", "n")
BUILTIN(__builtin_sve_svtrn2q_s32, "q4iq4iq4i", "n")
BUILTIN(__builtin_sve_svtrn2q_s64, "q2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svtrn2q_s8, "q16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svtrn2q_u16, "q8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svtrn2q_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svtrn2q_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svtrn2q_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svtsmul_f16, "q8hq8hq8Us", "n")
BUILTIN(__builtin_sve_svtsmul_f32, "q4fq4fq4Ui", "n")
BUILTIN(__builtin_sve_svtsmul_f64, "q2dq2dq2UWi", "n")
BUILTIN(__builtin_sve_svtssel_f16, "q8hq8hq8Us", "n")
BUILTIN(__builtin_sve_svtssel_f32, "q4fq4fq4Ui", "n")
BUILTIN(__builtin_sve_svtssel_f64, "q2dq2dq2UWi", "n")
BUILTIN(__builtin_sve_svundef2_bf16, "q16y", "n")
BUILTIN(__builtin_sve_svundef2_f16, "q16h", "n")
BUILTIN(__builtin_sve_svundef2_f32, "q8f", "n")
BUILTIN(__builtin_sve_svundef2_f64, "q4d", "n")
BUILTIN(__builtin_sve_svundef2_s16, "q16s", "n")
BUILTIN(__builtin_sve_svundef2_s32, "q8i", "n")
BUILTIN(__builtin_sve_svundef2_s64, "q4Wi", "n")
BUILTIN(__builtin_sve_svundef2_s8, "q32Sc", "n")
BUILTIN(__builtin_sve_svundef2_u16, "q16Us", "n")
BUILTIN(__builtin_sve_svundef2_u32, "q8Ui", "n")
BUILTIN(__builtin_sve_svundef2_u64, "q4UWi", "n")
BUILTIN(__builtin_sve_svundef2_u8, "q32Uc", "n")
BUILTIN(__builtin_sve_svundef3_bf16, "q24y", "n")
BUILTIN(__builtin_sve_svundef3_f16, "q24h", "n")
BUILTIN(__builtin_sve_svundef3_f32, "q12f", "n")
BUILTIN(__builtin_sve_svundef3_f64, "q6d", "n")
BUILTIN(__builtin_sve_svundef3_s16, "q24s", "n")
BUILTIN(__builtin_sve_svundef3_s32, "q12i", "n")
BUILTIN(__builtin_sve_svundef3_s64, "q6Wi", "n")
BUILTIN(__builtin_sve_svundef3_s8, "q48Sc", "n")
BUILTIN(__builtin_sve_svundef3_u16, "q24Us", "n")
BUILTIN(__builtin_sve_svundef3_u32, "q12Ui", "n")
BUILTIN(__builtin_sve_svundef3_u64, "q6UWi", "n")
BUILTIN(__builtin_sve_svundef3_u8, "q48Uc", "n")
BUILTIN(__builtin_sve_svundef4_bf16, "q32y", "n")
BUILTIN(__builtin_sve_svundef4_f16, "q32h", "n")
BUILTIN(__builtin_sve_svundef4_f32, "q16f", "n")
BUILTIN(__builtin_sve_svundef4_f64, "q8d", "n")
BUILTIN(__builtin_sve_svundef4_s16, "q32s", "n")
BUILTIN(__builtin_sve_svundef4_s32, "q16i", "n")
BUILTIN(__builtin_sve_svundef4_s64, "q8Wi", "n")
BUILTIN(__builtin_sve_svundef4_s8, "q64Sc", "n")
BUILTIN(__builtin_sve_svundef4_u16, "q32Us", "n")
BUILTIN(__builtin_sve_svundef4_u32, "q16Ui", "n")
BUILTIN(__builtin_sve_svundef4_u64, "q8UWi", "n")
BUILTIN(__builtin_sve_svundef4_u8, "q64Uc", "n")
BUILTIN(__builtin_sve_svundef_bf16, "q8y", "n")
BUILTIN(__builtin_sve_svundef_f16, "q8h", "n")
BUILTIN(__builtin_sve_svundef_f32, "q4f", "n")
BUILTIN(__builtin_sve_svundef_f64, "q2d", "n")
BUILTIN(__builtin_sve_svundef_s16, "q8s", "n")
BUILTIN(__builtin_sve_svundef_s32, "q4i", "n")
BUILTIN(__builtin_sve_svundef_s64, "q2Wi", "n")
BUILTIN(__builtin_sve_svundef_s8, "q16Sc", "n")
BUILTIN(__builtin_sve_svundef_u16, "q8Us", "n")
BUILTIN(__builtin_sve_svundef_u32, "q4Ui", "n")
BUILTIN(__builtin_sve_svundef_u64, "q2UWi", "n")
BUILTIN(__builtin_sve_svundef_u8, "q16Uc", "n")
BUILTIN(__builtin_sve_svunpkhi_b, "q16bq16b", "n")
BUILTIN(__builtin_sve_svunpkhi_s16, "q8sq16Sc", "n")
BUILTIN(__builtin_sve_svunpkhi_s32, "q4iq8s", "n")
BUILTIN(__builtin_sve_svunpkhi_s64, "q2Wiq4i", "n")
BUILTIN(__builtin_sve_svunpkhi_u16, "q8Usq16Uc", "n")
BUILTIN(__builtin_sve_svunpkhi_u32, "q4Uiq8Us", "n")
BUILTIN(__builtin_sve_svunpkhi_u64, "q2UWiq4Ui", "n")
BUILTIN(__builtin_sve_svunpklo_b, "q16bq16b", "n")
BUILTIN(__builtin_sve_svunpklo_s16, "q8sq16Sc", "n")
BUILTIN(__builtin_sve_svunpklo_s32, "q4iq8s", "n")
BUILTIN(__builtin_sve_svunpklo_s64, "q2Wiq4i", "n")
BUILTIN(__builtin_sve_svunpklo_u16, "q8Usq16Uc", "n")
BUILTIN(__builtin_sve_svunpklo_u32, "q4Uiq8Us", "n")
BUILTIN(__builtin_sve_svunpklo_u64, "q2UWiq4Ui", "n")
BUILTIN(__builtin_sve_svuqadd_n_s16_m, "q8sq16bq8sUs", "n")
BUILTIN(__builtin_sve_svuqadd_n_s16_x, "q8sq16bq8sUs", "n")
BUILTIN(__builtin_sve_svuqadd_n_s16_z, "q8sq16bq8sUs", "n")
BUILTIN(__builtin_sve_svuqadd_n_s32_m, "q4iq16bq4iUi", "n")
BUILTIN(__builtin_sve_svuqadd_n_s32_x, "q4iq16bq4iUi", "n")
BUILTIN(__builtin_sve_svuqadd_n_s32_z, "q4iq16bq4iUi", "n")
BUILTIN(__builtin_sve_svuqadd_n_s64_m, "q2Wiq16bq2WiUWi", "n")
BUILTIN(__builtin_sve_svuqadd_n_s64_x, "q2Wiq16bq2WiUWi", "n")
BUILTIN(__builtin_sve_svuqadd_n_s64_z, "q2Wiq16bq2WiUWi", "n")
BUILTIN(__builtin_sve_svuqadd_n_s8_m, "q16Scq16bq16ScUc", "n")
BUILTIN(__builtin_sve_svuqadd_n_s8_x, "q16Scq16bq16ScUc", "n")
BUILTIN(__builtin_sve_svuqadd_n_s8_z, "q16Scq16bq16ScUc", "n")
BUILTIN(__builtin_sve_svuqadd_s16_m, "q8sq16bq8sq8Us", "n")
BUILTIN(__builtin_sve_svuqadd_s16_x, "q8sq16bq8sq8Us", "n")
BUILTIN(__builtin_sve_svuqadd_s16_z, "q8sq16bq8sq8Us", "n")
BUILTIN(__builtin_sve_svuqadd_s32_m, "q4iq16bq4iq4Ui", "n")
BUILTIN(__builtin_sve_svuqadd_s32_x, "q4iq16bq4iq4Ui", "n")
BUILTIN(__builtin_sve_svuqadd_s32_z, "q4iq16bq4iq4Ui", "n")
BUILTIN(__builtin_sve_svuqadd_s64_m, "q2Wiq16bq2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svuqadd_s64_x, "q2Wiq16bq2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svuqadd_s64_z, "q2Wiq16bq2Wiq2UWi", "n")
BUILTIN(__builtin_sve_svuqadd_s8_m, "q16Scq16bq16Scq16Uc", "n")
BUILTIN(__builtin_sve_svuqadd_s8_x, "q16Scq16bq16Scq16Uc", "n")
BUILTIN(__builtin_sve_svuqadd_s8_z, "q16Scq16bq16Scq16Uc", "n")
BUILTIN(__builtin_sve_svusdot_lane_s32, "q4iq4iq16Ucq16ScIUWi", "n")
BUILTIN(__builtin_sve_svusdot_n_s32, "q4iq4iq16UcSc", "n")
BUILTIN(__builtin_sve_svusdot_s32, "q4iq4iq16Ucq16Sc", "n")
BUILTIN(__builtin_sve_svusmmla_s32, "q4iq4iq16Ucq16Sc", "n")
BUILTIN(__builtin_sve_svuzp1_b16, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svuzp1_b32, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svuzp1_b64, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svuzp1_b8, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svuzp1_bf16, "q8yq8yq8y", "n")
BUILTIN(__builtin_sve_svuzp1_f16, "q8hq8hq8h", "n")
BUILTIN(__builtin_sve_svuzp1_f32, "q4fq4fq4f", "n")
BUILTIN(__builtin_sve_svuzp1_f64, "q2dq2dq2d", "n")
BUILTIN(__builtin_sve_svuzp1_s16, "q8sq8sq8s", "n")
BUILTIN(__builtin_sve_svuzp1_s32, "q4iq4iq4i", "n")
BUILTIN(__builtin_sve_svuzp1_s64, "q2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svuzp1_s8, "q16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svuzp1_u16, "q8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svuzp1_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svuzp1_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svuzp1_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svuzp1q_bf16, "q8yq8yq8y", "n")
BUILTIN(__builtin_sve_svuzp1q_f16, "q8hq8hq8h", "n")
BUILTIN(__builtin_sve_svuzp1q_f32, "q4fq4fq4f", "n")
BUILTIN(__builtin_sve_svuzp1q_f64, "q2dq2dq2d", "n")
BUILTIN(__builtin_sve_svuzp1q_s16, "q8sq8sq8s", "n")
BUILTIN(__builtin_sve_svuzp1q_s32, "q4iq4iq4i", "n")
BUILTIN(__builtin_sve_svuzp1q_s64, "q2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svuzp1q_s8, "q16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svuzp1q_u16, "q8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svuzp1q_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svuzp1q_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svuzp1q_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svuzp2_b16, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svuzp2_b32, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svuzp2_b64, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svuzp2_b8, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svuzp2_bf16, "q8yq8yq8y", "n")
BUILTIN(__builtin_sve_svuzp2_f16, "q8hq8hq8h", "n")
BUILTIN(__builtin_sve_svuzp2_f32, "q4fq4fq4f", "n")
BUILTIN(__builtin_sve_svuzp2_f64, "q2dq2dq2d", "n")
BUILTIN(__builtin_sve_svuzp2_s16, "q8sq8sq8s", "n")
BUILTIN(__builtin_sve_svuzp2_s32, "q4iq4iq4i", "n")
BUILTIN(__builtin_sve_svuzp2_s64, "q2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svuzp2_s8, "q16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svuzp2_u16, "q8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svuzp2_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svuzp2_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svuzp2_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svuzp2q_bf16, "q8yq8yq8y", "n")
BUILTIN(__builtin_sve_svuzp2q_f16, "q8hq8hq8h", "n")
BUILTIN(__builtin_sve_svuzp2q_f32, "q4fq4fq4f", "n")
BUILTIN(__builtin_sve_svuzp2q_f64, "q2dq2dq2d", "n")
BUILTIN(__builtin_sve_svuzp2q_s16, "q8sq8sq8s", "n")
BUILTIN(__builtin_sve_svuzp2q_s32, "q4iq4iq4i", "n")
BUILTIN(__builtin_sve_svuzp2q_s64, "q2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svuzp2q_s8, "q16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svuzp2q_u16, "q8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svuzp2q_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svuzp2q_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svuzp2q_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svwhilege_b16_s32, "q16bii", "n")
BUILTIN(__builtin_sve_svwhilege_b16_s64, "q16bWiWi", "n")
BUILTIN(__builtin_sve_svwhilege_b16_u32, "q16bUiUi", "n")
BUILTIN(__builtin_sve_svwhilege_b16_u64, "q16bUWiUWi", "n")
BUILTIN(__builtin_sve_svwhilege_b32_s32, "q16bii", "n")
BUILTIN(__builtin_sve_svwhilege_b32_s64, "q16bWiWi", "n")
BUILTIN(__builtin_sve_svwhilege_b32_u32, "q16bUiUi", "n")
BUILTIN(__builtin_sve_svwhilege_b32_u64, "q16bUWiUWi", "n")
BUILTIN(__builtin_sve_svwhilege_b64_s32, "q16bii", "n")
BUILTIN(__builtin_sve_svwhilege_b64_s64, "q16bWiWi", "n")
BUILTIN(__builtin_sve_svwhilege_b64_u32, "q16bUiUi", "n")
BUILTIN(__builtin_sve_svwhilege_b64_u64, "q16bUWiUWi", "n")
BUILTIN(__builtin_sve_svwhilege_b8_s32, "q16bii", "n")
BUILTIN(__builtin_sve_svwhilege_b8_s64, "q16bWiWi", "n")
BUILTIN(__builtin_sve_svwhilege_b8_u32, "q16bUiUi", "n")
BUILTIN(__builtin_sve_svwhilege_b8_u64, "q16bUWiUWi", "n")
BUILTIN(__builtin_sve_svwhilegt_b16_s32, "q16bii", "n")
BUILTIN(__builtin_sve_svwhilegt_b16_s64, "q16bWiWi", "n")
BUILTIN(__builtin_sve_svwhilegt_b16_u32, "q16bUiUi", "n")
BUILTIN(__builtin_sve_svwhilegt_b16_u64, "q16bUWiUWi", "n")
BUILTIN(__builtin_sve_svwhilegt_b32_s32, "q16bii", "n")
BUILTIN(__builtin_sve_svwhilegt_b32_s64, "q16bWiWi", "n")
BUILTIN(__builtin_sve_svwhilegt_b32_u32, "q16bUiUi", "n")
BUILTIN(__builtin_sve_svwhilegt_b32_u64, "q16bUWiUWi", "n")
BUILTIN(__builtin_sve_svwhilegt_b64_s32, "q16bii", "n")
BUILTIN(__builtin_sve_svwhilegt_b64_s64, "q16bWiWi", "n")
BUILTIN(__builtin_sve_svwhilegt_b64_u32, "q16bUiUi", "n")
BUILTIN(__builtin_sve_svwhilegt_b64_u64, "q16bUWiUWi", "n")
BUILTIN(__builtin_sve_svwhilegt_b8_s32, "q16bii", "n")
BUILTIN(__builtin_sve_svwhilegt_b8_s64, "q16bWiWi", "n")
BUILTIN(__builtin_sve_svwhilegt_b8_u32, "q16bUiUi", "n")
BUILTIN(__builtin_sve_svwhilegt_b8_u64, "q16bUWiUWi", "n")
BUILTIN(__builtin_sve_svwhilele_b16_s32, "q16bii", "n")
BUILTIN(__builtin_sve_svwhilele_b16_s64, "q16bWiWi", "n")
BUILTIN(__builtin_sve_svwhilele_b16_u32, "q16bUiUi", "n")
BUILTIN(__builtin_sve_svwhilele_b16_u64, "q16bUWiUWi", "n")
BUILTIN(__builtin_sve_svwhilele_b32_s32, "q16bii", "n")
BUILTIN(__builtin_sve_svwhilele_b32_s64, "q16bWiWi", "n")
BUILTIN(__builtin_sve_svwhilele_b32_u32, "q16bUiUi", "n")
BUILTIN(__builtin_sve_svwhilele_b32_u64, "q16bUWiUWi", "n")
BUILTIN(__builtin_sve_svwhilele_b64_s32, "q16bii", "n")
BUILTIN(__builtin_sve_svwhilele_b64_s64, "q16bWiWi", "n")
BUILTIN(__builtin_sve_svwhilele_b64_u32, "q16bUiUi", "n")
BUILTIN(__builtin_sve_svwhilele_b64_u64, "q16bUWiUWi", "n")
BUILTIN(__builtin_sve_svwhilele_b8_s32, "q16bii", "n")
BUILTIN(__builtin_sve_svwhilele_b8_s64, "q16bWiWi", "n")
BUILTIN(__builtin_sve_svwhilele_b8_u32, "q16bUiUi", "n")
BUILTIN(__builtin_sve_svwhilele_b8_u64, "q16bUWiUWi", "n")
BUILTIN(__builtin_sve_svwhilelt_b16_s32, "q16bii", "n")
BUILTIN(__builtin_sve_svwhilelt_b16_s64, "q16bWiWi", "n")
BUILTIN(__builtin_sve_svwhilelt_b16_u32, "q16bUiUi", "n")
BUILTIN(__builtin_sve_svwhilelt_b16_u64, "q16bUWiUWi", "n")
BUILTIN(__builtin_sve_svwhilelt_b32_s32, "q16bii", "n")
BUILTIN(__builtin_sve_svwhilelt_b32_s64, "q16bWiWi", "n")
BUILTIN(__builtin_sve_svwhilelt_b32_u32, "q16bUiUi", "n")
BUILTIN(__builtin_sve_svwhilelt_b32_u64, "q16bUWiUWi", "n")
BUILTIN(__builtin_sve_svwhilelt_b64_s32, "q16bii", "n")
BUILTIN(__builtin_sve_svwhilelt_b64_s64, "q16bWiWi", "n")
BUILTIN(__builtin_sve_svwhilelt_b64_u32, "q16bUiUi", "n")
BUILTIN(__builtin_sve_svwhilelt_b64_u64, "q16bUWiUWi", "n")
BUILTIN(__builtin_sve_svwhilelt_b8_s32, "q16bii", "n")
BUILTIN(__builtin_sve_svwhilelt_b8_s64, "q16bWiWi", "n")
BUILTIN(__builtin_sve_svwhilelt_b8_u32, "q16bUiUi", "n")
BUILTIN(__builtin_sve_svwhilelt_b8_u64, "q16bUWiUWi", "n")
BUILTIN(__builtin_sve_svwhilerw_bf16, "q16byC*yC*", "n")
BUILTIN(__builtin_sve_svwhilerw_f16, "q16bhC*hC*", "n")
BUILTIN(__builtin_sve_svwhilerw_f32, "q16bfC*fC*", "n")
BUILTIN(__builtin_sve_svwhilerw_f64, "q16bdC*dC*", "n")
BUILTIN(__builtin_sve_svwhilerw_s16, "q16bSsC*SsC*", "n")
BUILTIN(__builtin_sve_svwhilerw_s32, "q16bSiC*SiC*", "n")
BUILTIN(__builtin_sve_svwhilerw_s64, "q16bSWiC*SWiC*", "n")
BUILTIN(__builtin_sve_svwhilerw_s8, "q16bScC*ScC*", "n")
BUILTIN(__builtin_sve_svwhilerw_u16, "q16bUsC*UsC*", "n")
BUILTIN(__builtin_sve_svwhilerw_u32, "q16bUiC*UiC*", "n")
BUILTIN(__builtin_sve_svwhilerw_u64, "q16bUWiC*UWiC*", "n")
BUILTIN(__builtin_sve_svwhilerw_u8, "q16bUcC*UcC*", "n")
BUILTIN(__builtin_sve_svwhilewr_bf16, "q16byC*yC*", "n")
BUILTIN(__builtin_sve_svwhilewr_f16, "q16bhC*hC*", "n")
BUILTIN(__builtin_sve_svwhilewr_f32, "q16bfC*fC*", "n")
BUILTIN(__builtin_sve_svwhilewr_f64, "q16bdC*dC*", "n")
BUILTIN(__builtin_sve_svwhilewr_s16, "q16bSsC*SsC*", "n")
BUILTIN(__builtin_sve_svwhilewr_s32, "q16bSiC*SiC*", "n")
BUILTIN(__builtin_sve_svwhilewr_s64, "q16bSWiC*SWiC*", "n")
BUILTIN(__builtin_sve_svwhilewr_s8, "q16bScC*ScC*", "n")
BUILTIN(__builtin_sve_svwhilewr_u16, "q16bUsC*UsC*", "n")
BUILTIN(__builtin_sve_svwhilewr_u32, "q16bUiC*UiC*", "n")
BUILTIN(__builtin_sve_svwhilewr_u64, "q16bUWiC*UWiC*", "n")
BUILTIN(__builtin_sve_svwhilewr_u8, "q16bUcC*UcC*", "n")
BUILTIN(__builtin_sve_svwrffr, "vq16b", "n")
BUILTIN(__builtin_sve_svxar_n_s16, "q8sq8sq8sIUWi", "n")
BUILTIN(__builtin_sve_svxar_n_s32, "q4iq4iq4iIUWi", "n")
BUILTIN(__builtin_sve_svxar_n_s64, "q2Wiq2Wiq2WiIUWi", "n")
BUILTIN(__builtin_sve_svxar_n_s8, "q16Scq16Scq16ScIUWi", "n")
BUILTIN(__builtin_sve_svxar_n_u16, "q8Usq8Usq8UsIUWi", "n")
BUILTIN(__builtin_sve_svxar_n_u32, "q4Uiq4Uiq4UiIUWi", "n")
BUILTIN(__builtin_sve_svxar_n_u64, "q2UWiq2UWiq2UWiIUWi", "n")
BUILTIN(__builtin_sve_svxar_n_u8, "q16Ucq16Ucq16UcIUWi", "n")
BUILTIN(__builtin_sve_svzip1_b16, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svzip1_b32, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svzip1_b64, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svzip1_b8, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svzip1_bf16, "q8yq8yq8y", "n")
BUILTIN(__builtin_sve_svzip1_f16, "q8hq8hq8h", "n")
BUILTIN(__builtin_sve_svzip1_f32, "q4fq4fq4f", "n")
BUILTIN(__builtin_sve_svzip1_f64, "q2dq2dq2d", "n")
BUILTIN(__builtin_sve_svzip1_s16, "q8sq8sq8s", "n")
BUILTIN(__builtin_sve_svzip1_s32, "q4iq4iq4i", "n")
BUILTIN(__builtin_sve_svzip1_s64, "q2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svzip1_s8, "q16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svzip1_u16, "q8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svzip1_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svzip1_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svzip1_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svzip1q_bf16, "q8yq8yq8y", "n")
BUILTIN(__builtin_sve_svzip1q_f16, "q8hq8hq8h", "n")
BUILTIN(__builtin_sve_svzip1q_f32, "q4fq4fq4f", "n")
BUILTIN(__builtin_sve_svzip1q_f64, "q2dq2dq2d", "n")
BUILTIN(__builtin_sve_svzip1q_s16, "q8sq8sq8s", "n")
BUILTIN(__builtin_sve_svzip1q_s32, "q4iq4iq4i", "n")
BUILTIN(__builtin_sve_svzip1q_s64, "q2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svzip1q_s8, "q16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svzip1q_u16, "q8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svzip1q_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svzip1q_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svzip1q_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svzip2_b16, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svzip2_b32, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svzip2_b64, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svzip2_b8, "q16bq16bq16b", "n")
BUILTIN(__builtin_sve_svzip2_bf16, "q8yq8yq8y", "n")
BUILTIN(__builtin_sve_svzip2_f16, "q8hq8hq8h", "n")
BUILTIN(__builtin_sve_svzip2_f32, "q4fq4fq4f", "n")
BUILTIN(__builtin_sve_svzip2_f64, "q2dq2dq2d", "n")
BUILTIN(__builtin_sve_svzip2_s16, "q8sq8sq8s", "n")
BUILTIN(__builtin_sve_svzip2_s32, "q4iq4iq4i", "n")
BUILTIN(__builtin_sve_svzip2_s64, "q2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svzip2_s8, "q16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svzip2_u16, "q8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svzip2_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svzip2_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svzip2_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_svzip2q_bf16, "q8yq8yq8y", "n")
BUILTIN(__builtin_sve_svzip2q_f16, "q8hq8hq8h", "n")
BUILTIN(__builtin_sve_svzip2q_f32, "q4fq4fq4f", "n")
BUILTIN(__builtin_sve_svzip2q_f64, "q2dq2dq2d", "n")
BUILTIN(__builtin_sve_svzip2q_s16, "q8sq8sq8s", "n")
BUILTIN(__builtin_sve_svzip2q_s32, "q4iq4iq4i", "n")
BUILTIN(__builtin_sve_svzip2q_s64, "q2Wiq2Wiq2Wi", "n")
BUILTIN(__builtin_sve_svzip2q_s8, "q16Scq16Scq16Sc", "n")
BUILTIN(__builtin_sve_svzip2q_u16, "q8Usq8Usq8Us", "n")
BUILTIN(__builtin_sve_svzip2q_u32, "q4Uiq4Uiq4Ui", "n")
BUILTIN(__builtin_sve_svzip2q_u64, "q2UWiq2UWiq2UWi", "n")
BUILTIN(__builtin_sve_svzip2q_u8, "q16Ucq16Ucq16Uc", "n")
BUILTIN(__builtin_sve_reinterpret_s8_s8, "q16Scq16Sc", "n")
BUILTIN(__builtin_sve_reinterpret_s8_s16, "q16Scq8Ss", "n")
BUILTIN(__builtin_sve_reinterpret_s8_s32, "q16Scq4Si", "n")
BUILTIN(__builtin_sve_reinterpret_s8_s64, "q16Scq2SWi", "n")
BUILTIN(__builtin_sve_reinterpret_s8_u8, "q16Scq16Uc", "n")
BUILTIN(__builtin_sve_reinterpret_s8_u16, "q16Scq8Us", "n")
BUILTIN(__builtin_sve_reinterpret_s8_u32, "q16Scq4Ui", "n")
BUILTIN(__builtin_sve_reinterpret_s8_u64, "q16Scq2UWi", "n")
BUILTIN(__builtin_sve_reinterpret_s8_f16, "q16Scq8h", "n")
BUILTIN(__builtin_sve_reinterpret_s8_bf16, "q16Scq8y", "n")
BUILTIN(__builtin_sve_reinterpret_s8_f32, "q16Scq4f", "n")
BUILTIN(__builtin_sve_reinterpret_s8_f64, "q16Scq2d", "n")
BUILTIN(__builtin_sve_reinterpret_s16_s8, "q8Ssq16Sc", "n")
BUILTIN(__builtin_sve_reinterpret_s16_s16, "q8Ssq8Ss", "n")
BUILTIN(__builtin_sve_reinterpret_s16_s32, "q8Ssq4Si", "n")
BUILTIN(__builtin_sve_reinterpret_s16_s64, "q8Ssq2SWi", "n")
BUILTIN(__builtin_sve_reinterpret_s16_u8, "q8Ssq16Uc", "n")
BUILTIN(__builtin_sve_reinterpret_s16_u16, "q8Ssq8Us", "n")
BUILTIN(__builtin_sve_reinterpret_s16_u32, "q8Ssq4Ui", "n")
BUILTIN(__builtin_sve_reinterpret_s16_u64, "q8Ssq2UWi", "n")
BUILTIN(__builtin_sve_reinterpret_s16_f16, "q8Ssq8h", "n")
BUILTIN(__builtin_sve_reinterpret_s16_bf16, "q8Ssq8y", "n")
BUILTIN(__builtin_sve_reinterpret_s16_f32, "q8Ssq4f", "n")
BUILTIN(__builtin_sve_reinterpret_s16_f64, "q8Ssq2d", "n")
BUILTIN(__builtin_sve_reinterpret_s32_s8, "q4Siq16Sc", "n")
BUILTIN(__builtin_sve_reinterpret_s32_s16, "q4Siq8Ss", "n")
BUILTIN(__builtin_sve_reinterpret_s32_s32, "q4Siq4Si", "n")
BUILTIN(__builtin_sve_reinterpret_s32_s64, "q4Siq2SWi", "n")
BUILTIN(__builtin_sve_reinterpret_s32_u8, "q4Siq16Uc", "n")
BUILTIN(__builtin_sve_reinterpret_s32_u16, "q4Siq8Us", "n")
BUILTIN(__builtin_sve_reinterpret_s32_u32, "q4Siq4Ui", "n")
BUILTIN(__builtin_sve_reinterpret_s32_u64, "q4Siq2UWi", "n")
BUILTIN(__builtin_sve_reinterpret_s32_f16, "q4Siq8h", "n")
BUILTIN(__builtin_sve_reinterpret_s32_bf16, "q4Siq8y", "n")
BUILTIN(__builtin_sve_reinterpret_s32_f32, "q4Siq4f", "n")
BUILTIN(__builtin_sve_reinterpret_s32_f64, "q4Siq2d", "n")
BUILTIN(__builtin_sve_reinterpret_s64_s8, "q2SWiq16Sc", "n")
BUILTIN(__builtin_sve_reinterpret_s64_s16, "q2SWiq8Ss", "n")
BUILTIN(__builtin_sve_reinterpret_s64_s32, "q2SWiq4Si", "n")
BUILTIN(__builtin_sve_reinterpret_s64_s64, "q2SWiq2SWi", "n")
BUILTIN(__builtin_sve_reinterpret_s64_u8, "q2SWiq16Uc", "n")
BUILTIN(__builtin_sve_reinterpret_s64_u16, "q2SWiq8Us", "n")
BUILTIN(__builtin_sve_reinterpret_s64_u32, "q2SWiq4Ui", "n")
BUILTIN(__builtin_sve_reinterpret_s64_u64, "q2SWiq2UWi", "n")
BUILTIN(__builtin_sve_reinterpret_s64_f16, "q2SWiq8h", "n")
BUILTIN(__builtin_sve_reinterpret_s64_bf16, "q2SWiq8y", "n")
BUILTIN(__builtin_sve_reinterpret_s64_f32, "q2SWiq4f", "n")
BUILTIN(__builtin_sve_reinterpret_s64_f64, "q2SWiq2d", "n")
BUILTIN(__builtin_sve_reinterpret_u8_s8, "q16Ucq16Sc", "n")
BUILTIN(__builtin_sve_reinterpret_u8_s16, "q16Ucq8Ss", "n")
BUILTIN(__builtin_sve_reinterpret_u8_s32, "q16Ucq4Si", "n")
BUILTIN(__builtin_sve_reinterpret_u8_s64, "q16Ucq2SWi", "n")
BUILTIN(__builtin_sve_reinterpret_u8_u8, "q16Ucq16Uc", "n")
BUILTIN(__builtin_sve_reinterpret_u8_u16, "q16Ucq8Us", "n")
BUILTIN(__builtin_sve_reinterpret_u8_u32, "q16Ucq4Ui", "n")
BUILTIN(__builtin_sve_reinterpret_u8_u64, "q16Ucq2UWi", "n")
BUILTIN(__builtin_sve_reinterpret_u8_f16, "q16Ucq8h", "n")
BUILTIN(__builtin_sve_reinterpret_u8_bf16, "q16Ucq8y", "n")
BUILTIN(__builtin_sve_reinterpret_u8_f32, "q16Ucq4f", "n")
BUILTIN(__builtin_sve_reinterpret_u8_f64, "q16Ucq2d", "n")
BUILTIN(__builtin_sve_reinterpret_u16_s8, "q8Usq16Sc", "n")
BUILTIN(__builtin_sve_reinterpret_u16_s16, "q8Usq8Ss", "n")
BUILTIN(__builtin_sve_reinterpret_u16_s32, "q8Usq4Si", "n")
BUILTIN(__builtin_sve_reinterpret_u16_s64, "q8Usq2SWi", "n")
BUILTIN(__builtin_sve_reinterpret_u16_u8, "q8Usq16Uc", "n")
BUILTIN(__builtin_sve_reinterpret_u16_u16, "q8Usq8Us", "n")
BUILTIN(__builtin_sve_reinterpret_u16_u32, "q8Usq4Ui", "n")
BUILTIN(__builtin_sve_reinterpret_u16_u64, "q8Usq2UWi", "n")
BUILTIN(__builtin_sve_reinterpret_u16_f16, "q8Usq8h", "n")
BUILTIN(__builtin_sve_reinterpret_u16_bf16, "q8Usq8y", "n")
BUILTIN(__builtin_sve_reinterpret_u16_f32, "q8Usq4f", "n")
BUILTIN(__builtin_sve_reinterpret_u16_f64, "q8Usq2d", "n")
BUILTIN(__builtin_sve_reinterpret_u32_s8, "q4Uiq16Sc", "n")
BUILTIN(__builtin_sve_reinterpret_u32_s16, "q4Uiq8Ss", "n")
BUILTIN(__builtin_sve_reinterpret_u32_s32, "q4Uiq4Si", "n")
BUILTIN(__builtin_sve_reinterpret_u32_s64, "q4Uiq2SWi", "n")
BUILTIN(__builtin_sve_reinterpret_u32_u8, "q4Uiq16Uc", "n")
BUILTIN(__builtin_sve_reinterpret_u32_u16, "q4Uiq8Us", "n")
BUILTIN(__builtin_sve_reinterpret_u32_u32, "q4Uiq4Ui", "n")
BUILTIN(__builtin_sve_reinterpret_u32_u64, "q4Uiq2UWi", "n")
BUILTIN(__builtin_sve_reinterpret_u32_f16, "q4Uiq8h", "n")
BUILTIN(__builtin_sve_reinterpret_u32_bf16, "q4Uiq8y", "n")
BUILTIN(__builtin_sve_reinterpret_u32_f32, "q4Uiq4f", "n")
BUILTIN(__builtin_sve_reinterpret_u32_f64, "q4Uiq2d", "n")
BUILTIN(__builtin_sve_reinterpret_u64_s8, "q2UWiq16Sc", "n")
BUILTIN(__builtin_sve_reinterpret_u64_s16, "q2UWiq8Ss", "n")
BUILTIN(__builtin_sve_reinterpret_u64_s32, "q2UWiq4Si", "n")
BUILTIN(__builtin_sve_reinterpret_u64_s64, "q2UWiq2SWi", "n")
BUILTIN(__builtin_sve_reinterpret_u64_u8, "q2UWiq16Uc", "n")
BUILTIN(__builtin_sve_reinterpret_u64_u16, "q2UWiq8Us", "n")
BUILTIN(__builtin_sve_reinterpret_u64_u32, "q2UWiq4Ui", "n")
BUILTIN(__builtin_sve_reinterpret_u64_u64, "q2UWiq2UWi", "n")
BUILTIN(__builtin_sve_reinterpret_u64_f16, "q2UWiq8h", "n")
BUILTIN(__builtin_sve_reinterpret_u64_bf16, "q2UWiq8y", "n")
BUILTIN(__builtin_sve_reinterpret_u64_f32, "q2UWiq4f", "n")
BUILTIN(__builtin_sve_reinterpret_u64_f64, "q2UWiq2d", "n")
BUILTIN(__builtin_sve_reinterpret_f16_s8, "q8hq16Sc", "n")
BUILTIN(__builtin_sve_reinterpret_f16_s16, "q8hq8Ss", "n")
BUILTIN(__builtin_sve_reinterpret_f16_s32, "q8hq4Si", "n")
BUILTIN(__builtin_sve_reinterpret_f16_s64, "q8hq2SWi", "n")
BUILTIN(__builtin_sve_reinterpret_f16_u8, "q8hq16Uc", "n")
BUILTIN(__builtin_sve_reinterpret_f16_u16, "q8hq8Us", "n")
BUILTIN(__builtin_sve_reinterpret_f16_u32, "q8hq4Ui", "n")
BUILTIN(__builtin_sve_reinterpret_f16_u64, "q8hq2UWi", "n")
BUILTIN(__builtin_sve_reinterpret_f16_f16, "q8hq8h", "n")
BUILTIN(__builtin_sve_reinterpret_f16_bf16, "q8hq8y", "n")
BUILTIN(__builtin_sve_reinterpret_f16_f32, "q8hq4f", "n")
BUILTIN(__builtin_sve_reinterpret_f16_f64, "q8hq2d", "n")
BUILTIN(__builtin_sve_reinterpret_bf16_s8, "q8yq16Sc", "n")
BUILTIN(__builtin_sve_reinterpret_bf16_s16, "q8yq8Ss", "n")
BUILTIN(__builtin_sve_reinterpret_bf16_s32, "q8yq4Si", "n")
BUILTIN(__builtin_sve_reinterpret_bf16_s64, "q8yq2SWi", "n")
BUILTIN(__builtin_sve_reinterpret_bf16_u8, "q8yq16Uc", "n")
BUILTIN(__builtin_sve_reinterpret_bf16_u16, "q8yq8Us", "n")
BUILTIN(__builtin_sve_reinterpret_bf16_u32, "q8yq4Ui", "n")
BUILTIN(__builtin_sve_reinterpret_bf16_u64, "q8yq2UWi", "n")
BUILTIN(__builtin_sve_reinterpret_bf16_f16, "q8yq8h", "n")
BUILTIN(__builtin_sve_reinterpret_bf16_bf16, "q8yq8y", "n")
BUILTIN(__builtin_sve_reinterpret_bf16_f32, "q8yq4f", "n")
BUILTIN(__builtin_sve_reinterpret_bf16_f64, "q8yq2d", "n")
BUILTIN(__builtin_sve_reinterpret_f32_s8, "q4fq16Sc", "n")
BUILTIN(__builtin_sve_reinterpret_f32_s16, "q4fq8Ss", "n")
BUILTIN(__builtin_sve_reinterpret_f32_s32, "q4fq4Si", "n")
BUILTIN(__builtin_sve_reinterpret_f32_s64, "q4fq2SWi", "n")
BUILTIN(__builtin_sve_reinterpret_f32_u8, "q4fq16Uc", "n")
BUILTIN(__builtin_sve_reinterpret_f32_u16, "q4fq8Us", "n")
BUILTIN(__builtin_sve_reinterpret_f32_u32, "q4fq4Ui", "n")
BUILTIN(__builtin_sve_reinterpret_f32_u64, "q4fq2UWi", "n")
BUILTIN(__builtin_sve_reinterpret_f32_f16, "q4fq8h", "n")
BUILTIN(__builtin_sve_reinterpret_f32_bf16, "q4fq8y", "n")
BUILTIN(__builtin_sve_reinterpret_f32_f32, "q4fq4f", "n")
BUILTIN(__builtin_sve_reinterpret_f32_f64, "q4fq2d", "n")
BUILTIN(__builtin_sve_reinterpret_f64_s8, "q2dq16Sc", "n")
BUILTIN(__builtin_sve_reinterpret_f64_s16, "q2dq8Ss", "n")
BUILTIN(__builtin_sve_reinterpret_f64_s32, "q2dq4Si", "n")
BUILTIN(__builtin_sve_reinterpret_f64_s64, "q2dq2SWi", "n")
BUILTIN(__builtin_sve_reinterpret_f64_u8, "q2dq16Uc", "n")
BUILTIN(__builtin_sve_reinterpret_f64_u16, "q2dq8Us", "n")
BUILTIN(__builtin_sve_reinterpret_f64_u32, "q2dq4Ui", "n")
BUILTIN(__builtin_sve_reinterpret_f64_u64, "q2dq2UWi", "n")
BUILTIN(__builtin_sve_reinterpret_f64_f16, "q2dq8h", "n")
BUILTIN(__builtin_sve_reinterpret_f64_bf16, "q2dq8y", "n")
BUILTIN(__builtin_sve_reinterpret_f64_f32, "q2dq4f", "n")
BUILTIN(__builtin_sve_reinterpret_f64_f64, "q2dq2d", "n")
#endif

