#ifdef GET_SVE_LLVM_INTRINSIC_MAP
SVEMAP1(svaba_n_s16, aarch64_sve_saba, 3074),
SVEMAP1(svaba_n_s32, aarch64_sve_saba, 3075),
SVEMAP1(svaba_n_s64, aarch64_sve_saba, 3076),
SVEMAP1(svaba_n_s8, aarch64_sve_saba, 3073),
SVEMAP1(svaba_n_u16, aarch64_sve_uaba, 3074),
SVEMAP1(svaba_n_u32, aarch64_sve_uaba, 3075),
SVEMAP1(svaba_n_u64, aarch64_sve_uaba, 3076),
SVEMAP1(svaba_n_u8, aarch64_sve_uaba, 3073),
SVEMAP1(svaba_s16, aarch64_sve_saba, 2),
SVEMAP1(svaba_s32, aarch64_sve_saba, 3),
SVEMAP1(svaba_s64, aarch64_sve_saba, 4),
SVEMAP1(svaba_s8, aarch64_sve_saba, 1),
SVEMAP1(svaba_u16, aarch64_sve_uaba, 2),
SVEMAP1(svaba_u32, aarch64_sve_uaba, 3),
SVEMAP1(svaba_u64, aarch64_sve_uaba, 4),
SVEMAP1(svaba_u8, aarch64_sve_uaba, 1),
SVEMAP1(svabalb_n_s16, aarch64_sve_sabalb, 3074),
SVEMAP1(svabalb_n_s32, aarch64_sve_sabalb, 3075),
SVEMAP1(svabalb_n_s64, aarch64_sve_sabalb, 3076),
SVEMAP1(svabalb_n_u16, aarch64_sve_uabalb, 3074),
SVEMAP1(svabalb_n_u32, aarch64_sve_uabalb, 3075),
SVEMAP1(svabalb_n_u64, aarch64_sve_uabalb, 3076),
SVEMAP1(svabalb_s16, aarch64_sve_sabalb, 2),
SVEMAP1(svabalb_s32, aarch64_sve_sabalb, 3),
SVEMAP1(svabalb_s64, aarch64_sve_sabalb, 4),
SVEMAP1(svabalb_u16, aarch64_sve_uabalb, 2),
SVEMAP1(svabalb_u32, aarch64_sve_uabalb, 3),
SVEMAP1(svabalb_u64, aarch64_sve_uabalb, 4),
SVEMAP1(svabalt_n_s16, aarch64_sve_sabalt, 3074),
SVEMAP1(svabalt_n_s32, aarch64_sve_sabalt, 3075),
SVEMAP1(svabalt_n_s64, aarch64_sve_sabalt, 3076),
SVEMAP1(svabalt_n_u16, aarch64_sve_uabalt, 3074),
SVEMAP1(svabalt_n_u32, aarch64_sve_uabalt, 3075),
SVEMAP1(svabalt_n_u64, aarch64_sve_uabalt, 3076),
SVEMAP1(svabalt_s16, aarch64_sve_sabalt, 2),
SVEMAP1(svabalt_s32, aarch64_sve_sabalt, 3),
SVEMAP1(svabalt_s64, aarch64_sve_sabalt, 4),
SVEMAP1(svabalt_u16, aarch64_sve_uabalt, 2),
SVEMAP1(svabalt_u32, aarch64_sve_uabalt, 3),
SVEMAP1(svabalt_u64, aarch64_sve_uabalt, 4),
SVEMAP1(svabd_f16_m, aarch64_sve_fabd, 261),
SVEMAP1(svabd_f16_x, aarch64_sve_fabd, 133),
SVEMAP1(svabd_f16_z, aarch64_sve_fabd, 389),
SVEMAP1(svabd_f32_m, aarch64_sve_fabd, 262),
SVEMAP1(svabd_f32_x, aarch64_sve_fabd, 134),
SVEMAP1(svabd_f32_z, aarch64_sve_fabd, 390),
SVEMAP1(svabd_f64_m, aarch64_sve_fabd, 263),
SVEMAP1(svabd_f64_x, aarch64_sve_fabd, 135),
SVEMAP1(svabd_f64_z, aarch64_sve_fabd, 391),
SVEMAP1(svabd_n_f16_m, aarch64_sve_fabd, 3333),
SVEMAP1(svabd_n_f16_x, aarch64_sve_fabd, 3205),
SVEMAP1(svabd_n_f16_z, aarch64_sve_fabd, 3461),
SVEMAP1(svabd_n_f32_m, aarch64_sve_fabd, 3334),
SVEMAP1(svabd_n_f32_x, aarch64_sve_fabd, 3206),
SVEMAP1(svabd_n_f32_z, aarch64_sve_fabd, 3462),
SVEMAP1(svabd_n_f64_m, aarch64_sve_fabd, 3335),
SVEMAP1(svabd_n_f64_x, aarch64_sve_fabd, 3207),
SVEMAP1(svabd_n_f64_z, aarch64_sve_fabd, 3463),
SVEMAP1(svabd_n_s16_m, aarch64_sve_sabd, 3330),
SVEMAP1(svabd_n_s16_x, aarch64_sve_sabd, 3202),
SVEMAP1(svabd_n_s16_z, aarch64_sve_sabd, 3458),
SVEMAP1(svabd_n_s32_m, aarch64_sve_sabd, 3331),
SVEMAP1(svabd_n_s32_x, aarch64_sve_sabd, 3203),
SVEMAP1(svabd_n_s32_z, aarch64_sve_sabd, 3459),
SVEMAP1(svabd_n_s64_m, aarch64_sve_sabd, 3332),
SVEMAP1(svabd_n_s64_x, aarch64_sve_sabd, 3204),
SVEMAP1(svabd_n_s64_z, aarch64_sve_sabd, 3460),
SVEMAP1(svabd_n_s8_m, aarch64_sve_sabd, 3329),
SVEMAP1(svabd_n_s8_x, aarch64_sve_sabd, 3201),
SVEMAP1(svabd_n_s8_z, aarch64_sve_sabd, 3457),
SVEMAP1(svabd_n_u16_m, aarch64_sve_uabd, 3330),
SVEMAP1(svabd_n_u16_x, aarch64_sve_uabd, 3202),
SVEMAP1(svabd_n_u16_z, aarch64_sve_uabd, 3458),
SVEMAP1(svabd_n_u32_m, aarch64_sve_uabd, 3331),
SVEMAP1(svabd_n_u32_x, aarch64_sve_uabd, 3203),
SVEMAP1(svabd_n_u32_z, aarch64_sve_uabd, 3459),
SVEMAP1(svabd_n_u64_m, aarch64_sve_uabd, 3332),
SVEMAP1(svabd_n_u64_x, aarch64_sve_uabd, 3204),
SVEMAP1(svabd_n_u64_z, aarch64_sve_uabd, 3460),
SVEMAP1(svabd_n_u8_m, aarch64_sve_uabd, 3329),
SVEMAP1(svabd_n_u8_x, aarch64_sve_uabd, 3201),
SVEMAP1(svabd_n_u8_z, aarch64_sve_uabd, 3457),
SVEMAP1(svabd_s16_m, aarch64_sve_sabd, 258),
SVEMAP1(svabd_s16_x, aarch64_sve_sabd, 130),
SVEMAP1(svabd_s16_z, aarch64_sve_sabd, 386),
SVEMAP1(svabd_s32_m, aarch64_sve_sabd, 259),
SVEMAP1(svabd_s32_x, aarch64_sve_sabd, 131),
SVEMAP1(svabd_s32_z, aarch64_sve_sabd, 387),
SVEMAP1(svabd_s64_m, aarch64_sve_sabd, 260),
SVEMAP1(svabd_s64_x, aarch64_sve_sabd, 132),
SVEMAP1(svabd_s64_z, aarch64_sve_sabd, 388),
SVEMAP1(svabd_s8_m, aarch64_sve_sabd, 257),
SVEMAP1(svabd_s8_x, aarch64_sve_sabd, 129),
SVEMAP1(svabd_s8_z, aarch64_sve_sabd, 385),
SVEMAP1(svabd_u16_m, aarch64_sve_uabd, 258),
SVEMAP1(svabd_u16_x, aarch64_sve_uabd, 130),
SVEMAP1(svabd_u16_z, aarch64_sve_uabd, 386),
SVEMAP1(svabd_u32_m, aarch64_sve_uabd, 259),
SVEMAP1(svabd_u32_x, aarch64_sve_uabd, 131),
SVEMAP1(svabd_u32_z, aarch64_sve_uabd, 387),
SVEMAP1(svabd_u64_m, aarch64_sve_uabd, 260),
SVEMAP1(svabd_u64_x, aarch64_sve_uabd, 132),
SVEMAP1(svabd_u64_z, aarch64_sve_uabd, 388),
SVEMAP1(svabd_u8_m, aarch64_sve_uabd, 257),
SVEMAP1(svabd_u8_x, aarch64_sve_uabd, 129),
SVEMAP1(svabd_u8_z, aarch64_sve_uabd, 385),
SVEMAP1(svabdlb_n_s16, aarch64_sve_sabdlb, 2050),
SVEMAP1(svabdlb_n_s32, aarch64_sve_sabdlb, 2051),
SVEMAP1(svabdlb_n_s64, aarch64_sve_sabdlb, 2052),
SVEMAP1(svabdlb_n_u16, aarch64_sve_uabdlb, 2050),
SVEMAP1(svabdlb_n_u32, aarch64_sve_uabdlb, 2051),
SVEMAP1(svabdlb_n_u64, aarch64_sve_uabdlb, 2052),
SVEMAP1(svabdlb_s16, aarch64_sve_sabdlb, 2),
SVEMAP1(svabdlb_s32, aarch64_sve_sabdlb, 3),
SVEMAP1(svabdlb_s64, aarch64_sve_sabdlb, 4),
SVEMAP1(svabdlb_u16, aarch64_sve_uabdlb, 2),
SVEMAP1(svabdlb_u32, aarch64_sve_uabdlb, 3),
SVEMAP1(svabdlb_u64, aarch64_sve_uabdlb, 4),
SVEMAP1(svabdlt_n_s16, aarch64_sve_sabdlt, 2050),
SVEMAP1(svabdlt_n_s32, aarch64_sve_sabdlt, 2051),
SVEMAP1(svabdlt_n_s64, aarch64_sve_sabdlt, 2052),
SVEMAP1(svabdlt_n_u16, aarch64_sve_uabdlt, 2050),
SVEMAP1(svabdlt_n_u32, aarch64_sve_uabdlt, 2051),
SVEMAP1(svabdlt_n_u64, aarch64_sve_uabdlt, 2052),
SVEMAP1(svabdlt_s16, aarch64_sve_sabdlt, 2),
SVEMAP1(svabdlt_s32, aarch64_sve_sabdlt, 3),
SVEMAP1(svabdlt_s64, aarch64_sve_sabdlt, 4),
SVEMAP1(svabdlt_u16, aarch64_sve_uabdlt, 2),
SVEMAP1(svabdlt_u32, aarch64_sve_uabdlt, 3),
SVEMAP1(svabdlt_u64, aarch64_sve_uabdlt, 4),
SVEMAP1(svabs_f16_m, aarch64_sve_fabs, 261),
SVEMAP1(svabs_f16_x, aarch64_sve_fabs, 517),
SVEMAP1(svabs_f16_z, aarch64_sve_fabs, 645),
SVEMAP1(svabs_f32_m, aarch64_sve_fabs, 262),
SVEMAP1(svabs_f32_x, aarch64_sve_fabs, 518),
SVEMAP1(svabs_f32_z, aarch64_sve_fabs, 646),
SVEMAP1(svabs_f64_m, aarch64_sve_fabs, 263),
SVEMAP1(svabs_f64_x, aarch64_sve_fabs, 519),
SVEMAP1(svabs_f64_z, aarch64_sve_fabs, 647),
SVEMAP1(svabs_s16_m, aarch64_sve_abs, 258),
SVEMAP1(svabs_s16_x, aarch64_sve_abs, 514),
SVEMAP1(svabs_s16_z, aarch64_sve_abs, 642),
SVEMAP1(svabs_s32_m, aarch64_sve_abs, 259),
SVEMAP1(svabs_s32_x, aarch64_sve_abs, 515),
SVEMAP1(svabs_s32_z, aarch64_sve_abs, 643),
SVEMAP1(svabs_s64_m, aarch64_sve_abs, 260),
SVEMAP1(svabs_s64_x, aarch64_sve_abs, 516),
SVEMAP1(svabs_s64_z, aarch64_sve_abs, 644),
SVEMAP1(svabs_s8_m, aarch64_sve_abs, 257),
SVEMAP1(svabs_s8_x, aarch64_sve_abs, 513),
SVEMAP1(svabs_s8_z, aarch64_sve_abs, 641),
SVEMAP1(svacge_f16, aarch64_sve_facge, 5),
SVEMAP1(svacge_f32, aarch64_sve_facge, 6),
SVEMAP1(svacge_f64, aarch64_sve_facge, 7),
SVEMAP1(svacge_n_f16, aarch64_sve_facge, 3077),
SVEMAP1(svacge_n_f32, aarch64_sve_facge, 3078),
SVEMAP1(svacge_n_f64, aarch64_sve_facge, 3079),
SVEMAP1(svacgt_f16, aarch64_sve_facgt, 5),
SVEMAP1(svacgt_f32, aarch64_sve_facgt, 6),
SVEMAP1(svacgt_f64, aarch64_sve_facgt, 7),
SVEMAP1(svacgt_n_f16, aarch64_sve_facgt, 3077),
SVEMAP1(svacgt_n_f32, aarch64_sve_facgt, 3078),
SVEMAP1(svacgt_n_f64, aarch64_sve_facgt, 3079),
SVEMAP1(svacle_f16, aarch64_sve_facge, 536870917),
SVEMAP1(svacle_f32, aarch64_sve_facge, 536870918),
SVEMAP1(svacle_f64, aarch64_sve_facge, 536870919),
SVEMAP1(svacle_n_f16, aarch64_sve_facge, 536873989),
SVEMAP1(svacle_n_f32, aarch64_sve_facge, 536873990),
SVEMAP1(svacle_n_f64, aarch64_sve_facge, 536873991),
SVEMAP1(svaclt_f16, aarch64_sve_facgt, 536870917),
SVEMAP1(svaclt_f32, aarch64_sve_facgt, 536870918),
SVEMAP1(svaclt_f64, aarch64_sve_facgt, 536870919),
SVEMAP1(svaclt_n_f16, aarch64_sve_facgt, 536873989),
SVEMAP1(svaclt_n_f32, aarch64_sve_facgt, 536873990),
SVEMAP1(svaclt_n_f64, aarch64_sve_facgt, 536873991),
SVEMAP1(svadalp_s16_m, aarch64_sve_sadalp, 258),
SVEMAP1(svadalp_s16_x, aarch64_sve_sadalp, 130),
SVEMAP1(svadalp_s16_z, aarch64_sve_sadalp, 386),
SVEMAP1(svadalp_s32_m, aarch64_sve_sadalp, 259),
SVEMAP1(svadalp_s32_x, aarch64_sve_sadalp, 131),
SVEMAP1(svadalp_s32_z, aarch64_sve_sadalp, 387),
SVEMAP1(svadalp_s64_m, aarch64_sve_sadalp, 260),
SVEMAP1(svadalp_s64_x, aarch64_sve_sadalp, 132),
SVEMAP1(svadalp_s64_z, aarch64_sve_sadalp, 388),
SVEMAP1(svadalp_u16_m, aarch64_sve_uadalp, 258),
SVEMAP1(svadalp_u16_x, aarch64_sve_uadalp, 130),
SVEMAP1(svadalp_u16_z, aarch64_sve_uadalp, 386),
SVEMAP1(svadalp_u32_m, aarch64_sve_uadalp, 259),
SVEMAP1(svadalp_u32_x, aarch64_sve_uadalp, 131),
SVEMAP1(svadalp_u32_z, aarch64_sve_uadalp, 387),
SVEMAP1(svadalp_u64_m, aarch64_sve_uadalp, 260),
SVEMAP1(svadalp_u64_x, aarch64_sve_uadalp, 132),
SVEMAP1(svadalp_u64_z, aarch64_sve_uadalp, 388),
SVEMAP1(svadclb_n_u32, aarch64_sve_adclb, 3075),
SVEMAP1(svadclb_n_u64, aarch64_sve_adclb, 3076),
SVEMAP1(svadclb_u32, aarch64_sve_adclb, 3),
SVEMAP1(svadclb_u64, aarch64_sve_adclb, 4),
SVEMAP1(svadclt_n_u32, aarch64_sve_adclt, 3075),
SVEMAP1(svadclt_n_u64, aarch64_sve_adclt, 3076),
SVEMAP1(svadclt_u32, aarch64_sve_adclt, 3),
SVEMAP1(svadclt_u64, aarch64_sve_adclt, 4),
SVEMAP1(svadd_f16_m, aarch64_sve_fadd, 261),
SVEMAP1(svadd_f16_x, aarch64_sve_fadd, 133),
SVEMAP1(svadd_f16_z, aarch64_sve_fadd, 389),
SVEMAP1(svadd_f32_m, aarch64_sve_fadd, 262),
SVEMAP1(svadd_f32_x, aarch64_sve_fadd, 134),
SVEMAP1(svadd_f32_z, aarch64_sve_fadd, 390),
SVEMAP1(svadd_f64_m, aarch64_sve_fadd, 263),
SVEMAP1(svadd_f64_x, aarch64_sve_fadd, 135),
SVEMAP1(svadd_f64_z, aarch64_sve_fadd, 391),
SVEMAP1(svadd_n_f16_m, aarch64_sve_fadd, 3333),
SVEMAP1(svadd_n_f16_x, aarch64_sve_fadd, 3205),
SVEMAP1(svadd_n_f16_z, aarch64_sve_fadd, 3461),
SVEMAP1(svadd_n_f32_m, aarch64_sve_fadd, 3334),
SVEMAP1(svadd_n_f32_x, aarch64_sve_fadd, 3206),
SVEMAP1(svadd_n_f32_z, aarch64_sve_fadd, 3462),
SVEMAP1(svadd_n_f64_m, aarch64_sve_fadd, 3335),
SVEMAP1(svadd_n_f64_x, aarch64_sve_fadd, 3207),
SVEMAP1(svadd_n_f64_z, aarch64_sve_fadd, 3463),
SVEMAP1(svadd_n_s16_m, aarch64_sve_add, 3330),
SVEMAP1(svadd_n_s16_x, aarch64_sve_add, 3202),
SVEMAP1(svadd_n_s16_z, aarch64_sve_add, 3458),
SVEMAP1(svadd_n_s32_m, aarch64_sve_add, 3331),
SVEMAP1(svadd_n_s32_x, aarch64_sve_add, 3203),
SVEMAP1(svadd_n_s32_z, aarch64_sve_add, 3459),
SVEMAP1(svadd_n_s64_m, aarch64_sve_add, 3332),
SVEMAP1(svadd_n_s64_x, aarch64_sve_add, 3204),
SVEMAP1(svadd_n_s64_z, aarch64_sve_add, 3460),
SVEMAP1(svadd_n_s8_m, aarch64_sve_add, 3329),
SVEMAP1(svadd_n_s8_x, aarch64_sve_add, 3201),
SVEMAP1(svadd_n_s8_z, aarch64_sve_add, 3457),
SVEMAP1(svadd_n_u16_m, aarch64_sve_add, 3330),
SVEMAP1(svadd_n_u16_x, aarch64_sve_add, 3202),
SVEMAP1(svadd_n_u16_z, aarch64_sve_add, 3458),
SVEMAP1(svadd_n_u32_m, aarch64_sve_add, 3331),
SVEMAP1(svadd_n_u32_x, aarch64_sve_add, 3203),
SVEMAP1(svadd_n_u32_z, aarch64_sve_add, 3459),
SVEMAP1(svadd_n_u64_m, aarch64_sve_add, 3332),
SVEMAP1(svadd_n_u64_x, aarch64_sve_add, 3204),
SVEMAP1(svadd_n_u64_z, aarch64_sve_add, 3460),
SVEMAP1(svadd_n_u8_m, aarch64_sve_add, 3329),
SVEMAP1(svadd_n_u8_x, aarch64_sve_add, 3201),
SVEMAP1(svadd_n_u8_z, aarch64_sve_add, 3457),
SVEMAP1(svadd_s16_m, aarch64_sve_add, 258),
SVEMAP1(svadd_s16_x, aarch64_sve_add, 130),
SVEMAP1(svadd_s16_z, aarch64_sve_add, 386),
SVEMAP1(svadd_s32_m, aarch64_sve_add, 259),
SVEMAP1(svadd_s32_x, aarch64_sve_add, 131),
SVEMAP1(svadd_s32_z, aarch64_sve_add, 387),
SVEMAP1(svadd_s64_m, aarch64_sve_add, 260),
SVEMAP1(svadd_s64_x, aarch64_sve_add, 132),
SVEMAP1(svadd_s64_z, aarch64_sve_add, 388),
SVEMAP1(svadd_s8_m, aarch64_sve_add, 257),
SVEMAP1(svadd_s8_x, aarch64_sve_add, 129),
SVEMAP1(svadd_s8_z, aarch64_sve_add, 385),
SVEMAP1(svadd_u16_m, aarch64_sve_add, 258),
SVEMAP1(svadd_u16_x, aarch64_sve_add, 130),
SVEMAP1(svadd_u16_z, aarch64_sve_add, 386),
SVEMAP1(svadd_u32_m, aarch64_sve_add, 259),
SVEMAP1(svadd_u32_x, aarch64_sve_add, 131),
SVEMAP1(svadd_u32_z, aarch64_sve_add, 387),
SVEMAP1(svadd_u64_m, aarch64_sve_add, 260),
SVEMAP1(svadd_u64_x, aarch64_sve_add, 132),
SVEMAP1(svadd_u64_z, aarch64_sve_add, 388),
SVEMAP1(svadd_u8_m, aarch64_sve_add, 257),
SVEMAP1(svadd_u8_x, aarch64_sve_add, 129),
SVEMAP1(svadd_u8_z, aarch64_sve_add, 385),
SVEMAP1(svadda_f16, aarch64_sve_fadda, 5),
SVEMAP1(svadda_f32, aarch64_sve_fadda, 6),
SVEMAP1(svadda_f64, aarch64_sve_fadda, 7),
SVEMAP1(svaddhnb_n_s16, aarch64_sve_addhnb, 2050),
SVEMAP1(svaddhnb_n_s32, aarch64_sve_addhnb, 2051),
SVEMAP1(svaddhnb_n_s64, aarch64_sve_addhnb, 2052),
SVEMAP1(svaddhnb_n_u16, aarch64_sve_addhnb, 2050),
SVEMAP1(svaddhnb_n_u32, aarch64_sve_addhnb, 2051),
SVEMAP1(svaddhnb_n_u64, aarch64_sve_addhnb, 2052),
SVEMAP1(svaddhnb_s16, aarch64_sve_addhnb, 2),
SVEMAP1(svaddhnb_s32, aarch64_sve_addhnb, 3),
SVEMAP1(svaddhnb_s64, aarch64_sve_addhnb, 4),
SVEMAP1(svaddhnb_u16, aarch64_sve_addhnb, 2),
SVEMAP1(svaddhnb_u32, aarch64_sve_addhnb, 3),
SVEMAP1(svaddhnb_u64, aarch64_sve_addhnb, 4),
SVEMAP1(svaddhnt_n_s16, aarch64_sve_addhnt, 3074),
SVEMAP1(svaddhnt_n_s32, aarch64_sve_addhnt, 3075),
SVEMAP1(svaddhnt_n_s64, aarch64_sve_addhnt, 3076),
SVEMAP1(svaddhnt_n_u16, aarch64_sve_addhnt, 3074),
SVEMAP1(svaddhnt_n_u32, aarch64_sve_addhnt, 3075),
SVEMAP1(svaddhnt_n_u64, aarch64_sve_addhnt, 3076),
SVEMAP1(svaddhnt_s16, aarch64_sve_addhnt, 2),
SVEMAP1(svaddhnt_s32, aarch64_sve_addhnt, 3),
SVEMAP1(svaddhnt_s64, aarch64_sve_addhnt, 4),
SVEMAP1(svaddhnt_u16, aarch64_sve_addhnt, 2),
SVEMAP1(svaddhnt_u32, aarch64_sve_addhnt, 3),
SVEMAP1(svaddhnt_u64, aarch64_sve_addhnt, 4),
SVEMAP1(svaddlb_n_s16, aarch64_sve_saddlb, 2050),
SVEMAP1(svaddlb_n_s32, aarch64_sve_saddlb, 2051),
SVEMAP1(svaddlb_n_s64, aarch64_sve_saddlb, 2052),
SVEMAP1(svaddlb_n_u16, aarch64_sve_uaddlb, 2050),
SVEMAP1(svaddlb_n_u32, aarch64_sve_uaddlb, 2051),
SVEMAP1(svaddlb_n_u64, aarch64_sve_uaddlb, 2052),
SVEMAP1(svaddlb_s16, aarch64_sve_saddlb, 2),
SVEMAP1(svaddlb_s32, aarch64_sve_saddlb, 3),
SVEMAP1(svaddlb_s64, aarch64_sve_saddlb, 4),
SVEMAP1(svaddlb_u16, aarch64_sve_uaddlb, 2),
SVEMAP1(svaddlb_u32, aarch64_sve_uaddlb, 3),
SVEMAP1(svaddlb_u64, aarch64_sve_uaddlb, 4),
SVEMAP1(svaddlbt_n_s16, aarch64_sve_saddlbt, 2050),
SVEMAP1(svaddlbt_n_s32, aarch64_sve_saddlbt, 2051),
SVEMAP1(svaddlbt_n_s64, aarch64_sve_saddlbt, 2052),
SVEMAP1(svaddlbt_s16, aarch64_sve_saddlbt, 2),
SVEMAP1(svaddlbt_s32, aarch64_sve_saddlbt, 3),
SVEMAP1(svaddlbt_s64, aarch64_sve_saddlbt, 4),
SVEMAP1(svaddlt_n_s16, aarch64_sve_saddlt, 2050),
SVEMAP1(svaddlt_n_s32, aarch64_sve_saddlt, 2051),
SVEMAP1(svaddlt_n_s64, aarch64_sve_saddlt, 2052),
SVEMAP1(svaddlt_n_u16, aarch64_sve_uaddlt, 2050),
SVEMAP1(svaddlt_n_u32, aarch64_sve_uaddlt, 2051),
SVEMAP1(svaddlt_n_u64, aarch64_sve_uaddlt, 2052),
SVEMAP1(svaddlt_s16, aarch64_sve_saddlt, 2),
SVEMAP1(svaddlt_s32, aarch64_sve_saddlt, 3),
SVEMAP1(svaddlt_s64, aarch64_sve_saddlt, 4),
SVEMAP1(svaddlt_u16, aarch64_sve_uaddlt, 2),
SVEMAP1(svaddlt_u32, aarch64_sve_uaddlt, 3),
SVEMAP1(svaddlt_u64, aarch64_sve_uaddlt, 4),
SVEMAP1(svaddp_f16_m, aarch64_sve_faddp, 261),
SVEMAP1(svaddp_f16_x, aarch64_sve_faddp, 133),
SVEMAP1(svaddp_f32_m, aarch64_sve_faddp, 262),
SVEMAP1(svaddp_f32_x, aarch64_sve_faddp, 134),
SVEMAP1(svaddp_f64_m, aarch64_sve_faddp, 263),
SVEMAP1(svaddp_f64_x, aarch64_sve_faddp, 135),
SVEMAP1(svaddp_s16_m, aarch64_sve_addp, 258),
SVEMAP1(svaddp_s16_x, aarch64_sve_addp, 130),
SVEMAP1(svaddp_s32_m, aarch64_sve_addp, 259),
SVEMAP1(svaddp_s32_x, aarch64_sve_addp, 131),
SVEMAP1(svaddp_s64_m, aarch64_sve_addp, 260),
SVEMAP1(svaddp_s64_x, aarch64_sve_addp, 132),
SVEMAP1(svaddp_s8_m, aarch64_sve_addp, 257),
SVEMAP1(svaddp_s8_x, aarch64_sve_addp, 129),
SVEMAP1(svaddp_u16_m, aarch64_sve_addp, 258),
SVEMAP1(svaddp_u16_x, aarch64_sve_addp, 130),
SVEMAP1(svaddp_u32_m, aarch64_sve_addp, 259),
SVEMAP1(svaddp_u32_x, aarch64_sve_addp, 131),
SVEMAP1(svaddp_u64_m, aarch64_sve_addp, 260),
SVEMAP1(svaddp_u64_x, aarch64_sve_addp, 132),
SVEMAP1(svaddp_u8_m, aarch64_sve_addp, 257),
SVEMAP1(svaddp_u8_x, aarch64_sve_addp, 129),
SVEMAP1(svaddv_f16, aarch64_sve_faddv, 5),
SVEMAP1(svaddv_f32, aarch64_sve_faddv, 6),
SVEMAP1(svaddv_f64, aarch64_sve_faddv, 7),
SVEMAP1(svaddv_s16, aarch64_sve_saddv, 2),
SVEMAP1(svaddv_s32, aarch64_sve_saddv, 3),
SVEMAP1(svaddv_s64, aarch64_sve_saddv, 4),
SVEMAP1(svaddv_s8, aarch64_sve_saddv, 1),
SVEMAP1(svaddv_u16, aarch64_sve_uaddv, 2),
SVEMAP1(svaddv_u32, aarch64_sve_uaddv, 3),
SVEMAP1(svaddv_u64, aarch64_sve_uaddv, 4),
SVEMAP1(svaddv_u8, aarch64_sve_uaddv, 1),
SVEMAP1(svaddwb_n_s16, aarch64_sve_saddwb, 2050),
SVEMAP1(svaddwb_n_s32, aarch64_sve_saddwb, 2051),
SVEMAP1(svaddwb_n_s64, aarch64_sve_saddwb, 2052),
SVEMAP1(svaddwb_n_u16, aarch64_sve_uaddwb, 2050),
SVEMAP1(svaddwb_n_u32, aarch64_sve_uaddwb, 2051),
SVEMAP1(svaddwb_n_u64, aarch64_sve_uaddwb, 2052),
SVEMAP1(svaddwb_s16, aarch64_sve_saddwb, 2),
SVEMAP1(svaddwb_s32, aarch64_sve_saddwb, 3),
SVEMAP1(svaddwb_s64, aarch64_sve_saddwb, 4),
SVEMAP1(svaddwb_u16, aarch64_sve_uaddwb, 2),
SVEMAP1(svaddwb_u32, aarch64_sve_uaddwb, 3),
SVEMAP1(svaddwb_u64, aarch64_sve_uaddwb, 4),
SVEMAP1(svaddwt_n_s16, aarch64_sve_saddwt, 2050),
SVEMAP1(svaddwt_n_s32, aarch64_sve_saddwt, 2051),
SVEMAP1(svaddwt_n_s64, aarch64_sve_saddwt, 2052),
SVEMAP1(svaddwt_n_u16, aarch64_sve_uaddwt, 2050),
SVEMAP1(svaddwt_n_u32, aarch64_sve_uaddwt, 2051),
SVEMAP1(svaddwt_n_u64, aarch64_sve_uaddwt, 2052),
SVEMAP1(svaddwt_s16, aarch64_sve_saddwt, 2),
SVEMAP1(svaddwt_s32, aarch64_sve_saddwt, 3),
SVEMAP1(svaddwt_s64, aarch64_sve_saddwt, 4),
SVEMAP1(svaddwt_u16, aarch64_sve_uaddwt, 2),
SVEMAP1(svaddwt_u32, aarch64_sve_uaddwt, 3),
SVEMAP1(svaddwt_u64, aarch64_sve_uaddwt, 4),
SVEMAP1(svadrb_u32base_s32offset, aarch64_sve_adrb, 3),
SVEMAP1(svadrb_u32base_u32offset, aarch64_sve_adrb, 3),
SVEMAP1(svadrb_u64base_s64offset, aarch64_sve_adrb, 4),
SVEMAP1(svadrb_u64base_u64offset, aarch64_sve_adrb, 4),
SVEMAP1(svadrd_u32base_s32index, aarch64_sve_adrd, 3),
SVEMAP1(svadrd_u32base_u32index, aarch64_sve_adrd, 3),
SVEMAP1(svadrd_u64base_s64index, aarch64_sve_adrd, 4),
SVEMAP1(svadrd_u64base_u64index, aarch64_sve_adrd, 4),
SVEMAP1(svadrh_u32base_s32index, aarch64_sve_adrh, 3),
SVEMAP1(svadrh_u32base_u32index, aarch64_sve_adrh, 3),
SVEMAP1(svadrh_u64base_s64index, aarch64_sve_adrh, 4),
SVEMAP1(svadrh_u64base_u64index, aarch64_sve_adrh, 4),
SVEMAP1(svadrw_u32base_s32index, aarch64_sve_adrw, 3),
SVEMAP1(svadrw_u32base_u32index, aarch64_sve_adrw, 3),
SVEMAP1(svadrw_u64base_s64index, aarch64_sve_adrw, 4),
SVEMAP1(svadrw_u64base_u64index, aarch64_sve_adrw, 4),
SVEMAP1(svaesd_u8, aarch64_sve_aesd, 1048577),
SVEMAP1(svaese_u8, aarch64_sve_aese, 1048577),
SVEMAP1(svaesimc_u8, aarch64_sve_aesimc, 1048577),
SVEMAP1(svaesmc_u8, aarch64_sve_aesmc, 1048577),
SVEMAP1(svand_b_z, aarch64_sve_and_z, 8),
SVEMAP1(svand_n_s16_m, aarch64_sve_and, 3330),
SVEMAP1(svand_n_s16_x, aarch64_sve_and, 3202),
SVEMAP1(svand_n_s16_z, aarch64_sve_and, 3458),
SVEMAP1(svand_n_s32_m, aarch64_sve_and, 3331),
SVEMAP1(svand_n_s32_x, aarch64_sve_and, 3203),
SVEMAP1(svand_n_s32_z, aarch64_sve_and, 3459),
SVEMAP1(svand_n_s64_m, aarch64_sve_and, 3332),
SVEMAP1(svand_n_s64_x, aarch64_sve_and, 3204),
SVEMAP1(svand_n_s64_z, aarch64_sve_and, 3460),
SVEMAP1(svand_n_s8_m, aarch64_sve_and, 3329),
SVEMAP1(svand_n_s8_x, aarch64_sve_and, 3201),
SVEMAP1(svand_n_s8_z, aarch64_sve_and, 3457),
SVEMAP1(svand_n_u16_m, aarch64_sve_and, 3330),
SVEMAP1(svand_n_u16_x, aarch64_sve_and, 3202),
SVEMAP1(svand_n_u16_z, aarch64_sve_and, 3458),
SVEMAP1(svand_n_u32_m, aarch64_sve_and, 3331),
SVEMAP1(svand_n_u32_x, aarch64_sve_and, 3203),
SVEMAP1(svand_n_u32_z, aarch64_sve_and, 3459),
SVEMAP1(svand_n_u64_m, aarch64_sve_and, 3332),
SVEMAP1(svand_n_u64_x, aarch64_sve_and, 3204),
SVEMAP1(svand_n_u64_z, aarch64_sve_and, 3460),
SVEMAP1(svand_n_u8_m, aarch64_sve_and, 3329),
SVEMAP1(svand_n_u8_x, aarch64_sve_and, 3201),
SVEMAP1(svand_n_u8_z, aarch64_sve_and, 3457),
SVEMAP1(svand_s16_m, aarch64_sve_and, 258),
SVEMAP1(svand_s16_x, aarch64_sve_and, 130),
SVEMAP1(svand_s16_z, aarch64_sve_and, 386),
SVEMAP1(svand_s32_m, aarch64_sve_and, 259),
SVEMAP1(svand_s32_x, aarch64_sve_and, 131),
SVEMAP1(svand_s32_z, aarch64_sve_and, 387),
SVEMAP1(svand_s64_m, aarch64_sve_and, 260),
SVEMAP1(svand_s64_x, aarch64_sve_and, 132),
SVEMAP1(svand_s64_z, aarch64_sve_and, 388),
SVEMAP1(svand_s8_m, aarch64_sve_and, 257),
SVEMAP1(svand_s8_x, aarch64_sve_and, 129),
SVEMAP1(svand_s8_z, aarch64_sve_and, 385),
SVEMAP1(svand_u16_m, aarch64_sve_and, 258),
SVEMAP1(svand_u16_x, aarch64_sve_and, 130),
SVEMAP1(svand_u16_z, aarch64_sve_and, 386),
SVEMAP1(svand_u32_m, aarch64_sve_and, 259),
SVEMAP1(svand_u32_x, aarch64_sve_and, 131),
SVEMAP1(svand_u32_z, aarch64_sve_and, 387),
SVEMAP1(svand_u64_m, aarch64_sve_and, 260),
SVEMAP1(svand_u64_x, aarch64_sve_and, 132),
SVEMAP1(svand_u64_z, aarch64_sve_and, 388),
SVEMAP1(svand_u8_m, aarch64_sve_and, 257),
SVEMAP1(svand_u8_x, aarch64_sve_and, 129),
SVEMAP1(svand_u8_z, aarch64_sve_and, 385),
SVEMAP1(svandv_s16, aarch64_sve_andv, 2),
SVEMAP1(svandv_s32, aarch64_sve_andv, 3),
SVEMAP1(svandv_s64, aarch64_sve_andv, 4),
SVEMAP1(svandv_s8, aarch64_sve_andv, 1),
SVEMAP1(svandv_u16, aarch64_sve_andv, 2),
SVEMAP1(svandv_u32, aarch64_sve_andv, 3),
SVEMAP1(svandv_u64, aarch64_sve_andv, 4),
SVEMAP1(svandv_u8, aarch64_sve_andv, 1),
SVEMAP1(svasr_n_s16_m, aarch64_sve_asr, 3330),
SVEMAP1(svasr_n_s16_x, aarch64_sve_asr, 3202),
SVEMAP1(svasr_n_s16_z, aarch64_sve_asr, 3458),
SVEMAP1(svasr_n_s32_m, aarch64_sve_asr, 3331),
SVEMAP1(svasr_n_s32_x, aarch64_sve_asr, 3203),
SVEMAP1(svasr_n_s32_z, aarch64_sve_asr, 3459),
SVEMAP1(svasr_n_s64_m, aarch64_sve_asr, 3332),
SVEMAP1(svasr_n_s64_x, aarch64_sve_asr, 3204),
SVEMAP1(svasr_n_s64_z, aarch64_sve_asr, 3460),
SVEMAP1(svasr_n_s8_m, aarch64_sve_asr, 3329),
SVEMAP1(svasr_n_s8_x, aarch64_sve_asr, 3201),
SVEMAP1(svasr_n_s8_z, aarch64_sve_asr, 3457),
SVEMAP1(svasr_s16_m, aarch64_sve_asr, 258),
SVEMAP1(svasr_s16_x, aarch64_sve_asr, 130),
SVEMAP1(svasr_s16_z, aarch64_sve_asr, 386),
SVEMAP1(svasr_s32_m, aarch64_sve_asr, 259),
SVEMAP1(svasr_s32_x, aarch64_sve_asr, 131),
SVEMAP1(svasr_s32_z, aarch64_sve_asr, 387),
SVEMAP1(svasr_s64_m, aarch64_sve_asr, 260),
SVEMAP1(svasr_s64_x, aarch64_sve_asr, 132),
SVEMAP1(svasr_s64_z, aarch64_sve_asr, 388),
SVEMAP1(svasr_s8_m, aarch64_sve_asr, 257),
SVEMAP1(svasr_s8_x, aarch64_sve_asr, 129),
SVEMAP1(svasr_s8_z, aarch64_sve_asr, 385),
SVEMAP1(svasr_wide_n_s16_m, aarch64_sve_asr_wide, 3330),
SVEMAP1(svasr_wide_n_s16_x, aarch64_sve_asr_wide, 3202),
SVEMAP1(svasr_wide_n_s16_z, aarch64_sve_asr_wide, 3458),
SVEMAP1(svasr_wide_n_s32_m, aarch64_sve_asr_wide, 3331),
SVEMAP1(svasr_wide_n_s32_x, aarch64_sve_asr_wide, 3203),
SVEMAP1(svasr_wide_n_s32_z, aarch64_sve_asr_wide, 3459),
SVEMAP1(svasr_wide_n_s8_m, aarch64_sve_asr_wide, 3329),
SVEMAP1(svasr_wide_n_s8_x, aarch64_sve_asr_wide, 3201),
SVEMAP1(svasr_wide_n_s8_z, aarch64_sve_asr_wide, 3457),
SVEMAP1(svasr_wide_s16_m, aarch64_sve_asr_wide, 258),
SVEMAP1(svasr_wide_s16_x, aarch64_sve_asr_wide, 130),
SVEMAP1(svasr_wide_s16_z, aarch64_sve_asr_wide, 386),
SVEMAP1(svasr_wide_s32_m, aarch64_sve_asr_wide, 259),
SVEMAP1(svasr_wide_s32_x, aarch64_sve_asr_wide, 131),
SVEMAP1(svasr_wide_s32_z, aarch64_sve_asr_wide, 387),
SVEMAP1(svasr_wide_s8_m, aarch64_sve_asr_wide, 257),
SVEMAP1(svasr_wide_s8_x, aarch64_sve_asr_wide, 129),
SVEMAP1(svasr_wide_s8_z, aarch64_sve_asr_wide, 385),
SVEMAP1(svasrd_n_s16_m, aarch64_sve_asrd, 258),
SVEMAP1(svasrd_n_s16_x, aarch64_sve_asrd, 130),
SVEMAP1(svasrd_n_s16_z, aarch64_sve_asrd, 386),
SVEMAP1(svasrd_n_s32_m, aarch64_sve_asrd, 259),
SVEMAP1(svasrd_n_s32_x, aarch64_sve_asrd, 131),
SVEMAP1(svasrd_n_s32_z, aarch64_sve_asrd, 387),
SVEMAP1(svasrd_n_s64_m, aarch64_sve_asrd, 260),
SVEMAP1(svasrd_n_s64_x, aarch64_sve_asrd, 132),
SVEMAP1(svasrd_n_s64_z, aarch64_sve_asrd, 388),
SVEMAP1(svasrd_n_s8_m, aarch64_sve_asrd, 257),
SVEMAP1(svasrd_n_s8_x, aarch64_sve_asrd, 129),
SVEMAP1(svasrd_n_s8_z, aarch64_sve_asrd, 385),
SVEMAP1(svbcax_n_s16, aarch64_sve_bcax, 3074),
SVEMAP1(svbcax_n_s32, aarch64_sve_bcax, 3075),
SVEMAP1(svbcax_n_s64, aarch64_sve_bcax, 3076),
SVEMAP1(svbcax_n_s8, aarch64_sve_bcax, 3073),
SVEMAP1(svbcax_n_u16, aarch64_sve_bcax, 3074),
SVEMAP1(svbcax_n_u32, aarch64_sve_bcax, 3075),
SVEMAP1(svbcax_n_u64, aarch64_sve_bcax, 3076),
SVEMAP1(svbcax_n_u8, aarch64_sve_bcax, 3073),
SVEMAP1(svbcax_s16, aarch64_sve_bcax, 2),
SVEMAP1(svbcax_s32, aarch64_sve_bcax, 3),
SVEMAP1(svbcax_s64, aarch64_sve_bcax, 4),
SVEMAP1(svbcax_s8, aarch64_sve_bcax, 1),
SVEMAP1(svbcax_u16, aarch64_sve_bcax, 2),
SVEMAP1(svbcax_u32, aarch64_sve_bcax, 3),
SVEMAP1(svbcax_u64, aarch64_sve_bcax, 4),
SVEMAP1(svbcax_u8, aarch64_sve_bcax, 1),
SVEMAP1(svbdep_n_u16, aarch64_sve_bdep_x, 2050),
SVEMAP1(svbdep_n_u32, aarch64_sve_bdep_x, 2051),
SVEMAP1(svbdep_n_u64, aarch64_sve_bdep_x, 2052),
SVEMAP1(svbdep_n_u8, aarch64_sve_bdep_x, 2049),
SVEMAP1(svbdep_u16, aarch64_sve_bdep_x, 2),
SVEMAP1(svbdep_u32, aarch64_sve_bdep_x, 3),
SVEMAP1(svbdep_u64, aarch64_sve_bdep_x, 4),
SVEMAP1(svbdep_u8, aarch64_sve_bdep_x, 1),
SVEMAP1(svbext_n_u16, aarch64_sve_bext_x, 2050),
SVEMAP1(svbext_n_u32, aarch64_sve_bext_x, 2051),
SVEMAP1(svbext_n_u64, aarch64_sve_bext_x, 2052),
SVEMAP1(svbext_n_u8, aarch64_sve_bext_x, 2049),
SVEMAP1(svbext_u16, aarch64_sve_bext_x, 2),
SVEMAP1(svbext_u32, aarch64_sve_bext_x, 3),
SVEMAP1(svbext_u64, aarch64_sve_bext_x, 4),
SVEMAP1(svbext_u8, aarch64_sve_bext_x, 1),
SVEMAP1(svbfdot_f32, aarch64_sve_bfdot, 1048588),
SVEMAP1(svbfdot_lane_f32, aarch64_sve_bfdot_lane, 1048588),
SVEMAP1(svbfdot_n_f32, aarch64_sve_bfdot, 1051660),
SVEMAP1(svbfmlalb_f32, aarch64_sve_bfmlalb, 1048588),
SVEMAP1(svbfmlalb_lane_f32, aarch64_sve_bfmlalb_lane, 1048588),
SVEMAP1(svbfmlalb_n_f32, aarch64_sve_bfmlalb, 1051660),
SVEMAP1(svbfmlalt_f32, aarch64_sve_bfmlalt, 1048588),
SVEMAP1(svbfmlalt_lane_f32, aarch64_sve_bfmlalt_lane, 1048588),
SVEMAP1(svbfmlalt_n_f32, aarch64_sve_bfmlalt, 1051660),
SVEMAP1(svbfmmla_f32, aarch64_sve_bfmmla, 1048588),
SVEMAP1(svbgrp_n_u16, aarch64_sve_bgrp_x, 2050),
SVEMAP1(svbgrp_n_u32, aarch64_sve_bgrp_x, 2051),
SVEMAP1(svbgrp_n_u64, aarch64_sve_bgrp_x, 2052),
SVEMAP1(svbgrp_n_u8, aarch64_sve_bgrp_x, 2049),
SVEMAP1(svbgrp_u16, aarch64_sve_bgrp_x, 2),
SVEMAP1(svbgrp_u32, aarch64_sve_bgrp_x, 3),
SVEMAP1(svbgrp_u64, aarch64_sve_bgrp_x, 4),
SVEMAP1(svbgrp_u8, aarch64_sve_bgrp_x, 1),
SVEMAP1(svbic_b_z, aarch64_sve_bic_z, 8),
SVEMAP1(svbic_n_s16_m, aarch64_sve_bic, 3330),
SVEMAP1(svbic_n_s16_x, aarch64_sve_bic, 3202),
SVEMAP1(svbic_n_s16_z, aarch64_sve_bic, 3458),
SVEMAP1(svbic_n_s32_m, aarch64_sve_bic, 3331),
SVEMAP1(svbic_n_s32_x, aarch64_sve_bic, 3203),
SVEMAP1(svbic_n_s32_z, aarch64_sve_bic, 3459),
SVEMAP1(svbic_n_s64_m, aarch64_sve_bic, 3332),
SVEMAP1(svbic_n_s64_x, aarch64_sve_bic, 3204),
SVEMAP1(svbic_n_s64_z, aarch64_sve_bic, 3460),
SVEMAP1(svbic_n_s8_m, aarch64_sve_bic, 3329),
SVEMAP1(svbic_n_s8_x, aarch64_sve_bic, 3201),
SVEMAP1(svbic_n_s8_z, aarch64_sve_bic, 3457),
SVEMAP1(svbic_n_u16_m, aarch64_sve_bic, 3330),
SVEMAP1(svbic_n_u16_x, aarch64_sve_bic, 3202),
SVEMAP1(svbic_n_u16_z, aarch64_sve_bic, 3458),
SVEMAP1(svbic_n_u32_m, aarch64_sve_bic, 3331),
SVEMAP1(svbic_n_u32_x, aarch64_sve_bic, 3203),
SVEMAP1(svbic_n_u32_z, aarch64_sve_bic, 3459),
SVEMAP1(svbic_n_u64_m, aarch64_sve_bic, 3332),
SVEMAP1(svbic_n_u64_x, aarch64_sve_bic, 3204),
SVEMAP1(svbic_n_u64_z, aarch64_sve_bic, 3460),
SVEMAP1(svbic_n_u8_m, aarch64_sve_bic, 3329),
SVEMAP1(svbic_n_u8_x, aarch64_sve_bic, 3201),
SVEMAP1(svbic_n_u8_z, aarch64_sve_bic, 3457),
SVEMAP1(svbic_s16_m, aarch64_sve_bic, 258),
SVEMAP1(svbic_s16_x, aarch64_sve_bic, 130),
SVEMAP1(svbic_s16_z, aarch64_sve_bic, 386),
SVEMAP1(svbic_s32_m, aarch64_sve_bic, 259),
SVEMAP1(svbic_s32_x, aarch64_sve_bic, 131),
SVEMAP1(svbic_s32_z, aarch64_sve_bic, 387),
SVEMAP1(svbic_s64_m, aarch64_sve_bic, 260),
SVEMAP1(svbic_s64_x, aarch64_sve_bic, 132),
SVEMAP1(svbic_s64_z, aarch64_sve_bic, 388),
SVEMAP1(svbic_s8_m, aarch64_sve_bic, 257),
SVEMAP1(svbic_s8_x, aarch64_sve_bic, 129),
SVEMAP1(svbic_s8_z, aarch64_sve_bic, 385),
SVEMAP1(svbic_u16_m, aarch64_sve_bic, 258),
SVEMAP1(svbic_u16_x, aarch64_sve_bic, 130),
SVEMAP1(svbic_u16_z, aarch64_sve_bic, 386),
SVEMAP1(svbic_u32_m, aarch64_sve_bic, 259),
SVEMAP1(svbic_u32_x, aarch64_sve_bic, 131),
SVEMAP1(svbic_u32_z, aarch64_sve_bic, 387),
SVEMAP1(svbic_u64_m, aarch64_sve_bic, 260),
SVEMAP1(svbic_u64_x, aarch64_sve_bic, 132),
SVEMAP1(svbic_u64_z, aarch64_sve_bic, 388),
SVEMAP1(svbic_u8_m, aarch64_sve_bic, 257),
SVEMAP1(svbic_u8_x, aarch64_sve_bic, 129),
SVEMAP1(svbic_u8_z, aarch64_sve_bic, 385),
SVEMAP1(svbrka_b_m, aarch64_sve_brka, 8),
SVEMAP1(svbrka_b_z, aarch64_sve_brka_z, 8),
SVEMAP1(svbrkb_b_m, aarch64_sve_brkb, 8),
SVEMAP1(svbrkb_b_z, aarch64_sve_brkb_z, 8),
SVEMAP1(svbrkn_b_z, aarch64_sve_brkn_z, 8),
SVEMAP1(svbrkpa_b_z, aarch64_sve_brkpa_z, 8),
SVEMAP1(svbrkpb_b_z, aarch64_sve_brkpb_z, 8),
SVEMAP1(svbsl1n_n_s16, aarch64_sve_bsl1n, 3074),
SVEMAP1(svbsl1n_n_s32, aarch64_sve_bsl1n, 3075),
SVEMAP1(svbsl1n_n_s64, aarch64_sve_bsl1n, 3076),
SVEMAP1(svbsl1n_n_s8, aarch64_sve_bsl1n, 3073),
SVEMAP1(svbsl1n_n_u16, aarch64_sve_bsl1n, 3074),
SVEMAP1(svbsl1n_n_u32, aarch64_sve_bsl1n, 3075),
SVEMAP1(svbsl1n_n_u64, aarch64_sve_bsl1n, 3076),
SVEMAP1(svbsl1n_n_u8, aarch64_sve_bsl1n, 3073),
SVEMAP1(svbsl1n_s16, aarch64_sve_bsl1n, 2),
SVEMAP1(svbsl1n_s32, aarch64_sve_bsl1n, 3),
SVEMAP1(svbsl1n_s64, aarch64_sve_bsl1n, 4),
SVEMAP1(svbsl1n_s8, aarch64_sve_bsl1n, 1),
SVEMAP1(svbsl1n_u16, aarch64_sve_bsl1n, 2),
SVEMAP1(svbsl1n_u32, aarch64_sve_bsl1n, 3),
SVEMAP1(svbsl1n_u64, aarch64_sve_bsl1n, 4),
SVEMAP1(svbsl1n_u8, aarch64_sve_bsl1n, 1),
SVEMAP1(svbsl2n_n_s16, aarch64_sve_bsl2n, 3074),
SVEMAP1(svbsl2n_n_s32, aarch64_sve_bsl2n, 3075),
SVEMAP1(svbsl2n_n_s64, aarch64_sve_bsl2n, 3076),
SVEMAP1(svbsl2n_n_s8, aarch64_sve_bsl2n, 3073),
SVEMAP1(svbsl2n_n_u16, aarch64_sve_bsl2n, 3074),
SVEMAP1(svbsl2n_n_u32, aarch64_sve_bsl2n, 3075),
SVEMAP1(svbsl2n_n_u64, aarch64_sve_bsl2n, 3076),
SVEMAP1(svbsl2n_n_u8, aarch64_sve_bsl2n, 3073),
SVEMAP1(svbsl2n_s16, aarch64_sve_bsl2n, 2),
SVEMAP1(svbsl2n_s32, aarch64_sve_bsl2n, 3),
SVEMAP1(svbsl2n_s64, aarch64_sve_bsl2n, 4),
SVEMAP1(svbsl2n_s8, aarch64_sve_bsl2n, 1),
SVEMAP1(svbsl2n_u16, aarch64_sve_bsl2n, 2),
SVEMAP1(svbsl2n_u32, aarch64_sve_bsl2n, 3),
SVEMAP1(svbsl2n_u64, aarch64_sve_bsl2n, 4),
SVEMAP1(svbsl2n_u8, aarch64_sve_bsl2n, 1),
SVEMAP1(svbsl_n_s16, aarch64_sve_bsl, 3074),
SVEMAP1(svbsl_n_s32, aarch64_sve_bsl, 3075),
SVEMAP1(svbsl_n_s64, aarch64_sve_bsl, 3076),
SVEMAP1(svbsl_n_s8, aarch64_sve_bsl, 3073),
SVEMAP1(svbsl_n_u16, aarch64_sve_bsl, 3074),
SVEMAP1(svbsl_n_u32, aarch64_sve_bsl, 3075),
SVEMAP1(svbsl_n_u64, aarch64_sve_bsl, 3076),
SVEMAP1(svbsl_n_u8, aarch64_sve_bsl, 3073),
SVEMAP1(svbsl_s16, aarch64_sve_bsl, 2),
SVEMAP1(svbsl_s32, aarch64_sve_bsl, 3),
SVEMAP1(svbsl_s64, aarch64_sve_bsl, 4),
SVEMAP1(svbsl_s8, aarch64_sve_bsl, 1),
SVEMAP1(svbsl_u16, aarch64_sve_bsl, 2),
SVEMAP1(svbsl_u32, aarch64_sve_bsl, 3),
SVEMAP1(svbsl_u64, aarch64_sve_bsl, 4),
SVEMAP1(svbsl_u8, aarch64_sve_bsl, 1),
SVEMAP1(svcadd_f16_m, aarch64_sve_fcadd, 261),
SVEMAP1(svcadd_f16_x, aarch64_sve_fcadd, 133),
SVEMAP1(svcadd_f16_z, aarch64_sve_fcadd, 389),
SVEMAP1(svcadd_f32_m, aarch64_sve_fcadd, 262),
SVEMAP1(svcadd_f32_x, aarch64_sve_fcadd, 134),
SVEMAP1(svcadd_f32_z, aarch64_sve_fcadd, 390),
SVEMAP1(svcadd_f64_m, aarch64_sve_fcadd, 263),
SVEMAP1(svcadd_f64_x, aarch64_sve_fcadd, 135),
SVEMAP1(svcadd_f64_z, aarch64_sve_fcadd, 391),
SVEMAP1(svcadd_s16, aarch64_sve_cadd_x, 2),
SVEMAP1(svcadd_s32, aarch64_sve_cadd_x, 3),
SVEMAP1(svcadd_s64, aarch64_sve_cadd_x, 4),
SVEMAP1(svcadd_s8, aarch64_sve_cadd_x, 1),
SVEMAP1(svcadd_u16, aarch64_sve_cadd_x, 2),
SVEMAP1(svcadd_u32, aarch64_sve_cadd_x, 3),
SVEMAP1(svcadd_u64, aarch64_sve_cadd_x, 4),
SVEMAP1(svcadd_u8, aarch64_sve_cadd_x, 1),
SVEMAP1(svcdot_lane_s32, aarch64_sve_cdot_lane, 3),
SVEMAP1(svcdot_lane_s64, aarch64_sve_cdot_lane, 4),
SVEMAP1(svcdot_s32, aarch64_sve_cdot, 3),
SVEMAP1(svcdot_s64, aarch64_sve_cdot, 4),
SVEMAP1(svclasta_bf16, aarch64_sve_clasta, 12),
SVEMAP1(svclasta_f16, aarch64_sve_clasta, 5),
SVEMAP1(svclasta_f32, aarch64_sve_clasta, 6),
SVEMAP1(svclasta_f64, aarch64_sve_clasta, 7),
SVEMAP1(svclasta_n_bf16, aarch64_sve_clasta_n, 12),
SVEMAP1(svclasta_n_f16, aarch64_sve_clasta_n, 5),
SVEMAP1(svclasta_n_f32, aarch64_sve_clasta_n, 6),
SVEMAP1(svclasta_n_f64, aarch64_sve_clasta_n, 7),
SVEMAP1(svclasta_n_s16, aarch64_sve_clasta_n, 2),
SVEMAP1(svclasta_n_s32, aarch64_sve_clasta_n, 3),
SVEMAP1(svclasta_n_s64, aarch64_sve_clasta_n, 4),
SVEMAP1(svclasta_n_s8, aarch64_sve_clasta_n, 1),
SVEMAP1(svclasta_n_u16, aarch64_sve_clasta_n, 2),
SVEMAP1(svclasta_n_u32, aarch64_sve_clasta_n, 3),
SVEMAP1(svclasta_n_u64, aarch64_sve_clasta_n, 4),
SVEMAP1(svclasta_n_u8, aarch64_sve_clasta_n, 1),
SVEMAP1(svclasta_s16, aarch64_sve_clasta, 2),
SVEMAP1(svclasta_s32, aarch64_sve_clasta, 3),
SVEMAP1(svclasta_s64, aarch64_sve_clasta, 4),
SVEMAP1(svclasta_s8, aarch64_sve_clasta, 1),
SVEMAP1(svclasta_u16, aarch64_sve_clasta, 2),
SVEMAP1(svclasta_u32, aarch64_sve_clasta, 3),
SVEMAP1(svclasta_u64, aarch64_sve_clasta, 4),
SVEMAP1(svclasta_u8, aarch64_sve_clasta, 1),
SVEMAP1(svclastb_bf16, aarch64_sve_clastb, 12),
SVEMAP1(svclastb_f16, aarch64_sve_clastb, 5),
SVEMAP1(svclastb_f32, aarch64_sve_clastb, 6),
SVEMAP1(svclastb_f64, aarch64_sve_clastb, 7),
SVEMAP1(svclastb_n_bf16, aarch64_sve_clastb_n, 12),
SVEMAP1(svclastb_n_f16, aarch64_sve_clastb_n, 5),
SVEMAP1(svclastb_n_f32, aarch64_sve_clastb_n, 6),
SVEMAP1(svclastb_n_f64, aarch64_sve_clastb_n, 7),
SVEMAP1(svclastb_n_s16, aarch64_sve_clastb_n, 2),
SVEMAP1(svclastb_n_s32, aarch64_sve_clastb_n, 3),
SVEMAP1(svclastb_n_s64, aarch64_sve_clastb_n, 4),
SVEMAP1(svclastb_n_s8, aarch64_sve_clastb_n, 1),
SVEMAP1(svclastb_n_u16, aarch64_sve_clastb_n, 2),
SVEMAP1(svclastb_n_u32, aarch64_sve_clastb_n, 3),
SVEMAP1(svclastb_n_u64, aarch64_sve_clastb_n, 4),
SVEMAP1(svclastb_n_u8, aarch64_sve_clastb_n, 1),
SVEMAP1(svclastb_s16, aarch64_sve_clastb, 2),
SVEMAP1(svclastb_s32, aarch64_sve_clastb, 3),
SVEMAP1(svclastb_s64, aarch64_sve_clastb, 4),
SVEMAP1(svclastb_s8, aarch64_sve_clastb, 1),
SVEMAP1(svclastb_u16, aarch64_sve_clastb, 2),
SVEMAP1(svclastb_u32, aarch64_sve_clastb, 3),
SVEMAP1(svclastb_u64, aarch64_sve_clastb, 4),
SVEMAP1(svclastb_u8, aarch64_sve_clastb, 1),
SVEMAP1(svcls_s16_m, aarch64_sve_cls, 258),
SVEMAP1(svcls_s16_x, aarch64_sve_cls, 514),
SVEMAP1(svcls_s16_z, aarch64_sve_cls, 642),
SVEMAP1(svcls_s32_m, aarch64_sve_cls, 259),
SVEMAP1(svcls_s32_x, aarch64_sve_cls, 515),
SVEMAP1(svcls_s32_z, aarch64_sve_cls, 643),
SVEMAP1(svcls_s64_m, aarch64_sve_cls, 260),
SVEMAP1(svcls_s64_x, aarch64_sve_cls, 516),
SVEMAP1(svcls_s64_z, aarch64_sve_cls, 644),
SVEMAP1(svcls_s8_m, aarch64_sve_cls, 257),
SVEMAP1(svcls_s8_x, aarch64_sve_cls, 513),
SVEMAP1(svcls_s8_z, aarch64_sve_cls, 641),
SVEMAP1(svclz_s16_m, aarch64_sve_clz, 258),
SVEMAP1(svclz_s16_x, aarch64_sve_clz, 514),
SVEMAP1(svclz_s16_z, aarch64_sve_clz, 642),
SVEMAP1(svclz_s32_m, aarch64_sve_clz, 259),
SVEMAP1(svclz_s32_x, aarch64_sve_clz, 515),
SVEMAP1(svclz_s32_z, aarch64_sve_clz, 643),
SVEMAP1(svclz_s64_m, aarch64_sve_clz, 260),
SVEMAP1(svclz_s64_x, aarch64_sve_clz, 516),
SVEMAP1(svclz_s64_z, aarch64_sve_clz, 644),
SVEMAP1(svclz_s8_m, aarch64_sve_clz, 257),
SVEMAP1(svclz_s8_x, aarch64_sve_clz, 513),
SVEMAP1(svclz_s8_z, aarch64_sve_clz, 641),
SVEMAP1(svclz_u16_m, aarch64_sve_clz, 258),
SVEMAP1(svclz_u16_x, aarch64_sve_clz, 514),
SVEMAP1(svclz_u16_z, aarch64_sve_clz, 642),
SVEMAP1(svclz_u32_m, aarch64_sve_clz, 259),
SVEMAP1(svclz_u32_x, aarch64_sve_clz, 515),
SVEMAP1(svclz_u32_z, aarch64_sve_clz, 643),
SVEMAP1(svclz_u64_m, aarch64_sve_clz, 260),
SVEMAP1(svclz_u64_x, aarch64_sve_clz, 516),
SVEMAP1(svclz_u64_z, aarch64_sve_clz, 644),
SVEMAP1(svclz_u8_m, aarch64_sve_clz, 257),
SVEMAP1(svclz_u8_x, aarch64_sve_clz, 513),
SVEMAP1(svclz_u8_z, aarch64_sve_clz, 641),
SVEMAP1(svcmla_f16_m, aarch64_sve_fcmla, 261),
SVEMAP1(svcmla_f16_x, aarch64_sve_fcmla, 133),
SVEMAP1(svcmla_f16_z, aarch64_sve_fcmla, 389),
SVEMAP1(svcmla_f32_m, aarch64_sve_fcmla, 262),
SVEMAP1(svcmla_f32_x, aarch64_sve_fcmla, 134),
SVEMAP1(svcmla_f32_z, aarch64_sve_fcmla, 390),
SVEMAP1(svcmla_f64_m, aarch64_sve_fcmla, 263),
SVEMAP1(svcmla_f64_x, aarch64_sve_fcmla, 135),
SVEMAP1(svcmla_f64_z, aarch64_sve_fcmla, 391),
SVEMAP1(svcmla_lane_f16, aarch64_sve_fcmla_lane, 5),
SVEMAP1(svcmla_lane_f32, aarch64_sve_fcmla_lane, 6),
SVEMAP1(svcmla_lane_s16, aarch64_sve_cmla_lane_x, 2),
SVEMAP1(svcmla_lane_s32, aarch64_sve_cmla_lane_x, 3),
SVEMAP1(svcmla_lane_u16, aarch64_sve_cmla_lane_x, 2),
SVEMAP1(svcmla_lane_u32, aarch64_sve_cmla_lane_x, 3),
SVEMAP1(svcmla_s16, aarch64_sve_cmla_x, 2),
SVEMAP1(svcmla_s32, aarch64_sve_cmla_x, 3),
SVEMAP1(svcmla_s64, aarch64_sve_cmla_x, 4),
SVEMAP1(svcmla_s8, aarch64_sve_cmla_x, 1),
SVEMAP1(svcmla_u16, aarch64_sve_cmla_x, 2),
SVEMAP1(svcmla_u32, aarch64_sve_cmla_x, 3),
SVEMAP1(svcmla_u64, aarch64_sve_cmla_x, 4),
SVEMAP1(svcmla_u8, aarch64_sve_cmla_x, 1),
SVEMAP1(svcmpeq_f16, aarch64_sve_fcmpeq, 5),
SVEMAP1(svcmpeq_f32, aarch64_sve_fcmpeq, 6),
SVEMAP1(svcmpeq_f64, aarch64_sve_fcmpeq, 7),
SVEMAP1(svcmpeq_n_f16, aarch64_sve_fcmpeq, 3077),
SVEMAP1(svcmpeq_n_f32, aarch64_sve_fcmpeq, 3078),
SVEMAP1(svcmpeq_n_f64, aarch64_sve_fcmpeq, 3079),
SVEMAP1(svcmpeq_n_s16, aarch64_sve_cmpeq, 3074),
SVEMAP1(svcmpeq_n_s32, aarch64_sve_cmpeq, 3075),
SVEMAP1(svcmpeq_n_s64, aarch64_sve_cmpeq, 3076),
SVEMAP1(svcmpeq_n_s8, aarch64_sve_cmpeq, 3073),
SVEMAP1(svcmpeq_n_u16, aarch64_sve_cmpeq, 3074),
SVEMAP1(svcmpeq_n_u32, aarch64_sve_cmpeq, 3075),
SVEMAP1(svcmpeq_n_u64, aarch64_sve_cmpeq, 3076),
SVEMAP1(svcmpeq_n_u8, aarch64_sve_cmpeq, 3073),
SVEMAP1(svcmpeq_s16, aarch64_sve_cmpeq, 2),
SVEMAP1(svcmpeq_s32, aarch64_sve_cmpeq, 3),
SVEMAP1(svcmpeq_s64, aarch64_sve_cmpeq, 4),
SVEMAP1(svcmpeq_s8, aarch64_sve_cmpeq, 1),
SVEMAP1(svcmpeq_u16, aarch64_sve_cmpeq, 2),
SVEMAP1(svcmpeq_u32, aarch64_sve_cmpeq, 3),
SVEMAP1(svcmpeq_u64, aarch64_sve_cmpeq, 4),
SVEMAP1(svcmpeq_u8, aarch64_sve_cmpeq, 1),
SVEMAP1(svcmpeq_wide_n_s16, aarch64_sve_cmpeq_wide, 3074),
SVEMAP1(svcmpeq_wide_n_s32, aarch64_sve_cmpeq_wide, 3075),
SVEMAP1(svcmpeq_wide_n_s8, aarch64_sve_cmpeq_wide, 3073),
SVEMAP1(svcmpeq_wide_s16, aarch64_sve_cmpeq_wide, 2),
SVEMAP1(svcmpeq_wide_s32, aarch64_sve_cmpeq_wide, 3),
SVEMAP1(svcmpeq_wide_s8, aarch64_sve_cmpeq_wide, 1),
SVEMAP1(svcmpge_f16, aarch64_sve_fcmpge, 5),
SVEMAP1(svcmpge_f32, aarch64_sve_fcmpge, 6),
SVEMAP1(svcmpge_f64, aarch64_sve_fcmpge, 7),
SVEMAP1(svcmpge_n_f16, aarch64_sve_fcmpge, 3077),
SVEMAP1(svcmpge_n_f32, aarch64_sve_fcmpge, 3078),
SVEMAP1(svcmpge_n_f64, aarch64_sve_fcmpge, 3079),
SVEMAP1(svcmpge_n_s16, aarch64_sve_cmpge, 3074),
SVEMAP1(svcmpge_n_s32, aarch64_sve_cmpge, 3075),
SVEMAP1(svcmpge_n_s64, aarch64_sve_cmpge, 3076),
SVEMAP1(svcmpge_n_s8, aarch64_sve_cmpge, 3073),
SVEMAP1(svcmpge_n_u16, aarch64_sve_cmphs, 3074),
SVEMAP1(svcmpge_n_u32, aarch64_sve_cmphs, 3075),
SVEMAP1(svcmpge_n_u64, aarch64_sve_cmphs, 3076),
SVEMAP1(svcmpge_n_u8, aarch64_sve_cmphs, 3073),
SVEMAP1(svcmpge_s16, aarch64_sve_cmpge, 2),
SVEMAP1(svcmpge_s32, aarch64_sve_cmpge, 3),
SVEMAP1(svcmpge_s64, aarch64_sve_cmpge, 4),
SVEMAP1(svcmpge_s8, aarch64_sve_cmpge, 1),
SVEMAP1(svcmpge_u16, aarch64_sve_cmphs, 2),
SVEMAP1(svcmpge_u32, aarch64_sve_cmphs, 3),
SVEMAP1(svcmpge_u64, aarch64_sve_cmphs, 4),
SVEMAP1(svcmpge_u8, aarch64_sve_cmphs, 1),
SVEMAP1(svcmpge_wide_n_s16, aarch64_sve_cmpge_wide, 3074),
SVEMAP1(svcmpge_wide_n_s32, aarch64_sve_cmpge_wide, 3075),
SVEMAP1(svcmpge_wide_n_s8, aarch64_sve_cmpge_wide, 3073),
SVEMAP1(svcmpge_wide_n_u16, aarch64_sve_cmphs_wide, 3074),
SVEMAP1(svcmpge_wide_n_u32, aarch64_sve_cmphs_wide, 3075),
SVEMAP1(svcmpge_wide_n_u8, aarch64_sve_cmphs_wide, 3073),
SVEMAP1(svcmpge_wide_s16, aarch64_sve_cmpge_wide, 2),
SVEMAP1(svcmpge_wide_s32, aarch64_sve_cmpge_wide, 3),
SVEMAP1(svcmpge_wide_s8, aarch64_sve_cmpge_wide, 1),
SVEMAP1(svcmpge_wide_u16, aarch64_sve_cmphs_wide, 2),
SVEMAP1(svcmpge_wide_u32, aarch64_sve_cmphs_wide, 3),
SVEMAP1(svcmpge_wide_u8, aarch64_sve_cmphs_wide, 1),
SVEMAP1(svcmpgt_f16, aarch64_sve_fcmpgt, 5),
SVEMAP1(svcmpgt_f32, aarch64_sve_fcmpgt, 6),
SVEMAP1(svcmpgt_f64, aarch64_sve_fcmpgt, 7),
SVEMAP1(svcmpgt_n_f16, aarch64_sve_fcmpgt, 3077),
SVEMAP1(svcmpgt_n_f32, aarch64_sve_fcmpgt, 3078),
SVEMAP1(svcmpgt_n_f64, aarch64_sve_fcmpgt, 3079),
SVEMAP1(svcmpgt_n_s16, aarch64_sve_cmpgt, 3074),
SVEMAP1(svcmpgt_n_s32, aarch64_sve_cmpgt, 3075),
SVEMAP1(svcmpgt_n_s64, aarch64_sve_cmpgt, 3076),
SVEMAP1(svcmpgt_n_s8, aarch64_sve_cmpgt, 3073),
SVEMAP1(svcmpgt_n_u16, aarch64_sve_cmphi, 3074),
SVEMAP1(svcmpgt_n_u32, aarch64_sve_cmphi, 3075),
SVEMAP1(svcmpgt_n_u64, aarch64_sve_cmphi, 3076),
SVEMAP1(svcmpgt_n_u8, aarch64_sve_cmphi, 3073),
SVEMAP1(svcmpgt_s16, aarch64_sve_cmpgt, 2),
SVEMAP1(svcmpgt_s32, aarch64_sve_cmpgt, 3),
SVEMAP1(svcmpgt_s64, aarch64_sve_cmpgt, 4),
SVEMAP1(svcmpgt_s8, aarch64_sve_cmpgt, 1),
SVEMAP1(svcmpgt_u16, aarch64_sve_cmphi, 2),
SVEMAP1(svcmpgt_u32, aarch64_sve_cmphi, 3),
SVEMAP1(svcmpgt_u64, aarch64_sve_cmphi, 4),
SVEMAP1(svcmpgt_u8, aarch64_sve_cmphi, 1),
SVEMAP1(svcmpgt_wide_n_s16, aarch64_sve_cmpgt_wide, 3074),
SVEMAP1(svcmpgt_wide_n_s32, aarch64_sve_cmpgt_wide, 3075),
SVEMAP1(svcmpgt_wide_n_s8, aarch64_sve_cmpgt_wide, 3073),
SVEMAP1(svcmpgt_wide_n_u16, aarch64_sve_cmphi_wide, 3074),
SVEMAP1(svcmpgt_wide_n_u32, aarch64_sve_cmphi_wide, 3075),
SVEMAP1(svcmpgt_wide_n_u8, aarch64_sve_cmphi_wide, 3073),
SVEMAP1(svcmpgt_wide_s16, aarch64_sve_cmpgt_wide, 2),
SVEMAP1(svcmpgt_wide_s32, aarch64_sve_cmpgt_wide, 3),
SVEMAP1(svcmpgt_wide_s8, aarch64_sve_cmpgt_wide, 1),
SVEMAP1(svcmpgt_wide_u16, aarch64_sve_cmphi_wide, 2),
SVEMAP1(svcmpgt_wide_u32, aarch64_sve_cmphi_wide, 3),
SVEMAP1(svcmpgt_wide_u8, aarch64_sve_cmphi_wide, 1),
SVEMAP1(svcmple_f16, aarch64_sve_fcmpge, 536870917),
SVEMAP1(svcmple_f32, aarch64_sve_fcmpge, 536870918),
SVEMAP1(svcmple_f64, aarch64_sve_fcmpge, 536870919),
SVEMAP1(svcmple_n_f16, aarch64_sve_fcmpge, 536873989),
SVEMAP1(svcmple_n_f32, aarch64_sve_fcmpge, 536873990),
SVEMAP1(svcmple_n_f64, aarch64_sve_fcmpge, 536873991),
SVEMAP1(svcmple_n_s16, aarch64_sve_cmpge, 536873986),
SVEMAP1(svcmple_n_s32, aarch64_sve_cmpge, 536873987),
SVEMAP1(svcmple_n_s64, aarch64_sve_cmpge, 536873988),
SVEMAP1(svcmple_n_s8, aarch64_sve_cmpge, 536873985),
SVEMAP1(svcmple_n_u16, aarch64_sve_cmphs, 536873986),
SVEMAP1(svcmple_n_u32, aarch64_sve_cmphs, 536873987),
SVEMAP1(svcmple_n_u64, aarch64_sve_cmphs, 536873988),
SVEMAP1(svcmple_n_u8, aarch64_sve_cmphs, 536873985),
SVEMAP1(svcmple_s16, aarch64_sve_cmpge, 536870914),
SVEMAP1(svcmple_s32, aarch64_sve_cmpge, 536870915),
SVEMAP1(svcmple_s64, aarch64_sve_cmpge, 536870916),
SVEMAP1(svcmple_s8, aarch64_sve_cmpge, 536870913),
SVEMAP1(svcmple_u16, aarch64_sve_cmphs, 536870914),
SVEMAP1(svcmple_u32, aarch64_sve_cmphs, 536870915),
SVEMAP1(svcmple_u64, aarch64_sve_cmphs, 536870916),
SVEMAP1(svcmple_u8, aarch64_sve_cmphs, 536870913),
SVEMAP1(svcmple_wide_n_s16, aarch64_sve_cmple_wide, 3074),
SVEMAP1(svcmple_wide_n_s32, aarch64_sve_cmple_wide, 3075),
SVEMAP1(svcmple_wide_n_s8, aarch64_sve_cmple_wide, 3073),
SVEMAP1(svcmple_wide_n_u16, aarch64_sve_cmpls_wide, 3074),
SVEMAP1(svcmple_wide_n_u32, aarch64_sve_cmpls_wide, 3075),
SVEMAP1(svcmple_wide_n_u8, aarch64_sve_cmpls_wide, 3073),
SVEMAP1(svcmple_wide_s16, aarch64_sve_cmple_wide, 2),
SVEMAP1(svcmple_wide_s32, aarch64_sve_cmple_wide, 3),
SVEMAP1(svcmple_wide_s8, aarch64_sve_cmple_wide, 1),
SVEMAP1(svcmple_wide_u16, aarch64_sve_cmpls_wide, 2),
SVEMAP1(svcmple_wide_u32, aarch64_sve_cmpls_wide, 3),
SVEMAP1(svcmple_wide_u8, aarch64_sve_cmpls_wide, 1),
SVEMAP1(svcmplt_f16, aarch64_sve_fcmpgt, 536870917),
SVEMAP1(svcmplt_f32, aarch64_sve_fcmpgt, 536870918),
SVEMAP1(svcmplt_f64, aarch64_sve_fcmpgt, 536870919),
SVEMAP1(svcmplt_n_f16, aarch64_sve_fcmpgt, 536873989),
SVEMAP1(svcmplt_n_f32, aarch64_sve_fcmpgt, 536873990),
SVEMAP1(svcmplt_n_f64, aarch64_sve_fcmpgt, 536873991),
SVEMAP1(svcmplt_n_s16, aarch64_sve_cmpgt, 536873986),
SVEMAP1(svcmplt_n_s32, aarch64_sve_cmpgt, 536873987),
SVEMAP1(svcmplt_n_s64, aarch64_sve_cmpgt, 536873988),
SVEMAP1(svcmplt_n_s8, aarch64_sve_cmpgt, 536873985),
SVEMAP1(svcmplt_n_u16, aarch64_sve_cmphi, 536873986),
SVEMAP1(svcmplt_n_u32, aarch64_sve_cmphi, 536873987),
SVEMAP1(svcmplt_n_u64, aarch64_sve_cmphi, 536873988),
SVEMAP1(svcmplt_n_u8, aarch64_sve_cmphi, 536873985),
SVEMAP1(svcmplt_s16, aarch64_sve_cmpgt, 536870914),
SVEMAP1(svcmplt_s32, aarch64_sve_cmpgt, 536870915),
SVEMAP1(svcmplt_s64, aarch64_sve_cmpgt, 536870916),
SVEMAP1(svcmplt_s8, aarch64_sve_cmpgt, 536870913),
SVEMAP1(svcmplt_u16, aarch64_sve_cmphi, 536870914),
SVEMAP1(svcmplt_u32, aarch64_sve_cmphi, 536870915),
SVEMAP1(svcmplt_u64, aarch64_sve_cmphi, 536870916),
SVEMAP1(svcmplt_u8, aarch64_sve_cmphi, 536870913),
SVEMAP1(svcmplt_wide_n_s16, aarch64_sve_cmplt_wide, 3074),
SVEMAP1(svcmplt_wide_n_s32, aarch64_sve_cmplt_wide, 3075),
SVEMAP1(svcmplt_wide_n_s8, aarch64_sve_cmplt_wide, 3073),
SVEMAP1(svcmplt_wide_n_u16, aarch64_sve_cmplo_wide, 3074),
SVEMAP1(svcmplt_wide_n_u32, aarch64_sve_cmplo_wide, 3075),
SVEMAP1(svcmplt_wide_n_u8, aarch64_sve_cmplo_wide, 3073),
SVEMAP1(svcmplt_wide_s16, aarch64_sve_cmplt_wide, 2),
SVEMAP1(svcmplt_wide_s32, aarch64_sve_cmplt_wide, 3),
SVEMAP1(svcmplt_wide_s8, aarch64_sve_cmplt_wide, 1),
SVEMAP1(svcmplt_wide_u16, aarch64_sve_cmplo_wide, 2),
SVEMAP1(svcmplt_wide_u32, aarch64_sve_cmplo_wide, 3),
SVEMAP1(svcmplt_wide_u8, aarch64_sve_cmplo_wide, 1),
SVEMAP1(svcmpne_f16, aarch64_sve_fcmpne, 5),
SVEMAP1(svcmpne_f32, aarch64_sve_fcmpne, 6),
SVEMAP1(svcmpne_f64, aarch64_sve_fcmpne, 7),
SVEMAP1(svcmpne_n_f16, aarch64_sve_fcmpne, 3077),
SVEMAP1(svcmpne_n_f32, aarch64_sve_fcmpne, 3078),
SVEMAP1(svcmpne_n_f64, aarch64_sve_fcmpne, 3079),
SVEMAP1(svcmpne_n_s16, aarch64_sve_cmpne, 3074),
SVEMAP1(svcmpne_n_s32, aarch64_sve_cmpne, 3075),
SVEMAP1(svcmpne_n_s64, aarch64_sve_cmpne, 3076),
SVEMAP1(svcmpne_n_s8, aarch64_sve_cmpne, 3073),
SVEMAP1(svcmpne_n_u16, aarch64_sve_cmpne, 3074),
SVEMAP1(svcmpne_n_u32, aarch64_sve_cmpne, 3075),
SVEMAP1(svcmpne_n_u64, aarch64_sve_cmpne, 3076),
SVEMAP1(svcmpne_n_u8, aarch64_sve_cmpne, 3073),
SVEMAP1(svcmpne_s16, aarch64_sve_cmpne, 2),
SVEMAP1(svcmpne_s32, aarch64_sve_cmpne, 3),
SVEMAP1(svcmpne_s64, aarch64_sve_cmpne, 4),
SVEMAP1(svcmpne_s8, aarch64_sve_cmpne, 1),
SVEMAP1(svcmpne_u16, aarch64_sve_cmpne, 2),
SVEMAP1(svcmpne_u32, aarch64_sve_cmpne, 3),
SVEMAP1(svcmpne_u64, aarch64_sve_cmpne, 4),
SVEMAP1(svcmpne_u8, aarch64_sve_cmpne, 1),
SVEMAP1(svcmpne_wide_n_s16, aarch64_sve_cmpne_wide, 3074),
SVEMAP1(svcmpne_wide_n_s32, aarch64_sve_cmpne_wide, 3075),
SVEMAP1(svcmpne_wide_n_s8, aarch64_sve_cmpne_wide, 3073),
SVEMAP1(svcmpne_wide_s16, aarch64_sve_cmpne_wide, 2),
SVEMAP1(svcmpne_wide_s32, aarch64_sve_cmpne_wide, 3),
SVEMAP1(svcmpne_wide_s8, aarch64_sve_cmpne_wide, 1),
SVEMAP1(svcmpuo_f16, aarch64_sve_fcmpuo, 5),
SVEMAP1(svcmpuo_f32, aarch64_sve_fcmpuo, 6),
SVEMAP1(svcmpuo_f64, aarch64_sve_fcmpuo, 7),
SVEMAP1(svcmpuo_n_f16, aarch64_sve_fcmpuo, 3077),
SVEMAP1(svcmpuo_n_f32, aarch64_sve_fcmpuo, 3078),
SVEMAP1(svcmpuo_n_f64, aarch64_sve_fcmpuo, 3079),
SVEMAP1(svcnot_s16_m, aarch64_sve_cnot, 258),
SVEMAP1(svcnot_s16_x, aarch64_sve_cnot, 514),
SVEMAP1(svcnot_s16_z, aarch64_sve_cnot, 642),
SVEMAP1(svcnot_s32_m, aarch64_sve_cnot, 259),
SVEMAP1(svcnot_s32_x, aarch64_sve_cnot, 515),
SVEMAP1(svcnot_s32_z, aarch64_sve_cnot, 643),
SVEMAP1(svcnot_s64_m, aarch64_sve_cnot, 260),
SVEMAP1(svcnot_s64_x, aarch64_sve_cnot, 516),
SVEMAP1(svcnot_s64_z, aarch64_sve_cnot, 644),
SVEMAP1(svcnot_s8_m, aarch64_sve_cnot, 257),
SVEMAP1(svcnot_s8_x, aarch64_sve_cnot, 513),
SVEMAP1(svcnot_s8_z, aarch64_sve_cnot, 641),
SVEMAP1(svcnot_u16_m, aarch64_sve_cnot, 258),
SVEMAP1(svcnot_u16_x, aarch64_sve_cnot, 514),
SVEMAP1(svcnot_u16_z, aarch64_sve_cnot, 642),
SVEMAP1(svcnot_u32_m, aarch64_sve_cnot, 259),
SVEMAP1(svcnot_u32_x, aarch64_sve_cnot, 515),
SVEMAP1(svcnot_u32_z, aarch64_sve_cnot, 643),
SVEMAP1(svcnot_u64_m, aarch64_sve_cnot, 260),
SVEMAP1(svcnot_u64_x, aarch64_sve_cnot, 516),
SVEMAP1(svcnot_u64_z, aarch64_sve_cnot, 644),
SVEMAP1(svcnot_u8_m, aarch64_sve_cnot, 257),
SVEMAP1(svcnot_u8_x, aarch64_sve_cnot, 513),
SVEMAP1(svcnot_u8_z, aarch64_sve_cnot, 641),
SVEMAP1(svcnt_bf16_m, aarch64_sve_cnt, 268),
SVEMAP1(svcnt_bf16_x, aarch64_sve_cnt, 524),
SVEMAP1(svcnt_bf16_z, aarch64_sve_cnt, 652),
SVEMAP1(svcnt_f16_m, aarch64_sve_cnt, 261),
SVEMAP1(svcnt_f16_x, aarch64_sve_cnt, 517),
SVEMAP1(svcnt_f16_z, aarch64_sve_cnt, 645),
SVEMAP1(svcnt_f32_m, aarch64_sve_cnt, 262),
SVEMAP1(svcnt_f32_x, aarch64_sve_cnt, 518),
SVEMAP1(svcnt_f32_z, aarch64_sve_cnt, 646),
SVEMAP1(svcnt_f64_m, aarch64_sve_cnt, 263),
SVEMAP1(svcnt_f64_x, aarch64_sve_cnt, 519),
SVEMAP1(svcnt_f64_z, aarch64_sve_cnt, 647),
SVEMAP1(svcnt_s16_m, aarch64_sve_cnt, 258),
SVEMAP1(svcnt_s16_x, aarch64_sve_cnt, 514),
SVEMAP1(svcnt_s16_z, aarch64_sve_cnt, 642),
SVEMAP1(svcnt_s32_m, aarch64_sve_cnt, 259),
SVEMAP1(svcnt_s32_x, aarch64_sve_cnt, 515),
SVEMAP1(svcnt_s32_z, aarch64_sve_cnt, 643),
SVEMAP1(svcnt_s64_m, aarch64_sve_cnt, 260),
SVEMAP1(svcnt_s64_x, aarch64_sve_cnt, 516),
SVEMAP1(svcnt_s64_z, aarch64_sve_cnt, 644),
SVEMAP1(svcnt_s8_m, aarch64_sve_cnt, 257),
SVEMAP1(svcnt_s8_x, aarch64_sve_cnt, 513),
SVEMAP1(svcnt_s8_z, aarch64_sve_cnt, 641),
SVEMAP1(svcnt_u16_m, aarch64_sve_cnt, 258),
SVEMAP1(svcnt_u16_x, aarch64_sve_cnt, 514),
SVEMAP1(svcnt_u16_z, aarch64_sve_cnt, 642),
SVEMAP1(svcnt_u32_m, aarch64_sve_cnt, 259),
SVEMAP1(svcnt_u32_x, aarch64_sve_cnt, 515),
SVEMAP1(svcnt_u32_z, aarch64_sve_cnt, 643),
SVEMAP1(svcnt_u64_m, aarch64_sve_cnt, 260),
SVEMAP1(svcnt_u64_x, aarch64_sve_cnt, 516),
SVEMAP1(svcnt_u64_z, aarch64_sve_cnt, 644),
SVEMAP1(svcnt_u8_m, aarch64_sve_cnt, 257),
SVEMAP1(svcnt_u8_x, aarch64_sve_cnt, 513),
SVEMAP1(svcnt_u8_z, aarch64_sve_cnt, 641),
SVEMAP1(svcntb, aarch64_sve_cntb, 34603011),
SVEMAP1(svcntb_pat, aarch64_sve_cntb, 1048579),
SVEMAP1(svcntd, aarch64_sve_cntd, 34603011),
SVEMAP1(svcntd_pat, aarch64_sve_cntd, 1048579),
SVEMAP1(svcnth, aarch64_sve_cnth, 34603011),
SVEMAP1(svcnth_pat, aarch64_sve_cnth, 1048579),
SVEMAP1(svcntp_b16, aarch64_sve_cntp, 9),
SVEMAP1(svcntp_b32, aarch64_sve_cntp, 10),
SVEMAP1(svcntp_b64, aarch64_sve_cntp, 11),
SVEMAP1(svcntp_b8, aarch64_sve_cntp, 8),
SVEMAP1(svcntw, aarch64_sve_cntw, 34603011),
SVEMAP1(svcntw_pat, aarch64_sve_cntw, 1048579),
SVEMAP1(svcompact_f32, aarch64_sve_compact, 6),
SVEMAP1(svcompact_f64, aarch64_sve_compact, 7),
SVEMAP1(svcompact_s32, aarch64_sve_compact, 3),
SVEMAP1(svcompact_s64, aarch64_sve_compact, 4),
SVEMAP1(svcompact_u32, aarch64_sve_compact, 3),
SVEMAP1(svcompact_u64, aarch64_sve_compact, 4),
SVEMAP1(svcreate2_bf16, aarch64_sve_tuple_create2, 4294967308),
SVEMAP1(svcreate2_f16, aarch64_sve_tuple_create2, 4294967301),
SVEMAP1(svcreate2_f32, aarch64_sve_tuple_create2, 4294967302),
SVEMAP1(svcreate2_f64, aarch64_sve_tuple_create2, 4294967303),
SVEMAP1(svcreate2_s16, aarch64_sve_tuple_create2, 4294967298),
SVEMAP1(svcreate2_s32, aarch64_sve_tuple_create2, 4294967299),
SVEMAP1(svcreate2_s64, aarch64_sve_tuple_create2, 4294967300),
SVEMAP1(svcreate2_s8, aarch64_sve_tuple_create2, 4294967297),
SVEMAP1(svcreate2_u16, aarch64_sve_tuple_create2, 4294967298),
SVEMAP1(svcreate2_u32, aarch64_sve_tuple_create2, 4294967299),
SVEMAP1(svcreate2_u64, aarch64_sve_tuple_create2, 4294967300),
SVEMAP1(svcreate2_u8, aarch64_sve_tuple_create2, 4294967297),
SVEMAP1(svcreate3_bf16, aarch64_sve_tuple_create3, 4294967308),
SVEMAP1(svcreate3_f16, aarch64_sve_tuple_create3, 4294967301),
SVEMAP1(svcreate3_f32, aarch64_sve_tuple_create3, 4294967302),
SVEMAP1(svcreate3_f64, aarch64_sve_tuple_create3, 4294967303),
SVEMAP1(svcreate3_s16, aarch64_sve_tuple_create3, 4294967298),
SVEMAP1(svcreate3_s32, aarch64_sve_tuple_create3, 4294967299),
SVEMAP1(svcreate3_s64, aarch64_sve_tuple_create3, 4294967300),
SVEMAP1(svcreate3_s8, aarch64_sve_tuple_create3, 4294967297),
SVEMAP1(svcreate3_u16, aarch64_sve_tuple_create3, 4294967298),
SVEMAP1(svcreate3_u32, aarch64_sve_tuple_create3, 4294967299),
SVEMAP1(svcreate3_u64, aarch64_sve_tuple_create3, 4294967300),
SVEMAP1(svcreate3_u8, aarch64_sve_tuple_create3, 4294967297),
SVEMAP1(svcreate4_bf16, aarch64_sve_tuple_create4, 4294967308),
SVEMAP1(svcreate4_f16, aarch64_sve_tuple_create4, 4294967301),
SVEMAP1(svcreate4_f32, aarch64_sve_tuple_create4, 4294967302),
SVEMAP1(svcreate4_f64, aarch64_sve_tuple_create4, 4294967303),
SVEMAP1(svcreate4_s16, aarch64_sve_tuple_create4, 4294967298),
SVEMAP1(svcreate4_s32, aarch64_sve_tuple_create4, 4294967299),
SVEMAP1(svcreate4_s64, aarch64_sve_tuple_create4, 4294967300),
SVEMAP1(svcreate4_s8, aarch64_sve_tuple_create4, 4294967297),
SVEMAP1(svcreate4_u16, aarch64_sve_tuple_create4, 4294967298),
SVEMAP1(svcreate4_u32, aarch64_sve_tuple_create4, 4294967299),
SVEMAP1(svcreate4_u64, aarch64_sve_tuple_create4, 4294967300),
SVEMAP1(svcreate4_u8, aarch64_sve_tuple_create4, 4294967297),
SVEMAP1(svcvt_bf16_f32_m, aarch64_sve_fcvt_bf16f32, 1048844),
SVEMAP1(svcvt_bf16_f32_x, aarch64_sve_fcvt_bf16f32, 1049100),
SVEMAP1(svcvt_bf16_f32_z, aarch64_sve_fcvt_bf16f32, 1049228),
SVEMAP1(svcvt_f16_f32_m, aarch64_sve_fcvt_f16f32, 1048838),
SVEMAP1(svcvt_f16_f32_x, aarch64_sve_fcvt_f16f32, 1049094),
SVEMAP1(svcvt_f16_f32_z, aarch64_sve_fcvt_f16f32, 1049222),
SVEMAP1(svcvt_f16_f64_m, aarch64_sve_fcvt_f16f64, 1048839),
SVEMAP1(svcvt_f16_f64_x, aarch64_sve_fcvt_f16f64, 1049095),
SVEMAP1(svcvt_f16_f64_z, aarch64_sve_fcvt_f16f64, 1049223),
SVEMAP1(svcvt_f16_s16_m, aarch64_sve_scvtf, 8388866),
SVEMAP1(svcvt_f16_s16_x, aarch64_sve_scvtf, 8389122),
SVEMAP1(svcvt_f16_s16_z, aarch64_sve_scvtf, 8389250),
SVEMAP1(svcvt_f16_s32_m, aarch64_sve_scvtf_f16i32, 1048835),
SVEMAP1(svcvt_f16_s32_x, aarch64_sve_scvtf_f16i32, 1049091),
SVEMAP1(svcvt_f16_s32_z, aarch64_sve_scvtf_f16i32, 1049219),
SVEMAP1(svcvt_f16_s64_m, aarch64_sve_scvtf_f16i64, 1048836),
SVEMAP1(svcvt_f16_s64_x, aarch64_sve_scvtf_f16i64, 1049092),
SVEMAP1(svcvt_f16_s64_z, aarch64_sve_scvtf_f16i64, 1049220),
SVEMAP1(svcvt_f16_u16_m, aarch64_sve_ucvtf, 8388866),
SVEMAP1(svcvt_f16_u16_x, aarch64_sve_ucvtf, 8389122),
SVEMAP1(svcvt_f16_u16_z, aarch64_sve_ucvtf, 8389250),
SVEMAP1(svcvt_f16_u32_m, aarch64_sve_ucvtf_f16i32, 1048835),
SVEMAP1(svcvt_f16_u32_x, aarch64_sve_ucvtf_f16i32, 1049091),
SVEMAP1(svcvt_f16_u32_z, aarch64_sve_ucvtf_f16i32, 1049219),
SVEMAP1(svcvt_f16_u64_m, aarch64_sve_ucvtf_f16i64, 1048836),
SVEMAP1(svcvt_f16_u64_x, aarch64_sve_ucvtf_f16i64, 1049092),
SVEMAP1(svcvt_f16_u64_z, aarch64_sve_ucvtf_f16i64, 1049220),
SVEMAP1(svcvt_f32_f16_m, aarch64_sve_fcvt_f32f16, 1048838),
SVEMAP1(svcvt_f32_f16_x, aarch64_sve_fcvt_f32f16, 1049094),
SVEMAP1(svcvt_f32_f16_z, aarch64_sve_fcvt_f32f16, 1049222),
SVEMAP1(svcvt_f32_f64_m, aarch64_sve_fcvt_f32f64, 1048839),
SVEMAP1(svcvt_f32_f64_x, aarch64_sve_fcvt_f32f64, 1049095),
SVEMAP1(svcvt_f32_f64_z, aarch64_sve_fcvt_f32f64, 1049223),
SVEMAP1(svcvt_f32_s32_m, aarch64_sve_scvtf, 8388867),
SVEMAP1(svcvt_f32_s32_x, aarch64_sve_scvtf, 8389123),
SVEMAP1(svcvt_f32_s32_z, aarch64_sve_scvtf, 8389251),
SVEMAP1(svcvt_f32_s64_m, aarch64_sve_scvtf_f32i64, 1048836),
SVEMAP1(svcvt_f32_s64_x, aarch64_sve_scvtf_f32i64, 1049092),
SVEMAP1(svcvt_f32_s64_z, aarch64_sve_scvtf_f32i64, 1049220),
SVEMAP1(svcvt_f32_u32_m, aarch64_sve_ucvtf, 8388867),
SVEMAP1(svcvt_f32_u32_x, aarch64_sve_ucvtf, 8389123),
SVEMAP1(svcvt_f32_u32_z, aarch64_sve_ucvtf, 8389251),
SVEMAP1(svcvt_f32_u64_m, aarch64_sve_ucvtf_f32i64, 1048836),
SVEMAP1(svcvt_f32_u64_x, aarch64_sve_ucvtf_f32i64, 1049092),
SVEMAP1(svcvt_f32_u64_z, aarch64_sve_ucvtf_f32i64, 1049220),
SVEMAP1(svcvt_f64_f16_m, aarch64_sve_fcvt_f64f16, 1048839),
SVEMAP1(svcvt_f64_f16_x, aarch64_sve_fcvt_f64f16, 1049095),
SVEMAP1(svcvt_f64_f16_z, aarch64_sve_fcvt_f64f16, 1049223),
SVEMAP1(svcvt_f64_f32_m, aarch64_sve_fcvt_f64f32, 1048839),
SVEMAP1(svcvt_f64_f32_x, aarch64_sve_fcvt_f64f32, 1049095),
SVEMAP1(svcvt_f64_f32_z, aarch64_sve_fcvt_f64f32, 1049223),
SVEMAP1(svcvt_f64_s32_m, aarch64_sve_scvtf_f64i32, 1048839),
SVEMAP1(svcvt_f64_s32_x, aarch64_sve_scvtf_f64i32, 1049095),
SVEMAP1(svcvt_f64_s32_z, aarch64_sve_scvtf_f64i32, 1049223),
SVEMAP1(svcvt_f64_s64_m, aarch64_sve_scvtf, 8388868),
SVEMAP1(svcvt_f64_s64_x, aarch64_sve_scvtf, 8389124),
SVEMAP1(svcvt_f64_s64_z, aarch64_sve_scvtf, 8389252),
SVEMAP1(svcvt_f64_u32_m, aarch64_sve_ucvtf_f64i32, 1048839),
SVEMAP1(svcvt_f64_u32_x, aarch64_sve_ucvtf_f64i32, 1049095),
SVEMAP1(svcvt_f64_u32_z, aarch64_sve_ucvtf_f64i32, 1049223),
SVEMAP1(svcvt_f64_u64_m, aarch64_sve_ucvtf, 8388868),
SVEMAP1(svcvt_f64_u64_x, aarch64_sve_ucvtf, 8389124),
SVEMAP1(svcvt_f64_u64_z, aarch64_sve_ucvtf, 8389252),
SVEMAP1(svcvt_s16_f16_m, aarch64_sve_fcvtzs, 8388866),
SVEMAP1(svcvt_s16_f16_x, aarch64_sve_fcvtzs, 8389122),
SVEMAP1(svcvt_s16_f16_z, aarch64_sve_fcvtzs, 8389250),
SVEMAP1(svcvt_s32_f16_m, aarch64_sve_fcvtzs_i32f16, 1048835),
SVEMAP1(svcvt_s32_f16_x, aarch64_sve_fcvtzs_i32f16, 1049091),
SVEMAP1(svcvt_s32_f16_z, aarch64_sve_fcvtzs_i32f16, 1049219),
SVEMAP1(svcvt_s32_f32_m, aarch64_sve_fcvtzs, 8388867),
SVEMAP1(svcvt_s32_f32_x, aarch64_sve_fcvtzs, 8389123),
SVEMAP1(svcvt_s32_f32_z, aarch64_sve_fcvtzs, 8389251),
SVEMAP1(svcvt_s32_f64_m, aarch64_sve_fcvtzs_i32f64, 1048839),
SVEMAP1(svcvt_s32_f64_x, aarch64_sve_fcvtzs_i32f64, 1049095),
SVEMAP1(svcvt_s32_f64_z, aarch64_sve_fcvtzs_i32f64, 1049223),
SVEMAP1(svcvt_s64_f16_m, aarch64_sve_fcvtzs_i64f16, 1048836),
SVEMAP1(svcvt_s64_f16_x, aarch64_sve_fcvtzs_i64f16, 1049092),
SVEMAP1(svcvt_s64_f16_z, aarch64_sve_fcvtzs_i64f16, 1049220),
SVEMAP1(svcvt_s64_f32_m, aarch64_sve_fcvtzs_i64f32, 1048836),
SVEMAP1(svcvt_s64_f32_x, aarch64_sve_fcvtzs_i64f32, 1049092),
SVEMAP1(svcvt_s64_f32_z, aarch64_sve_fcvtzs_i64f32, 1049220),
SVEMAP1(svcvt_s64_f64_m, aarch64_sve_fcvtzs, 8388868),
SVEMAP1(svcvt_s64_f64_x, aarch64_sve_fcvtzs, 8389124),
SVEMAP1(svcvt_s64_f64_z, aarch64_sve_fcvtzs, 8389252),
SVEMAP1(svcvt_u16_f16_m, aarch64_sve_fcvtzu, 8388866),
SVEMAP1(svcvt_u16_f16_x, aarch64_sve_fcvtzu, 8389122),
SVEMAP1(svcvt_u16_f16_z, aarch64_sve_fcvtzu, 8389250),
SVEMAP1(svcvt_u32_f16_m, aarch64_sve_fcvtzu_i32f16, 1048835),
SVEMAP1(svcvt_u32_f16_x, aarch64_sve_fcvtzu_i32f16, 1049091),
SVEMAP1(svcvt_u32_f16_z, aarch64_sve_fcvtzu_i32f16, 1049219),
SVEMAP1(svcvt_u32_f32_m, aarch64_sve_fcvtzu, 8388867),
SVEMAP1(svcvt_u32_f32_x, aarch64_sve_fcvtzu, 8389123),
SVEMAP1(svcvt_u32_f32_z, aarch64_sve_fcvtzu, 8389251),
SVEMAP1(svcvt_u32_f64_m, aarch64_sve_fcvtzu_i32f64, 1048839),
SVEMAP1(svcvt_u32_f64_x, aarch64_sve_fcvtzu_i32f64, 1049095),
SVEMAP1(svcvt_u32_f64_z, aarch64_sve_fcvtzu_i32f64, 1049223),
SVEMAP1(svcvt_u64_f16_m, aarch64_sve_fcvtzu_i64f16, 1048836),
SVEMAP1(svcvt_u64_f16_x, aarch64_sve_fcvtzu_i64f16, 1049092),
SVEMAP1(svcvt_u64_f16_z, aarch64_sve_fcvtzu_i64f16, 1049220),
SVEMAP1(svcvt_u64_f32_m, aarch64_sve_fcvtzu_i64f32, 1048836),
SVEMAP1(svcvt_u64_f32_x, aarch64_sve_fcvtzu_i64f32, 1049092),
SVEMAP1(svcvt_u64_f32_z, aarch64_sve_fcvtzu_i64f32, 1049220),
SVEMAP1(svcvt_u64_f64_m, aarch64_sve_fcvtzu, 8388868),
SVEMAP1(svcvt_u64_f64_x, aarch64_sve_fcvtzu, 8389124),
SVEMAP1(svcvt_u64_f64_z, aarch64_sve_fcvtzu, 8389252),
SVEMAP1(svcvtlt_f32_f16_m, aarch64_sve_fcvtlt_f32f16, 1048838),
SVEMAP1(svcvtlt_f32_f16_x, aarch64_sve_fcvtlt_f32f16, 1049094),
SVEMAP1(svcvtlt_f64_f32_m, aarch64_sve_fcvtlt_f64f32, 1048839),
SVEMAP1(svcvtlt_f64_f32_x, aarch64_sve_fcvtlt_f64f32, 1049095),
SVEMAP1(svcvtnt_bf16_f32_m, aarch64_sve_fcvtnt_bf16f32, 1048844),
SVEMAP1(svcvtnt_f16_f32_m, aarch64_sve_fcvtnt_f16f32, 1048838),
SVEMAP1(svcvtnt_f32_f64_m, aarch64_sve_fcvtnt_f32f64, 1048839),
SVEMAP1(svcvtx_f32_f64_m, aarch64_sve_fcvtx_f32f64, 1048839),
SVEMAP1(svcvtx_f32_f64_x, aarch64_sve_fcvtx_f32f64, 1049095),
SVEMAP1(svcvtx_f32_f64_z, aarch64_sve_fcvtx_f32f64, 1049223),
SVEMAP1(svcvtxnt_f32_f64_m, aarch64_sve_fcvtxnt_f32f64, 1048839),
SVEMAP1(svdiv_f16_m, aarch64_sve_fdiv, 261),
SVEMAP1(svdiv_f16_x, aarch64_sve_fdiv, 133),
SVEMAP1(svdiv_f16_z, aarch64_sve_fdiv, 389),
SVEMAP1(svdiv_f32_m, aarch64_sve_fdiv, 262),
SVEMAP1(svdiv_f32_x, aarch64_sve_fdiv, 134),
SVEMAP1(svdiv_f32_z, aarch64_sve_fdiv, 390),
SVEMAP1(svdiv_f64_m, aarch64_sve_fdiv, 263),
SVEMAP1(svdiv_f64_x, aarch64_sve_fdiv, 135),
SVEMAP1(svdiv_f64_z, aarch64_sve_fdiv, 391),
SVEMAP1(svdiv_n_f16_m, aarch64_sve_fdiv, 3333),
SVEMAP1(svdiv_n_f16_x, aarch64_sve_fdiv, 3205),
SVEMAP1(svdiv_n_f16_z, aarch64_sve_fdiv, 3461),
SVEMAP1(svdiv_n_f32_m, aarch64_sve_fdiv, 3334),
SVEMAP1(svdiv_n_f32_x, aarch64_sve_fdiv, 3206),
SVEMAP1(svdiv_n_f32_z, aarch64_sve_fdiv, 3462),
SVEMAP1(svdiv_n_f64_m, aarch64_sve_fdiv, 3335),
SVEMAP1(svdiv_n_f64_x, aarch64_sve_fdiv, 3207),
SVEMAP1(svdiv_n_f64_z, aarch64_sve_fdiv, 3463),
SVEMAP1(svdiv_n_s32_m, aarch64_sve_sdiv, 3331),
SVEMAP1(svdiv_n_s32_x, aarch64_sve_sdiv, 3203),
SVEMAP1(svdiv_n_s32_z, aarch64_sve_sdiv, 3459),
SVEMAP1(svdiv_n_s64_m, aarch64_sve_sdiv, 3332),
SVEMAP1(svdiv_n_s64_x, aarch64_sve_sdiv, 3204),
SVEMAP1(svdiv_n_s64_z, aarch64_sve_sdiv, 3460),
SVEMAP1(svdiv_n_u32_m, aarch64_sve_udiv, 3331),
SVEMAP1(svdiv_n_u32_x, aarch64_sve_udiv, 3203),
SVEMAP1(svdiv_n_u32_z, aarch64_sve_udiv, 3459),
SVEMAP1(svdiv_n_u64_m, aarch64_sve_udiv, 3332),
SVEMAP1(svdiv_n_u64_x, aarch64_sve_udiv, 3204),
SVEMAP1(svdiv_n_u64_z, aarch64_sve_udiv, 3460),
SVEMAP1(svdiv_s32_m, aarch64_sve_sdiv, 259),
SVEMAP1(svdiv_s32_x, aarch64_sve_sdiv, 131),
SVEMAP1(svdiv_s32_z, aarch64_sve_sdiv, 387),
SVEMAP1(svdiv_s64_m, aarch64_sve_sdiv, 260),
SVEMAP1(svdiv_s64_x, aarch64_sve_sdiv, 132),
SVEMAP1(svdiv_s64_z, aarch64_sve_sdiv, 388),
SVEMAP1(svdiv_u32_m, aarch64_sve_udiv, 259),
SVEMAP1(svdiv_u32_x, aarch64_sve_udiv, 131),
SVEMAP1(svdiv_u32_z, aarch64_sve_udiv, 387),
SVEMAP1(svdiv_u64_m, aarch64_sve_udiv, 260),
SVEMAP1(svdiv_u64_x, aarch64_sve_udiv, 132),
SVEMAP1(svdiv_u64_z, aarch64_sve_udiv, 388),
SVEMAP1(svdivr_f16_m, aarch64_sve_fdivr, 261),
SVEMAP1(svdivr_f16_x, aarch64_sve_fdivr, 133),
SVEMAP1(svdivr_f16_z, aarch64_sve_fdivr, 389),
SVEMAP1(svdivr_f32_m, aarch64_sve_fdivr, 262),
SVEMAP1(svdivr_f32_x, aarch64_sve_fdivr, 134),
SVEMAP1(svdivr_f32_z, aarch64_sve_fdivr, 390),
SVEMAP1(svdivr_f64_m, aarch64_sve_fdivr, 263),
SVEMAP1(svdivr_f64_x, aarch64_sve_fdivr, 135),
SVEMAP1(svdivr_f64_z, aarch64_sve_fdivr, 391),
SVEMAP1(svdivr_n_f16_m, aarch64_sve_fdivr, 3333),
SVEMAP1(svdivr_n_f16_x, aarch64_sve_fdivr, 3205),
SVEMAP1(svdivr_n_f16_z, aarch64_sve_fdivr, 3461),
SVEMAP1(svdivr_n_f32_m, aarch64_sve_fdivr, 3334),
SVEMAP1(svdivr_n_f32_x, aarch64_sve_fdivr, 3206),
SVEMAP1(svdivr_n_f32_z, aarch64_sve_fdivr, 3462),
SVEMAP1(svdivr_n_f64_m, aarch64_sve_fdivr, 3335),
SVEMAP1(svdivr_n_f64_x, aarch64_sve_fdivr, 3207),
SVEMAP1(svdivr_n_f64_z, aarch64_sve_fdivr, 3463),
SVEMAP1(svdivr_n_s32_m, aarch64_sve_sdivr, 3331),
SVEMAP1(svdivr_n_s32_x, aarch64_sve_sdivr, 3203),
SVEMAP1(svdivr_n_s32_z, aarch64_sve_sdivr, 3459),
SVEMAP1(svdivr_n_s64_m, aarch64_sve_sdivr, 3332),
SVEMAP1(svdivr_n_s64_x, aarch64_sve_sdivr, 3204),
SVEMAP1(svdivr_n_s64_z, aarch64_sve_sdivr, 3460),
SVEMAP1(svdivr_n_u32_m, aarch64_sve_udivr, 3331),
SVEMAP1(svdivr_n_u32_x, aarch64_sve_udivr, 3203),
SVEMAP1(svdivr_n_u32_z, aarch64_sve_udivr, 3459),
SVEMAP1(svdivr_n_u64_m, aarch64_sve_udivr, 3332),
SVEMAP1(svdivr_n_u64_x, aarch64_sve_udivr, 3204),
SVEMAP1(svdivr_n_u64_z, aarch64_sve_udivr, 3460),
SVEMAP1(svdivr_s32_m, aarch64_sve_sdivr, 259),
SVEMAP1(svdivr_s32_x, aarch64_sve_sdivr, 131),
SVEMAP1(svdivr_s32_z, aarch64_sve_sdivr, 387),
SVEMAP1(svdivr_s64_m, aarch64_sve_sdivr, 260),
SVEMAP1(svdivr_s64_x, aarch64_sve_sdivr, 132),
SVEMAP1(svdivr_s64_z, aarch64_sve_sdivr, 388),
SVEMAP1(svdivr_u32_m, aarch64_sve_udivr, 259),
SVEMAP1(svdivr_u32_x, aarch64_sve_udivr, 131),
SVEMAP1(svdivr_u32_z, aarch64_sve_udivr, 387),
SVEMAP1(svdivr_u64_m, aarch64_sve_udivr, 260),
SVEMAP1(svdivr_u64_x, aarch64_sve_udivr, 132),
SVEMAP1(svdivr_u64_z, aarch64_sve_udivr, 388),
SVEMAP1(svdot_lane_s32, aarch64_sve_sdot_lane, 3),
SVEMAP1(svdot_lane_s64, aarch64_sve_sdot_lane, 4),
SVEMAP1(svdot_lane_u32, aarch64_sve_udot_lane, 3),
SVEMAP1(svdot_lane_u64, aarch64_sve_udot_lane, 4),
SVEMAP1(svdot_n_s32, aarch64_sve_sdot, 3075),
SVEMAP1(svdot_n_s64, aarch64_sve_sdot, 3076),
SVEMAP1(svdot_n_u32, aarch64_sve_udot, 3075),
SVEMAP1(svdot_n_u64, aarch64_sve_udot, 3076),
SVEMAP1(svdot_s32, aarch64_sve_sdot, 3),
SVEMAP1(svdot_s64, aarch64_sve_sdot, 4),
SVEMAP1(svdot_u32, aarch64_sve_udot, 3),
SVEMAP1(svdot_u64, aarch64_sve_udot, 4),
SVEMAP1(svdup_lane_bf16, aarch64_sve_tbl, 2060),
SVEMAP1(svdup_lane_f16, aarch64_sve_tbl, 2053),
SVEMAP1(svdup_lane_f32, aarch64_sve_tbl, 2054),
SVEMAP1(svdup_lane_f64, aarch64_sve_tbl, 2055),
SVEMAP1(svdup_lane_s16, aarch64_sve_tbl, 2050),
SVEMAP1(svdup_lane_s32, aarch64_sve_tbl, 2051),
SVEMAP1(svdup_lane_s64, aarch64_sve_tbl, 2052),
SVEMAP1(svdup_lane_s8, aarch64_sve_tbl, 2049),
SVEMAP1(svdup_lane_u16, aarch64_sve_tbl, 2050),
SVEMAP1(svdup_lane_u32, aarch64_sve_tbl, 2051),
SVEMAP1(svdup_lane_u64, aarch64_sve_tbl, 2052),
SVEMAP1(svdup_lane_u8, aarch64_sve_tbl, 2049),
SVEMAP2(svdup_n_b16, 9),
SVEMAP2(svdup_n_b32, 10),
SVEMAP2(svdup_n_b64, 11),
SVEMAP2(svdup_n_b8, 8),
SVEMAP1(svdup_n_bf16, aarch64_sve_dup_x, 12),
SVEMAP1(svdup_n_bf16_m, aarch64_sve_dup, 268),
SVEMAP1(svdup_n_bf16_x, aarch64_sve_dup, 524),
SVEMAP1(svdup_n_bf16_z, aarch64_sve_dup, 652),
SVEMAP1(svdup_n_f16, aarch64_sve_dup_x, 5),
SVEMAP1(svdup_n_f16_m, aarch64_sve_dup, 261),
SVEMAP1(svdup_n_f16_x, aarch64_sve_dup, 517),
SVEMAP1(svdup_n_f16_z, aarch64_sve_dup, 645),
SVEMAP1(svdup_n_f32, aarch64_sve_dup_x, 6),
SVEMAP1(svdup_n_f32_m, aarch64_sve_dup, 262),
SVEMAP1(svdup_n_f32_x, aarch64_sve_dup, 518),
SVEMAP1(svdup_n_f32_z, aarch64_sve_dup, 646),
SVEMAP1(svdup_n_f64, aarch64_sve_dup_x, 7),
SVEMAP1(svdup_n_f64_m, aarch64_sve_dup, 263),
SVEMAP1(svdup_n_f64_x, aarch64_sve_dup, 519),
SVEMAP1(svdup_n_f64_z, aarch64_sve_dup, 647),
SVEMAP1(svdup_n_s16, aarch64_sve_dup_x, 2),
SVEMAP1(svdup_n_s16_m, aarch64_sve_dup, 258),
SVEMAP1(svdup_n_s16_x, aarch64_sve_dup, 514),
SVEMAP1(svdup_n_s16_z, aarch64_sve_dup, 642),
SVEMAP1(svdup_n_s32, aarch64_sve_dup_x, 3),
SVEMAP1(svdup_n_s32_m, aarch64_sve_dup, 259),
SVEMAP1(svdup_n_s32_x, aarch64_sve_dup, 515),
SVEMAP1(svdup_n_s32_z, aarch64_sve_dup, 643),
SVEMAP1(svdup_n_s64, aarch64_sve_dup_x, 4),
SVEMAP1(svdup_n_s64_m, aarch64_sve_dup, 260),
SVEMAP1(svdup_n_s64_x, aarch64_sve_dup, 516),
SVEMAP1(svdup_n_s64_z, aarch64_sve_dup, 644),
SVEMAP1(svdup_n_s8, aarch64_sve_dup_x, 1),
SVEMAP1(svdup_n_s8_m, aarch64_sve_dup, 257),
SVEMAP1(svdup_n_s8_x, aarch64_sve_dup, 513),
SVEMAP1(svdup_n_s8_z, aarch64_sve_dup, 641),
SVEMAP1(svdup_n_u16, aarch64_sve_dup_x, 2),
SVEMAP1(svdup_n_u16_m, aarch64_sve_dup, 258),
SVEMAP1(svdup_n_u16_x, aarch64_sve_dup, 514),
SVEMAP1(svdup_n_u16_z, aarch64_sve_dup, 642),
SVEMAP1(svdup_n_u32, aarch64_sve_dup_x, 3),
SVEMAP1(svdup_n_u32_m, aarch64_sve_dup, 259),
SVEMAP1(svdup_n_u32_x, aarch64_sve_dup, 515),
SVEMAP1(svdup_n_u32_z, aarch64_sve_dup, 643),
SVEMAP1(svdup_n_u64, aarch64_sve_dup_x, 4),
SVEMAP1(svdup_n_u64_m, aarch64_sve_dup, 260),
SVEMAP1(svdup_n_u64_x, aarch64_sve_dup, 516),
SVEMAP1(svdup_n_u64_z, aarch64_sve_dup, 644),
SVEMAP1(svdup_n_u8, aarch64_sve_dup_x, 1),
SVEMAP1(svdup_n_u8_m, aarch64_sve_dup, 257),
SVEMAP1(svdup_n_u8_x, aarch64_sve_dup, 513),
SVEMAP1(svdup_n_u8_z, aarch64_sve_dup, 641),
SVEMAP1(svdupq_lane_bf16, aarch64_sve_dupq_lane, 12),
SVEMAP1(svdupq_lane_f16, aarch64_sve_dupq_lane, 5),
SVEMAP1(svdupq_lane_f32, aarch64_sve_dupq_lane, 6),
SVEMAP1(svdupq_lane_f64, aarch64_sve_dupq_lane, 7),
SVEMAP1(svdupq_lane_s16, aarch64_sve_dupq_lane, 2),
SVEMAP1(svdupq_lane_s32, aarch64_sve_dupq_lane, 3),
SVEMAP1(svdupq_lane_s64, aarch64_sve_dupq_lane, 4),
SVEMAP1(svdupq_lane_s8, aarch64_sve_dupq_lane, 1),
SVEMAP1(svdupq_lane_u16, aarch64_sve_dupq_lane, 2),
SVEMAP1(svdupq_lane_u32, aarch64_sve_dupq_lane, 3),
SVEMAP1(svdupq_lane_u64, aarch64_sve_dupq_lane, 4),
SVEMAP1(svdupq_lane_u8, aarch64_sve_dupq_lane, 1),
SVEMAP2(svdupq_n_b16, 9),
SVEMAP2(svdupq_n_b32, 10),
SVEMAP2(svdupq_n_b64, 11),
SVEMAP2(svdupq_n_b8, 8),
SVEMAP2(svdupq_n_bf16, 12),
SVEMAP2(svdupq_n_f16, 5),
SVEMAP2(svdupq_n_f32, 6),
SVEMAP2(svdupq_n_f64, 7),
SVEMAP2(svdupq_n_s16, 2),
SVEMAP2(svdupq_n_s32, 3),
SVEMAP2(svdupq_n_s64, 4),
SVEMAP2(svdupq_n_s8, 1),
SVEMAP2(svdupq_n_u16, 2),
SVEMAP2(svdupq_n_u32, 3),
SVEMAP2(svdupq_n_u64, 4),
SVEMAP2(svdupq_n_u8, 1),
SVEMAP1(sveor3_n_s16, aarch64_sve_eor3, 3074),
SVEMAP1(sveor3_n_s32, aarch64_sve_eor3, 3075),
SVEMAP1(sveor3_n_s64, aarch64_sve_eor3, 3076),
SVEMAP1(sveor3_n_s8, aarch64_sve_eor3, 3073),
SVEMAP1(sveor3_n_u16, aarch64_sve_eor3, 3074),
SVEMAP1(sveor3_n_u32, aarch64_sve_eor3, 3075),
SVEMAP1(sveor3_n_u64, aarch64_sve_eor3, 3076),
SVEMAP1(sveor3_n_u8, aarch64_sve_eor3, 3073),
SVEMAP1(sveor3_s16, aarch64_sve_eor3, 2),
SVEMAP1(sveor3_s32, aarch64_sve_eor3, 3),
SVEMAP1(sveor3_s64, aarch64_sve_eor3, 4),
SVEMAP1(sveor3_s8, aarch64_sve_eor3, 1),
SVEMAP1(sveor3_u16, aarch64_sve_eor3, 2),
SVEMAP1(sveor3_u32, aarch64_sve_eor3, 3),
SVEMAP1(sveor3_u64, aarch64_sve_eor3, 4),
SVEMAP1(sveor3_u8, aarch64_sve_eor3, 1),
SVEMAP1(sveor_b_z, aarch64_sve_eor_z, 8),
SVEMAP1(sveor_n_s16_m, aarch64_sve_eor, 3330),
SVEMAP1(sveor_n_s16_x, aarch64_sve_eor, 3202),
SVEMAP1(sveor_n_s16_z, aarch64_sve_eor, 3458),
SVEMAP1(sveor_n_s32_m, aarch64_sve_eor, 3331),
SVEMAP1(sveor_n_s32_x, aarch64_sve_eor, 3203),
SVEMAP1(sveor_n_s32_z, aarch64_sve_eor, 3459),
SVEMAP1(sveor_n_s64_m, aarch64_sve_eor, 3332),
SVEMAP1(sveor_n_s64_x, aarch64_sve_eor, 3204),
SVEMAP1(sveor_n_s64_z, aarch64_sve_eor, 3460),
SVEMAP1(sveor_n_s8_m, aarch64_sve_eor, 3329),
SVEMAP1(sveor_n_s8_x, aarch64_sve_eor, 3201),
SVEMAP1(sveor_n_s8_z, aarch64_sve_eor, 3457),
SVEMAP1(sveor_n_u16_m, aarch64_sve_eor, 3330),
SVEMAP1(sveor_n_u16_x, aarch64_sve_eor, 3202),
SVEMAP1(sveor_n_u16_z, aarch64_sve_eor, 3458),
SVEMAP1(sveor_n_u32_m, aarch64_sve_eor, 3331),
SVEMAP1(sveor_n_u32_x, aarch64_sve_eor, 3203),
SVEMAP1(sveor_n_u32_z, aarch64_sve_eor, 3459),
SVEMAP1(sveor_n_u64_m, aarch64_sve_eor, 3332),
SVEMAP1(sveor_n_u64_x, aarch64_sve_eor, 3204),
SVEMAP1(sveor_n_u64_z, aarch64_sve_eor, 3460),
SVEMAP1(sveor_n_u8_m, aarch64_sve_eor, 3329),
SVEMAP1(sveor_n_u8_x, aarch64_sve_eor, 3201),
SVEMAP1(sveor_n_u8_z, aarch64_sve_eor, 3457),
SVEMAP1(sveor_s16_m, aarch64_sve_eor, 258),
SVEMAP1(sveor_s16_x, aarch64_sve_eor, 130),
SVEMAP1(sveor_s16_z, aarch64_sve_eor, 386),
SVEMAP1(sveor_s32_m, aarch64_sve_eor, 259),
SVEMAP1(sveor_s32_x, aarch64_sve_eor, 131),
SVEMAP1(sveor_s32_z, aarch64_sve_eor, 387),
SVEMAP1(sveor_s64_m, aarch64_sve_eor, 260),
SVEMAP1(sveor_s64_x, aarch64_sve_eor, 132),
SVEMAP1(sveor_s64_z, aarch64_sve_eor, 388),
SVEMAP1(sveor_s8_m, aarch64_sve_eor, 257),
SVEMAP1(sveor_s8_x, aarch64_sve_eor, 129),
SVEMAP1(sveor_s8_z, aarch64_sve_eor, 385),
SVEMAP1(sveor_u16_m, aarch64_sve_eor, 258),
SVEMAP1(sveor_u16_x, aarch64_sve_eor, 130),
SVEMAP1(sveor_u16_z, aarch64_sve_eor, 386),
SVEMAP1(sveor_u32_m, aarch64_sve_eor, 259),
SVEMAP1(sveor_u32_x, aarch64_sve_eor, 131),
SVEMAP1(sveor_u32_z, aarch64_sve_eor, 387),
SVEMAP1(sveor_u64_m, aarch64_sve_eor, 260),
SVEMAP1(sveor_u64_x, aarch64_sve_eor, 132),
SVEMAP1(sveor_u64_z, aarch64_sve_eor, 388),
SVEMAP1(sveor_u8_m, aarch64_sve_eor, 257),
SVEMAP1(sveor_u8_x, aarch64_sve_eor, 129),
SVEMAP1(sveor_u8_z, aarch64_sve_eor, 385),
SVEMAP1(sveorbt_n_s16, aarch64_sve_eorbt, 3074),
SVEMAP1(sveorbt_n_s32, aarch64_sve_eorbt, 3075),
SVEMAP1(sveorbt_n_s64, aarch64_sve_eorbt, 3076),
SVEMAP1(sveorbt_n_s8, aarch64_sve_eorbt, 3073),
SVEMAP1(sveorbt_n_u16, aarch64_sve_eorbt, 3074),
SVEMAP1(sveorbt_n_u32, aarch64_sve_eorbt, 3075),
SVEMAP1(sveorbt_n_u64, aarch64_sve_eorbt, 3076),
SVEMAP1(sveorbt_n_u8, aarch64_sve_eorbt, 3073),
SVEMAP1(sveorbt_s16, aarch64_sve_eorbt, 2),
SVEMAP1(sveorbt_s32, aarch64_sve_eorbt, 3),
SVEMAP1(sveorbt_s64, aarch64_sve_eorbt, 4),
SVEMAP1(sveorbt_s8, aarch64_sve_eorbt, 1),
SVEMAP1(sveorbt_u16, aarch64_sve_eorbt, 2),
SVEMAP1(sveorbt_u32, aarch64_sve_eorbt, 3),
SVEMAP1(sveorbt_u64, aarch64_sve_eorbt, 4),
SVEMAP1(sveorbt_u8, aarch64_sve_eorbt, 1),
SVEMAP1(sveortb_n_s16, aarch64_sve_eortb, 3074),
SVEMAP1(sveortb_n_s32, aarch64_sve_eortb, 3075),
SVEMAP1(sveortb_n_s64, aarch64_sve_eortb, 3076),
SVEMAP1(sveortb_n_s8, aarch64_sve_eortb, 3073),
SVEMAP1(sveortb_n_u16, aarch64_sve_eortb, 3074),
SVEMAP1(sveortb_n_u32, aarch64_sve_eortb, 3075),
SVEMAP1(sveortb_n_u64, aarch64_sve_eortb, 3076),
SVEMAP1(sveortb_n_u8, aarch64_sve_eortb, 3073),
SVEMAP1(sveortb_s16, aarch64_sve_eortb, 2),
SVEMAP1(sveortb_s32, aarch64_sve_eortb, 3),
SVEMAP1(sveortb_s64, aarch64_sve_eortb, 4),
SVEMAP1(sveortb_s8, aarch64_sve_eortb, 1),
SVEMAP1(sveortb_u16, aarch64_sve_eortb, 2),
SVEMAP1(sveortb_u32, aarch64_sve_eortb, 3),
SVEMAP1(sveortb_u64, aarch64_sve_eortb, 4),
SVEMAP1(sveortb_u8, aarch64_sve_eortb, 1),
SVEMAP1(sveorv_s16, aarch64_sve_eorv, 2),
SVEMAP1(sveorv_s32, aarch64_sve_eorv, 3),
SVEMAP1(sveorv_s64, aarch64_sve_eorv, 4),
SVEMAP1(sveorv_s8, aarch64_sve_eorv, 1),
SVEMAP1(sveorv_u16, aarch64_sve_eorv, 2),
SVEMAP1(sveorv_u32, aarch64_sve_eorv, 3),
SVEMAP1(sveorv_u64, aarch64_sve_eorv, 4),
SVEMAP1(sveorv_u8, aarch64_sve_eorv, 1),
SVEMAP1(svexpa_f16, aarch64_sve_fexpa_x, 5),
SVEMAP1(svexpa_f32, aarch64_sve_fexpa_x, 6),
SVEMAP1(svexpa_f64, aarch64_sve_fexpa_x, 7),
SVEMAP1(svext_bf16, aarch64_sve_ext, 12),
SVEMAP1(svext_f16, aarch64_sve_ext, 5),
SVEMAP1(svext_f32, aarch64_sve_ext, 6),
SVEMAP1(svext_f64, aarch64_sve_ext, 7),
SVEMAP1(svext_s16, aarch64_sve_ext, 2),
SVEMAP1(svext_s32, aarch64_sve_ext, 3),
SVEMAP1(svext_s64, aarch64_sve_ext, 4),
SVEMAP1(svext_s8, aarch64_sve_ext, 1),
SVEMAP1(svext_u16, aarch64_sve_ext, 2),
SVEMAP1(svext_u32, aarch64_sve_ext, 3),
SVEMAP1(svext_u64, aarch64_sve_ext, 4),
SVEMAP1(svext_u8, aarch64_sve_ext, 1),
SVEMAP1(svextb_s16_m, aarch64_sve_sxtb, 258),
SVEMAP1(svextb_s16_x, aarch64_sve_sxtb, 514),
SVEMAP1(svextb_s16_z, aarch64_sve_sxtb, 642),
SVEMAP1(svextb_s32_m, aarch64_sve_sxtb, 259),
SVEMAP1(svextb_s32_x, aarch64_sve_sxtb, 515),
SVEMAP1(svextb_s32_z, aarch64_sve_sxtb, 643),
SVEMAP1(svextb_s64_m, aarch64_sve_sxtb, 260),
SVEMAP1(svextb_s64_x, aarch64_sve_sxtb, 516),
SVEMAP1(svextb_s64_z, aarch64_sve_sxtb, 644),
SVEMAP1(svextb_u16_m, aarch64_sve_uxtb, 258),
SVEMAP1(svextb_u16_x, aarch64_sve_uxtb, 514),
SVEMAP1(svextb_u16_z, aarch64_sve_uxtb, 642),
SVEMAP1(svextb_u32_m, aarch64_sve_uxtb, 259),
SVEMAP1(svextb_u32_x, aarch64_sve_uxtb, 515),
SVEMAP1(svextb_u32_z, aarch64_sve_uxtb, 643),
SVEMAP1(svextb_u64_m, aarch64_sve_uxtb, 260),
SVEMAP1(svextb_u64_x, aarch64_sve_uxtb, 516),
SVEMAP1(svextb_u64_z, aarch64_sve_uxtb, 644),
SVEMAP1(svexth_s32_m, aarch64_sve_sxth, 259),
SVEMAP1(svexth_s32_x, aarch64_sve_sxth, 515),
SVEMAP1(svexth_s32_z, aarch64_sve_sxth, 643),
SVEMAP1(svexth_s64_m, aarch64_sve_sxth, 260),
SVEMAP1(svexth_s64_x, aarch64_sve_sxth, 516),
SVEMAP1(svexth_s64_z, aarch64_sve_sxth, 644),
SVEMAP1(svexth_u32_m, aarch64_sve_uxth, 259),
SVEMAP1(svexth_u32_x, aarch64_sve_uxth, 515),
SVEMAP1(svexth_u32_z, aarch64_sve_uxth, 643),
SVEMAP1(svexth_u64_m, aarch64_sve_uxth, 260),
SVEMAP1(svexth_u64_x, aarch64_sve_uxth, 516),
SVEMAP1(svexth_u64_z, aarch64_sve_uxth, 644),
SVEMAP1(svextw_s64_m, aarch64_sve_sxtw, 260),
SVEMAP1(svextw_s64_x, aarch64_sve_sxtw, 516),
SVEMAP1(svextw_s64_z, aarch64_sve_sxtw, 644),
SVEMAP1(svextw_u64_m, aarch64_sve_uxtw, 260),
SVEMAP1(svextw_u64_x, aarch64_sve_uxtw, 516),
SVEMAP1(svextw_u64_z, aarch64_sve_uxtw, 644),
SVEMAP1(svget2_bf16, aarch64_sve_tuple_get, 8589934604),
SVEMAP1(svget2_f16, aarch64_sve_tuple_get, 8589934597),
SVEMAP1(svget2_f32, aarch64_sve_tuple_get, 8589934598),
SVEMAP1(svget2_f64, aarch64_sve_tuple_get, 8589934599),
SVEMAP1(svget2_s16, aarch64_sve_tuple_get, 8589934594),
SVEMAP1(svget2_s32, aarch64_sve_tuple_get, 8589934595),
SVEMAP1(svget2_s64, aarch64_sve_tuple_get, 8589934596),
SVEMAP1(svget2_s8, aarch64_sve_tuple_get, 8589934593),
SVEMAP1(svget2_u16, aarch64_sve_tuple_get, 8589934594),
SVEMAP1(svget2_u32, aarch64_sve_tuple_get, 8589934595),
SVEMAP1(svget2_u64, aarch64_sve_tuple_get, 8589934596),
SVEMAP1(svget2_u8, aarch64_sve_tuple_get, 8589934593),
SVEMAP1(svget3_bf16, aarch64_sve_tuple_get, 8589934604),
SVEMAP1(svget3_f16, aarch64_sve_tuple_get, 8589934597),
SVEMAP1(svget3_f32, aarch64_sve_tuple_get, 8589934598),
SVEMAP1(svget3_f64, aarch64_sve_tuple_get, 8589934599),
SVEMAP1(svget3_s16, aarch64_sve_tuple_get, 8589934594),
SVEMAP1(svget3_s32, aarch64_sve_tuple_get, 8589934595),
SVEMAP1(svget3_s64, aarch64_sve_tuple_get, 8589934596),
SVEMAP1(svget3_s8, aarch64_sve_tuple_get, 8589934593),
SVEMAP1(svget3_u16, aarch64_sve_tuple_get, 8589934594),
SVEMAP1(svget3_u32, aarch64_sve_tuple_get, 8589934595),
SVEMAP1(svget3_u64, aarch64_sve_tuple_get, 8589934596),
SVEMAP1(svget3_u8, aarch64_sve_tuple_get, 8589934593),
SVEMAP1(svget4_bf16, aarch64_sve_tuple_get, 8589934604),
SVEMAP1(svget4_f16, aarch64_sve_tuple_get, 8589934597),
SVEMAP1(svget4_f32, aarch64_sve_tuple_get, 8589934598),
SVEMAP1(svget4_f64, aarch64_sve_tuple_get, 8589934599),
SVEMAP1(svget4_s16, aarch64_sve_tuple_get, 8589934594),
SVEMAP1(svget4_s32, aarch64_sve_tuple_get, 8589934595),
SVEMAP1(svget4_s64, aarch64_sve_tuple_get, 8589934596),
SVEMAP1(svget4_s8, aarch64_sve_tuple_get, 8589934593),
SVEMAP1(svget4_u16, aarch64_sve_tuple_get, 8589934594),
SVEMAP1(svget4_u32, aarch64_sve_tuple_get, 8589934595),
SVEMAP1(svget4_u64, aarch64_sve_tuple_get, 8589934596),
SVEMAP1(svget4_u8, aarch64_sve_tuple_get, 8589934593),
SVEMAP1(svhadd_n_s16_m, aarch64_sve_shadd, 3330),
SVEMAP1(svhadd_n_s16_x, aarch64_sve_shadd, 3202),
SVEMAP1(svhadd_n_s16_z, aarch64_sve_shadd, 3458),
SVEMAP1(svhadd_n_s32_m, aarch64_sve_shadd, 3331),
SVEMAP1(svhadd_n_s32_x, aarch64_sve_shadd, 3203),
SVEMAP1(svhadd_n_s32_z, aarch64_sve_shadd, 3459),
SVEMAP1(svhadd_n_s64_m, aarch64_sve_shadd, 3332),
SVEMAP1(svhadd_n_s64_x, aarch64_sve_shadd, 3204),
SVEMAP1(svhadd_n_s64_z, aarch64_sve_shadd, 3460),
SVEMAP1(svhadd_n_s8_m, aarch64_sve_shadd, 3329),
SVEMAP1(svhadd_n_s8_x, aarch64_sve_shadd, 3201),
SVEMAP1(svhadd_n_s8_z, aarch64_sve_shadd, 3457),
SVEMAP1(svhadd_n_u16_m, aarch64_sve_uhadd, 3330),
SVEMAP1(svhadd_n_u16_x, aarch64_sve_uhadd, 3202),
SVEMAP1(svhadd_n_u16_z, aarch64_sve_uhadd, 3458),
SVEMAP1(svhadd_n_u32_m, aarch64_sve_uhadd, 3331),
SVEMAP1(svhadd_n_u32_x, aarch64_sve_uhadd, 3203),
SVEMAP1(svhadd_n_u32_z, aarch64_sve_uhadd, 3459),
SVEMAP1(svhadd_n_u64_m, aarch64_sve_uhadd, 3332),
SVEMAP1(svhadd_n_u64_x, aarch64_sve_uhadd, 3204),
SVEMAP1(svhadd_n_u64_z, aarch64_sve_uhadd, 3460),
SVEMAP1(svhadd_n_u8_m, aarch64_sve_uhadd, 3329),
SVEMAP1(svhadd_n_u8_x, aarch64_sve_uhadd, 3201),
SVEMAP1(svhadd_n_u8_z, aarch64_sve_uhadd, 3457),
SVEMAP1(svhadd_s16_m, aarch64_sve_shadd, 258),
SVEMAP1(svhadd_s16_x, aarch64_sve_shadd, 130),
SVEMAP1(svhadd_s16_z, aarch64_sve_shadd, 386),
SVEMAP1(svhadd_s32_m, aarch64_sve_shadd, 259),
SVEMAP1(svhadd_s32_x, aarch64_sve_shadd, 131),
SVEMAP1(svhadd_s32_z, aarch64_sve_shadd, 387),
SVEMAP1(svhadd_s64_m, aarch64_sve_shadd, 260),
SVEMAP1(svhadd_s64_x, aarch64_sve_shadd, 132),
SVEMAP1(svhadd_s64_z, aarch64_sve_shadd, 388),
SVEMAP1(svhadd_s8_m, aarch64_sve_shadd, 257),
SVEMAP1(svhadd_s8_x, aarch64_sve_shadd, 129),
SVEMAP1(svhadd_s8_z, aarch64_sve_shadd, 385),
SVEMAP1(svhadd_u16_m, aarch64_sve_uhadd, 258),
SVEMAP1(svhadd_u16_x, aarch64_sve_uhadd, 130),
SVEMAP1(svhadd_u16_z, aarch64_sve_uhadd, 386),
SVEMAP1(svhadd_u32_m, aarch64_sve_uhadd, 259),
SVEMAP1(svhadd_u32_x, aarch64_sve_uhadd, 131),
SVEMAP1(svhadd_u32_z, aarch64_sve_uhadd, 387),
SVEMAP1(svhadd_u64_m, aarch64_sve_uhadd, 260),
SVEMAP1(svhadd_u64_x, aarch64_sve_uhadd, 132),
SVEMAP1(svhadd_u64_z, aarch64_sve_uhadd, 388),
SVEMAP1(svhadd_u8_m, aarch64_sve_uhadd, 257),
SVEMAP1(svhadd_u8_x, aarch64_sve_uhadd, 129),
SVEMAP1(svhadd_u8_z, aarch64_sve_uhadd, 385),
SVEMAP1(svhistcnt_s32_z, aarch64_sve_histcnt, 3),
SVEMAP1(svhistcnt_s64_z, aarch64_sve_histcnt, 4),
SVEMAP1(svhistcnt_u32_z, aarch64_sve_histcnt, 3),
SVEMAP1(svhistcnt_u64_z, aarch64_sve_histcnt, 4),
SVEMAP1(svhistseg_s8, aarch64_sve_histseg, 1),
SVEMAP1(svhistseg_u8, aarch64_sve_histseg, 1),
SVEMAP1(svhsub_n_s16_m, aarch64_sve_shsub, 3330),
SVEMAP1(svhsub_n_s16_x, aarch64_sve_shsub, 3202),
SVEMAP1(svhsub_n_s16_z, aarch64_sve_shsub, 3458),
SVEMAP1(svhsub_n_s32_m, aarch64_sve_shsub, 3331),
SVEMAP1(svhsub_n_s32_x, aarch64_sve_shsub, 3203),
SVEMAP1(svhsub_n_s32_z, aarch64_sve_shsub, 3459),
SVEMAP1(svhsub_n_s64_m, aarch64_sve_shsub, 3332),
SVEMAP1(svhsub_n_s64_x, aarch64_sve_shsub, 3204),
SVEMAP1(svhsub_n_s64_z, aarch64_sve_shsub, 3460),
SVEMAP1(svhsub_n_s8_m, aarch64_sve_shsub, 3329),
SVEMAP1(svhsub_n_s8_x, aarch64_sve_shsub, 3201),
SVEMAP1(svhsub_n_s8_z, aarch64_sve_shsub, 3457),
SVEMAP1(svhsub_n_u16_m, aarch64_sve_uhsub, 3330),
SVEMAP1(svhsub_n_u16_x, aarch64_sve_uhsub, 3202),
SVEMAP1(svhsub_n_u16_z, aarch64_sve_uhsub, 3458),
SVEMAP1(svhsub_n_u32_m, aarch64_sve_uhsub, 3331),
SVEMAP1(svhsub_n_u32_x, aarch64_sve_uhsub, 3203),
SVEMAP1(svhsub_n_u32_z, aarch64_sve_uhsub, 3459),
SVEMAP1(svhsub_n_u64_m, aarch64_sve_uhsub, 3332),
SVEMAP1(svhsub_n_u64_x, aarch64_sve_uhsub, 3204),
SVEMAP1(svhsub_n_u64_z, aarch64_sve_uhsub, 3460),
SVEMAP1(svhsub_n_u8_m, aarch64_sve_uhsub, 3329),
SVEMAP1(svhsub_n_u8_x, aarch64_sve_uhsub, 3201),
SVEMAP1(svhsub_n_u8_z, aarch64_sve_uhsub, 3457),
SVEMAP1(svhsub_s16_m, aarch64_sve_shsub, 258),
SVEMAP1(svhsub_s16_x, aarch64_sve_shsub, 130),
SVEMAP1(svhsub_s16_z, aarch64_sve_shsub, 386),
SVEMAP1(svhsub_s32_m, aarch64_sve_shsub, 259),
SVEMAP1(svhsub_s32_x, aarch64_sve_shsub, 131),
SVEMAP1(svhsub_s32_z, aarch64_sve_shsub, 387),
SVEMAP1(svhsub_s64_m, aarch64_sve_shsub, 260),
SVEMAP1(svhsub_s64_x, aarch64_sve_shsub, 132),
SVEMAP1(svhsub_s64_z, aarch64_sve_shsub, 388),
SVEMAP1(svhsub_s8_m, aarch64_sve_shsub, 257),
SVEMAP1(svhsub_s8_x, aarch64_sve_shsub, 129),
SVEMAP1(svhsub_s8_z, aarch64_sve_shsub, 385),
SVEMAP1(svhsub_u16_m, aarch64_sve_uhsub, 258),
SVEMAP1(svhsub_u16_x, aarch64_sve_uhsub, 130),
SVEMAP1(svhsub_u16_z, aarch64_sve_uhsub, 386),
SVEMAP1(svhsub_u32_m, aarch64_sve_uhsub, 259),
SVEMAP1(svhsub_u32_x, aarch64_sve_uhsub, 131),
SVEMAP1(svhsub_u32_z, aarch64_sve_uhsub, 387),
SVEMAP1(svhsub_u64_m, aarch64_sve_uhsub, 260),
SVEMAP1(svhsub_u64_x, aarch64_sve_uhsub, 132),
SVEMAP1(svhsub_u64_z, aarch64_sve_uhsub, 388),
SVEMAP1(svhsub_u8_m, aarch64_sve_uhsub, 257),
SVEMAP1(svhsub_u8_x, aarch64_sve_uhsub, 129),
SVEMAP1(svhsub_u8_z, aarch64_sve_uhsub, 385),
SVEMAP1(svhsubr_n_s16_m, aarch64_sve_shsubr, 3330),
SVEMAP1(svhsubr_n_s16_x, aarch64_sve_shsubr, 3202),
SVEMAP1(svhsubr_n_s16_z, aarch64_sve_shsubr, 3458),
SVEMAP1(svhsubr_n_s32_m, aarch64_sve_shsubr, 3331),
SVEMAP1(svhsubr_n_s32_x, aarch64_sve_shsubr, 3203),
SVEMAP1(svhsubr_n_s32_z, aarch64_sve_shsubr, 3459),
SVEMAP1(svhsubr_n_s64_m, aarch64_sve_shsubr, 3332),
SVEMAP1(svhsubr_n_s64_x, aarch64_sve_shsubr, 3204),
SVEMAP1(svhsubr_n_s64_z, aarch64_sve_shsubr, 3460),
SVEMAP1(svhsubr_n_s8_m, aarch64_sve_shsubr, 3329),
SVEMAP1(svhsubr_n_s8_x, aarch64_sve_shsubr, 3201),
SVEMAP1(svhsubr_n_s8_z, aarch64_sve_shsubr, 3457),
SVEMAP1(svhsubr_n_u16_m, aarch64_sve_uhsubr, 3330),
SVEMAP1(svhsubr_n_u16_x, aarch64_sve_uhsubr, 3202),
SVEMAP1(svhsubr_n_u16_z, aarch64_sve_uhsubr, 3458),
SVEMAP1(svhsubr_n_u32_m, aarch64_sve_uhsubr, 3331),
SVEMAP1(svhsubr_n_u32_x, aarch64_sve_uhsubr, 3203),
SVEMAP1(svhsubr_n_u32_z, aarch64_sve_uhsubr, 3459),
SVEMAP1(svhsubr_n_u64_m, aarch64_sve_uhsubr, 3332),
SVEMAP1(svhsubr_n_u64_x, aarch64_sve_uhsubr, 3204),
SVEMAP1(svhsubr_n_u64_z, aarch64_sve_uhsubr, 3460),
SVEMAP1(svhsubr_n_u8_m, aarch64_sve_uhsubr, 3329),
SVEMAP1(svhsubr_n_u8_x, aarch64_sve_uhsubr, 3201),
SVEMAP1(svhsubr_n_u8_z, aarch64_sve_uhsubr, 3457),
SVEMAP1(svhsubr_s16_m, aarch64_sve_shsubr, 258),
SVEMAP1(svhsubr_s16_x, aarch64_sve_shsubr, 130),
SVEMAP1(svhsubr_s16_z, aarch64_sve_shsubr, 386),
SVEMAP1(svhsubr_s32_m, aarch64_sve_shsubr, 259),
SVEMAP1(svhsubr_s32_x, aarch64_sve_shsubr, 131),
SVEMAP1(svhsubr_s32_z, aarch64_sve_shsubr, 387),
SVEMAP1(svhsubr_s64_m, aarch64_sve_shsubr, 260),
SVEMAP1(svhsubr_s64_x, aarch64_sve_shsubr, 132),
SVEMAP1(svhsubr_s64_z, aarch64_sve_shsubr, 388),
SVEMAP1(svhsubr_s8_m, aarch64_sve_shsubr, 257),
SVEMAP1(svhsubr_s8_x, aarch64_sve_shsubr, 129),
SVEMAP1(svhsubr_s8_z, aarch64_sve_shsubr, 385),
SVEMAP1(svhsubr_u16_m, aarch64_sve_uhsubr, 258),
SVEMAP1(svhsubr_u16_x, aarch64_sve_uhsubr, 130),
SVEMAP1(svhsubr_u16_z, aarch64_sve_uhsubr, 386),
SVEMAP1(svhsubr_u32_m, aarch64_sve_uhsubr, 259),
SVEMAP1(svhsubr_u32_x, aarch64_sve_uhsubr, 131),
SVEMAP1(svhsubr_u32_z, aarch64_sve_uhsubr, 387),
SVEMAP1(svhsubr_u64_m, aarch64_sve_uhsubr, 260),
SVEMAP1(svhsubr_u64_x, aarch64_sve_uhsubr, 132),
SVEMAP1(svhsubr_u64_z, aarch64_sve_uhsubr, 388),
SVEMAP1(svhsubr_u8_m, aarch64_sve_uhsubr, 257),
SVEMAP1(svhsubr_u8_x, aarch64_sve_uhsubr, 129),
SVEMAP1(svhsubr_u8_z, aarch64_sve_uhsubr, 385),
SVEMAP1(svindex_s16, aarch64_sve_index, 2),
SVEMAP1(svindex_s32, aarch64_sve_index, 3),
SVEMAP1(svindex_s64, aarch64_sve_index, 4),
SVEMAP1(svindex_s8, aarch64_sve_index, 1),
SVEMAP1(svindex_u16, aarch64_sve_index, 2),
SVEMAP1(svindex_u32, aarch64_sve_index, 3),
SVEMAP1(svindex_u64, aarch64_sve_index, 4),
SVEMAP1(svindex_u8, aarch64_sve_index, 1),
SVEMAP1(svinsr_n_bf16, aarch64_sve_insr, 12),
SVEMAP1(svinsr_n_f16, aarch64_sve_insr, 5),
SVEMAP1(svinsr_n_f32, aarch64_sve_insr, 6),
SVEMAP1(svinsr_n_f64, aarch64_sve_insr, 7),
SVEMAP1(svinsr_n_s16, aarch64_sve_insr, 2),
SVEMAP1(svinsr_n_s32, aarch64_sve_insr, 3),
SVEMAP1(svinsr_n_s64, aarch64_sve_insr, 4),
SVEMAP1(svinsr_n_s8, aarch64_sve_insr, 1),
SVEMAP1(svinsr_n_u16, aarch64_sve_insr, 2),
SVEMAP1(svinsr_n_u32, aarch64_sve_insr, 3),
SVEMAP1(svinsr_n_u64, aarch64_sve_insr, 4),
SVEMAP1(svinsr_n_u8, aarch64_sve_insr, 1),
SVEMAP1(svlasta_bf16, aarch64_sve_lasta, 12),
SVEMAP1(svlasta_f16, aarch64_sve_lasta, 5),
SVEMAP1(svlasta_f32, aarch64_sve_lasta, 6),
SVEMAP1(svlasta_f64, aarch64_sve_lasta, 7),
SVEMAP1(svlasta_s16, aarch64_sve_lasta, 2),
SVEMAP1(svlasta_s32, aarch64_sve_lasta, 3),
SVEMAP1(svlasta_s64, aarch64_sve_lasta, 4),
SVEMAP1(svlasta_s8, aarch64_sve_lasta, 1),
SVEMAP1(svlasta_u16, aarch64_sve_lasta, 2),
SVEMAP1(svlasta_u32, aarch64_sve_lasta, 3),
SVEMAP1(svlasta_u64, aarch64_sve_lasta, 4),
SVEMAP1(svlasta_u8, aarch64_sve_lasta, 1),
SVEMAP1(svlastb_bf16, aarch64_sve_lastb, 12),
SVEMAP1(svlastb_f16, aarch64_sve_lastb, 5),
SVEMAP1(svlastb_f32, aarch64_sve_lastb, 6),
SVEMAP1(svlastb_f64, aarch64_sve_lastb, 7),
SVEMAP1(svlastb_s16, aarch64_sve_lastb, 2),
SVEMAP1(svlastb_s32, aarch64_sve_lastb, 3),
SVEMAP1(svlastb_s64, aarch64_sve_lastb, 4),
SVEMAP1(svlastb_s8, aarch64_sve_lastb, 1),
SVEMAP1(svlastb_u16, aarch64_sve_lastb, 2),
SVEMAP1(svlastb_u32, aarch64_sve_lastb, 3),
SVEMAP1(svlastb_u64, aarch64_sve_lastb, 4),
SVEMAP1(svlastb_u8, aarch64_sve_lastb, 1),
SVEMAP1(svld1_bf16, aarch64_sve_ld1, 8204),
SVEMAP1(svld1_f16, aarch64_sve_ld1, 8197),
SVEMAP1(svld1_f32, aarch64_sve_ld1, 8198),
SVEMAP1(svld1_f64, aarch64_sve_ld1, 8199),
SVEMAP1(svld1_gather_s32index_f32, aarch64_sve_ld1_gather_sxtw_index, 32774),
SVEMAP1(svld1_gather_s32index_s32, aarch64_sve_ld1_gather_sxtw_index, 32771),
SVEMAP1(svld1_gather_s32index_u32, aarch64_sve_ld1_gather_sxtw_index, 32771),
SVEMAP1(svld1_gather_s32offset_f32, aarch64_sve_ld1_gather_sxtw, 16809990),
SVEMAP1(svld1_gather_s32offset_s32, aarch64_sve_ld1_gather_sxtw, 16809987),
SVEMAP1(svld1_gather_s32offset_u32, aarch64_sve_ld1_gather_sxtw, 16809987),
SVEMAP1(svld1_gather_s64index_f64, aarch64_sve_ld1_gather_index, 32775),
SVEMAP1(svld1_gather_s64index_s64, aarch64_sve_ld1_gather_index, 32772),
SVEMAP1(svld1_gather_s64index_u64, aarch64_sve_ld1_gather_index, 32772),
SVEMAP1(svld1_gather_s64offset_f64, aarch64_sve_ld1_gather, 16809991),
SVEMAP1(svld1_gather_s64offset_s64, aarch64_sve_ld1_gather, 16809988),
SVEMAP1(svld1_gather_s64offset_u64, aarch64_sve_ld1_gather, 16809988),
SVEMAP1(svld1_gather_u32base_f32, aarch64_sve_ld1_gather_scalar_offset, 32774),
SVEMAP1(svld1_gather_u32base_index_f32, aarch64_sve_ld1_gather_scalar_offset, 32774),
SVEMAP1(svld1_gather_u32base_index_s32, aarch64_sve_ld1_gather_scalar_offset, 32771),
SVEMAP1(svld1_gather_u32base_index_u32, aarch64_sve_ld1_gather_scalar_offset, 32771),
SVEMAP1(svld1_gather_u32base_offset_f32, aarch64_sve_ld1_gather_scalar_offset, 16809990),
SVEMAP1(svld1_gather_u32base_offset_s32, aarch64_sve_ld1_gather_scalar_offset, 16809987),
SVEMAP1(svld1_gather_u32base_offset_u32, aarch64_sve_ld1_gather_scalar_offset, 16809987),
SVEMAP1(svld1_gather_u32base_s32, aarch64_sve_ld1_gather_scalar_offset, 32771),
SVEMAP1(svld1_gather_u32base_u32, aarch64_sve_ld1_gather_scalar_offset, 32771),
SVEMAP1(svld1_gather_u32index_f32, aarch64_sve_ld1_gather_uxtw_index, 32774),
SVEMAP1(svld1_gather_u32index_s32, aarch64_sve_ld1_gather_uxtw_index, 32771),
SVEMAP1(svld1_gather_u32index_u32, aarch64_sve_ld1_gather_uxtw_index, 32771),
SVEMAP1(svld1_gather_u32offset_f32, aarch64_sve_ld1_gather_uxtw, 16809990),
SVEMAP1(svld1_gather_u32offset_s32, aarch64_sve_ld1_gather_uxtw, 16809987),
SVEMAP1(svld1_gather_u32offset_u32, aarch64_sve_ld1_gather_uxtw, 16809987),
SVEMAP1(svld1_gather_u64base_f64, aarch64_sve_ld1_gather_scalar_offset, 32775),
SVEMAP1(svld1_gather_u64base_index_f64, aarch64_sve_ld1_gather_scalar_offset, 32775),
SVEMAP1(svld1_gather_u64base_index_s64, aarch64_sve_ld1_gather_scalar_offset, 32772),
SVEMAP1(svld1_gather_u64base_index_u64, aarch64_sve_ld1_gather_scalar_offset, 32772),
SVEMAP1(svld1_gather_u64base_offset_f64, aarch64_sve_ld1_gather_scalar_offset, 16809991),
SVEMAP1(svld1_gather_u64base_offset_s64, aarch64_sve_ld1_gather_scalar_offset, 16809988),
SVEMAP1(svld1_gather_u64base_offset_u64, aarch64_sve_ld1_gather_scalar_offset, 16809988),
SVEMAP1(svld1_gather_u64base_s64, aarch64_sve_ld1_gather_scalar_offset, 32772),
SVEMAP1(svld1_gather_u64base_u64, aarch64_sve_ld1_gather_scalar_offset, 32772),
SVEMAP1(svld1_gather_u64index_f64, aarch64_sve_ld1_gather_index, 32775),
SVEMAP1(svld1_gather_u64index_s64, aarch64_sve_ld1_gather_index, 32772),
SVEMAP1(svld1_gather_u64index_u64, aarch64_sve_ld1_gather_index, 32772),
SVEMAP1(svld1_gather_u64offset_f64, aarch64_sve_ld1_gather, 16809991),
SVEMAP1(svld1_gather_u64offset_s64, aarch64_sve_ld1_gather, 16809988),
SVEMAP1(svld1_gather_u64offset_u64, aarch64_sve_ld1_gather, 16809988),
SVEMAP1(svld1_s16, aarch64_sve_ld1, 8194),
SVEMAP1(svld1_s32, aarch64_sve_ld1, 8195),
SVEMAP1(svld1_s64, aarch64_sve_ld1, 8196),
SVEMAP1(svld1_s8, aarch64_sve_ld1, 8193),
SVEMAP1(svld1_u16, aarch64_sve_ld1, 8194),
SVEMAP1(svld1_u32, aarch64_sve_ld1, 8195),
SVEMAP1(svld1_u64, aarch64_sve_ld1, 8196),
SVEMAP1(svld1_u8, aarch64_sve_ld1, 8193),
SVEMAP1(svld1_vnum_bf16, aarch64_sve_ld1, 8204),
SVEMAP1(svld1_vnum_f16, aarch64_sve_ld1, 8197),
SVEMAP1(svld1_vnum_f32, aarch64_sve_ld1, 8198),
SVEMAP1(svld1_vnum_f64, aarch64_sve_ld1, 8199),
SVEMAP1(svld1_vnum_s16, aarch64_sve_ld1, 8194),
SVEMAP1(svld1_vnum_s32, aarch64_sve_ld1, 8195),
SVEMAP1(svld1_vnum_s64, aarch64_sve_ld1, 8196),
SVEMAP1(svld1_vnum_s8, aarch64_sve_ld1, 8193),
SVEMAP1(svld1_vnum_u16, aarch64_sve_ld1, 8194),
SVEMAP1(svld1_vnum_u32, aarch64_sve_ld1, 8195),
SVEMAP1(svld1_vnum_u64, aarch64_sve_ld1, 8196),
SVEMAP1(svld1_vnum_u8, aarch64_sve_ld1, 8193),
SVEMAP1(svld1ro_bf16, aarch64_sve_ld1ro, 12),
SVEMAP1(svld1ro_f16, aarch64_sve_ld1ro, 5),
SVEMAP1(svld1ro_f32, aarch64_sve_ld1ro, 6),
SVEMAP1(svld1ro_f64, aarch64_sve_ld1ro, 7),
SVEMAP1(svld1ro_s16, aarch64_sve_ld1ro, 2),
SVEMAP1(svld1ro_s32, aarch64_sve_ld1ro, 3),
SVEMAP1(svld1ro_s64, aarch64_sve_ld1ro, 4),
SVEMAP1(svld1ro_s8, aarch64_sve_ld1ro, 1),
SVEMAP1(svld1ro_u16, aarch64_sve_ld1ro, 2),
SVEMAP1(svld1ro_u32, aarch64_sve_ld1ro, 3),
SVEMAP1(svld1ro_u64, aarch64_sve_ld1ro, 4),
SVEMAP1(svld1ro_u8, aarch64_sve_ld1ro, 1),
SVEMAP1(svld1rq_bf16, aarch64_sve_ld1rq, 12),
SVEMAP1(svld1rq_f16, aarch64_sve_ld1rq, 5),
SVEMAP1(svld1rq_f32, aarch64_sve_ld1rq, 6),
SVEMAP1(svld1rq_f64, aarch64_sve_ld1rq, 7),
SVEMAP1(svld1rq_s16, aarch64_sve_ld1rq, 2),
SVEMAP1(svld1rq_s32, aarch64_sve_ld1rq, 3),
SVEMAP1(svld1rq_s64, aarch64_sve_ld1rq, 4),
SVEMAP1(svld1rq_s8, aarch64_sve_ld1rq, 1),
SVEMAP1(svld1rq_u16, aarch64_sve_ld1rq, 2),
SVEMAP1(svld1rq_u32, aarch64_sve_ld1rq, 3),
SVEMAP1(svld1rq_u64, aarch64_sve_ld1rq, 4),
SVEMAP1(svld1rq_u8, aarch64_sve_ld1rq, 1),
SVEMAP1(svld1sb_gather_s32offset_s32, aarch64_sve_ld1_gather_sxtw, 16810003),
SVEMAP1(svld1sb_gather_s32offset_u32, aarch64_sve_ld1_gather_sxtw, 16810003),
SVEMAP1(svld1sb_gather_s64offset_s64, aarch64_sve_ld1_gather, 16810004),
SVEMAP1(svld1sb_gather_s64offset_u64, aarch64_sve_ld1_gather, 16810004),
SVEMAP1(svld1sb_gather_u32base_offset_s32, aarch64_sve_ld1_gather_scalar_offset, 16810003),
SVEMAP1(svld1sb_gather_u32base_offset_u32, aarch64_sve_ld1_gather_scalar_offset, 16810003),
SVEMAP1(svld1sb_gather_u32base_s32, aarch64_sve_ld1_gather_scalar_offset, 32787),
SVEMAP1(svld1sb_gather_u32base_u32, aarch64_sve_ld1_gather_scalar_offset, 32787),
SVEMAP1(svld1sb_gather_u32offset_s32, aarch64_sve_ld1_gather_uxtw, 16810003),
SVEMAP1(svld1sb_gather_u32offset_u32, aarch64_sve_ld1_gather_uxtw, 16810003),
SVEMAP1(svld1sb_gather_u64base_offset_s64, aarch64_sve_ld1_gather_scalar_offset, 16810004),
SVEMAP1(svld1sb_gather_u64base_offset_u64, aarch64_sve_ld1_gather_scalar_offset, 16810004),
SVEMAP1(svld1sb_gather_u64base_s64, aarch64_sve_ld1_gather_scalar_offset, 32788),
SVEMAP1(svld1sb_gather_u64base_u64, aarch64_sve_ld1_gather_scalar_offset, 32788),
SVEMAP1(svld1sb_gather_u64offset_s64, aarch64_sve_ld1_gather, 16810004),
SVEMAP1(svld1sb_gather_u64offset_u64, aarch64_sve_ld1_gather, 16810004),
SVEMAP1(svld1sb_s16, aarch64_sve_ld1, 8210),
SVEMAP1(svld1sb_s32, aarch64_sve_ld1, 8211),
SVEMAP1(svld1sb_s64, aarch64_sve_ld1, 8212),
SVEMAP1(svld1sb_u16, aarch64_sve_ld1, 8210),
SVEMAP1(svld1sb_u32, aarch64_sve_ld1, 8211),
SVEMAP1(svld1sb_u64, aarch64_sve_ld1, 8212),
SVEMAP1(svld1sb_vnum_s16, aarch64_sve_ld1, 8210),
SVEMAP1(svld1sb_vnum_s32, aarch64_sve_ld1, 8211),
SVEMAP1(svld1sb_vnum_s64, aarch64_sve_ld1, 8212),
SVEMAP1(svld1sb_vnum_u16, aarch64_sve_ld1, 8210),
SVEMAP1(svld1sb_vnum_u32, aarch64_sve_ld1, 8211),
SVEMAP1(svld1sb_vnum_u64, aarch64_sve_ld1, 8212),
SVEMAP1(svld1sh_gather_s32index_s32, aarch64_sve_ld1_gather_sxtw_index, 32803),
SVEMAP1(svld1sh_gather_s32index_u32, aarch64_sve_ld1_gather_sxtw_index, 32803),
SVEMAP1(svld1sh_gather_s32offset_s32, aarch64_sve_ld1_gather_sxtw, 16810019),
SVEMAP1(svld1sh_gather_s32offset_u32, aarch64_sve_ld1_gather_sxtw, 16810019),
SVEMAP1(svld1sh_gather_s64index_s64, aarch64_sve_ld1_gather_index, 32804),
SVEMAP1(svld1sh_gather_s64index_u64, aarch64_sve_ld1_gather_index, 32804),
SVEMAP1(svld1sh_gather_s64offset_s64, aarch64_sve_ld1_gather, 16810020),
SVEMAP1(svld1sh_gather_s64offset_u64, aarch64_sve_ld1_gather, 16810020),
SVEMAP1(svld1sh_gather_u32base_index_s32, aarch64_sve_ld1_gather_scalar_offset, 32803),
SVEMAP1(svld1sh_gather_u32base_index_u32, aarch64_sve_ld1_gather_scalar_offset, 32803),
SVEMAP1(svld1sh_gather_u32base_offset_s32, aarch64_sve_ld1_gather_scalar_offset, 16810019),
SVEMAP1(svld1sh_gather_u32base_offset_u32, aarch64_sve_ld1_gather_scalar_offset, 16810019),
SVEMAP1(svld1sh_gather_u32base_s32, aarch64_sve_ld1_gather_scalar_offset, 32803),
SVEMAP1(svld1sh_gather_u32base_u32, aarch64_sve_ld1_gather_scalar_offset, 32803),
SVEMAP1(svld1sh_gather_u32index_s32, aarch64_sve_ld1_gather_uxtw_index, 32803),
SVEMAP1(svld1sh_gather_u32index_u32, aarch64_sve_ld1_gather_uxtw_index, 32803),
SVEMAP1(svld1sh_gather_u32offset_s32, aarch64_sve_ld1_gather_uxtw, 16810019),
SVEMAP1(svld1sh_gather_u32offset_u32, aarch64_sve_ld1_gather_uxtw, 16810019),
SVEMAP1(svld1sh_gather_u64base_index_s64, aarch64_sve_ld1_gather_scalar_offset, 32804),
SVEMAP1(svld1sh_gather_u64base_index_u64, aarch64_sve_ld1_gather_scalar_offset, 32804),
SVEMAP1(svld1sh_gather_u64base_offset_s64, aarch64_sve_ld1_gather_scalar_offset, 16810020),
SVEMAP1(svld1sh_gather_u64base_offset_u64, aarch64_sve_ld1_gather_scalar_offset, 16810020),
SVEMAP1(svld1sh_gather_u64base_s64, aarch64_sve_ld1_gather_scalar_offset, 32804),
SVEMAP1(svld1sh_gather_u64base_u64, aarch64_sve_ld1_gather_scalar_offset, 32804),
SVEMAP1(svld1sh_gather_u64index_s64, aarch64_sve_ld1_gather_index, 32804),
SVEMAP1(svld1sh_gather_u64index_u64, aarch64_sve_ld1_gather_index, 32804),
SVEMAP1(svld1sh_gather_u64offset_s64, aarch64_sve_ld1_gather, 16810020),
SVEMAP1(svld1sh_gather_u64offset_u64, aarch64_sve_ld1_gather, 16810020),
SVEMAP1(svld1sh_s32, aarch64_sve_ld1, 8227),
SVEMAP1(svld1sh_s64, aarch64_sve_ld1, 8228),
SVEMAP1(svld1sh_u32, aarch64_sve_ld1, 8227),
SVEMAP1(svld1sh_u64, aarch64_sve_ld1, 8228),
SVEMAP1(svld1sh_vnum_s32, aarch64_sve_ld1, 8227),
SVEMAP1(svld1sh_vnum_s64, aarch64_sve_ld1, 8228),
SVEMAP1(svld1sh_vnum_u32, aarch64_sve_ld1, 8227),
SVEMAP1(svld1sh_vnum_u64, aarch64_sve_ld1, 8228),
SVEMAP1(svld1sw_gather_s64index_s64, aarch64_sve_ld1_gather_index, 32820),
SVEMAP1(svld1sw_gather_s64index_u64, aarch64_sve_ld1_gather_index, 32820),
SVEMAP1(svld1sw_gather_s64offset_s64, aarch64_sve_ld1_gather, 16810036),
SVEMAP1(svld1sw_gather_s64offset_u64, aarch64_sve_ld1_gather, 16810036),
SVEMAP1(svld1sw_gather_u64base_index_s64, aarch64_sve_ld1_gather_scalar_offset, 32820),
SVEMAP1(svld1sw_gather_u64base_index_u64, aarch64_sve_ld1_gather_scalar_offset, 32820),
SVEMAP1(svld1sw_gather_u64base_offset_s64, aarch64_sve_ld1_gather_scalar_offset, 16810036),
SVEMAP1(svld1sw_gather_u64base_offset_u64, aarch64_sve_ld1_gather_scalar_offset, 16810036),
SVEMAP1(svld1sw_gather_u64base_s64, aarch64_sve_ld1_gather_scalar_offset, 32820),
SVEMAP1(svld1sw_gather_u64base_u64, aarch64_sve_ld1_gather_scalar_offset, 32820),
SVEMAP1(svld1sw_gather_u64index_s64, aarch64_sve_ld1_gather_index, 32820),
SVEMAP1(svld1sw_gather_u64index_u64, aarch64_sve_ld1_gather_index, 32820),
SVEMAP1(svld1sw_gather_u64offset_s64, aarch64_sve_ld1_gather, 16810036),
SVEMAP1(svld1sw_gather_u64offset_u64, aarch64_sve_ld1_gather, 16810036),
SVEMAP1(svld1sw_s64, aarch64_sve_ld1, 8244),
SVEMAP1(svld1sw_u64, aarch64_sve_ld1, 8244),
SVEMAP1(svld1sw_vnum_s64, aarch64_sve_ld1, 8244),
SVEMAP1(svld1sw_vnum_u64, aarch64_sve_ld1, 8244),
SVEMAP1(svld1ub_gather_s32offset_s32, aarch64_sve_ld1_gather_sxtw, 17334291),
SVEMAP1(svld1ub_gather_s32offset_u32, aarch64_sve_ld1_gather_sxtw, 17334291),
SVEMAP1(svld1ub_gather_s64offset_s64, aarch64_sve_ld1_gather, 17334292),
SVEMAP1(svld1ub_gather_s64offset_u64, aarch64_sve_ld1_gather, 17334292),
SVEMAP1(svld1ub_gather_u32base_offset_s32, aarch64_sve_ld1_gather_scalar_offset, 17334291),
SVEMAP1(svld1ub_gather_u32base_offset_u32, aarch64_sve_ld1_gather_scalar_offset, 17334291),
SVEMAP1(svld1ub_gather_u32base_s32, aarch64_sve_ld1_gather_scalar_offset, 557075),
SVEMAP1(svld1ub_gather_u32base_u32, aarch64_sve_ld1_gather_scalar_offset, 557075),
SVEMAP1(svld1ub_gather_u32offset_s32, aarch64_sve_ld1_gather_uxtw, 17334291),
SVEMAP1(svld1ub_gather_u32offset_u32, aarch64_sve_ld1_gather_uxtw, 17334291),
SVEMAP1(svld1ub_gather_u64base_offset_s64, aarch64_sve_ld1_gather_scalar_offset, 17334292),
SVEMAP1(svld1ub_gather_u64base_offset_u64, aarch64_sve_ld1_gather_scalar_offset, 17334292),
SVEMAP1(svld1ub_gather_u64base_s64, aarch64_sve_ld1_gather_scalar_offset, 557076),
SVEMAP1(svld1ub_gather_u64base_u64, aarch64_sve_ld1_gather_scalar_offset, 557076),
SVEMAP1(svld1ub_gather_u64offset_s64, aarch64_sve_ld1_gather, 17334292),
SVEMAP1(svld1ub_gather_u64offset_u64, aarch64_sve_ld1_gather, 17334292),
SVEMAP1(svld1ub_s16, aarch64_sve_ld1, 532498),
SVEMAP1(svld1ub_s32, aarch64_sve_ld1, 532499),
SVEMAP1(svld1ub_s64, aarch64_sve_ld1, 532500),
SVEMAP1(svld1ub_u16, aarch64_sve_ld1, 532498),
SVEMAP1(svld1ub_u32, aarch64_sve_ld1, 532499),
SVEMAP1(svld1ub_u64, aarch64_sve_ld1, 532500),
SVEMAP1(svld1ub_vnum_s16, aarch64_sve_ld1, 532498),
SVEMAP1(svld1ub_vnum_s32, aarch64_sve_ld1, 532499),
SVEMAP1(svld1ub_vnum_s64, aarch64_sve_ld1, 532500),
SVEMAP1(svld1ub_vnum_u16, aarch64_sve_ld1, 532498),
SVEMAP1(svld1ub_vnum_u32, aarch64_sve_ld1, 532499),
SVEMAP1(svld1ub_vnum_u64, aarch64_sve_ld1, 532500),
SVEMAP1(svld1uh_gather_s32index_s32, aarch64_sve_ld1_gather_sxtw_index, 557091),
SVEMAP1(svld1uh_gather_s32index_u32, aarch64_sve_ld1_gather_sxtw_index, 557091),
SVEMAP1(svld1uh_gather_s32offset_s32, aarch64_sve_ld1_gather_sxtw, 17334307),
SVEMAP1(svld1uh_gather_s32offset_u32, aarch64_sve_ld1_gather_sxtw, 17334307),
SVEMAP1(svld1uh_gather_s64index_s64, aarch64_sve_ld1_gather_index, 557092),
SVEMAP1(svld1uh_gather_s64index_u64, aarch64_sve_ld1_gather_index, 557092),
SVEMAP1(svld1uh_gather_s64offset_s64, aarch64_sve_ld1_gather, 17334308),
SVEMAP1(svld1uh_gather_s64offset_u64, aarch64_sve_ld1_gather, 17334308),
SVEMAP1(svld1uh_gather_u32base_index_s32, aarch64_sve_ld1_gather_scalar_offset, 557091),
SVEMAP1(svld1uh_gather_u32base_index_u32, aarch64_sve_ld1_gather_scalar_offset, 557091),
SVEMAP1(svld1uh_gather_u32base_offset_s32, aarch64_sve_ld1_gather_scalar_offset, 17334307),
SVEMAP1(svld1uh_gather_u32base_offset_u32, aarch64_sve_ld1_gather_scalar_offset, 17334307),
SVEMAP1(svld1uh_gather_u32base_s32, aarch64_sve_ld1_gather_scalar_offset, 557091),
SVEMAP1(svld1uh_gather_u32base_u32, aarch64_sve_ld1_gather_scalar_offset, 557091),
SVEMAP1(svld1uh_gather_u32index_s32, aarch64_sve_ld1_gather_uxtw_index, 557091),
SVEMAP1(svld1uh_gather_u32index_u32, aarch64_sve_ld1_gather_uxtw_index, 557091),
SVEMAP1(svld1uh_gather_u32offset_s32, aarch64_sve_ld1_gather_uxtw, 17334307),
SVEMAP1(svld1uh_gather_u32offset_u32, aarch64_sve_ld1_gather_uxtw, 17334307),
SVEMAP1(svld1uh_gather_u64base_index_s64, aarch64_sve_ld1_gather_scalar_offset, 557092),
SVEMAP1(svld1uh_gather_u64base_index_u64, aarch64_sve_ld1_gather_scalar_offset, 557092),
SVEMAP1(svld1uh_gather_u64base_offset_s64, aarch64_sve_ld1_gather_scalar_offset, 17334308),
SVEMAP1(svld1uh_gather_u64base_offset_u64, aarch64_sve_ld1_gather_scalar_offset, 17334308),
SVEMAP1(svld1uh_gather_u64base_s64, aarch64_sve_ld1_gather_scalar_offset, 557092),
SVEMAP1(svld1uh_gather_u64base_u64, aarch64_sve_ld1_gather_scalar_offset, 557092),
SVEMAP1(svld1uh_gather_u64index_s64, aarch64_sve_ld1_gather_index, 557092),
SVEMAP1(svld1uh_gather_u64index_u64, aarch64_sve_ld1_gather_index, 557092),
SVEMAP1(svld1uh_gather_u64offset_s64, aarch64_sve_ld1_gather, 17334308),
SVEMAP1(svld1uh_gather_u64offset_u64, aarch64_sve_ld1_gather, 17334308),
SVEMAP1(svld1uh_s32, aarch64_sve_ld1, 532515),
SVEMAP1(svld1uh_s64, aarch64_sve_ld1, 532516),
SVEMAP1(svld1uh_u32, aarch64_sve_ld1, 532515),
SVEMAP1(svld1uh_u64, aarch64_sve_ld1, 532516),
SVEMAP1(svld1uh_vnum_s32, aarch64_sve_ld1, 532515),
SVEMAP1(svld1uh_vnum_s64, aarch64_sve_ld1, 532516),
SVEMAP1(svld1uh_vnum_u32, aarch64_sve_ld1, 532515),
SVEMAP1(svld1uh_vnum_u64, aarch64_sve_ld1, 532516),
SVEMAP1(svld1uw_gather_s64index_s64, aarch64_sve_ld1_gather_index, 557108),
SVEMAP1(svld1uw_gather_s64index_u64, aarch64_sve_ld1_gather_index, 557108),
SVEMAP1(svld1uw_gather_s64offset_s64, aarch64_sve_ld1_gather, 17334324),
SVEMAP1(svld1uw_gather_s64offset_u64, aarch64_sve_ld1_gather, 17334324),
SVEMAP1(svld1uw_gather_u64base_index_s64, aarch64_sve_ld1_gather_scalar_offset, 557108),
SVEMAP1(svld1uw_gather_u64base_index_u64, aarch64_sve_ld1_gather_scalar_offset, 557108),
SVEMAP1(svld1uw_gather_u64base_offset_s64, aarch64_sve_ld1_gather_scalar_offset, 17334324),
SVEMAP1(svld1uw_gather_u64base_offset_u64, aarch64_sve_ld1_gather_scalar_offset, 17334324),
SVEMAP1(svld1uw_gather_u64base_s64, aarch64_sve_ld1_gather_scalar_offset, 557108),
SVEMAP1(svld1uw_gather_u64base_u64, aarch64_sve_ld1_gather_scalar_offset, 557108),
SVEMAP1(svld1uw_gather_u64index_s64, aarch64_sve_ld1_gather_index, 557108),
SVEMAP1(svld1uw_gather_u64index_u64, aarch64_sve_ld1_gather_index, 557108),
SVEMAP1(svld1uw_gather_u64offset_s64, aarch64_sve_ld1_gather, 17334324),
SVEMAP1(svld1uw_gather_u64offset_u64, aarch64_sve_ld1_gather, 17334324),
SVEMAP1(svld1uw_s64, aarch64_sve_ld1, 532532),
SVEMAP1(svld1uw_u64, aarch64_sve_ld1, 532532),
SVEMAP1(svld1uw_vnum_s64, aarch64_sve_ld1, 532532),
SVEMAP1(svld1uw_vnum_u64, aarch64_sve_ld1, 532532),
SVEMAP1(svld2_bf16, aarch64_sve_ld2, 131084),
SVEMAP1(svld2_f16, aarch64_sve_ld2, 131077),
SVEMAP1(svld2_f32, aarch64_sve_ld2, 131078),
SVEMAP1(svld2_f64, aarch64_sve_ld2, 131079),
SVEMAP1(svld2_s16, aarch64_sve_ld2, 131074),
SVEMAP1(svld2_s32, aarch64_sve_ld2, 131075),
SVEMAP1(svld2_s64, aarch64_sve_ld2, 131076),
SVEMAP1(svld2_s8, aarch64_sve_ld2, 131073),
SVEMAP1(svld2_u16, aarch64_sve_ld2, 131074),
SVEMAP1(svld2_u32, aarch64_sve_ld2, 131075),
SVEMAP1(svld2_u64, aarch64_sve_ld2, 131076),
SVEMAP1(svld2_u8, aarch64_sve_ld2, 131073),
SVEMAP1(svld2_vnum_bf16, aarch64_sve_ld2, 131084),
SVEMAP1(svld2_vnum_f16, aarch64_sve_ld2, 131077),
SVEMAP1(svld2_vnum_f32, aarch64_sve_ld2, 131078),
SVEMAP1(svld2_vnum_f64, aarch64_sve_ld2, 131079),
SVEMAP1(svld2_vnum_s16, aarch64_sve_ld2, 131074),
SVEMAP1(svld2_vnum_s32, aarch64_sve_ld2, 131075),
SVEMAP1(svld2_vnum_s64, aarch64_sve_ld2, 131076),
SVEMAP1(svld2_vnum_s8, aarch64_sve_ld2, 131073),
SVEMAP1(svld2_vnum_u16, aarch64_sve_ld2, 131074),
SVEMAP1(svld2_vnum_u32, aarch64_sve_ld2, 131075),
SVEMAP1(svld2_vnum_u64, aarch64_sve_ld2, 131076),
SVEMAP1(svld2_vnum_u8, aarch64_sve_ld2, 131073),
SVEMAP1(svld3_bf16, aarch64_sve_ld3, 131084),
SVEMAP1(svld3_f16, aarch64_sve_ld3, 131077),
SVEMAP1(svld3_f32, aarch64_sve_ld3, 131078),
SVEMAP1(svld3_f64, aarch64_sve_ld3, 131079),
SVEMAP1(svld3_s16, aarch64_sve_ld3, 131074),
SVEMAP1(svld3_s32, aarch64_sve_ld3, 131075),
SVEMAP1(svld3_s64, aarch64_sve_ld3, 131076),
SVEMAP1(svld3_s8, aarch64_sve_ld3, 131073),
SVEMAP1(svld3_u16, aarch64_sve_ld3, 131074),
SVEMAP1(svld3_u32, aarch64_sve_ld3, 131075),
SVEMAP1(svld3_u64, aarch64_sve_ld3, 131076),
SVEMAP1(svld3_u8, aarch64_sve_ld3, 131073),
SVEMAP1(svld3_vnum_bf16, aarch64_sve_ld3, 131084),
SVEMAP1(svld3_vnum_f16, aarch64_sve_ld3, 131077),
SVEMAP1(svld3_vnum_f32, aarch64_sve_ld3, 131078),
SVEMAP1(svld3_vnum_f64, aarch64_sve_ld3, 131079),
SVEMAP1(svld3_vnum_s16, aarch64_sve_ld3, 131074),
SVEMAP1(svld3_vnum_s32, aarch64_sve_ld3, 131075),
SVEMAP1(svld3_vnum_s64, aarch64_sve_ld3, 131076),
SVEMAP1(svld3_vnum_s8, aarch64_sve_ld3, 131073),
SVEMAP1(svld3_vnum_u16, aarch64_sve_ld3, 131074),
SVEMAP1(svld3_vnum_u32, aarch64_sve_ld3, 131075),
SVEMAP1(svld3_vnum_u64, aarch64_sve_ld3, 131076),
SVEMAP1(svld3_vnum_u8, aarch64_sve_ld3, 131073),
SVEMAP1(svld4_bf16, aarch64_sve_ld4, 131084),
SVEMAP1(svld4_f16, aarch64_sve_ld4, 131077),
SVEMAP1(svld4_f32, aarch64_sve_ld4, 131078),
SVEMAP1(svld4_f64, aarch64_sve_ld4, 131079),
SVEMAP1(svld4_s16, aarch64_sve_ld4, 131074),
SVEMAP1(svld4_s32, aarch64_sve_ld4, 131075),
SVEMAP1(svld4_s64, aarch64_sve_ld4, 131076),
SVEMAP1(svld4_s8, aarch64_sve_ld4, 131073),
SVEMAP1(svld4_u16, aarch64_sve_ld4, 131074),
SVEMAP1(svld4_u32, aarch64_sve_ld4, 131075),
SVEMAP1(svld4_u64, aarch64_sve_ld4, 131076),
SVEMAP1(svld4_u8, aarch64_sve_ld4, 131073),
SVEMAP1(svld4_vnum_bf16, aarch64_sve_ld4, 131084),
SVEMAP1(svld4_vnum_f16, aarch64_sve_ld4, 131077),
SVEMAP1(svld4_vnum_f32, aarch64_sve_ld4, 131078),
SVEMAP1(svld4_vnum_f64, aarch64_sve_ld4, 131079),
SVEMAP1(svld4_vnum_s16, aarch64_sve_ld4, 131074),
SVEMAP1(svld4_vnum_s32, aarch64_sve_ld4, 131075),
SVEMAP1(svld4_vnum_s64, aarch64_sve_ld4, 131076),
SVEMAP1(svld4_vnum_s8, aarch64_sve_ld4, 131073),
SVEMAP1(svld4_vnum_u16, aarch64_sve_ld4, 131074),
SVEMAP1(svld4_vnum_u32, aarch64_sve_ld4, 131075),
SVEMAP1(svld4_vnum_u64, aarch64_sve_ld4, 131076),
SVEMAP1(svld4_vnum_u8, aarch64_sve_ld4, 131073),
SVEMAP1(svldff1_bf16, aarch64_sve_ldff1, 8204),
SVEMAP1(svldff1_f16, aarch64_sve_ldff1, 8197),
SVEMAP1(svldff1_f32, aarch64_sve_ldff1, 8198),
SVEMAP1(svldff1_f64, aarch64_sve_ldff1, 8199),
SVEMAP1(svldff1_gather_s32index_f32, aarch64_sve_ldff1_gather_sxtw_index, 32774),
SVEMAP1(svldff1_gather_s32index_s32, aarch64_sve_ldff1_gather_sxtw_index, 32771),
SVEMAP1(svldff1_gather_s32index_u32, aarch64_sve_ldff1_gather_sxtw_index, 32771),
SVEMAP1(svldff1_gather_s32offset_f32, aarch64_sve_ldff1_gather_sxtw, 16809990),
SVEMAP1(svldff1_gather_s32offset_s32, aarch64_sve_ldff1_gather_sxtw, 16809987),
SVEMAP1(svldff1_gather_s32offset_u32, aarch64_sve_ldff1_gather_sxtw, 16809987),
SVEMAP1(svldff1_gather_s64index_f64, aarch64_sve_ldff1_gather_index, 32775),
SVEMAP1(svldff1_gather_s64index_s64, aarch64_sve_ldff1_gather_index, 32772),
SVEMAP1(svldff1_gather_s64index_u64, aarch64_sve_ldff1_gather_index, 32772),
SVEMAP1(svldff1_gather_s64offset_f64, aarch64_sve_ldff1_gather, 16809991),
SVEMAP1(svldff1_gather_s64offset_s64, aarch64_sve_ldff1_gather, 16809988),
SVEMAP1(svldff1_gather_s64offset_u64, aarch64_sve_ldff1_gather, 16809988),
SVEMAP1(svldff1_gather_u32base_f32, aarch64_sve_ldff1_gather_scalar_offset, 32774),
SVEMAP1(svldff1_gather_u32base_index_f32, aarch64_sve_ldff1_gather_scalar_offset, 32774),
SVEMAP1(svldff1_gather_u32base_index_s32, aarch64_sve_ldff1_gather_scalar_offset, 32771),
SVEMAP1(svldff1_gather_u32base_index_u32, aarch64_sve_ldff1_gather_scalar_offset, 32771),
SVEMAP1(svldff1_gather_u32base_offset_f32, aarch64_sve_ldff1_gather_scalar_offset, 16809990),
SVEMAP1(svldff1_gather_u32base_offset_s32, aarch64_sve_ldff1_gather_scalar_offset, 16809987),
SVEMAP1(svldff1_gather_u32base_offset_u32, aarch64_sve_ldff1_gather_scalar_offset, 16809987),
SVEMAP1(svldff1_gather_u32base_s32, aarch64_sve_ldff1_gather_scalar_offset, 32771),
SVEMAP1(svldff1_gather_u32base_u32, aarch64_sve_ldff1_gather_scalar_offset, 32771),
SVEMAP1(svldff1_gather_u32index_f32, aarch64_sve_ldff1_gather_uxtw_index, 32774),
SVEMAP1(svldff1_gather_u32index_s32, aarch64_sve_ldff1_gather_uxtw_index, 32771),
SVEMAP1(svldff1_gather_u32index_u32, aarch64_sve_ldff1_gather_uxtw_index, 32771),
SVEMAP1(svldff1_gather_u32offset_f32, aarch64_sve_ldff1_gather_uxtw, 16809990),
SVEMAP1(svldff1_gather_u32offset_s32, aarch64_sve_ldff1_gather_uxtw, 16809987),
SVEMAP1(svldff1_gather_u32offset_u32, aarch64_sve_ldff1_gather_uxtw, 16809987),
SVEMAP1(svldff1_gather_u64base_f64, aarch64_sve_ldff1_gather_scalar_offset, 32775),
SVEMAP1(svldff1_gather_u64base_index_f64, aarch64_sve_ldff1_gather_scalar_offset, 32775),
SVEMAP1(svldff1_gather_u64base_index_s64, aarch64_sve_ldff1_gather_scalar_offset, 32772),
SVEMAP1(svldff1_gather_u64base_index_u64, aarch64_sve_ldff1_gather_scalar_offset, 32772),
SVEMAP1(svldff1_gather_u64base_offset_f64, aarch64_sve_ldff1_gather_scalar_offset, 16809991),
SVEMAP1(svldff1_gather_u64base_offset_s64, aarch64_sve_ldff1_gather_scalar_offset, 16809988),
SVEMAP1(svldff1_gather_u64base_offset_u64, aarch64_sve_ldff1_gather_scalar_offset, 16809988),
SVEMAP1(svldff1_gather_u64base_s64, aarch64_sve_ldff1_gather_scalar_offset, 32772),
SVEMAP1(svldff1_gather_u64base_u64, aarch64_sve_ldff1_gather_scalar_offset, 32772),
SVEMAP1(svldff1_gather_u64index_f64, aarch64_sve_ldff1_gather_index, 32775),
SVEMAP1(svldff1_gather_u64index_s64, aarch64_sve_ldff1_gather_index, 32772),
SVEMAP1(svldff1_gather_u64index_u64, aarch64_sve_ldff1_gather_index, 32772),
SVEMAP1(svldff1_gather_u64offset_f64, aarch64_sve_ldff1_gather, 16809991),
SVEMAP1(svldff1_gather_u64offset_s64, aarch64_sve_ldff1_gather, 16809988),
SVEMAP1(svldff1_gather_u64offset_u64, aarch64_sve_ldff1_gather, 16809988),
SVEMAP1(svldff1_s16, aarch64_sve_ldff1, 8194),
SVEMAP1(svldff1_s32, aarch64_sve_ldff1, 8195),
SVEMAP1(svldff1_s64, aarch64_sve_ldff1, 8196),
SVEMAP1(svldff1_s8, aarch64_sve_ldff1, 8193),
SVEMAP1(svldff1_u16, aarch64_sve_ldff1, 8194),
SVEMAP1(svldff1_u32, aarch64_sve_ldff1, 8195),
SVEMAP1(svldff1_u64, aarch64_sve_ldff1, 8196),
SVEMAP1(svldff1_u8, aarch64_sve_ldff1, 8193),
SVEMAP1(svldff1_vnum_bf16, aarch64_sve_ldff1, 8204),
SVEMAP1(svldff1_vnum_f16, aarch64_sve_ldff1, 8197),
SVEMAP1(svldff1_vnum_f32, aarch64_sve_ldff1, 8198),
SVEMAP1(svldff1_vnum_f64, aarch64_sve_ldff1, 8199),
SVEMAP1(svldff1_vnum_s16, aarch64_sve_ldff1, 8194),
SVEMAP1(svldff1_vnum_s32, aarch64_sve_ldff1, 8195),
SVEMAP1(svldff1_vnum_s64, aarch64_sve_ldff1, 8196),
SVEMAP1(svldff1_vnum_s8, aarch64_sve_ldff1, 8193),
SVEMAP1(svldff1_vnum_u16, aarch64_sve_ldff1, 8194),
SVEMAP1(svldff1_vnum_u32, aarch64_sve_ldff1, 8195),
SVEMAP1(svldff1_vnum_u64, aarch64_sve_ldff1, 8196),
SVEMAP1(svldff1_vnum_u8, aarch64_sve_ldff1, 8193),
SVEMAP1(svldff1sb_gather_s32offset_s32, aarch64_sve_ldff1_gather_sxtw, 16810003),
SVEMAP1(svldff1sb_gather_s32offset_u32, aarch64_sve_ldff1_gather_sxtw, 16810003),
SVEMAP1(svldff1sb_gather_s64offset_s64, aarch64_sve_ldff1_gather, 16810004),
SVEMAP1(svldff1sb_gather_s64offset_u64, aarch64_sve_ldff1_gather, 16810004),
SVEMAP1(svldff1sb_gather_u32base_offset_s32, aarch64_sve_ldff1_gather_scalar_offset, 16810003),
SVEMAP1(svldff1sb_gather_u32base_offset_u32, aarch64_sve_ldff1_gather_scalar_offset, 16810003),
SVEMAP1(svldff1sb_gather_u32base_s32, aarch64_sve_ldff1_gather_scalar_offset, 32787),
SVEMAP1(svldff1sb_gather_u32base_u32, aarch64_sve_ldff1_gather_scalar_offset, 32787),
SVEMAP1(svldff1sb_gather_u32offset_s32, aarch64_sve_ldff1_gather_uxtw, 16810003),
SVEMAP1(svldff1sb_gather_u32offset_u32, aarch64_sve_ldff1_gather_uxtw, 16810003),
SVEMAP1(svldff1sb_gather_u64base_offset_s64, aarch64_sve_ldff1_gather_scalar_offset, 16810004),
SVEMAP1(svldff1sb_gather_u64base_offset_u64, aarch64_sve_ldff1_gather_scalar_offset, 16810004),
SVEMAP1(svldff1sb_gather_u64base_s64, aarch64_sve_ldff1_gather_scalar_offset, 32788),
SVEMAP1(svldff1sb_gather_u64base_u64, aarch64_sve_ldff1_gather_scalar_offset, 32788),
SVEMAP1(svldff1sb_gather_u64offset_s64, aarch64_sve_ldff1_gather, 16810004),
SVEMAP1(svldff1sb_gather_u64offset_u64, aarch64_sve_ldff1_gather, 16810004),
SVEMAP1(svldff1sb_s16, aarch64_sve_ldff1, 8210),
SVEMAP1(svldff1sb_s32, aarch64_sve_ldff1, 8211),
SVEMAP1(svldff1sb_s64, aarch64_sve_ldff1, 8212),
SVEMAP1(svldff1sb_u16, aarch64_sve_ldff1, 8210),
SVEMAP1(svldff1sb_u32, aarch64_sve_ldff1, 8211),
SVEMAP1(svldff1sb_u64, aarch64_sve_ldff1, 8212),
SVEMAP1(svldff1sb_vnum_s16, aarch64_sve_ldff1, 8210),
SVEMAP1(svldff1sb_vnum_s32, aarch64_sve_ldff1, 8211),
SVEMAP1(svldff1sb_vnum_s64, aarch64_sve_ldff1, 8212),
SVEMAP1(svldff1sb_vnum_u16, aarch64_sve_ldff1, 8210),
SVEMAP1(svldff1sb_vnum_u32, aarch64_sve_ldff1, 8211),
SVEMAP1(svldff1sb_vnum_u64, aarch64_sve_ldff1, 8212),
SVEMAP1(svldff1sh_gather_s32index_s32, aarch64_sve_ldff1_gather_sxtw_index, 32803),
SVEMAP1(svldff1sh_gather_s32index_u32, aarch64_sve_ldff1_gather_sxtw_index, 32803),
SVEMAP1(svldff1sh_gather_s32offset_s32, aarch64_sve_ldff1_gather_sxtw, 16810019),
SVEMAP1(svldff1sh_gather_s32offset_u32, aarch64_sve_ldff1_gather_sxtw, 16810019),
SVEMAP1(svldff1sh_gather_s64index_s64, aarch64_sve_ldff1_gather_index, 32804),
SVEMAP1(svldff1sh_gather_s64index_u64, aarch64_sve_ldff1_gather_index, 32804),
SVEMAP1(svldff1sh_gather_s64offset_s64, aarch64_sve_ldff1_gather, 16810020),
SVEMAP1(svldff1sh_gather_s64offset_u64, aarch64_sve_ldff1_gather, 16810020),
SVEMAP1(svldff1sh_gather_u32base_index_s32, aarch64_sve_ldff1_gather_scalar_offset, 32803),
SVEMAP1(svldff1sh_gather_u32base_index_u32, aarch64_sve_ldff1_gather_scalar_offset, 32803),
SVEMAP1(svldff1sh_gather_u32base_offset_s32, aarch64_sve_ldff1_gather_scalar_offset, 16810019),
SVEMAP1(svldff1sh_gather_u32base_offset_u32, aarch64_sve_ldff1_gather_scalar_offset, 16810019),
SVEMAP1(svldff1sh_gather_u32base_s32, aarch64_sve_ldff1_gather_scalar_offset, 32803),
SVEMAP1(svldff1sh_gather_u32base_u32, aarch64_sve_ldff1_gather_scalar_offset, 32803),
SVEMAP1(svldff1sh_gather_u32index_s32, aarch64_sve_ldff1_gather_uxtw_index, 32803),
SVEMAP1(svldff1sh_gather_u32index_u32, aarch64_sve_ldff1_gather_uxtw_index, 32803),
SVEMAP1(svldff1sh_gather_u32offset_s32, aarch64_sve_ldff1_gather_uxtw, 16810019),
SVEMAP1(svldff1sh_gather_u32offset_u32, aarch64_sve_ldff1_gather_uxtw, 16810019),
SVEMAP1(svldff1sh_gather_u64base_index_s64, aarch64_sve_ldff1_gather_scalar_offset, 32804),
SVEMAP1(svldff1sh_gather_u64base_index_u64, aarch64_sve_ldff1_gather_scalar_offset, 32804),
SVEMAP1(svldff1sh_gather_u64base_offset_s64, aarch64_sve_ldff1_gather_scalar_offset, 16810020),
SVEMAP1(svldff1sh_gather_u64base_offset_u64, aarch64_sve_ldff1_gather_scalar_offset, 16810020),
SVEMAP1(svldff1sh_gather_u64base_s64, aarch64_sve_ldff1_gather_scalar_offset, 32804),
SVEMAP1(svldff1sh_gather_u64base_u64, aarch64_sve_ldff1_gather_scalar_offset, 32804),
SVEMAP1(svldff1sh_gather_u64index_s64, aarch64_sve_ldff1_gather_index, 32804),
SVEMAP1(svldff1sh_gather_u64index_u64, aarch64_sve_ldff1_gather_index, 32804),
SVEMAP1(svldff1sh_gather_u64offset_s64, aarch64_sve_ldff1_gather, 16810020),
SVEMAP1(svldff1sh_gather_u64offset_u64, aarch64_sve_ldff1_gather, 16810020),
SVEMAP1(svldff1sh_s32, aarch64_sve_ldff1, 8227),
SVEMAP1(svldff1sh_s64, aarch64_sve_ldff1, 8228),
SVEMAP1(svldff1sh_u32, aarch64_sve_ldff1, 8227),
SVEMAP1(svldff1sh_u64, aarch64_sve_ldff1, 8228),
SVEMAP1(svldff1sh_vnum_s32, aarch64_sve_ldff1, 8227),
SVEMAP1(svldff1sh_vnum_s64, aarch64_sve_ldff1, 8228),
SVEMAP1(svldff1sh_vnum_u32, aarch64_sve_ldff1, 8227),
SVEMAP1(svldff1sh_vnum_u64, aarch64_sve_ldff1, 8228),
SVEMAP1(svldff1sw_gather_s64index_s64, aarch64_sve_ldff1_gather_index, 32820),
SVEMAP1(svldff1sw_gather_s64index_u64, aarch64_sve_ldff1_gather_index, 32820),
SVEMAP1(svldff1sw_gather_s64offset_s64, aarch64_sve_ldff1_gather, 16810036),
SVEMAP1(svldff1sw_gather_s64offset_u64, aarch64_sve_ldff1_gather, 16810036),
SVEMAP1(svldff1sw_gather_u64base_index_s64, aarch64_sve_ldff1_gather_scalar_offset, 32820),
SVEMAP1(svldff1sw_gather_u64base_index_u64, aarch64_sve_ldff1_gather_scalar_offset, 32820),
SVEMAP1(svldff1sw_gather_u64base_offset_s64, aarch64_sve_ldff1_gather_scalar_offset, 16810036),
SVEMAP1(svldff1sw_gather_u64base_offset_u64, aarch64_sve_ldff1_gather_scalar_offset, 16810036),
SVEMAP1(svldff1sw_gather_u64base_s64, aarch64_sve_ldff1_gather_scalar_offset, 32820),
SVEMAP1(svldff1sw_gather_u64base_u64, aarch64_sve_ldff1_gather_scalar_offset, 32820),
SVEMAP1(svldff1sw_gather_u64index_s64, aarch64_sve_ldff1_gather_index, 32820),
SVEMAP1(svldff1sw_gather_u64index_u64, aarch64_sve_ldff1_gather_index, 32820),
SVEMAP1(svldff1sw_gather_u64offset_s64, aarch64_sve_ldff1_gather, 16810036),
SVEMAP1(svldff1sw_gather_u64offset_u64, aarch64_sve_ldff1_gather, 16810036),
SVEMAP1(svldff1sw_s64, aarch64_sve_ldff1, 8244),
SVEMAP1(svldff1sw_u64, aarch64_sve_ldff1, 8244),
SVEMAP1(svldff1sw_vnum_s64, aarch64_sve_ldff1, 8244),
SVEMAP1(svldff1sw_vnum_u64, aarch64_sve_ldff1, 8244),
SVEMAP1(svldff1ub_gather_s32offset_s32, aarch64_sve_ldff1_gather_sxtw, 17334291),
SVEMAP1(svldff1ub_gather_s32offset_u32, aarch64_sve_ldff1_gather_sxtw, 17334291),
SVEMAP1(svldff1ub_gather_s64offset_s64, aarch64_sve_ldff1_gather, 17334292),
SVEMAP1(svldff1ub_gather_s64offset_u64, aarch64_sve_ldff1_gather, 17334292),
SVEMAP1(svldff1ub_gather_u32base_offset_s32, aarch64_sve_ldff1_gather_scalar_offset, 17334291),
SVEMAP1(svldff1ub_gather_u32base_offset_u32, aarch64_sve_ldff1_gather_scalar_offset, 17334291),
SVEMAP1(svldff1ub_gather_u32base_s32, aarch64_sve_ldff1_gather_scalar_offset, 557075),
SVEMAP1(svldff1ub_gather_u32base_u32, aarch64_sve_ldff1_gather_scalar_offset, 557075),
SVEMAP1(svldff1ub_gather_u32offset_s32, aarch64_sve_ldff1_gather_uxtw, 17334291),
SVEMAP1(svldff1ub_gather_u32offset_u32, aarch64_sve_ldff1_gather_uxtw, 17334291),
SVEMAP1(svldff1ub_gather_u64base_offset_s64, aarch64_sve_ldff1_gather_scalar_offset, 17334292),
SVEMAP1(svldff1ub_gather_u64base_offset_u64, aarch64_sve_ldff1_gather_scalar_offset, 17334292),
SVEMAP1(svldff1ub_gather_u64base_s64, aarch64_sve_ldff1_gather_scalar_offset, 557076),
SVEMAP1(svldff1ub_gather_u64base_u64, aarch64_sve_ldff1_gather_scalar_offset, 557076),
SVEMAP1(svldff1ub_gather_u64offset_s64, aarch64_sve_ldff1_gather, 17334292),
SVEMAP1(svldff1ub_gather_u64offset_u64, aarch64_sve_ldff1_gather, 17334292),
SVEMAP1(svldff1ub_s16, aarch64_sve_ldff1, 532498),
SVEMAP1(svldff1ub_s32, aarch64_sve_ldff1, 532499),
SVEMAP1(svldff1ub_s64, aarch64_sve_ldff1, 532500),
SVEMAP1(svldff1ub_u16, aarch64_sve_ldff1, 532498),
SVEMAP1(svldff1ub_u32, aarch64_sve_ldff1, 532499),
SVEMAP1(svldff1ub_u64, aarch64_sve_ldff1, 532500),
SVEMAP1(svldff1ub_vnum_s16, aarch64_sve_ldff1, 532498),
SVEMAP1(svldff1ub_vnum_s32, aarch64_sve_ldff1, 532499),
SVEMAP1(svldff1ub_vnum_s64, aarch64_sve_ldff1, 532500),
SVEMAP1(svldff1ub_vnum_u16, aarch64_sve_ldff1, 532498),
SVEMAP1(svldff1ub_vnum_u32, aarch64_sve_ldff1, 532499),
SVEMAP1(svldff1ub_vnum_u64, aarch64_sve_ldff1, 532500),
SVEMAP1(svldff1uh_gather_s32index_s32, aarch64_sve_ldff1_gather_sxtw_index, 557091),
SVEMAP1(svldff1uh_gather_s32index_u32, aarch64_sve_ldff1_gather_sxtw_index, 557091),
SVEMAP1(svldff1uh_gather_s32offset_s32, aarch64_sve_ldff1_gather_sxtw, 17334307),
SVEMAP1(svldff1uh_gather_s32offset_u32, aarch64_sve_ldff1_gather_sxtw, 17334307),
SVEMAP1(svldff1uh_gather_s64index_s64, aarch64_sve_ldff1_gather_index, 557092),
SVEMAP1(svldff1uh_gather_s64index_u64, aarch64_sve_ldff1_gather_index, 557092),
SVEMAP1(svldff1uh_gather_s64offset_s64, aarch64_sve_ldff1_gather, 17334308),
SVEMAP1(svldff1uh_gather_s64offset_u64, aarch64_sve_ldff1_gather, 17334308),
SVEMAP1(svldff1uh_gather_u32base_index_s32, aarch64_sve_ldff1_gather_scalar_offset, 557091),
SVEMAP1(svldff1uh_gather_u32base_index_u32, aarch64_sve_ldff1_gather_scalar_offset, 557091),
SVEMAP1(svldff1uh_gather_u32base_offset_s32, aarch64_sve_ldff1_gather_scalar_offset, 17334307),
SVEMAP1(svldff1uh_gather_u32base_offset_u32, aarch64_sve_ldff1_gather_scalar_offset, 17334307),
SVEMAP1(svldff1uh_gather_u32base_s32, aarch64_sve_ldff1_gather_scalar_offset, 557091),
SVEMAP1(svldff1uh_gather_u32base_u32, aarch64_sve_ldff1_gather_scalar_offset, 557091),
SVEMAP1(svldff1uh_gather_u32index_s32, aarch64_sve_ldff1_gather_uxtw_index, 557091),
SVEMAP1(svldff1uh_gather_u32index_u32, aarch64_sve_ldff1_gather_uxtw_index, 557091),
SVEMAP1(svldff1uh_gather_u32offset_s32, aarch64_sve_ldff1_gather_uxtw, 17334307),
SVEMAP1(svldff1uh_gather_u32offset_u32, aarch64_sve_ldff1_gather_uxtw, 17334307),
SVEMAP1(svldff1uh_gather_u64base_index_s64, aarch64_sve_ldff1_gather_scalar_offset, 557092),
SVEMAP1(svldff1uh_gather_u64base_index_u64, aarch64_sve_ldff1_gather_scalar_offset, 557092),
SVEMAP1(svldff1uh_gather_u64base_offset_s64, aarch64_sve_ldff1_gather_scalar_offset, 17334308),
SVEMAP1(svldff1uh_gather_u64base_offset_u64, aarch64_sve_ldff1_gather_scalar_offset, 17334308),
SVEMAP1(svldff1uh_gather_u64base_s64, aarch64_sve_ldff1_gather_scalar_offset, 557092),
SVEMAP1(svldff1uh_gather_u64base_u64, aarch64_sve_ldff1_gather_scalar_offset, 557092),
SVEMAP1(svldff1uh_gather_u64index_s64, aarch64_sve_ldff1_gather_index, 557092),
SVEMAP1(svldff1uh_gather_u64index_u64, aarch64_sve_ldff1_gather_index, 557092),
SVEMAP1(svldff1uh_gather_u64offset_s64, aarch64_sve_ldff1_gather, 17334308),
SVEMAP1(svldff1uh_gather_u64offset_u64, aarch64_sve_ldff1_gather, 17334308),
SVEMAP1(svldff1uh_s32, aarch64_sve_ldff1, 532515),
SVEMAP1(svldff1uh_s64, aarch64_sve_ldff1, 532516),
SVEMAP1(svldff1uh_u32, aarch64_sve_ldff1, 532515),
SVEMAP1(svldff1uh_u64, aarch64_sve_ldff1, 532516),
SVEMAP1(svldff1uh_vnum_s32, aarch64_sve_ldff1, 532515),
SVEMAP1(svldff1uh_vnum_s64, aarch64_sve_ldff1, 532516),
SVEMAP1(svldff1uh_vnum_u32, aarch64_sve_ldff1, 532515),
SVEMAP1(svldff1uh_vnum_u64, aarch64_sve_ldff1, 532516),
SVEMAP1(svldff1uw_gather_s64index_s64, aarch64_sve_ldff1_gather_index, 557108),
SVEMAP1(svldff1uw_gather_s64index_u64, aarch64_sve_ldff1_gather_index, 557108),
SVEMAP1(svldff1uw_gather_s64offset_s64, aarch64_sve_ldff1_gather, 17334324),
SVEMAP1(svldff1uw_gather_s64offset_u64, aarch64_sve_ldff1_gather, 17334324),
SVEMAP1(svldff1uw_gather_u64base_index_s64, aarch64_sve_ldff1_gather_scalar_offset, 557108),
SVEMAP1(svldff1uw_gather_u64base_index_u64, aarch64_sve_ldff1_gather_scalar_offset, 557108),
SVEMAP1(svldff1uw_gather_u64base_offset_s64, aarch64_sve_ldff1_gather_scalar_offset, 17334324),
SVEMAP1(svldff1uw_gather_u64base_offset_u64, aarch64_sve_ldff1_gather_scalar_offset, 17334324),
SVEMAP1(svldff1uw_gather_u64base_s64, aarch64_sve_ldff1_gather_scalar_offset, 557108),
SVEMAP1(svldff1uw_gather_u64base_u64, aarch64_sve_ldff1_gather_scalar_offset, 557108),
SVEMAP1(svldff1uw_gather_u64index_s64, aarch64_sve_ldff1_gather_index, 557108),
SVEMAP1(svldff1uw_gather_u64index_u64, aarch64_sve_ldff1_gather_index, 557108),
SVEMAP1(svldff1uw_gather_u64offset_s64, aarch64_sve_ldff1_gather, 17334324),
SVEMAP1(svldff1uw_gather_u64offset_u64, aarch64_sve_ldff1_gather, 17334324),
SVEMAP1(svldff1uw_s64, aarch64_sve_ldff1, 532532),
SVEMAP1(svldff1uw_u64, aarch64_sve_ldff1, 532532),
SVEMAP1(svldff1uw_vnum_s64, aarch64_sve_ldff1, 532532),
SVEMAP1(svldff1uw_vnum_u64, aarch64_sve_ldff1, 532532),
SVEMAP1(svldnf1_bf16, aarch64_sve_ldnf1, 8204),
SVEMAP1(svldnf1_f16, aarch64_sve_ldnf1, 8197),
SVEMAP1(svldnf1_f32, aarch64_sve_ldnf1, 8198),
SVEMAP1(svldnf1_f64, aarch64_sve_ldnf1, 8199),
SVEMAP1(svldnf1_s16, aarch64_sve_ldnf1, 8194),
SVEMAP1(svldnf1_s32, aarch64_sve_ldnf1, 8195),
SVEMAP1(svldnf1_s64, aarch64_sve_ldnf1, 8196),
SVEMAP1(svldnf1_s8, aarch64_sve_ldnf1, 8193),
SVEMAP1(svldnf1_u16, aarch64_sve_ldnf1, 8194),
SVEMAP1(svldnf1_u32, aarch64_sve_ldnf1, 8195),
SVEMAP1(svldnf1_u64, aarch64_sve_ldnf1, 8196),
SVEMAP1(svldnf1_u8, aarch64_sve_ldnf1, 8193),
SVEMAP1(svldnf1_vnum_bf16, aarch64_sve_ldnf1, 8204),
SVEMAP1(svldnf1_vnum_f16, aarch64_sve_ldnf1, 8197),
SVEMAP1(svldnf1_vnum_f32, aarch64_sve_ldnf1, 8198),
SVEMAP1(svldnf1_vnum_f64, aarch64_sve_ldnf1, 8199),
SVEMAP1(svldnf1_vnum_s16, aarch64_sve_ldnf1, 8194),
SVEMAP1(svldnf1_vnum_s32, aarch64_sve_ldnf1, 8195),
SVEMAP1(svldnf1_vnum_s64, aarch64_sve_ldnf1, 8196),
SVEMAP1(svldnf1_vnum_s8, aarch64_sve_ldnf1, 8193),
SVEMAP1(svldnf1_vnum_u16, aarch64_sve_ldnf1, 8194),
SVEMAP1(svldnf1_vnum_u32, aarch64_sve_ldnf1, 8195),
SVEMAP1(svldnf1_vnum_u64, aarch64_sve_ldnf1, 8196),
SVEMAP1(svldnf1_vnum_u8, aarch64_sve_ldnf1, 8193),
SVEMAP1(svldnf1sb_s16, aarch64_sve_ldnf1, 8210),
SVEMAP1(svldnf1sb_s32, aarch64_sve_ldnf1, 8211),
SVEMAP1(svldnf1sb_s64, aarch64_sve_ldnf1, 8212),
SVEMAP1(svldnf1sb_u16, aarch64_sve_ldnf1, 8210),
SVEMAP1(svldnf1sb_u32, aarch64_sve_ldnf1, 8211),
SVEMAP1(svldnf1sb_u64, aarch64_sve_ldnf1, 8212),
SVEMAP1(svldnf1sb_vnum_s16, aarch64_sve_ldnf1, 8210),
SVEMAP1(svldnf1sb_vnum_s32, aarch64_sve_ldnf1, 8211),
SVEMAP1(svldnf1sb_vnum_s64, aarch64_sve_ldnf1, 8212),
SVEMAP1(svldnf1sb_vnum_u16, aarch64_sve_ldnf1, 8210),
SVEMAP1(svldnf1sb_vnum_u32, aarch64_sve_ldnf1, 8211),
SVEMAP1(svldnf1sb_vnum_u64, aarch64_sve_ldnf1, 8212),
SVEMAP1(svldnf1sh_s32, aarch64_sve_ldnf1, 8227),
SVEMAP1(svldnf1sh_s64, aarch64_sve_ldnf1, 8228),
SVEMAP1(svldnf1sh_u32, aarch64_sve_ldnf1, 8227),
SVEMAP1(svldnf1sh_u64, aarch64_sve_ldnf1, 8228),
SVEMAP1(svldnf1sh_vnum_s32, aarch64_sve_ldnf1, 8227),
SVEMAP1(svldnf1sh_vnum_s64, aarch64_sve_ldnf1, 8228),
SVEMAP1(svldnf1sh_vnum_u32, aarch64_sve_ldnf1, 8227),
SVEMAP1(svldnf1sh_vnum_u64, aarch64_sve_ldnf1, 8228),
SVEMAP1(svldnf1sw_s64, aarch64_sve_ldnf1, 8244),
SVEMAP1(svldnf1sw_u64, aarch64_sve_ldnf1, 8244),
SVEMAP1(svldnf1sw_vnum_s64, aarch64_sve_ldnf1, 8244),
SVEMAP1(svldnf1sw_vnum_u64, aarch64_sve_ldnf1, 8244),
SVEMAP1(svldnf1ub_s16, aarch64_sve_ldnf1, 532498),
SVEMAP1(svldnf1ub_s32, aarch64_sve_ldnf1, 532499),
SVEMAP1(svldnf1ub_s64, aarch64_sve_ldnf1, 532500),
SVEMAP1(svldnf1ub_u16, aarch64_sve_ldnf1, 532498),
SVEMAP1(svldnf1ub_u32, aarch64_sve_ldnf1, 532499),
SVEMAP1(svldnf1ub_u64, aarch64_sve_ldnf1, 532500),
SVEMAP1(svldnf1ub_vnum_s16, aarch64_sve_ldnf1, 532498),
SVEMAP1(svldnf1ub_vnum_s32, aarch64_sve_ldnf1, 532499),
SVEMAP1(svldnf1ub_vnum_s64, aarch64_sve_ldnf1, 532500),
SVEMAP1(svldnf1ub_vnum_u16, aarch64_sve_ldnf1, 532498),
SVEMAP1(svldnf1ub_vnum_u32, aarch64_sve_ldnf1, 532499),
SVEMAP1(svldnf1ub_vnum_u64, aarch64_sve_ldnf1, 532500),
SVEMAP1(svldnf1uh_s32, aarch64_sve_ldnf1, 532515),
SVEMAP1(svldnf1uh_s64, aarch64_sve_ldnf1, 532516),
SVEMAP1(svldnf1uh_u32, aarch64_sve_ldnf1, 532515),
SVEMAP1(svldnf1uh_u64, aarch64_sve_ldnf1, 532516),
SVEMAP1(svldnf1uh_vnum_s32, aarch64_sve_ldnf1, 532515),
SVEMAP1(svldnf1uh_vnum_s64, aarch64_sve_ldnf1, 532516),
SVEMAP1(svldnf1uh_vnum_u32, aarch64_sve_ldnf1, 532515),
SVEMAP1(svldnf1uh_vnum_u64, aarch64_sve_ldnf1, 532516),
SVEMAP1(svldnf1uw_s64, aarch64_sve_ldnf1, 532532),
SVEMAP1(svldnf1uw_u64, aarch64_sve_ldnf1, 532532),
SVEMAP1(svldnf1uw_vnum_s64, aarch64_sve_ldnf1, 532532),
SVEMAP1(svldnf1uw_vnum_u64, aarch64_sve_ldnf1, 532532),
SVEMAP1(svldnt1_bf16, aarch64_sve_ldnt1, 8204),
SVEMAP1(svldnt1_f16, aarch64_sve_ldnt1, 8197),
SVEMAP1(svldnt1_f32, aarch64_sve_ldnt1, 8198),
SVEMAP1(svldnt1_f64, aarch64_sve_ldnt1, 8199),
SVEMAP1(svldnt1_gather_s64index_f64, aarch64_sve_ldnt1_gather_index, 32775),
SVEMAP1(svldnt1_gather_s64index_s64, aarch64_sve_ldnt1_gather_index, 32772),
SVEMAP1(svldnt1_gather_s64index_u64, aarch64_sve_ldnt1_gather_index, 32772),
SVEMAP1(svldnt1_gather_s64offset_f64, aarch64_sve_ldnt1_gather, 16809991),
SVEMAP1(svldnt1_gather_s64offset_s64, aarch64_sve_ldnt1_gather, 16809988),
SVEMAP1(svldnt1_gather_s64offset_u64, aarch64_sve_ldnt1_gather, 16809988),
SVEMAP1(svldnt1_gather_u32base_f32, aarch64_sve_ldnt1_gather_scalar_offset, 32774),
SVEMAP1(svldnt1_gather_u32base_index_f32, aarch64_sve_ldnt1_gather_scalar_offset, 32774),
SVEMAP1(svldnt1_gather_u32base_index_s32, aarch64_sve_ldnt1_gather_scalar_offset, 32771),
SVEMAP1(svldnt1_gather_u32base_index_u32, aarch64_sve_ldnt1_gather_scalar_offset, 32771),
SVEMAP1(svldnt1_gather_u32base_offset_f32, aarch64_sve_ldnt1_gather_scalar_offset, 16809990),
SVEMAP1(svldnt1_gather_u32base_offset_s32, aarch64_sve_ldnt1_gather_scalar_offset, 16809987),
SVEMAP1(svldnt1_gather_u32base_offset_u32, aarch64_sve_ldnt1_gather_scalar_offset, 16809987),
SVEMAP1(svldnt1_gather_u32base_s32, aarch64_sve_ldnt1_gather_scalar_offset, 32771),
SVEMAP1(svldnt1_gather_u32base_u32, aarch64_sve_ldnt1_gather_scalar_offset, 32771),
SVEMAP1(svldnt1_gather_u32offset_f32, aarch64_sve_ldnt1_gather_uxtw, 16809990),
SVEMAP1(svldnt1_gather_u32offset_s32, aarch64_sve_ldnt1_gather_uxtw, 16809987),
SVEMAP1(svldnt1_gather_u32offset_u32, aarch64_sve_ldnt1_gather_uxtw, 16809987),
SVEMAP1(svldnt1_gather_u64base_f64, aarch64_sve_ldnt1_gather_scalar_offset, 32775),
SVEMAP1(svldnt1_gather_u64base_index_f64, aarch64_sve_ldnt1_gather_scalar_offset, 32775),
SVEMAP1(svldnt1_gather_u64base_index_s64, aarch64_sve_ldnt1_gather_scalar_offset, 32772),
SVEMAP1(svldnt1_gather_u64base_index_u64, aarch64_sve_ldnt1_gather_scalar_offset, 32772),
SVEMAP1(svldnt1_gather_u64base_offset_f64, aarch64_sve_ldnt1_gather_scalar_offset, 16809991),
SVEMAP1(svldnt1_gather_u64base_offset_s64, aarch64_sve_ldnt1_gather_scalar_offset, 16809988),
SVEMAP1(svldnt1_gather_u64base_offset_u64, aarch64_sve_ldnt1_gather_scalar_offset, 16809988),
SVEMAP1(svldnt1_gather_u64base_s64, aarch64_sve_ldnt1_gather_scalar_offset, 32772),
SVEMAP1(svldnt1_gather_u64base_u64, aarch64_sve_ldnt1_gather_scalar_offset, 32772),
SVEMAP1(svldnt1_gather_u64index_f64, aarch64_sve_ldnt1_gather_index, 32775),
SVEMAP1(svldnt1_gather_u64index_s64, aarch64_sve_ldnt1_gather_index, 32772),
SVEMAP1(svldnt1_gather_u64index_u64, aarch64_sve_ldnt1_gather_index, 32772),
SVEMAP1(svldnt1_gather_u64offset_f64, aarch64_sve_ldnt1_gather, 16809991),
SVEMAP1(svldnt1_gather_u64offset_s64, aarch64_sve_ldnt1_gather, 16809988),
SVEMAP1(svldnt1_gather_u64offset_u64, aarch64_sve_ldnt1_gather, 16809988),
SVEMAP1(svldnt1_s16, aarch64_sve_ldnt1, 8194),
SVEMAP1(svldnt1_s32, aarch64_sve_ldnt1, 8195),
SVEMAP1(svldnt1_s64, aarch64_sve_ldnt1, 8196),
SVEMAP1(svldnt1_s8, aarch64_sve_ldnt1, 8193),
SVEMAP1(svldnt1_u16, aarch64_sve_ldnt1, 8194),
SVEMAP1(svldnt1_u32, aarch64_sve_ldnt1, 8195),
SVEMAP1(svldnt1_u64, aarch64_sve_ldnt1, 8196),
SVEMAP1(svldnt1_u8, aarch64_sve_ldnt1, 8193),
SVEMAP1(svldnt1_vnum_bf16, aarch64_sve_ldnt1, 8204),
SVEMAP1(svldnt1_vnum_f16, aarch64_sve_ldnt1, 8197),
SVEMAP1(svldnt1_vnum_f32, aarch64_sve_ldnt1, 8198),
SVEMAP1(svldnt1_vnum_f64, aarch64_sve_ldnt1, 8199),
SVEMAP1(svldnt1_vnum_s16, aarch64_sve_ldnt1, 8194),
SVEMAP1(svldnt1_vnum_s32, aarch64_sve_ldnt1, 8195),
SVEMAP1(svldnt1_vnum_s64, aarch64_sve_ldnt1, 8196),
SVEMAP1(svldnt1_vnum_s8, aarch64_sve_ldnt1, 8193),
SVEMAP1(svldnt1_vnum_u16, aarch64_sve_ldnt1, 8194),
SVEMAP1(svldnt1_vnum_u32, aarch64_sve_ldnt1, 8195),
SVEMAP1(svldnt1_vnum_u64, aarch64_sve_ldnt1, 8196),
SVEMAP1(svldnt1_vnum_u8, aarch64_sve_ldnt1, 8193),
SVEMAP1(svldnt1sb_gather_s64offset_s64, aarch64_sve_ldnt1_gather, 16810004),
SVEMAP1(svldnt1sb_gather_s64offset_u64, aarch64_sve_ldnt1_gather, 16810004),
SVEMAP1(svldnt1sb_gather_u32base_offset_s32, aarch64_sve_ldnt1_gather_scalar_offset, 16810003),
SVEMAP1(svldnt1sb_gather_u32base_offset_u32, aarch64_sve_ldnt1_gather_scalar_offset, 16810003),
SVEMAP1(svldnt1sb_gather_u32base_s32, aarch64_sve_ldnt1_gather_scalar_offset, 32787),
SVEMAP1(svldnt1sb_gather_u32base_u32, aarch64_sve_ldnt1_gather_scalar_offset, 32787),
SVEMAP1(svldnt1sb_gather_u32offset_s32, aarch64_sve_ldnt1_gather_uxtw, 16810003),
SVEMAP1(svldnt1sb_gather_u32offset_u32, aarch64_sve_ldnt1_gather_uxtw, 16810003),
SVEMAP1(svldnt1sb_gather_u64base_offset_s64, aarch64_sve_ldnt1_gather_scalar_offset, 16810004),
SVEMAP1(svldnt1sb_gather_u64base_offset_u64, aarch64_sve_ldnt1_gather_scalar_offset, 16810004),
SVEMAP1(svldnt1sb_gather_u64base_s64, aarch64_sve_ldnt1_gather_scalar_offset, 32788),
SVEMAP1(svldnt1sb_gather_u64base_u64, aarch64_sve_ldnt1_gather_scalar_offset, 32788),
SVEMAP1(svldnt1sb_gather_u64offset_s64, aarch64_sve_ldnt1_gather, 16810004),
SVEMAP1(svldnt1sb_gather_u64offset_u64, aarch64_sve_ldnt1_gather, 16810004),
SVEMAP1(svldnt1sh_gather_s64index_s64, aarch64_sve_ldnt1_gather_index, 32804),
SVEMAP1(svldnt1sh_gather_s64index_u64, aarch64_sve_ldnt1_gather_index, 32804),
SVEMAP1(svldnt1sh_gather_s64offset_s64, aarch64_sve_ldnt1_gather, 16810020),
SVEMAP1(svldnt1sh_gather_s64offset_u64, aarch64_sve_ldnt1_gather, 16810020),
SVEMAP1(svldnt1sh_gather_u32base_index_s32, aarch64_sve_ldnt1_gather_scalar_offset, 32803),
SVEMAP1(svldnt1sh_gather_u32base_index_u32, aarch64_sve_ldnt1_gather_scalar_offset, 32803),
SVEMAP1(svldnt1sh_gather_u32base_offset_s32, aarch64_sve_ldnt1_gather_scalar_offset, 16810019),
SVEMAP1(svldnt1sh_gather_u32base_offset_u32, aarch64_sve_ldnt1_gather_scalar_offset, 16810019),
SVEMAP1(svldnt1sh_gather_u32base_s32, aarch64_sve_ldnt1_gather_scalar_offset, 32803),
SVEMAP1(svldnt1sh_gather_u32base_u32, aarch64_sve_ldnt1_gather_scalar_offset, 32803),
SVEMAP1(svldnt1sh_gather_u32offset_s32, aarch64_sve_ldnt1_gather_uxtw, 16810019),
SVEMAP1(svldnt1sh_gather_u32offset_u32, aarch64_sve_ldnt1_gather_uxtw, 16810019),
SVEMAP1(svldnt1sh_gather_u64base_index_s64, aarch64_sve_ldnt1_gather_scalar_offset, 32804),
SVEMAP1(svldnt1sh_gather_u64base_index_u64, aarch64_sve_ldnt1_gather_scalar_offset, 32804),
SVEMAP1(svldnt1sh_gather_u64base_offset_s64, aarch64_sve_ldnt1_gather_scalar_offset, 16810020),
SVEMAP1(svldnt1sh_gather_u64base_offset_u64, aarch64_sve_ldnt1_gather_scalar_offset, 16810020),
SVEMAP1(svldnt1sh_gather_u64base_s64, aarch64_sve_ldnt1_gather_scalar_offset, 32804),
SVEMAP1(svldnt1sh_gather_u64base_u64, aarch64_sve_ldnt1_gather_scalar_offset, 32804),
SVEMAP1(svldnt1sh_gather_u64index_s64, aarch64_sve_ldnt1_gather_index, 32804),
SVEMAP1(svldnt1sh_gather_u64index_u64, aarch64_sve_ldnt1_gather_index, 32804),
SVEMAP1(svldnt1sh_gather_u64offset_s64, aarch64_sve_ldnt1_gather, 16810020),
SVEMAP1(svldnt1sh_gather_u64offset_u64, aarch64_sve_ldnt1_gather, 16810020),
SVEMAP1(svldnt1sw_gather_s64index_s64, aarch64_sve_ldnt1_gather_index, 32820),
SVEMAP1(svldnt1sw_gather_s64index_u64, aarch64_sve_ldnt1_gather_index, 32820),
SVEMAP1(svldnt1sw_gather_s64offset_s64, aarch64_sve_ldnt1_gather, 16810036),
SVEMAP1(svldnt1sw_gather_s64offset_u64, aarch64_sve_ldnt1_gather, 16810036),
SVEMAP1(svldnt1sw_gather_u64base_index_s64, aarch64_sve_ldnt1_gather_scalar_offset, 32820),
SVEMAP1(svldnt1sw_gather_u64base_index_u64, aarch64_sve_ldnt1_gather_scalar_offset, 32820),
SVEMAP1(svldnt1sw_gather_u64base_offset_s64, aarch64_sve_ldnt1_gather_scalar_offset, 16810036),
SVEMAP1(svldnt1sw_gather_u64base_offset_u64, aarch64_sve_ldnt1_gather_scalar_offset, 16810036),
SVEMAP1(svldnt1sw_gather_u64base_s64, aarch64_sve_ldnt1_gather_scalar_offset, 32820),
SVEMAP1(svldnt1sw_gather_u64base_u64, aarch64_sve_ldnt1_gather_scalar_offset, 32820),
SVEMAP1(svldnt1sw_gather_u64index_s64, aarch64_sve_ldnt1_gather_index, 32820),
SVEMAP1(svldnt1sw_gather_u64index_u64, aarch64_sve_ldnt1_gather_index, 32820),
SVEMAP1(svldnt1sw_gather_u64offset_s64, aarch64_sve_ldnt1_gather, 16810036),
SVEMAP1(svldnt1sw_gather_u64offset_u64, aarch64_sve_ldnt1_gather, 16810036),
SVEMAP1(svldnt1ub_gather_s64offset_s64, aarch64_sve_ldnt1_gather, 17334292),
SVEMAP1(svldnt1ub_gather_s64offset_u64, aarch64_sve_ldnt1_gather, 17334292),
SVEMAP1(svldnt1ub_gather_u32base_offset_s32, aarch64_sve_ldnt1_gather_scalar_offset, 17334291),
SVEMAP1(svldnt1ub_gather_u32base_offset_u32, aarch64_sve_ldnt1_gather_scalar_offset, 17334291),
SVEMAP1(svldnt1ub_gather_u32base_s32, aarch64_sve_ldnt1_gather_scalar_offset, 557075),
SVEMAP1(svldnt1ub_gather_u32base_u32, aarch64_sve_ldnt1_gather_scalar_offset, 557075),
SVEMAP1(svldnt1ub_gather_u32offset_s32, aarch64_sve_ldnt1_gather_uxtw, 17334291),
SVEMAP1(svldnt1ub_gather_u32offset_u32, aarch64_sve_ldnt1_gather_uxtw, 17334291),
SVEMAP1(svldnt1ub_gather_u64base_offset_s64, aarch64_sve_ldnt1_gather_scalar_offset, 17334292),
SVEMAP1(svldnt1ub_gather_u64base_offset_u64, aarch64_sve_ldnt1_gather_scalar_offset, 17334292),
SVEMAP1(svldnt1ub_gather_u64base_s64, aarch64_sve_ldnt1_gather_scalar_offset, 557076),
SVEMAP1(svldnt1ub_gather_u64base_u64, aarch64_sve_ldnt1_gather_scalar_offset, 557076),
SVEMAP1(svldnt1ub_gather_u64offset_s64, aarch64_sve_ldnt1_gather, 17334292),
SVEMAP1(svldnt1ub_gather_u64offset_u64, aarch64_sve_ldnt1_gather, 17334292),
SVEMAP1(svldnt1uh_gather_s64index_s64, aarch64_sve_ldnt1_gather_index, 557092),
SVEMAP1(svldnt1uh_gather_s64index_u64, aarch64_sve_ldnt1_gather_index, 557092),
SVEMAP1(svldnt1uh_gather_s64offset_s64, aarch64_sve_ldnt1_gather, 17334308),
SVEMAP1(svldnt1uh_gather_s64offset_u64, aarch64_sve_ldnt1_gather, 17334308),
SVEMAP1(svldnt1uh_gather_u32base_index_s32, aarch64_sve_ldnt1_gather_scalar_offset, 557091),
SVEMAP1(svldnt1uh_gather_u32base_index_u32, aarch64_sve_ldnt1_gather_scalar_offset, 557091),
SVEMAP1(svldnt1uh_gather_u32base_offset_s32, aarch64_sve_ldnt1_gather_scalar_offset, 17334307),
SVEMAP1(svldnt1uh_gather_u32base_offset_u32, aarch64_sve_ldnt1_gather_scalar_offset, 17334307),
SVEMAP1(svldnt1uh_gather_u32base_s32, aarch64_sve_ldnt1_gather_scalar_offset, 557091),
SVEMAP1(svldnt1uh_gather_u32base_u32, aarch64_sve_ldnt1_gather_scalar_offset, 557091),
SVEMAP1(svldnt1uh_gather_u32offset_s32, aarch64_sve_ldnt1_gather_uxtw, 17334307),
SVEMAP1(svldnt1uh_gather_u32offset_u32, aarch64_sve_ldnt1_gather_uxtw, 17334307),
SVEMAP1(svldnt1uh_gather_u64base_index_s64, aarch64_sve_ldnt1_gather_scalar_offset, 557092),
SVEMAP1(svldnt1uh_gather_u64base_index_u64, aarch64_sve_ldnt1_gather_scalar_offset, 557092),
SVEMAP1(svldnt1uh_gather_u64base_offset_s64, aarch64_sve_ldnt1_gather_scalar_offset, 17334308),
SVEMAP1(svldnt1uh_gather_u64base_offset_u64, aarch64_sve_ldnt1_gather_scalar_offset, 17334308),
SVEMAP1(svldnt1uh_gather_u64base_s64, aarch64_sve_ldnt1_gather_scalar_offset, 557092),
SVEMAP1(svldnt1uh_gather_u64base_u64, aarch64_sve_ldnt1_gather_scalar_offset, 557092),
SVEMAP1(svldnt1uh_gather_u64index_s64, aarch64_sve_ldnt1_gather_index, 557092),
SVEMAP1(svldnt1uh_gather_u64index_u64, aarch64_sve_ldnt1_gather_index, 557092),
SVEMAP1(svldnt1uh_gather_u64offset_s64, aarch64_sve_ldnt1_gather, 17334308),
SVEMAP1(svldnt1uh_gather_u64offset_u64, aarch64_sve_ldnt1_gather, 17334308),
SVEMAP1(svldnt1uw_gather_s64index_s64, aarch64_sve_ldnt1_gather_index, 557108),
SVEMAP1(svldnt1uw_gather_s64index_u64, aarch64_sve_ldnt1_gather_index, 557108),
SVEMAP1(svldnt1uw_gather_s64offset_s64, aarch64_sve_ldnt1_gather, 17334324),
SVEMAP1(svldnt1uw_gather_s64offset_u64, aarch64_sve_ldnt1_gather, 17334324),
SVEMAP1(svldnt1uw_gather_u64base_index_s64, aarch64_sve_ldnt1_gather_scalar_offset, 557108),
SVEMAP1(svldnt1uw_gather_u64base_index_u64, aarch64_sve_ldnt1_gather_scalar_offset, 557108),
SVEMAP1(svldnt1uw_gather_u64base_offset_s64, aarch64_sve_ldnt1_gather_scalar_offset, 17334324),
SVEMAP1(svldnt1uw_gather_u64base_offset_u64, aarch64_sve_ldnt1_gather_scalar_offset, 17334324),
SVEMAP1(svldnt1uw_gather_u64base_s64, aarch64_sve_ldnt1_gather_scalar_offset, 557108),
SVEMAP1(svldnt1uw_gather_u64base_u64, aarch64_sve_ldnt1_gather_scalar_offset, 557108),
SVEMAP1(svldnt1uw_gather_u64index_s64, aarch64_sve_ldnt1_gather_index, 557108),
SVEMAP1(svldnt1uw_gather_u64index_u64, aarch64_sve_ldnt1_gather_index, 557108),
SVEMAP1(svldnt1uw_gather_u64offset_s64, aarch64_sve_ldnt1_gather, 17334324),
SVEMAP1(svldnt1uw_gather_u64offset_u64, aarch64_sve_ldnt1_gather, 17334324),
SVEMAP2(svlen_bf16, 12),
SVEMAP2(svlen_f16, 5),
SVEMAP2(svlen_f32, 6),
SVEMAP2(svlen_f64, 7),
SVEMAP2(svlen_s16, 2),
SVEMAP2(svlen_s32, 3),
SVEMAP2(svlen_s64, 4),
SVEMAP2(svlen_s8, 1),
SVEMAP2(svlen_u16, 2),
SVEMAP2(svlen_u32, 3),
SVEMAP2(svlen_u64, 4),
SVEMAP2(svlen_u8, 1),
SVEMAP1(svlogb_f16_m, aarch64_sve_flogb, 261),
SVEMAP1(svlogb_f16_x, aarch64_sve_flogb, 517),
SVEMAP1(svlogb_f16_z, aarch64_sve_flogb, 645),
SVEMAP1(svlogb_f32_m, aarch64_sve_flogb, 262),
SVEMAP1(svlogb_f32_x, aarch64_sve_flogb, 518),
SVEMAP1(svlogb_f32_z, aarch64_sve_flogb, 646),
SVEMAP1(svlogb_f64_m, aarch64_sve_flogb, 263),
SVEMAP1(svlogb_f64_x, aarch64_sve_flogb, 519),
SVEMAP1(svlogb_f64_z, aarch64_sve_flogb, 647),
SVEMAP1(svlsl_n_s16_m, aarch64_sve_lsl, 3330),
SVEMAP1(svlsl_n_s16_x, aarch64_sve_lsl, 3202),
SVEMAP1(svlsl_n_s16_z, aarch64_sve_lsl, 3458),
SVEMAP1(svlsl_n_s32_m, aarch64_sve_lsl, 3331),
SVEMAP1(svlsl_n_s32_x, aarch64_sve_lsl, 3203),
SVEMAP1(svlsl_n_s32_z, aarch64_sve_lsl, 3459),
SVEMAP1(svlsl_n_s64_m, aarch64_sve_lsl, 3332),
SVEMAP1(svlsl_n_s64_x, aarch64_sve_lsl, 3204),
SVEMAP1(svlsl_n_s64_z, aarch64_sve_lsl, 3460),
SVEMAP1(svlsl_n_s8_m, aarch64_sve_lsl, 3329),
SVEMAP1(svlsl_n_s8_x, aarch64_sve_lsl, 3201),
SVEMAP1(svlsl_n_s8_z, aarch64_sve_lsl, 3457),
SVEMAP1(svlsl_n_u16_m, aarch64_sve_lsl, 3330),
SVEMAP1(svlsl_n_u16_x, aarch64_sve_lsl, 3202),
SVEMAP1(svlsl_n_u16_z, aarch64_sve_lsl, 3458),
SVEMAP1(svlsl_n_u32_m, aarch64_sve_lsl, 3331),
SVEMAP1(svlsl_n_u32_x, aarch64_sve_lsl, 3203),
SVEMAP1(svlsl_n_u32_z, aarch64_sve_lsl, 3459),
SVEMAP1(svlsl_n_u64_m, aarch64_sve_lsl, 3332),
SVEMAP1(svlsl_n_u64_x, aarch64_sve_lsl, 3204),
SVEMAP1(svlsl_n_u64_z, aarch64_sve_lsl, 3460),
SVEMAP1(svlsl_n_u8_m, aarch64_sve_lsl, 3329),
SVEMAP1(svlsl_n_u8_x, aarch64_sve_lsl, 3201),
SVEMAP1(svlsl_n_u8_z, aarch64_sve_lsl, 3457),
SVEMAP1(svlsl_s16_m, aarch64_sve_lsl, 258),
SVEMAP1(svlsl_s16_x, aarch64_sve_lsl, 130),
SVEMAP1(svlsl_s16_z, aarch64_sve_lsl, 386),
SVEMAP1(svlsl_s32_m, aarch64_sve_lsl, 259),
SVEMAP1(svlsl_s32_x, aarch64_sve_lsl, 131),
SVEMAP1(svlsl_s32_z, aarch64_sve_lsl, 387),
SVEMAP1(svlsl_s64_m, aarch64_sve_lsl, 260),
SVEMAP1(svlsl_s64_x, aarch64_sve_lsl, 132),
SVEMAP1(svlsl_s64_z, aarch64_sve_lsl, 388),
SVEMAP1(svlsl_s8_m, aarch64_sve_lsl, 257),
SVEMAP1(svlsl_s8_x, aarch64_sve_lsl, 129),
SVEMAP1(svlsl_s8_z, aarch64_sve_lsl, 385),
SVEMAP1(svlsl_u16_m, aarch64_sve_lsl, 258),
SVEMAP1(svlsl_u16_x, aarch64_sve_lsl, 130),
SVEMAP1(svlsl_u16_z, aarch64_sve_lsl, 386),
SVEMAP1(svlsl_u32_m, aarch64_sve_lsl, 259),
SVEMAP1(svlsl_u32_x, aarch64_sve_lsl, 131),
SVEMAP1(svlsl_u32_z, aarch64_sve_lsl, 387),
SVEMAP1(svlsl_u64_m, aarch64_sve_lsl, 260),
SVEMAP1(svlsl_u64_x, aarch64_sve_lsl, 132),
SVEMAP1(svlsl_u64_z, aarch64_sve_lsl, 388),
SVEMAP1(svlsl_u8_m, aarch64_sve_lsl, 257),
SVEMAP1(svlsl_u8_x, aarch64_sve_lsl, 129),
SVEMAP1(svlsl_u8_z, aarch64_sve_lsl, 385),
SVEMAP1(svlsl_wide_n_s16_m, aarch64_sve_lsl_wide, 3330),
SVEMAP1(svlsl_wide_n_s16_x, aarch64_sve_lsl_wide, 3202),
SVEMAP1(svlsl_wide_n_s16_z, aarch64_sve_lsl_wide, 3458),
SVEMAP1(svlsl_wide_n_s32_m, aarch64_sve_lsl_wide, 3331),
SVEMAP1(svlsl_wide_n_s32_x, aarch64_sve_lsl_wide, 3203),
SVEMAP1(svlsl_wide_n_s32_z, aarch64_sve_lsl_wide, 3459),
SVEMAP1(svlsl_wide_n_s8_m, aarch64_sve_lsl_wide, 3329),
SVEMAP1(svlsl_wide_n_s8_x, aarch64_sve_lsl_wide, 3201),
SVEMAP1(svlsl_wide_n_s8_z, aarch64_sve_lsl_wide, 3457),
SVEMAP1(svlsl_wide_n_u16_m, aarch64_sve_lsl_wide, 3330),
SVEMAP1(svlsl_wide_n_u16_x, aarch64_sve_lsl_wide, 3202),
SVEMAP1(svlsl_wide_n_u16_z, aarch64_sve_lsl_wide, 3458),
SVEMAP1(svlsl_wide_n_u32_m, aarch64_sve_lsl_wide, 3331),
SVEMAP1(svlsl_wide_n_u32_x, aarch64_sve_lsl_wide, 3203),
SVEMAP1(svlsl_wide_n_u32_z, aarch64_sve_lsl_wide, 3459),
SVEMAP1(svlsl_wide_n_u8_m, aarch64_sve_lsl_wide, 3329),
SVEMAP1(svlsl_wide_n_u8_x, aarch64_sve_lsl_wide, 3201),
SVEMAP1(svlsl_wide_n_u8_z, aarch64_sve_lsl_wide, 3457),
SVEMAP1(svlsl_wide_s16_m, aarch64_sve_lsl_wide, 258),
SVEMAP1(svlsl_wide_s16_x, aarch64_sve_lsl_wide, 130),
SVEMAP1(svlsl_wide_s16_z, aarch64_sve_lsl_wide, 386),
SVEMAP1(svlsl_wide_s32_m, aarch64_sve_lsl_wide, 259),
SVEMAP1(svlsl_wide_s32_x, aarch64_sve_lsl_wide, 131),
SVEMAP1(svlsl_wide_s32_z, aarch64_sve_lsl_wide, 387),
SVEMAP1(svlsl_wide_s8_m, aarch64_sve_lsl_wide, 257),
SVEMAP1(svlsl_wide_s8_x, aarch64_sve_lsl_wide, 129),
SVEMAP1(svlsl_wide_s8_z, aarch64_sve_lsl_wide, 385),
SVEMAP1(svlsl_wide_u16_m, aarch64_sve_lsl_wide, 258),
SVEMAP1(svlsl_wide_u16_x, aarch64_sve_lsl_wide, 130),
SVEMAP1(svlsl_wide_u16_z, aarch64_sve_lsl_wide, 386),
SVEMAP1(svlsl_wide_u32_m, aarch64_sve_lsl_wide, 259),
SVEMAP1(svlsl_wide_u32_x, aarch64_sve_lsl_wide, 131),
SVEMAP1(svlsl_wide_u32_z, aarch64_sve_lsl_wide, 387),
SVEMAP1(svlsl_wide_u8_m, aarch64_sve_lsl_wide, 257),
SVEMAP1(svlsl_wide_u8_x, aarch64_sve_lsl_wide, 129),
SVEMAP1(svlsl_wide_u8_z, aarch64_sve_lsl_wide, 385),
SVEMAP1(svlsr_n_u16_m, aarch64_sve_lsr, 3330),
SVEMAP1(svlsr_n_u16_x, aarch64_sve_lsr, 3202),
SVEMAP1(svlsr_n_u16_z, aarch64_sve_lsr, 3458),
SVEMAP1(svlsr_n_u32_m, aarch64_sve_lsr, 3331),
SVEMAP1(svlsr_n_u32_x, aarch64_sve_lsr, 3203),
SVEMAP1(svlsr_n_u32_z, aarch64_sve_lsr, 3459),
SVEMAP1(svlsr_n_u64_m, aarch64_sve_lsr, 3332),
SVEMAP1(svlsr_n_u64_x, aarch64_sve_lsr, 3204),
SVEMAP1(svlsr_n_u64_z, aarch64_sve_lsr, 3460),
SVEMAP1(svlsr_n_u8_m, aarch64_sve_lsr, 3329),
SVEMAP1(svlsr_n_u8_x, aarch64_sve_lsr, 3201),
SVEMAP1(svlsr_n_u8_z, aarch64_sve_lsr, 3457),
SVEMAP1(svlsr_u16_m, aarch64_sve_lsr, 258),
SVEMAP1(svlsr_u16_x, aarch64_sve_lsr, 130),
SVEMAP1(svlsr_u16_z, aarch64_sve_lsr, 386),
SVEMAP1(svlsr_u32_m, aarch64_sve_lsr, 259),
SVEMAP1(svlsr_u32_x, aarch64_sve_lsr, 131),
SVEMAP1(svlsr_u32_z, aarch64_sve_lsr, 387),
SVEMAP1(svlsr_u64_m, aarch64_sve_lsr, 260),
SVEMAP1(svlsr_u64_x, aarch64_sve_lsr, 132),
SVEMAP1(svlsr_u64_z, aarch64_sve_lsr, 388),
SVEMAP1(svlsr_u8_m, aarch64_sve_lsr, 257),
SVEMAP1(svlsr_u8_x, aarch64_sve_lsr, 129),
SVEMAP1(svlsr_u8_z, aarch64_sve_lsr, 385),
SVEMAP1(svlsr_wide_n_u16_m, aarch64_sve_lsr_wide, 3330),
SVEMAP1(svlsr_wide_n_u16_x, aarch64_sve_lsr_wide, 3202),
SVEMAP1(svlsr_wide_n_u16_z, aarch64_sve_lsr_wide, 3458),
SVEMAP1(svlsr_wide_n_u32_m, aarch64_sve_lsr_wide, 3331),
SVEMAP1(svlsr_wide_n_u32_x, aarch64_sve_lsr_wide, 3203),
SVEMAP1(svlsr_wide_n_u32_z, aarch64_sve_lsr_wide, 3459),
SVEMAP1(svlsr_wide_n_u8_m, aarch64_sve_lsr_wide, 3329),
SVEMAP1(svlsr_wide_n_u8_x, aarch64_sve_lsr_wide, 3201),
SVEMAP1(svlsr_wide_n_u8_z, aarch64_sve_lsr_wide, 3457),
SVEMAP1(svlsr_wide_u16_m, aarch64_sve_lsr_wide, 258),
SVEMAP1(svlsr_wide_u16_x, aarch64_sve_lsr_wide, 130),
SVEMAP1(svlsr_wide_u16_z, aarch64_sve_lsr_wide, 386),
SVEMAP1(svlsr_wide_u32_m, aarch64_sve_lsr_wide, 259),
SVEMAP1(svlsr_wide_u32_x, aarch64_sve_lsr_wide, 131),
SVEMAP1(svlsr_wide_u32_z, aarch64_sve_lsr_wide, 387),
SVEMAP1(svlsr_wide_u8_m, aarch64_sve_lsr_wide, 257),
SVEMAP1(svlsr_wide_u8_x, aarch64_sve_lsr_wide, 129),
SVEMAP1(svlsr_wide_u8_z, aarch64_sve_lsr_wide, 385),
SVEMAP1(svmad_f16_m, aarch64_sve_fmad, 261),
SVEMAP1(svmad_f16_x, aarch64_sve_fmad, 133),
SVEMAP1(svmad_f16_z, aarch64_sve_fmad, 389),
SVEMAP1(svmad_f32_m, aarch64_sve_fmad, 262),
SVEMAP1(svmad_f32_x, aarch64_sve_fmad, 134),
SVEMAP1(svmad_f32_z, aarch64_sve_fmad, 390),
SVEMAP1(svmad_f64_m, aarch64_sve_fmad, 263),
SVEMAP1(svmad_f64_x, aarch64_sve_fmad, 135),
SVEMAP1(svmad_f64_z, aarch64_sve_fmad, 391),
SVEMAP1(svmad_n_f16_m, aarch64_sve_fmad, 4357),
SVEMAP1(svmad_n_f16_x, aarch64_sve_fmad, 4229),
SVEMAP1(svmad_n_f16_z, aarch64_sve_fmad, 4485),
SVEMAP1(svmad_n_f32_m, aarch64_sve_fmad, 4358),
SVEMAP1(svmad_n_f32_x, aarch64_sve_fmad, 4230),
SVEMAP1(svmad_n_f32_z, aarch64_sve_fmad, 4486),
SVEMAP1(svmad_n_f64_m, aarch64_sve_fmad, 4359),
SVEMAP1(svmad_n_f64_x, aarch64_sve_fmad, 4231),
SVEMAP1(svmad_n_f64_z, aarch64_sve_fmad, 4487),
SVEMAP1(svmad_n_s16_m, aarch64_sve_mad, 4354),
SVEMAP1(svmad_n_s16_x, aarch64_sve_mad, 4226),
SVEMAP1(svmad_n_s16_z, aarch64_sve_mad, 4482),
SVEMAP1(svmad_n_s32_m, aarch64_sve_mad, 4355),
SVEMAP1(svmad_n_s32_x, aarch64_sve_mad, 4227),
SVEMAP1(svmad_n_s32_z, aarch64_sve_mad, 4483),
SVEMAP1(svmad_n_s64_m, aarch64_sve_mad, 4356),
SVEMAP1(svmad_n_s64_x, aarch64_sve_mad, 4228),
SVEMAP1(svmad_n_s64_z, aarch64_sve_mad, 4484),
SVEMAP1(svmad_n_s8_m, aarch64_sve_mad, 4353),
SVEMAP1(svmad_n_s8_x, aarch64_sve_mad, 4225),
SVEMAP1(svmad_n_s8_z, aarch64_sve_mad, 4481),
SVEMAP1(svmad_n_u16_m, aarch64_sve_mad, 4354),
SVEMAP1(svmad_n_u16_x, aarch64_sve_mad, 4226),
SVEMAP1(svmad_n_u16_z, aarch64_sve_mad, 4482),
SVEMAP1(svmad_n_u32_m, aarch64_sve_mad, 4355),
SVEMAP1(svmad_n_u32_x, aarch64_sve_mad, 4227),
SVEMAP1(svmad_n_u32_z, aarch64_sve_mad, 4483),
SVEMAP1(svmad_n_u64_m, aarch64_sve_mad, 4356),
SVEMAP1(svmad_n_u64_x, aarch64_sve_mad, 4228),
SVEMAP1(svmad_n_u64_z, aarch64_sve_mad, 4484),
SVEMAP1(svmad_n_u8_m, aarch64_sve_mad, 4353),
SVEMAP1(svmad_n_u8_x, aarch64_sve_mad, 4225),
SVEMAP1(svmad_n_u8_z, aarch64_sve_mad, 4481),
SVEMAP1(svmad_s16_m, aarch64_sve_mad, 258),
SVEMAP1(svmad_s16_x, aarch64_sve_mad, 130),
SVEMAP1(svmad_s16_z, aarch64_sve_mad, 386),
SVEMAP1(svmad_s32_m, aarch64_sve_mad, 259),
SVEMAP1(svmad_s32_x, aarch64_sve_mad, 131),
SVEMAP1(svmad_s32_z, aarch64_sve_mad, 387),
SVEMAP1(svmad_s64_m, aarch64_sve_mad, 260),
SVEMAP1(svmad_s64_x, aarch64_sve_mad, 132),
SVEMAP1(svmad_s64_z, aarch64_sve_mad, 388),
SVEMAP1(svmad_s8_m, aarch64_sve_mad, 257),
SVEMAP1(svmad_s8_x, aarch64_sve_mad, 129),
SVEMAP1(svmad_s8_z, aarch64_sve_mad, 385),
SVEMAP1(svmad_u16_m, aarch64_sve_mad, 258),
SVEMAP1(svmad_u16_x, aarch64_sve_mad, 130),
SVEMAP1(svmad_u16_z, aarch64_sve_mad, 386),
SVEMAP1(svmad_u32_m, aarch64_sve_mad, 259),
SVEMAP1(svmad_u32_x, aarch64_sve_mad, 131),
SVEMAP1(svmad_u32_z, aarch64_sve_mad, 387),
SVEMAP1(svmad_u64_m, aarch64_sve_mad, 260),
SVEMAP1(svmad_u64_x, aarch64_sve_mad, 132),
SVEMAP1(svmad_u64_z, aarch64_sve_mad, 388),
SVEMAP1(svmad_u8_m, aarch64_sve_mad, 257),
SVEMAP1(svmad_u8_x, aarch64_sve_mad, 129),
SVEMAP1(svmad_u8_z, aarch64_sve_mad, 385),
SVEMAP1(svmatch_s16, aarch64_sve_match, 2),
SVEMAP1(svmatch_s8, aarch64_sve_match, 1),
SVEMAP1(svmatch_u16, aarch64_sve_match, 2),
SVEMAP1(svmatch_u8, aarch64_sve_match, 1),
SVEMAP1(svmax_f16_m, aarch64_sve_fmax, 261),
SVEMAP1(svmax_f16_x, aarch64_sve_fmax, 133),
SVEMAP1(svmax_f16_z, aarch64_sve_fmax, 389),
SVEMAP1(svmax_f32_m, aarch64_sve_fmax, 262),
SVEMAP1(svmax_f32_x, aarch64_sve_fmax, 134),
SVEMAP1(svmax_f32_z, aarch64_sve_fmax, 390),
SVEMAP1(svmax_f64_m, aarch64_sve_fmax, 263),
SVEMAP1(svmax_f64_x, aarch64_sve_fmax, 135),
SVEMAP1(svmax_f64_z, aarch64_sve_fmax, 391),
SVEMAP1(svmax_n_f16_m, aarch64_sve_fmax, 3333),
SVEMAP1(svmax_n_f16_x, aarch64_sve_fmax, 3205),
SVEMAP1(svmax_n_f16_z, aarch64_sve_fmax, 3461),
SVEMAP1(svmax_n_f32_m, aarch64_sve_fmax, 3334),
SVEMAP1(svmax_n_f32_x, aarch64_sve_fmax, 3206),
SVEMAP1(svmax_n_f32_z, aarch64_sve_fmax, 3462),
SVEMAP1(svmax_n_f64_m, aarch64_sve_fmax, 3335),
SVEMAP1(svmax_n_f64_x, aarch64_sve_fmax, 3207),
SVEMAP1(svmax_n_f64_z, aarch64_sve_fmax, 3463),
SVEMAP1(svmax_n_s16_m, aarch64_sve_smax, 3330),
SVEMAP1(svmax_n_s16_x, aarch64_sve_smax, 3202),
SVEMAP1(svmax_n_s16_z, aarch64_sve_smax, 3458),
SVEMAP1(svmax_n_s32_m, aarch64_sve_smax, 3331),
SVEMAP1(svmax_n_s32_x, aarch64_sve_smax, 3203),
SVEMAP1(svmax_n_s32_z, aarch64_sve_smax, 3459),
SVEMAP1(svmax_n_s64_m, aarch64_sve_smax, 3332),
SVEMAP1(svmax_n_s64_x, aarch64_sve_smax, 3204),
SVEMAP1(svmax_n_s64_z, aarch64_sve_smax, 3460),
SVEMAP1(svmax_n_s8_m, aarch64_sve_smax, 3329),
SVEMAP1(svmax_n_s8_x, aarch64_sve_smax, 3201),
SVEMAP1(svmax_n_s8_z, aarch64_sve_smax, 3457),
SVEMAP1(svmax_n_u16_m, aarch64_sve_umax, 3330),
SVEMAP1(svmax_n_u16_x, aarch64_sve_umax, 3202),
SVEMAP1(svmax_n_u16_z, aarch64_sve_umax, 3458),
SVEMAP1(svmax_n_u32_m, aarch64_sve_umax, 3331),
SVEMAP1(svmax_n_u32_x, aarch64_sve_umax, 3203),
SVEMAP1(svmax_n_u32_z, aarch64_sve_umax, 3459),
SVEMAP1(svmax_n_u64_m, aarch64_sve_umax, 3332),
SVEMAP1(svmax_n_u64_x, aarch64_sve_umax, 3204),
SVEMAP1(svmax_n_u64_z, aarch64_sve_umax, 3460),
SVEMAP1(svmax_n_u8_m, aarch64_sve_umax, 3329),
SVEMAP1(svmax_n_u8_x, aarch64_sve_umax, 3201),
SVEMAP1(svmax_n_u8_z, aarch64_sve_umax, 3457),
SVEMAP1(svmax_s16_m, aarch64_sve_smax, 258),
SVEMAP1(svmax_s16_x, aarch64_sve_smax, 130),
SVEMAP1(svmax_s16_z, aarch64_sve_smax, 386),
SVEMAP1(svmax_s32_m, aarch64_sve_smax, 259),
SVEMAP1(svmax_s32_x, aarch64_sve_smax, 131),
SVEMAP1(svmax_s32_z, aarch64_sve_smax, 387),
SVEMAP1(svmax_s64_m, aarch64_sve_smax, 260),
SVEMAP1(svmax_s64_x, aarch64_sve_smax, 132),
SVEMAP1(svmax_s64_z, aarch64_sve_smax, 388),
SVEMAP1(svmax_s8_m, aarch64_sve_smax, 257),
SVEMAP1(svmax_s8_x, aarch64_sve_smax, 129),
SVEMAP1(svmax_s8_z, aarch64_sve_smax, 385),
SVEMAP1(svmax_u16_m, aarch64_sve_umax, 258),
SVEMAP1(svmax_u16_x, aarch64_sve_umax, 130),
SVEMAP1(svmax_u16_z, aarch64_sve_umax, 386),
SVEMAP1(svmax_u32_m, aarch64_sve_umax, 259),
SVEMAP1(svmax_u32_x, aarch64_sve_umax, 131),
SVEMAP1(svmax_u32_z, aarch64_sve_umax, 387),
SVEMAP1(svmax_u64_m, aarch64_sve_umax, 260),
SVEMAP1(svmax_u64_x, aarch64_sve_umax, 132),
SVEMAP1(svmax_u64_z, aarch64_sve_umax, 388),
SVEMAP1(svmax_u8_m, aarch64_sve_umax, 257),
SVEMAP1(svmax_u8_x, aarch64_sve_umax, 129),
SVEMAP1(svmax_u8_z, aarch64_sve_umax, 385),
SVEMAP1(svmaxnm_f16_m, aarch64_sve_fmaxnm, 261),
SVEMAP1(svmaxnm_f16_x, aarch64_sve_fmaxnm, 133),
SVEMAP1(svmaxnm_f16_z, aarch64_sve_fmaxnm, 389),
SVEMAP1(svmaxnm_f32_m, aarch64_sve_fmaxnm, 262),
SVEMAP1(svmaxnm_f32_x, aarch64_sve_fmaxnm, 134),
SVEMAP1(svmaxnm_f32_z, aarch64_sve_fmaxnm, 390),
SVEMAP1(svmaxnm_f64_m, aarch64_sve_fmaxnm, 263),
SVEMAP1(svmaxnm_f64_x, aarch64_sve_fmaxnm, 135),
SVEMAP1(svmaxnm_f64_z, aarch64_sve_fmaxnm, 391),
SVEMAP1(svmaxnm_n_f16_m, aarch64_sve_fmaxnm, 3333),
SVEMAP1(svmaxnm_n_f16_x, aarch64_sve_fmaxnm, 3205),
SVEMAP1(svmaxnm_n_f16_z, aarch64_sve_fmaxnm, 3461),
SVEMAP1(svmaxnm_n_f32_m, aarch64_sve_fmaxnm, 3334),
SVEMAP1(svmaxnm_n_f32_x, aarch64_sve_fmaxnm, 3206),
SVEMAP1(svmaxnm_n_f32_z, aarch64_sve_fmaxnm, 3462),
SVEMAP1(svmaxnm_n_f64_m, aarch64_sve_fmaxnm, 3335),
SVEMAP1(svmaxnm_n_f64_x, aarch64_sve_fmaxnm, 3207),
SVEMAP1(svmaxnm_n_f64_z, aarch64_sve_fmaxnm, 3463),
SVEMAP1(svmaxnmp_f16_m, aarch64_sve_fmaxnmp, 261),
SVEMAP1(svmaxnmp_f16_x, aarch64_sve_fmaxnmp, 133),
SVEMAP1(svmaxnmp_f32_m, aarch64_sve_fmaxnmp, 262),
SVEMAP1(svmaxnmp_f32_x, aarch64_sve_fmaxnmp, 134),
SVEMAP1(svmaxnmp_f64_m, aarch64_sve_fmaxnmp, 263),
SVEMAP1(svmaxnmp_f64_x, aarch64_sve_fmaxnmp, 135),
SVEMAP1(svmaxnmv_f16, aarch64_sve_fmaxnmv, 5),
SVEMAP1(svmaxnmv_f32, aarch64_sve_fmaxnmv, 6),
SVEMAP1(svmaxnmv_f64, aarch64_sve_fmaxnmv, 7),
SVEMAP1(svmaxp_f16_m, aarch64_sve_fmaxp, 261),
SVEMAP1(svmaxp_f16_x, aarch64_sve_fmaxp, 133),
SVEMAP1(svmaxp_f32_m, aarch64_sve_fmaxp, 262),
SVEMAP1(svmaxp_f32_x, aarch64_sve_fmaxp, 134),
SVEMAP1(svmaxp_f64_m, aarch64_sve_fmaxp, 263),
SVEMAP1(svmaxp_f64_x, aarch64_sve_fmaxp, 135),
SVEMAP1(svmaxp_s16_m, aarch64_sve_smaxp, 258),
SVEMAP1(svmaxp_s16_x, aarch64_sve_smaxp, 130),
SVEMAP1(svmaxp_s32_m, aarch64_sve_smaxp, 259),
SVEMAP1(svmaxp_s32_x, aarch64_sve_smaxp, 131),
SVEMAP1(svmaxp_s64_m, aarch64_sve_smaxp, 260),
SVEMAP1(svmaxp_s64_x, aarch64_sve_smaxp, 132),
SVEMAP1(svmaxp_s8_m, aarch64_sve_smaxp, 257),
SVEMAP1(svmaxp_s8_x, aarch64_sve_smaxp, 129),
SVEMAP1(svmaxp_u16_m, aarch64_sve_umaxp, 258),
SVEMAP1(svmaxp_u16_x, aarch64_sve_umaxp, 130),
SVEMAP1(svmaxp_u32_m, aarch64_sve_umaxp, 259),
SVEMAP1(svmaxp_u32_x, aarch64_sve_umaxp, 131),
SVEMAP1(svmaxp_u64_m, aarch64_sve_umaxp, 260),
SVEMAP1(svmaxp_u64_x, aarch64_sve_umaxp, 132),
SVEMAP1(svmaxp_u8_m, aarch64_sve_umaxp, 257),
SVEMAP1(svmaxp_u8_x, aarch64_sve_umaxp, 129),
SVEMAP1(svmaxv_f16, aarch64_sve_fmaxv, 5),
SVEMAP1(svmaxv_f32, aarch64_sve_fmaxv, 6),
SVEMAP1(svmaxv_f64, aarch64_sve_fmaxv, 7),
SVEMAP1(svmaxv_s16, aarch64_sve_smaxv, 2),
SVEMAP1(svmaxv_s32, aarch64_sve_smaxv, 3),
SVEMAP1(svmaxv_s64, aarch64_sve_smaxv, 4),
SVEMAP1(svmaxv_s8, aarch64_sve_smaxv, 1),
SVEMAP1(svmaxv_u16, aarch64_sve_umaxv, 2),
SVEMAP1(svmaxv_u32, aarch64_sve_umaxv, 3),
SVEMAP1(svmaxv_u64, aarch64_sve_umaxv, 4),
SVEMAP1(svmaxv_u8, aarch64_sve_umaxv, 1),
SVEMAP1(svmin_f16_m, aarch64_sve_fmin, 261),
SVEMAP1(svmin_f16_x, aarch64_sve_fmin, 133),
SVEMAP1(svmin_f16_z, aarch64_sve_fmin, 389),
SVEMAP1(svmin_f32_m, aarch64_sve_fmin, 262),
SVEMAP1(svmin_f32_x, aarch64_sve_fmin, 134),
SVEMAP1(svmin_f32_z, aarch64_sve_fmin, 390),
SVEMAP1(svmin_f64_m, aarch64_sve_fmin, 263),
SVEMAP1(svmin_f64_x, aarch64_sve_fmin, 135),
SVEMAP1(svmin_f64_z, aarch64_sve_fmin, 391),
SVEMAP1(svmin_n_f16_m, aarch64_sve_fmin, 3333),
SVEMAP1(svmin_n_f16_x, aarch64_sve_fmin, 3205),
SVEMAP1(svmin_n_f16_z, aarch64_sve_fmin, 3461),
SVEMAP1(svmin_n_f32_m, aarch64_sve_fmin, 3334),
SVEMAP1(svmin_n_f32_x, aarch64_sve_fmin, 3206),
SVEMAP1(svmin_n_f32_z, aarch64_sve_fmin, 3462),
SVEMAP1(svmin_n_f64_m, aarch64_sve_fmin, 3335),
SVEMAP1(svmin_n_f64_x, aarch64_sve_fmin, 3207),
SVEMAP1(svmin_n_f64_z, aarch64_sve_fmin, 3463),
SVEMAP1(svmin_n_s16_m, aarch64_sve_smin, 3330),
SVEMAP1(svmin_n_s16_x, aarch64_sve_smin, 3202),
SVEMAP1(svmin_n_s16_z, aarch64_sve_smin, 3458),
SVEMAP1(svmin_n_s32_m, aarch64_sve_smin, 3331),
SVEMAP1(svmin_n_s32_x, aarch64_sve_smin, 3203),
SVEMAP1(svmin_n_s32_z, aarch64_sve_smin, 3459),
SVEMAP1(svmin_n_s64_m, aarch64_sve_smin, 3332),
SVEMAP1(svmin_n_s64_x, aarch64_sve_smin, 3204),
SVEMAP1(svmin_n_s64_z, aarch64_sve_smin, 3460),
SVEMAP1(svmin_n_s8_m, aarch64_sve_smin, 3329),
SVEMAP1(svmin_n_s8_x, aarch64_sve_smin, 3201),
SVEMAP1(svmin_n_s8_z, aarch64_sve_smin, 3457),
SVEMAP1(svmin_n_u16_m, aarch64_sve_umin, 3330),
SVEMAP1(svmin_n_u16_x, aarch64_sve_umin, 3202),
SVEMAP1(svmin_n_u16_z, aarch64_sve_umin, 3458),
SVEMAP1(svmin_n_u32_m, aarch64_sve_umin, 3331),
SVEMAP1(svmin_n_u32_x, aarch64_sve_umin, 3203),
SVEMAP1(svmin_n_u32_z, aarch64_sve_umin, 3459),
SVEMAP1(svmin_n_u64_m, aarch64_sve_umin, 3332),
SVEMAP1(svmin_n_u64_x, aarch64_sve_umin, 3204),
SVEMAP1(svmin_n_u64_z, aarch64_sve_umin, 3460),
SVEMAP1(svmin_n_u8_m, aarch64_sve_umin, 3329),
SVEMAP1(svmin_n_u8_x, aarch64_sve_umin, 3201),
SVEMAP1(svmin_n_u8_z, aarch64_sve_umin, 3457),
SVEMAP1(svmin_s16_m, aarch64_sve_smin, 258),
SVEMAP1(svmin_s16_x, aarch64_sve_smin, 130),
SVEMAP1(svmin_s16_z, aarch64_sve_smin, 386),
SVEMAP1(svmin_s32_m, aarch64_sve_smin, 259),
SVEMAP1(svmin_s32_x, aarch64_sve_smin, 131),
SVEMAP1(svmin_s32_z, aarch64_sve_smin, 387),
SVEMAP1(svmin_s64_m, aarch64_sve_smin, 260),
SVEMAP1(svmin_s64_x, aarch64_sve_smin, 132),
SVEMAP1(svmin_s64_z, aarch64_sve_smin, 388),
SVEMAP1(svmin_s8_m, aarch64_sve_smin, 257),
SVEMAP1(svmin_s8_x, aarch64_sve_smin, 129),
SVEMAP1(svmin_s8_z, aarch64_sve_smin, 385),
SVEMAP1(svmin_u16_m, aarch64_sve_umin, 258),
SVEMAP1(svmin_u16_x, aarch64_sve_umin, 130),
SVEMAP1(svmin_u16_z, aarch64_sve_umin, 386),
SVEMAP1(svmin_u32_m, aarch64_sve_umin, 259),
SVEMAP1(svmin_u32_x, aarch64_sve_umin, 131),
SVEMAP1(svmin_u32_z, aarch64_sve_umin, 387),
SVEMAP1(svmin_u64_m, aarch64_sve_umin, 260),
SVEMAP1(svmin_u64_x, aarch64_sve_umin, 132),
SVEMAP1(svmin_u64_z, aarch64_sve_umin, 388),
SVEMAP1(svmin_u8_m, aarch64_sve_umin, 257),
SVEMAP1(svmin_u8_x, aarch64_sve_umin, 129),
SVEMAP1(svmin_u8_z, aarch64_sve_umin, 385),
SVEMAP1(svminnm_f16_m, aarch64_sve_fminnm, 261),
SVEMAP1(svminnm_f16_x, aarch64_sve_fminnm, 133),
SVEMAP1(svminnm_f16_z, aarch64_sve_fminnm, 389),
SVEMAP1(svminnm_f32_m, aarch64_sve_fminnm, 262),
SVEMAP1(svminnm_f32_x, aarch64_sve_fminnm, 134),
SVEMAP1(svminnm_f32_z, aarch64_sve_fminnm, 390),
SVEMAP1(svminnm_f64_m, aarch64_sve_fminnm, 263),
SVEMAP1(svminnm_f64_x, aarch64_sve_fminnm, 135),
SVEMAP1(svminnm_f64_z, aarch64_sve_fminnm, 391),
SVEMAP1(svminnm_n_f16_m, aarch64_sve_fminnm, 3333),
SVEMAP1(svminnm_n_f16_x, aarch64_sve_fminnm, 3205),
SVEMAP1(svminnm_n_f16_z, aarch64_sve_fminnm, 3461),
SVEMAP1(svminnm_n_f32_m, aarch64_sve_fminnm, 3334),
SVEMAP1(svminnm_n_f32_x, aarch64_sve_fminnm, 3206),
SVEMAP1(svminnm_n_f32_z, aarch64_sve_fminnm, 3462),
SVEMAP1(svminnm_n_f64_m, aarch64_sve_fminnm, 3335),
SVEMAP1(svminnm_n_f64_x, aarch64_sve_fminnm, 3207),
SVEMAP1(svminnm_n_f64_z, aarch64_sve_fminnm, 3463),
SVEMAP1(svminnmp_f16_m, aarch64_sve_fminnmp, 261),
SVEMAP1(svminnmp_f16_x, aarch64_sve_fminnmp, 133),
SVEMAP1(svminnmp_f32_m, aarch64_sve_fminnmp, 262),
SVEMAP1(svminnmp_f32_x, aarch64_sve_fminnmp, 134),
SVEMAP1(svminnmp_f64_m, aarch64_sve_fminnmp, 263),
SVEMAP1(svminnmp_f64_x, aarch64_sve_fminnmp, 135),
SVEMAP1(svminnmv_f16, aarch64_sve_fminnmv, 5),
SVEMAP1(svminnmv_f32, aarch64_sve_fminnmv, 6),
SVEMAP1(svminnmv_f64, aarch64_sve_fminnmv, 7),
SVEMAP1(svminp_f16_m, aarch64_sve_fminp, 261),
SVEMAP1(svminp_f16_x, aarch64_sve_fminp, 133),
SVEMAP1(svminp_f32_m, aarch64_sve_fminp, 262),
SVEMAP1(svminp_f32_x, aarch64_sve_fminp, 134),
SVEMAP1(svminp_f64_m, aarch64_sve_fminp, 263),
SVEMAP1(svminp_f64_x, aarch64_sve_fminp, 135),
SVEMAP1(svminp_s16_m, aarch64_sve_sminp, 258),
SVEMAP1(svminp_s16_x, aarch64_sve_sminp, 130),
SVEMAP1(svminp_s32_m, aarch64_sve_sminp, 259),
SVEMAP1(svminp_s32_x, aarch64_sve_sminp, 131),
SVEMAP1(svminp_s64_m, aarch64_sve_sminp, 260),
SVEMAP1(svminp_s64_x, aarch64_sve_sminp, 132),
SVEMAP1(svminp_s8_m, aarch64_sve_sminp, 257),
SVEMAP1(svminp_s8_x, aarch64_sve_sminp, 129),
SVEMAP1(svminp_u16_m, aarch64_sve_uminp, 258),
SVEMAP1(svminp_u16_x, aarch64_sve_uminp, 130),
SVEMAP1(svminp_u32_m, aarch64_sve_uminp, 259),
SVEMAP1(svminp_u32_x, aarch64_sve_uminp, 131),
SVEMAP1(svminp_u64_m, aarch64_sve_uminp, 260),
SVEMAP1(svminp_u64_x, aarch64_sve_uminp, 132),
SVEMAP1(svminp_u8_m, aarch64_sve_uminp, 257),
SVEMAP1(svminp_u8_x, aarch64_sve_uminp, 129),
SVEMAP1(svminv_f16, aarch64_sve_fminv, 5),
SVEMAP1(svminv_f32, aarch64_sve_fminv, 6),
SVEMAP1(svminv_f64, aarch64_sve_fminv, 7),
SVEMAP1(svminv_s16, aarch64_sve_sminv, 2),
SVEMAP1(svminv_s32, aarch64_sve_sminv, 3),
SVEMAP1(svminv_s64, aarch64_sve_sminv, 4),
SVEMAP1(svminv_s8, aarch64_sve_sminv, 1),
SVEMAP1(svminv_u16, aarch64_sve_uminv, 2),
SVEMAP1(svminv_u32, aarch64_sve_uminv, 3),
SVEMAP1(svminv_u64, aarch64_sve_uminv, 4),
SVEMAP1(svminv_u8, aarch64_sve_uminv, 1),
SVEMAP1(svmla_f16_m, aarch64_sve_fmla, 261),
SVEMAP1(svmla_f16_x, aarch64_sve_fmla, 133),
SVEMAP1(svmla_f16_z, aarch64_sve_fmla, 389),
SVEMAP1(svmla_f32_m, aarch64_sve_fmla, 262),
SVEMAP1(svmla_f32_x, aarch64_sve_fmla, 134),
SVEMAP1(svmla_f32_z, aarch64_sve_fmla, 390),
SVEMAP1(svmla_f64_m, aarch64_sve_fmla, 263),
SVEMAP1(svmla_f64_x, aarch64_sve_fmla, 135),
SVEMAP1(svmla_f64_z, aarch64_sve_fmla, 391),
SVEMAP1(svmla_lane_f16, aarch64_sve_fmla_lane, 5),
SVEMAP1(svmla_lane_f32, aarch64_sve_fmla_lane, 6),
SVEMAP1(svmla_lane_f64, aarch64_sve_fmla_lane, 7),
SVEMAP1(svmla_lane_s16, aarch64_sve_mla_lane, 2),
SVEMAP1(svmla_lane_s32, aarch64_sve_mla_lane, 3),
SVEMAP1(svmla_lane_s64, aarch64_sve_mla_lane, 4),
SVEMAP1(svmla_lane_u16, aarch64_sve_mla_lane, 2),
SVEMAP1(svmla_lane_u32, aarch64_sve_mla_lane, 3),
SVEMAP1(svmla_lane_u64, aarch64_sve_mla_lane, 4),
SVEMAP1(svmla_n_f16_m, aarch64_sve_fmla, 4357),
SVEMAP1(svmla_n_f16_x, aarch64_sve_fmla, 4229),
SVEMAP1(svmla_n_f16_z, aarch64_sve_fmla, 4485),
SVEMAP1(svmla_n_f32_m, aarch64_sve_fmla, 4358),
SVEMAP1(svmla_n_f32_x, aarch64_sve_fmla, 4230),
SVEMAP1(svmla_n_f32_z, aarch64_sve_fmla, 4486),
SVEMAP1(svmla_n_f64_m, aarch64_sve_fmla, 4359),
SVEMAP1(svmla_n_f64_x, aarch64_sve_fmla, 4231),
SVEMAP1(svmla_n_f64_z, aarch64_sve_fmla, 4487),
SVEMAP1(svmla_n_s16_m, aarch64_sve_mla, 4354),
SVEMAP1(svmla_n_s16_x, aarch64_sve_mla, 4226),
SVEMAP1(svmla_n_s16_z, aarch64_sve_mla, 4482),
SVEMAP1(svmla_n_s32_m, aarch64_sve_mla, 4355),
SVEMAP1(svmla_n_s32_x, aarch64_sve_mla, 4227),
SVEMAP1(svmla_n_s32_z, aarch64_sve_mla, 4483),
SVEMAP1(svmla_n_s64_m, aarch64_sve_mla, 4356),
SVEMAP1(svmla_n_s64_x, aarch64_sve_mla, 4228),
SVEMAP1(svmla_n_s64_z, aarch64_sve_mla, 4484),
SVEMAP1(svmla_n_s8_m, aarch64_sve_mla, 4353),
SVEMAP1(svmla_n_s8_x, aarch64_sve_mla, 4225),
SVEMAP1(svmla_n_s8_z, aarch64_sve_mla, 4481),
SVEMAP1(svmla_n_u16_m, aarch64_sve_mla, 4354),
SVEMAP1(svmla_n_u16_x, aarch64_sve_mla, 4226),
SVEMAP1(svmla_n_u16_z, aarch64_sve_mla, 4482),
SVEMAP1(svmla_n_u32_m, aarch64_sve_mla, 4355),
SVEMAP1(svmla_n_u32_x, aarch64_sve_mla, 4227),
SVEMAP1(svmla_n_u32_z, aarch64_sve_mla, 4483),
SVEMAP1(svmla_n_u64_m, aarch64_sve_mla, 4356),
SVEMAP1(svmla_n_u64_x, aarch64_sve_mla, 4228),
SVEMAP1(svmla_n_u64_z, aarch64_sve_mla, 4484),
SVEMAP1(svmla_n_u8_m, aarch64_sve_mla, 4353),
SVEMAP1(svmla_n_u8_x, aarch64_sve_mla, 4225),
SVEMAP1(svmla_n_u8_z, aarch64_sve_mla, 4481),
SVEMAP1(svmla_s16_m, aarch64_sve_mla, 258),
SVEMAP1(svmla_s16_x, aarch64_sve_mla, 130),
SVEMAP1(svmla_s16_z, aarch64_sve_mla, 386),
SVEMAP1(svmla_s32_m, aarch64_sve_mla, 259),
SVEMAP1(svmla_s32_x, aarch64_sve_mla, 131),
SVEMAP1(svmla_s32_z, aarch64_sve_mla, 387),
SVEMAP1(svmla_s64_m, aarch64_sve_mla, 260),
SVEMAP1(svmla_s64_x, aarch64_sve_mla, 132),
SVEMAP1(svmla_s64_z, aarch64_sve_mla, 388),
SVEMAP1(svmla_s8_m, aarch64_sve_mla, 257),
SVEMAP1(svmla_s8_x, aarch64_sve_mla, 129),
SVEMAP1(svmla_s8_z, aarch64_sve_mla, 385),
SVEMAP1(svmla_u16_m, aarch64_sve_mla, 258),
SVEMAP1(svmla_u16_x, aarch64_sve_mla, 130),
SVEMAP1(svmla_u16_z, aarch64_sve_mla, 386),
SVEMAP1(svmla_u32_m, aarch64_sve_mla, 259),
SVEMAP1(svmla_u32_x, aarch64_sve_mla, 131),
SVEMAP1(svmla_u32_z, aarch64_sve_mla, 387),
SVEMAP1(svmla_u64_m, aarch64_sve_mla, 260),
SVEMAP1(svmla_u64_x, aarch64_sve_mla, 132),
SVEMAP1(svmla_u64_z, aarch64_sve_mla, 388),
SVEMAP1(svmla_u8_m, aarch64_sve_mla, 257),
SVEMAP1(svmla_u8_x, aarch64_sve_mla, 129),
SVEMAP1(svmla_u8_z, aarch64_sve_mla, 385),
SVEMAP1(svmlalb_f32, aarch64_sve_fmlalb, 6),
SVEMAP1(svmlalb_lane_f32, aarch64_sve_fmlalb_lane, 6),
SVEMAP1(svmlalb_lane_s32, aarch64_sve_smlalb_lane, 3),
SVEMAP1(svmlalb_lane_s64, aarch64_sve_smlalb_lane, 4),
SVEMAP1(svmlalb_lane_u32, aarch64_sve_umlalb_lane, 3),
SVEMAP1(svmlalb_lane_u64, aarch64_sve_umlalb_lane, 4),
SVEMAP1(svmlalb_n_f32, aarch64_sve_fmlalb, 3078),
SVEMAP1(svmlalb_n_s16, aarch64_sve_smlalb, 3074),
SVEMAP1(svmlalb_n_s32, aarch64_sve_smlalb, 3075),
SVEMAP1(svmlalb_n_s64, aarch64_sve_smlalb, 3076),
SVEMAP1(svmlalb_n_u16, aarch64_sve_umlalb, 3074),
SVEMAP1(svmlalb_n_u32, aarch64_sve_umlalb, 3075),
SVEMAP1(svmlalb_n_u64, aarch64_sve_umlalb, 3076),
SVEMAP1(svmlalb_s16, aarch64_sve_smlalb, 2),
SVEMAP1(svmlalb_s32, aarch64_sve_smlalb, 3),
SVEMAP1(svmlalb_s64, aarch64_sve_smlalb, 4),
SVEMAP1(svmlalb_u16, aarch64_sve_umlalb, 2),
SVEMAP1(svmlalb_u32, aarch64_sve_umlalb, 3),
SVEMAP1(svmlalb_u64, aarch64_sve_umlalb, 4),
SVEMAP1(svmlalt_f32, aarch64_sve_fmlalt, 6),
SVEMAP1(svmlalt_lane_f32, aarch64_sve_fmlalt_lane, 6),
SVEMAP1(svmlalt_lane_s32, aarch64_sve_smlalt_lane, 3),
SVEMAP1(svmlalt_lane_s64, aarch64_sve_smlalt_lane, 4),
SVEMAP1(svmlalt_lane_u32, aarch64_sve_umlalt_lane, 3),
SVEMAP1(svmlalt_lane_u64, aarch64_sve_umlalt_lane, 4),
SVEMAP1(svmlalt_n_f32, aarch64_sve_fmlalt, 3078),
SVEMAP1(svmlalt_n_s16, aarch64_sve_smlalt, 3074),
SVEMAP1(svmlalt_n_s32, aarch64_sve_smlalt, 3075),
SVEMAP1(svmlalt_n_s64, aarch64_sve_smlalt, 3076),
SVEMAP1(svmlalt_n_u16, aarch64_sve_umlalt, 3074),
SVEMAP1(svmlalt_n_u32, aarch64_sve_umlalt, 3075),
SVEMAP1(svmlalt_n_u64, aarch64_sve_umlalt, 3076),
SVEMAP1(svmlalt_s16, aarch64_sve_smlalt, 2),
SVEMAP1(svmlalt_s32, aarch64_sve_smlalt, 3),
SVEMAP1(svmlalt_s64, aarch64_sve_smlalt, 4),
SVEMAP1(svmlalt_u16, aarch64_sve_umlalt, 2),
SVEMAP1(svmlalt_u32, aarch64_sve_umlalt, 3),
SVEMAP1(svmlalt_u64, aarch64_sve_umlalt, 4),
SVEMAP1(svmls_f16_m, aarch64_sve_fmls, 261),
SVEMAP1(svmls_f16_x, aarch64_sve_fmls, 133),
SVEMAP1(svmls_f16_z, aarch64_sve_fmls, 389),
SVEMAP1(svmls_f32_m, aarch64_sve_fmls, 262),
SVEMAP1(svmls_f32_x, aarch64_sve_fmls, 134),
SVEMAP1(svmls_f32_z, aarch64_sve_fmls, 390),
SVEMAP1(svmls_f64_m, aarch64_sve_fmls, 263),
SVEMAP1(svmls_f64_x, aarch64_sve_fmls, 135),
SVEMAP1(svmls_f64_z, aarch64_sve_fmls, 391),
SVEMAP1(svmls_lane_f16, aarch64_sve_fmls_lane, 5),
SVEMAP1(svmls_lane_f32, aarch64_sve_fmls_lane, 6),
SVEMAP1(svmls_lane_f64, aarch64_sve_fmls_lane, 7),
SVEMAP1(svmls_lane_s16, aarch64_sve_mls_lane, 2),
SVEMAP1(svmls_lane_s32, aarch64_sve_mls_lane, 3),
SVEMAP1(svmls_lane_s64, aarch64_sve_mls_lane, 4),
SVEMAP1(svmls_lane_u16, aarch64_sve_mls_lane, 2),
SVEMAP1(svmls_lane_u32, aarch64_sve_mls_lane, 3),
SVEMAP1(svmls_lane_u64, aarch64_sve_mls_lane, 4),
SVEMAP1(svmls_n_f16_m, aarch64_sve_fmls, 4357),
SVEMAP1(svmls_n_f16_x, aarch64_sve_fmls, 4229),
SVEMAP1(svmls_n_f16_z, aarch64_sve_fmls, 4485),
SVEMAP1(svmls_n_f32_m, aarch64_sve_fmls, 4358),
SVEMAP1(svmls_n_f32_x, aarch64_sve_fmls, 4230),
SVEMAP1(svmls_n_f32_z, aarch64_sve_fmls, 4486),
SVEMAP1(svmls_n_f64_m, aarch64_sve_fmls, 4359),
SVEMAP1(svmls_n_f64_x, aarch64_sve_fmls, 4231),
SVEMAP1(svmls_n_f64_z, aarch64_sve_fmls, 4487),
SVEMAP1(svmls_n_s16_m, aarch64_sve_mls, 4354),
SVEMAP1(svmls_n_s16_x, aarch64_sve_mls, 4226),
SVEMAP1(svmls_n_s16_z, aarch64_sve_mls, 4482),
SVEMAP1(svmls_n_s32_m, aarch64_sve_mls, 4355),
SVEMAP1(svmls_n_s32_x, aarch64_sve_mls, 4227),
SVEMAP1(svmls_n_s32_z, aarch64_sve_mls, 4483),
SVEMAP1(svmls_n_s64_m, aarch64_sve_mls, 4356),
SVEMAP1(svmls_n_s64_x, aarch64_sve_mls, 4228),
SVEMAP1(svmls_n_s64_z, aarch64_sve_mls, 4484),
SVEMAP1(svmls_n_s8_m, aarch64_sve_mls, 4353),
SVEMAP1(svmls_n_s8_x, aarch64_sve_mls, 4225),
SVEMAP1(svmls_n_s8_z, aarch64_sve_mls, 4481),
SVEMAP1(svmls_n_u16_m, aarch64_sve_mls, 4354),
SVEMAP1(svmls_n_u16_x, aarch64_sve_mls, 4226),
SVEMAP1(svmls_n_u16_z, aarch64_sve_mls, 4482),
SVEMAP1(svmls_n_u32_m, aarch64_sve_mls, 4355),
SVEMAP1(svmls_n_u32_x, aarch64_sve_mls, 4227),
SVEMAP1(svmls_n_u32_z, aarch64_sve_mls, 4483),
SVEMAP1(svmls_n_u64_m, aarch64_sve_mls, 4356),
SVEMAP1(svmls_n_u64_x, aarch64_sve_mls, 4228),
SVEMAP1(svmls_n_u64_z, aarch64_sve_mls, 4484),
SVEMAP1(svmls_n_u8_m, aarch64_sve_mls, 4353),
SVEMAP1(svmls_n_u8_x, aarch64_sve_mls, 4225),
SVEMAP1(svmls_n_u8_z, aarch64_sve_mls, 4481),
SVEMAP1(svmls_s16_m, aarch64_sve_mls, 258),
SVEMAP1(svmls_s16_x, aarch64_sve_mls, 130),
SVEMAP1(svmls_s16_z, aarch64_sve_mls, 386),
SVEMAP1(svmls_s32_m, aarch64_sve_mls, 259),
SVEMAP1(svmls_s32_x, aarch64_sve_mls, 131),
SVEMAP1(svmls_s32_z, aarch64_sve_mls, 387),
SVEMAP1(svmls_s64_m, aarch64_sve_mls, 260),
SVEMAP1(svmls_s64_x, aarch64_sve_mls, 132),
SVEMAP1(svmls_s64_z, aarch64_sve_mls, 388),
SVEMAP1(svmls_s8_m, aarch64_sve_mls, 257),
SVEMAP1(svmls_s8_x, aarch64_sve_mls, 129),
SVEMAP1(svmls_s8_z, aarch64_sve_mls, 385),
SVEMAP1(svmls_u16_m, aarch64_sve_mls, 258),
SVEMAP1(svmls_u16_x, aarch64_sve_mls, 130),
SVEMAP1(svmls_u16_z, aarch64_sve_mls, 386),
SVEMAP1(svmls_u32_m, aarch64_sve_mls, 259),
SVEMAP1(svmls_u32_x, aarch64_sve_mls, 131),
SVEMAP1(svmls_u32_z, aarch64_sve_mls, 387),
SVEMAP1(svmls_u64_m, aarch64_sve_mls, 260),
SVEMAP1(svmls_u64_x, aarch64_sve_mls, 132),
SVEMAP1(svmls_u64_z, aarch64_sve_mls, 388),
SVEMAP1(svmls_u8_m, aarch64_sve_mls, 257),
SVEMAP1(svmls_u8_x, aarch64_sve_mls, 129),
SVEMAP1(svmls_u8_z, aarch64_sve_mls, 385),
SVEMAP1(svmlslb_f32, aarch64_sve_fmlslb, 6),
SVEMAP1(svmlslb_lane_f32, aarch64_sve_fmlslb_lane, 6),
SVEMAP1(svmlslb_lane_s32, aarch64_sve_smlslb_lane, 3),
SVEMAP1(svmlslb_lane_s64, aarch64_sve_smlslb_lane, 4),
SVEMAP1(svmlslb_lane_u32, aarch64_sve_umlslb_lane, 3),
SVEMAP1(svmlslb_lane_u64, aarch64_sve_umlslb_lane, 4),
SVEMAP1(svmlslb_n_f32, aarch64_sve_fmlslb, 3078),
SVEMAP1(svmlslb_n_s16, aarch64_sve_smlslb, 3074),
SVEMAP1(svmlslb_n_s32, aarch64_sve_smlslb, 3075),
SVEMAP1(svmlslb_n_s64, aarch64_sve_smlslb, 3076),
SVEMAP1(svmlslb_n_u16, aarch64_sve_umlslb, 3074),
SVEMAP1(svmlslb_n_u32, aarch64_sve_umlslb, 3075),
SVEMAP1(svmlslb_n_u64, aarch64_sve_umlslb, 3076),
SVEMAP1(svmlslb_s16, aarch64_sve_smlslb, 2),
SVEMAP1(svmlslb_s32, aarch64_sve_smlslb, 3),
SVEMAP1(svmlslb_s64, aarch64_sve_smlslb, 4),
SVEMAP1(svmlslb_u16, aarch64_sve_umlslb, 2),
SVEMAP1(svmlslb_u32, aarch64_sve_umlslb, 3),
SVEMAP1(svmlslb_u64, aarch64_sve_umlslb, 4),
SVEMAP1(svmlslt_f32, aarch64_sve_fmlslt, 6),
SVEMAP1(svmlslt_lane_f32, aarch64_sve_fmlslt_lane, 6),
SVEMAP1(svmlslt_lane_s32, aarch64_sve_smlslt_lane, 3),
SVEMAP1(svmlslt_lane_s64, aarch64_sve_smlslt_lane, 4),
SVEMAP1(svmlslt_lane_u32, aarch64_sve_umlslt_lane, 3),
SVEMAP1(svmlslt_lane_u64, aarch64_sve_umlslt_lane, 4),
SVEMAP1(svmlslt_n_f32, aarch64_sve_fmlslt, 3078),
SVEMAP1(svmlslt_n_s16, aarch64_sve_smlslt, 3074),
SVEMAP1(svmlslt_n_s32, aarch64_sve_smlslt, 3075),
SVEMAP1(svmlslt_n_s64, aarch64_sve_smlslt, 3076),
SVEMAP1(svmlslt_n_u16, aarch64_sve_umlslt, 3074),
SVEMAP1(svmlslt_n_u32, aarch64_sve_umlslt, 3075),
SVEMAP1(svmlslt_n_u64, aarch64_sve_umlslt, 3076),
SVEMAP1(svmlslt_s16, aarch64_sve_smlslt, 2),
SVEMAP1(svmlslt_s32, aarch64_sve_smlslt, 3),
SVEMAP1(svmlslt_s64, aarch64_sve_smlslt, 4),
SVEMAP1(svmlslt_u16, aarch64_sve_umlslt, 2),
SVEMAP1(svmlslt_u32, aarch64_sve_umlslt, 3),
SVEMAP1(svmlslt_u64, aarch64_sve_umlslt, 4),
SVEMAP1(svmmla_f32, aarch64_sve_fmmla, 6),
SVEMAP1(svmmla_f64, aarch64_sve_fmmla, 7),
SVEMAP1(svmmla_s32, aarch64_sve_smmla, 3),
SVEMAP1(svmmla_u32, aarch64_sve_ummla, 3),
SVEMAP2(svmov_b_z, 8),
SVEMAP2(svmovlb_s16, 2),
SVEMAP2(svmovlb_s32, 3),
SVEMAP2(svmovlb_s64, 4),
SVEMAP2(svmovlb_u16, 2),
SVEMAP2(svmovlb_u32, 3),
SVEMAP2(svmovlb_u64, 4),
SVEMAP2(svmovlt_s16, 2),
SVEMAP2(svmovlt_s32, 3),
SVEMAP2(svmovlt_s64, 4),
SVEMAP2(svmovlt_u16, 2),
SVEMAP2(svmovlt_u32, 3),
SVEMAP2(svmovlt_u64, 4),
SVEMAP1(svmsb_f16_m, aarch64_sve_fmsb, 261),
SVEMAP1(svmsb_f16_x, aarch64_sve_fmsb, 133),
SVEMAP1(svmsb_f16_z, aarch64_sve_fmsb, 389),
SVEMAP1(svmsb_f32_m, aarch64_sve_fmsb, 262),
SVEMAP1(svmsb_f32_x, aarch64_sve_fmsb, 134),
SVEMAP1(svmsb_f32_z, aarch64_sve_fmsb, 390),
SVEMAP1(svmsb_f64_m, aarch64_sve_fmsb, 263),
SVEMAP1(svmsb_f64_x, aarch64_sve_fmsb, 135),
SVEMAP1(svmsb_f64_z, aarch64_sve_fmsb, 391),
SVEMAP1(svmsb_n_f16_m, aarch64_sve_fmsb, 4357),
SVEMAP1(svmsb_n_f16_x, aarch64_sve_fmsb, 4229),
SVEMAP1(svmsb_n_f16_z, aarch64_sve_fmsb, 4485),
SVEMAP1(svmsb_n_f32_m, aarch64_sve_fmsb, 4358),
SVEMAP1(svmsb_n_f32_x, aarch64_sve_fmsb, 4230),
SVEMAP1(svmsb_n_f32_z, aarch64_sve_fmsb, 4486),
SVEMAP1(svmsb_n_f64_m, aarch64_sve_fmsb, 4359),
SVEMAP1(svmsb_n_f64_x, aarch64_sve_fmsb, 4231),
SVEMAP1(svmsb_n_f64_z, aarch64_sve_fmsb, 4487),
SVEMAP1(svmsb_n_s16_m, aarch64_sve_msb, 4354),
SVEMAP1(svmsb_n_s16_x, aarch64_sve_msb, 4226),
SVEMAP1(svmsb_n_s16_z, aarch64_sve_msb, 4482),
SVEMAP1(svmsb_n_s32_m, aarch64_sve_msb, 4355),
SVEMAP1(svmsb_n_s32_x, aarch64_sve_msb, 4227),
SVEMAP1(svmsb_n_s32_z, aarch64_sve_msb, 4483),
SVEMAP1(svmsb_n_s64_m, aarch64_sve_msb, 4356),
SVEMAP1(svmsb_n_s64_x, aarch64_sve_msb, 4228),
SVEMAP1(svmsb_n_s64_z, aarch64_sve_msb, 4484),
SVEMAP1(svmsb_n_s8_m, aarch64_sve_msb, 4353),
SVEMAP1(svmsb_n_s8_x, aarch64_sve_msb, 4225),
SVEMAP1(svmsb_n_s8_z, aarch64_sve_msb, 4481),
SVEMAP1(svmsb_n_u16_m, aarch64_sve_msb, 4354),
SVEMAP1(svmsb_n_u16_x, aarch64_sve_msb, 4226),
SVEMAP1(svmsb_n_u16_z, aarch64_sve_msb, 4482),
SVEMAP1(svmsb_n_u32_m, aarch64_sve_msb, 4355),
SVEMAP1(svmsb_n_u32_x, aarch64_sve_msb, 4227),
SVEMAP1(svmsb_n_u32_z, aarch64_sve_msb, 4483),
SVEMAP1(svmsb_n_u64_m, aarch64_sve_msb, 4356),
SVEMAP1(svmsb_n_u64_x, aarch64_sve_msb, 4228),
SVEMAP1(svmsb_n_u64_z, aarch64_sve_msb, 4484),
SVEMAP1(svmsb_n_u8_m, aarch64_sve_msb, 4353),
SVEMAP1(svmsb_n_u8_x, aarch64_sve_msb, 4225),
SVEMAP1(svmsb_n_u8_z, aarch64_sve_msb, 4481),
SVEMAP1(svmsb_s16_m, aarch64_sve_msb, 258),
SVEMAP1(svmsb_s16_x, aarch64_sve_msb, 130),
SVEMAP1(svmsb_s16_z, aarch64_sve_msb, 386),
SVEMAP1(svmsb_s32_m, aarch64_sve_msb, 259),
SVEMAP1(svmsb_s32_x, aarch64_sve_msb, 131),
SVEMAP1(svmsb_s32_z, aarch64_sve_msb, 387),
SVEMAP1(svmsb_s64_m, aarch64_sve_msb, 260),
SVEMAP1(svmsb_s64_x, aarch64_sve_msb, 132),
SVEMAP1(svmsb_s64_z, aarch64_sve_msb, 388),
SVEMAP1(svmsb_s8_m, aarch64_sve_msb, 257),
SVEMAP1(svmsb_s8_x, aarch64_sve_msb, 129),
SVEMAP1(svmsb_s8_z, aarch64_sve_msb, 385),
SVEMAP1(svmsb_u16_m, aarch64_sve_msb, 258),
SVEMAP1(svmsb_u16_x, aarch64_sve_msb, 130),
SVEMAP1(svmsb_u16_z, aarch64_sve_msb, 386),
SVEMAP1(svmsb_u32_m, aarch64_sve_msb, 259),
SVEMAP1(svmsb_u32_x, aarch64_sve_msb, 131),
SVEMAP1(svmsb_u32_z, aarch64_sve_msb, 387),
SVEMAP1(svmsb_u64_m, aarch64_sve_msb, 260),
SVEMAP1(svmsb_u64_x, aarch64_sve_msb, 132),
SVEMAP1(svmsb_u64_z, aarch64_sve_msb, 388),
SVEMAP1(svmsb_u8_m, aarch64_sve_msb, 257),
SVEMAP1(svmsb_u8_x, aarch64_sve_msb, 129),
SVEMAP1(svmsb_u8_z, aarch64_sve_msb, 385),
SVEMAP1(svmul_f16_m, aarch64_sve_fmul, 261),
SVEMAP1(svmul_f16_x, aarch64_sve_fmul, 133),
SVEMAP1(svmul_f16_z, aarch64_sve_fmul, 389),
SVEMAP1(svmul_f32_m, aarch64_sve_fmul, 262),
SVEMAP1(svmul_f32_x, aarch64_sve_fmul, 134),
SVEMAP1(svmul_f32_z, aarch64_sve_fmul, 390),
SVEMAP1(svmul_f64_m, aarch64_sve_fmul, 263),
SVEMAP1(svmul_f64_x, aarch64_sve_fmul, 135),
SVEMAP1(svmul_f64_z, aarch64_sve_fmul, 391),
SVEMAP1(svmul_lane_f16, aarch64_sve_fmul_lane, 5),
SVEMAP1(svmul_lane_f32, aarch64_sve_fmul_lane, 6),
SVEMAP1(svmul_lane_f64, aarch64_sve_fmul_lane, 7),
SVEMAP1(svmul_lane_s16, aarch64_sve_mul_lane, 2),
SVEMAP1(svmul_lane_s32, aarch64_sve_mul_lane, 3),
SVEMAP1(svmul_lane_s64, aarch64_sve_mul_lane, 4),
SVEMAP1(svmul_lane_u16, aarch64_sve_mul_lane, 2),
SVEMAP1(svmul_lane_u32, aarch64_sve_mul_lane, 3),
SVEMAP1(svmul_lane_u64, aarch64_sve_mul_lane, 4),
SVEMAP1(svmul_n_f16_m, aarch64_sve_fmul, 3333),
SVEMAP1(svmul_n_f16_x, aarch64_sve_fmul, 3205),
SVEMAP1(svmul_n_f16_z, aarch64_sve_fmul, 3461),
SVEMAP1(svmul_n_f32_m, aarch64_sve_fmul, 3334),
SVEMAP1(svmul_n_f32_x, aarch64_sve_fmul, 3206),
SVEMAP1(svmul_n_f32_z, aarch64_sve_fmul, 3462),
SVEMAP1(svmul_n_f64_m, aarch64_sve_fmul, 3335),
SVEMAP1(svmul_n_f64_x, aarch64_sve_fmul, 3207),
SVEMAP1(svmul_n_f64_z, aarch64_sve_fmul, 3463),
SVEMAP1(svmul_n_s16_m, aarch64_sve_mul, 3330),
SVEMAP1(svmul_n_s16_x, aarch64_sve_mul, 3202),
SVEMAP1(svmul_n_s16_z, aarch64_sve_mul, 3458),
SVEMAP1(svmul_n_s32_m, aarch64_sve_mul, 3331),
SVEMAP1(svmul_n_s32_x, aarch64_sve_mul, 3203),
SVEMAP1(svmul_n_s32_z, aarch64_sve_mul, 3459),
SVEMAP1(svmul_n_s64_m, aarch64_sve_mul, 3332),
SVEMAP1(svmul_n_s64_x, aarch64_sve_mul, 3204),
SVEMAP1(svmul_n_s64_z, aarch64_sve_mul, 3460),
SVEMAP1(svmul_n_s8_m, aarch64_sve_mul, 3329),
SVEMAP1(svmul_n_s8_x, aarch64_sve_mul, 3201),
SVEMAP1(svmul_n_s8_z, aarch64_sve_mul, 3457),
SVEMAP1(svmul_n_u16_m, aarch64_sve_mul, 3330),
SVEMAP1(svmul_n_u16_x, aarch64_sve_mul, 3202),
SVEMAP1(svmul_n_u16_z, aarch64_sve_mul, 3458),
SVEMAP1(svmul_n_u32_m, aarch64_sve_mul, 3331),
SVEMAP1(svmul_n_u32_x, aarch64_sve_mul, 3203),
SVEMAP1(svmul_n_u32_z, aarch64_sve_mul, 3459),
SVEMAP1(svmul_n_u64_m, aarch64_sve_mul, 3332),
SVEMAP1(svmul_n_u64_x, aarch64_sve_mul, 3204),
SVEMAP1(svmul_n_u64_z, aarch64_sve_mul, 3460),
SVEMAP1(svmul_n_u8_m, aarch64_sve_mul, 3329),
SVEMAP1(svmul_n_u8_x, aarch64_sve_mul, 3201),
SVEMAP1(svmul_n_u8_z, aarch64_sve_mul, 3457),
SVEMAP1(svmul_s16_m, aarch64_sve_mul, 258),
SVEMAP1(svmul_s16_x, aarch64_sve_mul, 130),
SVEMAP1(svmul_s16_z, aarch64_sve_mul, 386),
SVEMAP1(svmul_s32_m, aarch64_sve_mul, 259),
SVEMAP1(svmul_s32_x, aarch64_sve_mul, 131),
SVEMAP1(svmul_s32_z, aarch64_sve_mul, 387),
SVEMAP1(svmul_s64_m, aarch64_sve_mul, 260),
SVEMAP1(svmul_s64_x, aarch64_sve_mul, 132),
SVEMAP1(svmul_s64_z, aarch64_sve_mul, 388),
SVEMAP1(svmul_s8_m, aarch64_sve_mul, 257),
SVEMAP1(svmul_s8_x, aarch64_sve_mul, 129),
SVEMAP1(svmul_s8_z, aarch64_sve_mul, 385),
SVEMAP1(svmul_u16_m, aarch64_sve_mul, 258),
SVEMAP1(svmul_u16_x, aarch64_sve_mul, 130),
SVEMAP1(svmul_u16_z, aarch64_sve_mul, 386),
SVEMAP1(svmul_u32_m, aarch64_sve_mul, 259),
SVEMAP1(svmul_u32_x, aarch64_sve_mul, 131),
SVEMAP1(svmul_u32_z, aarch64_sve_mul, 387),
SVEMAP1(svmul_u64_m, aarch64_sve_mul, 260),
SVEMAP1(svmul_u64_x, aarch64_sve_mul, 132),
SVEMAP1(svmul_u64_z, aarch64_sve_mul, 388),
SVEMAP1(svmul_u8_m, aarch64_sve_mul, 257),
SVEMAP1(svmul_u8_x, aarch64_sve_mul, 129),
SVEMAP1(svmul_u8_z, aarch64_sve_mul, 385),
SVEMAP1(svmulh_n_s16_m, aarch64_sve_smulh, 3330),
SVEMAP1(svmulh_n_s16_x, aarch64_sve_smulh, 3202),
SVEMAP1(svmulh_n_s16_z, aarch64_sve_smulh, 3458),
SVEMAP1(svmulh_n_s32_m, aarch64_sve_smulh, 3331),
SVEMAP1(svmulh_n_s32_x, aarch64_sve_smulh, 3203),
SVEMAP1(svmulh_n_s32_z, aarch64_sve_smulh, 3459),
SVEMAP1(svmulh_n_s64_m, aarch64_sve_smulh, 3332),
SVEMAP1(svmulh_n_s64_x, aarch64_sve_smulh, 3204),
SVEMAP1(svmulh_n_s64_z, aarch64_sve_smulh, 3460),
SVEMAP1(svmulh_n_s8_m, aarch64_sve_smulh, 3329),
SVEMAP1(svmulh_n_s8_x, aarch64_sve_smulh, 3201),
SVEMAP1(svmulh_n_s8_z, aarch64_sve_smulh, 3457),
SVEMAP1(svmulh_n_u16_m, aarch64_sve_umulh, 3330),
SVEMAP1(svmulh_n_u16_x, aarch64_sve_umulh, 3202),
SVEMAP1(svmulh_n_u16_z, aarch64_sve_umulh, 3458),
SVEMAP1(svmulh_n_u32_m, aarch64_sve_umulh, 3331),
SVEMAP1(svmulh_n_u32_x, aarch64_sve_umulh, 3203),
SVEMAP1(svmulh_n_u32_z, aarch64_sve_umulh, 3459),
SVEMAP1(svmulh_n_u64_m, aarch64_sve_umulh, 3332),
SVEMAP1(svmulh_n_u64_x, aarch64_sve_umulh, 3204),
SVEMAP1(svmulh_n_u64_z, aarch64_sve_umulh, 3460),
SVEMAP1(svmulh_n_u8_m, aarch64_sve_umulh, 3329),
SVEMAP1(svmulh_n_u8_x, aarch64_sve_umulh, 3201),
SVEMAP1(svmulh_n_u8_z, aarch64_sve_umulh, 3457),
SVEMAP1(svmulh_s16_m, aarch64_sve_smulh, 258),
SVEMAP1(svmulh_s16_x, aarch64_sve_smulh, 130),
SVEMAP1(svmulh_s16_z, aarch64_sve_smulh, 386),
SVEMAP1(svmulh_s32_m, aarch64_sve_smulh, 259),
SVEMAP1(svmulh_s32_x, aarch64_sve_smulh, 131),
SVEMAP1(svmulh_s32_z, aarch64_sve_smulh, 387),
SVEMAP1(svmulh_s64_m, aarch64_sve_smulh, 260),
SVEMAP1(svmulh_s64_x, aarch64_sve_smulh, 132),
SVEMAP1(svmulh_s64_z, aarch64_sve_smulh, 388),
SVEMAP1(svmulh_s8_m, aarch64_sve_smulh, 257),
SVEMAP1(svmulh_s8_x, aarch64_sve_smulh, 129),
SVEMAP1(svmulh_s8_z, aarch64_sve_smulh, 385),
SVEMAP1(svmulh_u16_m, aarch64_sve_umulh, 258),
SVEMAP1(svmulh_u16_x, aarch64_sve_umulh, 130),
SVEMAP1(svmulh_u16_z, aarch64_sve_umulh, 386),
SVEMAP1(svmulh_u32_m, aarch64_sve_umulh, 259),
SVEMAP1(svmulh_u32_x, aarch64_sve_umulh, 131),
SVEMAP1(svmulh_u32_z, aarch64_sve_umulh, 387),
SVEMAP1(svmulh_u64_m, aarch64_sve_umulh, 260),
SVEMAP1(svmulh_u64_x, aarch64_sve_umulh, 132),
SVEMAP1(svmulh_u64_z, aarch64_sve_umulh, 388),
SVEMAP1(svmulh_u8_m, aarch64_sve_umulh, 257),
SVEMAP1(svmulh_u8_x, aarch64_sve_umulh, 129),
SVEMAP1(svmulh_u8_z, aarch64_sve_umulh, 385),
SVEMAP1(svmullb_lane_s32, aarch64_sve_smullb_lane, 3),
SVEMAP1(svmullb_lane_s64, aarch64_sve_smullb_lane, 4),
SVEMAP1(svmullb_lane_u32, aarch64_sve_umullb_lane, 3),
SVEMAP1(svmullb_lane_u64, aarch64_sve_umullb_lane, 4),
SVEMAP1(svmullb_n_s16, aarch64_sve_smullb, 2050),
SVEMAP1(svmullb_n_s32, aarch64_sve_smullb, 2051),
SVEMAP1(svmullb_n_s64, aarch64_sve_smullb, 2052),
SVEMAP1(svmullb_n_u16, aarch64_sve_umullb, 2050),
SVEMAP1(svmullb_n_u32, aarch64_sve_umullb, 2051),
SVEMAP1(svmullb_n_u64, aarch64_sve_umullb, 2052),
SVEMAP1(svmullb_s16, aarch64_sve_smullb, 2),
SVEMAP1(svmullb_s32, aarch64_sve_smullb, 3),
SVEMAP1(svmullb_s64, aarch64_sve_smullb, 4),
SVEMAP1(svmullb_u16, aarch64_sve_umullb, 2),
SVEMAP1(svmullb_u32, aarch64_sve_umullb, 3),
SVEMAP1(svmullb_u64, aarch64_sve_umullb, 4),
SVEMAP1(svmullt_lane_s32, aarch64_sve_smullt_lane, 3),
SVEMAP1(svmullt_lane_s64, aarch64_sve_smullt_lane, 4),
SVEMAP1(svmullt_lane_u32, aarch64_sve_umullt_lane, 3),
SVEMAP1(svmullt_lane_u64, aarch64_sve_umullt_lane, 4),
SVEMAP1(svmullt_n_s16, aarch64_sve_smullt, 2050),
SVEMAP1(svmullt_n_s32, aarch64_sve_smullt, 2051),
SVEMAP1(svmullt_n_s64, aarch64_sve_smullt, 2052),
SVEMAP1(svmullt_n_u16, aarch64_sve_umullt, 2050),
SVEMAP1(svmullt_n_u32, aarch64_sve_umullt, 2051),
SVEMAP1(svmullt_n_u64, aarch64_sve_umullt, 2052),
SVEMAP1(svmullt_s16, aarch64_sve_smullt, 2),
SVEMAP1(svmullt_s32, aarch64_sve_smullt, 3),
SVEMAP1(svmullt_s64, aarch64_sve_smullt, 4),
SVEMAP1(svmullt_u16, aarch64_sve_umullt, 2),
SVEMAP1(svmullt_u32, aarch64_sve_umullt, 3),
SVEMAP1(svmullt_u64, aarch64_sve_umullt, 4),
SVEMAP1(svmulx_f16_m, aarch64_sve_fmulx, 261),
SVEMAP1(svmulx_f16_x, aarch64_sve_fmulx, 133),
SVEMAP1(svmulx_f16_z, aarch64_sve_fmulx, 389),
SVEMAP1(svmulx_f32_m, aarch64_sve_fmulx, 262),
SVEMAP1(svmulx_f32_x, aarch64_sve_fmulx, 134),
SVEMAP1(svmulx_f32_z, aarch64_sve_fmulx, 390),
SVEMAP1(svmulx_f64_m, aarch64_sve_fmulx, 263),
SVEMAP1(svmulx_f64_x, aarch64_sve_fmulx, 135),
SVEMAP1(svmulx_f64_z, aarch64_sve_fmulx, 391),
SVEMAP1(svmulx_n_f16_m, aarch64_sve_fmulx, 3333),
SVEMAP1(svmulx_n_f16_x, aarch64_sve_fmulx, 3205),
SVEMAP1(svmulx_n_f16_z, aarch64_sve_fmulx, 3461),
SVEMAP1(svmulx_n_f32_m, aarch64_sve_fmulx, 3334),
SVEMAP1(svmulx_n_f32_x, aarch64_sve_fmulx, 3206),
SVEMAP1(svmulx_n_f32_z, aarch64_sve_fmulx, 3462),
SVEMAP1(svmulx_n_f64_m, aarch64_sve_fmulx, 3335),
SVEMAP1(svmulx_n_f64_x, aarch64_sve_fmulx, 3207),
SVEMAP1(svmulx_n_f64_z, aarch64_sve_fmulx, 3463),
SVEMAP1(svnand_b_z, aarch64_sve_nand_z, 8),
SVEMAP1(svnbsl_n_s16, aarch64_sve_nbsl, 3074),
SVEMAP1(svnbsl_n_s32, aarch64_sve_nbsl, 3075),
SVEMAP1(svnbsl_n_s64, aarch64_sve_nbsl, 3076),
SVEMAP1(svnbsl_n_s8, aarch64_sve_nbsl, 3073),
SVEMAP1(svnbsl_n_u16, aarch64_sve_nbsl, 3074),
SVEMAP1(svnbsl_n_u32, aarch64_sve_nbsl, 3075),
SVEMAP1(svnbsl_n_u64, aarch64_sve_nbsl, 3076),
SVEMAP1(svnbsl_n_u8, aarch64_sve_nbsl, 3073),
SVEMAP1(svnbsl_s16, aarch64_sve_nbsl, 2),
SVEMAP1(svnbsl_s32, aarch64_sve_nbsl, 3),
SVEMAP1(svnbsl_s64, aarch64_sve_nbsl, 4),
SVEMAP1(svnbsl_s8, aarch64_sve_nbsl, 1),
SVEMAP1(svnbsl_u16, aarch64_sve_nbsl, 2),
SVEMAP1(svnbsl_u32, aarch64_sve_nbsl, 3),
SVEMAP1(svnbsl_u64, aarch64_sve_nbsl, 4),
SVEMAP1(svnbsl_u8, aarch64_sve_nbsl, 1),
SVEMAP1(svneg_f16_m, aarch64_sve_fneg, 261),
SVEMAP1(svneg_f16_x, aarch64_sve_fneg, 517),
SVEMAP1(svneg_f16_z, aarch64_sve_fneg, 645),
SVEMAP1(svneg_f32_m, aarch64_sve_fneg, 262),
SVEMAP1(svneg_f32_x, aarch64_sve_fneg, 518),
SVEMAP1(svneg_f32_z, aarch64_sve_fneg, 646),
SVEMAP1(svneg_f64_m, aarch64_sve_fneg, 263),
SVEMAP1(svneg_f64_x, aarch64_sve_fneg, 519),
SVEMAP1(svneg_f64_z, aarch64_sve_fneg, 647),
SVEMAP1(svneg_s16_m, aarch64_sve_neg, 258),
SVEMAP1(svneg_s16_x, aarch64_sve_neg, 514),
SVEMAP1(svneg_s16_z, aarch64_sve_neg, 642),
SVEMAP1(svneg_s32_m, aarch64_sve_neg, 259),
SVEMAP1(svneg_s32_x, aarch64_sve_neg, 515),
SVEMAP1(svneg_s32_z, aarch64_sve_neg, 643),
SVEMAP1(svneg_s64_m, aarch64_sve_neg, 260),
SVEMAP1(svneg_s64_x, aarch64_sve_neg, 516),
SVEMAP1(svneg_s64_z, aarch64_sve_neg, 644),
SVEMAP1(svneg_s8_m, aarch64_sve_neg, 257),
SVEMAP1(svneg_s8_x, aarch64_sve_neg, 513),
SVEMAP1(svneg_s8_z, aarch64_sve_neg, 641),
SVEMAP1(svnmad_f16_m, aarch64_sve_fnmad, 261),
SVEMAP1(svnmad_f16_x, aarch64_sve_fnmad, 133),
SVEMAP1(svnmad_f16_z, aarch64_sve_fnmad, 389),
SVEMAP1(svnmad_f32_m, aarch64_sve_fnmad, 262),
SVEMAP1(svnmad_f32_x, aarch64_sve_fnmad, 134),
SVEMAP1(svnmad_f32_z, aarch64_sve_fnmad, 390),
SVEMAP1(svnmad_f64_m, aarch64_sve_fnmad, 263),
SVEMAP1(svnmad_f64_x, aarch64_sve_fnmad, 135),
SVEMAP1(svnmad_f64_z, aarch64_sve_fnmad, 391),
SVEMAP1(svnmad_n_f16_m, aarch64_sve_fnmad, 4357),
SVEMAP1(svnmad_n_f16_x, aarch64_sve_fnmad, 4229),
SVEMAP1(svnmad_n_f16_z, aarch64_sve_fnmad, 4485),
SVEMAP1(svnmad_n_f32_m, aarch64_sve_fnmad, 4358),
SVEMAP1(svnmad_n_f32_x, aarch64_sve_fnmad, 4230),
SVEMAP1(svnmad_n_f32_z, aarch64_sve_fnmad, 4486),
SVEMAP1(svnmad_n_f64_m, aarch64_sve_fnmad, 4359),
SVEMAP1(svnmad_n_f64_x, aarch64_sve_fnmad, 4231),
SVEMAP1(svnmad_n_f64_z, aarch64_sve_fnmad, 4487),
SVEMAP1(svnmatch_s16, aarch64_sve_nmatch, 2),
SVEMAP1(svnmatch_s8, aarch64_sve_nmatch, 1),
SVEMAP1(svnmatch_u16, aarch64_sve_nmatch, 2),
SVEMAP1(svnmatch_u8, aarch64_sve_nmatch, 1),
SVEMAP1(svnmla_f16_m, aarch64_sve_fnmla, 261),
SVEMAP1(svnmla_f16_x, aarch64_sve_fnmla, 133),
SVEMAP1(svnmla_f16_z, aarch64_sve_fnmla, 389),
SVEMAP1(svnmla_f32_m, aarch64_sve_fnmla, 262),
SVEMAP1(svnmla_f32_x, aarch64_sve_fnmla, 134),
SVEMAP1(svnmla_f32_z, aarch64_sve_fnmla, 390),
SVEMAP1(svnmla_f64_m, aarch64_sve_fnmla, 263),
SVEMAP1(svnmla_f64_x, aarch64_sve_fnmla, 135),
SVEMAP1(svnmla_f64_z, aarch64_sve_fnmla, 391),
SVEMAP1(svnmla_n_f16_m, aarch64_sve_fnmla, 4357),
SVEMAP1(svnmla_n_f16_x, aarch64_sve_fnmla, 4229),
SVEMAP1(svnmla_n_f16_z, aarch64_sve_fnmla, 4485),
SVEMAP1(svnmla_n_f32_m, aarch64_sve_fnmla, 4358),
SVEMAP1(svnmla_n_f32_x, aarch64_sve_fnmla, 4230),
SVEMAP1(svnmla_n_f32_z, aarch64_sve_fnmla, 4486),
SVEMAP1(svnmla_n_f64_m, aarch64_sve_fnmla, 4359),
SVEMAP1(svnmla_n_f64_x, aarch64_sve_fnmla, 4231),
SVEMAP1(svnmla_n_f64_z, aarch64_sve_fnmla, 4487),
SVEMAP1(svnmls_f16_m, aarch64_sve_fnmls, 261),
SVEMAP1(svnmls_f16_x, aarch64_sve_fnmls, 133),
SVEMAP1(svnmls_f16_z, aarch64_sve_fnmls, 389),
SVEMAP1(svnmls_f32_m, aarch64_sve_fnmls, 262),
SVEMAP1(svnmls_f32_x, aarch64_sve_fnmls, 134),
SVEMAP1(svnmls_f32_z, aarch64_sve_fnmls, 390),
SVEMAP1(svnmls_f64_m, aarch64_sve_fnmls, 263),
SVEMAP1(svnmls_f64_x, aarch64_sve_fnmls, 135),
SVEMAP1(svnmls_f64_z, aarch64_sve_fnmls, 391),
SVEMAP1(svnmls_n_f16_m, aarch64_sve_fnmls, 4357),
SVEMAP1(svnmls_n_f16_x, aarch64_sve_fnmls, 4229),
SVEMAP1(svnmls_n_f16_z, aarch64_sve_fnmls, 4485),
SVEMAP1(svnmls_n_f32_m, aarch64_sve_fnmls, 4358),
SVEMAP1(svnmls_n_f32_x, aarch64_sve_fnmls, 4230),
SVEMAP1(svnmls_n_f32_z, aarch64_sve_fnmls, 4486),
SVEMAP1(svnmls_n_f64_m, aarch64_sve_fnmls, 4359),
SVEMAP1(svnmls_n_f64_x, aarch64_sve_fnmls, 4231),
SVEMAP1(svnmls_n_f64_z, aarch64_sve_fnmls, 4487),
SVEMAP1(svnmsb_f16_m, aarch64_sve_fnmsb, 261),
SVEMAP1(svnmsb_f16_x, aarch64_sve_fnmsb, 133),
SVEMAP1(svnmsb_f16_z, aarch64_sve_fnmsb, 389),
SVEMAP1(svnmsb_f32_m, aarch64_sve_fnmsb, 262),
SVEMAP1(svnmsb_f32_x, aarch64_sve_fnmsb, 134),
SVEMAP1(svnmsb_f32_z, aarch64_sve_fnmsb, 390),
SVEMAP1(svnmsb_f64_m, aarch64_sve_fnmsb, 263),
SVEMAP1(svnmsb_f64_x, aarch64_sve_fnmsb, 135),
SVEMAP1(svnmsb_f64_z, aarch64_sve_fnmsb, 391),
SVEMAP1(svnmsb_n_f16_m, aarch64_sve_fnmsb, 4357),
SVEMAP1(svnmsb_n_f16_x, aarch64_sve_fnmsb, 4229),
SVEMAP1(svnmsb_n_f16_z, aarch64_sve_fnmsb, 4485),
SVEMAP1(svnmsb_n_f32_m, aarch64_sve_fnmsb, 4358),
SVEMAP1(svnmsb_n_f32_x, aarch64_sve_fnmsb, 4230),
SVEMAP1(svnmsb_n_f32_z, aarch64_sve_fnmsb, 4486),
SVEMAP1(svnmsb_n_f64_m, aarch64_sve_fnmsb, 4359),
SVEMAP1(svnmsb_n_f64_x, aarch64_sve_fnmsb, 4231),
SVEMAP1(svnmsb_n_f64_z, aarch64_sve_fnmsb, 4487),
SVEMAP1(svnor_b_z, aarch64_sve_nor_z, 8),
SVEMAP2(svnot_b_z, 8),
SVEMAP1(svnot_s16_m, aarch64_sve_not, 258),
SVEMAP1(svnot_s16_x, aarch64_sve_not, 514),
SVEMAP1(svnot_s16_z, aarch64_sve_not, 642),
SVEMAP1(svnot_s32_m, aarch64_sve_not, 259),
SVEMAP1(svnot_s32_x, aarch64_sve_not, 515),
SVEMAP1(svnot_s32_z, aarch64_sve_not, 643),
SVEMAP1(svnot_s64_m, aarch64_sve_not, 260),
SVEMAP1(svnot_s64_x, aarch64_sve_not, 516),
SVEMAP1(svnot_s64_z, aarch64_sve_not, 644),
SVEMAP1(svnot_s8_m, aarch64_sve_not, 257),
SVEMAP1(svnot_s8_x, aarch64_sve_not, 513),
SVEMAP1(svnot_s8_z, aarch64_sve_not, 641),
SVEMAP1(svnot_u16_m, aarch64_sve_not, 258),
SVEMAP1(svnot_u16_x, aarch64_sve_not, 514),
SVEMAP1(svnot_u16_z, aarch64_sve_not, 642),
SVEMAP1(svnot_u32_m, aarch64_sve_not, 259),
SVEMAP1(svnot_u32_x, aarch64_sve_not, 515),
SVEMAP1(svnot_u32_z, aarch64_sve_not, 643),
SVEMAP1(svnot_u64_m, aarch64_sve_not, 260),
SVEMAP1(svnot_u64_x, aarch64_sve_not, 516),
SVEMAP1(svnot_u64_z, aarch64_sve_not, 644),
SVEMAP1(svnot_u8_m, aarch64_sve_not, 257),
SVEMAP1(svnot_u8_x, aarch64_sve_not, 513),
SVEMAP1(svnot_u8_z, aarch64_sve_not, 641),
SVEMAP1(svorn_b_z, aarch64_sve_orn_z, 8),
SVEMAP1(svorr_b_z, aarch64_sve_orr_z, 8),
SVEMAP1(svorr_n_s16_m, aarch64_sve_orr, 3330),
SVEMAP1(svorr_n_s16_x, aarch64_sve_orr, 3202),
SVEMAP1(svorr_n_s16_z, aarch64_sve_orr, 3458),
SVEMAP1(svorr_n_s32_m, aarch64_sve_orr, 3331),
SVEMAP1(svorr_n_s32_x, aarch64_sve_orr, 3203),
SVEMAP1(svorr_n_s32_z, aarch64_sve_orr, 3459),
SVEMAP1(svorr_n_s64_m, aarch64_sve_orr, 3332),
SVEMAP1(svorr_n_s64_x, aarch64_sve_orr, 3204),
SVEMAP1(svorr_n_s64_z, aarch64_sve_orr, 3460),
SVEMAP1(svorr_n_s8_m, aarch64_sve_orr, 3329),
SVEMAP1(svorr_n_s8_x, aarch64_sve_orr, 3201),
SVEMAP1(svorr_n_s8_z, aarch64_sve_orr, 3457),
SVEMAP1(svorr_n_u16_m, aarch64_sve_orr, 3330),
SVEMAP1(svorr_n_u16_x, aarch64_sve_orr, 3202),
SVEMAP1(svorr_n_u16_z, aarch64_sve_orr, 3458),
SVEMAP1(svorr_n_u32_m, aarch64_sve_orr, 3331),
SVEMAP1(svorr_n_u32_x, aarch64_sve_orr, 3203),
SVEMAP1(svorr_n_u32_z, aarch64_sve_orr, 3459),
SVEMAP1(svorr_n_u64_m, aarch64_sve_orr, 3332),
SVEMAP1(svorr_n_u64_x, aarch64_sve_orr, 3204),
SVEMAP1(svorr_n_u64_z, aarch64_sve_orr, 3460),
SVEMAP1(svorr_n_u8_m, aarch64_sve_orr, 3329),
SVEMAP1(svorr_n_u8_x, aarch64_sve_orr, 3201),
SVEMAP1(svorr_n_u8_z, aarch64_sve_orr, 3457),
SVEMAP1(svorr_s16_m, aarch64_sve_orr, 258),
SVEMAP1(svorr_s16_x, aarch64_sve_orr, 130),
SVEMAP1(svorr_s16_z, aarch64_sve_orr, 386),
SVEMAP1(svorr_s32_m, aarch64_sve_orr, 259),
SVEMAP1(svorr_s32_x, aarch64_sve_orr, 131),
SVEMAP1(svorr_s32_z, aarch64_sve_orr, 387),
SVEMAP1(svorr_s64_m, aarch64_sve_orr, 260),
SVEMAP1(svorr_s64_x, aarch64_sve_orr, 132),
SVEMAP1(svorr_s64_z, aarch64_sve_orr, 388),
SVEMAP1(svorr_s8_m, aarch64_sve_orr, 257),
SVEMAP1(svorr_s8_x, aarch64_sve_orr, 129),
SVEMAP1(svorr_s8_z, aarch64_sve_orr, 385),
SVEMAP1(svorr_u16_m, aarch64_sve_orr, 258),
SVEMAP1(svorr_u16_x, aarch64_sve_orr, 130),
SVEMAP1(svorr_u16_z, aarch64_sve_orr, 386),
SVEMAP1(svorr_u32_m, aarch64_sve_orr, 259),
SVEMAP1(svorr_u32_x, aarch64_sve_orr, 131),
SVEMAP1(svorr_u32_z, aarch64_sve_orr, 387),
SVEMAP1(svorr_u64_m, aarch64_sve_orr, 260),
SVEMAP1(svorr_u64_x, aarch64_sve_orr, 132),
SVEMAP1(svorr_u64_z, aarch64_sve_orr, 388),
SVEMAP1(svorr_u8_m, aarch64_sve_orr, 257),
SVEMAP1(svorr_u8_x, aarch64_sve_orr, 129),
SVEMAP1(svorr_u8_z, aarch64_sve_orr, 385),
SVEMAP1(svorv_s16, aarch64_sve_orv, 2),
SVEMAP1(svorv_s32, aarch64_sve_orv, 3),
SVEMAP1(svorv_s64, aarch64_sve_orv, 4),
SVEMAP1(svorv_s8, aarch64_sve_orv, 1),
SVEMAP1(svorv_u16, aarch64_sve_orv, 2),
SVEMAP1(svorv_u32, aarch64_sve_orv, 3),
SVEMAP1(svorv_u64, aarch64_sve_orv, 4),
SVEMAP1(svorv_u8, aarch64_sve_orv, 1),
SVEMAP2(svpfalse_b, 1048579),
SVEMAP1(svpfirst_b, aarch64_sve_pfirst, 8),
SVEMAP1(svpmul_n_u8, aarch64_sve_pmul, 2049),
SVEMAP1(svpmul_u8, aarch64_sve_pmul, 1),
SVEMAP2(svpmullb_n_u16, 2050),
SVEMAP2(svpmullb_n_u64, 2052),
SVEMAP1(svpmullb_pair_n_u32, aarch64_sve_pmullb_pair, 2051),
SVEMAP1(svpmullb_pair_n_u64, aarch64_sve_pmullb_pair, 2052),
SVEMAP1(svpmullb_pair_n_u8, aarch64_sve_pmullb_pair, 2049),
SVEMAP1(svpmullb_pair_u32, aarch64_sve_pmullb_pair, 3),
SVEMAP1(svpmullb_pair_u64, aarch64_sve_pmullb_pair, 4),
SVEMAP1(svpmullb_pair_u8, aarch64_sve_pmullb_pair, 1),
SVEMAP2(svpmullb_u16, 2),
SVEMAP2(svpmullb_u64, 4),
SVEMAP2(svpmullt_n_u16, 2050),
SVEMAP2(svpmullt_n_u64, 2052),
SVEMAP1(svpmullt_pair_n_u32, aarch64_sve_pmullt_pair, 2051),
SVEMAP1(svpmullt_pair_n_u64, aarch64_sve_pmullt_pair, 2052),
SVEMAP1(svpmullt_pair_n_u8, aarch64_sve_pmullt_pair, 2049),
SVEMAP1(svpmullt_pair_u32, aarch64_sve_pmullt_pair, 3),
SVEMAP1(svpmullt_pair_u64, aarch64_sve_pmullt_pair, 4),
SVEMAP1(svpmullt_pair_u8, aarch64_sve_pmullt_pair, 1),
SVEMAP2(svpmullt_u16, 2),
SVEMAP2(svpmullt_u64, 4),
SVEMAP1(svpnext_b16, aarch64_sve_pnext, 9),
SVEMAP1(svpnext_b32, aarch64_sve_pnext, 10),
SVEMAP1(svpnext_b64, aarch64_sve_pnext, 11),
SVEMAP1(svpnext_b8, aarch64_sve_pnext, 8),
SVEMAP1(svprfb, aarch64_sve_prf, 134217745),
SVEMAP1(svprfb_gather_s32offset, aarch64_sve_prfb_gather_sxtw_index, 268435475),
SVEMAP1(svprfb_gather_s64offset, aarch64_sve_prfb_gather_index, 268435476),
SVEMAP1(svprfb_gather_u32base, aarch64_sve_prfb_gather_scalar_offset, 268435475),
SVEMAP1(svprfb_gather_u32base_offset, aarch64_sve_prfb_gather_scalar_offset, 268435475),
SVEMAP1(svprfb_gather_u32offset, aarch64_sve_prfb_gather_uxtw_index, 268435475),
SVEMAP1(svprfb_gather_u64base, aarch64_sve_prfb_gather_scalar_offset, 268435476),
SVEMAP1(svprfb_gather_u64base_offset, aarch64_sve_prfb_gather_scalar_offset, 268435476),
SVEMAP1(svprfb_gather_u64offset, aarch64_sve_prfb_gather_index, 268435476),
SVEMAP1(svprfb_vnum, aarch64_sve_prf, 134217745),
SVEMAP1(svprfd, aarch64_sve_prf, 134217796),
SVEMAP1(svprfd_gather_s32index, aarch64_sve_prfd_gather_sxtw_index, 268435523),
SVEMAP1(svprfd_gather_s64index, aarch64_sve_prfd_gather_index, 268435524),
SVEMAP1(svprfd_gather_u32base, aarch64_sve_prfd_gather_scalar_offset, 268435523),
SVEMAP1(svprfd_gather_u32base_index, aarch64_sve_prfd_gather_scalar_offset, 268435523),
SVEMAP1(svprfd_gather_u32index, aarch64_sve_prfd_gather_uxtw_index, 268435523),
SVEMAP1(svprfd_gather_u64base, aarch64_sve_prfd_gather_scalar_offset, 268435524),
SVEMAP1(svprfd_gather_u64base_index, aarch64_sve_prfd_gather_scalar_offset, 268435524),
SVEMAP1(svprfd_gather_u64index, aarch64_sve_prfd_gather_index, 268435524),
SVEMAP1(svprfd_vnum, aarch64_sve_prf, 134217796),
SVEMAP1(svprfh, aarch64_sve_prf, 134217762),
SVEMAP1(svprfh_gather_s32index, aarch64_sve_prfh_gather_sxtw_index, 268435491),
SVEMAP1(svprfh_gather_s64index, aarch64_sve_prfh_gather_index, 268435492),
SVEMAP1(svprfh_gather_u32base, aarch64_sve_prfh_gather_scalar_offset, 268435491),
SVEMAP1(svprfh_gather_u32base_index, aarch64_sve_prfh_gather_scalar_offset, 268435491),
SVEMAP1(svprfh_gather_u32index, aarch64_sve_prfh_gather_uxtw_index, 268435491),
SVEMAP1(svprfh_gather_u64base, aarch64_sve_prfh_gather_scalar_offset, 268435492),
SVEMAP1(svprfh_gather_u64base_index, aarch64_sve_prfh_gather_scalar_offset, 268435492),
SVEMAP1(svprfh_gather_u64index, aarch64_sve_prfh_gather_index, 268435492),
SVEMAP1(svprfh_vnum, aarch64_sve_prf, 134217762),
SVEMAP1(svprfw, aarch64_sve_prf, 134217779),
SVEMAP1(svprfw_gather_s32index, aarch64_sve_prfw_gather_sxtw_index, 268435507),
SVEMAP1(svprfw_gather_s64index, aarch64_sve_prfw_gather_index, 268435508),
SVEMAP1(svprfw_gather_u32base, aarch64_sve_prfw_gather_scalar_offset, 268435507),
SVEMAP1(svprfw_gather_u32base_index, aarch64_sve_prfw_gather_scalar_offset, 268435507),
SVEMAP1(svprfw_gather_u32index, aarch64_sve_prfw_gather_uxtw_index, 268435507),
SVEMAP1(svprfw_gather_u64base, aarch64_sve_prfw_gather_scalar_offset, 268435508),
SVEMAP1(svprfw_gather_u64base_index, aarch64_sve_prfw_gather_scalar_offset, 268435508),
SVEMAP1(svprfw_gather_u64index, aarch64_sve_prfw_gather_index, 268435508),
SVEMAP1(svprfw_vnum, aarch64_sve_prf, 134217779),
SVEMAP1(svptest_any, aarch64_sve_ptest_any, 8),
SVEMAP1(svptest_first, aarch64_sve_ptest_first, 8),
SVEMAP1(svptest_last, aarch64_sve_ptest_last, 8),
SVEMAP1(svptrue_b16, aarch64_sve_ptrue, 33554441),
SVEMAP1(svptrue_b32, aarch64_sve_ptrue, 33554442),
SVEMAP1(svptrue_b64, aarch64_sve_ptrue, 33554443),
SVEMAP1(svptrue_b8, aarch64_sve_ptrue, 33554440),
SVEMAP1(svptrue_pat_b16, aarch64_sve_ptrue, 9),
SVEMAP1(svptrue_pat_b32, aarch64_sve_ptrue, 10),
SVEMAP1(svptrue_pat_b64, aarch64_sve_ptrue, 11),
SVEMAP1(svptrue_pat_b8, aarch64_sve_ptrue, 8),
SVEMAP1(svqabs_s16_m, aarch64_sve_sqabs, 258),
SVEMAP1(svqabs_s16_x, aarch64_sve_sqabs, 514),
SVEMAP1(svqabs_s16_z, aarch64_sve_sqabs, 642),
SVEMAP1(svqabs_s32_m, aarch64_sve_sqabs, 259),
SVEMAP1(svqabs_s32_x, aarch64_sve_sqabs, 515),
SVEMAP1(svqabs_s32_z, aarch64_sve_sqabs, 643),
SVEMAP1(svqabs_s64_m, aarch64_sve_sqabs, 260),
SVEMAP1(svqabs_s64_x, aarch64_sve_sqabs, 516),
SVEMAP1(svqabs_s64_z, aarch64_sve_sqabs, 644),
SVEMAP1(svqabs_s8_m, aarch64_sve_sqabs, 257),
SVEMAP1(svqabs_s8_x, aarch64_sve_sqabs, 513),
SVEMAP1(svqabs_s8_z, aarch64_sve_sqabs, 641),
SVEMAP1(svqadd_n_s16, aarch64_sve_sqadd_x, 2050),
SVEMAP1(svqadd_n_s16_m, aarch64_sve_sqadd, 3330),
SVEMAP1(svqadd_n_s16_x, aarch64_sve_sqadd, 3202),
SVEMAP1(svqadd_n_s16_z, aarch64_sve_sqadd, 3458),
SVEMAP1(svqadd_n_s32, aarch64_sve_sqadd_x, 2051),
SVEMAP1(svqadd_n_s32_m, aarch64_sve_sqadd, 3331),
SVEMAP1(svqadd_n_s32_x, aarch64_sve_sqadd, 3203),
SVEMAP1(svqadd_n_s32_z, aarch64_sve_sqadd, 3459),
SVEMAP1(svqadd_n_s64, aarch64_sve_sqadd_x, 2052),
SVEMAP1(svqadd_n_s64_m, aarch64_sve_sqadd, 3332),
SVEMAP1(svqadd_n_s64_x, aarch64_sve_sqadd, 3204),
SVEMAP1(svqadd_n_s64_z, aarch64_sve_sqadd, 3460),
SVEMAP1(svqadd_n_s8, aarch64_sve_sqadd_x, 2049),
SVEMAP1(svqadd_n_s8_m, aarch64_sve_sqadd, 3329),
SVEMAP1(svqadd_n_s8_x, aarch64_sve_sqadd, 3201),
SVEMAP1(svqadd_n_s8_z, aarch64_sve_sqadd, 3457),
SVEMAP1(svqadd_n_u16, aarch64_sve_uqadd_x, 2050),
SVEMAP1(svqadd_n_u16_m, aarch64_sve_uqadd, 3330),
SVEMAP1(svqadd_n_u16_x, aarch64_sve_uqadd, 3202),
SVEMAP1(svqadd_n_u16_z, aarch64_sve_uqadd, 3458),
SVEMAP1(svqadd_n_u32, aarch64_sve_uqadd_x, 2051),
SVEMAP1(svqadd_n_u32_m, aarch64_sve_uqadd, 3331),
SVEMAP1(svqadd_n_u32_x, aarch64_sve_uqadd, 3203),
SVEMAP1(svqadd_n_u32_z, aarch64_sve_uqadd, 3459),
SVEMAP1(svqadd_n_u64, aarch64_sve_uqadd_x, 2052),
SVEMAP1(svqadd_n_u64_m, aarch64_sve_uqadd, 3332),
SVEMAP1(svqadd_n_u64_x, aarch64_sve_uqadd, 3204),
SVEMAP1(svqadd_n_u64_z, aarch64_sve_uqadd, 3460),
SVEMAP1(svqadd_n_u8, aarch64_sve_uqadd_x, 2049),
SVEMAP1(svqadd_n_u8_m, aarch64_sve_uqadd, 3329),
SVEMAP1(svqadd_n_u8_x, aarch64_sve_uqadd, 3201),
SVEMAP1(svqadd_n_u8_z, aarch64_sve_uqadd, 3457),
SVEMAP1(svqadd_s16, aarch64_sve_sqadd_x, 2),
SVEMAP1(svqadd_s16_m, aarch64_sve_sqadd, 258),
SVEMAP1(svqadd_s16_x, aarch64_sve_sqadd, 130),
SVEMAP1(svqadd_s16_z, aarch64_sve_sqadd, 386),
SVEMAP1(svqadd_s32, aarch64_sve_sqadd_x, 3),
SVEMAP1(svqadd_s32_m, aarch64_sve_sqadd, 259),
SVEMAP1(svqadd_s32_x, aarch64_sve_sqadd, 131),
SVEMAP1(svqadd_s32_z, aarch64_sve_sqadd, 387),
SVEMAP1(svqadd_s64, aarch64_sve_sqadd_x, 4),
SVEMAP1(svqadd_s64_m, aarch64_sve_sqadd, 260),
SVEMAP1(svqadd_s64_x, aarch64_sve_sqadd, 132),
SVEMAP1(svqadd_s64_z, aarch64_sve_sqadd, 388),
SVEMAP1(svqadd_s8, aarch64_sve_sqadd_x, 1),
SVEMAP1(svqadd_s8_m, aarch64_sve_sqadd, 257),
SVEMAP1(svqadd_s8_x, aarch64_sve_sqadd, 129),
SVEMAP1(svqadd_s8_z, aarch64_sve_sqadd, 385),
SVEMAP1(svqadd_u16, aarch64_sve_uqadd_x, 2),
SVEMAP1(svqadd_u16_m, aarch64_sve_uqadd, 258),
SVEMAP1(svqadd_u16_x, aarch64_sve_uqadd, 130),
SVEMAP1(svqadd_u16_z, aarch64_sve_uqadd, 386),
SVEMAP1(svqadd_u32, aarch64_sve_uqadd_x, 3),
SVEMAP1(svqadd_u32_m, aarch64_sve_uqadd, 259),
SVEMAP1(svqadd_u32_x, aarch64_sve_uqadd, 131),
SVEMAP1(svqadd_u32_z, aarch64_sve_uqadd, 387),
SVEMAP1(svqadd_u64, aarch64_sve_uqadd_x, 4),
SVEMAP1(svqadd_u64_m, aarch64_sve_uqadd, 260),
SVEMAP1(svqadd_u64_x, aarch64_sve_uqadd, 132),
SVEMAP1(svqadd_u64_z, aarch64_sve_uqadd, 388),
SVEMAP1(svqadd_u8, aarch64_sve_uqadd_x, 1),
SVEMAP1(svqadd_u8_m, aarch64_sve_uqadd, 257),
SVEMAP1(svqadd_u8_x, aarch64_sve_uqadd, 129),
SVEMAP1(svqadd_u8_z, aarch64_sve_uqadd, 385),
SVEMAP1(svqcadd_s16, aarch64_sve_sqcadd_x, 2),
SVEMAP1(svqcadd_s32, aarch64_sve_sqcadd_x, 3),
SVEMAP1(svqcadd_s64, aarch64_sve_sqcadd_x, 4),
SVEMAP1(svqcadd_s8, aarch64_sve_sqcadd_x, 1),
SVEMAP1(svqdecb_n_s32, aarch64_sve_sqdecb_n32, 68157443),
SVEMAP1(svqdecb_n_s64, aarch64_sve_sqdecb_n64, 68157444),
SVEMAP1(svqdecb_n_u32, aarch64_sve_uqdecb_n32, 68157443),
SVEMAP1(svqdecb_n_u64, aarch64_sve_uqdecb_n64, 68157444),
SVEMAP1(svqdecb_pat_n_s32, aarch64_sve_sqdecb_n32, 1048579),
SVEMAP1(svqdecb_pat_n_s64, aarch64_sve_sqdecb_n64, 1048580),
SVEMAP1(svqdecb_pat_n_u32, aarch64_sve_uqdecb_n32, 1048579),
SVEMAP1(svqdecb_pat_n_u64, aarch64_sve_uqdecb_n64, 1048580),
SVEMAP1(svqdecd_n_s32, aarch64_sve_sqdecd_n32, 68157443),
SVEMAP1(svqdecd_n_s64, aarch64_sve_sqdecd_n64, 68157444),
SVEMAP1(svqdecd_n_u32, aarch64_sve_uqdecd_n32, 68157443),
SVEMAP1(svqdecd_n_u64, aarch64_sve_uqdecd_n64, 68157444),
SVEMAP1(svqdecd_pat_n_s32, aarch64_sve_sqdecd_n32, 1048579),
SVEMAP1(svqdecd_pat_n_s64, aarch64_sve_sqdecd_n64, 1048580),
SVEMAP1(svqdecd_pat_n_u32, aarch64_sve_uqdecd_n32, 1048579),
SVEMAP1(svqdecd_pat_n_u64, aarch64_sve_uqdecd_n64, 1048580),
SVEMAP1(svqdecd_pat_s64, aarch64_sve_sqdecd, 4),
SVEMAP1(svqdecd_pat_u64, aarch64_sve_uqdecd, 4),
SVEMAP1(svqdecd_s64, aarch64_sve_sqdecd, 67108868),
SVEMAP1(svqdecd_u64, aarch64_sve_uqdecd, 67108868),
SVEMAP1(svqdech_n_s32, aarch64_sve_sqdech_n32, 68157443),
SVEMAP1(svqdech_n_s64, aarch64_sve_sqdech_n64, 68157444),
SVEMAP1(svqdech_n_u32, aarch64_sve_uqdech_n32, 68157443),
SVEMAP1(svqdech_n_u64, aarch64_sve_uqdech_n64, 68157444),
SVEMAP1(svqdech_pat_n_s32, aarch64_sve_sqdech_n32, 1048579),
SVEMAP1(svqdech_pat_n_s64, aarch64_sve_sqdech_n64, 1048580),
SVEMAP1(svqdech_pat_n_u32, aarch64_sve_uqdech_n32, 1048579),
SVEMAP1(svqdech_pat_n_u64, aarch64_sve_uqdech_n64, 1048580),
SVEMAP1(svqdech_pat_s16, aarch64_sve_sqdech, 2),
SVEMAP1(svqdech_pat_u16, aarch64_sve_uqdech, 2),
SVEMAP1(svqdech_s16, aarch64_sve_sqdech, 67108866),
SVEMAP1(svqdech_u16, aarch64_sve_uqdech, 67108866),
SVEMAP1(svqdecp_n_s32_b16, aarch64_sve_sqdecp_n32, 9),
SVEMAP1(svqdecp_n_s32_b32, aarch64_sve_sqdecp_n32, 10),
SVEMAP1(svqdecp_n_s32_b64, aarch64_sve_sqdecp_n32, 11),
SVEMAP1(svqdecp_n_s32_b8, aarch64_sve_sqdecp_n32, 8),
SVEMAP1(svqdecp_n_s64_b16, aarch64_sve_sqdecp_n64, 9),
SVEMAP1(svqdecp_n_s64_b32, aarch64_sve_sqdecp_n64, 10),
SVEMAP1(svqdecp_n_s64_b64, aarch64_sve_sqdecp_n64, 11),
SVEMAP1(svqdecp_n_s64_b8, aarch64_sve_sqdecp_n64, 8),
SVEMAP1(svqdecp_n_u32_b16, aarch64_sve_uqdecp_n32, 9),
SVEMAP1(svqdecp_n_u32_b32, aarch64_sve_uqdecp_n32, 10),
SVEMAP1(svqdecp_n_u32_b64, aarch64_sve_uqdecp_n32, 11),
SVEMAP1(svqdecp_n_u32_b8, aarch64_sve_uqdecp_n32, 8),
SVEMAP1(svqdecp_n_u64_b16, aarch64_sve_uqdecp_n64, 9),
SVEMAP1(svqdecp_n_u64_b32, aarch64_sve_uqdecp_n64, 10),
SVEMAP1(svqdecp_n_u64_b64, aarch64_sve_uqdecp_n64, 11),
SVEMAP1(svqdecp_n_u64_b8, aarch64_sve_uqdecp_n64, 8),
SVEMAP1(svqdecp_s16, aarch64_sve_sqdecp, 2),
SVEMAP1(svqdecp_s32, aarch64_sve_sqdecp, 3),
SVEMAP1(svqdecp_s64, aarch64_sve_sqdecp, 4),
SVEMAP1(svqdecp_u16, aarch64_sve_uqdecp, 2),
SVEMAP1(svqdecp_u32, aarch64_sve_uqdecp, 3),
SVEMAP1(svqdecp_u64, aarch64_sve_uqdecp, 4),
SVEMAP1(svqdecw_n_s32, aarch64_sve_sqdecw_n32, 68157443),
SVEMAP1(svqdecw_n_s64, aarch64_sve_sqdecw_n64, 68157444),
SVEMAP1(svqdecw_n_u32, aarch64_sve_uqdecw_n32, 68157443),
SVEMAP1(svqdecw_n_u64, aarch64_sve_uqdecw_n64, 68157444),
SVEMAP1(svqdecw_pat_n_s32, aarch64_sve_sqdecw_n32, 1048579),
SVEMAP1(svqdecw_pat_n_s64, aarch64_sve_sqdecw_n64, 1048580),
SVEMAP1(svqdecw_pat_n_u32, aarch64_sve_uqdecw_n32, 1048579),
SVEMAP1(svqdecw_pat_n_u64, aarch64_sve_uqdecw_n64, 1048580),
SVEMAP1(svqdecw_pat_s32, aarch64_sve_sqdecw, 3),
SVEMAP1(svqdecw_pat_u32, aarch64_sve_uqdecw, 3),
SVEMAP1(svqdecw_s32, aarch64_sve_sqdecw, 67108867),
SVEMAP1(svqdecw_u32, aarch64_sve_uqdecw, 67108867),
SVEMAP1(svqdmlalb_lane_s32, aarch64_sve_sqdmlalb_lane, 3),
SVEMAP1(svqdmlalb_lane_s64, aarch64_sve_sqdmlalb_lane, 4),
SVEMAP1(svqdmlalb_n_s16, aarch64_sve_sqdmlalb, 3074),
SVEMAP1(svqdmlalb_n_s32, aarch64_sve_sqdmlalb, 3075),
SVEMAP1(svqdmlalb_n_s64, aarch64_sve_sqdmlalb, 3076),
SVEMAP1(svqdmlalb_s16, aarch64_sve_sqdmlalb, 2),
SVEMAP1(svqdmlalb_s32, aarch64_sve_sqdmlalb, 3),
SVEMAP1(svqdmlalb_s64, aarch64_sve_sqdmlalb, 4),
SVEMAP1(svqdmlalbt_n_s16, aarch64_sve_sqdmlalbt, 3074),
SVEMAP1(svqdmlalbt_n_s32, aarch64_sve_sqdmlalbt, 3075),
SVEMAP1(svqdmlalbt_n_s64, aarch64_sve_sqdmlalbt, 3076),
SVEMAP1(svqdmlalbt_s16, aarch64_sve_sqdmlalbt, 2),
SVEMAP1(svqdmlalbt_s32, aarch64_sve_sqdmlalbt, 3),
SVEMAP1(svqdmlalbt_s64, aarch64_sve_sqdmlalbt, 4),
SVEMAP1(svqdmlalt_lane_s32, aarch64_sve_sqdmlalt_lane, 3),
SVEMAP1(svqdmlalt_lane_s64, aarch64_sve_sqdmlalt_lane, 4),
SVEMAP1(svqdmlalt_n_s16, aarch64_sve_sqdmlalt, 3074),
SVEMAP1(svqdmlalt_n_s32, aarch64_sve_sqdmlalt, 3075),
SVEMAP1(svqdmlalt_n_s64, aarch64_sve_sqdmlalt, 3076),
SVEMAP1(svqdmlalt_s16, aarch64_sve_sqdmlalt, 2),
SVEMAP1(svqdmlalt_s32, aarch64_sve_sqdmlalt, 3),
SVEMAP1(svqdmlalt_s64, aarch64_sve_sqdmlalt, 4),
SVEMAP1(svqdmlslb_lane_s32, aarch64_sve_sqdmlslb_lane, 3),
SVEMAP1(svqdmlslb_lane_s64, aarch64_sve_sqdmlslb_lane, 4),
SVEMAP1(svqdmlslb_n_s16, aarch64_sve_sqdmlslb, 3074),
SVEMAP1(svqdmlslb_n_s32, aarch64_sve_sqdmlslb, 3075),
SVEMAP1(svqdmlslb_n_s64, aarch64_sve_sqdmlslb, 3076),
SVEMAP1(svqdmlslb_s16, aarch64_sve_sqdmlslb, 2),
SVEMAP1(svqdmlslb_s32, aarch64_sve_sqdmlslb, 3),
SVEMAP1(svqdmlslb_s64, aarch64_sve_sqdmlslb, 4),
SVEMAP1(svqdmlslbt_n_s16, aarch64_sve_sqdmlslbt, 3074),
SVEMAP1(svqdmlslbt_n_s32, aarch64_sve_sqdmlslbt, 3075),
SVEMAP1(svqdmlslbt_n_s64, aarch64_sve_sqdmlslbt, 3076),
SVEMAP1(svqdmlslbt_s16, aarch64_sve_sqdmlslbt, 2),
SVEMAP1(svqdmlslbt_s32, aarch64_sve_sqdmlslbt, 3),
SVEMAP1(svqdmlslbt_s64, aarch64_sve_sqdmlslbt, 4),
SVEMAP1(svqdmlslt_lane_s32, aarch64_sve_sqdmlslt_lane, 3),
SVEMAP1(svqdmlslt_lane_s64, aarch64_sve_sqdmlslt_lane, 4),
SVEMAP1(svqdmlslt_n_s16, aarch64_sve_sqdmlslt, 3074),
SVEMAP1(svqdmlslt_n_s32, aarch64_sve_sqdmlslt, 3075),
SVEMAP1(svqdmlslt_n_s64, aarch64_sve_sqdmlslt, 3076),
SVEMAP1(svqdmlslt_s16, aarch64_sve_sqdmlslt, 2),
SVEMAP1(svqdmlslt_s32, aarch64_sve_sqdmlslt, 3),
SVEMAP1(svqdmlslt_s64, aarch64_sve_sqdmlslt, 4),
SVEMAP1(svqdmulh_lane_s16, aarch64_sve_sqdmulh_lane, 2),
SVEMAP1(svqdmulh_lane_s32, aarch64_sve_sqdmulh_lane, 3),
SVEMAP1(svqdmulh_lane_s64, aarch64_sve_sqdmulh_lane, 4),
SVEMAP1(svqdmulh_n_s16, aarch64_sve_sqdmulh, 2050),
SVEMAP1(svqdmulh_n_s32, aarch64_sve_sqdmulh, 2051),
SVEMAP1(svqdmulh_n_s64, aarch64_sve_sqdmulh, 2052),
SVEMAP1(svqdmulh_n_s8, aarch64_sve_sqdmulh, 2049),
SVEMAP1(svqdmulh_s16, aarch64_sve_sqdmulh, 2),
SVEMAP1(svqdmulh_s32, aarch64_sve_sqdmulh, 3),
SVEMAP1(svqdmulh_s64, aarch64_sve_sqdmulh, 4),
SVEMAP1(svqdmulh_s8, aarch64_sve_sqdmulh, 1),
SVEMAP1(svqdmullb_lane_s32, aarch64_sve_sqdmullb_lane, 3),
SVEMAP1(svqdmullb_lane_s64, aarch64_sve_sqdmullb_lane, 4),
SVEMAP1(svqdmullb_n_s16, aarch64_sve_sqdmullb, 2050),
SVEMAP1(svqdmullb_n_s32, aarch64_sve_sqdmullb, 2051),
SVEMAP1(svqdmullb_n_s64, aarch64_sve_sqdmullb, 2052),
SVEMAP1(svqdmullb_s16, aarch64_sve_sqdmullb, 2),
SVEMAP1(svqdmullb_s32, aarch64_sve_sqdmullb, 3),
SVEMAP1(svqdmullb_s64, aarch64_sve_sqdmullb, 4),
SVEMAP1(svqdmullt_lane_s32, aarch64_sve_sqdmullt_lane, 3),
SVEMAP1(svqdmullt_lane_s64, aarch64_sve_sqdmullt_lane, 4),
SVEMAP1(svqdmullt_n_s16, aarch64_sve_sqdmullt, 2050),
SVEMAP1(svqdmullt_n_s32, aarch64_sve_sqdmullt, 2051),
SVEMAP1(svqdmullt_n_s64, aarch64_sve_sqdmullt, 2052),
SVEMAP1(svqdmullt_s16, aarch64_sve_sqdmullt, 2),
SVEMAP1(svqdmullt_s32, aarch64_sve_sqdmullt, 3),
SVEMAP1(svqdmullt_s64, aarch64_sve_sqdmullt, 4),
SVEMAP1(svqincb_n_s32, aarch64_sve_sqincb_n32, 68157443),
SVEMAP1(svqincb_n_s64, aarch64_sve_sqincb_n64, 68157444),
SVEMAP1(svqincb_n_u32, aarch64_sve_uqincb_n32, 68157443),
SVEMAP1(svqincb_n_u64, aarch64_sve_uqincb_n64, 68157444),
SVEMAP1(svqincb_pat_n_s32, aarch64_sve_sqincb_n32, 1048579),
SVEMAP1(svqincb_pat_n_s64, aarch64_sve_sqincb_n64, 1048580),
SVEMAP1(svqincb_pat_n_u32, aarch64_sve_uqincb_n32, 1048579),
SVEMAP1(svqincb_pat_n_u64, aarch64_sve_uqincb_n64, 1048580),
SVEMAP1(svqincd_n_s32, aarch64_sve_sqincd_n32, 68157443),
SVEMAP1(svqincd_n_s64, aarch64_sve_sqincd_n64, 68157444),
SVEMAP1(svqincd_n_u32, aarch64_sve_uqincd_n32, 68157443),
SVEMAP1(svqincd_n_u64, aarch64_sve_uqincd_n64, 68157444),
SVEMAP1(svqincd_pat_n_s32, aarch64_sve_sqincd_n32, 1048579),
SVEMAP1(svqincd_pat_n_s64, aarch64_sve_sqincd_n64, 1048580),
SVEMAP1(svqincd_pat_n_u32, aarch64_sve_uqincd_n32, 1048579),
SVEMAP1(svqincd_pat_n_u64, aarch64_sve_uqincd_n64, 1048580),
SVEMAP1(svqincd_pat_s64, aarch64_sve_sqincd, 4),
SVEMAP1(svqincd_pat_u64, aarch64_sve_uqincd, 4),
SVEMAP1(svqincd_s64, aarch64_sve_sqincd, 67108868),
SVEMAP1(svqincd_u64, aarch64_sve_uqincd, 67108868),
SVEMAP1(svqinch_n_s32, aarch64_sve_sqinch_n32, 68157443),
SVEMAP1(svqinch_n_s64, aarch64_sve_sqinch_n64, 68157444),
SVEMAP1(svqinch_n_u32, aarch64_sve_uqinch_n32, 68157443),
SVEMAP1(svqinch_n_u64, aarch64_sve_uqinch_n64, 68157444),
SVEMAP1(svqinch_pat_n_s32, aarch64_sve_sqinch_n32, 1048579),
SVEMAP1(svqinch_pat_n_s64, aarch64_sve_sqinch_n64, 1048580),
SVEMAP1(svqinch_pat_n_u32, aarch64_sve_uqinch_n32, 1048579),
SVEMAP1(svqinch_pat_n_u64, aarch64_sve_uqinch_n64, 1048580),
SVEMAP1(svqinch_pat_s16, aarch64_sve_sqinch, 2),
SVEMAP1(svqinch_pat_u16, aarch64_sve_uqinch, 2),
SVEMAP1(svqinch_s16, aarch64_sve_sqinch, 67108866),
SVEMAP1(svqinch_u16, aarch64_sve_uqinch, 67108866),
SVEMAP1(svqincp_n_s32_b16, aarch64_sve_sqincp_n32, 9),
SVEMAP1(svqincp_n_s32_b32, aarch64_sve_sqincp_n32, 10),
SVEMAP1(svqincp_n_s32_b64, aarch64_sve_sqincp_n32, 11),
SVEMAP1(svqincp_n_s32_b8, aarch64_sve_sqincp_n32, 8),
SVEMAP1(svqincp_n_s64_b16, aarch64_sve_sqincp_n64, 9),
SVEMAP1(svqincp_n_s64_b32, aarch64_sve_sqincp_n64, 10),
SVEMAP1(svqincp_n_s64_b64, aarch64_sve_sqincp_n64, 11),
SVEMAP1(svqincp_n_s64_b8, aarch64_sve_sqincp_n64, 8),
SVEMAP1(svqincp_n_u32_b16, aarch64_sve_uqincp_n32, 9),
SVEMAP1(svqincp_n_u32_b32, aarch64_sve_uqincp_n32, 10),
SVEMAP1(svqincp_n_u32_b64, aarch64_sve_uqincp_n32, 11),
SVEMAP1(svqincp_n_u32_b8, aarch64_sve_uqincp_n32, 8),
SVEMAP1(svqincp_n_u64_b16, aarch64_sve_uqincp_n64, 9),
SVEMAP1(svqincp_n_u64_b32, aarch64_sve_uqincp_n64, 10),
SVEMAP1(svqincp_n_u64_b64, aarch64_sve_uqincp_n64, 11),
SVEMAP1(svqincp_n_u64_b8, aarch64_sve_uqincp_n64, 8),
SVEMAP1(svqincp_s16, aarch64_sve_sqincp, 2),
SVEMAP1(svqincp_s32, aarch64_sve_sqincp, 3),
SVEMAP1(svqincp_s64, aarch64_sve_sqincp, 4),
SVEMAP1(svqincp_u16, aarch64_sve_uqincp, 2),
SVEMAP1(svqincp_u32, aarch64_sve_uqincp, 3),
SVEMAP1(svqincp_u64, aarch64_sve_uqincp, 4),
SVEMAP1(svqincw_n_s32, aarch64_sve_sqincw_n32, 68157443),
SVEMAP1(svqincw_n_s64, aarch64_sve_sqincw_n64, 68157444),
SVEMAP1(svqincw_n_u32, aarch64_sve_uqincw_n32, 68157443),
SVEMAP1(svqincw_n_u64, aarch64_sve_uqincw_n64, 68157444),
SVEMAP1(svqincw_pat_n_s32, aarch64_sve_sqincw_n32, 1048579),
SVEMAP1(svqincw_pat_n_s64, aarch64_sve_sqincw_n64, 1048580),
SVEMAP1(svqincw_pat_n_u32, aarch64_sve_uqincw_n32, 1048579),
SVEMAP1(svqincw_pat_n_u64, aarch64_sve_uqincw_n64, 1048580),
SVEMAP1(svqincw_pat_s32, aarch64_sve_sqincw, 3),
SVEMAP1(svqincw_pat_u32, aarch64_sve_uqincw, 3),
SVEMAP1(svqincw_s32, aarch64_sve_sqincw, 67108867),
SVEMAP1(svqincw_u32, aarch64_sve_uqincw, 67108867),
SVEMAP1(svqneg_s16_m, aarch64_sve_sqneg, 258),
SVEMAP1(svqneg_s16_x, aarch64_sve_sqneg, 514),
SVEMAP1(svqneg_s16_z, aarch64_sve_sqneg, 642),
SVEMAP1(svqneg_s32_m, aarch64_sve_sqneg, 259),
SVEMAP1(svqneg_s32_x, aarch64_sve_sqneg, 515),
SVEMAP1(svqneg_s32_z, aarch64_sve_sqneg, 643),
SVEMAP1(svqneg_s64_m, aarch64_sve_sqneg, 260),
SVEMAP1(svqneg_s64_x, aarch64_sve_sqneg, 516),
SVEMAP1(svqneg_s64_z, aarch64_sve_sqneg, 644),
SVEMAP1(svqneg_s8_m, aarch64_sve_sqneg, 257),
SVEMAP1(svqneg_s8_x, aarch64_sve_sqneg, 513),
SVEMAP1(svqneg_s8_z, aarch64_sve_sqneg, 641),
SVEMAP1(svqrdcmlah_lane_s16, aarch64_sve_sqrdcmlah_lane_x, 2),
SVEMAP1(svqrdcmlah_lane_s32, aarch64_sve_sqrdcmlah_lane_x, 3),
SVEMAP1(svqrdcmlah_s16, aarch64_sve_sqrdcmlah_x, 2),
SVEMAP1(svqrdcmlah_s32, aarch64_sve_sqrdcmlah_x, 3),
SVEMAP1(svqrdcmlah_s64, aarch64_sve_sqrdcmlah_x, 4),
SVEMAP1(svqrdcmlah_s8, aarch64_sve_sqrdcmlah_x, 1),
SVEMAP1(svqrdmlah_lane_s16, aarch64_sve_sqrdmlah_lane, 2),
SVEMAP1(svqrdmlah_lane_s32, aarch64_sve_sqrdmlah_lane, 3),
SVEMAP1(svqrdmlah_lane_s64, aarch64_sve_sqrdmlah_lane, 4),
SVEMAP1(svqrdmlah_n_s16, aarch64_sve_sqrdmlah, 3074),
SVEMAP1(svqrdmlah_n_s32, aarch64_sve_sqrdmlah, 3075),
SVEMAP1(svqrdmlah_n_s64, aarch64_sve_sqrdmlah, 3076),
SVEMAP1(svqrdmlah_n_s8, aarch64_sve_sqrdmlah, 3073),
SVEMAP1(svqrdmlah_s16, aarch64_sve_sqrdmlah, 2),
SVEMAP1(svqrdmlah_s32, aarch64_sve_sqrdmlah, 3),
SVEMAP1(svqrdmlah_s64, aarch64_sve_sqrdmlah, 4),
SVEMAP1(svqrdmlah_s8, aarch64_sve_sqrdmlah, 1),
SVEMAP1(svqrdmlsh_lane_s16, aarch64_sve_sqrdmlsh_lane, 2),
SVEMAP1(svqrdmlsh_lane_s32, aarch64_sve_sqrdmlsh_lane, 3),
SVEMAP1(svqrdmlsh_lane_s64, aarch64_sve_sqrdmlsh_lane, 4),
SVEMAP1(svqrdmlsh_n_s16, aarch64_sve_sqrdmlsh, 3074),
SVEMAP1(svqrdmlsh_n_s32, aarch64_sve_sqrdmlsh, 3075),
SVEMAP1(svqrdmlsh_n_s64, aarch64_sve_sqrdmlsh, 3076),
SVEMAP1(svqrdmlsh_n_s8, aarch64_sve_sqrdmlsh, 3073),
SVEMAP1(svqrdmlsh_s16, aarch64_sve_sqrdmlsh, 2),
SVEMAP1(svqrdmlsh_s32, aarch64_sve_sqrdmlsh, 3),
SVEMAP1(svqrdmlsh_s64, aarch64_sve_sqrdmlsh, 4),
SVEMAP1(svqrdmlsh_s8, aarch64_sve_sqrdmlsh, 1),
SVEMAP1(svqrdmulh_lane_s16, aarch64_sve_sqrdmulh_lane, 2),
SVEMAP1(svqrdmulh_lane_s32, aarch64_sve_sqrdmulh_lane, 3),
SVEMAP1(svqrdmulh_lane_s64, aarch64_sve_sqrdmulh_lane, 4),
SVEMAP1(svqrdmulh_n_s16, aarch64_sve_sqrdmulh, 2050),
SVEMAP1(svqrdmulh_n_s32, aarch64_sve_sqrdmulh, 2051),
SVEMAP1(svqrdmulh_n_s64, aarch64_sve_sqrdmulh, 2052),
SVEMAP1(svqrdmulh_n_s8, aarch64_sve_sqrdmulh, 2049),
SVEMAP1(svqrdmulh_s16, aarch64_sve_sqrdmulh, 2),
SVEMAP1(svqrdmulh_s32, aarch64_sve_sqrdmulh, 3),
SVEMAP1(svqrdmulh_s64, aarch64_sve_sqrdmulh, 4),
SVEMAP1(svqrdmulh_s8, aarch64_sve_sqrdmulh, 1),
SVEMAP1(svqrshl_n_s16_m, aarch64_sve_sqrshl, 3330),
SVEMAP1(svqrshl_n_s16_x, aarch64_sve_sqrshl, 3202),
SVEMAP1(svqrshl_n_s16_z, aarch64_sve_sqrshl, 3458),
SVEMAP1(svqrshl_n_s32_m, aarch64_sve_sqrshl, 3331),
SVEMAP1(svqrshl_n_s32_x, aarch64_sve_sqrshl, 3203),
SVEMAP1(svqrshl_n_s32_z, aarch64_sve_sqrshl, 3459),
SVEMAP1(svqrshl_n_s64_m, aarch64_sve_sqrshl, 3332),
SVEMAP1(svqrshl_n_s64_x, aarch64_sve_sqrshl, 3204),
SVEMAP1(svqrshl_n_s64_z, aarch64_sve_sqrshl, 3460),
SVEMAP1(svqrshl_n_s8_m, aarch64_sve_sqrshl, 3329),
SVEMAP1(svqrshl_n_s8_x, aarch64_sve_sqrshl, 3201),
SVEMAP1(svqrshl_n_s8_z, aarch64_sve_sqrshl, 3457),
SVEMAP1(svqrshl_n_u16_m, aarch64_sve_uqrshl, 3330),
SVEMAP1(svqrshl_n_u16_x, aarch64_sve_uqrshl, 3202),
SVEMAP1(svqrshl_n_u16_z, aarch64_sve_uqrshl, 3458),
SVEMAP1(svqrshl_n_u32_m, aarch64_sve_uqrshl, 3331),
SVEMAP1(svqrshl_n_u32_x, aarch64_sve_uqrshl, 3203),
SVEMAP1(svqrshl_n_u32_z, aarch64_sve_uqrshl, 3459),
SVEMAP1(svqrshl_n_u64_m, aarch64_sve_uqrshl, 3332),
SVEMAP1(svqrshl_n_u64_x, aarch64_sve_uqrshl, 3204),
SVEMAP1(svqrshl_n_u64_z, aarch64_sve_uqrshl, 3460),
SVEMAP1(svqrshl_n_u8_m, aarch64_sve_uqrshl, 3329),
SVEMAP1(svqrshl_n_u8_x, aarch64_sve_uqrshl, 3201),
SVEMAP1(svqrshl_n_u8_z, aarch64_sve_uqrshl, 3457),
SVEMAP1(svqrshl_s16_m, aarch64_sve_sqrshl, 258),
SVEMAP1(svqrshl_s16_x, aarch64_sve_sqrshl, 130),
SVEMAP1(svqrshl_s16_z, aarch64_sve_sqrshl, 386),
SVEMAP1(svqrshl_s32_m, aarch64_sve_sqrshl, 259),
SVEMAP1(svqrshl_s32_x, aarch64_sve_sqrshl, 131),
SVEMAP1(svqrshl_s32_z, aarch64_sve_sqrshl, 387),
SVEMAP1(svqrshl_s64_m, aarch64_sve_sqrshl, 260),
SVEMAP1(svqrshl_s64_x, aarch64_sve_sqrshl, 132),
SVEMAP1(svqrshl_s64_z, aarch64_sve_sqrshl, 388),
SVEMAP1(svqrshl_s8_m, aarch64_sve_sqrshl, 257),
SVEMAP1(svqrshl_s8_x, aarch64_sve_sqrshl, 129),
SVEMAP1(svqrshl_s8_z, aarch64_sve_sqrshl, 385),
SVEMAP1(svqrshl_u16_m, aarch64_sve_uqrshl, 258),
SVEMAP1(svqrshl_u16_x, aarch64_sve_uqrshl, 130),
SVEMAP1(svqrshl_u16_z, aarch64_sve_uqrshl, 386),
SVEMAP1(svqrshl_u32_m, aarch64_sve_uqrshl, 259),
SVEMAP1(svqrshl_u32_x, aarch64_sve_uqrshl, 131),
SVEMAP1(svqrshl_u32_z, aarch64_sve_uqrshl, 387),
SVEMAP1(svqrshl_u64_m, aarch64_sve_uqrshl, 260),
SVEMAP1(svqrshl_u64_x, aarch64_sve_uqrshl, 132),
SVEMAP1(svqrshl_u64_z, aarch64_sve_uqrshl, 388),
SVEMAP1(svqrshl_u8_m, aarch64_sve_uqrshl, 257),
SVEMAP1(svqrshl_u8_x, aarch64_sve_uqrshl, 129),
SVEMAP1(svqrshl_u8_z, aarch64_sve_uqrshl, 385),
SVEMAP1(svqrshrnb_n_s16, aarch64_sve_sqrshrnb, 2),
SVEMAP1(svqrshrnb_n_s32, aarch64_sve_sqrshrnb, 3),
SVEMAP1(svqrshrnb_n_s64, aarch64_sve_sqrshrnb, 4),
SVEMAP1(svqrshrnb_n_u16, aarch64_sve_uqrshrnb, 2),
SVEMAP1(svqrshrnb_n_u32, aarch64_sve_uqrshrnb, 3),
SVEMAP1(svqrshrnb_n_u64, aarch64_sve_uqrshrnb, 4),
SVEMAP1(svqrshrnt_n_s16, aarch64_sve_sqrshrnt, 2),
SVEMAP1(svqrshrnt_n_s32, aarch64_sve_sqrshrnt, 3),
SVEMAP1(svqrshrnt_n_s64, aarch64_sve_sqrshrnt, 4),
SVEMAP1(svqrshrnt_n_u16, aarch64_sve_uqrshrnt, 2),
SVEMAP1(svqrshrnt_n_u32, aarch64_sve_uqrshrnt, 3),
SVEMAP1(svqrshrnt_n_u64, aarch64_sve_uqrshrnt, 4),
SVEMAP1(svqrshrunb_n_s16, aarch64_sve_sqrshrunb, 2),
SVEMAP1(svqrshrunb_n_s32, aarch64_sve_sqrshrunb, 3),
SVEMAP1(svqrshrunb_n_s64, aarch64_sve_sqrshrunb, 4),
SVEMAP1(svqrshrunt_n_s16, aarch64_sve_sqrshrunt, 2),
SVEMAP1(svqrshrunt_n_s32, aarch64_sve_sqrshrunt, 3),
SVEMAP1(svqrshrunt_n_s64, aarch64_sve_sqrshrunt, 4),
SVEMAP1(svqshl_n_s16_m, aarch64_sve_sqshl, 3330),
SVEMAP1(svqshl_n_s16_x, aarch64_sve_sqshl, 3202),
SVEMAP1(svqshl_n_s16_z, aarch64_sve_sqshl, 3458),
SVEMAP1(svqshl_n_s32_m, aarch64_sve_sqshl, 3331),
SVEMAP1(svqshl_n_s32_x, aarch64_sve_sqshl, 3203),
SVEMAP1(svqshl_n_s32_z, aarch64_sve_sqshl, 3459),
SVEMAP1(svqshl_n_s64_m, aarch64_sve_sqshl, 3332),
SVEMAP1(svqshl_n_s64_x, aarch64_sve_sqshl, 3204),
SVEMAP1(svqshl_n_s64_z, aarch64_sve_sqshl, 3460),
SVEMAP1(svqshl_n_s8_m, aarch64_sve_sqshl, 3329),
SVEMAP1(svqshl_n_s8_x, aarch64_sve_sqshl, 3201),
SVEMAP1(svqshl_n_s8_z, aarch64_sve_sqshl, 3457),
SVEMAP1(svqshl_n_u16_m, aarch64_sve_uqshl, 3330),
SVEMAP1(svqshl_n_u16_x, aarch64_sve_uqshl, 3202),
SVEMAP1(svqshl_n_u16_z, aarch64_sve_uqshl, 3458),
SVEMAP1(svqshl_n_u32_m, aarch64_sve_uqshl, 3331),
SVEMAP1(svqshl_n_u32_x, aarch64_sve_uqshl, 3203),
SVEMAP1(svqshl_n_u32_z, aarch64_sve_uqshl, 3459),
SVEMAP1(svqshl_n_u64_m, aarch64_sve_uqshl, 3332),
SVEMAP1(svqshl_n_u64_x, aarch64_sve_uqshl, 3204),
SVEMAP1(svqshl_n_u64_z, aarch64_sve_uqshl, 3460),
SVEMAP1(svqshl_n_u8_m, aarch64_sve_uqshl, 3329),
SVEMAP1(svqshl_n_u8_x, aarch64_sve_uqshl, 3201),
SVEMAP1(svqshl_n_u8_z, aarch64_sve_uqshl, 3457),
SVEMAP1(svqshl_s16_m, aarch64_sve_sqshl, 258),
SVEMAP1(svqshl_s16_x, aarch64_sve_sqshl, 130),
SVEMAP1(svqshl_s16_z, aarch64_sve_sqshl, 386),
SVEMAP1(svqshl_s32_m, aarch64_sve_sqshl, 259),
SVEMAP1(svqshl_s32_x, aarch64_sve_sqshl, 131),
SVEMAP1(svqshl_s32_z, aarch64_sve_sqshl, 387),
SVEMAP1(svqshl_s64_m, aarch64_sve_sqshl, 260),
SVEMAP1(svqshl_s64_x, aarch64_sve_sqshl, 132),
SVEMAP1(svqshl_s64_z, aarch64_sve_sqshl, 388),
SVEMAP1(svqshl_s8_m, aarch64_sve_sqshl, 257),
SVEMAP1(svqshl_s8_x, aarch64_sve_sqshl, 129),
SVEMAP1(svqshl_s8_z, aarch64_sve_sqshl, 385),
SVEMAP1(svqshl_u16_m, aarch64_sve_uqshl, 258),
SVEMAP1(svqshl_u16_x, aarch64_sve_uqshl, 130),
SVEMAP1(svqshl_u16_z, aarch64_sve_uqshl, 386),
SVEMAP1(svqshl_u32_m, aarch64_sve_uqshl, 259),
SVEMAP1(svqshl_u32_x, aarch64_sve_uqshl, 131),
SVEMAP1(svqshl_u32_z, aarch64_sve_uqshl, 387),
SVEMAP1(svqshl_u64_m, aarch64_sve_uqshl, 260),
SVEMAP1(svqshl_u64_x, aarch64_sve_uqshl, 132),
SVEMAP1(svqshl_u64_z, aarch64_sve_uqshl, 388),
SVEMAP1(svqshl_u8_m, aarch64_sve_uqshl, 257),
SVEMAP1(svqshl_u8_x, aarch64_sve_uqshl, 129),
SVEMAP1(svqshl_u8_z, aarch64_sve_uqshl, 385),
SVEMAP1(svqshlu_n_s16_m, aarch64_sve_sqshlu, 258),
SVEMAP1(svqshlu_n_s16_x, aarch64_sve_sqshlu, 130),
SVEMAP1(svqshlu_n_s16_z, aarch64_sve_sqshlu, 386),
SVEMAP1(svqshlu_n_s32_m, aarch64_sve_sqshlu, 259),
SVEMAP1(svqshlu_n_s32_x, aarch64_sve_sqshlu, 131),
SVEMAP1(svqshlu_n_s32_z, aarch64_sve_sqshlu, 387),
SVEMAP1(svqshlu_n_s64_m, aarch64_sve_sqshlu, 260),
SVEMAP1(svqshlu_n_s64_x, aarch64_sve_sqshlu, 132),
SVEMAP1(svqshlu_n_s64_z, aarch64_sve_sqshlu, 388),
SVEMAP1(svqshlu_n_s8_m, aarch64_sve_sqshlu, 257),
SVEMAP1(svqshlu_n_s8_x, aarch64_sve_sqshlu, 129),
SVEMAP1(svqshlu_n_s8_z, aarch64_sve_sqshlu, 385),
SVEMAP1(svqshrnb_n_s16, aarch64_sve_sqshrnb, 2),
SVEMAP1(svqshrnb_n_s32, aarch64_sve_sqshrnb, 3),
SVEMAP1(svqshrnb_n_s64, aarch64_sve_sqshrnb, 4),
SVEMAP1(svqshrnb_n_u16, aarch64_sve_uqshrnb, 2),
SVEMAP1(svqshrnb_n_u32, aarch64_sve_uqshrnb, 3),
SVEMAP1(svqshrnb_n_u64, aarch64_sve_uqshrnb, 4),
SVEMAP1(svqshrnt_n_s16, aarch64_sve_sqshrnt, 2),
SVEMAP1(svqshrnt_n_s32, aarch64_sve_sqshrnt, 3),
SVEMAP1(svqshrnt_n_s64, aarch64_sve_sqshrnt, 4),
SVEMAP1(svqshrnt_n_u16, aarch64_sve_uqshrnt, 2),
SVEMAP1(svqshrnt_n_u32, aarch64_sve_uqshrnt, 3),
SVEMAP1(svqshrnt_n_u64, aarch64_sve_uqshrnt, 4),
SVEMAP1(svqshrunb_n_s16, aarch64_sve_sqshrunb, 2),
SVEMAP1(svqshrunb_n_s32, aarch64_sve_sqshrunb, 3),
SVEMAP1(svqshrunb_n_s64, aarch64_sve_sqshrunb, 4),
SVEMAP1(svqshrunt_n_s16, aarch64_sve_sqshrunt, 2),
SVEMAP1(svqshrunt_n_s32, aarch64_sve_sqshrunt, 3),
SVEMAP1(svqshrunt_n_s64, aarch64_sve_sqshrunt, 4),
SVEMAP1(svqsub_n_s16, aarch64_sve_sqsub_x, 2050),
SVEMAP1(svqsub_n_s16_m, aarch64_sve_sqsub, 3330),
SVEMAP1(svqsub_n_s16_x, aarch64_sve_sqsub, 3202),
SVEMAP1(svqsub_n_s16_z, aarch64_sve_sqsub, 3458),
SVEMAP1(svqsub_n_s32, aarch64_sve_sqsub_x, 2051),
SVEMAP1(svqsub_n_s32_m, aarch64_sve_sqsub, 3331),
SVEMAP1(svqsub_n_s32_x, aarch64_sve_sqsub, 3203),
SVEMAP1(svqsub_n_s32_z, aarch64_sve_sqsub, 3459),
SVEMAP1(svqsub_n_s64, aarch64_sve_sqsub_x, 2052),
SVEMAP1(svqsub_n_s64_m, aarch64_sve_sqsub, 3332),
SVEMAP1(svqsub_n_s64_x, aarch64_sve_sqsub, 3204),
SVEMAP1(svqsub_n_s64_z, aarch64_sve_sqsub, 3460),
SVEMAP1(svqsub_n_s8, aarch64_sve_sqsub_x, 2049),
SVEMAP1(svqsub_n_s8_m, aarch64_sve_sqsub, 3329),
SVEMAP1(svqsub_n_s8_x, aarch64_sve_sqsub, 3201),
SVEMAP1(svqsub_n_s8_z, aarch64_sve_sqsub, 3457),
SVEMAP1(svqsub_n_u16, aarch64_sve_uqsub_x, 2050),
SVEMAP1(svqsub_n_u16_m, aarch64_sve_uqsub, 3330),
SVEMAP1(svqsub_n_u16_x, aarch64_sve_uqsub, 3202),
SVEMAP1(svqsub_n_u16_z, aarch64_sve_uqsub, 3458),
SVEMAP1(svqsub_n_u32, aarch64_sve_uqsub_x, 2051),
SVEMAP1(svqsub_n_u32_m, aarch64_sve_uqsub, 3331),
SVEMAP1(svqsub_n_u32_x, aarch64_sve_uqsub, 3203),
SVEMAP1(svqsub_n_u32_z, aarch64_sve_uqsub, 3459),
SVEMAP1(svqsub_n_u64, aarch64_sve_uqsub_x, 2052),
SVEMAP1(svqsub_n_u64_m, aarch64_sve_uqsub, 3332),
SVEMAP1(svqsub_n_u64_x, aarch64_sve_uqsub, 3204),
SVEMAP1(svqsub_n_u64_z, aarch64_sve_uqsub, 3460),
SVEMAP1(svqsub_n_u8, aarch64_sve_uqsub_x, 2049),
SVEMAP1(svqsub_n_u8_m, aarch64_sve_uqsub, 3329),
SVEMAP1(svqsub_n_u8_x, aarch64_sve_uqsub, 3201),
SVEMAP1(svqsub_n_u8_z, aarch64_sve_uqsub, 3457),
SVEMAP1(svqsub_s16, aarch64_sve_sqsub_x, 2),
SVEMAP1(svqsub_s16_m, aarch64_sve_sqsub, 258),
SVEMAP1(svqsub_s16_x, aarch64_sve_sqsub, 130),
SVEMAP1(svqsub_s16_z, aarch64_sve_sqsub, 386),
SVEMAP1(svqsub_s32, aarch64_sve_sqsub_x, 3),
SVEMAP1(svqsub_s32_m, aarch64_sve_sqsub, 259),
SVEMAP1(svqsub_s32_x, aarch64_sve_sqsub, 131),
SVEMAP1(svqsub_s32_z, aarch64_sve_sqsub, 387),
SVEMAP1(svqsub_s64, aarch64_sve_sqsub_x, 4),
SVEMAP1(svqsub_s64_m, aarch64_sve_sqsub, 260),
SVEMAP1(svqsub_s64_x, aarch64_sve_sqsub, 132),
SVEMAP1(svqsub_s64_z, aarch64_sve_sqsub, 388),
SVEMAP1(svqsub_s8, aarch64_sve_sqsub_x, 1),
SVEMAP1(svqsub_s8_m, aarch64_sve_sqsub, 257),
SVEMAP1(svqsub_s8_x, aarch64_sve_sqsub, 129),
SVEMAP1(svqsub_s8_z, aarch64_sve_sqsub, 385),
SVEMAP1(svqsub_u16, aarch64_sve_uqsub_x, 2),
SVEMAP1(svqsub_u16_m, aarch64_sve_uqsub, 258),
SVEMAP1(svqsub_u16_x, aarch64_sve_uqsub, 130),
SVEMAP1(svqsub_u16_z, aarch64_sve_uqsub, 386),
SVEMAP1(svqsub_u32, aarch64_sve_uqsub_x, 3),
SVEMAP1(svqsub_u32_m, aarch64_sve_uqsub, 259),
SVEMAP1(svqsub_u32_x, aarch64_sve_uqsub, 131),
SVEMAP1(svqsub_u32_z, aarch64_sve_uqsub, 387),
SVEMAP1(svqsub_u64, aarch64_sve_uqsub_x, 4),
SVEMAP1(svqsub_u64_m, aarch64_sve_uqsub, 260),
SVEMAP1(svqsub_u64_x, aarch64_sve_uqsub, 132),
SVEMAP1(svqsub_u64_z, aarch64_sve_uqsub, 388),
SVEMAP1(svqsub_u8, aarch64_sve_uqsub_x, 1),
SVEMAP1(svqsub_u8_m, aarch64_sve_uqsub, 257),
SVEMAP1(svqsub_u8_x, aarch64_sve_uqsub, 129),
SVEMAP1(svqsub_u8_z, aarch64_sve_uqsub, 385),
SVEMAP1(svqsubr_n_s16_m, aarch64_sve_sqsubr, 3330),
SVEMAP1(svqsubr_n_s16_x, aarch64_sve_sqsubr, 3202),
SVEMAP1(svqsubr_n_s16_z, aarch64_sve_sqsubr, 3458),
SVEMAP1(svqsubr_n_s32_m, aarch64_sve_sqsubr, 3331),
SVEMAP1(svqsubr_n_s32_x, aarch64_sve_sqsubr, 3203),
SVEMAP1(svqsubr_n_s32_z, aarch64_sve_sqsubr, 3459),
SVEMAP1(svqsubr_n_s64_m, aarch64_sve_sqsubr, 3332),
SVEMAP1(svqsubr_n_s64_x, aarch64_sve_sqsubr, 3204),
SVEMAP1(svqsubr_n_s64_z, aarch64_sve_sqsubr, 3460),
SVEMAP1(svqsubr_n_s8_m, aarch64_sve_sqsubr, 3329),
SVEMAP1(svqsubr_n_s8_x, aarch64_sve_sqsubr, 3201),
SVEMAP1(svqsubr_n_s8_z, aarch64_sve_sqsubr, 3457),
SVEMAP1(svqsubr_n_u16_m, aarch64_sve_uqsubr, 3330),
SVEMAP1(svqsubr_n_u16_x, aarch64_sve_uqsubr, 3202),
SVEMAP1(svqsubr_n_u16_z, aarch64_sve_uqsubr, 3458),
SVEMAP1(svqsubr_n_u32_m, aarch64_sve_uqsubr, 3331),
SVEMAP1(svqsubr_n_u32_x, aarch64_sve_uqsubr, 3203),
SVEMAP1(svqsubr_n_u32_z, aarch64_sve_uqsubr, 3459),
SVEMAP1(svqsubr_n_u64_m, aarch64_sve_uqsubr, 3332),
SVEMAP1(svqsubr_n_u64_x, aarch64_sve_uqsubr, 3204),
SVEMAP1(svqsubr_n_u64_z, aarch64_sve_uqsubr, 3460),
SVEMAP1(svqsubr_n_u8_m, aarch64_sve_uqsubr, 3329),
SVEMAP1(svqsubr_n_u8_x, aarch64_sve_uqsubr, 3201),
SVEMAP1(svqsubr_n_u8_z, aarch64_sve_uqsubr, 3457),
SVEMAP1(svqsubr_s16_m, aarch64_sve_sqsubr, 258),
SVEMAP1(svqsubr_s16_x, aarch64_sve_sqsubr, 130),
SVEMAP1(svqsubr_s16_z, aarch64_sve_sqsubr, 386),
SVEMAP1(svqsubr_s32_m, aarch64_sve_sqsubr, 259),
SVEMAP1(svqsubr_s32_x, aarch64_sve_sqsubr, 131),
SVEMAP1(svqsubr_s32_z, aarch64_sve_sqsubr, 387),
SVEMAP1(svqsubr_s64_m, aarch64_sve_sqsubr, 260),
SVEMAP1(svqsubr_s64_x, aarch64_sve_sqsubr, 132),
SVEMAP1(svqsubr_s64_z, aarch64_sve_sqsubr, 388),
SVEMAP1(svqsubr_s8_m, aarch64_sve_sqsubr, 257),
SVEMAP1(svqsubr_s8_x, aarch64_sve_sqsubr, 129),
SVEMAP1(svqsubr_s8_z, aarch64_sve_sqsubr, 385),
SVEMAP1(svqsubr_u16_m, aarch64_sve_uqsubr, 258),
SVEMAP1(svqsubr_u16_x, aarch64_sve_uqsubr, 130),
SVEMAP1(svqsubr_u16_z, aarch64_sve_uqsubr, 386),
SVEMAP1(svqsubr_u32_m, aarch64_sve_uqsubr, 259),
SVEMAP1(svqsubr_u32_x, aarch64_sve_uqsubr, 131),
SVEMAP1(svqsubr_u32_z, aarch64_sve_uqsubr, 387),
SVEMAP1(svqsubr_u64_m, aarch64_sve_uqsubr, 260),
SVEMAP1(svqsubr_u64_x, aarch64_sve_uqsubr, 132),
SVEMAP1(svqsubr_u64_z, aarch64_sve_uqsubr, 388),
SVEMAP1(svqsubr_u8_m, aarch64_sve_uqsubr, 257),
SVEMAP1(svqsubr_u8_x, aarch64_sve_uqsubr, 129),
SVEMAP1(svqsubr_u8_z, aarch64_sve_uqsubr, 385),
SVEMAP1(svqxtnb_s16, aarch64_sve_sqxtnb, 2),
SVEMAP1(svqxtnb_s32, aarch64_sve_sqxtnb, 3),
SVEMAP1(svqxtnb_s64, aarch64_sve_sqxtnb, 4),
SVEMAP1(svqxtnb_u16, aarch64_sve_uqxtnb, 2),
SVEMAP1(svqxtnb_u32, aarch64_sve_uqxtnb, 3),
SVEMAP1(svqxtnb_u64, aarch64_sve_uqxtnb, 4),
SVEMAP1(svqxtnt_s16, aarch64_sve_sqxtnt, 2),
SVEMAP1(svqxtnt_s32, aarch64_sve_sqxtnt, 3),
SVEMAP1(svqxtnt_s64, aarch64_sve_sqxtnt, 4),
SVEMAP1(svqxtnt_u16, aarch64_sve_uqxtnt, 2),
SVEMAP1(svqxtnt_u32, aarch64_sve_uqxtnt, 3),
SVEMAP1(svqxtnt_u64, aarch64_sve_uqxtnt, 4),
SVEMAP1(svqxtunb_s16, aarch64_sve_sqxtunb, 2),
SVEMAP1(svqxtunb_s32, aarch64_sve_sqxtunb, 3),
SVEMAP1(svqxtunb_s64, aarch64_sve_sqxtunb, 4),
SVEMAP1(svqxtunt_s16, aarch64_sve_sqxtunt, 2),
SVEMAP1(svqxtunt_s32, aarch64_sve_sqxtunt, 3),
SVEMAP1(svqxtunt_s64, aarch64_sve_sqxtunt, 4),
SVEMAP1(svraddhnb_n_s16, aarch64_sve_raddhnb, 2050),
SVEMAP1(svraddhnb_n_s32, aarch64_sve_raddhnb, 2051),
SVEMAP1(svraddhnb_n_s64, aarch64_sve_raddhnb, 2052),
SVEMAP1(svraddhnb_n_u16, aarch64_sve_raddhnb, 2050),
SVEMAP1(svraddhnb_n_u32, aarch64_sve_raddhnb, 2051),
SVEMAP1(svraddhnb_n_u64, aarch64_sve_raddhnb, 2052),
SVEMAP1(svraddhnb_s16, aarch64_sve_raddhnb, 2),
SVEMAP1(svraddhnb_s32, aarch64_sve_raddhnb, 3),
SVEMAP1(svraddhnb_s64, aarch64_sve_raddhnb, 4),
SVEMAP1(svraddhnb_u16, aarch64_sve_raddhnb, 2),
SVEMAP1(svraddhnb_u32, aarch64_sve_raddhnb, 3),
SVEMAP1(svraddhnb_u64, aarch64_sve_raddhnb, 4),
SVEMAP1(svraddhnt_n_s16, aarch64_sve_raddhnt, 3074),
SVEMAP1(svraddhnt_n_s32, aarch64_sve_raddhnt, 3075),
SVEMAP1(svraddhnt_n_s64, aarch64_sve_raddhnt, 3076),
SVEMAP1(svraddhnt_n_u16, aarch64_sve_raddhnt, 3074),
SVEMAP1(svraddhnt_n_u32, aarch64_sve_raddhnt, 3075),
SVEMAP1(svraddhnt_n_u64, aarch64_sve_raddhnt, 3076),
SVEMAP1(svraddhnt_s16, aarch64_sve_raddhnt, 2),
SVEMAP1(svraddhnt_s32, aarch64_sve_raddhnt, 3),
SVEMAP1(svraddhnt_s64, aarch64_sve_raddhnt, 4),
SVEMAP1(svraddhnt_u16, aarch64_sve_raddhnt, 2),
SVEMAP1(svraddhnt_u32, aarch64_sve_raddhnt, 3),
SVEMAP1(svraddhnt_u64, aarch64_sve_raddhnt, 4),
SVEMAP1(svrax1_s64, aarch64_sve_rax1, 1048580),
SVEMAP1(svrax1_u64, aarch64_sve_rax1, 1048580),
SVEMAP1(svrbit_s16_m, aarch64_sve_rbit, 258),
SVEMAP1(svrbit_s16_x, aarch64_sve_rbit, 514),
SVEMAP1(svrbit_s16_z, aarch64_sve_rbit, 642),
SVEMAP1(svrbit_s32_m, aarch64_sve_rbit, 259),
SVEMAP1(svrbit_s32_x, aarch64_sve_rbit, 515),
SVEMAP1(svrbit_s32_z, aarch64_sve_rbit, 643),
SVEMAP1(svrbit_s64_m, aarch64_sve_rbit, 260),
SVEMAP1(svrbit_s64_x, aarch64_sve_rbit, 516),
SVEMAP1(svrbit_s64_z, aarch64_sve_rbit, 644),
SVEMAP1(svrbit_s8_m, aarch64_sve_rbit, 257),
SVEMAP1(svrbit_s8_x, aarch64_sve_rbit, 513),
SVEMAP1(svrbit_s8_z, aarch64_sve_rbit, 641),
SVEMAP1(svrbit_u16_m, aarch64_sve_rbit, 258),
SVEMAP1(svrbit_u16_x, aarch64_sve_rbit, 514),
SVEMAP1(svrbit_u16_z, aarch64_sve_rbit, 642),
SVEMAP1(svrbit_u32_m, aarch64_sve_rbit, 259),
SVEMAP1(svrbit_u32_x, aarch64_sve_rbit, 515),
SVEMAP1(svrbit_u32_z, aarch64_sve_rbit, 643),
SVEMAP1(svrbit_u64_m, aarch64_sve_rbit, 260),
SVEMAP1(svrbit_u64_x, aarch64_sve_rbit, 516),
SVEMAP1(svrbit_u64_z, aarch64_sve_rbit, 644),
SVEMAP1(svrbit_u8_m, aarch64_sve_rbit, 257),
SVEMAP1(svrbit_u8_x, aarch64_sve_rbit, 513),
SVEMAP1(svrbit_u8_z, aarch64_sve_rbit, 641),
SVEMAP2(svrdffr, 1048584),
SVEMAP2(svrdffr_z, 1048584),
SVEMAP1(svrecpe_f16, aarch64_sve_frecpe_x, 5),
SVEMAP1(svrecpe_f32, aarch64_sve_frecpe_x, 6),
SVEMAP1(svrecpe_f64, aarch64_sve_frecpe_x, 7),
SVEMAP1(svrecpe_u32_m, aarch64_sve_urecpe, 259),
SVEMAP1(svrecpe_u32_x, aarch64_sve_urecpe, 515),
SVEMAP1(svrecpe_u32_z, aarch64_sve_urecpe, 643),
SVEMAP1(svrecps_f16, aarch64_sve_frecps_x, 5),
SVEMAP1(svrecps_f32, aarch64_sve_frecps_x, 6),
SVEMAP1(svrecps_f64, aarch64_sve_frecps_x, 7),
SVEMAP1(svrecpx_f16_m, aarch64_sve_frecpx, 261),
SVEMAP1(svrecpx_f16_x, aarch64_sve_frecpx, 517),
SVEMAP1(svrecpx_f16_z, aarch64_sve_frecpx, 645),
SVEMAP1(svrecpx_f32_m, aarch64_sve_frecpx, 262),
SVEMAP1(svrecpx_f32_x, aarch64_sve_frecpx, 518),
SVEMAP1(svrecpx_f32_z, aarch64_sve_frecpx, 646),
SVEMAP1(svrecpx_f64_m, aarch64_sve_frecpx, 263),
SVEMAP1(svrecpx_f64_x, aarch64_sve_frecpx, 519),
SVEMAP1(svrecpx_f64_z, aarch64_sve_frecpx, 647),
SVEMAP1(svrev_b16, aarch64_sve_rev, 9),
SVEMAP1(svrev_b32, aarch64_sve_rev, 10),
SVEMAP1(svrev_b64, aarch64_sve_rev, 11),
SVEMAP1(svrev_b8, aarch64_sve_rev, 8),
SVEMAP1(svrev_bf16, aarch64_sve_rev, 12),
SVEMAP1(svrev_f16, aarch64_sve_rev, 5),
SVEMAP1(svrev_f32, aarch64_sve_rev, 6),
SVEMAP1(svrev_f64, aarch64_sve_rev, 7),
SVEMAP1(svrev_s16, aarch64_sve_rev, 2),
SVEMAP1(svrev_s32, aarch64_sve_rev, 3),
SVEMAP1(svrev_s64, aarch64_sve_rev, 4),
SVEMAP1(svrev_s8, aarch64_sve_rev, 1),
SVEMAP1(svrev_u16, aarch64_sve_rev, 2),
SVEMAP1(svrev_u32, aarch64_sve_rev, 3),
SVEMAP1(svrev_u64, aarch64_sve_rev, 4),
SVEMAP1(svrev_u8, aarch64_sve_rev, 1),
SVEMAP1(svrevb_s16_m, aarch64_sve_revb, 258),
SVEMAP1(svrevb_s16_x, aarch64_sve_revb, 514),
SVEMAP1(svrevb_s16_z, aarch64_sve_revb, 642),
SVEMAP1(svrevb_s32_m, aarch64_sve_revb, 259),
SVEMAP1(svrevb_s32_x, aarch64_sve_revb, 515),
SVEMAP1(svrevb_s32_z, aarch64_sve_revb, 643),
SVEMAP1(svrevb_s64_m, aarch64_sve_revb, 260),
SVEMAP1(svrevb_s64_x, aarch64_sve_revb, 516),
SVEMAP1(svrevb_s64_z, aarch64_sve_revb, 644),
SVEMAP1(svrevb_u16_m, aarch64_sve_revb, 258),
SVEMAP1(svrevb_u16_x, aarch64_sve_revb, 514),
SVEMAP1(svrevb_u16_z, aarch64_sve_revb, 642),
SVEMAP1(svrevb_u32_m, aarch64_sve_revb, 259),
SVEMAP1(svrevb_u32_x, aarch64_sve_revb, 515),
SVEMAP1(svrevb_u32_z, aarch64_sve_revb, 643),
SVEMAP1(svrevb_u64_m, aarch64_sve_revb, 260),
SVEMAP1(svrevb_u64_x, aarch64_sve_revb, 516),
SVEMAP1(svrevb_u64_z, aarch64_sve_revb, 644),
SVEMAP1(svrevh_s32_m, aarch64_sve_revh, 259),
SVEMAP1(svrevh_s32_x, aarch64_sve_revh, 515),
SVEMAP1(svrevh_s32_z, aarch64_sve_revh, 643),
SVEMAP1(svrevh_s64_m, aarch64_sve_revh, 260),
SVEMAP1(svrevh_s64_x, aarch64_sve_revh, 516),
SVEMAP1(svrevh_s64_z, aarch64_sve_revh, 644),
SVEMAP1(svrevh_u32_m, aarch64_sve_revh, 259),
SVEMAP1(svrevh_u32_x, aarch64_sve_revh, 515),
SVEMAP1(svrevh_u32_z, aarch64_sve_revh, 643),
SVEMAP1(svrevh_u64_m, aarch64_sve_revh, 260),
SVEMAP1(svrevh_u64_x, aarch64_sve_revh, 516),
SVEMAP1(svrevh_u64_z, aarch64_sve_revh, 644),
SVEMAP1(svrevw_s64_m, aarch64_sve_revw, 260),
SVEMAP1(svrevw_s64_x, aarch64_sve_revw, 516),
SVEMAP1(svrevw_s64_z, aarch64_sve_revw, 644),
SVEMAP1(svrevw_u64_m, aarch64_sve_revw, 260),
SVEMAP1(svrevw_u64_x, aarch64_sve_revw, 516),
SVEMAP1(svrevw_u64_z, aarch64_sve_revw, 644),
SVEMAP1(svrhadd_n_s16_m, aarch64_sve_srhadd, 3330),
SVEMAP1(svrhadd_n_s16_x, aarch64_sve_srhadd, 3202),
SVEMAP1(svrhadd_n_s16_z, aarch64_sve_srhadd, 3458),
SVEMAP1(svrhadd_n_s32_m, aarch64_sve_srhadd, 3331),
SVEMAP1(svrhadd_n_s32_x, aarch64_sve_srhadd, 3203),
SVEMAP1(svrhadd_n_s32_z, aarch64_sve_srhadd, 3459),
SVEMAP1(svrhadd_n_s64_m, aarch64_sve_srhadd, 3332),
SVEMAP1(svrhadd_n_s64_x, aarch64_sve_srhadd, 3204),
SVEMAP1(svrhadd_n_s64_z, aarch64_sve_srhadd, 3460),
SVEMAP1(svrhadd_n_s8_m, aarch64_sve_srhadd, 3329),
SVEMAP1(svrhadd_n_s8_x, aarch64_sve_srhadd, 3201),
SVEMAP1(svrhadd_n_s8_z, aarch64_sve_srhadd, 3457),
SVEMAP1(svrhadd_n_u16_m, aarch64_sve_urhadd, 3330),
SVEMAP1(svrhadd_n_u16_x, aarch64_sve_urhadd, 3202),
SVEMAP1(svrhadd_n_u16_z, aarch64_sve_urhadd, 3458),
SVEMAP1(svrhadd_n_u32_m, aarch64_sve_urhadd, 3331),
SVEMAP1(svrhadd_n_u32_x, aarch64_sve_urhadd, 3203),
SVEMAP1(svrhadd_n_u32_z, aarch64_sve_urhadd, 3459),
SVEMAP1(svrhadd_n_u64_m, aarch64_sve_urhadd, 3332),
SVEMAP1(svrhadd_n_u64_x, aarch64_sve_urhadd, 3204),
SVEMAP1(svrhadd_n_u64_z, aarch64_sve_urhadd, 3460),
SVEMAP1(svrhadd_n_u8_m, aarch64_sve_urhadd, 3329),
SVEMAP1(svrhadd_n_u8_x, aarch64_sve_urhadd, 3201),
SVEMAP1(svrhadd_n_u8_z, aarch64_sve_urhadd, 3457),
SVEMAP1(svrhadd_s16_m, aarch64_sve_srhadd, 258),
SVEMAP1(svrhadd_s16_x, aarch64_sve_srhadd, 130),
SVEMAP1(svrhadd_s16_z, aarch64_sve_srhadd, 386),
SVEMAP1(svrhadd_s32_m, aarch64_sve_srhadd, 259),
SVEMAP1(svrhadd_s32_x, aarch64_sve_srhadd, 131),
SVEMAP1(svrhadd_s32_z, aarch64_sve_srhadd, 387),
SVEMAP1(svrhadd_s64_m, aarch64_sve_srhadd, 260),
SVEMAP1(svrhadd_s64_x, aarch64_sve_srhadd, 132),
SVEMAP1(svrhadd_s64_z, aarch64_sve_srhadd, 388),
SVEMAP1(svrhadd_s8_m, aarch64_sve_srhadd, 257),
SVEMAP1(svrhadd_s8_x, aarch64_sve_srhadd, 129),
SVEMAP1(svrhadd_s8_z, aarch64_sve_srhadd, 385),
SVEMAP1(svrhadd_u16_m, aarch64_sve_urhadd, 258),
SVEMAP1(svrhadd_u16_x, aarch64_sve_urhadd, 130),
SVEMAP1(svrhadd_u16_z, aarch64_sve_urhadd, 386),
SVEMAP1(svrhadd_u32_m, aarch64_sve_urhadd, 259),
SVEMAP1(svrhadd_u32_x, aarch64_sve_urhadd, 131),
SVEMAP1(svrhadd_u32_z, aarch64_sve_urhadd, 387),
SVEMAP1(svrhadd_u64_m, aarch64_sve_urhadd, 260),
SVEMAP1(svrhadd_u64_x, aarch64_sve_urhadd, 132),
SVEMAP1(svrhadd_u64_z, aarch64_sve_urhadd, 388),
SVEMAP1(svrhadd_u8_m, aarch64_sve_urhadd, 257),
SVEMAP1(svrhadd_u8_x, aarch64_sve_urhadd, 129),
SVEMAP1(svrhadd_u8_z, aarch64_sve_urhadd, 385),
SVEMAP1(svrinta_f16_m, aarch64_sve_frinta, 261),
SVEMAP1(svrinta_f16_x, aarch64_sve_frinta, 517),
SVEMAP1(svrinta_f16_z, aarch64_sve_frinta, 645),
SVEMAP1(svrinta_f32_m, aarch64_sve_frinta, 262),
SVEMAP1(svrinta_f32_x, aarch64_sve_frinta, 518),
SVEMAP1(svrinta_f32_z, aarch64_sve_frinta, 646),
SVEMAP1(svrinta_f64_m, aarch64_sve_frinta, 263),
SVEMAP1(svrinta_f64_x, aarch64_sve_frinta, 519),
SVEMAP1(svrinta_f64_z, aarch64_sve_frinta, 647),
SVEMAP1(svrinti_f16_m, aarch64_sve_frinti, 261),
SVEMAP1(svrinti_f16_x, aarch64_sve_frinti, 517),
SVEMAP1(svrinti_f16_z, aarch64_sve_frinti, 645),
SVEMAP1(svrinti_f32_m, aarch64_sve_frinti, 262),
SVEMAP1(svrinti_f32_x, aarch64_sve_frinti, 518),
SVEMAP1(svrinti_f32_z, aarch64_sve_frinti, 646),
SVEMAP1(svrinti_f64_m, aarch64_sve_frinti, 263),
SVEMAP1(svrinti_f64_x, aarch64_sve_frinti, 519),
SVEMAP1(svrinti_f64_z, aarch64_sve_frinti, 647),
SVEMAP1(svrintm_f16_m, aarch64_sve_frintm, 261),
SVEMAP1(svrintm_f16_x, aarch64_sve_frintm, 517),
SVEMAP1(svrintm_f16_z, aarch64_sve_frintm, 645),
SVEMAP1(svrintm_f32_m, aarch64_sve_frintm, 262),
SVEMAP1(svrintm_f32_x, aarch64_sve_frintm, 518),
SVEMAP1(svrintm_f32_z, aarch64_sve_frintm, 646),
SVEMAP1(svrintm_f64_m, aarch64_sve_frintm, 263),
SVEMAP1(svrintm_f64_x, aarch64_sve_frintm, 519),
SVEMAP1(svrintm_f64_z, aarch64_sve_frintm, 647),
SVEMAP1(svrintn_f16_m, aarch64_sve_frintn, 261),
SVEMAP1(svrintn_f16_x, aarch64_sve_frintn, 517),
SVEMAP1(svrintn_f16_z, aarch64_sve_frintn, 645),
SVEMAP1(svrintn_f32_m, aarch64_sve_frintn, 262),
SVEMAP1(svrintn_f32_x, aarch64_sve_frintn, 518),
SVEMAP1(svrintn_f32_z, aarch64_sve_frintn, 646),
SVEMAP1(svrintn_f64_m, aarch64_sve_frintn, 263),
SVEMAP1(svrintn_f64_x, aarch64_sve_frintn, 519),
SVEMAP1(svrintn_f64_z, aarch64_sve_frintn, 647),
SVEMAP1(svrintp_f16_m, aarch64_sve_frintp, 261),
SVEMAP1(svrintp_f16_x, aarch64_sve_frintp, 517),
SVEMAP1(svrintp_f16_z, aarch64_sve_frintp, 645),
SVEMAP1(svrintp_f32_m, aarch64_sve_frintp, 262),
SVEMAP1(svrintp_f32_x, aarch64_sve_frintp, 518),
SVEMAP1(svrintp_f32_z, aarch64_sve_frintp, 646),
SVEMAP1(svrintp_f64_m, aarch64_sve_frintp, 263),
SVEMAP1(svrintp_f64_x, aarch64_sve_frintp, 519),
SVEMAP1(svrintp_f64_z, aarch64_sve_frintp, 647),
SVEMAP1(svrintx_f16_m, aarch64_sve_frintx, 261),
SVEMAP1(svrintx_f16_x, aarch64_sve_frintx, 517),
SVEMAP1(svrintx_f16_z, aarch64_sve_frintx, 645),
SVEMAP1(svrintx_f32_m, aarch64_sve_frintx, 262),
SVEMAP1(svrintx_f32_x, aarch64_sve_frintx, 518),
SVEMAP1(svrintx_f32_z, aarch64_sve_frintx, 646),
SVEMAP1(svrintx_f64_m, aarch64_sve_frintx, 263),
SVEMAP1(svrintx_f64_x, aarch64_sve_frintx, 519),
SVEMAP1(svrintx_f64_z, aarch64_sve_frintx, 647),
SVEMAP1(svrintz_f16_m, aarch64_sve_frintz, 261),
SVEMAP1(svrintz_f16_x, aarch64_sve_frintz, 517),
SVEMAP1(svrintz_f16_z, aarch64_sve_frintz, 645),
SVEMAP1(svrintz_f32_m, aarch64_sve_frintz, 262),
SVEMAP1(svrintz_f32_x, aarch64_sve_frintz, 518),
SVEMAP1(svrintz_f32_z, aarch64_sve_frintz, 646),
SVEMAP1(svrintz_f64_m, aarch64_sve_frintz, 263),
SVEMAP1(svrintz_f64_x, aarch64_sve_frintz, 519),
SVEMAP1(svrintz_f64_z, aarch64_sve_frintz, 647),
SVEMAP1(svrshl_n_s16_m, aarch64_sve_srshl, 3330),
SVEMAP1(svrshl_n_s16_x, aarch64_sve_srshl, 3202),
SVEMAP1(svrshl_n_s16_z, aarch64_sve_srshl, 3458),
SVEMAP1(svrshl_n_s32_m, aarch64_sve_srshl, 3331),
SVEMAP1(svrshl_n_s32_x, aarch64_sve_srshl, 3203),
SVEMAP1(svrshl_n_s32_z, aarch64_sve_srshl, 3459),
SVEMAP1(svrshl_n_s64_m, aarch64_sve_srshl, 3332),
SVEMAP1(svrshl_n_s64_x, aarch64_sve_srshl, 3204),
SVEMAP1(svrshl_n_s64_z, aarch64_sve_srshl, 3460),
SVEMAP1(svrshl_n_s8_m, aarch64_sve_srshl, 3329),
SVEMAP1(svrshl_n_s8_x, aarch64_sve_srshl, 3201),
SVEMAP1(svrshl_n_s8_z, aarch64_sve_srshl, 3457),
SVEMAP1(svrshl_n_u16_m, aarch64_sve_urshl, 3330),
SVEMAP1(svrshl_n_u16_x, aarch64_sve_urshl, 3202),
SVEMAP1(svrshl_n_u16_z, aarch64_sve_urshl, 3458),
SVEMAP1(svrshl_n_u32_m, aarch64_sve_urshl, 3331),
SVEMAP1(svrshl_n_u32_x, aarch64_sve_urshl, 3203),
SVEMAP1(svrshl_n_u32_z, aarch64_sve_urshl, 3459),
SVEMAP1(svrshl_n_u64_m, aarch64_sve_urshl, 3332),
SVEMAP1(svrshl_n_u64_x, aarch64_sve_urshl, 3204),
SVEMAP1(svrshl_n_u64_z, aarch64_sve_urshl, 3460),
SVEMAP1(svrshl_n_u8_m, aarch64_sve_urshl, 3329),
SVEMAP1(svrshl_n_u8_x, aarch64_sve_urshl, 3201),
SVEMAP1(svrshl_n_u8_z, aarch64_sve_urshl, 3457),
SVEMAP1(svrshl_s16_m, aarch64_sve_srshl, 258),
SVEMAP1(svrshl_s16_x, aarch64_sve_srshl, 130),
SVEMAP1(svrshl_s16_z, aarch64_sve_srshl, 386),
SVEMAP1(svrshl_s32_m, aarch64_sve_srshl, 259),
SVEMAP1(svrshl_s32_x, aarch64_sve_srshl, 131),
SVEMAP1(svrshl_s32_z, aarch64_sve_srshl, 387),
SVEMAP1(svrshl_s64_m, aarch64_sve_srshl, 260),
SVEMAP1(svrshl_s64_x, aarch64_sve_srshl, 132),
SVEMAP1(svrshl_s64_z, aarch64_sve_srshl, 388),
SVEMAP1(svrshl_s8_m, aarch64_sve_srshl, 257),
SVEMAP1(svrshl_s8_x, aarch64_sve_srshl, 129),
SVEMAP1(svrshl_s8_z, aarch64_sve_srshl, 385),
SVEMAP1(svrshl_u16_m, aarch64_sve_urshl, 258),
SVEMAP1(svrshl_u16_x, aarch64_sve_urshl, 130),
SVEMAP1(svrshl_u16_z, aarch64_sve_urshl, 386),
SVEMAP1(svrshl_u32_m, aarch64_sve_urshl, 259),
SVEMAP1(svrshl_u32_x, aarch64_sve_urshl, 131),
SVEMAP1(svrshl_u32_z, aarch64_sve_urshl, 387),
SVEMAP1(svrshl_u64_m, aarch64_sve_urshl, 260),
SVEMAP1(svrshl_u64_x, aarch64_sve_urshl, 132),
SVEMAP1(svrshl_u64_z, aarch64_sve_urshl, 388),
SVEMAP1(svrshl_u8_m, aarch64_sve_urshl, 257),
SVEMAP1(svrshl_u8_x, aarch64_sve_urshl, 129),
SVEMAP1(svrshl_u8_z, aarch64_sve_urshl, 385),
SVEMAP1(svrshr_n_s16_m, aarch64_sve_srshr, 258),
SVEMAP1(svrshr_n_s16_x, aarch64_sve_srshr, 130),
SVEMAP1(svrshr_n_s16_z, aarch64_sve_srshr, 386),
SVEMAP1(svrshr_n_s32_m, aarch64_sve_srshr, 259),
SVEMAP1(svrshr_n_s32_x, aarch64_sve_srshr, 131),
SVEMAP1(svrshr_n_s32_z, aarch64_sve_srshr, 387),
SVEMAP1(svrshr_n_s64_m, aarch64_sve_srshr, 260),
SVEMAP1(svrshr_n_s64_x, aarch64_sve_srshr, 132),
SVEMAP1(svrshr_n_s64_z, aarch64_sve_srshr, 388),
SVEMAP1(svrshr_n_s8_m, aarch64_sve_srshr, 257),
SVEMAP1(svrshr_n_s8_x, aarch64_sve_srshr, 129),
SVEMAP1(svrshr_n_s8_z, aarch64_sve_srshr, 385),
SVEMAP1(svrshr_n_u16_m, aarch64_sve_urshr, 258),
SVEMAP1(svrshr_n_u16_x, aarch64_sve_urshr, 130),
SVEMAP1(svrshr_n_u16_z, aarch64_sve_urshr, 386),
SVEMAP1(svrshr_n_u32_m, aarch64_sve_urshr, 259),
SVEMAP1(svrshr_n_u32_x, aarch64_sve_urshr, 131),
SVEMAP1(svrshr_n_u32_z, aarch64_sve_urshr, 387),
SVEMAP1(svrshr_n_u64_m, aarch64_sve_urshr, 260),
SVEMAP1(svrshr_n_u64_x, aarch64_sve_urshr, 132),
SVEMAP1(svrshr_n_u64_z, aarch64_sve_urshr, 388),
SVEMAP1(svrshr_n_u8_m, aarch64_sve_urshr, 257),
SVEMAP1(svrshr_n_u8_x, aarch64_sve_urshr, 129),
SVEMAP1(svrshr_n_u8_z, aarch64_sve_urshr, 385),
SVEMAP1(svrshrnb_n_s16, aarch64_sve_rshrnb, 2),
SVEMAP1(svrshrnb_n_s32, aarch64_sve_rshrnb, 3),
SVEMAP1(svrshrnb_n_s64, aarch64_sve_rshrnb, 4),
SVEMAP1(svrshrnb_n_u16, aarch64_sve_rshrnb, 2),
SVEMAP1(svrshrnb_n_u32, aarch64_sve_rshrnb, 3),
SVEMAP1(svrshrnb_n_u64, aarch64_sve_rshrnb, 4),
SVEMAP1(svrshrnt_n_s16, aarch64_sve_rshrnt, 2),
SVEMAP1(svrshrnt_n_s32, aarch64_sve_rshrnt, 3),
SVEMAP1(svrshrnt_n_s64, aarch64_sve_rshrnt, 4),
SVEMAP1(svrshrnt_n_u16, aarch64_sve_rshrnt, 2),
SVEMAP1(svrshrnt_n_u32, aarch64_sve_rshrnt, 3),
SVEMAP1(svrshrnt_n_u64, aarch64_sve_rshrnt, 4),
SVEMAP1(svrsqrte_f16, aarch64_sve_frsqrte_x, 5),
SVEMAP1(svrsqrte_f32, aarch64_sve_frsqrte_x, 6),
SVEMAP1(svrsqrte_f64, aarch64_sve_frsqrte_x, 7),
SVEMAP1(svrsqrte_u32_m, aarch64_sve_ursqrte, 259),
SVEMAP1(svrsqrte_u32_x, aarch64_sve_ursqrte, 515),
SVEMAP1(svrsqrte_u32_z, aarch64_sve_ursqrte, 643),
SVEMAP1(svrsqrts_f16, aarch64_sve_frsqrts_x, 5),
SVEMAP1(svrsqrts_f32, aarch64_sve_frsqrts_x, 6),
SVEMAP1(svrsqrts_f64, aarch64_sve_frsqrts_x, 7),
SVEMAP1(svrsra_n_s16, aarch64_sve_srsra, 2),
SVEMAP1(svrsra_n_s32, aarch64_sve_srsra, 3),
SVEMAP1(svrsra_n_s64, aarch64_sve_srsra, 4),
SVEMAP1(svrsra_n_s8, aarch64_sve_srsra, 1),
SVEMAP1(svrsra_n_u16, aarch64_sve_ursra, 2),
SVEMAP1(svrsra_n_u32, aarch64_sve_ursra, 3),
SVEMAP1(svrsra_n_u64, aarch64_sve_ursra, 4),
SVEMAP1(svrsra_n_u8, aarch64_sve_ursra, 1),
SVEMAP1(svrsubhnb_n_s16, aarch64_sve_rsubhnb, 2050),
SVEMAP1(svrsubhnb_n_s32, aarch64_sve_rsubhnb, 2051),
SVEMAP1(svrsubhnb_n_s64, aarch64_sve_rsubhnb, 2052),
SVEMAP1(svrsubhnb_n_u16, aarch64_sve_rsubhnb, 2050),
SVEMAP1(svrsubhnb_n_u32, aarch64_sve_rsubhnb, 2051),
SVEMAP1(svrsubhnb_n_u64, aarch64_sve_rsubhnb, 2052),
SVEMAP1(svrsubhnb_s16, aarch64_sve_rsubhnb, 2),
SVEMAP1(svrsubhnb_s32, aarch64_sve_rsubhnb, 3),
SVEMAP1(svrsubhnb_s64, aarch64_sve_rsubhnb, 4),
SVEMAP1(svrsubhnb_u16, aarch64_sve_rsubhnb, 2),
SVEMAP1(svrsubhnb_u32, aarch64_sve_rsubhnb, 3),
SVEMAP1(svrsubhnb_u64, aarch64_sve_rsubhnb, 4),
SVEMAP1(svrsubhnt_n_s16, aarch64_sve_rsubhnt, 3074),
SVEMAP1(svrsubhnt_n_s32, aarch64_sve_rsubhnt, 3075),
SVEMAP1(svrsubhnt_n_s64, aarch64_sve_rsubhnt, 3076),
SVEMAP1(svrsubhnt_n_u16, aarch64_sve_rsubhnt, 3074),
SVEMAP1(svrsubhnt_n_u32, aarch64_sve_rsubhnt, 3075),
SVEMAP1(svrsubhnt_n_u64, aarch64_sve_rsubhnt, 3076),
SVEMAP1(svrsubhnt_s16, aarch64_sve_rsubhnt, 2),
SVEMAP1(svrsubhnt_s32, aarch64_sve_rsubhnt, 3),
SVEMAP1(svrsubhnt_s64, aarch64_sve_rsubhnt, 4),
SVEMAP1(svrsubhnt_u16, aarch64_sve_rsubhnt, 2),
SVEMAP1(svrsubhnt_u32, aarch64_sve_rsubhnt, 3),
SVEMAP1(svrsubhnt_u64, aarch64_sve_rsubhnt, 4),
SVEMAP1(svsbclb_n_u32, aarch64_sve_sbclb, 3075),
SVEMAP1(svsbclb_n_u64, aarch64_sve_sbclb, 3076),
SVEMAP1(svsbclb_u32, aarch64_sve_sbclb, 3),
SVEMAP1(svsbclb_u64, aarch64_sve_sbclb, 4),
SVEMAP1(svsbclt_n_u32, aarch64_sve_sbclt, 3075),
SVEMAP1(svsbclt_n_u64, aarch64_sve_sbclt, 3076),
SVEMAP1(svsbclt_u32, aarch64_sve_sbclt, 3),
SVEMAP1(svsbclt_u64, aarch64_sve_sbclt, 4),
SVEMAP1(svscale_f16_m, aarch64_sve_fscale, 261),
SVEMAP1(svscale_f16_x, aarch64_sve_fscale, 133),
SVEMAP1(svscale_f16_z, aarch64_sve_fscale, 389),
SVEMAP1(svscale_f32_m, aarch64_sve_fscale, 262),
SVEMAP1(svscale_f32_x, aarch64_sve_fscale, 134),
SVEMAP1(svscale_f32_z, aarch64_sve_fscale, 390),
SVEMAP1(svscale_f64_m, aarch64_sve_fscale, 263),
SVEMAP1(svscale_f64_x, aarch64_sve_fscale, 135),
SVEMAP1(svscale_f64_z, aarch64_sve_fscale, 391),
SVEMAP1(svscale_n_f16_m, aarch64_sve_fscale, 3333),
SVEMAP1(svscale_n_f16_x, aarch64_sve_fscale, 3205),
SVEMAP1(svscale_n_f16_z, aarch64_sve_fscale, 3461),
SVEMAP1(svscale_n_f32_m, aarch64_sve_fscale, 3334),
SVEMAP1(svscale_n_f32_x, aarch64_sve_fscale, 3206),
SVEMAP1(svscale_n_f32_z, aarch64_sve_fscale, 3462),
SVEMAP1(svscale_n_f64_m, aarch64_sve_fscale, 3335),
SVEMAP1(svscale_n_f64_x, aarch64_sve_fscale, 3207),
SVEMAP1(svscale_n_f64_z, aarch64_sve_fscale, 3463),
SVEMAP1(svsel_b, aarch64_sve_sel, 8),
SVEMAP1(svsel_bf16, aarch64_sve_sel, 12),
SVEMAP1(svsel_f16, aarch64_sve_sel, 5),
SVEMAP1(svsel_f32, aarch64_sve_sel, 6),
SVEMAP1(svsel_f64, aarch64_sve_sel, 7),
SVEMAP1(svsel_s16, aarch64_sve_sel, 2),
SVEMAP1(svsel_s32, aarch64_sve_sel, 3),
SVEMAP1(svsel_s64, aarch64_sve_sel, 4),
SVEMAP1(svsel_s8, aarch64_sve_sel, 1),
SVEMAP1(svsel_u16, aarch64_sve_sel, 2),
SVEMAP1(svsel_u32, aarch64_sve_sel, 3),
SVEMAP1(svsel_u64, aarch64_sve_sel, 4),
SVEMAP1(svsel_u8, aarch64_sve_sel, 1),
SVEMAP1(svset2_bf16, aarch64_sve_tuple_set, 17179869196),
SVEMAP1(svset2_f16, aarch64_sve_tuple_set, 17179869189),
SVEMAP1(svset2_f32, aarch64_sve_tuple_set, 17179869190),
SVEMAP1(svset2_f64, aarch64_sve_tuple_set, 17179869191),
SVEMAP1(svset2_s16, aarch64_sve_tuple_set, 17179869186),
SVEMAP1(svset2_s32, aarch64_sve_tuple_set, 17179869187),
SVEMAP1(svset2_s64, aarch64_sve_tuple_set, 17179869188),
SVEMAP1(svset2_s8, aarch64_sve_tuple_set, 17179869185),
SVEMAP1(svset2_u16, aarch64_sve_tuple_set, 17179869186),
SVEMAP1(svset2_u32, aarch64_sve_tuple_set, 17179869187),
SVEMAP1(svset2_u64, aarch64_sve_tuple_set, 17179869188),
SVEMAP1(svset2_u8, aarch64_sve_tuple_set, 17179869185),
SVEMAP1(svset3_bf16, aarch64_sve_tuple_set, 17179869196),
SVEMAP1(svset3_f16, aarch64_sve_tuple_set, 17179869189),
SVEMAP1(svset3_f32, aarch64_sve_tuple_set, 17179869190),
SVEMAP1(svset3_f64, aarch64_sve_tuple_set, 17179869191),
SVEMAP1(svset3_s16, aarch64_sve_tuple_set, 17179869186),
SVEMAP1(svset3_s32, aarch64_sve_tuple_set, 17179869187),
SVEMAP1(svset3_s64, aarch64_sve_tuple_set, 17179869188),
SVEMAP1(svset3_s8, aarch64_sve_tuple_set, 17179869185),
SVEMAP1(svset3_u16, aarch64_sve_tuple_set, 17179869186),
SVEMAP1(svset3_u32, aarch64_sve_tuple_set, 17179869187),
SVEMAP1(svset3_u64, aarch64_sve_tuple_set, 17179869188),
SVEMAP1(svset3_u8, aarch64_sve_tuple_set, 17179869185),
SVEMAP1(svset4_bf16, aarch64_sve_tuple_set, 17179869196),
SVEMAP1(svset4_f16, aarch64_sve_tuple_set, 17179869189),
SVEMAP1(svset4_f32, aarch64_sve_tuple_set, 17179869190),
SVEMAP1(svset4_f64, aarch64_sve_tuple_set, 17179869191),
SVEMAP1(svset4_s16, aarch64_sve_tuple_set, 17179869186),
SVEMAP1(svset4_s32, aarch64_sve_tuple_set, 17179869187),
SVEMAP1(svset4_s64, aarch64_sve_tuple_set, 17179869188),
SVEMAP1(svset4_s8, aarch64_sve_tuple_set, 17179869185),
SVEMAP1(svset4_u16, aarch64_sve_tuple_set, 17179869186),
SVEMAP1(svset4_u32, aarch64_sve_tuple_set, 17179869187),
SVEMAP1(svset4_u64, aarch64_sve_tuple_set, 17179869188),
SVEMAP1(svset4_u8, aarch64_sve_tuple_set, 17179869185),
SVEMAP2(svsetffr, 1048579),
SVEMAP1(svshllb_n_s16, aarch64_sve_sshllb, 2),
SVEMAP1(svshllb_n_s32, aarch64_sve_sshllb, 3),
SVEMAP1(svshllb_n_s64, aarch64_sve_sshllb, 4),
SVEMAP1(svshllb_n_u16, aarch64_sve_ushllb, 2),
SVEMAP1(svshllb_n_u32, aarch64_sve_ushllb, 3),
SVEMAP1(svshllb_n_u64, aarch64_sve_ushllb, 4),
SVEMAP1(svshllt_n_s16, aarch64_sve_sshllt, 2),
SVEMAP1(svshllt_n_s32, aarch64_sve_sshllt, 3),
SVEMAP1(svshllt_n_s64, aarch64_sve_sshllt, 4),
SVEMAP1(svshllt_n_u16, aarch64_sve_ushllt, 2),
SVEMAP1(svshllt_n_u32, aarch64_sve_ushllt, 3),
SVEMAP1(svshllt_n_u64, aarch64_sve_ushllt, 4),
SVEMAP1(svshrnb_n_s16, aarch64_sve_shrnb, 2),
SVEMAP1(svshrnb_n_s32, aarch64_sve_shrnb, 3),
SVEMAP1(svshrnb_n_s64, aarch64_sve_shrnb, 4),
SVEMAP1(svshrnb_n_u16, aarch64_sve_shrnb, 2),
SVEMAP1(svshrnb_n_u32, aarch64_sve_shrnb, 3),
SVEMAP1(svshrnb_n_u64, aarch64_sve_shrnb, 4),
SVEMAP1(svshrnt_n_s16, aarch64_sve_shrnt, 2),
SVEMAP1(svshrnt_n_s32, aarch64_sve_shrnt, 3),
SVEMAP1(svshrnt_n_s64, aarch64_sve_shrnt, 4),
SVEMAP1(svshrnt_n_u16, aarch64_sve_shrnt, 2),
SVEMAP1(svshrnt_n_u32, aarch64_sve_shrnt, 3),
SVEMAP1(svshrnt_n_u64, aarch64_sve_shrnt, 4),
SVEMAP1(svsli_n_s16, aarch64_sve_sli, 2),
SVEMAP1(svsli_n_s32, aarch64_sve_sli, 3),
SVEMAP1(svsli_n_s64, aarch64_sve_sli, 4),
SVEMAP1(svsli_n_s8, aarch64_sve_sli, 1),
SVEMAP1(svsli_n_u16, aarch64_sve_sli, 2),
SVEMAP1(svsli_n_u32, aarch64_sve_sli, 3),
SVEMAP1(svsli_n_u64, aarch64_sve_sli, 4),
SVEMAP1(svsli_n_u8, aarch64_sve_sli, 1),
SVEMAP1(svsm4e_u32, aarch64_sve_sm4e, 1048579),
SVEMAP1(svsm4ekey_u32, aarch64_sve_sm4ekey, 1048579),
SVEMAP1(svsplice_bf16, aarch64_sve_splice, 12),
SVEMAP1(svsplice_f16, aarch64_sve_splice, 5),
SVEMAP1(svsplice_f32, aarch64_sve_splice, 6),
SVEMAP1(svsplice_f64, aarch64_sve_splice, 7),
SVEMAP1(svsplice_s16, aarch64_sve_splice, 2),
SVEMAP1(svsplice_s32, aarch64_sve_splice, 3),
SVEMAP1(svsplice_s64, aarch64_sve_splice, 4),
SVEMAP1(svsplice_s8, aarch64_sve_splice, 1),
SVEMAP1(svsplice_u16, aarch64_sve_splice, 2),
SVEMAP1(svsplice_u32, aarch64_sve_splice, 3),
SVEMAP1(svsplice_u64, aarch64_sve_splice, 4),
SVEMAP1(svsplice_u8, aarch64_sve_splice, 1),
SVEMAP1(svsqadd_n_u16_m, aarch64_sve_usqadd, 3330),
SVEMAP1(svsqadd_n_u16_x, aarch64_sve_usqadd, 3202),
SVEMAP1(svsqadd_n_u16_z, aarch64_sve_usqadd, 3458),
SVEMAP1(svsqadd_n_u32_m, aarch64_sve_usqadd, 3331),
SVEMAP1(svsqadd_n_u32_x, aarch64_sve_usqadd, 3203),
SVEMAP1(svsqadd_n_u32_z, aarch64_sve_usqadd, 3459),
SVEMAP1(svsqadd_n_u64_m, aarch64_sve_usqadd, 3332),
SVEMAP1(svsqadd_n_u64_x, aarch64_sve_usqadd, 3204),
SVEMAP1(svsqadd_n_u64_z, aarch64_sve_usqadd, 3460),
SVEMAP1(svsqadd_n_u8_m, aarch64_sve_usqadd, 3329),
SVEMAP1(svsqadd_n_u8_x, aarch64_sve_usqadd, 3201),
SVEMAP1(svsqadd_n_u8_z, aarch64_sve_usqadd, 3457),
SVEMAP1(svsqadd_u16_m, aarch64_sve_usqadd, 258),
SVEMAP1(svsqadd_u16_x, aarch64_sve_usqadd, 130),
SVEMAP1(svsqadd_u16_z, aarch64_sve_usqadd, 386),
SVEMAP1(svsqadd_u32_m, aarch64_sve_usqadd, 259),
SVEMAP1(svsqadd_u32_x, aarch64_sve_usqadd, 131),
SVEMAP1(svsqadd_u32_z, aarch64_sve_usqadd, 387),
SVEMAP1(svsqadd_u64_m, aarch64_sve_usqadd, 260),
SVEMAP1(svsqadd_u64_x, aarch64_sve_usqadd, 132),
SVEMAP1(svsqadd_u64_z, aarch64_sve_usqadd, 388),
SVEMAP1(svsqadd_u8_m, aarch64_sve_usqadd, 257),
SVEMAP1(svsqadd_u8_x, aarch64_sve_usqadd, 129),
SVEMAP1(svsqadd_u8_z, aarch64_sve_usqadd, 385),
SVEMAP1(svsqrt_f16_m, aarch64_sve_fsqrt, 261),
SVEMAP1(svsqrt_f16_x, aarch64_sve_fsqrt, 517),
SVEMAP1(svsqrt_f16_z, aarch64_sve_fsqrt, 645),
SVEMAP1(svsqrt_f32_m, aarch64_sve_fsqrt, 262),
SVEMAP1(svsqrt_f32_x, aarch64_sve_fsqrt, 518),
SVEMAP1(svsqrt_f32_z, aarch64_sve_fsqrt, 646),
SVEMAP1(svsqrt_f64_m, aarch64_sve_fsqrt, 263),
SVEMAP1(svsqrt_f64_x, aarch64_sve_fsqrt, 519),
SVEMAP1(svsqrt_f64_z, aarch64_sve_fsqrt, 647),
SVEMAP1(svsra_n_s16, aarch64_sve_ssra, 2),
SVEMAP1(svsra_n_s32, aarch64_sve_ssra, 3),
SVEMAP1(svsra_n_s64, aarch64_sve_ssra, 4),
SVEMAP1(svsra_n_s8, aarch64_sve_ssra, 1),
SVEMAP1(svsra_n_u16, aarch64_sve_usra, 2),
SVEMAP1(svsra_n_u32, aarch64_sve_usra, 3),
SVEMAP1(svsra_n_u64, aarch64_sve_usra, 4),
SVEMAP1(svsra_n_u8, aarch64_sve_usra, 1),
SVEMAP1(svsri_n_s16, aarch64_sve_sri, 2),
SVEMAP1(svsri_n_s32, aarch64_sve_sri, 3),
SVEMAP1(svsri_n_s64, aarch64_sve_sri, 4),
SVEMAP1(svsri_n_s8, aarch64_sve_sri, 1),
SVEMAP1(svsri_n_u16, aarch64_sve_sri, 2),
SVEMAP1(svsri_n_u32, aarch64_sve_sri, 3),
SVEMAP1(svsri_n_u64, aarch64_sve_sri, 4),
SVEMAP1(svsri_n_u8, aarch64_sve_sri, 1),
SVEMAP1(svst1_bf16, aarch64_sve_st1, 16396),
SVEMAP1(svst1_f16, aarch64_sve_st1, 16389),
SVEMAP1(svst1_f32, aarch64_sve_st1, 16390),
SVEMAP1(svst1_f64, aarch64_sve_st1, 16391),
SVEMAP1(svst1_s16, aarch64_sve_st1, 16386),
SVEMAP1(svst1_s32, aarch64_sve_st1, 16387),
SVEMAP1(svst1_s64, aarch64_sve_st1, 16388),
SVEMAP1(svst1_s8, aarch64_sve_st1, 16385),
SVEMAP1(svst1_scatter_s32index_f32, aarch64_sve_st1_scatter_sxtw_index, 65542),
SVEMAP1(svst1_scatter_s32index_s32, aarch64_sve_st1_scatter_sxtw_index, 65539),
SVEMAP1(svst1_scatter_s32index_u32, aarch64_sve_st1_scatter_sxtw_index, 65539),
SVEMAP1(svst1_scatter_s32offset_f32, aarch64_sve_st1_scatter_sxtw, 16842758),
SVEMAP1(svst1_scatter_s32offset_s32, aarch64_sve_st1_scatter_sxtw, 16842755),
SVEMAP1(svst1_scatter_s32offset_u32, aarch64_sve_st1_scatter_sxtw, 16842755),
SVEMAP1(svst1_scatter_s64index_f64, aarch64_sve_st1_scatter_index, 65543),
SVEMAP1(svst1_scatter_s64index_s64, aarch64_sve_st1_scatter_index, 65540),
SVEMAP1(svst1_scatter_s64index_u64, aarch64_sve_st1_scatter_index, 65540),
SVEMAP1(svst1_scatter_s64offset_f64, aarch64_sve_st1_scatter, 16842759),
SVEMAP1(svst1_scatter_s64offset_s64, aarch64_sve_st1_scatter, 16842756),
SVEMAP1(svst1_scatter_s64offset_u64, aarch64_sve_st1_scatter, 16842756),
SVEMAP1(svst1_scatter_u32base_f32, aarch64_sve_st1_scatter_scalar_offset, 65542),
SVEMAP1(svst1_scatter_u32base_index_f32, aarch64_sve_st1_scatter_scalar_offset, 65542),
SVEMAP1(svst1_scatter_u32base_index_s32, aarch64_sve_st1_scatter_scalar_offset, 65539),
SVEMAP1(svst1_scatter_u32base_index_u32, aarch64_sve_st1_scatter_scalar_offset, 65539),
SVEMAP1(svst1_scatter_u32base_offset_f32, aarch64_sve_st1_scatter_scalar_offset, 16842758),
SVEMAP1(svst1_scatter_u32base_offset_s32, aarch64_sve_st1_scatter_scalar_offset, 16842755),
SVEMAP1(svst1_scatter_u32base_offset_u32, aarch64_sve_st1_scatter_scalar_offset, 16842755),
SVEMAP1(svst1_scatter_u32base_s32, aarch64_sve_st1_scatter_scalar_offset, 65539),
SVEMAP1(svst1_scatter_u32base_u32, aarch64_sve_st1_scatter_scalar_offset, 65539),
SVEMAP1(svst1_scatter_u32index_f32, aarch64_sve_st1_scatter_uxtw_index, 65542),
SVEMAP1(svst1_scatter_u32index_s32, aarch64_sve_st1_scatter_uxtw_index, 65539),
SVEMAP1(svst1_scatter_u32index_u32, aarch64_sve_st1_scatter_uxtw_index, 65539),
SVEMAP1(svst1_scatter_u32offset_f32, aarch64_sve_st1_scatter_uxtw, 16842758),
SVEMAP1(svst1_scatter_u32offset_s32, aarch64_sve_st1_scatter_uxtw, 16842755),
SVEMAP1(svst1_scatter_u32offset_u32, aarch64_sve_st1_scatter_uxtw, 16842755),
SVEMAP1(svst1_scatter_u64base_f64, aarch64_sve_st1_scatter_scalar_offset, 65543),
SVEMAP1(svst1_scatter_u64base_index_f64, aarch64_sve_st1_scatter_scalar_offset, 65543),
SVEMAP1(svst1_scatter_u64base_index_s64, aarch64_sve_st1_scatter_scalar_offset, 65540),
SVEMAP1(svst1_scatter_u64base_index_u64, aarch64_sve_st1_scatter_scalar_offset, 65540),
SVEMAP1(svst1_scatter_u64base_offset_f64, aarch64_sve_st1_scatter_scalar_offset, 16842759),
SVEMAP1(svst1_scatter_u64base_offset_s64, aarch64_sve_st1_scatter_scalar_offset, 16842756),
SVEMAP1(svst1_scatter_u64base_offset_u64, aarch64_sve_st1_scatter_scalar_offset, 16842756),
SVEMAP1(svst1_scatter_u64base_s64, aarch64_sve_st1_scatter_scalar_offset, 65540),
SVEMAP1(svst1_scatter_u64base_u64, aarch64_sve_st1_scatter_scalar_offset, 65540),
SVEMAP1(svst1_scatter_u64index_f64, aarch64_sve_st1_scatter_index, 65543),
SVEMAP1(svst1_scatter_u64index_s64, aarch64_sve_st1_scatter_index, 65540),
SVEMAP1(svst1_scatter_u64index_u64, aarch64_sve_st1_scatter_index, 65540),
SVEMAP1(svst1_scatter_u64offset_f64, aarch64_sve_st1_scatter, 16842759),
SVEMAP1(svst1_scatter_u64offset_s64, aarch64_sve_st1_scatter, 16842756),
SVEMAP1(svst1_scatter_u64offset_u64, aarch64_sve_st1_scatter, 16842756),
SVEMAP1(svst1_u16, aarch64_sve_st1, 16386),
SVEMAP1(svst1_u32, aarch64_sve_st1, 16387),
SVEMAP1(svst1_u64, aarch64_sve_st1, 16388),
SVEMAP1(svst1_u8, aarch64_sve_st1, 16385),
SVEMAP1(svst1_vnum_bf16, aarch64_sve_st1, 16396),
SVEMAP1(svst1_vnum_f16, aarch64_sve_st1, 16389),
SVEMAP1(svst1_vnum_f32, aarch64_sve_st1, 16390),
SVEMAP1(svst1_vnum_f64, aarch64_sve_st1, 16391),
SVEMAP1(svst1_vnum_s16, aarch64_sve_st1, 16386),
SVEMAP1(svst1_vnum_s32, aarch64_sve_st1, 16387),
SVEMAP1(svst1_vnum_s64, aarch64_sve_st1, 16388),
SVEMAP1(svst1_vnum_s8, aarch64_sve_st1, 16385),
SVEMAP1(svst1_vnum_u16, aarch64_sve_st1, 16386),
SVEMAP1(svst1_vnum_u32, aarch64_sve_st1, 16387),
SVEMAP1(svst1_vnum_u64, aarch64_sve_st1, 16388),
SVEMAP1(svst1_vnum_u8, aarch64_sve_st1, 16385),
SVEMAP1(svst1b_s16, aarch64_sve_st1, 16402),
SVEMAP1(svst1b_s32, aarch64_sve_st1, 16403),
SVEMAP1(svst1b_s64, aarch64_sve_st1, 16404),
SVEMAP1(svst1b_scatter_s32offset_s32, aarch64_sve_st1_scatter_sxtw, 16842771),
SVEMAP1(svst1b_scatter_s32offset_u32, aarch64_sve_st1_scatter_sxtw, 16842771),
SVEMAP1(svst1b_scatter_s64offset_s64, aarch64_sve_st1_scatter, 16842772),
SVEMAP1(svst1b_scatter_s64offset_u64, aarch64_sve_st1_scatter, 16842772),
SVEMAP1(svst1b_scatter_u32base_offset_s32, aarch64_sve_st1_scatter_scalar_offset, 16842771),
SVEMAP1(svst1b_scatter_u32base_offset_u32, aarch64_sve_st1_scatter_scalar_offset, 16842771),
SVEMAP1(svst1b_scatter_u32base_s32, aarch64_sve_st1_scatter_scalar_offset, 65555),
SVEMAP1(svst1b_scatter_u32base_u32, aarch64_sve_st1_scatter_scalar_offset, 65555),
SVEMAP1(svst1b_scatter_u32offset_s32, aarch64_sve_st1_scatter_uxtw, 16842771),
SVEMAP1(svst1b_scatter_u32offset_u32, aarch64_sve_st1_scatter_uxtw, 16842771),
SVEMAP1(svst1b_scatter_u64base_offset_s64, aarch64_sve_st1_scatter_scalar_offset, 16842772),
SVEMAP1(svst1b_scatter_u64base_offset_u64, aarch64_sve_st1_scatter_scalar_offset, 16842772),
SVEMAP1(svst1b_scatter_u64base_s64, aarch64_sve_st1_scatter_scalar_offset, 65556),
SVEMAP1(svst1b_scatter_u64base_u64, aarch64_sve_st1_scatter_scalar_offset, 65556),
SVEMAP1(svst1b_scatter_u64offset_s64, aarch64_sve_st1_scatter, 16842772),
SVEMAP1(svst1b_scatter_u64offset_u64, aarch64_sve_st1_scatter, 16842772),
SVEMAP1(svst1b_u16, aarch64_sve_st1, 16402),
SVEMAP1(svst1b_u32, aarch64_sve_st1, 16403),
SVEMAP1(svst1b_u64, aarch64_sve_st1, 16404),
SVEMAP1(svst1b_vnum_s16, aarch64_sve_st1, 16402),
SVEMAP1(svst1b_vnum_s32, aarch64_sve_st1, 16403),
SVEMAP1(svst1b_vnum_s64, aarch64_sve_st1, 16404),
SVEMAP1(svst1b_vnum_u16, aarch64_sve_st1, 16402),
SVEMAP1(svst1b_vnum_u32, aarch64_sve_st1, 16403),
SVEMAP1(svst1b_vnum_u64, aarch64_sve_st1, 16404),
SVEMAP1(svst1h_s32, aarch64_sve_st1, 16419),
SVEMAP1(svst1h_s64, aarch64_sve_st1, 16420),
SVEMAP1(svst1h_scatter_s32index_s32, aarch64_sve_st1_scatter_sxtw_index, 65571),
SVEMAP1(svst1h_scatter_s32index_u32, aarch64_sve_st1_scatter_sxtw_index, 65571),
SVEMAP1(svst1h_scatter_s32offset_s32, aarch64_sve_st1_scatter_sxtw, 16842787),
SVEMAP1(svst1h_scatter_s32offset_u32, aarch64_sve_st1_scatter_sxtw, 16842787),
SVEMAP1(svst1h_scatter_s64index_s64, aarch64_sve_st1_scatter_index, 65572),
SVEMAP1(svst1h_scatter_s64index_u64, aarch64_sve_st1_scatter_index, 65572),
SVEMAP1(svst1h_scatter_s64offset_s64, aarch64_sve_st1_scatter, 16842788),
SVEMAP1(svst1h_scatter_s64offset_u64, aarch64_sve_st1_scatter, 16842788),
SVEMAP1(svst1h_scatter_u32base_index_s32, aarch64_sve_st1_scatter_scalar_offset, 65571),
SVEMAP1(svst1h_scatter_u32base_index_u32, aarch64_sve_st1_scatter_scalar_offset, 65571),
SVEMAP1(svst1h_scatter_u32base_offset_s32, aarch64_sve_st1_scatter_scalar_offset, 16842787),
SVEMAP1(svst1h_scatter_u32base_offset_u32, aarch64_sve_st1_scatter_scalar_offset, 16842787),
SVEMAP1(svst1h_scatter_u32base_s32, aarch64_sve_st1_scatter_scalar_offset, 65571),
SVEMAP1(svst1h_scatter_u32base_u32, aarch64_sve_st1_scatter_scalar_offset, 65571),
SVEMAP1(svst1h_scatter_u32index_s32, aarch64_sve_st1_scatter_uxtw_index, 65571),
SVEMAP1(svst1h_scatter_u32index_u32, aarch64_sve_st1_scatter_uxtw_index, 65571),
SVEMAP1(svst1h_scatter_u32offset_s32, aarch64_sve_st1_scatter_uxtw, 16842787),
SVEMAP1(svst1h_scatter_u32offset_u32, aarch64_sve_st1_scatter_uxtw, 16842787),
SVEMAP1(svst1h_scatter_u64base_index_s64, aarch64_sve_st1_scatter_scalar_offset, 65572),
SVEMAP1(svst1h_scatter_u64base_index_u64, aarch64_sve_st1_scatter_scalar_offset, 65572),
SVEMAP1(svst1h_scatter_u64base_offset_s64, aarch64_sve_st1_scatter_scalar_offset, 16842788),
SVEMAP1(svst1h_scatter_u64base_offset_u64, aarch64_sve_st1_scatter_scalar_offset, 16842788),
SVEMAP1(svst1h_scatter_u64base_s64, aarch64_sve_st1_scatter_scalar_offset, 65572),
SVEMAP1(svst1h_scatter_u64base_u64, aarch64_sve_st1_scatter_scalar_offset, 65572),
SVEMAP1(svst1h_scatter_u64index_s64, aarch64_sve_st1_scatter_index, 65572),
SVEMAP1(svst1h_scatter_u64index_u64, aarch64_sve_st1_scatter_index, 65572),
SVEMAP1(svst1h_scatter_u64offset_s64, aarch64_sve_st1_scatter, 16842788),
SVEMAP1(svst1h_scatter_u64offset_u64, aarch64_sve_st1_scatter, 16842788),
SVEMAP1(svst1h_u32, aarch64_sve_st1, 16419),
SVEMAP1(svst1h_u64, aarch64_sve_st1, 16420),
SVEMAP1(svst1h_vnum_s32, aarch64_sve_st1, 16419),
SVEMAP1(svst1h_vnum_s64, aarch64_sve_st1, 16420),
SVEMAP1(svst1h_vnum_u32, aarch64_sve_st1, 16419),
SVEMAP1(svst1h_vnum_u64, aarch64_sve_st1, 16420),
SVEMAP1(svst1w_s64, aarch64_sve_st1, 16436),
SVEMAP1(svst1w_scatter_s64index_s64, aarch64_sve_st1_scatter_index, 65588),
SVEMAP1(svst1w_scatter_s64index_u64, aarch64_sve_st1_scatter_index, 65588),
SVEMAP1(svst1w_scatter_s64offset_s64, aarch64_sve_st1_scatter, 16842804),
SVEMAP1(svst1w_scatter_s64offset_u64, aarch64_sve_st1_scatter, 16842804),
SVEMAP1(svst1w_scatter_u64base_index_s64, aarch64_sve_st1_scatter_scalar_offset, 65588),
SVEMAP1(svst1w_scatter_u64base_index_u64, aarch64_sve_st1_scatter_scalar_offset, 65588),
SVEMAP1(svst1w_scatter_u64base_offset_s64, aarch64_sve_st1_scatter_scalar_offset, 16842804),
SVEMAP1(svst1w_scatter_u64base_offset_u64, aarch64_sve_st1_scatter_scalar_offset, 16842804),
SVEMAP1(svst1w_scatter_u64base_s64, aarch64_sve_st1_scatter_scalar_offset, 65588),
SVEMAP1(svst1w_scatter_u64base_u64, aarch64_sve_st1_scatter_scalar_offset, 65588),
SVEMAP1(svst1w_scatter_u64index_s64, aarch64_sve_st1_scatter_index, 65588),
SVEMAP1(svst1w_scatter_u64index_u64, aarch64_sve_st1_scatter_index, 65588),
SVEMAP1(svst1w_scatter_u64offset_s64, aarch64_sve_st1_scatter, 16842804),
SVEMAP1(svst1w_scatter_u64offset_u64, aarch64_sve_st1_scatter, 16842804),
SVEMAP1(svst1w_u64, aarch64_sve_st1, 16436),
SVEMAP1(svst1w_vnum_s64, aarch64_sve_st1, 16436),
SVEMAP1(svst1w_vnum_u64, aarch64_sve_st1, 16436),
SVEMAP1(svst2_bf16, aarch64_sve_st2, 262156),
SVEMAP1(svst2_f16, aarch64_sve_st2, 262149),
SVEMAP1(svst2_f32, aarch64_sve_st2, 262150),
SVEMAP1(svst2_f64, aarch64_sve_st2, 262151),
SVEMAP1(svst2_s16, aarch64_sve_st2, 262146),
SVEMAP1(svst2_s32, aarch64_sve_st2, 262147),
SVEMAP1(svst2_s64, aarch64_sve_st2, 262148),
SVEMAP1(svst2_s8, aarch64_sve_st2, 262145),
SVEMAP1(svst2_u16, aarch64_sve_st2, 262146),
SVEMAP1(svst2_u32, aarch64_sve_st2, 262147),
SVEMAP1(svst2_u64, aarch64_sve_st2, 262148),
SVEMAP1(svst2_u8, aarch64_sve_st2, 262145),
SVEMAP1(svst2_vnum_bf16, aarch64_sve_st2, 262156),
SVEMAP1(svst2_vnum_f16, aarch64_sve_st2, 262149),
SVEMAP1(svst2_vnum_f32, aarch64_sve_st2, 262150),
SVEMAP1(svst2_vnum_f64, aarch64_sve_st2, 262151),
SVEMAP1(svst2_vnum_s16, aarch64_sve_st2, 262146),
SVEMAP1(svst2_vnum_s32, aarch64_sve_st2, 262147),
SVEMAP1(svst2_vnum_s64, aarch64_sve_st2, 262148),
SVEMAP1(svst2_vnum_s8, aarch64_sve_st2, 262145),
SVEMAP1(svst2_vnum_u16, aarch64_sve_st2, 262146),
SVEMAP1(svst2_vnum_u32, aarch64_sve_st2, 262147),
SVEMAP1(svst2_vnum_u64, aarch64_sve_st2, 262148),
SVEMAP1(svst2_vnum_u8, aarch64_sve_st2, 262145),
SVEMAP1(svst3_bf16, aarch64_sve_st3, 262156),
SVEMAP1(svst3_f16, aarch64_sve_st3, 262149),
SVEMAP1(svst3_f32, aarch64_sve_st3, 262150),
SVEMAP1(svst3_f64, aarch64_sve_st3, 262151),
SVEMAP1(svst3_s16, aarch64_sve_st3, 262146),
SVEMAP1(svst3_s32, aarch64_sve_st3, 262147),
SVEMAP1(svst3_s64, aarch64_sve_st3, 262148),
SVEMAP1(svst3_s8, aarch64_sve_st3, 262145),
SVEMAP1(svst3_u16, aarch64_sve_st3, 262146),
SVEMAP1(svst3_u32, aarch64_sve_st3, 262147),
SVEMAP1(svst3_u64, aarch64_sve_st3, 262148),
SVEMAP1(svst3_u8, aarch64_sve_st3, 262145),
SVEMAP1(svst3_vnum_bf16, aarch64_sve_st3, 262156),
SVEMAP1(svst3_vnum_f16, aarch64_sve_st3, 262149),
SVEMAP1(svst3_vnum_f32, aarch64_sve_st3, 262150),
SVEMAP1(svst3_vnum_f64, aarch64_sve_st3, 262151),
SVEMAP1(svst3_vnum_s16, aarch64_sve_st3, 262146),
SVEMAP1(svst3_vnum_s32, aarch64_sve_st3, 262147),
SVEMAP1(svst3_vnum_s64, aarch64_sve_st3, 262148),
SVEMAP1(svst3_vnum_s8, aarch64_sve_st3, 262145),
SVEMAP1(svst3_vnum_u16, aarch64_sve_st3, 262146),
SVEMAP1(svst3_vnum_u32, aarch64_sve_st3, 262147),
SVEMAP1(svst3_vnum_u64, aarch64_sve_st3, 262148),
SVEMAP1(svst3_vnum_u8, aarch64_sve_st3, 262145),
SVEMAP1(svst4_bf16, aarch64_sve_st4, 262156),
SVEMAP1(svst4_f16, aarch64_sve_st4, 262149),
SVEMAP1(svst4_f32, aarch64_sve_st4, 262150),
SVEMAP1(svst4_f64, aarch64_sve_st4, 262151),
SVEMAP1(svst4_s16, aarch64_sve_st4, 262146),
SVEMAP1(svst4_s32, aarch64_sve_st4, 262147),
SVEMAP1(svst4_s64, aarch64_sve_st4, 262148),
SVEMAP1(svst4_s8, aarch64_sve_st4, 262145),
SVEMAP1(svst4_u16, aarch64_sve_st4, 262146),
SVEMAP1(svst4_u32, aarch64_sve_st4, 262147),
SVEMAP1(svst4_u64, aarch64_sve_st4, 262148),
SVEMAP1(svst4_u8, aarch64_sve_st4, 262145),
SVEMAP1(svst4_vnum_bf16, aarch64_sve_st4, 262156),
SVEMAP1(svst4_vnum_f16, aarch64_sve_st4, 262149),
SVEMAP1(svst4_vnum_f32, aarch64_sve_st4, 262150),
SVEMAP1(svst4_vnum_f64, aarch64_sve_st4, 262151),
SVEMAP1(svst4_vnum_s16, aarch64_sve_st4, 262146),
SVEMAP1(svst4_vnum_s32, aarch64_sve_st4, 262147),
SVEMAP1(svst4_vnum_s64, aarch64_sve_st4, 262148),
SVEMAP1(svst4_vnum_s8, aarch64_sve_st4, 262145),
SVEMAP1(svst4_vnum_u16, aarch64_sve_st4, 262146),
SVEMAP1(svst4_vnum_u32, aarch64_sve_st4, 262147),
SVEMAP1(svst4_vnum_u64, aarch64_sve_st4, 262148),
SVEMAP1(svst4_vnum_u8, aarch64_sve_st4, 262145),
SVEMAP1(svstnt1_bf16, aarch64_sve_stnt1, 16396),
SVEMAP1(svstnt1_f16, aarch64_sve_stnt1, 16389),
SVEMAP1(svstnt1_f32, aarch64_sve_stnt1, 16390),
SVEMAP1(svstnt1_f64, aarch64_sve_stnt1, 16391),
SVEMAP1(svstnt1_s16, aarch64_sve_stnt1, 16386),
SVEMAP1(svstnt1_s32, aarch64_sve_stnt1, 16387),
SVEMAP1(svstnt1_s64, aarch64_sve_stnt1, 16388),
SVEMAP1(svstnt1_s8, aarch64_sve_stnt1, 16385),
SVEMAP1(svstnt1_scatter_s64index_f64, aarch64_sve_stnt1_scatter_index, 65543),
SVEMAP1(svstnt1_scatter_s64index_s64, aarch64_sve_stnt1_scatter_index, 65540),
SVEMAP1(svstnt1_scatter_s64index_u64, aarch64_sve_stnt1_scatter_index, 65540),
SVEMAP1(svstnt1_scatter_s64offset_f64, aarch64_sve_stnt1_scatter, 16842759),
SVEMAP1(svstnt1_scatter_s64offset_s64, aarch64_sve_stnt1_scatter, 16842756),
SVEMAP1(svstnt1_scatter_s64offset_u64, aarch64_sve_stnt1_scatter, 16842756),
SVEMAP1(svstnt1_scatter_u32base_f32, aarch64_sve_stnt1_scatter_scalar_offset, 65542),
SVEMAP1(svstnt1_scatter_u32base_index_f32, aarch64_sve_stnt1_scatter_scalar_offset, 65542),
SVEMAP1(svstnt1_scatter_u32base_index_s32, aarch64_sve_stnt1_scatter_scalar_offset, 65539),
SVEMAP1(svstnt1_scatter_u32base_index_u32, aarch64_sve_stnt1_scatter_scalar_offset, 65539),
SVEMAP1(svstnt1_scatter_u32base_offset_f32, aarch64_sve_stnt1_scatter_scalar_offset, 16842758),
SVEMAP1(svstnt1_scatter_u32base_offset_s32, aarch64_sve_stnt1_scatter_scalar_offset, 16842755),
SVEMAP1(svstnt1_scatter_u32base_offset_u32, aarch64_sve_stnt1_scatter_scalar_offset, 16842755),
SVEMAP1(svstnt1_scatter_u32base_s32, aarch64_sve_stnt1_scatter_scalar_offset, 65539),
SVEMAP1(svstnt1_scatter_u32base_u32, aarch64_sve_stnt1_scatter_scalar_offset, 65539),
SVEMAP1(svstnt1_scatter_u32offset_f32, aarch64_sve_stnt1_scatter_uxtw, 16842758),
SVEMAP1(svstnt1_scatter_u32offset_s32, aarch64_sve_stnt1_scatter_uxtw, 16842755),
SVEMAP1(svstnt1_scatter_u32offset_u32, aarch64_sve_stnt1_scatter_uxtw, 16842755),
SVEMAP1(svstnt1_scatter_u64base_f64, aarch64_sve_stnt1_scatter_scalar_offset, 65543),
SVEMAP1(svstnt1_scatter_u64base_index_f64, aarch64_sve_stnt1_scatter_scalar_offset, 65543),
SVEMAP1(svstnt1_scatter_u64base_index_s64, aarch64_sve_stnt1_scatter_scalar_offset, 65540),
SVEMAP1(svstnt1_scatter_u64base_index_u64, aarch64_sve_stnt1_scatter_scalar_offset, 65540),
SVEMAP1(svstnt1_scatter_u64base_offset_f64, aarch64_sve_stnt1_scatter_scalar_offset, 16842759),
SVEMAP1(svstnt1_scatter_u64base_offset_s64, aarch64_sve_stnt1_scatter_scalar_offset, 16842756),
SVEMAP1(svstnt1_scatter_u64base_offset_u64, aarch64_sve_stnt1_scatter_scalar_offset, 16842756),
SVEMAP1(svstnt1_scatter_u64base_s64, aarch64_sve_stnt1_scatter_scalar_offset, 65540),
SVEMAP1(svstnt1_scatter_u64base_u64, aarch64_sve_stnt1_scatter_scalar_offset, 65540),
SVEMAP1(svstnt1_scatter_u64index_f64, aarch64_sve_stnt1_scatter_index, 65543),
SVEMAP1(svstnt1_scatter_u64index_s64, aarch64_sve_stnt1_scatter_index, 65540),
SVEMAP1(svstnt1_scatter_u64index_u64, aarch64_sve_stnt1_scatter_index, 65540),
SVEMAP1(svstnt1_scatter_u64offset_f64, aarch64_sve_stnt1_scatter, 16842759),
SVEMAP1(svstnt1_scatter_u64offset_s64, aarch64_sve_stnt1_scatter, 16842756),
SVEMAP1(svstnt1_scatter_u64offset_u64, aarch64_sve_stnt1_scatter, 16842756),
SVEMAP1(svstnt1_u16, aarch64_sve_stnt1, 16386),
SVEMAP1(svstnt1_u32, aarch64_sve_stnt1, 16387),
SVEMAP1(svstnt1_u64, aarch64_sve_stnt1, 16388),
SVEMAP1(svstnt1_u8, aarch64_sve_stnt1, 16385),
SVEMAP1(svstnt1_vnum_bf16, aarch64_sve_stnt1, 16396),
SVEMAP1(svstnt1_vnum_f16, aarch64_sve_stnt1, 16389),
SVEMAP1(svstnt1_vnum_f32, aarch64_sve_stnt1, 16390),
SVEMAP1(svstnt1_vnum_f64, aarch64_sve_stnt1, 16391),
SVEMAP1(svstnt1_vnum_s16, aarch64_sve_stnt1, 16386),
SVEMAP1(svstnt1_vnum_s32, aarch64_sve_stnt1, 16387),
SVEMAP1(svstnt1_vnum_s64, aarch64_sve_stnt1, 16388),
SVEMAP1(svstnt1_vnum_s8, aarch64_sve_stnt1, 16385),
SVEMAP1(svstnt1_vnum_u16, aarch64_sve_stnt1, 16386),
SVEMAP1(svstnt1_vnum_u32, aarch64_sve_stnt1, 16387),
SVEMAP1(svstnt1_vnum_u64, aarch64_sve_stnt1, 16388),
SVEMAP1(svstnt1_vnum_u8, aarch64_sve_stnt1, 16385),
SVEMAP1(svstnt1b_scatter_s64offset_s64, aarch64_sve_stnt1_scatter, 16842772),
SVEMAP1(svstnt1b_scatter_s64offset_u64, aarch64_sve_stnt1_scatter, 16842772),
SVEMAP1(svstnt1b_scatter_u32base_offset_s32, aarch64_sve_stnt1_scatter_scalar_offset, 16842771),
SVEMAP1(svstnt1b_scatter_u32base_offset_u32, aarch64_sve_stnt1_scatter_scalar_offset, 16842771),
SVEMAP1(svstnt1b_scatter_u32base_s32, aarch64_sve_stnt1_scatter_scalar_offset, 65555),
SVEMAP1(svstnt1b_scatter_u32base_u32, aarch64_sve_stnt1_scatter_scalar_offset, 65555),
SVEMAP1(svstnt1b_scatter_u32offset_s32, aarch64_sve_stnt1_scatter_uxtw, 16842771),
SVEMAP1(svstnt1b_scatter_u32offset_u32, aarch64_sve_stnt1_scatter_uxtw, 16842771),
SVEMAP1(svstnt1b_scatter_u64base_offset_s64, aarch64_sve_stnt1_scatter_scalar_offset, 16842772),
SVEMAP1(svstnt1b_scatter_u64base_offset_u64, aarch64_sve_stnt1_scatter_scalar_offset, 16842772),
SVEMAP1(svstnt1b_scatter_u64base_s64, aarch64_sve_stnt1_scatter_scalar_offset, 65556),
SVEMAP1(svstnt1b_scatter_u64base_u64, aarch64_sve_stnt1_scatter_scalar_offset, 65556),
SVEMAP1(svstnt1b_scatter_u64offset_s64, aarch64_sve_stnt1_scatter, 16842772),
SVEMAP1(svstnt1b_scatter_u64offset_u64, aarch64_sve_stnt1_scatter, 16842772),
SVEMAP1(svstnt1h_scatter_s64index_s64, aarch64_sve_stnt1_scatter_index, 65572),
SVEMAP1(svstnt1h_scatter_s64index_u64, aarch64_sve_stnt1_scatter_index, 65572),
SVEMAP1(svstnt1h_scatter_s64offset_s64, aarch64_sve_stnt1_scatter, 16842788),
SVEMAP1(svstnt1h_scatter_s64offset_u64, aarch64_sve_stnt1_scatter, 16842788),
SVEMAP1(svstnt1h_scatter_u32base_index_s32, aarch64_sve_stnt1_scatter_scalar_offset, 65571),
SVEMAP1(svstnt1h_scatter_u32base_index_u32, aarch64_sve_stnt1_scatter_scalar_offset, 65571),
SVEMAP1(svstnt1h_scatter_u32base_offset_s32, aarch64_sve_stnt1_scatter_scalar_offset, 16842787),
SVEMAP1(svstnt1h_scatter_u32base_offset_u32, aarch64_sve_stnt1_scatter_scalar_offset, 16842787),
SVEMAP1(svstnt1h_scatter_u32base_s32, aarch64_sve_stnt1_scatter_scalar_offset, 65571),
SVEMAP1(svstnt1h_scatter_u32base_u32, aarch64_sve_stnt1_scatter_scalar_offset, 65571),
SVEMAP1(svstnt1h_scatter_u32offset_s32, aarch64_sve_stnt1_scatter_uxtw, 16842787),
SVEMAP1(svstnt1h_scatter_u32offset_u32, aarch64_sve_stnt1_scatter_uxtw, 16842787),
SVEMAP1(svstnt1h_scatter_u64base_index_s64, aarch64_sve_stnt1_scatter_scalar_offset, 65572),
SVEMAP1(svstnt1h_scatter_u64base_index_u64, aarch64_sve_stnt1_scatter_scalar_offset, 65572),
SVEMAP1(svstnt1h_scatter_u64base_offset_s64, aarch64_sve_stnt1_scatter_scalar_offset, 16842788),
SVEMAP1(svstnt1h_scatter_u64base_offset_u64, aarch64_sve_stnt1_scatter_scalar_offset, 16842788),
SVEMAP1(svstnt1h_scatter_u64base_s64, aarch64_sve_stnt1_scatter_scalar_offset, 65572),
SVEMAP1(svstnt1h_scatter_u64base_u64, aarch64_sve_stnt1_scatter_scalar_offset, 65572),
SVEMAP1(svstnt1h_scatter_u64index_s64, aarch64_sve_stnt1_scatter_index, 65572),
SVEMAP1(svstnt1h_scatter_u64index_u64, aarch64_sve_stnt1_scatter_index, 65572),
SVEMAP1(svstnt1h_scatter_u64offset_s64, aarch64_sve_stnt1_scatter, 16842788),
SVEMAP1(svstnt1h_scatter_u64offset_u64, aarch64_sve_stnt1_scatter, 16842788),
SVEMAP1(svstnt1w_scatter_s64index_s64, aarch64_sve_stnt1_scatter_index, 65588),
SVEMAP1(svstnt1w_scatter_s64index_u64, aarch64_sve_stnt1_scatter_index, 65588),
SVEMAP1(svstnt1w_scatter_s64offset_s64, aarch64_sve_stnt1_scatter, 16842804),
SVEMAP1(svstnt1w_scatter_s64offset_u64, aarch64_sve_stnt1_scatter, 16842804),
SVEMAP1(svstnt1w_scatter_u64base_index_s64, aarch64_sve_stnt1_scatter_scalar_offset, 65588),
SVEMAP1(svstnt1w_scatter_u64base_index_u64, aarch64_sve_stnt1_scatter_scalar_offset, 65588),
SVEMAP1(svstnt1w_scatter_u64base_offset_s64, aarch64_sve_stnt1_scatter_scalar_offset, 16842804),
SVEMAP1(svstnt1w_scatter_u64base_offset_u64, aarch64_sve_stnt1_scatter_scalar_offset, 16842804),
SVEMAP1(svstnt1w_scatter_u64base_s64, aarch64_sve_stnt1_scatter_scalar_offset, 65588),
SVEMAP1(svstnt1w_scatter_u64base_u64, aarch64_sve_stnt1_scatter_scalar_offset, 65588),
SVEMAP1(svstnt1w_scatter_u64index_s64, aarch64_sve_stnt1_scatter_index, 65588),
SVEMAP1(svstnt1w_scatter_u64index_u64, aarch64_sve_stnt1_scatter_index, 65588),
SVEMAP1(svstnt1w_scatter_u64offset_s64, aarch64_sve_stnt1_scatter, 16842804),
SVEMAP1(svstnt1w_scatter_u64offset_u64, aarch64_sve_stnt1_scatter, 16842804),
SVEMAP1(svsub_f16_m, aarch64_sve_fsub, 261),
SVEMAP1(svsub_f16_x, aarch64_sve_fsub, 133),
SVEMAP1(svsub_f16_z, aarch64_sve_fsub, 389),
SVEMAP1(svsub_f32_m, aarch64_sve_fsub, 262),
SVEMAP1(svsub_f32_x, aarch64_sve_fsub, 134),
SVEMAP1(svsub_f32_z, aarch64_sve_fsub, 390),
SVEMAP1(svsub_f64_m, aarch64_sve_fsub, 263),
SVEMAP1(svsub_f64_x, aarch64_sve_fsub, 135),
SVEMAP1(svsub_f64_z, aarch64_sve_fsub, 391),
SVEMAP1(svsub_n_f16_m, aarch64_sve_fsub, 3333),
SVEMAP1(svsub_n_f16_x, aarch64_sve_fsub, 3205),
SVEMAP1(svsub_n_f16_z, aarch64_sve_fsub, 3461),
SVEMAP1(svsub_n_f32_m, aarch64_sve_fsub, 3334),
SVEMAP1(svsub_n_f32_x, aarch64_sve_fsub, 3206),
SVEMAP1(svsub_n_f32_z, aarch64_sve_fsub, 3462),
SVEMAP1(svsub_n_f64_m, aarch64_sve_fsub, 3335),
SVEMAP1(svsub_n_f64_x, aarch64_sve_fsub, 3207),
SVEMAP1(svsub_n_f64_z, aarch64_sve_fsub, 3463),
SVEMAP1(svsub_n_s16_m, aarch64_sve_sub, 3330),
SVEMAP1(svsub_n_s16_x, aarch64_sve_sub, 3202),
SVEMAP1(svsub_n_s16_z, aarch64_sve_sub, 3458),
SVEMAP1(svsub_n_s32_m, aarch64_sve_sub, 3331),
SVEMAP1(svsub_n_s32_x, aarch64_sve_sub, 3203),
SVEMAP1(svsub_n_s32_z, aarch64_sve_sub, 3459),
SVEMAP1(svsub_n_s64_m, aarch64_sve_sub, 3332),
SVEMAP1(svsub_n_s64_x, aarch64_sve_sub, 3204),
SVEMAP1(svsub_n_s64_z, aarch64_sve_sub, 3460),
SVEMAP1(svsub_n_s8_m, aarch64_sve_sub, 3329),
SVEMAP1(svsub_n_s8_x, aarch64_sve_sub, 3201),
SVEMAP1(svsub_n_s8_z, aarch64_sve_sub, 3457),
SVEMAP1(svsub_n_u16_m, aarch64_sve_sub, 3330),
SVEMAP1(svsub_n_u16_x, aarch64_sve_sub, 3202),
SVEMAP1(svsub_n_u16_z, aarch64_sve_sub, 3458),
SVEMAP1(svsub_n_u32_m, aarch64_sve_sub, 3331),
SVEMAP1(svsub_n_u32_x, aarch64_sve_sub, 3203),
SVEMAP1(svsub_n_u32_z, aarch64_sve_sub, 3459),
SVEMAP1(svsub_n_u64_m, aarch64_sve_sub, 3332),
SVEMAP1(svsub_n_u64_x, aarch64_sve_sub, 3204),
SVEMAP1(svsub_n_u64_z, aarch64_sve_sub, 3460),
SVEMAP1(svsub_n_u8_m, aarch64_sve_sub, 3329),
SVEMAP1(svsub_n_u8_x, aarch64_sve_sub, 3201),
SVEMAP1(svsub_n_u8_z, aarch64_sve_sub, 3457),
SVEMAP1(svsub_s16_m, aarch64_sve_sub, 258),
SVEMAP1(svsub_s16_x, aarch64_sve_sub, 130),
SVEMAP1(svsub_s16_z, aarch64_sve_sub, 386),
SVEMAP1(svsub_s32_m, aarch64_sve_sub, 259),
SVEMAP1(svsub_s32_x, aarch64_sve_sub, 131),
SVEMAP1(svsub_s32_z, aarch64_sve_sub, 387),
SVEMAP1(svsub_s64_m, aarch64_sve_sub, 260),
SVEMAP1(svsub_s64_x, aarch64_sve_sub, 132),
SVEMAP1(svsub_s64_z, aarch64_sve_sub, 388),
SVEMAP1(svsub_s8_m, aarch64_sve_sub, 257),
SVEMAP1(svsub_s8_x, aarch64_sve_sub, 129),
SVEMAP1(svsub_s8_z, aarch64_sve_sub, 385),
SVEMAP1(svsub_u16_m, aarch64_sve_sub, 258),
SVEMAP1(svsub_u16_x, aarch64_sve_sub, 130),
SVEMAP1(svsub_u16_z, aarch64_sve_sub, 386),
SVEMAP1(svsub_u32_m, aarch64_sve_sub, 259),
SVEMAP1(svsub_u32_x, aarch64_sve_sub, 131),
SVEMAP1(svsub_u32_z, aarch64_sve_sub, 387),
SVEMAP1(svsub_u64_m, aarch64_sve_sub, 260),
SVEMAP1(svsub_u64_x, aarch64_sve_sub, 132),
SVEMAP1(svsub_u64_z, aarch64_sve_sub, 388),
SVEMAP1(svsub_u8_m, aarch64_sve_sub, 257),
SVEMAP1(svsub_u8_x, aarch64_sve_sub, 129),
SVEMAP1(svsub_u8_z, aarch64_sve_sub, 385),
SVEMAP1(svsubhnb_n_s16, aarch64_sve_subhnb, 2050),
SVEMAP1(svsubhnb_n_s32, aarch64_sve_subhnb, 2051),
SVEMAP1(svsubhnb_n_s64, aarch64_sve_subhnb, 2052),
SVEMAP1(svsubhnb_n_u16, aarch64_sve_subhnb, 2050),
SVEMAP1(svsubhnb_n_u32, aarch64_sve_subhnb, 2051),
SVEMAP1(svsubhnb_n_u64, aarch64_sve_subhnb, 2052),
SVEMAP1(svsubhnb_s16, aarch64_sve_subhnb, 2),
SVEMAP1(svsubhnb_s32, aarch64_sve_subhnb, 3),
SVEMAP1(svsubhnb_s64, aarch64_sve_subhnb, 4),
SVEMAP1(svsubhnb_u16, aarch64_sve_subhnb, 2),
SVEMAP1(svsubhnb_u32, aarch64_sve_subhnb, 3),
SVEMAP1(svsubhnb_u64, aarch64_sve_subhnb, 4),
SVEMAP1(svsubhnt_n_s16, aarch64_sve_subhnt, 3074),
SVEMAP1(svsubhnt_n_s32, aarch64_sve_subhnt, 3075),
SVEMAP1(svsubhnt_n_s64, aarch64_sve_subhnt, 3076),
SVEMAP1(svsubhnt_n_u16, aarch64_sve_subhnt, 3074),
SVEMAP1(svsubhnt_n_u32, aarch64_sve_subhnt, 3075),
SVEMAP1(svsubhnt_n_u64, aarch64_sve_subhnt, 3076),
SVEMAP1(svsubhnt_s16, aarch64_sve_subhnt, 2),
SVEMAP1(svsubhnt_s32, aarch64_sve_subhnt, 3),
SVEMAP1(svsubhnt_s64, aarch64_sve_subhnt, 4),
SVEMAP1(svsubhnt_u16, aarch64_sve_subhnt, 2),
SVEMAP1(svsubhnt_u32, aarch64_sve_subhnt, 3),
SVEMAP1(svsubhnt_u64, aarch64_sve_subhnt, 4),
SVEMAP1(svsublb_n_s16, aarch64_sve_ssublb, 2050),
SVEMAP1(svsublb_n_s32, aarch64_sve_ssublb, 2051),
SVEMAP1(svsublb_n_s64, aarch64_sve_ssublb, 2052),
SVEMAP1(svsublb_n_u16, aarch64_sve_usublb, 2050),
SVEMAP1(svsublb_n_u32, aarch64_sve_usublb, 2051),
SVEMAP1(svsublb_n_u64, aarch64_sve_usublb, 2052),
SVEMAP1(svsublb_s16, aarch64_sve_ssublb, 2),
SVEMAP1(svsublb_s32, aarch64_sve_ssublb, 3),
SVEMAP1(svsublb_s64, aarch64_sve_ssublb, 4),
SVEMAP1(svsublb_u16, aarch64_sve_usublb, 2),
SVEMAP1(svsublb_u32, aarch64_sve_usublb, 3),
SVEMAP1(svsublb_u64, aarch64_sve_usublb, 4),
SVEMAP1(svsublbt_n_s16, aarch64_sve_ssublbt, 2050),
SVEMAP1(svsublbt_n_s32, aarch64_sve_ssublbt, 2051),
SVEMAP1(svsublbt_n_s64, aarch64_sve_ssublbt, 2052),
SVEMAP1(svsublbt_s16, aarch64_sve_ssublbt, 2),
SVEMAP1(svsublbt_s32, aarch64_sve_ssublbt, 3),
SVEMAP1(svsublbt_s64, aarch64_sve_ssublbt, 4),
SVEMAP1(svsublt_n_s16, aarch64_sve_ssublt, 2050),
SVEMAP1(svsublt_n_s32, aarch64_sve_ssublt, 2051),
SVEMAP1(svsublt_n_s64, aarch64_sve_ssublt, 2052),
SVEMAP1(svsublt_n_u16, aarch64_sve_usublt, 2050),
SVEMAP1(svsublt_n_u32, aarch64_sve_usublt, 2051),
SVEMAP1(svsublt_n_u64, aarch64_sve_usublt, 2052),
SVEMAP1(svsublt_s16, aarch64_sve_ssublt, 2),
SVEMAP1(svsublt_s32, aarch64_sve_ssublt, 3),
SVEMAP1(svsublt_s64, aarch64_sve_ssublt, 4),
SVEMAP1(svsublt_u16, aarch64_sve_usublt, 2),
SVEMAP1(svsublt_u32, aarch64_sve_usublt, 3),
SVEMAP1(svsublt_u64, aarch64_sve_usublt, 4),
SVEMAP1(svsubltb_n_s16, aarch64_sve_ssubltb, 2050),
SVEMAP1(svsubltb_n_s32, aarch64_sve_ssubltb, 2051),
SVEMAP1(svsubltb_n_s64, aarch64_sve_ssubltb, 2052),
SVEMAP1(svsubltb_s16, aarch64_sve_ssubltb, 2),
SVEMAP1(svsubltb_s32, aarch64_sve_ssubltb, 3),
SVEMAP1(svsubltb_s64, aarch64_sve_ssubltb, 4),
SVEMAP1(svsubr_f16_m, aarch64_sve_fsubr, 261),
SVEMAP1(svsubr_f16_x, aarch64_sve_fsubr, 133),
SVEMAP1(svsubr_f16_z, aarch64_sve_fsubr, 389),
SVEMAP1(svsubr_f32_m, aarch64_sve_fsubr, 262),
SVEMAP1(svsubr_f32_x, aarch64_sve_fsubr, 134),
SVEMAP1(svsubr_f32_z, aarch64_sve_fsubr, 390),
SVEMAP1(svsubr_f64_m, aarch64_sve_fsubr, 263),
SVEMAP1(svsubr_f64_x, aarch64_sve_fsubr, 135),
SVEMAP1(svsubr_f64_z, aarch64_sve_fsubr, 391),
SVEMAP1(svsubr_n_f16_m, aarch64_sve_fsubr, 3333),
SVEMAP1(svsubr_n_f16_x, aarch64_sve_fsubr, 3205),
SVEMAP1(svsubr_n_f16_z, aarch64_sve_fsubr, 3461),
SVEMAP1(svsubr_n_f32_m, aarch64_sve_fsubr, 3334),
SVEMAP1(svsubr_n_f32_x, aarch64_sve_fsubr, 3206),
SVEMAP1(svsubr_n_f32_z, aarch64_sve_fsubr, 3462),
SVEMAP1(svsubr_n_f64_m, aarch64_sve_fsubr, 3335),
SVEMAP1(svsubr_n_f64_x, aarch64_sve_fsubr, 3207),
SVEMAP1(svsubr_n_f64_z, aarch64_sve_fsubr, 3463),
SVEMAP1(svsubr_n_s16_m, aarch64_sve_subr, 3330),
SVEMAP1(svsubr_n_s16_x, aarch64_sve_subr, 3202),
SVEMAP1(svsubr_n_s16_z, aarch64_sve_subr, 3458),
SVEMAP1(svsubr_n_s32_m, aarch64_sve_subr, 3331),
SVEMAP1(svsubr_n_s32_x, aarch64_sve_subr, 3203),
SVEMAP1(svsubr_n_s32_z, aarch64_sve_subr, 3459),
SVEMAP1(svsubr_n_s64_m, aarch64_sve_subr, 3332),
SVEMAP1(svsubr_n_s64_x, aarch64_sve_subr, 3204),
SVEMAP1(svsubr_n_s64_z, aarch64_sve_subr, 3460),
SVEMAP1(svsubr_n_s8_m, aarch64_sve_subr, 3329),
SVEMAP1(svsubr_n_s8_x, aarch64_sve_subr, 3201),
SVEMAP1(svsubr_n_s8_z, aarch64_sve_subr, 3457),
SVEMAP1(svsubr_n_u16_m, aarch64_sve_subr, 3330),
SVEMAP1(svsubr_n_u16_x, aarch64_sve_subr, 3202),
SVEMAP1(svsubr_n_u16_z, aarch64_sve_subr, 3458),
SVEMAP1(svsubr_n_u32_m, aarch64_sve_subr, 3331),
SVEMAP1(svsubr_n_u32_x, aarch64_sve_subr, 3203),
SVEMAP1(svsubr_n_u32_z, aarch64_sve_subr, 3459),
SVEMAP1(svsubr_n_u64_m, aarch64_sve_subr, 3332),
SVEMAP1(svsubr_n_u64_x, aarch64_sve_subr, 3204),
SVEMAP1(svsubr_n_u64_z, aarch64_sve_subr, 3460),
SVEMAP1(svsubr_n_u8_m, aarch64_sve_subr, 3329),
SVEMAP1(svsubr_n_u8_x, aarch64_sve_subr, 3201),
SVEMAP1(svsubr_n_u8_z, aarch64_sve_subr, 3457),
SVEMAP1(svsubr_s16_m, aarch64_sve_subr, 258),
SVEMAP1(svsubr_s16_x, aarch64_sve_subr, 130),
SVEMAP1(svsubr_s16_z, aarch64_sve_subr, 386),
SVEMAP1(svsubr_s32_m, aarch64_sve_subr, 259),
SVEMAP1(svsubr_s32_x, aarch64_sve_subr, 131),
SVEMAP1(svsubr_s32_z, aarch64_sve_subr, 387),
SVEMAP1(svsubr_s64_m, aarch64_sve_subr, 260),
SVEMAP1(svsubr_s64_x, aarch64_sve_subr, 132),
SVEMAP1(svsubr_s64_z, aarch64_sve_subr, 388),
SVEMAP1(svsubr_s8_m, aarch64_sve_subr, 257),
SVEMAP1(svsubr_s8_x, aarch64_sve_subr, 129),
SVEMAP1(svsubr_s8_z, aarch64_sve_subr, 385),
SVEMAP1(svsubr_u16_m, aarch64_sve_subr, 258),
SVEMAP1(svsubr_u16_x, aarch64_sve_subr, 130),
SVEMAP1(svsubr_u16_z, aarch64_sve_subr, 386),
SVEMAP1(svsubr_u32_m, aarch64_sve_subr, 259),
SVEMAP1(svsubr_u32_x, aarch64_sve_subr, 131),
SVEMAP1(svsubr_u32_z, aarch64_sve_subr, 387),
SVEMAP1(svsubr_u64_m, aarch64_sve_subr, 260),
SVEMAP1(svsubr_u64_x, aarch64_sve_subr, 132),
SVEMAP1(svsubr_u64_z, aarch64_sve_subr, 388),
SVEMAP1(svsubr_u8_m, aarch64_sve_subr, 257),
SVEMAP1(svsubr_u8_x, aarch64_sve_subr, 129),
SVEMAP1(svsubr_u8_z, aarch64_sve_subr, 385),
SVEMAP1(svsubwb_n_s16, aarch64_sve_ssubwb, 2050),
SVEMAP1(svsubwb_n_s32, aarch64_sve_ssubwb, 2051),
SVEMAP1(svsubwb_n_s64, aarch64_sve_ssubwb, 2052),
SVEMAP1(svsubwb_n_u16, aarch64_sve_usubwb, 2050),
SVEMAP1(svsubwb_n_u32, aarch64_sve_usubwb, 2051),
SVEMAP1(svsubwb_n_u64, aarch64_sve_usubwb, 2052),
SVEMAP1(svsubwb_s16, aarch64_sve_ssubwb, 2),
SVEMAP1(svsubwb_s32, aarch64_sve_ssubwb, 3),
SVEMAP1(svsubwb_s64, aarch64_sve_ssubwb, 4),
SVEMAP1(svsubwb_u16, aarch64_sve_usubwb, 2),
SVEMAP1(svsubwb_u32, aarch64_sve_usubwb, 3),
SVEMAP1(svsubwb_u64, aarch64_sve_usubwb, 4),
SVEMAP1(svsubwt_n_s16, aarch64_sve_ssubwt, 2050),
SVEMAP1(svsubwt_n_s32, aarch64_sve_ssubwt, 2051),
SVEMAP1(svsubwt_n_s64, aarch64_sve_ssubwt, 2052),
SVEMAP1(svsubwt_n_u16, aarch64_sve_usubwt, 2050),
SVEMAP1(svsubwt_n_u32, aarch64_sve_usubwt, 2051),
SVEMAP1(svsubwt_n_u64, aarch64_sve_usubwt, 2052),
SVEMAP1(svsubwt_s16, aarch64_sve_ssubwt, 2),
SVEMAP1(svsubwt_s32, aarch64_sve_ssubwt, 3),
SVEMAP1(svsubwt_s64, aarch64_sve_ssubwt, 4),
SVEMAP1(svsubwt_u16, aarch64_sve_usubwt, 2),
SVEMAP1(svsubwt_u32, aarch64_sve_usubwt, 3),
SVEMAP1(svsubwt_u64, aarch64_sve_usubwt, 4),
SVEMAP1(svsudot_lane_s32, aarch64_sve_sudot_lane, 3),
SVEMAP1(svsudot_n_s32, aarch64_sve_usdot, 1073744899),
SVEMAP1(svsudot_s32, aarch64_sve_usdot, 1073741827),
SVEMAP2(svtbl2_bf16, 12),
SVEMAP2(svtbl2_f16, 5),
SVEMAP2(svtbl2_f32, 6),
SVEMAP2(svtbl2_f64, 7),
SVEMAP2(svtbl2_s16, 2),
SVEMAP2(svtbl2_s32, 3),
SVEMAP2(svtbl2_s64, 4),
SVEMAP2(svtbl2_s8, 1),
SVEMAP2(svtbl2_u16, 2),
SVEMAP2(svtbl2_u32, 3),
SVEMAP2(svtbl2_u64, 4),
SVEMAP2(svtbl2_u8, 1),
SVEMAP1(svtbl_bf16, aarch64_sve_tbl, 12),
SVEMAP1(svtbl_f16, aarch64_sve_tbl, 5),
SVEMAP1(svtbl_f32, aarch64_sve_tbl, 6),
SVEMAP1(svtbl_f64, aarch64_sve_tbl, 7),
SVEMAP1(svtbl_s16, aarch64_sve_tbl, 2),
SVEMAP1(svtbl_s32, aarch64_sve_tbl, 3),
SVEMAP1(svtbl_s64, aarch64_sve_tbl, 4),
SVEMAP1(svtbl_s8, aarch64_sve_tbl, 1),
SVEMAP1(svtbl_u16, aarch64_sve_tbl, 2),
SVEMAP1(svtbl_u32, aarch64_sve_tbl, 3),
SVEMAP1(svtbl_u64, aarch64_sve_tbl, 4),
SVEMAP1(svtbl_u8, aarch64_sve_tbl, 1),
SVEMAP1(svtbx_bf16, aarch64_sve_tbx, 12),
SVEMAP1(svtbx_f16, aarch64_sve_tbx, 5),
SVEMAP1(svtbx_f32, aarch64_sve_tbx, 6),
SVEMAP1(svtbx_f64, aarch64_sve_tbx, 7),
SVEMAP1(svtbx_s16, aarch64_sve_tbx, 2),
SVEMAP1(svtbx_s32, aarch64_sve_tbx, 3),
SVEMAP1(svtbx_s64, aarch64_sve_tbx, 4),
SVEMAP1(svtbx_s8, aarch64_sve_tbx, 1),
SVEMAP1(svtbx_u16, aarch64_sve_tbx, 2),
SVEMAP1(svtbx_u32, aarch64_sve_tbx, 3),
SVEMAP1(svtbx_u64, aarch64_sve_tbx, 4),
SVEMAP1(svtbx_u8, aarch64_sve_tbx, 1),
SVEMAP1(svtmad_f16, aarch64_sve_ftmad_x, 5),
SVEMAP1(svtmad_f32, aarch64_sve_ftmad_x, 6),
SVEMAP1(svtmad_f64, aarch64_sve_ftmad_x, 7),
SVEMAP1(svtrn1_b16, aarch64_sve_trn1, 9),
SVEMAP1(svtrn1_b32, aarch64_sve_trn1, 10),
SVEMAP1(svtrn1_b64, aarch64_sve_trn1, 11),
SVEMAP1(svtrn1_b8, aarch64_sve_trn1, 8),
SVEMAP1(svtrn1_bf16, aarch64_sve_trn1, 12),
SVEMAP1(svtrn1_f16, aarch64_sve_trn1, 5),
SVEMAP1(svtrn1_f32, aarch64_sve_trn1, 6),
SVEMAP1(svtrn1_f64, aarch64_sve_trn1, 7),
SVEMAP1(svtrn1_s16, aarch64_sve_trn1, 2),
SVEMAP1(svtrn1_s32, aarch64_sve_trn1, 3),
SVEMAP1(svtrn1_s64, aarch64_sve_trn1, 4),
SVEMAP1(svtrn1_s8, aarch64_sve_trn1, 1),
SVEMAP1(svtrn1_u16, aarch64_sve_trn1, 2),
SVEMAP1(svtrn1_u32, aarch64_sve_trn1, 3),
SVEMAP1(svtrn1_u64, aarch64_sve_trn1, 4),
SVEMAP1(svtrn1_u8, aarch64_sve_trn1, 1),
SVEMAP1(svtrn1q_bf16, aarch64_sve_trn1q, 12),
SVEMAP1(svtrn1q_f16, aarch64_sve_trn1q, 5),
SVEMAP1(svtrn1q_f32, aarch64_sve_trn1q, 6),
SVEMAP1(svtrn1q_f64, aarch64_sve_trn1q, 7),
SVEMAP1(svtrn1q_s16, aarch64_sve_trn1q, 2),
SVEMAP1(svtrn1q_s32, aarch64_sve_trn1q, 3),
SVEMAP1(svtrn1q_s64, aarch64_sve_trn1q, 4),
SVEMAP1(svtrn1q_s8, aarch64_sve_trn1q, 1),
SVEMAP1(svtrn1q_u16, aarch64_sve_trn1q, 2),
SVEMAP1(svtrn1q_u32, aarch64_sve_trn1q, 3),
SVEMAP1(svtrn1q_u64, aarch64_sve_trn1q, 4),
SVEMAP1(svtrn1q_u8, aarch64_sve_trn1q, 1),
SVEMAP1(svtrn2_b16, aarch64_sve_trn2, 9),
SVEMAP1(svtrn2_b32, aarch64_sve_trn2, 10),
SVEMAP1(svtrn2_b64, aarch64_sve_trn2, 11),
SVEMAP1(svtrn2_b8, aarch64_sve_trn2, 8),
SVEMAP1(svtrn2_bf16, aarch64_sve_trn2, 12),
SVEMAP1(svtrn2_f16, aarch64_sve_trn2, 5),
SVEMAP1(svtrn2_f32, aarch64_sve_trn2, 6),
SVEMAP1(svtrn2_f64, aarch64_sve_trn2, 7),
SVEMAP1(svtrn2_s16, aarch64_sve_trn2, 2),
SVEMAP1(svtrn2_s32, aarch64_sve_trn2, 3),
SVEMAP1(svtrn2_s64, aarch64_sve_trn2, 4),
SVEMAP1(svtrn2_s8, aarch64_sve_trn2, 1),
SVEMAP1(svtrn2_u16, aarch64_sve_trn2, 2),
SVEMAP1(svtrn2_u32, aarch64_sve_trn2, 3),
SVEMAP1(svtrn2_u64, aarch64_sve_trn2, 4),
SVEMAP1(svtrn2_u8, aarch64_sve_trn2, 1),
SVEMAP1(svtrn2q_bf16, aarch64_sve_trn2q, 12),
SVEMAP1(svtrn2q_f16, aarch64_sve_trn2q, 5),
SVEMAP1(svtrn2q_f32, aarch64_sve_trn2q, 6),
SVEMAP1(svtrn2q_f64, aarch64_sve_trn2q, 7),
SVEMAP1(svtrn2q_s16, aarch64_sve_trn2q, 2),
SVEMAP1(svtrn2q_s32, aarch64_sve_trn2q, 3),
SVEMAP1(svtrn2q_s64, aarch64_sve_trn2q, 4),
SVEMAP1(svtrn2q_s8, aarch64_sve_trn2q, 1),
SVEMAP1(svtrn2q_u16, aarch64_sve_trn2q, 2),
SVEMAP1(svtrn2q_u32, aarch64_sve_trn2q, 3),
SVEMAP1(svtrn2q_u64, aarch64_sve_trn2q, 4),
SVEMAP1(svtrn2q_u8, aarch64_sve_trn2q, 1),
SVEMAP1(svtsmul_f16, aarch64_sve_ftsmul_x, 5),
SVEMAP1(svtsmul_f32, aarch64_sve_ftsmul_x, 6),
SVEMAP1(svtsmul_f64, aarch64_sve_ftsmul_x, 7),
SVEMAP1(svtssel_f16, aarch64_sve_ftssel_x, 5),
SVEMAP1(svtssel_f32, aarch64_sve_ftssel_x, 6),
SVEMAP1(svtssel_f64, aarch64_sve_ftssel_x, 7),
SVEMAP2(svundef2_bf16, 2147483660),
SVEMAP2(svundef2_f16, 2147483653),
SVEMAP2(svundef2_f32, 2147483654),
SVEMAP2(svundef2_f64, 2147483655),
SVEMAP2(svundef2_s16, 2147483650),
SVEMAP2(svundef2_s32, 2147483651),
SVEMAP2(svundef2_s64, 2147483652),
SVEMAP2(svundef2_s8, 2147483649),
SVEMAP2(svundef2_u16, 2147483650),
SVEMAP2(svundef2_u32, 2147483651),
SVEMAP2(svundef2_u64, 2147483652),
SVEMAP2(svundef2_u8, 2147483649),
SVEMAP2(svundef3_bf16, 2147483660),
SVEMAP2(svundef3_f16, 2147483653),
SVEMAP2(svundef3_f32, 2147483654),
SVEMAP2(svundef3_f64, 2147483655),
SVEMAP2(svundef3_s16, 2147483650),
SVEMAP2(svundef3_s32, 2147483651),
SVEMAP2(svundef3_s64, 2147483652),
SVEMAP2(svundef3_s8, 2147483649),
SVEMAP2(svundef3_u16, 2147483650),
SVEMAP2(svundef3_u32, 2147483651),
SVEMAP2(svundef3_u64, 2147483652),
SVEMAP2(svundef3_u8, 2147483649),
SVEMAP2(svundef4_bf16, 2147483660),
SVEMAP2(svundef4_f16, 2147483653),
SVEMAP2(svundef4_f32, 2147483654),
SVEMAP2(svundef4_f64, 2147483655),
SVEMAP2(svundef4_s16, 2147483650),
SVEMAP2(svundef4_s32, 2147483651),
SVEMAP2(svundef4_s64, 2147483652),
SVEMAP2(svundef4_s8, 2147483649),
SVEMAP2(svundef4_u16, 2147483650),
SVEMAP2(svundef4_u32, 2147483651),
SVEMAP2(svundef4_u64, 2147483652),
SVEMAP2(svundef4_u8, 2147483649),
SVEMAP2(svundef_bf16, 2147483660),
SVEMAP2(svundef_f16, 2147483653),
SVEMAP2(svundef_f32, 2147483654),
SVEMAP2(svundef_f64, 2147483655),
SVEMAP2(svundef_s16, 2147483650),
SVEMAP2(svundef_s32, 2147483651),
SVEMAP2(svundef_s64, 2147483652),
SVEMAP2(svundef_s8, 2147483649),
SVEMAP2(svundef_u16, 2147483650),
SVEMAP2(svundef_u32, 2147483651),
SVEMAP2(svundef_u64, 2147483652),
SVEMAP2(svundef_u8, 2147483649),
SVEMAP1(svunpkhi_b, aarch64_sve_punpkhi, 8),
SVEMAP1(svunpkhi_s16, aarch64_sve_sunpkhi, 2),
SVEMAP1(svunpkhi_s32, aarch64_sve_sunpkhi, 3),
SVEMAP1(svunpkhi_s64, aarch64_sve_sunpkhi, 4),
SVEMAP1(svunpkhi_u16, aarch64_sve_uunpkhi, 2),
SVEMAP1(svunpkhi_u32, aarch64_sve_uunpkhi, 3),
SVEMAP1(svunpkhi_u64, aarch64_sve_uunpkhi, 4),
SVEMAP1(svunpklo_b, aarch64_sve_punpklo, 8),
SVEMAP1(svunpklo_s16, aarch64_sve_sunpklo, 2),
SVEMAP1(svunpklo_s32, aarch64_sve_sunpklo, 3),
SVEMAP1(svunpklo_s64, aarch64_sve_sunpklo, 4),
SVEMAP1(svunpklo_u16, aarch64_sve_uunpklo, 2),
SVEMAP1(svunpklo_u32, aarch64_sve_uunpklo, 3),
SVEMAP1(svunpklo_u64, aarch64_sve_uunpklo, 4),
SVEMAP1(svuqadd_n_s16_m, aarch64_sve_suqadd, 3330),
SVEMAP1(svuqadd_n_s16_x, aarch64_sve_suqadd, 3202),
SVEMAP1(svuqadd_n_s16_z, aarch64_sve_suqadd, 3458),
SVEMAP1(svuqadd_n_s32_m, aarch64_sve_suqadd, 3331),
SVEMAP1(svuqadd_n_s32_x, aarch64_sve_suqadd, 3203),
SVEMAP1(svuqadd_n_s32_z, aarch64_sve_suqadd, 3459),
SVEMAP1(svuqadd_n_s64_m, aarch64_sve_suqadd, 3332),
SVEMAP1(svuqadd_n_s64_x, aarch64_sve_suqadd, 3204),
SVEMAP1(svuqadd_n_s64_z, aarch64_sve_suqadd, 3460),
SVEMAP1(svuqadd_n_s8_m, aarch64_sve_suqadd, 3329),
SVEMAP1(svuqadd_n_s8_x, aarch64_sve_suqadd, 3201),
SVEMAP1(svuqadd_n_s8_z, aarch64_sve_suqadd, 3457),
SVEMAP1(svuqadd_s16_m, aarch64_sve_suqadd, 258),
SVEMAP1(svuqadd_s16_x, aarch64_sve_suqadd, 130),
SVEMAP1(svuqadd_s16_z, aarch64_sve_suqadd, 386),
SVEMAP1(svuqadd_s32_m, aarch64_sve_suqadd, 259),
SVEMAP1(svuqadd_s32_x, aarch64_sve_suqadd, 131),
SVEMAP1(svuqadd_s32_z, aarch64_sve_suqadd, 387),
SVEMAP1(svuqadd_s64_m, aarch64_sve_suqadd, 260),
SVEMAP1(svuqadd_s64_x, aarch64_sve_suqadd, 132),
SVEMAP1(svuqadd_s64_z, aarch64_sve_suqadd, 388),
SVEMAP1(svuqadd_s8_m, aarch64_sve_suqadd, 257),
SVEMAP1(svuqadd_s8_x, aarch64_sve_suqadd, 129),
SVEMAP1(svuqadd_s8_z, aarch64_sve_suqadd, 385),
SVEMAP1(svusdot_lane_s32, aarch64_sve_usdot_lane, 3),
SVEMAP1(svusdot_n_s32, aarch64_sve_usdot, 3075),
SVEMAP1(svusdot_s32, aarch64_sve_usdot, 3),
SVEMAP1(svusmmla_s32, aarch64_sve_usmmla, 3),
SVEMAP1(svuzp1_b16, aarch64_sve_uzp1, 9),
SVEMAP1(svuzp1_b32, aarch64_sve_uzp1, 10),
SVEMAP1(svuzp1_b64, aarch64_sve_uzp1, 11),
SVEMAP1(svuzp1_b8, aarch64_sve_uzp1, 8),
SVEMAP1(svuzp1_bf16, aarch64_sve_uzp1, 12),
SVEMAP1(svuzp1_f16, aarch64_sve_uzp1, 5),
SVEMAP1(svuzp1_f32, aarch64_sve_uzp1, 6),
SVEMAP1(svuzp1_f64, aarch64_sve_uzp1, 7),
SVEMAP1(svuzp1_s16, aarch64_sve_uzp1, 2),
SVEMAP1(svuzp1_s32, aarch64_sve_uzp1, 3),
SVEMAP1(svuzp1_s64, aarch64_sve_uzp1, 4),
SVEMAP1(svuzp1_s8, aarch64_sve_uzp1, 1),
SVEMAP1(svuzp1_u16, aarch64_sve_uzp1, 2),
SVEMAP1(svuzp1_u32, aarch64_sve_uzp1, 3),
SVEMAP1(svuzp1_u64, aarch64_sve_uzp1, 4),
SVEMAP1(svuzp1_u8, aarch64_sve_uzp1, 1),
SVEMAP1(svuzp1q_bf16, aarch64_sve_uzp1q, 12),
SVEMAP1(svuzp1q_f16, aarch64_sve_uzp1q, 5),
SVEMAP1(svuzp1q_f32, aarch64_sve_uzp1q, 6),
SVEMAP1(svuzp1q_f64, aarch64_sve_uzp1q, 7),
SVEMAP1(svuzp1q_s16, aarch64_sve_uzp1q, 2),
SVEMAP1(svuzp1q_s32, aarch64_sve_uzp1q, 3),
SVEMAP1(svuzp1q_s64, aarch64_sve_uzp1q, 4),
SVEMAP1(svuzp1q_s8, aarch64_sve_uzp1q, 1),
SVEMAP1(svuzp1q_u16, aarch64_sve_uzp1q, 2),
SVEMAP1(svuzp1q_u32, aarch64_sve_uzp1q, 3),
SVEMAP1(svuzp1q_u64, aarch64_sve_uzp1q, 4),
SVEMAP1(svuzp1q_u8, aarch64_sve_uzp1q, 1),
SVEMAP1(svuzp2_b16, aarch64_sve_uzp2, 9),
SVEMAP1(svuzp2_b32, aarch64_sve_uzp2, 10),
SVEMAP1(svuzp2_b64, aarch64_sve_uzp2, 11),
SVEMAP1(svuzp2_b8, aarch64_sve_uzp2, 8),
SVEMAP1(svuzp2_bf16, aarch64_sve_uzp2, 12),
SVEMAP1(svuzp2_f16, aarch64_sve_uzp2, 5),
SVEMAP1(svuzp2_f32, aarch64_sve_uzp2, 6),
SVEMAP1(svuzp2_f64, aarch64_sve_uzp2, 7),
SVEMAP1(svuzp2_s16, aarch64_sve_uzp2, 2),
SVEMAP1(svuzp2_s32, aarch64_sve_uzp2, 3),
SVEMAP1(svuzp2_s64, aarch64_sve_uzp2, 4),
SVEMAP1(svuzp2_s8, aarch64_sve_uzp2, 1),
SVEMAP1(svuzp2_u16, aarch64_sve_uzp2, 2),
SVEMAP1(svuzp2_u32, aarch64_sve_uzp2, 3),
SVEMAP1(svuzp2_u64, aarch64_sve_uzp2, 4),
SVEMAP1(svuzp2_u8, aarch64_sve_uzp2, 1),
SVEMAP1(svuzp2q_bf16, aarch64_sve_uzp2q, 12),
SVEMAP1(svuzp2q_f16, aarch64_sve_uzp2q, 5),
SVEMAP1(svuzp2q_f32, aarch64_sve_uzp2q, 6),
SVEMAP1(svuzp2q_f64, aarch64_sve_uzp2q, 7),
SVEMAP1(svuzp2q_s16, aarch64_sve_uzp2q, 2),
SVEMAP1(svuzp2q_s32, aarch64_sve_uzp2q, 3),
SVEMAP1(svuzp2q_s64, aarch64_sve_uzp2q, 4),
SVEMAP1(svuzp2q_s8, aarch64_sve_uzp2q, 1),
SVEMAP1(svuzp2q_u16, aarch64_sve_uzp2q, 2),
SVEMAP1(svuzp2q_u32, aarch64_sve_uzp2q, 3),
SVEMAP1(svuzp2q_u64, aarch64_sve_uzp2q, 4),
SVEMAP1(svuzp2q_u8, aarch64_sve_uzp2q, 1),
SVEMAP1(svwhilege_b16_s32, aarch64_sve_whilege, 2097161),
SVEMAP1(svwhilege_b16_s64, aarch64_sve_whilege, 2097161),
SVEMAP1(svwhilege_b16_u32, aarch64_sve_whilehs, 2097161),
SVEMAP1(svwhilege_b16_u64, aarch64_sve_whilehs, 2097161),
SVEMAP1(svwhilege_b32_s32, aarch64_sve_whilege, 2097162),
SVEMAP1(svwhilege_b32_s64, aarch64_sve_whilege, 2097162),
SVEMAP1(svwhilege_b32_u32, aarch64_sve_whilehs, 2097162),
SVEMAP1(svwhilege_b32_u64, aarch64_sve_whilehs, 2097162),
SVEMAP1(svwhilege_b64_s32, aarch64_sve_whilege, 2097163),
SVEMAP1(svwhilege_b64_s64, aarch64_sve_whilege, 2097163),
SVEMAP1(svwhilege_b64_u32, aarch64_sve_whilehs, 2097163),
SVEMAP1(svwhilege_b64_u64, aarch64_sve_whilehs, 2097163),
SVEMAP1(svwhilege_b8_s32, aarch64_sve_whilege, 2097160),
SVEMAP1(svwhilege_b8_s64, aarch64_sve_whilege, 2097160),
SVEMAP1(svwhilege_b8_u32, aarch64_sve_whilehs, 2097160),
SVEMAP1(svwhilege_b8_u64, aarch64_sve_whilehs, 2097160),
SVEMAP1(svwhilegt_b16_s32, aarch64_sve_whilegt, 2097161),
SVEMAP1(svwhilegt_b16_s64, aarch64_sve_whilegt, 2097161),
SVEMAP1(svwhilegt_b16_u32, aarch64_sve_whilehi, 2097161),
SVEMAP1(svwhilegt_b16_u64, aarch64_sve_whilehi, 2097161),
SVEMAP1(svwhilegt_b32_s32, aarch64_sve_whilegt, 2097162),
SVEMAP1(svwhilegt_b32_s64, aarch64_sve_whilegt, 2097162),
SVEMAP1(svwhilegt_b32_u32, aarch64_sve_whilehi, 2097162),
SVEMAP1(svwhilegt_b32_u64, aarch64_sve_whilehi, 2097162),
SVEMAP1(svwhilegt_b64_s32, aarch64_sve_whilegt, 2097163),
SVEMAP1(svwhilegt_b64_s64, aarch64_sve_whilegt, 2097163),
SVEMAP1(svwhilegt_b64_u32, aarch64_sve_whilehi, 2097163),
SVEMAP1(svwhilegt_b64_u64, aarch64_sve_whilehi, 2097163),
SVEMAP1(svwhilegt_b8_s32, aarch64_sve_whilegt, 2097160),
SVEMAP1(svwhilegt_b8_s64, aarch64_sve_whilegt, 2097160),
SVEMAP1(svwhilegt_b8_u32, aarch64_sve_whilehi, 2097160),
SVEMAP1(svwhilegt_b8_u64, aarch64_sve_whilehi, 2097160),
SVEMAP1(svwhilele_b16_s32, aarch64_sve_whilele, 2097161),
SVEMAP1(svwhilele_b16_s64, aarch64_sve_whilele, 2097161),
SVEMAP1(svwhilele_b16_u32, aarch64_sve_whilels, 2097161),
SVEMAP1(svwhilele_b16_u64, aarch64_sve_whilels, 2097161),
SVEMAP1(svwhilele_b32_s32, aarch64_sve_whilele, 2097162),
SVEMAP1(svwhilele_b32_s64, aarch64_sve_whilele, 2097162),
SVEMAP1(svwhilele_b32_u32, aarch64_sve_whilels, 2097162),
SVEMAP1(svwhilele_b32_u64, aarch64_sve_whilels, 2097162),
SVEMAP1(svwhilele_b64_s32, aarch64_sve_whilele, 2097163),
SVEMAP1(svwhilele_b64_s64, aarch64_sve_whilele, 2097163),
SVEMAP1(svwhilele_b64_u32, aarch64_sve_whilels, 2097163),
SVEMAP1(svwhilele_b64_u64, aarch64_sve_whilels, 2097163),
SVEMAP1(svwhilele_b8_s32, aarch64_sve_whilele, 2097160),
SVEMAP1(svwhilele_b8_s64, aarch64_sve_whilele, 2097160),
SVEMAP1(svwhilele_b8_u32, aarch64_sve_whilels, 2097160),
SVEMAP1(svwhilele_b8_u64, aarch64_sve_whilels, 2097160),
SVEMAP1(svwhilelt_b16_s32, aarch64_sve_whilelt, 2097161),
SVEMAP1(svwhilelt_b16_s64, aarch64_sve_whilelt, 2097161),
SVEMAP1(svwhilelt_b16_u32, aarch64_sve_whilelo, 2097161),
SVEMAP1(svwhilelt_b16_u64, aarch64_sve_whilelo, 2097161),
SVEMAP1(svwhilelt_b32_s32, aarch64_sve_whilelt, 2097162),
SVEMAP1(svwhilelt_b32_s64, aarch64_sve_whilelt, 2097162),
SVEMAP1(svwhilelt_b32_u32, aarch64_sve_whilelo, 2097162),
SVEMAP1(svwhilelt_b32_u64, aarch64_sve_whilelo, 2097162),
SVEMAP1(svwhilelt_b64_s32, aarch64_sve_whilelt, 2097163),
SVEMAP1(svwhilelt_b64_s64, aarch64_sve_whilelt, 2097163),
SVEMAP1(svwhilelt_b64_u32, aarch64_sve_whilelo, 2097163),
SVEMAP1(svwhilelt_b64_u64, aarch64_sve_whilelo, 2097163),
SVEMAP1(svwhilelt_b8_s32, aarch64_sve_whilelt, 2097160),
SVEMAP1(svwhilelt_b8_s64, aarch64_sve_whilelt, 2097160),
SVEMAP1(svwhilelt_b8_u32, aarch64_sve_whilelo, 2097160),
SVEMAP1(svwhilelt_b8_u64, aarch64_sve_whilelo, 2097160),
SVEMAP1(svwhilerw_bf16, aarch64_sve_whilerw_h, 4194316),
SVEMAP1(svwhilerw_f16, aarch64_sve_whilerw_h, 4194309),
SVEMAP1(svwhilerw_f32, aarch64_sve_whilerw_s, 4194310),
SVEMAP1(svwhilerw_f64, aarch64_sve_whilerw_d, 4194311),
SVEMAP1(svwhilerw_s16, aarch64_sve_whilerw_h, 4194306),
SVEMAP1(svwhilerw_s32, aarch64_sve_whilerw_s, 4194307),
SVEMAP1(svwhilerw_s64, aarch64_sve_whilerw_d, 4194308),
SVEMAP1(svwhilerw_s8, aarch64_sve_whilerw_b, 4194305),
SVEMAP1(svwhilerw_u16, aarch64_sve_whilerw_h, 4194306),
SVEMAP1(svwhilerw_u32, aarch64_sve_whilerw_s, 4194307),
SVEMAP1(svwhilerw_u64, aarch64_sve_whilerw_d, 4194308),
SVEMAP1(svwhilerw_u8, aarch64_sve_whilerw_b, 4194305),
SVEMAP1(svwhilewr_bf16, aarch64_sve_whilewr_h, 4194316),
SVEMAP1(svwhilewr_f16, aarch64_sve_whilewr_h, 4194309),
SVEMAP1(svwhilewr_f32, aarch64_sve_whilewr_s, 4194310),
SVEMAP1(svwhilewr_f64, aarch64_sve_whilewr_d, 4194311),
SVEMAP1(svwhilewr_s16, aarch64_sve_whilewr_h, 4194306),
SVEMAP1(svwhilewr_s32, aarch64_sve_whilewr_s, 4194307),
SVEMAP1(svwhilewr_s64, aarch64_sve_whilewr_d, 4194308),
SVEMAP1(svwhilewr_s8, aarch64_sve_whilewr_b, 4194305),
SVEMAP1(svwhilewr_u16, aarch64_sve_whilewr_h, 4194306),
SVEMAP1(svwhilewr_u32, aarch64_sve_whilewr_s, 4194307),
SVEMAP1(svwhilewr_u64, aarch64_sve_whilewr_d, 4194308),
SVEMAP1(svwhilewr_u8, aarch64_sve_whilewr_b, 4194305),
SVEMAP2(svwrffr, 1048584),
SVEMAP1(svxar_n_s16, aarch64_sve_xar, 2),
SVEMAP1(svxar_n_s32, aarch64_sve_xar, 3),
SVEMAP1(svxar_n_s64, aarch64_sve_xar, 4),
SVEMAP1(svxar_n_s8, aarch64_sve_xar, 1),
SVEMAP1(svxar_n_u16, aarch64_sve_xar, 2),
SVEMAP1(svxar_n_u32, aarch64_sve_xar, 3),
SVEMAP1(svxar_n_u64, aarch64_sve_xar, 4),
SVEMAP1(svxar_n_u8, aarch64_sve_xar, 1),
SVEMAP1(svzip1_b16, aarch64_sve_zip1, 9),
SVEMAP1(svzip1_b32, aarch64_sve_zip1, 10),
SVEMAP1(svzip1_b64, aarch64_sve_zip1, 11),
SVEMAP1(svzip1_b8, aarch64_sve_zip1, 8),
SVEMAP1(svzip1_bf16, aarch64_sve_zip1, 12),
SVEMAP1(svzip1_f16, aarch64_sve_zip1, 5),
SVEMAP1(svzip1_f32, aarch64_sve_zip1, 6),
SVEMAP1(svzip1_f64, aarch64_sve_zip1, 7),
SVEMAP1(svzip1_s16, aarch64_sve_zip1, 2),
SVEMAP1(svzip1_s32, aarch64_sve_zip1, 3),
SVEMAP1(svzip1_s64, aarch64_sve_zip1, 4),
SVEMAP1(svzip1_s8, aarch64_sve_zip1, 1),
SVEMAP1(svzip1_u16, aarch64_sve_zip1, 2),
SVEMAP1(svzip1_u32, aarch64_sve_zip1, 3),
SVEMAP1(svzip1_u64, aarch64_sve_zip1, 4),
SVEMAP1(svzip1_u8, aarch64_sve_zip1, 1),
SVEMAP1(svzip1q_bf16, aarch64_sve_zip1q, 12),
SVEMAP1(svzip1q_f16, aarch64_sve_zip1q, 5),
SVEMAP1(svzip1q_f32, aarch64_sve_zip1q, 6),
SVEMAP1(svzip1q_f64, aarch64_sve_zip1q, 7),
SVEMAP1(svzip1q_s16, aarch64_sve_zip1q, 2),
SVEMAP1(svzip1q_s32, aarch64_sve_zip1q, 3),
SVEMAP1(svzip1q_s64, aarch64_sve_zip1q, 4),
SVEMAP1(svzip1q_s8, aarch64_sve_zip1q, 1),
SVEMAP1(svzip1q_u16, aarch64_sve_zip1q, 2),
SVEMAP1(svzip1q_u32, aarch64_sve_zip1q, 3),
SVEMAP1(svzip1q_u64, aarch64_sve_zip1q, 4),
SVEMAP1(svzip1q_u8, aarch64_sve_zip1q, 1),
SVEMAP1(svzip2_b16, aarch64_sve_zip2, 9),
SVEMAP1(svzip2_b32, aarch64_sve_zip2, 10),
SVEMAP1(svzip2_b64, aarch64_sve_zip2, 11),
SVEMAP1(svzip2_b8, aarch64_sve_zip2, 8),
SVEMAP1(svzip2_bf16, aarch64_sve_zip2, 12),
SVEMAP1(svzip2_f16, aarch64_sve_zip2, 5),
SVEMAP1(svzip2_f32, aarch64_sve_zip2, 6),
SVEMAP1(svzip2_f64, aarch64_sve_zip2, 7),
SVEMAP1(svzip2_s16, aarch64_sve_zip2, 2),
SVEMAP1(svzip2_s32, aarch64_sve_zip2, 3),
SVEMAP1(svzip2_s64, aarch64_sve_zip2, 4),
SVEMAP1(svzip2_s8, aarch64_sve_zip2, 1),
SVEMAP1(svzip2_u16, aarch64_sve_zip2, 2),
SVEMAP1(svzip2_u32, aarch64_sve_zip2, 3),
SVEMAP1(svzip2_u64, aarch64_sve_zip2, 4),
SVEMAP1(svzip2_u8, aarch64_sve_zip2, 1),
SVEMAP1(svzip2q_bf16, aarch64_sve_zip2q, 12),
SVEMAP1(svzip2q_f16, aarch64_sve_zip2q, 5),
SVEMAP1(svzip2q_f32, aarch64_sve_zip2q, 6),
SVEMAP1(svzip2q_f64, aarch64_sve_zip2q, 7),
SVEMAP1(svzip2q_s16, aarch64_sve_zip2q, 2),
SVEMAP1(svzip2q_s32, aarch64_sve_zip2q, 3),
SVEMAP1(svzip2q_s64, aarch64_sve_zip2q, 4),
SVEMAP1(svzip2q_s8, aarch64_sve_zip2q, 1),
SVEMAP1(svzip2q_u16, aarch64_sve_zip2q, 2),
SVEMAP1(svzip2q_u32, aarch64_sve_zip2q, 3),
SVEMAP1(svzip2q_u64, aarch64_sve_zip2q, 4),
SVEMAP1(svzip2q_u8, aarch64_sve_zip2q, 1),
#endif

